/*
 * Copyright 2016-2017 Golden Code Development Corporation
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this software
 * and associated documentation files (the "Software"),  to deal in the Software without
 * restriction, including without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all copies or
 * substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
 * BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

package com.goldencode.hotel;

import com.goldencode.p2j.util.*;
import com.goldencode.p2j.persist.*;
import com.goldencode.hotel.dmo._temp.*;
import com.goldencode.hotel.dmo.hotel.*;
import com.goldencode.p2j.ui.*;
import com.goldencode.util.*;
import com.goldencode.p2j.persist.lock.*;

import static com.goldencode.p2j.util.BlockManager.*;
import static com.goldencode.p2j.util.CompareOps.*;
import static com.goldencode.p2j.util.MathOps.*;
import static com.goldencode.p2j.util.logical.*;
import static com.goldencode.p2j.util.TextOps.*;
import static com.goldencode.p2j.util.ProcedureManager.*;
import static com.goldencode.p2j.util.character.*;
import static com.goldencode.p2j.ui.LogicalTerminal.*;

/**
 * Business logic (converted to Java from the 4GL source code
 * in ehotel.p).
 */
public class Ehotel
{
   public static final String FMT_STR_1 = "z9.99";

   /* temp-table of aggregated data for room types over a period */   
   TtRoomTypeAgg_1_1.Buf ttRoomTypeAgg = SharedVariableManager.addTempTable("ttRoomTypeAgg", TtRoomTypeAgg_1_1.Buf.class, "ttRoomTypeAgg", "tt-room-type-agg");

   /* _ADM-CODE-BLOCK-START _CUSTOM _INCLUDED-LIB-CUSTOM CUSTOM */   
   /*********************************************************************
   * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
   * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
   * below.  All Rights Reserved.                                       *
   *                                                                    *
   * The Initial Developer of the Original Code is PSC.  The Original   *
   * Code is Progress IDE code released to open source December 1, 2000.*
   *                                                                    *
   * The contents of this file are subject to the Possenet Public       *
   * License Version 1.0 (the "License"); you may not use this file     *
   * except in compliance with the License.  A copy of the License is   *
   * available as of the date of this notice at                         *
   * http://www.possenet.org/license.html                               *
   *                                                                    *
   * Software distributed under the License is distributed on an "AS IS"*
   * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
   * should refer to the License for the specific language governing    *
   * rights and limitations under the License.                          *
   *                                                                    *
   * Contributors:                                                      *
   *                                                                    *
   *********************************************************************/   
   /*------------------------------------------------------------------------
       File        : containrcustom.i
       Purpose     : References the start of the custom super procedure.
                     Allows properties initialization.
   
       Syntax      : {src/adm2/custom/containrcustom.i}
   
       Description :
   
       Created     : 06/03/1999
       Notes       : Referenced in {src/adm2/containr.i}
     ----------------------------------------------------------------------*/   
   /*          This .W file was created with the Progress AppBuilder.      */   
   /*----------------------------------------------------------------------*/   
   /* ***************************  Definitions  ************************** */   
   /* _UIB-CODE-BLOCK-END */   
   /* ********************  Preprocessor Definitions  ******************** */   
   /* _UIB-PREPROCESSOR-BLOCK-END */   
   /* *********************** Procedure Settings ************************ */   
   /* Settings for THIS-PROCEDURE
      Type: Include
      Allow: 
      Frames: 0
      Add Fields to: Neither
      Other Settings: INCLUDE-ONLY
    */   
   /* *************************  Create Window  ************************** */   
   /* DESIGN Window definition (used by the UIB) 
     CREATE WINDOW Include ASSIGN
            HEIGHT             = 15
            WIDTH              = 60.
   _* END WINDOW DEFINITION *_
                                                                           */   
   /* ***************************  Main Block  *************************** */   
   /* Starts here the custom super procedure 
      Uncomment to run it */   
   /*RUN start-super-proc ("adm2/custom/containrcustom.p":U).*/   
   /* _UIB-CODE-BLOCK-END */   
   /* _ADM-CODE-BLOCK-END */   
   /* see explanation in setWindowFrameHandle */   
   /* _UIB-CODE-BLOCK-END */   
   /* temp-table to count rooms utilized (reserved or occupied) by date */   
   TtRoomsUsed_1_1.Buf ttRoomsUsed = SharedVariableManager.addTempTable("ttRoomsUsed", TtRoomsUsed_1_1.Buf.class, "ttRoomsUsed", "tt-rooms-used");

   Rate.Buf rate = RecordBuffer.define(Rate.Buf.class, "hotel", "rate", "rate");

   RoomType.Buf roomType = RecordBuffer.define(RoomType.Buf.class, "hotel", "roomType", "room-type");

   Stay.Buf stay = RecordBuffer.define(Stay.Buf.class, "hotel", "stay", "stay");

   Reservation.Buf reservation = RecordBuffer.define(Reservation.Buf.class, "hotel", "reservation", "reservation");

   Room.Buf room = RecordBuffer.define(Room.Buf.class, "hotel", "room", "room");

   /* These preprocessors tell at compile time which properties can be
        retrieved directly from the temp-table. */   
   /* This property contains the security for all objects found in the AllFieldHandles Property */    
   /* Include the next property file up the chain (in this case, the top
        of the chain); it starts the property temp-table definition and we
        add our field definitions to it. */   
   /*********************************************************************
   * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
   * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
   * below.  All Rights Reserved.                                       *
   *                                                                    *
   * The Initial Developer of the Original Code is PSC.  The Original   *
   * Code is Progress IDE code released to open source December 1, 2000.*
   *                                                                    *
   * The contents of this file are subject to the Possenet Public       *
   * License Version 1.0 (the "License"); you may not use this file     *
   * except in compliance with the License.  A copy of the License is   *
   * available as of the date of this notice at                         *
   * http://www.possenet.org/license.html                               *
   *                                                                    *
   * Software distributed under the License is distributed on an "AS IS"*
   * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
   * should refer to the License for the specific language governing    *
   * rights and limitations under the License.                          *
   *                                                                    *
   * Contributors:                                                      *
   *                                                                    *
   *********************************************************************/   
   /*--------------------------------------------------------------------------
       File        : smrtprop.i
       Purpose     : Starts smart.p super procedure and defines general
                     SmartObject properties and other values.
       Syntax      : {src/adm2/smrtprop.i}
   
       Description :
   
       Modified    : February 4, 2000 - Version 9.1B
     ------------------------------------------------------------------------*/   
   /*          This .W file was created with the Progress UIB.             */   
   /*----------------------------------------------------------------------*/   
   /* ***************************  Definitions  ************************** */   
   /*********************************************************************
   * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
   * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
   * below.  All Rights Reserved.                                       *
   *                                                                    *
   * The Initial Developer of the Original Code is PSC.  The Original   *
   * Code is Progress IDE code released to open source December 1, 2000.*
   *                                                                    *
   * The contents of this file are subject to the Possenet Public       *
   * License Version 1.0 (the "License"); you may not use this file     *
   * except in compliance with the License.  A copy of the License is   *
   * available as of the date of this notice at                         *
   * http://www.possenet.org/license.html                               *
   *                                                                    *
   * Software distributed under the License is distributed on an "AS IS"*
   * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
   * should refer to the License for the specific language governing    *
   * rights and limitations under the License.                          *
   *                                                                    *
   * Contributors:                                                      *
   *                                                                    *
   *********************************************************************/   
   /*--------------------------------------------------------------------------
       File        : smartdefscustom.i
       Purpose     : Entry point to define:
                       New definitions
                       {&xcInstanceProperties}
                       {&ADM-PROPERTY-DLG}
       Syntax      : {src/adm2/custom/smartdefscustom.i}
   
       Modified    : 06/03/1999
       
       Notes       : Referenced in {src/adm2/smrtprop.i}
     ------------------------------------------------------------------------*/   
   /* Service Managers */   
   /*********************************************************************
   * Copyright (C) 2000-2002 by Progress Software Corporation ("PSC"),  *
   * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
   * below.  All Rights Reserved.                                       *
   *                                                                    *
   * The Initial Developer of the Original Code is PSC.  The Original   *
   * Code is Progress IDE code released to open source December 1, 2000.*
   *                                                                    *
   * The contents of this file are subject to the Possenet Public       *
   * License Version 1.0 (the "License"); you may not use this file     *
   * except in compliance with the License.  A copy of the License is   *
   * available as of the date of this notice at                         *
   * http://www.possenet.org/license.html                               *
   *                                                                    *
   * Software distributed under the License is distributed on an "AS IS"*
   * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
   * should refer to the License for the specific language governing    *
   * rights and limitations under the License.                          *
   *                                                                    *
   * Contributors:                                                      *
   *                                                                    *
   *********************************************************************/   
   /*---------------------------------------------------------------------------------
     File: globals.i
   
     Description:  ICF Global Shared Variables
   
     Purpose:      ICF Global Shared Variables
                   The global shared variables available within the Dynamics environment.
                   
                   NOTE: (Dynamic Only )
                   Shared variables do not span the client / server boundary. On the
                   client these are set-up in afstartupp.p and on the server in the agent
                   as_startup.p
                   Also note if we include the guts of a procedure as an internal
                   procedure for shared client/server code, we sometime need to
                   ensure the shared variables are not redefined.
   
     Parameters:   <none>
   
     History:
     --------
   
     Modified: 11/08/2001 - Mark Davies (MIP)
               Copied code for ICF afglobals.i and added here to allow
               ICF changed adm code to compile. afglobals.i now includes
               this file for definitions.
               Any ADM code will now reference globals.i which excludes any other
               ICF specific includes.
   -----------------------------------------------------------*/   
   /*                   This .W file was created with the Progress UIB.             */   
   /*-------------------------------------------------------------------------------*/   
   /* ***************************  Definitions  ************************** */   
   /* _UIB-CODE-BLOCK-END */   
   /* ********************  Preprocessor Definitions  ******************** */   
   /* _UIB-PREPROCESSOR-BLOCK-END */   
   /* *********************** Procedure Settings ************************ */   
   /* Settings for THIS-PROCEDURE
      Type: Include
      Allow: 
      Frames: 0
      Add Fields to: Neither
      Other Settings: INCLUDE-ONLY
    */   
   /* *************************  Create Window  ************************** */   
   /* DESIGN Window definition (used by the UIB) 
     CREATE WINDOW Include ASSIGN
            HEIGHT             = 5.57
            WIDTH              = 64.4.
   _* END WINDOW DEFINITION *_
                                                                           */   
   /* ***************************  Main Block  *************************** */   
   /* Used whereever Crystal is used and when defining the tt_datasource temp-table 
      to define the maximum columns supported when printing to crystal.
      See browsercustom.p and afcrpplip2p.p for example uses
   */   
   /* Used for filter checks */   
   /* The table delimiter used in assignlist, *ColumnsByTable props */   
   handle gshAstraAppserver /* quick access to all handles on object, e.g. viewer, whether enabled or not */  /* quick access to all fields on object, e.g. viewer, whether enabled or not */ = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshastraappserver", true, new handle());

   /* Handle to Astra Application Server Partition */   
   handle gshSessionManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshsessionmanager", true, new handle());

   /* Handle Astra Session Manager */   
   handle gshRimanager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshrimanager", true, new handle());

   /* Handle Astra Session Manager */   
   handle gshSecurityManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshsecuritymanager", true, new handle());

   /* Handle Astra Security Manager */   
   handle gshProfileManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshprofilemanager", true, new handle());

   /* Handle Astra Profile Manager */   
   handle gshRepositoryManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshrepositorymanager", true, new handle());

   /* Handle Astra Repository Manager */   
   handle gshTranslationManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshtranslationmanager", true, new handle());

   /* Handle Astra Translation Manager */   
   handle gshWebManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshwebmanager", true, new handle());

   /* Handle Astra Web Manager */   
   character gscSessionId = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gscsessionid", true, new character(""));

   /* Unique session id */   
   decimal gsdSessionObj = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gsdsessionobj", true, new decimal(0));

   /* Unique session id */   
   handle gshFinManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshfinmanager", true, new handle());

   /* Handle AstraFin Generic Manager */   
   handle gshGenManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshgenmanager", true, new handle());

   /* Handle AstraFramework Generic Manager */   
   handle gshAgnManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshagnmanager", true, new handle());

   /* Handle AstraGen Generic Manager */   
   decimal gsdTempUniqueId = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gsdtempuniqueid", true, new decimal(0));

   /* A number for unique objects */   
   decimal gsdUserObj = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gsduserobj", true, new decimal(0));

   /* User Object ID */   
   decimal gsdRenderTypeObj = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gsdrendertypeobj", true, new decimal(0));

   /* Render Type Object ID */   
   decimal gsdSessionScopeObj = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gsdsessionscopeobj", true, new decimal(0));

   /* Render Type Object ID */   
   /*********************************************************************
   * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
   * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
   * below.  All Rights Reserved.                                       *
   *                                                                    *
   * The Initial Developer of the Original Code is PSC.  The Original   *
   * Code is Progress IDE code released to open source December 1, 2000.*
   *                                                                    *
   * The contents of this file are subject to the Possenet Public       *
   * License Version 1.0 (the "License"); you may not use this file     *
   * except in compliance with the License.  A copy of the License is   *
   * available as of the date of this notice at                         *
   * http://www.possenet.org/license.html                               *
   *                                                                    *
   * Software distributed under the License is distributed on an "AS IS"*
   * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
   * should refer to the License for the specific language governing    *
   * rights and limitations under the License.                          *
   *                                                                    *
   * Contributors:                                                      *
   *                                                                    *
   *********************************************************************/   
   /*------------------------------------------------------------------------
       File        : globalscustom.i
       Purpose     : The developer can create his own version of this to 
                     allow 'global' definitions to be added to the 
                     framework.   
       Syntax      : {src/adm2/custom/globalscustom.i}
   
       Description :
   
       Created     : 06/03/1999
       Notes       : Referenced in {src/adm2/globals.i}
     ----------------------------------------------------------------------*/   
   /*          This .W file was created with the Progress AppBuilder.      */   
   /*----------------------------------------------------------------------*/   
   /* ***************************  Definitions  ************************** */   
   /* _UIB-CODE-BLOCK-END */   
   /* define the ADM Version and broker handle for all SmartObjects */   
   /* Note that adm-props-from-repository is documented as possible to define
        as a variable for flexible override, so it cannot be used for conditional 
        compile.  */   
   /* adm-props-from-repository is supported to be defined as a variable,
       so it is not used for conditional compile */   
   handle ghProp = TypeFactory.handle();

   handle hpage = UndoableFactory.handle();

   handle wWin = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "wwin", new handle());

   /**
    * External procedure (converted to Java from the 4GL source code
    * in ehotel.p).
    */
   public void execute()
   {
      /* For {get/set} */
      handle ghAdmprops = TypeFactory.handle();
      /* Handle of prop t-t */
      handle ghAdmpropsBuf = TypeFactory.handle();
      /*  and its buffer    */
      logical glAdmloadFromRepos = TypeFactory.logical();
      character cObjectName = TypeFactory.character();
      integer iStart = TypeFactory.integer();
      
      externalProcedure(Ehotel.this, new Block((Init) () -> 
      {
         gsdSessionObj.setPrecision(9);
         gsdTempUniqueId.setPrecision(9);
         gsdUserObj.setPrecision(9);
         gsdRenderTypeObj.setPrecision(9);
         gsdSessionScopeObj.setPrecision(9);
      }, 
      (Body) () -> 
      {
         RecordBuffer.openScope(ttRoomsUsed, ttRoomTypeAgg, room, reservation, stay, roomType, rate);
         
         /* These preprocessors let the get and set methods know at compile time
             which property values are located in the temp-table and which must
             be accessed through the property functions.
          */
         /* for support of dynamic objects */
         /* physical name - no path */
         /* for support of runtime parameters passed from the menus */
         /* Properties for super procedure management */
         /* The name of the super procedure; can be a CSV list */ 
 /* STATELESS, STATEFUL; the mode that all supers for the object will be launched in */ 
 /* Runtime property storing handles to the object's super procedures, defined in SuperProcedure. */
         /* Layout support for dynamic objects */
         /* contains a layout position that allows an object to be laid out correctly. */
         /* determine if we need to create the prop fields here or from the */
         /* repository (if ICF is running ) */
         /* COMMENT THE FOLLOWING LINE TO USE THE "<class>prop.i" DEFINITIONS */
         /* This temp-table defines all the propertt fields for an object.
             This include file contributes the DEFINE statement header and
             all basic smart object properties. Each other property class include file
             adds its own fields and then the parent object ends the statement.
             Define the fields for smart objects only, not for their super procedures.
          */
         /* Note that adm-props-from-repository is intended to be replaced with
              a variable for flexible override, so it cannot be used for conditional 
              compile.  */
         if ((gshRepositoryManager.isValid()).booleanValue())
         {
            if (_not(new logical(ControlFlowOps.invokeDynamicFunctionInWithMode("prepareInstance", gshRepositoryManager, "II", valueOf(thisProcedure()), sourceProcedure()))))
            {
               stop();
            }
            
            ghAdmpropsBuf.assign(handle.fromString(entry(1, thisProcedure().unwrapADMData().getADMData(), chr(1))));
            glAdmloadFromRepos.assign(ghAdmpropsBuf.isValid());
         }
         
         /* VALID-HANDLE(gshRepositoryManager) */
         if (_not(handle.fromString(entry(1, thisProcedure().unwrapADMData().getADMData(), chr(1))).isValid()))
         {
            TempTableBuilder.create(ghAdmprops);
            ghAdmprops.unwrapTempTable().setCanUndo(new logical(false));
            ghAdmprops.unwrapTempTable().addNewField(new character("ObjectName"), new character("CHAR"), new integer(0), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("ObjectVersion"), new character("CHAR"), new integer(0), new character(), new character("ADM2.2"));
            ghAdmprops.unwrapTempTable().addNewField(new character("ObjectType"), new character("CHAR"), new integer(0), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("ContainerType"), new character("CHAR"), new integer(0), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("PropertyDialog"), new character("CHAR"), new integer(0), new character(), new character("adm2/support/visuald.w"));
            ghAdmprops.unwrapTempTable().addNewField(new character("QueryObject"), new character("LOGICAL"), new integer(0), new character(), new logical(false));
            ghAdmprops.unwrapTempTable().addNewField(new character("ContainerHandle"), new character("HANDLE"));
            ghAdmprops.unwrapTempTable().addNewField(new character("InstanceProperties"), new character("CHAR"), new integer(0), new character(), new character("LogicalObjectName,PhysicalObjectName,DynamicObject,RunAttribute, HideOnInit,DisableOnInit,ObjectLayout"));
            /* NOTE: Any need to support &User-Supported-Links?? */
            ghAdmprops.unwrapTempTable().addNewField(new character("SupportedLinks"), new character("CHAR"), new integer(0), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("ContainerHidden"), new character("LOGICAL"), new integer(0), new character(), new logical(false));
            ghAdmprops.unwrapTempTable().addNewField(new character("ObjectInitialized"), new character("LOGICAL"), new integer(0), new character(), new logical(false));
            /*  "hidden" is a logical concept. */
            ghAdmprops.unwrapTempTable().addNewField(new character("ObjectHidden"), new character("LOGICAL"), new integer(0), new character(), new logical(true));
            ghAdmprops.unwrapTempTable().addNewField(new character("HideOnInit"), new character("LOGICAL"), new integer(0), new character(), new logical(false));
            ghAdmprops.unwrapTempTable().addNewField(new character("UIBMode"), new character("CHAR"), new integer(0), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("ContainerSource"), new character("HANDLE"));
            /* Note that datavis adds some events that are only required for updating objects */
            /* isUpdateActive event is needed only for data, datavis and container classes,
                 but we currently define it here.. (this simplifies the logic for datavis, 
                 which may or may not inherit from container)
                 confirmOk and confirmCancel only for container ans datavis */
            ghAdmprops.unwrapTempTable().addNewField(new character("ContainerSourceEvents"), new character("CHAR"), new integer(0), new character(), new character("initializeObject,hideObject,viewObject,destroyObject,enableObject,confirmExit,confirmCancel,confirmOk,isUpdateActive"));
            ghAdmprops.unwrapTempTable().addNewField(new character("DataSource"), new character("HANDLE"));
            /* Note that DataSourceEvents is overidden in data.i since some of these 
                 events are intended for visual targets; queryPosition, fetchDataSet, 
                 assignMaxdataguess and the most resent addition confirmUndo and 
                 confirmCommit. These two are NOT dataSourceEvents for SDOs as the message 
                 should not to be published to objects not affected by the commit/undo. */
            ghAdmprops.unwrapTempTable().addNewField(new character("DataSourceEvents"), new character("CHAR"), new integer(0), new character(), new character("dataAvailable,queryPosition,updateState,deleteComplete,fetchDataSet,confirmContinue,confirmCommit,confirmUndo,assignMaxGuess,isUpdatePending"));
            ghAdmprops.unwrapTempTable().addNewField(new character("TranslatableProperties"), new character("CHAR"), new integer(0), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("ObjectPage"), new character("INT"), new integer(0), new character(), new integer(0));
            ghAdmprops.unwrapTempTable().addNewField(new character("DBAware"), new character("LOGICAL"), new integer(0), new character(), new logical(false));
            ghAdmprops.unwrapTempTable().addNewField(new character("DesignDataObject"), new character("CHAR"), new integer(0), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("DataSourceNames"), new character("CHAR"), new integer(0), new character(), new unknown());
            ghAdmprops.unwrapTempTable().addNewField(new character("DataTarget"), new character("CHAR"), new integer(0), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("DataTargetEvents"), new character("CHARACTER"), new integer(0), new character(), new character("updateState,rowObjectState,fetchBatch,LinkState"));
            ghAdmprops.unwrapTempTable().addNewField(new character("LogicalObjectName"), new character("CHARACTER"));
            ghAdmprops.unwrapTempTable().addNewField(new character("PhysicalObjectName"), new character("CHARACTER"), new integer(), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("LogicalVersion"), new character("CHARACTER"));
            ghAdmprops.unwrapTempTable().addNewField(new character("PhysicalVersion"), new character("CHARACTER"), new integer(), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("DynamicObject"), new character("LOGICAL"));
            ghAdmprops.unwrapTempTable().addNewField(new character("RunAttribute"), new character("CHARACTER"));
            ghAdmprops.unwrapTempTable().addNewField(new character("ChildDataKey"), new character("CHARACTER"));
            ghAdmprops.unwrapTempTable().addNewField(new character("ParentDataKey"), new character("CHARACTER"));
            ghAdmprops.unwrapTempTable().addNewField(new character("DataLinksEnabled"), new character("LOGICAL"), new integer(), new character(), new logical(true));
            ghAdmprops.unwrapTempTable().addNewField(new character("InactiveLinks"), new character("CHARACTER"));
            ghAdmprops.unwrapTempTable().addNewField(new character("InstanceId"), new character("DECIMAL"));
            /* SuperProcedure management */
            ghAdmprops.unwrapTempTable().addNewField(new character("SuperProcedure"), new character("CHARACTER"));
            ghAdmprops.unwrapTempTable().addNewField(new character("SuperProcedureMode"), new character("CHARACTER"));
            ghAdmprops.unwrapTempTable().addNewField(new character("SuperProcedureHandle"), new character("CHARACTER"));
            ghAdmprops.unwrapTempTable().addNewField(new character("LayoutPosition"), new character("CHARACTER"));
            ghAdmprops.unwrapTempTable().addNewField(new character("ClassName"), new character("CHARACTER"));
            ghAdmprops.unwrapTempTable().addNewField(new character("RenderingProcedure"), new character("CHARACTER"));
            ghAdmprops.unwrapTempTable().addNewField(new character("ThinRenderingProcedure"), new character("CHARACTER"));
            ghAdmprops.unwrapTempTable().addNewField(new character("Label"), new character("CHAR"), new integer(0), new character(), new unknown());
         }
         
         /* _UIB-CODE-BLOCK-END */
         /* else if appserver aware (adecomm/appserv.i) include appserver props */
         if (_not(handle.fromString(entry(1, thisProcedure().unwrapADMData().getADMData(), chr(1))).isValid()))
         {
            ghAdmprops.unwrapTempTable().addNewField(new character("ObjectLayout"), new character("CHARACTER"), new integer(0), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("LayoutOptions"), new character("CHARACTER"), new integer(0), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("ObjectEnabled"), new character("LOGICAL"), new integer(0), new character(), new logical(false));
            ghAdmprops.unwrapTempTable().addNewField(new character("LayoutVariable"), new character("CHARACTER"), new integer(0), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("DefaultLayout"), new character("CHARACTER"), new integer(0), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("DisableOnInit"), new character("LOGICAL"), new integer(0), new character(), new logical(false));
            ghAdmprops.unwrapTempTable().addNewField(new character("EnabledObjFlds"), new character("CHARACTER"), new integer(0), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("EnabledObjHdls"), new character("CHARACTER"), new integer(0), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("FieldSecurity"), new character("CHARACTER"), new integer(0), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("SecuredTokens"), new character("CHARACTER"), new integer(0), new character(), new unknown());
            /* Initial must be unknown. */
            ghAdmprops.unwrapTempTable().addNewField(new character("AllFieldHandles"), new character("CHARACTER"), new integer(0), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("AllFieldNames"), new character("CHARACTER"), new integer(0), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("MinHeight"), new character("DECIMAL"), new integer(0), new character(), new integer(0));
            ghAdmprops.unwrapTempTable().addNewField(new character("MinWidth"), new character("DECIMAL"), new integer(0), new character(), new integer(0));
            ghAdmprops.unwrapTempTable().addNewField(new character("ResizeHorizontal"), new character("LOGICAL"), new integer(0), new character(), new unknown());
            ghAdmprops.unwrapTempTable().addNewField(new character("ResizeVertical"), new character("LOGICAL"), new integer(0), new character(), new unknown());
            ghAdmprops.unwrapTempTable().addNewField(new character("ObjectSecured"), new character("LOGICAL"), new integer(0), new character(), new logical(false));
            ghAdmprops.unwrapTempTable().addNewField(new character("ObjectTranslated"), new character("LOGICAL"), new integer(0), new character(), new logical(false));
            ghAdmprops.unwrapTempTable().addNewField(new character("PopupButtonsInFields"), new character("LOGICAL"), new integer(0), new character(), new logical(false));
            ghAdmprops.unwrapTempTable().addNewField(new character("ColorInfoBG"), new character("INTEGER"), new integer(0), new character(), new integer(10));
            ghAdmprops.unwrapTempTable().addNewField(new character("ColorInfoFG"), new character("INTEGER"), new integer(0), new character(), new unknown());
            ghAdmprops.unwrapTempTable().addNewField(new character("ColorWarnBG"), new character("INTEGER"), new integer(0), new character(), new integer(3));
            ghAdmprops.unwrapTempTable().addNewField(new character("ColorWarnFG"), new character("INTEGER"), new integer(0), new character(), new unknown());
            ghAdmprops.unwrapTempTable().addNewField(new character("ColorErrorBG"), new character("INTEGER"), new integer(0), new character(), new integer(12));
            ghAdmprops.unwrapTempTable().addNewField(new character("ColorErrorFG"), new character("INTEGER"), new integer(0), new character(), new unknown());
            ghAdmprops.unwrapTempTable().addNewField(new character("BGColor"), new character("INTEGER"), new integer(0), new character(), new integer(12));
            ghAdmprops.unwrapTempTable().addNewField(new character("FGColor"), new character("INTEGER"), new integer(0), new character(), new unknown());
            ghAdmprops.unwrapTempTable().addNewField(new character("FieldPopupMapping"), new character("CHARACTER"), new integer(0), new character(), new character(""));
         }
         
         /* _UIB-CODE-BLOCK-END */
         if (_not(handle.fromString(entry(1, thisProcedure().unwrapADMData().getADMData(), chr(1))).isValid()))
         {
            ghAdmprops.unwrapTempTable().addNewField(new character("CurrentPage"), new character("INT"), new integer(0), new character(), new integer(0));
            ghAdmprops.unwrapTempTable().addNewField(new character("PendingPage"), new character("INT"), new integer(0), new character(), new unknown());
            ghAdmprops.unwrapTempTable().addNewField(new character("ContainerTarget"), new character("CHAR"), new integer(0), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("ContainerTargetEvents"), new character("CHAR"), new integer(0), new character(), new character("exitObject,okObject,cancelObject,updateActive"));
            ghAdmprops.unwrapTempTable().addNewField(new character("ContainerToolbarSource"), new character("CHAR"), new integer(0), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("ContainerToolbarSourceEvents"), new character("CHAR"), new integer(0), new character(), new character("toolbar,okObject,cancelObject"));
            ghAdmprops.unwrapTempTable().addNewField(new character("OutMessageTarget"), new character("HANDLE"));
            ghAdmprops.unwrapTempTable().addNewField(new character("PageNTarget"), new character("CHAR"), new integer(0), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("PageSource"), new character("HANDLE"));
            ghAdmprops.unwrapTempTable().addNewField(new character("FilterSource"), new character("HANDLE"));
            ghAdmprops.unwrapTempTable().addNewField(new character("UpdateSource"), new character("CHARACTER"), new integer(0), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("UpdateTarget"), new character("CHAR"), new integer(0), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("CommitSource"), new character("HANDLE"), new integer(0), new character(), new unknown());
            ghAdmprops.unwrapTempTable().addNewField(new character("CommitSourceEvents"), new character("CHAR"), new integer(0), new character(), new character("commitTransaction,undoTransaction"));
            ghAdmprops.unwrapTempTable().addNewField(new character("CommitTarget"), new character("CHAR"), new integer(0), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("CommitTargetEvents"), new character("CHAR"), new integer(0), new character(), new character("rowObjectState"));
            ghAdmprops.unwrapTempTable().addNewField(new character("StartPage"), new character("INT"), new integer(0), new character(), new unknown());
            ghAdmprops.unwrapTempTable().addNewField(new character("RunMultiple"), new character("LOGICAL"), new integer(0), new character(), new logical(false));
            ghAdmprops.unwrapTempTable().addNewField(new character("WaitForObject"), new character("HANDLE"));
            ghAdmprops.unwrapTempTable().addNewField(new character("DynamicSDOProcedure"), new character("CHAR"), new integer(0), new character(), new character("adm2/dyndata.w"));
            ghAdmprops.unwrapTempTable().addNewField(new character("RunDOOptions"), new character("CHARACTER"), new integer(0), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("InitialPageList"), new character("CHARACTER"), new integer(0), new character(), new character(""));
            /* comma delimited list of pages to construct at startup or * for all or empty for just start page and page 0 */
            ghAdmprops.unwrapTempTable().addNewField(new character("WindowFrameHandle"), new character("HANDLE"));
            ghAdmprops.unwrapTempTable().addNewField(new character("Page0LayoutManager"), new character("CHARACTER"));
            ghAdmprops.unwrapTempTable().addNewField(new character("MultiInstanceSupported"), new character("LOGICAL"), new integer(), new character(), new logical(false));
            ghAdmprops.unwrapTempTable().addNewField(new character("MultiInstanceActivated"), new character("LOGICAL"));
            ghAdmprops.unwrapTempTable().addNewField(new character("ContainerMode"), new character("CHARACTER"));
            ghAdmprops.unwrapTempTable().addNewField(new character("SavedContainerMode"), new character("CHARACTER"));
            ghAdmprops.unwrapTempTable().addNewField(new character("SdoForeignFields"), new character("CHARACTER"));
            ghAdmprops.unwrapTempTable().addNewField(new character("NavigationSource"), new character("CHARACTER"));
            ghAdmprops.unwrapTempTable().addNewField(new character("NavigationTarget"), new character("CHARACTER"));
            ghAdmprops.unwrapTempTable().addNewField(new character("PrimarySdoTarget"), new character("HANDLE"));
            ghAdmprops.unwrapTempTable().addNewField(new character("NavigationSourceEvents"), new character("CHAR"), new integer(0), new character(), new character("fetchFirst,fetchNext,fetchPrev,fetchLast,startFilter"));
            ghAdmprops.unwrapTempTable().addNewField(new character("CallerWindow"), new character("HANDLE"));
            ghAdmprops.unwrapTempTable().addNewField(new character("CallerProcedure"), new character("HANDLE"));
            ghAdmprops.unwrapTempTable().addNewField(new character("CallerObject"), new character("HANDLE"));
            ghAdmprops.unwrapTempTable().addNewField(new character("DisabledAddModeTabs"), new character("CHARACTER"));
            ghAdmprops.unwrapTempTable().addNewField(new character("ReEnableDataLinks"), new character("CHARACTER"));
            ghAdmprops.unwrapTempTable().addNewField(new character("WindowTitleViewer"), new character("CHARACTER"));
            ghAdmprops.unwrapTempTable().addNewField(new character("UpdateActive"), new character("LOGICAL"));
            ghAdmprops.unwrapTempTable().addNewField(new character("ObjectsCreated"), new character("LOGICAL"));
            ghAdmprops.unwrapTempTable().addNewField(new character("InstanceNames"), new character("CHARACTER"), new integer(0), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("ClientNames"), new character("CHARACTER"), new integer(0), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("ContainedDataObjects"), new character("CHARACTER"), new integer(0), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("ContainedAppServices"), new character("CHARACTER"), new integer(0), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("DataContainer"), new character("LOGICAL"), new integer(0), new character(), new logical(false));
            ghAdmprops.unwrapTempTable().addNewField(new character("HasDynamicProxy"), new character("LOGICAL"), new integer(0), new character(), new logical(false));
            ghAdmprops.unwrapTempTable().addNewField(new character("HideOnClose"), new character("LOGICAL"), new integer(0), new character(), new logical(false));
            ghAdmprops.unwrapTempTable().addNewField(new character("HideChildContainersOnClose"), new character("LOGICAL"), new integer(0), new character(), new unknown());
            ghAdmprops.unwrapTempTable().addNewField(new character("HasObjectMenu"), new character("LOGICAL"), new integer(0), new character(), new logical(false));
            ghAdmprops.unwrapTempTable().addNewField(new character("RequiredPages"), new character("CHARACTER"));
            ghAdmprops.unwrapTempTable().addNewField(new character("RemoveMenuOnHide"), new character("LOGICAL"), new integer(0), new character(), new logical(true));
         }
         
         /* _UIB-CODE-BLOCK-END */
         /* ********************  Preprocessor Definitions  ******************** */
         /* _UIB-PREPROCESSOR-BLOCK-END */
         /* *********************** Procedure Settings ************************ */
         /* Settings for THIS-PROCEDURE
            Type: Method-Library
            Allow: 
            Frames: 0
            Add Fields to: Neither
            Other Settings: INCLUDE-ONLY
          */
         /* *************************  Create Window  ************************** */
         /* DESIGN Window definition (used by the UIB) 
           CREATE WINDOW Method-Library ASSIGN
                  HEIGHT             = 8.62
                  WIDTH              = 52.
         _* END WINDOW DEFINITION *_
                                                                                 */
         /* ************************* Included-Libraries *********************** */
         /* _UIB-CODE-BLOCK-END */
         /* ***************************  Main Block  *************************** */
         if (_not(glAdmloadFromRepos))
         {
            ControlFlowOps.invokeWithMode("start-super-proc", "I", new character("adm2/smart.p"));
         }
         
         /* Use the old adm definition */
         if (_not(handle.fromString(entry(1, thisProcedure().unwrapADMData().getADMData(), chr(1))).isValid()))
         {
            /* Now create the one record in this property temp-table, and store its
                   handle in ADM-DATA. The CHR(1) delimiters are to set aside spots
                   for UserProperties and UserLinks. */
            ghAdmprops.unwrapTempTable().tempTablePrepare("ADMProps");
            ghAdmpropsBuf.assign(ghAdmprops.unwrapTempTable().defaultBufferHandle());
            ghAdmpropsBuf.unwrapBuffer().bufferCreate();
            /* ... and save it. This ADM object is now open for business! */
            thisProcedure().unwrapADMData().setADMData(concat(valueOf(ghAdmpropsBuf), chr(1), chr(1)));
            /* from this point  {&ADM-PROPS-DEFINED} is true for all cases */
            cObjectName.assign("LogicalObjectName,PhysicalObjectName,DynamicObject,RunAttribute, HideOnInit,DisableOnInit,ObjectLayout");
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("InstanceProperties"));
            ghProp.unwrapBufferField().changeValue(cObjectName);
            cObjectName.assign(replaceAll(thisProcedure().unwrapFile().getFileName(), "\\", "/"));
            iStart.assign(plus(lastIndexOf(cObjectName, "/"), 1));
            cObjectName.assign(substring(cObjectName, iStart, minus(lastIndexOf(thisProcedure().unwrapFile().getFileName(), "."), iStart)));
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("ObjectName"));
            ghProp.unwrapBufferField().changeValue(cObjectName);
         }
         else
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("ObjectType"));
            ghProp.unwrapBufferField().changeValue(new character(""));
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("ContainerType"));
            ghProp.unwrapBufferField().changeValue(new character(""));
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("PhysicalVersion"));
            ghProp.unwrapBufferField().changeValue(new character(""));
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("PhysicalObjectName"));
            ghProp.unwrapBufferField().changeValue((_isNotEqual("", "") ? new character("") : thisProcedure().unwrapFile().getFileName()));
         }
         
         if (_not(glAdmloadFromRepos))
         {
            ControlFlowOps.invokeWithMode("start-super-proc", "I", new character("adm2/visual.p"));
         }
         
         /* _ADM-CODE-BLOCK-START _CUSTOM _INCLUDED-LIB-CUSTOM CUSTOM */
         /*********************************************************************
         * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
         * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
         * below.  All Rights Reserved.                                       *
         *                                                                    *
         * The Initial Developer of the Original Code is PSC.  The Original   *
         * Code is Progress IDE code released to open source December 1, 2000.*
         *                                                                    *
         * The contents of this file are subject to the Possenet Public       *
         * License Version 1.0 (the "License"); you may not use this file     *
         * except in compliance with the License.  A copy of the License is   *
         * available as of the date of this notice at                         *
         * http://www.possenet.org/license.html                               *
         *                                                                    *
         * Software distributed under the License is distributed on an "AS IS"*
         * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
         * should refer to the License for the specific language governing    *
         * rights and limitations under the License.                          *
         *                                                                    *
         * Contributors:                                                      *
         *                                                                    *
         *********************************************************************/
         /*------------------------------------------------------------------------
             File        : visualcustom.i
             Purpose     : References the start of the custom super procedure.
                           Allows properties initialization.
         
             Syntax      : {src/adm2/custom/visualcustom.i}
         
             Description :
         
             Created     : 06/03/1999
             Notes       : Referenced in {src/adm2/visual.i}
           ----------------------------------------------------------------------*/
         /*          This .W file was created with the Progress AppBuilder.      */
         /*----------------------------------------------------------------------*/
         /* ***************************  Definitions  ************************** */
         /* _UIB-CODE-BLOCK-END */
         /* ********************  Preprocessor Definitions  ******************** */
         /* _UIB-PREPROCESSOR-BLOCK-END */
         /* *********************** Procedure Settings ************************ */
         /* Settings for THIS-PROCEDURE
            Type: Include
            Allow: 
            Frames: 0
            Add Fields to: Neither
            Other Settings: INCLUDE-ONLY
          */
         /* *************************  Create Window  ************************** */
         /* DESIGN Window definition (used by the UIB) 
           CREATE WINDOW Include ASSIGN
                  HEIGHT             = 15
                  WIDTH              = 60.
         _* END WINDOW DEFINITION *_
                                                                                 */
         /* ***************************  Main Block  *************************** */
         /* Starts here the custom super procedure 
            Uncomment to run it */
         /*RUN start-super-proc ("adm2/custom/visualcustom.p":U).*/
         /* _UIB-CODE-BLOCK-END */
         /* _ADM-CODE-BLOCK-END */
         /* _UIB-CODE-BLOCK-END */
         /* _UIB-CODE-BLOCK-END */
         /* ***************************  Main Block  *************************** */
         if (_not(glAdmloadFromRepos))
         {
            ControlFlowOps.invokeWithMode("start-super-proc", "I", new character("adm2/containr.p"));
            ControlFlowOps.invokeInWithMode("modifyListProperty", targetProcedure(), "IIII", targetProcedure(), new character("Add"), new character("ContainerSourceEvents"), new character("initializeDataObjects"));
            
            /* {fn <func-name> [handle] */ 
            if (_not(matchesList(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getSupportedLinks", targetProcedure()), "containertoolbar-target")))
            {
               ControlFlowOps.invokeInWithMode("modifyListProperty", targetProcedure(), "IIII", targetProcedure(), new character("Add"), new character("SupportedLinks"), new character("ContainerToolbar-Target"));
            }
         }
         /* if not adm-load-from-repos */
         /* Best default for GUI applications - this will apply to the whole session: */
         pauseBeforeHide(0);
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * ***********************  Function Implementations****************
    */
   public character getObjectType()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Returns the Type of the SmartObject, such as "SmartDataObject", etc.
                  For an ADM Super procedure, this function returns "SUPER".
         Params:  <none>
      ------------------------------------------------------------------------------*/
      character cType = TypeFactory.character();
      
      return function("getObjectType", character.class, new Block((Body) () -> 
      {
         ErrorManager.silentErrorEnable();
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ObjectType"));
         cType.assign(ghProp.unwrapBufferField().value());
         ErrorManager.silentErrorDisable();
         returnNormal(cType);
      }));
   }

   /** *********************  Internal Procedures ********************** */
   /**
    *    Purpose:  Clone the current adm property temp-table<br>
    * Parameters: <br>
    *      Notes:  This is used to clone the dynamic property record for static <br>
    *              objects that does not call deleteProperties from destroyObject<br>
    *              in the target-procedure. This way the properties are kept alive <br>
    *              until destroyObject is completed without creating a memory leak,<br>
    *              since this TT is created in the object's widget-pool.
    */
   public void admCloneProps()
   {
      handle hReposBuffer = TypeFactory.handle();
      handle hPropTable = TypeFactory.handle();
      handle hBuffer = TypeFactory.handle();
      
      internalProcedure(new Block((Body) () -> 
      {
         hReposBuffer.assign(handle.fromString(entry(1, thisProcedure().unwrapADMData().getADMData(), chr(1))));
         
         if (_and(hReposBuffer.isValid(), () -> isNotEqual(hReposBuffer.unwrap().name(), "ADMProps")))
         {
            ErrorManager.silentErrorEnable();
            hReposBuffer.unwrapBuffer().findFirst(concat("WHERE Target = WIDGET-H(\"", valueOf(thisProcedure()), "\")"));
            ErrorManager.silentErrorDisable();
            
            if ((hReposBuffer.unwrapBuffer().available()).booleanValue())
            {
               TempTableBuilder.create(hPropTable);
               hPropTable.unwrapTempTable().createTableLike(hReposBuffer);
               hPropTable.unwrapTempTable().tempTablePrepare("ADMProps");
               hBuffer.assign(hPropTable.unwrapTempTable().defaultBufferHandle());
               hBuffer.unwrapBuffer().bufferCreate();
               hBuffer.unwrapBuffer().bufferCopy(hReposBuffer);
               /* delete the dynamic's property record */
               /* {fn <func-name> [handle] */ 
               ControlFlowOps.invokeDynamicFunctionIn(logical.class, "deleteProperties", targetProcedure());
               /* ... and save it. This ADM object is now open for business! */
               thisProcedure().unwrapADMData().setADMData(concat(valueOf(hBuffer), chr(1), chr(1)));
            }
         }
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     Procedure to start a super proc if it's not already running, <br>
    *              and to add it as a super proc in any case.<br>
    * Parameters:  Procedure name to make super.<br>
    * Notes:       NOTE: This presumes that we want only one copy of an ADM<br>
    *              super procedure running per session, meaning that they are <br>
    *              stateless and "multi-threaded". This is intended to be the case<br>
    *              for ours, but may not be true for all super procs.<br>
    *           -  The LAST-SUPER-PROCEDURE-PROP preprocessor allows classes<br>
    *              to specify a property of char or handle that stores<br>
    *              super-procedures that need to be kept at the bottom of the list <br>
    *              (defined by data.i and sbo.i if they are extended by other classes)
    */
   public void startSuperProc(final character _pcProcName)
   {
      character pcProcName = TypeFactory.initInput(_pcProcName);
      handle hProc = TypeFactory.handle();
      
      internalProcedure(new Block((Body) () -> 
      {
         hProc.assign(ProcedureManager.firstProcedure());
         
         loopLabel0:
         while (_and(hProc.isValid(), () -> isNotEqual(hProc.unwrapFile().getFileName(), pcProcName)))
         {
            hProc.assign(hProc.unwrap().getNextSibling());
         }
         
         if (_not(hProc.isValid()))
         {
            ControlFlowOps.invokePersistentSet((pcProcName).toStringMessage(), hProc);
         }
         
         thisProcedure().unwrapCommonProcedure().addSuperProcedure(hProc, ProcedureManager.SEARCH_TARGET);
         returnNormal();
      }));
   }

   /**
    * Up-call from JavaScript to notify of client disconnect, delete the current procedure, and quit.
    */
   public void inMsgNotifyEhotelDisconnect()
   {
      internalProcedure(new Block((Body) () -> 
      {
         HandleOps.delete(thisProcedure());
         quit();
      }));
   }

   /**
    * Up-call from JavaScript to gather hotel room utilization data for a range of dates.<br>
    * <p>
    * Parameters:<br>
    * <p>
    *    messageId (input)    - Unique message ID for the up-call.<br>
    *    start-date (input)   - Start of date range.<br>
    *    end-date (input)     - End of date range.
    */
   public void inMsgUtilizationByDate(final integer _messageId, final date _startDate, final date _endDate)
   {
      integer messageId = UndoableFactory.initInput(_messageId);
      date startDate = UndoableFactory.initInput(_startDate);
      date endDate = UndoableFactory.initInput(_endDate);
      
      internalProcedure(new Block((Body) () -> 
      {
         ControlFlowOps.invokeWithMode("sendMsg_utilizationByDate", "III", messageId, startDate, endDate);
      }));
   }

   /**
    * Collect hotel room utilization data for a range of dates and supply that data back to<br>
    * JavaScript code as a JSON message, via a remote call.<br>
    * <p>
    * Parameters:<br>
    * <p>
    *    messageId (input)    - Unique message ID for the up-call.<br>
    *    start-date (input)   - Start of date range.<br>
    *    end-date (input)     - End of date range.
    */
   public void sendMsgUtilizationByDate(final integer _messageId, final date _startDate, final date _endDate)
   {
      integer messageId = UndoableFactory.initInput(_messageId);
      date startDate = UndoableFactory.initInput(_startDate);
      date endDate = UndoableFactory.initInput(_endDate);
      date nextDate = TypeFactory.date();
      date d1 = TypeFactory.date();
      date d2 = TypeFactory.date();
      decimal useCount = UndoableFactory.decimal();
      decimal totalRooms = UndoableFactory.decimal(new decimal(0));
      decimal full = UndoableFactory.decimal();
      character jsonMsg = UndoableFactory.character();
      character rowData = UndoableFactory.character();
      logical comma = UndoableFactory.logical(false);
      
      internalProcedure(new Block((Init) () -> 
      {
         full.setPrecision(4);
      }, 
      (Body) () -> 
      {
         AdaptiveQuery query0 = new AdaptiveQuery();
         
         forEach("loopLabel1", new Block((Init) () -> 
         {
            query0.initialize(room, ((String) null), null, "room.roomNum asc", LockType.NONE);
         }, 
         (Body) () -> 
         {
            query0.next();
            totalRooms.assign(plus(totalRooms, 1));
         }));
         
         AdaptiveQuery query1 = new AdaptiveQuery();
         
         forEach("loopLabel2", new Block((Init) () -> 
         {
            query1.initialize(reservation, "reservation.startDate <= ? and reservation.endDate > ?", null, "reservation.roomNum asc, reservation.startDate asc, reservation.endDate asc", new Object[]
            {
               endDate,
               startDate
            }, LockType.NONE);
         }, 
         (Body) () -> 
         {
            /* collect reservations; include check-in date, exclude check-out date */
            query1.next();
            d1.assign((_isGreaterThan(reservation.getStartDate(), startDate) ? reservation.getStartDate() : startDate));
            d2.assign((_isLessThanOrEqual(reservation.getEndDate(), endDate) ? date.minusDays(reservation.getEndDate(), 1) : endDate));
            
            loopLabel3:
            for (nextDate.assign(d1); _isLessThanOrEqual(nextDate, d2); nextDate.increment())
            {
               if (_not(new FindQuery(ttRoomsUsed, "ttRoomsUsed.useDate = ? and ttRoomsUsed.roomNum = ?", null, "ttRoomsUsed.useDate asc, ttRoomsUsed.roomNum asc", new Object[]
               {
                  nextDate,
                  (P2JQuery.Parameter) () -> reservation.getRoomNum()
               }, LockType.NONE).hasOne()))
               {
                  ttRoomsUsed.create();
                  RecordBuffer.startBatch();
                  ttRoomsUsed.setUseDate(nextDate);
                  ttRoomsUsed.setRoomNum(reservation.getRoomNum());
                  RecordBuffer.endBatch();
               }
            }
         }));
         
         AdaptiveQuery query2 = new AdaptiveQuery();
         
         forEach("loopLabel4", new Block((Init) () -> 
         {
            query2.initialize(stay, "stay.startDate <= ? and stay.endDate > ?", null, "stay.stayId asc", new Object[]
            {
               endDate,
               startDate
            }, LockType.NONE);
         }, 
         (Body) () -> 
         {
            /* collect stays; include check-in date, exclude check-out date */
            query2.next();
            d1.assign((_isGreaterThan(stay.getStartDate(), startDate) ? stay.getStartDate() : startDate));
            d2.assign((_isLessThanOrEqual(stay.getEndDate(), endDate) ? date.minusDays(stay.getEndDate(), 1) : endDate));
            
            loopLabel5:
            for (nextDate.assign(d1); _isLessThanOrEqual(nextDate, d2); nextDate.increment())
            {
               if (_not(new FindQuery(ttRoomsUsed, "ttRoomsUsed.useDate = ? and ttRoomsUsed.roomNum = ?", null, "ttRoomsUsed.useDate asc, ttRoomsUsed.roomNum asc", new Object[]
               {
                  nextDate,
                  (P2JQuery.Parameter) () -> stay.getRoomNum()
               }, LockType.NONE).hasOne()))
               {
                  ttRoomsUsed.create();
                  RecordBuffer.startBatch();
                  ttRoomsUsed.setUseDate(nextDate);
                  ttRoomsUsed.setRoomNum(stay.getRoomNum());
                  RecordBuffer.endBatch();
               }
            }
         }));
         
         PresortQuery query3 = new PresortQuery();
         FieldReference byExpr0 = new FieldReference(ttRoomsUsed, "useDate");
         
         forEach("loopLabel6", new Block((Init) () -> 
         {
            query3.initialize(ttRoomsUsed, ((String) null), null, "ttRoomsUsed.useDate asc, ttRoomsUsed.roomNum asc");
            query3.enableBreakGroups();
            query3.setNonScrolling();
            query3.addSortCriterion(byExpr0);
         }, 
         (Body) () -> 
         {
            /* count up used rooms by date */
            query3.next();
            
            if ((query3.isFirstOfGroup(byExpr0)).booleanValue())
            {
               useCount.assign(0);
            }
            
            useCount.assign(plus(useCount, 1));
            
            if ((query3.isLastOfGroup(byExpr0)).booleanValue())
            {
               /* treat overbookings as fully reserved */
               if (_isGreaterThan(useCount, totalRooms))
               {
                  useCount.assign(totalRooms);
               }
               
               rowData.assign(concat(rowData, ((comma).booleanValue() ? new character(",\n") : new character(""))));
               full.assign(divide(useCount, totalRooms));
               rowData.assign(concat(rowData, "{"));
               rowData.assign(concat(rowData, "\"date\":\"", valueOf(ttRoomsUsed.getUseDate(), "99/99/9999"), "\","));
               rowData.assign(concat(rowData, "\"score\":", valueOf(full, "z9.9999")));
               rowData.assign(concat(rowData, "}"));
               comma.assign(true);
            }
         }));
         
         ErrorManager.silentErrorEnable();
         ttRoomsUsed.deleteAll();
         ErrorManager.silentErrorDisable();
         jsonMsg.assign("{\n");
         jsonMsg.assign(concat(jsonMsg, " \"api\":\"utilizationByDateResponse\",\n"));
         jsonMsg.assign(concat(jsonMsg, " \"messageId\":\"", valueOf(messageId), "\",\n"));
         jsonMsg.assign(concat(jsonMsg, " \"rows\": [", rowData, "],\n"));
         jsonMsg.assign(concat(jsonMsg, " \"domain\": [ 0.0, 1.0 ],\n"));
         jsonMsg.assign(concat(jsonMsg, " \"start\":\"", valueOf(startDate, "99/99/9999"), "\",\n"));
         jsonMsg.assign(concat(jsonMsg, " \"end\":\"", valueOf(endDate, "99/99/9999"), "\"\n"));
         jsonMsg.assign(concat(jsonMsg, "}"));
         ControlFlowOps.remoteCall(jsonMsg);
      }));
   }

   /**
    * Up-call from JavaScript to gather hotel room utilization data, grouped by room type, for a<br>
    * range of dates.<br>
    * <p>
    * Parameters:<br>
    * <p>
    *    messageId (input)    - Unique message ID for the up-call.<br>
    *    start-date (input)   - Start of date range.<br>
    *    end-date (input)     - End of date range.
    */
   public void inMsgUtilizationByType(final integer _messageId, final date _startDate, final date _endDate)
   {
      integer messageId = UndoableFactory.initInput(_messageId);
      date startDate = UndoableFactory.initInput(_startDate);
      date endDate = UndoableFactory.initInput(_endDate);
      
      internalProcedure(new Block((Body) () -> 
      {
         ControlFlowOps.invokeWithMode("sendMsg_utilizationByType", "III", messageId, startDate, endDate);
      }));
   }

   /**
    * Collect hotel room utilization data, grouped by room type, for a range of dates. Supply that<br>
    * data back to JavaScript code as a JSON message, via a remote call.<br>
    * <p>
    * Parameters:<br>
    * <p>
    *    messageId (input)    - Unique message ID for the up-call.<br>
    *    start-date (input)   - Start of date range.<br>
    *    end-date (input)     - End of date range.
    */
   public void sendMsgUtilizationByType(final integer _messageId, final date _startDate, final date _endDate)
   {
      integer messageId = UndoableFactory.initInput(_messageId);
      date startDate = UndoableFactory.initInput(_startDate);
      date endDate = UndoableFactory.initInput(_endDate);
      character jsonMsg = TypeFactory.character();
      character msg = TypeFactory.character("");
      decimal weight = TypeFactory.decimal();
      decimal score = TypeFactory.decimal();
      integer totalRoomCount = TypeFactory.integer((long) 0);
      decimal actualCount = TypeFactory.decimal();
      decimal revenue = TypeFactory.decimal();
      decimal totalPotRev = TypeFactory.decimal(new decimal(0));
      decimal actualRevenue = TypeFactory.decimal();
      date d1 = TypeFactory.date();
      date d2 = TypeFactory.date();
      integer dayCount = TypeFactory.integer();
      date nextDate = TypeFactory.date();
      logical comma = TypeFactory.logical(false);
      logical rateWeighted = TypeFactory.logical(true);
      decimal r1 = TypeFactory.decimal();
      
      internalProcedure(new Block((Init) () -> 
      {
         totalPotRev.setPrecision(2);
         actualRevenue.setPrecision(2);
         r1.setPrecision(2);
      }, 
      (Body) () -> 
      {
         PresortQuery query4 = new PresortQuery();
         FieldReference byExpr1 = new FieldReference(room, "roomType");
         FieldReference fieldRef0 = new FieldReference(room, "roomNum");
         final CountAccumulator accumCount0 = new CountAccumulator();
         
         forEach("loopLabel7", new Block((Init) () -> 
         {
            query4.initialize();
            query4.enableBreakGroups();
            query4.setNonScrolling();
            query4.addComponent(roomType, ((String) null), LockType.NONE);
            query4.addComponent(room, "room.roomType = roomType.roomType and room.active = true", LockType.NONE);
            query4.addSortCriterion(byExpr1);
            query4.addAccumulator(accumCount0);
            accumCount0.setSubOnly();
            accumCount0.addBreakGroup(byExpr1);
            accumCount0.reset();
         }, 
         (Body) () -> 
         {
            /* count the number of active rooms of each type */
            query4.next();
            accumCount0.accumulate();
            
            if ((query4.isLastOfGroup(byExpr1)).booleanValue())
            {
               ttRoomTypeAgg.create();
               RecordBuffer.startBatch();
               ttRoomTypeAgg.setRoomType(room.getRoomType());
               ttRoomTypeAgg.setCount(accumCount0.getResult(byExpr1));
               ttRoomTypeAgg.setDescription(roomType.getDescription());
               RecordBuffer.endBatch();
               totalRoomCount.assign(plus(totalRoomCount, ttRoomTypeAgg.getCount()));
            }
         }));
         
         /* get total count of rooms that could potentially be rented for the period */
         dayCount.assign(plus(date.differenceNum(endDate, startDate), 1));
         totalRoomCount.assign(multiply(totalRoomCount, dayCount));
         
         CompoundQuery query5 = new CompoundQuery();
         
         forEach("loopLabel8", new Block((Init) () -> 
         {
            query5.initialize(false, false);
            query5.addComponent(new AdaptiveQuery().initialize(ttRoomTypeAgg, ((String) null), null, "ttRoomTypeAgg.roomType asc"));
            query5.addComponent(new AdaptiveQuery().initialize(rate, "rate.roomType = ? and rate.startDate <= ? and rate.endDate >= ?", null, "rate.roomType asc, rate.startDate asc, rate.endDate asc", new Object[]
            {
               new FieldReference(ttRoomTypeAgg, "roomType"),
               endDate,
               startDate
            }, LockType.NONE));
         }, 
         (Body) () -> 
         {
            /* calculate potential revenue for the period by room type using the associated room rates */
            /* note: this algorithm assumes rates do not overlap! */
            query5.iterate();
            d1.assign((_isGreaterThan(rate.getStartDate(), startDate) ? rate.getStartDate() : startDate));
            d2.assign((_isLessThanOrEqual(rate.getEndDate(), endDate) ? rate.getEndDate() : endDate));
            dayCount.assign(plus(date.differenceNum(d2, d1), 1));
            revenue.assign(multiply(multiply(dayCount, rate.getRate()), ttRoomTypeAgg.getCount()));
            ttRoomTypeAgg.setPotentialRevenue(plus(ttRoomTypeAgg.getPotentialRevenue(), revenue));
            totalPotRev.assign(plus(totalPotRev, revenue));
         }));
         
         CompoundQuery query6 = new CompoundQuery();
         
         forEach("loopLabel9", new Block((Init) () -> 
         {
            query6.initialize(false, false);
            query6.addComponent(new AdaptiveQuery().initialize(room, "room.active = true", null, "room.roomNum asc", LockType.NONE));
            query6.addComponent(new AdaptiveQuery().initialize(reservation, "reservation.roomNum = ? and reservation.startDate <= ? and reservation.endDate > ?", null, "reservation.roomNum asc, reservation.startDate asc, reservation.endDate asc", new Object[]
            {
               new FieldReference(room, "roomNum"),
               endDate,
               startDate
            }, LockType.NONE));
         }, 
         (Body) () -> 
         {
            /* collect daily room utilization data for each reservation for the period */
            query6.iterate();
            d1.assign((_isGreaterThan(reservation.getStartDate(), startDate) ? reservation.getStartDate() : startDate));
            d2.assign((_isLessThanOrEqual(reservation.getEndDate(), endDate) ? date.minusDays(reservation.getEndDate(), 1) : endDate));
            
            loopLabel10:
            for (nextDate.assign(d1); _isLessThanOrEqual(nextDate, d2); nextDate.increment())
            {
               r1.assign(0);
               
               if ((rateWeighted).booleanValue())
               {
                  ErrorManager.silentErrorEnable();
                  new FindQuery(rate, "rate.roomType = ? and rate.startDate <= ? and rate.endDate >= ?", null, "rate.roomType asc, rate.startDate asc, rate.endDate asc", new Object[]
                  {
                     room.getRoomType(),
                     nextDate,
                     nextDate
                  }, LockType.NONE).first();
                  ErrorManager.silentErrorDisable();
                  
                  if (rate._available())
                  {
                     r1.assign(rate.getRate());
                  }
                  else
                  {
                     rateWeighted.assign(false);
                  }
               }
               ttRoomsUsed.create();
               RecordBuffer.startBatch();
               ttRoomsUsed.setUseDate(nextDate);
               ttRoomsUsed.setRoomNum(reservation.getRoomNum());
               ttRoomsUsed.setRoomType(room.getRoomType());
               ttRoomsUsed.setRate(r1);
               RecordBuffer.endBatch();
            }
         }));
         
         CompoundQuery query7 = new CompoundQuery();
         
         forEach("loopLabel11", new Block((Init) () -> 
         {
            query7.initialize(false, false);
            query7.addComponent(new AdaptiveQuery().initialize(room, "room.active = true", null, "room.roomNum asc", LockType.NONE));
            query7.addComponent(new AdaptiveQuery().initialize(stay, "stay.roomNum = ? and stay.startDate <= ? and stay.endDate > ?", null, "stay.stayId asc", new Object[]
            {
               new FieldReference(room, "roomNum"),
               endDate,
               startDate
            }, LockType.NONE));
         }, 
         (Body) () -> 
         {
            /* collect daily room utilization data for each stay for the period */
            query7.iterate();
            d1.assign((_isGreaterThan(stay.getStartDate(), startDate) ? stay.getStartDate() : startDate));
            d2.assign((_isLessThanOrEqual(stay.getEndDate(), endDate) ? date.minusDays(stay.getEndDate(), 1) : endDate));
            
            loopLabel12:
            for (nextDate.assign(d1); _isLessThanOrEqual(nextDate, d2); nextDate.increment())
            {
               r1.assign(0);
               
               if ((rateWeighted).booleanValue())
               {
                  ErrorManager.silentErrorEnable();
                  new FindQuery(rate, "rate.roomType = ? and rate.startDate <= ? and rate.endDate >= ?", null, "rate.roomType asc, rate.startDate asc, rate.endDate asc", new Object[]
                  {
                     room.getRoomType(),
                     nextDate,
                     nextDate
                  }, LockType.NONE).first();
                  ErrorManager.silentErrorDisable();
                  
                  if (rate._available())
                  {
                     r1.assign(rate.getRate());
                  }
                  else
                  {
                     rateWeighted.assign(false);
                  }
               }
               ttRoomsUsed.create();
               RecordBuffer.startBatch();
               ttRoomsUsed.setUseDate(nextDate);
               ttRoomsUsed.setRoomNum(stay.getRoomNum());
               ttRoomsUsed.setRoomType(room.getRoomType());
               ttRoomsUsed.setRate(r1);
               RecordBuffer.endBatch();
            }
         }));
         
         AdaptiveQuery query8 = new AdaptiveQuery();
         
         forEach("loopLabel13", new Block((Init) () -> 
         {
            query8.initialize(ttRoomTypeAgg, ((String) null), null, "ttRoomTypeAgg.roomType asc");
         }, 
         (Body) () -> 
         {
            /* walk the collected, aggregate data and compose summary data for each room type in JSON */
            query8.next();
            actualRevenue.assign(0);
            actualCount.assign(0);
            
            AdaptiveQuery query9 = new AdaptiveQuery();
            
            forEach("loopLabel14", new Block((Init) () -> 
            {
               query9.initialize(ttRoomsUsed, "ttRoomsUsed.roomType = ?", null, "ttRoomsUsed.roomType asc", new Object[]
               {
                  (P2JQuery.Parameter) () -> ttRoomTypeAgg.getRoomType()
               });
            }, 
            (Body) () -> 
            {
               query9.next();
               actualRevenue.assign(plus(actualRevenue, ttRoomsUsed.getRate()));
               actualCount.assign(plus(actualCount, 1));
            }));
            
            if ((rateWeighted).booleanValue())
            {
               weight.assign(multiply(divide(ttRoomTypeAgg.getPotentialRevenue(), totalPotRev), 100));
               score.assign(multiply(divide(actualRevenue, ttRoomTypeAgg.getPotentialRevenue()), 100));
            }
            else
            {
               weight.assign(multiply(divide(ttRoomTypeAgg.getCount(), totalRoomCount), 100));
               score.assign(multiply(divide(actualCount, multiply(ttRoomTypeAgg.getCount(), dayCount)), 100));
            }
            
            msg.assign(concat(msg, ((comma).booleanValue() ? new character(",\n") : new character(""))));
            msg.assign(concat(msg, "{"));
            msg.assign(concat(msg, "\"id\":\"", ttRoomTypeAgg.getRoomType(), "\","));
            msg.assign(concat(msg, "\"score\":\"", valueOf(score, "zz9.99"), "\","));
            msg.assign(concat(msg, "\"weight\":\"", valueOf(weight, "zz9.99"), "\","));
            msg.assign(concat(msg, "\"actualRevenue\":\"", actualRevenue, "\","));
            msg.assign(concat(msg, "\"potentialRevenue\":\"", ttRoomTypeAgg.getPotentialRevenue(), "\","));
            msg.assign(concat(msg, "\"roomsOccupied\":\"", valueOf(actualCount, "zzzzz9"), "\","));
            msg.assign(concat(msg, "\"totalRooms\":\"", valueOf(multiply(ttRoomTypeAgg.getCount(), dayCount), "zzzzz9"), "\","));
            msg.assign(concat(msg, "\"label\":\"", ttRoomTypeAgg.getDescription(), "\""));
            msg.assign(concat(msg, "}"));
            comma.assign(true);
         }));
         
         jsonMsg.assign("{\n");
         jsonMsg.assign(concat(jsonMsg, " \"api\":\"utilizationByTypeResponse\",\n"));
         jsonMsg.assign(concat(jsonMsg, " \"messageId\":\"", valueOf(messageId), "\",\n"));
         jsonMsg.assign(concat(jsonMsg, " \"rows\": [\n", msg, "],\n"));
         jsonMsg.assign(concat(jsonMsg, " \"rateWeighted\":\"", ((rateWeighted).booleanValue() ? new character("true") : new character("false")), "\",\n"));
         jsonMsg.assign(concat(jsonMsg, " \"start\":\"", valueOf(startDate, "99/99/9999"), "\",\n"));
         jsonMsg.assign(concat(jsonMsg, " \"end\":\"", valueOf(endDate, "99/99/9999"), "\"\n"));
         jsonMsg.assign(concat(jsonMsg, "}"));
         ttRoomTypeAgg.deleteAll();
         ttRoomsUsed.deleteAll();
         ControlFlowOps.remoteCall(jsonMsg);
      }));
   }

   /**
    * Display an embedded ADM2 window in an iframe.<br>
    * <p>
    * Parameters:<br>
    * <p>
    *    pname (input)  -  Name of external procedure which loads the desired window.
    */
   public void showEmbeddedWindow(final character _pname)
   {
      character pname = UndoableFactory.initInput(_pname);
      
      internalProcedure(new Block((Body) () -> 
      {
         ControlFlowOps.invokeWithMode("constructObject", "IIIO", pname, wWin, new character("HideOnInitnoDisableOnInitnoObjectLayout"), hpage);
         ErrorManager.silentErrorEnable();
         ControlFlowOps.invokeInWithMode("repositionObject", hpage, "II", new integer(1), new integer(1));
         ErrorManager.silentErrorDisable();
         ControlFlowOps.invokeIn("initializeObject", hpage);
      }));
   }

   /**
    * Up-call from JavaScript which executes a command to display a particular, embedded ADM2 window<br>
    * in an iframe.<br>
    * <p>
    * Parameters:<br>
    * <p>
    *    messageId (input)    -  Unique message ID for the up-call.<br>
    *    command (input)      -  Identifying string associated with the external procedure which<br>
    *                            loads the window.
    */
   public void inMsgExecuteCommand(final integer _messageId, final character _command)
   {
      integer messageId = UndoableFactory.initInput(_messageId);
      character command = UndoableFactory.initInput(_command);
      
      internalProcedure(new Block((Body) () -> 
      {
         if (_notUnknown(hpage))
         {
            ControlFlowOps.invokeIn("destroyObject", hpage);
         }
         
         switch (StringHelper.safeTrimTrailing((command).toStringMessage()).toUpperCase())
         {
            case "SHOWAVAILROOMS":
               {
                  ControlFlowOps.invokeWithMode("showEmbeddedWindow", "I", new character("avail-rooms-frame.w"));
               }
               break;
            case "SHOWGUESTS":
               {
                  ControlFlowOps.invokeWithMode("showEmbeddedWindow", "I", new character("guests-frame.w"));
               }
               break;
            case "SHOWRESERVATIONS":
               {
                  ControlFlowOps.invokeWithMode("showEmbeddedWindow", "I", new character("reservations-frame.w"));
               }
               break;
            case "SHOWRATES":
               {
                  ControlFlowOps.invokeWithMode("showEmbeddedWindow", "I", new character("rates-frame.w"));
               }
               break;
            case "SHOWROOMS":
               {
                  ControlFlowOps.invokeWithMode("showEmbeddedWindow", "I", new character("rooms-frame.w"));
               }
               break;
            default:
               {
                  messageBox(new Object[]
                  {
                     "Invalid command:",
                     command
                  }, ALERT_MESSAGE, BTN_OK, (String) null);
               }
               break;
         }
      }));
   }
}
