/*
 * Copyright 2016-2017 Golden Code Development Corporation
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this software
 * and associated documentation files (the "Software"),  to deal in the Software without
 * restriction, including without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all copies or
 * substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
 * BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

package com.goldencode.hotel;

import com.goldencode.p2j.util.*;
import com.goldencode.p2j.persist.*;
import com.goldencode.hotel.dmo._meta.*;
import com.goldencode.util.*;
import com.goldencode.p2j.persist.lock.*;

import static com.goldencode.p2j.util.BlockManager.*;
import static com.goldencode.p2j.util.MathOps.*;
import static com.goldencode.p2j.util.CompareOps.*;
import static com.goldencode.p2j.util.logical.*;
import static com.goldencode.p2j.util.TextOps.*;
import static com.goldencode.p2j.util.character.*;

/**
 * Business logic (converted to Java from the 4GL source code
 * in export-data.p).
 */
public class ExportData
{
   MetaFile.Buf metaFile = RecordBuffer.define(MetaFile.Buf.class, "hotel", "metaFile", "_file");

   Stream dumpStreamStream = new StreamWrapper("dump-stream");

   character ldb = TypeFactory.character();

   integer recs = TypeFactory.integer();

   datetime dtnow = TypeFactory.datetime();

   /**
    * External procedure (converted to Java from the 4GL source code
    * in export-data.p).
    */
   public void execute()
   {
      handle hQry = TypeFactory.handle();
      handle hBuf = TypeFactory.handle();
      handle hFld = TypeFactory.handle();
      character xDir = TypeFactory.character("./export");
      character delim = TypeFactory.character(" ");
      character line = TypeFactory.character();
      integer num = TypeFactory.integer();
      integer ext = TypeFactory.integer();
      integer idx = TypeFactory.integer();
      
      externalProcedure(ExportData.this, new Block((Body) () -> 
      {
         TransactionManager.registerTopLevelFinalizable(dumpStreamStream, true);
         /* Main Block */
         /* delete any previous version of the export directory, then create a fresh one */
         FileSystemOps.delete(new String[]
         {
            (xDir).toStringMessage()
         }, true);
         FileSystemOps.mkdir(new String[]
         {
            (xDir).toStringMessage()
         });
         
         AdaptiveQuery query0 = new AdaptiveQuery();
         
         forEach("loopLabel0", new Block((Init) () -> 
         {
            RecordBuffer.openScope(metaFile);
            query0.initialize(metaFile, "upper(metaFile.owner) = 'PUB' and not (upper(metaFile.fileName) like '\\_%')", null, "metaFile.fileName asc", LockType.NONE);
         }, 
         (Body) () -> 
         {
            /* loop through all public tables and dump each one to a .d file */
            query0.next();
            dumpStreamStream.assign(StreamFactory.openFileStream((concat(xDir, "/", metaFile.getDumpName(), ".d")).toStringMessage(), true, false));
            RecordBuffer.create(hBuf, metaFile.getFileName());
            QueryWrapper.createQuery(hQry);
            ldb.assign(hBuf.unwrapDatabaseInfo().getDbName());
            hQry.unwrapQuery().setBuffers(hBuf);
            hQry.unwrapQuery().prepare(concat("for each ", metaFile.getFileName()));
            hQry.unwrapQuery().queryOpen();
            hQry.unwrapQuery().getFirst();
            recs.assign(0);
            
            loopLabel1:
            while (_not(hQry.unwrapQuery().isOffEnd()))
            {
               line.assign("");
               
               IntegerExpression resExpr0 = new IntegerExpression()
               {
                  public integer execute()
                  {
                     return hBuf.unwrapBuffer().numFields();
                  }
               };
               ToClause toClause0 = new ToClause(num, 1, resExpr0);
               
               doTo("loopLabel2", toClause0, new Block((Body) () -> 
               {
                  hFld.assign(hBuf.unwrapBuffer().bufferField(num));
                  ext.assign(hFld.unwrapBufferField().extent());
                  
                  if (_isEqual(ext, 0))
                  {
                     line.assign(concat(line, processValue(hFld, new integer(0), delim)));
                  }
                  else
                  {
                     ToClause toClause1 = new ToClause(idx, 1, ext);
                     
                     doTo("loopLabel3", toClause1, new Block((Body) () -> 
                     {
                        line.assign(concat(line, processValue(hFld, idx, delim)));
                     }));
                  }
               }));
               
               /* right-trim result */
               line.assign(rightTrim(line, delim));
               dumpStreamStream.putUnformatted(new FieldEntry[]
               {
                  new PutField(line),
                  new SkipField()
               });
               recs.assign(plus(recs, 1));
               hQry.unwrapQuery().getNext();
            }
            
            /* write export file footer */
            dtnow.assign(datetimetz.now());
            dumpStreamStream.putUnformatted(new FieldEntry[]
            {
               new PutField("."),
               new SkipField(),
               new PutField("PSC"),
               new SkipField(),
               new PutField(() -> concat("filename=", metaFile.getFileName())),
               new SkipField(),
               new PutField(() -> concat("records=", valueOf(recs, "9999999999999"))),
               new SkipField(),
               new PutField(() -> concat("ldbname=", ldb)),
               new SkipField(),
               new PutField(() -> concat("timestamp=", valueOf(date.year(dtnow), "9999"), "/", valueOf(date.month(dtnow), "99"), "/", valueOf(date.day(dtnow), "99"), "-", valueOf(new integer(divide(datetime.millisecondsSinceMidnight(dtnow), 1000)), "hh:mm:ss"))),
               new SkipField(),
               new PutField(() -> concat("numformat=", valueOf(asc(NumberType.getNumericSeparator())), ",", valueOf(asc(NumberType.getNumericDecimalPoint())))),
               new SkipField(),
               new PutField(() -> concat("dateformat=", SessionUtils.getDateFormat(), "-", valueOf(SessionUtils.getYearOffset()))),
               new SkipField(),
               new PutField("map=NO-MAP"),
               new SkipField(),
               new PutField(() -> concat("cpstream=", EnvironmentOps.getCPStream())),
               new SkipField(),
               new PutField("."),
               new SkipField(),
               new PutField("0000000000"),
               new SkipField()
            });
            dumpStreamStream.closeOut();
            hQry.unwrapQuery().queryClose();
         }));
         
         quit();
      }));
   }

   /**
    * Purpose:    Process a single scalar or extent value from a buffer field.<br>
    * Parameters: hFld<br>
    *             Handle to the buffer field.<br>
    *             idx<br>
    *             1-based index of the field value for an extent field; 0 for a scalar field.<br>
    *             delim<br>
    *             Delimiter character.<br>
    * <p>
    * Returns:    Value, formatted for export, followed by delimiter character.
    */
   public character processValue(final handle _hFld_1, final integer _idx_1, final character _delim_1)
   {
      handle hFld_1 = TypeFactory.initInput(_hFld_1);
      integer idx_1 = TypeFactory.initInput(_idx_1);
      character delim_1 = TypeFactory.initInput(_delim_1);
      character result = TypeFactory.character();
      
      return function("process-value", character.class, new Block((Body) () -> 
      {
         if (_isEqual(idx_1, 0))
         {
            if (_isUnknown(hFld_1.unwrapBufferField().value()))
            {
               result.assign("?");
            }
            else
            {
               switch (StringHelper.safeTrimTrailing((hFld_1.unwrapCommonField().getDataType()).toStringMessage()))
               {
                  case "character":
                     {
                        result.assign(character.quoter(hFld_1.unwrapBufferField().value()));
                     }
                     break;
                  default:
                     {
                        result.assign(valueOf(hFld_1.unwrapBufferField().value()));
                     }
                     break;
               }
            }
         }
         else
         {
            if (_isUnknown(hFld_1.unwrapBufferField().value(idx_1)))
            {
               result.assign("?");
            }
            else
            {
               switch (StringHelper.safeTrimTrailing((hFld_1.unwrapCommonField().getDataType()).toStringMessage()))
               {
                  case "character":
                     {
                        result.assign(character.quoter(hFld_1.unwrapBufferField().value(idx_1)));
                     }
                     break;
                  default:
                     {
                        result.assign(valueOf(hFld_1.unwrapBufferField().value(idx_1)));
                     }
                     break;
               }
            }
         }
         /* scalar field */
         returnNormal(concat(result, delim_1));
      }));
   }
}
