/*--------------------------------------------------------------------------
    File        : adecomm/AsUtils.java
    Purpose     : Appserver Utilities

    Syntax      :

    Description : Utilities to aid in the deployment of distributed applications

    Author(s)   : Ross Hunter
    Created     : 12/01/97
    Notes       :
    
    Modified    : 
      05/10/2000  - BSG - JMS Support. Multiple Partition Types.
      06/01/2000  - GFS - Fixed problem with not passing URLs
  ------------------------------------------------------------------------*/

package com.goldencode.hotel.common.adecomm;

import com.goldencode.p2j.util.*;
import com.goldencode.p2j.persist.*;
import com.goldencode.hotel.dmo._temp.*;
import com.goldencode.p2j.ui.*;
import com.goldencode.hotel.ui.common.adecomm.*;
import com.goldencode.p2j.persist.lock.*;

import static com.goldencode.p2j.util.BlockManager.*;
import static com.goldencode.p2j.util.TextOps.*;
import static com.goldencode.p2j.util.MathOps.*;
import static com.goldencode.p2j.util.CompareOps.*;
import static com.goldencode.p2j.util.logical.*;
import static com.goldencode.p2j.util.character.*;
import static com.goldencode.p2j.ui.LogicalTerminal.*;

/**
 * Business logic (converted to Java from the 4GL source code
 * in common/adecomm/as-utils.w).
 */
public class AsUtils
{
   public static final String FMT_STR_1 = "X(256)";

   /*********************************************************************
   * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
   * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
   * below.  All Rights Reserved.                                       *
   *                                                                    *
   * The Initial Developer of the Original Code is PSC.  The Original   *
   * Code is Progress IDE code released to open source December 1, 2000.*
   *                                                                    *
   * The contents of this file are subject to the Possenet Public       *
   * License Version 1.0 (the "License"); you may not use this file     *
   * except in compliance with the License.  A copy of the License is   *
   * available as of the date of this notice at                         *
   * http://www.possenet.org/license.html                               *
   *                                                                    *
   * Software distributed under the License is distributed on an "AS IS"*
   * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
   * should refer to the License for the specific language governing    *
   * rights and limitations under the License.                          *
   *                                                                    *
   * Contributors:                                                      *
   *                                                                    *
   *********************************************************************/   
   /*          This .W file was created with the Progress UIB.             */   
   /*----------------------------------------------------------------------*/   
   /* ***************************  Definitions  ************************** */   
   /*********************************************************************
   * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
   * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
   * below.  All Rights Reserved.                                       *
   *                                                                    *
   * The Initial Developer of the Original Code is PSC.  The Original   *
   * Code is Progress IDE code released to open source December 1, 2000.*
   *                                                                    *
   * The contents of this file are subject to the Possenet Public       *
   * License Version 1.0 (the "License"); you may not use this file     *
   * except in compliance with the License.  A copy of the License is   *
   * available as of the date of this notice at                         *
   * http://www.possenet.org/license.html                               *
   *                                                                    *
   * Software distributed under the License is distributed on an "AS IS"*
   * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
   * should refer to the License for the specific language governing    *
   * rights and limitations under the License.                          *
   *                                                                    *
   * Contributors:                                                      *
   *                                                                    *
   *********************************************************************/   
   /* adecomm/appsrvtt.i Contains the definition for the AppServer Service
                         temp-table
   
      This definition is declared "NEW GLOBAL" in adecomm/as-utils.w which is 
      called once per PROGRESS session.  adecomm/as-utils is also responsible
      for loading the records when it is called.  protools/as-partn.w is
      responsible for maintaining and dumping the record in the table.  Any
      procedure needing to connect or disconnect needs to call methods in
      adecomm/as-utils.w to lookup information in AppSrv-TT.
      
      Created 11/26/97 by Ross Hunter                                          
      Modified:
        05/09/2000 BSG  Support for multiple partition types and Server URLs */   
   AppsrvTt_1_1.Buf appsrvTt = SharedVariableManager.addTempTable(ScopeLevel.GLOBAL, "appsrvTt", AppsrvTt_1_1.Buf.class, "appsrvTt", "AppSrv-TT");

   character usrId = TypeFactory.character();

   character psswrd = TypeFactory.character();

   AsUtilsSecurityPromptSecurityDialog securityPromptSecurityDialogFrame = null;

   AppsrvTt_1_1.Buf bappsrvtt;

   /**
    * External procedure (converted to Java from the 4GL source code
    * in common/adecomm/as-utils.w).
    */
   public void execute()
   {
      externalProcedure(AsUtils.this, new Block((Body) () -> 
      {
         RecordBuffer.openScope(appsrvTt);
         
         /* _UIB-CODE-BLOCK-END */
         /* *********************** Procedure Settings ************************ */
         /* Settings for THIS-PROCEDURE
            Type: Procedure
            Allow: 
            Frames: 0
            Add Fields to: Neither
            Other Settings: CODE-ONLY COMPILE
          */
         /* *************************  Create Window  ************************** */
         /* DESIGN Window definition (used by the UIB) 
           CREATE WINDOW Procedure ASSIGN
                  HEIGHT             = 22.81
                  WIDTH              = 62.2.
         _* END WINDOW DEFINITION *_
                                                                                 */
         /* ***************************  Main Block  *************************** */
         /* The first thing to do is load up the service records into the
            AppSrv-TT.  This is only done here and this adecomm/as-utils.w
            should only be called once per session so there never should be
            any records already there.  But it is cheap to check it anyway.     */
         if ((new FindQuery(appsrvTt, (String) null, null, "appsrvTt.partition asc", LockType.NONE).hasAny()).booleanValue())
         {
            messageBox("The Application Server Service Table has been previously defined.", ALERT_ERROR, BTN_OK, (String) null);
         }
         else
         {
            ControlFlowOps.invoke("loadPartitionInfo");
         }
      }));
   }

   /**
    * *************************************************************************<br>
    *     Procedure:  _adetool.i<br>
    *                                        <br>
    *     Purpose:    Defines an internal procedure called 'ADEPersistent'. This<br>
    *                 should be included in any Persistent procedure which you do<br>
    *                 not want to be automatically deleted by any ADE tools (e.g.<br>
    *                 Editor, UIB). The program _runtool.p will check for the<br>
    *                 existence of ADEPersistent in a procedure and will delete<br>
    *                 it only if it doesn't exist.<br>
    * <p>
    *     Syntax :    {adecomm/_adetool.i}<br>
    * <p>
    *     Parameters: None<br>
    *         <br>
    *     Description:<br>
    *     Notes  :<br>
    *     Authors: Gerry Seidl<br>
    *     Date   : 1/18/95<br>
    * *************************************************************************
    */
   public void adepersistent()
   {
      internalProcedure(new Block((Body) () -> 
      {
         returnNormal("OK");
      }));
   }

   /** *********************  Internal Procedures ********************** */
   /**
    * Purpose:  appServerConnect calls the AppServer CONNECT method.  The extra<br>
    *           layer facilitates the switching between running locally and running<br>
    *           remotely (through AppServer Technology) at run time without<br>
    *           modifying source code.  This is achieved by managing service name<br>
    *           records in the AppSrv-TT temp-table.<br>
    * Parameters:  service_name    the name of partition or service name to be <br>
    *                              connected.<br>
    *              security_prompt Yes causes a prompt for a userid/password which<br>
    *                              is included in the AppServer CONNECT method.<br>
    *                              No passes blanks to the AppServer.<br>
    *              app_server_info Any string is put into the 4Th argument of the<br>
    *                              AppServer CONNECT method.<br>
    *              conn-hdl        The connection handle for the AppServer session<br>
    * Notes:
    */
   public void appServerConnect(final character _partitionName, final logical _securityPrompt, final character _appServerInfo, final handle connHdl)
   {
      AppsrvTt_1_1.Buf xappsrvTt = TemporaryBuffer.define(appsrvTt, "xappsrvTt", "xAppSrv-TT");
      character partitionName = TypeFactory.initInput(_partitionName);
      logical securityPrompt = TypeFactory.initInput(_securityPrompt);
      character appServerInfo = TypeFactory.initInput(_appServerInfo);
      logical ret = TypeFactory.logical();
      character cConnString = TypeFactory.character();
      logical lcancel = TypeFactory.logical();
      
      internalProcedure(new Block((Init) () -> 
      {
         TypeFactory.initOutput(connHdl);
      }, 
      (Body) () -> 
      {
         RecordBuffer.openScope(xappsrvTt);
         ErrorManager.silentErrorEnable();
         new FindQuery(appsrvTt, "upper(appsrvTt.partition) = ?", null, "appsrvTt.partition asc", new Object[]
         {
            toUpperCase(partitionName)
         }).first();
         ErrorManager.silentErrorDisable();
         
         if (_not(appsrvTt.available()))
         {
            messageBox(new Object[]
            {
               "Attempt to connect a partition named",
               concat(partitionName, new character(".")),
               new SkipEntity(true),
               "This partition is not currently defined in the",
               "Service Parameter Maintenance tool."
            }, ALERT_ERROR, BTN_OK, (String) null);
            returnNormal("ERROR");
         }
         
         /* If not record is available for this partiton */
         if (_isNotEqual(appsrvTt.getPartitiontype(), "A"))
         {
            messageBox(new Object[]
            {
               "Attempt to connect a partition named",
               concat(partitionName, new character(".")),
               new SkipEntity(true),
               "This partition is not defined as an AppServer Partition in the",
               "Service Parameter Maintenance tool."
            }, ALERT_ERROR, BTN_OK, (String) null);
            returnNormal("ERROR");
         }
         
         /* Determine whether or not to override the defaults */
         if (_isUnknown(securityPrompt))
         {
            securityPrompt.assign(appsrvTt.isSecurity());
         }
         
         if (_isUnknown(appServerInfo))
         {
            appServerInfo.assign(appsrvTt.getInfo());
         }
         
         if (_and(appsrvTt.getAsHandle().isValid(), () -> isEqual(AsUtils.this.appsrvTt.getAsHandle(), SessionUtils.asHandle())))
         {
            appsrvTt.setAsHandle(new handle());
         }
         
         /* Return the Session handle if this service isn't remote */
         if (_not(appsrvTt.isConfiguration()))
         {
            appsrvTt.setAsHandle(new handle(SessionUtils.asHandle()));
            connHdl.assign(SessionUtils.asHandle());
            returnNormal();
         }
         
         connHdl.assign(appsrvTt.getAsHandle());
         appsrvTt.setConnCount(plus(appsrvTt.getConnCount(), 1));
         
         if (_and(connHdl.isValid(), () -> connHdl.unwrapConnectable().connected()))
         {
            returnNormal();
         }
         else
         {
            if (_and(notUnknown(connHdl), () -> isNotEqual(connHdl, SessionUtils.asHandle())))
            {
               if ((connHdl.isValid()).booleanValue())
               {
                  ErrorManager.silentErrorEnable();
                  HandleOps.delete(connHdl);
                  ErrorManager.silentErrorDisable();
               }
               
               AdaptiveQuery query0 = new AdaptiveQuery();
               
               forEach("loopLabel0", new Block((Init) () -> 
               {
                  query0.initialize(xappsrvTt, "xappsrvTt.asHandle = ?", null, "xappsrvTt.partition asc", new Object[]
                  {
                     connHdl
                  });
               }, 
               (Body) () -> 
               {
                  query0.next();
                  xappsrvTt.setAsHandle(new handle());
               }));
            }
 /* If we are dealing with an AppServer handle */;
         }
 /* This is now an invalid AppServer Handle */;
         connHdl.setUnknown();
         ErrorManager.silentErrorEnable();
         /* See if any other partition with the same AppService is connected - if so
            use that handle                                                        */
         new FindQuery(xappsrvTt, "upper(xappsrvTt.appService) = ? and upper(xappsrvTt.partition) != ?", () -> xappsrvTt.getAsHandle().isValid(), "xappsrvTt.appService asc", new Object[]
         {
            toUpperCase(appsrvTt.getAppService()),
            toUpperCase(appsrvTt.getPartition())
         }).first();
         ErrorManager.silentErrorDisable();
         
         if (xappsrvTt._available())
         {
            connHdl.assign(xappsrvTt.getAsHandle());
            
            if ((connHdl.unwrapConnectable().connected()).booleanValue())
            {
               appsrvTt.setAsHandle(xappsrvTt.getAsHandle());
               returnNormal();
            }
            else
            {
               ErrorManager.silentErrorEnable();
               HandleOps.delete(connHdl);
               ErrorManager.silentErrorDisable();
               connHdl.setUnknown();
               xappsrvTt.setAsHandle(new handle());
            }
         }
         /* We found another partition that is currently connected to this AppService */
         /* We need to make a connection. */
         ServerFactory.createServer(connHdl);
         
         /*  Check to see if we need to prompt for userid/password */
         if ((securityPrompt).booleanValue())
         {
            if (_isGreaterThan(ConnectionManager.numDBs(), 0))
            {
               ControlFlowOps.invokeWithMode("adecomm/get-user.p", "O", usrId);
            }
            
            ControlFlowOps.invokeWithMode("security_prompt", "O", lcancel);
            
            if ((lcancel).booleanValue())
            {
               connHdl.setUnknown();
               returnNormal("ERROR");
            }
         }
         cConnString.assign(getAsconnectString(appsrvTt));
         ret.assign(connHdl.unwrapConnectableServer().connect(cConnString, usrId, psswrd, appServerInfo));
         
         if (_not(ret))
         {
            messageBox(new Object[]
            {
               "Unable to connect to the AppServer for the",
               (character) new FieldReference(appsrvTt, "appService").getValue(),
               "service."
            }, ALERT_ERROR, BTN_OK, (String) null);
            connHdl.assign(SessionUtils.asHandle());
         }
         
         appsrvTt.setAsHandle(connHdl);
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     To disconnect from a named service (a Partition)<br>
    * Parameters:  service_name  The name of the service to be disconnected from<br>
    * Notes:
    */
   public void appServerDisconnect(final character _partitionName)
   {
      AppsrvTt_1_1.Buf xappsrvTtBuf2 = TemporaryBuffer.define(appsrvTt, "xappsrvTtBuf2", "xAppSrv-TT");
      character partitionName = TypeFactory.initInput(_partitionName);
      logical ret = TypeFactory.logical();
      
      internalProcedure(new Block((Body) () -> 
      {
         RecordBuffer.openScope(xappsrvTtBuf2);
         ErrorManager.silentErrorEnable();
         new FindQuery(appsrvTt, "upper(appsrvTt.partition) = ?", null, "appsrvTt.partition asc", new Object[]
         {
            toUpperCase(partitionName)
         }).first();
         ErrorManager.silentErrorDisable();
         
         if (_not(appsrvTt.available()))
         {
            returnNormal();
         }
         
         /* This was being run locally don't disconnect, just null out the handle */
         if (_or(not(appsrvTt.getAsHandle().isValid()), () -> isEqual(AsUtils.this.appsrvTt.getAsHandle(), SessionUtils.asHandle())))
         {
            appsrvTt.setAsHandle(new handle());
            returnNormal();
         }
         
         appsrvTt.setConnCount(minus(appsrvTt.getConnCount(), 1));
         
         if (_isLessThan(appsrvTt.getConnCount(), 1))
         {
            /* This partition is tapped out - disconnect it */
            /* Check to see if another partition is still using the connection */
            if ((new FindQuery(xappsrvTtBuf2, "upper(xappsrvTtBuf2.appService) = ? and upper(xappsrvTtBuf2.partition) != ?", () -> xappsrvTtBuf2.getAsHandle().isValid(), "xappsrvTtBuf2.appService asc", new Object[]
            {
               (P2JQuery.Parameter) () -> toUpperCase(appsrvTt.getAppService()),
               toUpperCase(partitionName)
            }, LockType.NONE).hasAny()).booleanValue())
            {
               appsrvTt.setAsHandle(new handle());
               /* Keep the connection for this other partition */
               returnNormal();
            }
            
            /* Found another active connection */
            /* Disconnect here */
            ret.assign(appsrvTt.getAsHandle().unwrapConnectable().disconnect());
            
            if ((ret).booleanValue())
            {
               /* Disconnection was succesful */
               HandleOps.delete(appsrvTt.getAsHandle());
               appsrvTt.setAsHandle(new handle());
            }
            else
            {
               messageBox(new Object[]
               {
                  "There is an error disconnecting from the",
                  (character) new FieldReference(appsrvTt, "appService").getValue(),
                  "AppServer Service."
               }, ALERT_ERROR, BTN_OK, (String) null);
            }
         }
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     <br>
    * Parameters:  &lt;none&gt;<br>
    * Notes:
    */
   public void loadPartitionInfo()
   {
      character ttFile = TypeFactory.character();
      
      internalProcedure(new Block((Body) () -> 
      {
         ttFile.assign(FileSystemOps.searchPath("appsrvtt.d"));
         
         if (_notUnknown(ttFile))
         {
            UnnamedStreams.assignIn(StreamFactory.openFileStream((ttFile).toStringMessage(), false, false));
            UnnamedStreams.safeInput().setEcho(false);
            
            repeat(TransactionType.FULL, "loopLabel1", new Block((Body) () -> 
            {
               appsrvTt.create();
               UnnamedStreams.safeInput().readField(new FieldReference(appsrvTt, "partition"));
               UnnamedStreams.safeInput().readField(new FieldReference(appsrvTt, "host"));
               UnnamedStreams.safeInput().readField(new FieldReference(appsrvTt, "service"));
               UnnamedStreams.safeInput().readField(new FieldReference(appsrvTt, "configuration"));
               UnnamedStreams.safeInput().readField(new FieldReference(appsrvTt, "security"));
               UnnamedStreams.safeInput().readField(new FieldReference(appsrvTt, "info"));
               UnnamedStreams.safeInput().readField(new FieldReference(appsrvTt, "appService"));
               UnnamedStreams.safeInput().readField(new FieldReference(appsrvTt, "partitiontype"));
               UnnamedStreams.safeInput().readField(new FieldReference(appsrvTt, "serverurl"));
               UnnamedStreams.safeInput().resetCurrentLine();
               
               if (_isEqual(appsrvTt.getAppService(), ""))
               {
                  appsrvTt.setAppService(appsrvTt.getPartition());
               }
 /* Convert from 9.0A to 9.0B */;
               
               if (_isEqual(appsrvTt.getPartitiontype(), ""))
               {
                  appsrvTt.setPartitiontype(new character("A"));
               }
               
               appsrvTt.setAsHandle(new handle());
               appsrvTt.validate();
            }));
            
            UnnamedStreams.closeIn();
         }
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public void savePartitionInfo()
   {
      internalProcedure(new Block((Body) () -> 
      {
         UnnamedStreams.assignOut(StreamFactory.openFileStream("appsrvtt.d", true, false));
         
         AdaptiveQuery query1 = new AdaptiveQuery();
         
         forEach("loopLabel2", new Block((Init) () -> 
         {
            query1.initialize(appsrvTt, ((String) null), null, "appsrvTt.partition asc", LockType.NONE);
         }, 
         (Body) () -> 
         {
            query1.next();
            
            /* This next 3 lines shouldn't be necessary, but there seems to be a bug
                 in the updatable browse */
            if (_and(and(and(isEqual(appsrvTt.getAppService(), ""), () -> isEqual(AsUtils.this.appsrvTt.getHost(), "")), () -> isEqual(AsUtils.this.appsrvTt.getService(), "")), () -> isEqual(AsUtils.this.appsrvTt.isConfiguration(), false)))
            {
               appsrvTt.deleteRecord();
            }
            else
            {
               UnnamedStreams.safeOutput().export(new FieldEntry[]
               {
                  new ExportField(new FieldReference(appsrvTt, "partition")),
                  new ExportField(new FieldReference(appsrvTt, "host")),
                  new ExportField(new FieldReference(appsrvTt, "service")),
                  new ExportField(new FieldReference(appsrvTt, "configuration")),
                  new ExportField(new FieldReference(appsrvTt, "security")),
                  new ExportField(new FieldReference(appsrvTt, "info")),
                  new ExportField(new FieldReference(appsrvTt, "appService")),
                  new ExportField(new FieldReference(appsrvTt, "partitiontype")),
                  new ExportField(new FieldReference(appsrvTt, "serverurl"))
               });
            }
         }));
         
         UnnamedStreams.closeOut();
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:  Prompts for userid and password   <br>
    * Parameters:  returns lCancel = TRUE if cancel was pressed.<br>
    *              returns lCancel = FALSE if ok was pressed.<br>
    * Notes:
    */
   public void securityPrompt(final logical lCancel)
   {
      internalProcedure(new Block((Pre) () -> 
      {
         securityPromptSecurityDialogFrame = GenericFrame.createFrame(AsUtilsSecurityPromptSecurityDialog.class, "securityPrompt-security-dialog");
      }, 
      (Init) () -> 
      {
         TypeFactory.initOutput(lCancel);
      }, 
      (Body) () -> 
      {
         securityPromptSecurityDialogFrame.openScope();
         lCancel.assign(true);
         
         FrameElement[] elementList0 = new FrameElement[]
         {
            new Element(usrId, securityPromptSecurityDialogFrame.widgetUsrId()),
            new Element(psswrd, securityPromptSecurityDialogFrame.widgetPsswrd()),
            new WidgetElement(securityPromptSecurityDialogFrame.widgetBtnOk()),
            new WidgetElement(securityPromptSecurityDialogFrame.widgetBtnCancel()),
            new WidgetElement(securityPromptSecurityDialogFrame.widgetBtnHelp())
         };
         
         securityPromptSecurityDialogFrame.update(elementList0);
         
         lCancel.assign(false);
         
         /* If we get to here then OK was pressed (not cancel)*/
         securityPromptSecurityDialogFrame.hide();
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * ***********************  Function Implementations****************
    */
   public character definedPartitions()
   {
      return function("definedPartitions", character.class, new Block((Body) () -> 
      {
         /*------------------------------------------------------------------------------
           Purpose:  Returns a list of defined Partitions (separated by CHR(3))
             Notes: 
             Modifications: 
                05/09/00 - BSG - Changed to use getPartitionsByType
         ------------------------------------------------------------------------------*/
         returnNormal(getPartitionsByType(new character("A")));
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getAsconnectString(final AppsrvTt_1_1.Buf bappsrvtt)
   {
      /*------------------------------------------------------------------------------
        Purpose:  Creates the connect string for the AppServer connect method
          Notes:  
      ------------------------------------------------------------------------------*/
      logical lDirect = TypeFactory.logical((Boolean) null);
      
      return function("getASConnectString", character.class, new Block((Init) () -> 
      {
         AsUtils.this.bappsrvtt = bappsrvtt;
      }, 
      (Body) () -> 
      {
         RecordBuffer.openScope(bappsrvtt);
         
         /* If the direct connection parameters are blank, don't make a 
              direct connection */
         if (_and(and(or(isEqual(bappsrvtt.getHost(), ""), () -> isUnknown(AsUtils.this.bappsrvtt.getHost())), () -> or(isEqual(AsUtils.this.bappsrvtt.getService(), ""), () -> isUnknown(AsUtils.this.bappsrvtt.getService()))), () -> or(isEqual(AsUtils.this.bappsrvtt.getAppService(), ""), () -> isUnknown(AsUtils.this.bappsrvtt.getAppService()))))
         {
            lDirect.assign(false);
         }
         
         /* If the URL parameters are blank, use a direct connetion */
         if (_or(isEqual(bappsrvtt.getServerurl(), ""), () -> isUnknown(AsUtils.this.bappsrvtt.getServerurl())))
         {
            lDirect.assign(true);
         }
         else
         {
            lDirect.assign(false);
         }
         
         /* Either WEBCLIENT or 4GL client using a URL */
         /* If we haven't decided the type of connection, decide from the 
              new CLIENT-TYPE attribute */
         if (_and(isUnknown(lDirect), () -> isNotEqual(LogicalTerminal.getClientType(), "WebClient")))
         {
            lDirect.assign(true);
         }
         
         /* Return the appropriate connection string */
         if ((lDirect).booleanValue())
         {
            returnNormal(concat((_isNotEqual(bappsrvtt.getHost(), "") ? concat(" -H ", bappsrvtt.getHost()) : new character("")), (_isNotEqual(bappsrvtt.getService(), "") ? concat(" -S ", bappsrvtt.getService()) : new character("")), (_isNotEqual(bappsrvtt.getAppService(), "") ? concat(" -AppService ", bappsrvtt.getAppService()) : new character(""))));
         }
         else
         {
            returnNormal(concat("-URL ", bappsrvtt.getServerurl()));
         }
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getJmspartitions()
   {
      return function("getJMSPartitions", character.class, new Block((Body) () -> 
      {
         /*------------------------------------------------------------------------------
           Purpose:  Returns a CHR(3) delimited list of JMS partition names  
             Notes:  
         ------------------------------------------------------------------------------*/
         returnNormal(getPartitionsByType(new character("J")));
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getJmsptnInfo(final character _cPartition_1)
   {
      character cPartition_1 = TypeFactory.initInput(_cPartition_1);
      /*------------------------------------------------------------------------------
        Purpose:  Gets the URL and connection string for the JMS Partition Connection
          Notes:  
      ------------------------------------------------------------------------------*/
      AppsrvTt_1_1.Buf lappsrvtt = TemporaryBuffer.define(appsrvTt, "lappsrvtt", "lAppSrvTT");
      
      return function("getJMSPtnInfo", character.class, new Block((Body) () -> 
      {
         RecordBuffer.openScope(lappsrvtt);
         ErrorManager.silentErrorEnable();
         new FindQuery(lappsrvtt, "upper(lappsrvtt.partition) = ?", null, "lappsrvtt.partition asc", new Object[]
         {
            toUpperCase(cPartition_1)
         }).first();
         ErrorManager.silentErrorDisable();
         
         if (_not(lappsrvtt.available()))
         {
            returnNormal(concat("?", chr(3), "Partition not defined"));
         }
         
         if (_isNotEqual(lappsrvtt.getPartitiontype(), "J"))
         {
            returnNormal(concat("?", chr(3), "Not a JMS Partition"));
         }
         
         returnNormal(concat(getAsconnectString(lappsrvtt), chr(3), lappsrvtt.getInfo()));
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getPartitionsByType(final character _cType_1)
   {
      character cType_1 = TypeFactory.initInput(_cType_1);
      /*------------------------------------------------------------------------------
        Purpose:  Returns a CHR(3) delimited list of partitions of a particular type
          Notes:  
      ------------------------------------------------------------------------------*/
      character cPartitionList = TypeFactory.character();
      
      return function("getPartitionsByType", character.class, new Block((Body) () -> 
      {
         AdaptiveQuery query2 = new AdaptiveQuery();
         
         forEach("loopLabel3", new Block((Init) () -> 
         {
            query2.initialize(appsrvTt, "upper(appsrvTt.partitiontype) = ?", null, "appsrvTt.partitiontype asc", new Object[]
            {
               toUpperCase(cType_1)
            }, LockType.NONE);
         }, 
         (Body) () -> 
         {
            query2.next();
            cPartitionList.assign(concat(cPartitionList, (_isEqual(cPartitionList, "") ? new character("") : chr(3)), appsrvTt.getPartition()));
         }));
         
         returnNormal(cPartitionList);
      }));
   }
}
