/*--------------------------------------------------------------------------
    File        : Dataext.java
    Purpose     : Support procedure for Data Object.  This is an extension
                  of data.p.  The extension is necessary to avoid an overflow
                  of the action segment.  From 9.1B this extension file contains
                  all of the get and set property functions. 
                  From Dynamics 2.1/ v10 it also includes the do* procedures. 
                  These will probably be rolled back into data.p when segment 
                  size increases. (rolled? )

    Syntax      : adm2/dataext.p
------------------------------------------------------------------------*/

package com.goldencode.hotel.common.adm2;

import com.goldencode.p2j.util.*;
import com.goldencode.p2j.persist.*;
import com.goldencode.util.*;

import static com.goldencode.p2j.util.BlockManager.*;
import static com.goldencode.p2j.util.CompareOps.*;
import static com.goldencode.p2j.util.MathOps.*;
import static com.goldencode.p2j.util.logical.*;
import static com.goldencode.p2j.util.TextOps.*;
import static com.goldencode.p2j.util.ProcedureManager.*;
import static com.goldencode.p2j.util.character.*;

/**
 * Business logic (converted to Java from the 4GL source code
 * in common/adm2/dataext.p).
 */
public class Dataext
{
   /* _UIB-CODE-BLOCK-END */   
   /* ********************  Preprocessor Definitions  ******************** */   
   /* _UIB-PREPROCESSOR-BLOCK-END */   
   /* *********************** Procedure Settings ************************ */   
   /* Settings for THIS-PROCEDURE
      Type: Include
      Allow: 
      Frames: 0
      Add Fields to: Neither
      Other Settings: INCLUDE-ONLY
    */   
   /* *************************  Create Window  ************************** */   
   /* DESIGN Window definition (used by the UIB) 
     CREATE WINDOW Include ASSIGN
            HEIGHT             = 4.57
            WIDTH              = 42.4.
   _* END WINDOW DEFINITION *_
                                                                           */   
   /* ***************************  Main Block  *************************** */   
   /* Include the file which defines prototypes for all of the super
        procedure's entry points. Also, start or attach to the super procedure.
        Skip start-super-proc if we *are* the super procedure. 
        And skip including the prototypes if we are *any* super procedure. */   
   /* Preprocessors to define properties that can be retrieved
       directly from the property temp-table. */   
   /* if appserver aware add appserver properties */   
   /*********************************************************************
   * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
   * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
   * below.  All Rights Reserved.                                       *
   *                                                                    *
   * The Initial Developer of the Original Code is PSC.  The Original   *
   * Code is Progress IDE code released to open source December 1, 2000.*
   *                                                                    *
   * The contents of this file are subject to the Possenet Public       *
   * License Version 1.0 (the "License"); you may not use this file     *
   * except in compliance with the License.  A copy of the License is   *
   * available as of the date of this notice at                         *
   * http://www.possenet.org/license.html                               *
   *                                                                    *
   * Software distributed under the License is distributed on an "AS IS"*
   * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
   * should refer to the License for the specific language governing    *
   * rights and limitations under the License.                          *
   *                                                                    *
   * Contributors:                                                      *
   *                                                                    *
   *********************************************************************/   
   /*--------------------------------------------------------------------------
       File        : smrtprop.i
       Purpose     : Starts smart.p super procedure and defines general
                     SmartObject properties and other values.
       Syntax      : {src/adm2/smrtprop.i}
   
       Description :
   
       Modified    : February 4, 2000 - Version 9.1B
     ------------------------------------------------------------------------*/   
   /*          This .W file was created with the Progress UIB.             */   
   /*----------------------------------------------------------------------*/   
   /* ***************************  Definitions  ************************** */   
   /*********************************************************************
   * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
   * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
   * below.  All Rights Reserved.                                       *
   *                                                                    *
   * The Initial Developer of the Original Code is PSC.  The Original   *
   * Code is Progress IDE code released to open source December 1, 2000.*
   *                                                                    *
   * The contents of this file are subject to the Possenet Public       *
   * License Version 1.0 (the "License"); you may not use this file     *
   * except in compliance with the License.  A copy of the License is   *
   * available as of the date of this notice at                         *
   * http://www.possenet.org/license.html                               *
   *                                                                    *
   * Software distributed under the License is distributed on an "AS IS"*
   * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
   * should refer to the License for the specific language governing    *
   * rights and limitations under the License.                          *
   *                                                                    *
   * Contributors:                                                      *
   *                                                                    *
   *********************************************************************/   
   /*--------------------------------------------------------------------------
       File        : smartdefscustom.i
       Purpose     : Entry point to define:
                       New definitions
                       {&xcInstanceProperties}
                       {&ADM-PROPERTY-DLG}
       Syntax      : {src/adm2/custom/smartdefscustom.i}
   
       Modified    : 06/03/1999
       
       Notes       : Referenced in {src/adm2/smrtprop.i}
     ------------------------------------------------------------------------*/   
   /* Service Managers */   
   /*********************************************************************
   * Copyright (C) 2000-2002 by Progress Software Corporation ("PSC"),  *
   * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
   * below.  All Rights Reserved.                                       *
   *                                                                    *
   * The Initial Developer of the Original Code is PSC.  The Original   *
   * Code is Progress IDE code released to open source December 1, 2000.*
   *                                                                    *
   * The contents of this file are subject to the Possenet Public       *
   * License Version 1.0 (the "License"); you may not use this file     *
   * except in compliance with the License.  A copy of the License is   *
   * available as of the date of this notice at                         *
   * http://www.possenet.org/license.html                               *
   *                                                                    *
   * Software distributed under the License is distributed on an "AS IS"*
   * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
   * should refer to the License for the specific language governing    *
   * rights and limitations under the License.                          *
   *                                                                    *
   * Contributors:                                                      *
   *                                                                    *
   *********************************************************************/   
   /*---------------------------------------------------------------------------------
     File: globals.i
   
     Description:  ICF Global Shared Variables
   
     Purpose:      ICF Global Shared Variables
                   The global shared variables available within the Dynamics environment.
                   
                   NOTE: (Dynamic Only )
                   Shared variables do not span the client / server boundary. On the
                   client these are set-up in afstartupp.p and on the server in the agent
                   as_startup.p
                   Also note if we include the guts of a procedure as an internal
                   procedure for shared client/server code, we sometime need to
                   ensure the shared variables are not redefined.
   
     Parameters:   <none>
   
     History:
     --------
   
     Modified: 11/08/2001 - Mark Davies (MIP)
               Copied code for ICF afglobals.i and added here to allow
               ICF changed adm code to compile. afglobals.i now includes
               this file for definitions.
               Any ADM code will now reference globals.i which excludes any other
               ICF specific includes.
   -----------------------------------------------------------*/   
   /*                   This .W file was created with the Progress UIB.             */   
   /*-------------------------------------------------------------------------------*/   
   /* ***************************  Definitions  ************************** */   
   /* _UIB-CODE-BLOCK-END */   
   /* ********************  Preprocessor Definitions  ******************** */   
   /* _UIB-PREPROCESSOR-BLOCK-END */   
   /* *********************** Procedure Settings ************************ */   
   /* Settings for THIS-PROCEDURE
      Type: Include
      Allow: 
      Frames: 0
      Add Fields to: Neither
      Other Settings: INCLUDE-ONLY
    */   
   /* *************************  Create Window  ************************** */   
   /* DESIGN Window definition (used by the UIB) 
     CREATE WINDOW Include ASSIGN
            HEIGHT             = 5.57
            WIDTH              = 64.4.
   _* END WINDOW DEFINITION *_
                                                                           */   
   /* ***************************  Main Block  *************************** */   
   /* Used whereever Crystal is used and when defining the tt_datasource temp-table 
      to define the maximum columns supported when printing to crystal.
      See browsercustom.p and afcrpplip2p.p for example uses
   */   
   /* Used for filter checks */   
   /* The table delimiter used in assignlist, *ColumnsByTable props */   
   handle gshAstraAppserver = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshastraappserver", true, new handle());

   /* Handle to Astra Application Server Partition */   
   handle gshSessionManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshsessionmanager", true, new handle());

   /* Handle Astra Session Manager */   
   handle gshRimanager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshrimanager", true, new handle());

   /* Handle Astra Session Manager */   
   handle gshSecurityManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshsecuritymanager", true, new handle());

   /* Handle Astra Security Manager */   
   handle gshProfileManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshprofilemanager", true, new handle());

   /* Handle Astra Profile Manager */   
   handle gshRepositoryManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshrepositorymanager", true, new handle());

   /* Handle Astra Repository Manager */   
   handle gshTranslationManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshtranslationmanager", true, new handle());

   /* Handle Astra Translation Manager */   
   handle gshWebManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshwebmanager", true, new handle());

   /* Handle Astra Web Manager */   
   character gscSessionId = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gscsessionid", true, new character(""));

   /* Unique session id */   
   decimal gsdSessionObj = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gsdsessionobj", true, new decimal(0));

   /* Unique session id */   
   handle gshFinManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshfinmanager", true, new handle());

   /* Handle AstraFin Generic Manager */   
   handle gshGenManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshgenmanager", true, new handle());

   /* Handle AstraFramework Generic Manager */   
   handle gshAgnManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshagnmanager", true, new handle());

   /* Handle AstraGen Generic Manager */   
   decimal gsdTempUniqueId = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gsdtempuniqueid", true, new decimal(0));

   /* A number for unique objects */   
   decimal gsdUserObj = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gsduserobj", true, new decimal(0));

   /* User Object ID */   
   decimal gsdRenderTypeObj = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gsdrendertypeobj", true, new decimal(0));

   /* Render Type Object ID */   
   decimal gsdSessionScopeObj = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gsdsessionscopeobj", true, new decimal(0));

   /* Render Type Object ID */   
   /*********************************************************************
   * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
   * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
   * below.  All Rights Reserved.                                       *
   *                                                                    *
   * The Initial Developer of the Original Code is PSC.  The Original   *
   * Code is Progress IDE code released to open source December 1, 2000.*
   *                                                                    *
   * The contents of this file are subject to the Possenet Public       *
   * License Version 1.0 (the "License"); you may not use this file     *
   * except in compliance with the License.  A copy of the License is   *
   * available as of the date of this notice at                         *
   * http://www.possenet.org/license.html                               *
   *                                                                    *
   * Software distributed under the License is distributed on an "AS IS"*
   * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
   * should refer to the License for the specific language governing    *
   * rights and limitations under the License.                          *
   *                                                                    *
   * Contributors:                                                      *
   *                                                                    *
   *********************************************************************/   
   /*------------------------------------------------------------------------
       File        : globalscustom.i
       Purpose     : The developer can create his own version of this to 
                     allow 'global' definitions to be added to the 
                     framework.   
       Syntax      : {src/adm2/custom/globalscustom.i}
   
       Description :
   
       Created     : 06/03/1999
       Notes       : Referenced in {src/adm2/globals.i}
     ----------------------------------------------------------------------*/   
   /*          This .W file was created with the Progress AppBuilder.      */   
   /*----------------------------------------------------------------------*/   
   /* ***************************  Definitions  ************************** */   
   /* _UIB-CODE-BLOCK-END */   
   /* define the ADM Version and broker handle for all SmartObjects */   
   /* Note that adm-props-from-repository is documented as possible to define
        as a variable for flexible override, so it cannot be used for conditional 
        compile.  */   
   /* adm-props-from-repository is supported to be defined as a variable,
       so it is not used for conditional compile */   
   handle ghProp = TypeFactory.handle();

   /**
    * External procedure (converted to Java from the 4GL source code
    * in common/adm2/dataext.p).
    */
   public void execute()
   {
      externalProcedure(Dataext.this, new Block((Init) () -> 
      {
         gsdSessionObj.setPrecision(9);
         gsdTempUniqueId.setPrecision(9);
         gsdUserObj.setPrecision(9);
         gsdRenderTypeObj.setPrecision(9);
         gsdSessionScopeObj.setPrecision(9);
      }, 
      (Body) () -> 
      {
         /* _UIB-CODE-BLOCK-END */
         /* *********************** Procedure Settings ************************ */
         /* Settings for THIS-PROCEDURE
            Type: Method-Library
            Allow: 
            Frames: 0
            Add Fields to: Neither
            Other Settings: INCLUDE-ONLY
          */
         /* *************************  Create Window  ************************** */
         /* DESIGN Window definition (used by the UIB) 
           CREATE WINDOW Method-Library ASSIGN
                  HEIGHT             = 14.86
                  WIDTH              = 57.2.
         _* END WINDOW DEFINITION *_
                                                                                 */
         /* ***************************  Main Block  *************************** */
         /* Include the file which defines prototypes for all of the super
              procedure's entry points. Also, start or attach to the super procedure.
              Skip start-super-proc if we *are* the super procedure. 
              And skip including the prototypes if we are *any* super procedure. */
         /* These preprocessors let the get and set methods know at compile time
             which property values are located in the temp-table and which must
             be accessed through the property functions.
          */
         /* for support of dynamic objects */
         /* physical name - no path */
         /* for support of runtime parameters passed from the menus */
         /* Properties for super procedure management */
         /* The name of the super procedure; can be a CSV list */ 
 /* STATELESS, STATEFUL; the mode that all supers for the object will be launched in */ 
 /* Runtime property storing handles to the object's super procedures, defined in SuperProcedure. */
         /* Layout support for dynamic objects */
         /* contains a layout position that allows an object to be laid out correctly. */
         /* determine if we need to create the prop fields here or from the */
         /* repository (if ICF is running ) */
         /* COMMENT THE FOLLOWING LINE TO USE THE "<class>prop.i" DEFINITIONS */
         /* This temp-table defines all the propertt fields for an object.
             This include file contributes the DEFINE statement header and
             all basic smart object properties. Each other property class include file
             adds its own fields and then the parent object ends the statement.
             Define the fields for smart objects only, not for their super procedures.
          */
         if (_not(handle.fromString(entry(1, thisProcedure().unwrapADMData().getADMData(), chr(1))).isValid()))
         {
         }
         
         /* _UIB-CODE-BLOCK-END */
         if (_not(handle.fromString(entry(1, thisProcedure().unwrapADMData().getADMData(), chr(1))).isValid()))
         {
         }
         
         /* _UIB-CODE-BLOCK-END */
         if (_not(handle.fromString(entry(1, thisProcedure().unwrapADMData().getADMData(), chr(1))).isValid()))
         {
         }
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * ***********************  Function Implementations****************
    */
   public character getObjectType()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Returns the Type of the SmartObject, such as "SmartDataObject", etc.
                  For an ADM Super procedure, this function returns "SUPER".
         Params:  <none>
      ------------------------------------------------------------------------------*/
      character cType = TypeFactory.character();
      
      return function("getObjectType", character.class, new Block((Body) () -> 
      {
         cType.assign("SUPER");
         returnNormal(cType);
      }));
   }

   /** *********************  Internal Procedures ********************** */
   /**
    * Purpose:     Transfers changed rows into the Update Temp-Table and returns<br>
    *              it to the caller (the Commit function).<br>
    * <p>
    * Parameters:  None<br>
    * <p>
    * Notes:       For each existing row to be updated, there is already a "before"<br>
    *              copy in the RowObjUpd table, so we create an "after" row.<br>
    *              There is already a row in RowObjUpd for each Added/Copied row, <br>
    *              so we just update it with the latest values.<br>
    *              There is already a row in RowObjUpd for each Deleted row, so<br>
    *              we don't need to do anything for these rows.
    */
   public void doBuildUpd()
   {
      handle hRowObject = TypeFactory.handle();
      handle hRowObjUpd = TypeFactory.handle();
      handle hQuery = TypeFactory.handle();
      logical lAuto = TypeFactory.logical();
      
      internalProcedure(new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("RowObject"));
         hRowObject.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("RowObjUpd"));
         hRowObjUpd.assign(ghProp.unwrapBufferField().value());
         /* local buffer to the RowObject table to eliminate repositioning problems */
         RecordBuffer.create(hRowObject, hRowObject);
         QueryWrapper.createQuery(hQuery);
         hQuery.unwrapQuery().setBuffers(hRowObject);
         hQuery.unwrapQuery().prepare(concat("FOR EACH ", hRowObject.unwrap().name(), " WHERE RowMod = \"U\""));
         hQuery.unwrapQuery().queryOpen();
         hQuery.unwrapQuery().getFirst();
         
         loopLabel0:
         while (_not(hQuery.unwrapQuery().isOffEnd()))
         {
            hRowObjUpd.unwrapBuffer().bufferCreate();
            hRowObjUpd.unwrapBuffer().bufferCopy(hRowObject);
            hQuery.unwrapQuery().getNext();
         }
         
         hQuery.unwrapQuery().queryClose();
         hQuery.unwrapQuery().prepare(concat("FOR EACH ", hRowObject.unwrap().name(), " WHERE RowMod = \"A\" OR RowMod = \"C\""));
         hQuery.unwrapQuery().queryOpen();
         hQuery.unwrapQuery().getFirst();
         
         loopLabel1:
         while (_not(hQuery.unwrapQuery().isOffEnd()))
         {
            hRowObjUpd.unwrapBuffer().findFirst(concat("WHERE RowNum = ", valueOf(hRowObject.unwrapBuffer().bufferField("RowNum").unwrapBufferField().value())));
            hRowObjUpd.unwrapBuffer().bufferCopy(hRowObject);
            hQuery.unwrapQuery().getNext();
         }
         
         /* if this is an AutoCommit SDO, only one record is being processed, so leave it */
         /* available to be backwards compatible with earlier releases of the ADM */
         /* {get Prop var [handle]} */ 
         lAuto.assign(ControlFlowOps.invokeDynamicFunctionIn(logical.class, "getAutoCommit", targetProcedure()));
         
         if (_not(lAuto))
         {
            hRowObjUpd.unwrapBuffer().bufferRelease();
         }
         
         HandleOps.delete(hRowObject);
         HandleOps.delete(hQuery);
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     FINDs the specified row to be updated and creates a "backup" <br>
    *              (a before-image) copy of it in the RowObjUpd table, to support<br>
    *              Undo.  Run from submitRow when it receives a set of value <br>
    *              changes from a UI object.<br>
    * <p>
    * Parameters:<br>
    *   INPUT  pcRowIdent  - encoded "key" of the row to be updated.<br>
    *   INPUT  pcValueList - the list of changes made.  A CHR(1) delimited list<br>
    *                        of FieldName/NewValue pairs.<br>
    *   OUTPUT plReopen    - true if the row was new (either a copy or an add),<br>
    *                        so reopen RowObject query.<br>
    *   OUTPUT pcMessage   - error messages.<br>
    * <p>
    * Notes:       Run from submitRow.  Returns error message or "". <br>
    *              If the row is not available in the RowObject temp-table<br>
    *              (this would be because the SmartDataObject was not the <br>
    *              DataSource) this routine FINDs the database record(s) using <br>
    *              the RowIdent key before applying the changes, unless it's a <br>
    *              new row.
    */
   public void doCreateUpdate(final character _pcRowIdent_1, final character _pcValueList, final logical plReopen, final character pcMessage)
   {
      character pcRowIdent_1 = TypeFactory.initInput(_pcRowIdent_1);
      character pcValueList = TypeFactory.initInput(_pcValueList);
      handle hRowObject = TypeFactory.handle();
      handle hRowObjUpd = TypeFactory.handle();
      handle hDataQuery = TypeFactory.handle();
      integer iNextRowNum = TypeFactory.integer();
      
      internalProcedure(new Block((Init) () -> 
      {
         TypeFactory.initOutput(plReopen);
         TypeFactory.initOutput(pcMessage);
      }, 
      (Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("RowObject"));
         hRowObject.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("RowObjUpd"));
         hRowObjUpd.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DataHandle"));
         hDataQuery.assign(ghProp.unwrapBufferField().value());
         
         /* We postpone RowObjUpd table creation for dynamic SDOs until really needed, 
              which will be now... */
         if (_not(hRowObjUpd.isValid()))
         {
            /* {fn <func-name> [handle] */ 
            ControlFlowOps.invokeDynamicFunctionIn(logical.class, "createRowObjUpdTable", targetProcedure());
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("RowObjUpd"));
            hRowObjUpd.assign(ghProp.unwrapBufferField().value());
         }
         
         RecordBuffer.create(hRowObjUpd, hRowObjUpd);
         
         /* If it is empty, re-FIND the db rec(s), unless it's a newly */
         /* added row (no db rowids in RowIdent).                    */
         if (_and(and(or(not(hDataQuery.unwrapQuery().isOpen()), () -> isEqual(hDataQuery.unwrapQuery().getNumResults(), 0)), () -> isGreaterThan(numEntries(pcRowIdent_1), 1)), () -> isNotEqual(entry(2, pcRowIdent_1), "")))
         {
            /* Now create a RowObject record and assign a RowNum to it. */
            hRowObject.unwrapBuffer().bufferCreate();
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("LastRowNum"));
            iNextRowNum.assign(ghProp.unwrapBufferField().value());
            iNextRowNum.assign(plus(iNextRowNum, 1));
            hRowObject.unwrapBuffer().bufferField("RowNum").unwrapBufferField().changeValue(iNextRowNum);
            /* This creates a temp-table row from the db recs. */
            ControlFlowOps.invokeInWithMode("transferDBRow", targetProcedure(), "II", pcRowIdent_1, iNextRowNum);
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("LastRowNum"));
            ghProp.unwrapBufferField().changeValue(iNextRowNum);
         }
         else
         {
            hRowObject.unwrapBuffer().findByRowID(new rowid(entry(1, pcRowIdent_1)));
         }
         
         ErrorManager.silentErrorEnable();
         /* First check to see if there's already a saved pre-change version 
              of the record; this would be so if the same row is changed 
              multiple times before commit. */
         hRowObjUpd.unwrapBuffer().findFirst(concat("WHERE RowNum = ", valueOf(hRowObject.unwrapBuffer().bufferField("RowNum").unwrapBufferField().value())));
         ErrorManager.silentErrorDisable();
         
         doBlock(TransactionType.FULL, "blockLabel0", new Block((Body) () -> 
         {
            if (_not(hRowObjUpd.unwrapBuffer().available()))
            {
               hRowObjUpd.unwrapBuffer().bufferCreate();
               hRowObjUpd.unwrapBuffer().bufferCopy(hRowObject);
            }
            
            /* END DO IF NO ROU available */
            /* If this isn't an add/copy then  flag it as update. */
            if (_and(isNotEqual(hRowObject.unwrapBuffer().bufferField("RowMod").unwrapBufferField().value(), "A"), () -> isNotEqual(hRowObject.unwrapBuffer().bufferField("RowMod").unwrapBufferField().value(), "C")))
            {
               hRowObject.unwrapBuffer().bufferField("RowMod").unwrapBufferField().changeValue(new character("U"));
            }
            else
            {
               plReopen.assign(true);
            }
         }));
         
         /* END TRANSACTION */
         pcMessage.assign("");
         /* "Success" output value. */
         HandleOps.delete(hRowObjUpd);
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     <br>
    * Parameters:  &lt;none&gt;<br>
    * Notes:       Cannot use EMPTY-TEMP-TABLE method within a TRANSACTION if the<br>
    *              Temp-table is UNDO
    */
   public void doEmptyTempTable()
   {
      handle hRowObject = TypeFactory.handle();
      handle hQuery = TypeFactory.handle();
      
      internalProcedure(new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("RowObject"));
         hRowObject.assign(ghProp.unwrapBufferField().value());
         RecordBuffer.create(hRowObject, hRowObject);
         QueryWrapper.createQuery(hQuery);
         hQuery.unwrapQuery().setBuffers(hRowObject);
         hQuery.unwrapQuery().prepare(concat("FOR EACH ", hRowObject.unwrap().name()));
         hQuery.unwrapQuery().queryOpen();
         
         doBlock(TransactionType.FULL, "blockLabel1", new Block((Body) () -> 
         {
            hQuery.unwrapQuery().getFirst();
            
            loopLabel2:
            while (_not(hQuery.unwrapQuery().isOffEnd()))
            {
               hRowObject.unwrapBuffer().deleteRecord();
               hQuery.unwrapQuery().getNext();
            }
         }));
         
         HandleOps.delete(hQuery);
         HandleOps.delete(hRowObject);
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:    Return current record to addmode after an unsuccesful commit   <br>
    * Parameters:  &lt;none&gt;<br>
    * Notes:      This is only used by the SBO to return to addmode when an Add <br>
    *             failed when AutoCommit. Since the SDO has AutoCommit doReturnUpd<br>
    *             does not remove the ROwObjUpd.<br>
    *          -  This logic is expected to become obsolete as ROwObjUpd records <br>
    *             for add/copy really are created too early.
    */
   public void doReturnToAddMode()
   {
      handle hRowObject = TypeFactory.handle();
      handle hRowObjUpd = TypeFactory.handle();
      
      internalProcedure(new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("RowObject"));
         hRowObject.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("RowObjUpd"));
         hRowObjUpd.assign(ghProp.unwrapBufferField().value());
         
         if (_and(hRowObject.unwrapBuffer().available(), () -> hRowObjUpd.isValid()))
         {
            RecordBuffer.create(hRowObjUpd, hRowObjUpd);
            ErrorManager.silentErrorEnable();
            hRowObjUpd.unwrapBuffer().findFirst(concat("WHERE RowNum = ", valueOf(hRowObject.unwrapBuffer().bufferField("RowNum").unwrapBufferField().value()), " AND (RowMod = \"A\" OR RowMod = \"C\")"));
            ErrorManager.silentErrorDisable();
            
            if ((hRowObjUpd.unwrapBuffer().available()).booleanValue())
            {
               hRowObjUpd.unwrapBuffer().deleteRecord();
            }
            
            HandleOps.delete(hRowObjUpd);
         }
         returnNormal();
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     RUN from Commit on the client side to get back the Update <br>
    *              (RowObjUpd) table (from the server side) and undo any failed <br>
    *              changes or return final versions of record values to the client.<br>
    * <p>
    * Parameters:  <br>
    *   INPUT cUndoIds - list of any RowObject Rownum whose changes were <br>
    *                    rejected by a commit. Delimited list of the form:<br>
    *              "RowNumCHR(3)ADM-ERROR-STRING,RowNumCHR(3)ADM-ERROR-STRING,..."<br>
    *          - A '?' as RowNum means do NOT reposition, but stay on current record. <br>
    *           <br>
    * Notes:   - If the error string in cUndoIds is "ADM-FIELDS-CHANGED", then<br>
    *            another user has changed at least one field value.  In this <br>
    *            case, RowObjUpd fields will contain the refreshed db values <br>
    *            and we pass those values back to the client.<br>
    *          - If not autocommit we may also reposition here, but otherwise the caller <br>
    *            both has the rowident and has more info. (submitCommit knows whether <br>
    *            a reopen is required, deleteRow just uses fetchNext if required)
    */
   public void doReturnUpd(final character _pcUndoIds)
   {
      character pcUndoIds = TypeFactory.initInput(_pcUndoIds);
      handle hRowObject = TypeFactory.handle();
      handle hRowObject2 = TypeFactory.handle();
      handle hRowObjUpd = TypeFactory.handle();
      handle hRowObjUpd2 = TypeFactory.handle();
      handle hQuery = TypeFactory.handle();
      logical lAutoCommit = TypeFactory.logical();
      handle hDataQuery = TypeFactory.handle();
      logical lQueryContainer = TypeFactory.logical();
      logical lBrowsed = TypeFactory.logical();
      handle hMsgSource = TypeFactory.handle();
      integer iCurrentRowNum = TypeFactory.integer();
      integer iRowNum = TypeFactory.integer();
      logical lCommitOk = TypeFactory.logical();
      logical lQueryOpened = TypeFactory.logical();
      integer iTempRowNum = TypeFactory.integer();
      character cFirstError = TypeFactory.character();
      character cErrorType = TypeFactory.character();
      character cKeyFields = TypeFactory.character();
      character cDataDelimiter = TypeFactory.character();
      integer iLoop = TypeFactory.integer();
      integer iLoop2 = TypeFactory.integer();
      character cKeyValues = TypeFactory.character();
      handle hRowObjUpdTable = TypeFactory.handle();
      logical lHasRecords = TypeFactory.logical();
      handle hReadHandler = TypeFactory.handle();
      character cReadFormat = TypeFactory.character();
      character cReadcolumns = TypeFactory.character();
      handle hReadBuffer = TypeFactory.handle();
      character cContainerName = TypeFactory.character();
      character cUniqueObjectName = TypeFactory.character();
      character cCommittedAction = TypeFactory.character();
      
      internalProcedure(new Block((Body) () -> 
      {
         /* {get Prop var [handle]} */ 
         lAutoCommit.assign(ControlFlowOps.invokeDynamicFunctionIn(logical.class, "getAutoCommit", targetProcedure()));
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("RowObject"));
         hRowObject.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("RowObjUpd"));
         hRowObjUpd.assign(ghProp.unwrapBufferField().value());
         /* {get Prop var [handle]} */ 
         hRowObjUpdTable.assign(ControlFlowOps.invokeDynamicFunctionIn(handle.class, "getRowObjUpdTable", targetProcedure()));
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ObjectName"));
         cUniqueObjectName.assign(ghProp.unwrapBufferField().value());
         
         /* May be called from SBO when no table is defined (dynamic sdo with no updates)*/
         if (_not(hRowObjUpdTable.isValid()))
         {
            returnNormal();
         }
         
         lHasRecords.assign(hRowObjUpdTable.unwrapTempTable().hasRecords());
         /* local buffers */
         RecordBuffer.create(hRowObject2, hRowObject);
         RecordBuffer.create(hRowObjUpd2, hRowObjUpd);
         /* set up RowObjUpd query */
         QueryWrapper.createQuery(hQuery);
         hQuery.unwrapQuery().setBuffers(hRowObjUpd);
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryContainer"));
         lQueryContainer.assign(ghProp.unwrapBufferField().value());
         
         if ((lQueryContainer).booleanValue())
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("ContainerSource"));
            hMsgSource.assign(ghProp.unwrapBufferField().value());
            ghProp.assign(handle.fromString(entry(1, hMsgSource.unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("ObjectName"));
            cContainerName.assign(ghProp.unwrapBufferField().value());
            cUniqueObjectName.assign(concat(cContainerName, ".", cUniqueObjectName));
         }
         else
         {
            hMsgSource.assign(targetProcedure());
         }
         
         lCommitOk.assign(and(not(new logical(ControlFlowOps.invokeDynamicFunctionIn(logical.class, "anyMessage", hMsgSource))), () -> isEqual(pcUndoIds, "")));
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DataDelimiter"));
         cDataDelimiter.assign(ghProp.unwrapBufferField().value());
         
         /* Blank if no error.. */
         /* Commit successful. */
         /* for each RowObjUpd */
         if ((lCommitOk).booleanValue())
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("LastCommitErrorType"));
            ghProp.unwrapBufferField().changeValue(new character(""));
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("LastCommitErrorKeys"));
            ghProp.unwrapBufferField().changeValue(new character(""));
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("DataReadHandler"));
            hReadHandler.assign(ghProp.unwrapBufferField().value());
            
            if ((hReadHandler.isValid()).booleanValue())
            {
               ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
               ghProp.assign(ghProp.unwrapBuffer().bufferField("DataReadColumns"));
               cReadcolumns.assign(ghProp.unwrapBufferField().value());
               ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
               ghProp.assign(ghProp.unwrapBuffer().bufferField("DataReadFormat"));
               cReadFormat.assign(ghProp.unwrapBufferField().value());
               ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
               ghProp.assign(ghProp.unwrapBuffer().bufferField("DataReadBuffer"));
               hReadBuffer.assign(ghProp.unwrapBufferField().value());
            }
            
            /* Delete ALL RowObjUpd records, but before the delete, move updated versions of records
                   back into the RowObject table for the client to see (they may have new values from CREATE 
                   or WRITE triggers or other server side logic) "A" = Add, "C" = Copy, "U" = update. */
            hQuery.unwrapQuery().prepare(concat("FOR EACH ", hRowObjUpd.unwrap().name()));
            hQuery.unwrapQuery().queryOpen();
            hQuery.unwrapQuery().getFirst();
            
            LogicalOp whileClause0 = () -> hRowObjUpd.unwrapBuffer().available();
            
            doWhile("loopLabel3", whileClause0, new Block((Body) () -> 
            {
               cCommittedAction.assign(hRowObjUpd.unwrapBuffer().bufferField("RowMod").unwrapBufferField().value());
               cCommittedAction.assign(_isEqual(cCommittedAction, "A") ? new character("ADD") : _isEqual(cCommittedAction, "C") ? new character("COPY") : _isEqual(cCommittedAction, "U") ? new character("UPDATE") : _isEqual(cCommittedAction, "D") ? new character("DELETE") : new character(""));
               
               if (_isGreaterThan(lookup(cCommittedAction, "ADD,COPY,UPDATE"), 0))
               {
                  ErrorManager.silentErrorEnable();
                  hRowObject2.unwrapBuffer().findFirst(concat("WHERE RowNum = ", valueOf(hRowObjUpd.unwrapBuffer().bufferField("RowNum").unwrapBufferField().value())));
                  ErrorManager.silentErrorDisable();
                  
                  if ((hRowObject2.unwrapBuffer().available()).booleanValue())
                  {
                     hRowObject2.unwrapBuffer().bufferCopy(hRowObjUpd);
                     hRowObject2.unwrapBuffer().bufferField("RowMod").unwrapBufferField().changeValue(new character(""));
                  }
               }
               
               /* can-do('A,C,U',RowObjUpd.Rowmod)  */
               /* Open API callback. */
               /* We only do this if we got here from 'commitData' (AutoCommit is OFF). */
               /* Otherwise the individual APIs (updateData, createData etc.) return the new values directly */
               if (_and(and(not(lAutoCommit), () -> hReadHandler.isValid()), () -> isGreaterThan(cCommittedAction, "")))
               {
                  if ((hReadBuffer.isValid()).booleanValue())
                  {
                     hReadBuffer.unwrapBuffer().bufferCreate();
                     hReadBuffer.unwrapBuffer().bufferCopy(hRowObjUpd);
                     ControlFlowOps.invokeInWithMode("receiveCommittedBuffer", hReadHandler, "II", cUniqueObjectName, hReadBuffer);
                  }
                  else
                  {
                     ControlFlowOps.invokeInWithMode("receiveCommittedData", hReadHandler, "IIII", cUniqueObjectName, cReadcolumns, (_isGreaterThan(lookup(cCommittedAction, "ADD,COPY,UPDATE"), 0) ? ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "colStringValuesAnyRow", targetProcedure(), "IIII", hRowObject2, cReadcolumns, cReadFormat, cDataDelimiter) : hRowObjUpd.unwrapBuffer().bufferField("RowIdent").unwrapBufferField().value()), cCommittedAction);
                  }
               }
               hRowObjUpd.unwrapBuffer().deleteRecord();
               hQuery.unwrapQuery().getNext();
            }));
            
            hQuery.unwrapQuery().queryClose();
         }
         else
         {
            ErrorManager.silentErrorEnable();
            cFirstError.assign(entry(1, pcUndoIds));
            cFirstError.assign(trim(entry(2, cFirstError, chr(3))));
            ErrorManager.silentErrorDisable();
            
            switch (StringHelper.safeTrimTrailing((cFirstError).toStringMessage()).toUpperCase())
            {
               case "ADM-FIELDS-CHANGED":
                  {
                     cErrorType.assign("Conflict");
                  }
                  break;
               default:
                  {
                     cErrorType.assign("Error");
                  }
                  break;
            }
            
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("LastCommitErrorType"));
            ghProp.unwrapBufferField().changeValue(cErrorType);
            /* log the key values of all failed records */
            /* {get Prop var [handle]} */ 
            cKeyFields.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getKeyFields", targetProcedure()));
            
            loopLabel4:
            for (iLoop.assign(1); _isLessThanOrEqual(iLoop, numEntries(pcUndoIds)); iLoop.increment())
            {
               /* there is a trailing ',' in pcUndoIds */
               if (_isEqual(entry(iLoop, pcUndoIds), ""))
               {
                  break loopLabel4;
               }
               
               ErrorManager.silentErrorEnable();
               hRowObject2.unwrapBuffer().findFirst(concat("WHERE RowNum = ", entry(1, entry(iLoop, pcUndoIds), chr(3))));
               ErrorManager.silentErrorDisable();
               
               if ((hRowObject2.unwrapBuffer().available()).booleanValue())
               {
                  loopLabel5:
                  for (iLoop2.assign(1); _isLessThanOrEqual(iLoop2, numEntries(cKeyFields)); iLoop2.increment())
                  {
                     cKeyValues.assign(concat(cKeyValues, (_isEqual(cKeyValues, "") ? new character("") : cDataDelimiter), valueOf(hRowObject2.unwrapBuffer().bufferField(entry(iLoop2, cKeyFields)).unwrapBufferField().value())));
                  }
               }
            }
            
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("LastCommitErrorKeys"));
            ghProp.unwrapBufferField().changeValue(cKeyValues);
            
            /* If not autocommit we want to know where we shall position to after */
            if (_not(lAutoCommit))
            {
               /* Keep track of where we are for no AutoCommit repositioning logic below */
               if ((hRowObject.unwrapBuffer().available()).booleanValue())
               {
                  iCurrentRowNum.assign(hRowObject.unwrapBuffer().bufferField("RowNum").unwrapBufferField().value());
               }
               else
               {
                  iCurrentRowNum.setUnknown();
               }
               
               /* ? if not avail */
               /* We noramlly reposition to the firstr entry passed in, note that a 
                        value of '?' in the first RowNum entry  means that we must stay where 
                        we are */
               if (_isNotEqual(pcUndoIds, ""))
               {
                  iRowNum.assign(new integer(entry(1, entry(1, pcUndoIds, ","), chr(3))));
               }
               
               ErrorManager.silentErrorEnable();
               hRowObjUpd.unwrapBuffer().findFirst(new character("WHERE RowMod = \"D\""));
               ErrorManager.silentErrorDisable();
               
               if ((hRowObjUpd.unwrapBuffer().available()).booleanValue())
               {
                  /* If nowhere to reposition to yet, then position to the first undeleted row */
                  if (_isEqual(iRowNum, 0))
                  {
                     iRowNum.assign(hRowObjUpd.unwrapBuffer().bufferField("RowNum").unwrapBufferField().value());
                  }
                  
                  /* Restore deleted records */
                  ControlFlowOps.invokeIn("doUndoDelete", targetProcedure());
                  /* {fnarg <func-name> <arg> [handle] */ 
                  ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "openDataQuery", targetProcedure(), "I", "");
                  lQueryOpened.assign(true);
               }
            }
            /* not autocommit  */
            /* Go through each 'U' copy and use it to find the 'before-image' for delete
                   or refresh also copy the contents to rowObject if adm-fields-changed */
            hQuery.unwrapQuery().prepare(concat("FOR EACH ", hRowObjUpd.unwrap().name(), " WHERE RowMod = \"U\""));
            hQuery.unwrapQuery().queryOpen();
            hQuery.unwrapQuery().getFirst();
            
            loopLabel6:
            while ((hRowObjUpd.unwrapBuffer().available()).booleanValue())
            {
               iTempRowNum.assign(hRowObjUpd.unwrapBuffer().bufferField("RowNum").unwrapBufferField().value());
               
               /* Refreshing the update record if required (changed data was reset on server).*/
               if (_isNotEqual(indexOf(pcUndoIds, concat(valueOf(iTempRowNum), chr(3), "ADM-FIELDS-CHANGED")), 0))
               {
                  hRowObject.unwrapBuffer().findFirst(concat("WHERE RowNum = ", valueOf(iTempRowNum)));
                  /* Copy the refreshed db field values to row object. */
                  hRowObject.unwrapBuffer().bufferCopy(hRowObjUpd, new character("RowMod"));
                  hRowObjUpd2.unwrapBuffer().findFirst(concat("WHERE RowNum = ", valueOf(iTempRowNum), " AND RowMod = \"\""));
                  /* Copy the refreshed db field values to the pre-commit row.
                               Don't copy over the RowMod (that could change what the
                               record is used for). And don't copy over ChangedFields. Leave
                               that as the client set it. */
                  hRowObjUpd2.unwrapBuffer().bufferCopy(hRowObjUpd, new character("RowMod,ChangedFields"));
               }
               
               /* UndoIds matches RowNum and ADM-FIELDS-CHANGED */
               /* The pre-commit update record (RowMod = "") contains the right values for
                        an undo, so we no longer need the commit copy (RowMod = "U") of
                        the update record. */
               hRowObjUpd.unwrapBuffer().deleteRecord();
               hQuery.unwrapQuery().getNext();
            }
            
            /* for each RowObjUpd .. RowMod = "U" */
            hQuery.unwrapQuery().queryClose();
            
            /* Failed AutoCommit Add, copy and Delete should also be removed.
                   Non AutoCommit currently need to be kept as they are creted on save
                   not on the actual commit, but an Autocommit SBO also has the SDO in 
                   non autocommit and calls doReturnToAddMode to get rid of the RowObjUpd 
                   in cases where it was in add mode before the save.*/
            if ((lAutoCommit).booleanValue())
            {
               hQuery.unwrapQuery().prepare(concat("FOR EACH ", hRowObjUpd.unwrap().name(), " WHERE RowMod = \"A\" OR RowMod = \"C\" OR RowMod = \"D\""));
               hQuery.unwrapQuery().queryOpen();
               hQuery.unwrapQuery().getFirst();
               
               loopLabel7:
               while ((hRowObjUpd.unwrapBuffer().available()).booleanValue())
               {
                  hRowObjUpd.unwrapBuffer().deleteRecord();
                  hQuery.unwrapQuery().getNext();
               }
               
               hQuery.unwrapQuery().queryClose();
            }
         }
         
         /* commit not successful */
         if ((lHasRecords).booleanValue())
         {
            /* Deal with reopen, reposition and publish dataavailable for non auto commit */
            if (_not(lAutoCommit))
            {
               ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
               ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryContainer"));
               lQueryContainer.assign(ghProp.unwrapBufferField().value());
               /* {get Prop var [handle]} */ 
               lBrowsed.assign(ControlFlowOps.invokeDynamicFunctionIn(logical.class, "getDataQueryBrowsed", targetProcedure()));
               
               if ((lCommitOk).booleanValue())
               {
                  /* {set Prop value [handle]} */ 
                  ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "setRowObjectState", targetProcedure(), "I", "NoUpdates");
                  
                  /* If not Autocommit changes can be to more than one record, so if object
                               is being browsed then ensure all records in the browser are refreshed 
                               correctly.*/
                  if ((lBrowsed).booleanValue())
                  {
                     NamedEventManager.publish("refreshBrowse", targetProcedure());
                  }
                  
                  /* The SBO does publish dataavailable, when all objects have been returned,
                               so don't do it here. */
                  if (_not(lQueryContainer))
                  {
                     NamedEventManager.publish("dataAvailable", targetProcedure(), "I", new Object[]
                     {
                        "SAME"
                     });
                  }
               }
               else
               {
                  ErrorManager.silentErrorEnable();
                  hRowObject.unwrapBuffer().findFirst(concat("WHERE RowNum = ", (_or(isUnknown(iRowNum), () -> isEqual(iRowNum, 0)) ? valueOf(iCurrentRowNum) : valueOf(iRowNum))));
                  ErrorManager.silentErrorDisable();
                  
                  /* just in case the above failed (RowNum 0 and no current rec maybe.. ) */
                  if (_not(hRowObject.unwrapBuffer().available()))
                  {
                     ErrorManager.silentErrorEnable();
                     hRowObject.unwrapBuffer().findFirst();
                     ErrorManager.silentErrorDisable();
                  }
                  
                  /* If we have changed position we need to reposition the query also if
                               we are not browsed (fetchNext etc. assumes that the query is synched) */
                  if (_or(lQueryOpened, () -> and(hRowObject.unwrapBuffer().available(), () -> isNotEqual(hRowObject.unwrapBuffer().bufferField("RowNum").unwrapBufferField().value(), iCurrentRowNum))))
                  {
                     ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
                     ghProp.assign(ghProp.unwrapBuffer().bufferField("DataHandle"));
                     hDataQuery.assign(ghProp.unwrapBufferField().value());
                     ErrorManager.silentErrorEnable();
                     hDataQuery.unwrapQuery().queryRepositionByID(hRowObject.unwrapBuffer().rowID());
                     ErrorManager.silentErrorDisable();
                     
                     /* if not browsed, get-next to get the buffer */
                     if (_and(isEqual(ErrorManager.getErrorText(1), ""), not(lBrowsed)))
                     {
                        hDataQuery.unwrapQuery().getNext();
                     }
                  }
                  ErrorManager.silentErrorEnable();
                  hRowObject2.unwrapBuffer().findFirst(new character("WHERE RowMod <> \"\""));
                  ErrorManager.silentErrorDisable();
                  
                  if (_not(hRowObject2.unwrapBuffer().available()))
                  {
                     /* {set Prop value [handle]} */ 
                     ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "setRowObjectState", targetProcedure(), "I", "NoUpdates");
                  }
                  
                  ControlFlowOps.invokeIn("updateQueryPosition", targetProcedure());
                  
                  if (_not(lQueryContainer))
                  {
                     /* not avail is overkill as there always is a rowObject in 
                                       an undo, but.. */
                     NamedEventManager.publish("dataAvailable", targetProcedure(), "I", new Object[]
                     {
                        _or(not(hRowObject.unwrapBuffer().available()), () -> isNotEqual(hRowObject.unwrapBuffer().bufferField("RowNum").unwrapBufferField().value(), iCurrentRowNum)) ? new character("DIFFERENT") : new character("SAME")
                     });
                     ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
                     ghProp.assign(ghProp.unwrapBuffer().bufferField("NewBatchInfo"));
                     ghProp.unwrapBufferField().changeValue(new character(""));
                  }
               }
            }
         }
         HandleOps.delete(hQuery);
         HandleOps.delete(hRowObjUpd2);
         HandleOps.delete(hRowObject2);
         returnNormal();
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     Restore deleted rows. <br>
    * Parameters:  &lt;none&gt;<br>
    * <p>
    * Notes:       This is separated because: <br>
    *              - a failed commit should restore this, otherwise the user has to <br>
    *                undo to correct mistakes. <br>
    *              - The regualar undo need to restore these.
    */
   public void doUndoDelete()
   {
      handle hRowObject = TypeFactory.handle();
      handle hRowObjUpd = TypeFactory.handle();
      handle hQuery = TypeFactory.handle();
      integer iFirstRowNum = TypeFactory.integer();
      integer iLastRowNum = TypeFactory.integer();
      character cLastResultRow = TypeFactory.character();
      character cFirstResultRow = TypeFactory.character();
      handle hRowNum = TypeFactory.handle();
      handle hRowident = TypeFactory.handle();
      logical lNew = TypeFactory.logical();
      
      internalProcedure(new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("RowObjUpd"));
         hRowObjUpd.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("RowObject"));
         hRowObject.assign(ghProp.unwrapBufferField().value());
         
         if (_not(hRowObjUpd.isValid()))
         {
            returnNormal();
         }
         
         RecordBuffer.create(hRowObjUpd, hRowObjUpd);
         RecordBuffer.create(hRowObject, hRowObject);
         QueryWrapper.createQuery(hQuery);
         hQuery.unwrapQuery().setBuffers(hRowObjUpd);
         /* Start from last so the first delete is avail when we are finished */
         hQuery.unwrapQuery().prepare(concat("FOR EACH ", hRowObjUpd.unwrap().name(), " WHERE RowMod = \"D\" BY RowNum DESCENDING"));
         hQuery.unwrapQuery().queryOpen();
         hQuery.unwrapQuery().getFirst();
         hRowNum.assign(hRowObjUpd.unwrapBuffer().bufferField("RowNum"));
         hRowident.assign(hRowObjUpd.unwrapBuffer().bufferField("RowIdent"));
         
         loopLabel8:
         while (_not(hQuery.unwrapQuery().isOffEnd()))
         {
            lNew.assign(isEqual(hRowident.unwrapBufferField().value(), ""));
            hRowObject.unwrapBuffer().bufferCreate();
            hRowObject.unwrapBuffer().bufferCopy(hRowObjUpd);
            hRowObject.unwrapBuffer().bufferField("RowMod").unwrapBufferField().changeValue((lNew).booleanValue() ? new character("A") : new character(""));
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("FirstRowNum"));
            iFirstRowNum.assign(ghProp.unwrapBufferField().value());
            
            if (_isLessThan(hRowNum.unwrapBufferField().value(), iFirstRowNum))
            {
               ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
               ghProp.assign(ghProp.unwrapBuffer().bufferField("FirstRowNum"));
               ghProp.unwrapBufferField().changeValue(hRowNum.unwrapBufferField().value());
            }
            /* {set Prop value [handle]} */ 
            
            if (_not(lNew))
            {
               ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
               ghProp.assign(ghProp.unwrapBuffer().bufferField("FirstResultRow"));
               cFirstResultRow.assign(ghProp.unwrapBufferField().value());
               iFirstRowNum.assign(new integer(entry(1, cFirstResultRow, ";")));
               
               if (_isLessThan(hRowNum.unwrapBufferField().value(), iFirstRowNum))
               {
                  cFirstResultRow.assign(concat(hRowNum.unwrapBufferField().value(), ";", hRowident.unwrapBufferField().value()));
                  ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
                  ghProp.assign(ghProp.unwrapBuffer().bufferField("FirstResultRow"));
                  ghProp.unwrapBufferField().changeValue(cFirstResultRow);
               }
            }
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("LastRowNum"));
            iLastRowNum.assign(ghProp.unwrapBufferField().value());
            
            if (_isGreaterThan(hRowObjUpd.unwrapBuffer().bufferField("RowNum").unwrapBufferField().value(), iLastRowNum))
            {
               ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
               ghProp.assign(ghProp.unwrapBuffer().bufferField("LastRowNum"));
               ghProp.unwrapBufferField().changeValue(hRowObjUpd.unwrapBuffer().bufferField("RowNum").unwrapBufferField().value());
               ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
               ghProp.assign(ghProp.unwrapBuffer().bufferField("LastDbRowIdent"));
               ghProp.unwrapBufferField().changeValue(hRowObjUpd.unwrapBuffer().bufferField("RowIdent").unwrapBufferField().value());
            }
            
            if (_not(lNew))
            {
               ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
               ghProp.assign(ghProp.unwrapBuffer().bufferField("LastResultRow"));
               cLastResultRow.assign(ghProp.unwrapBufferField().value());
               iLastRowNum.assign(new integer(entry(1, cLastResultRow, ";")));
               
               if (_isGreaterThan(hRowNum.unwrapBufferField().value(), iLastRowNum))
               {
                  cLastResultRow.assign(concat(hRowNum.unwrapBufferField().value(), ";", hRowObjUpd.unwrapBuffer().bufferField("RowIdent").unwrapBufferField().value()));
                  ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
                  ghProp.assign(ghProp.unwrapBuffer().bufferField("LastResultRow"));
                  ghProp.unwrapBufferField().changeValue(cLastResultRow);
               }
            }
            /* not new */
            hRowObjUpd.unwrapBuffer().deleteRecord();
            hQuery.unwrapQuery().getNext();
         }
         
         HandleOps.delete(hQuery);
         HandleOps.delete(hRowObjUpd);
         HandleOps.delete(hRowObject);
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:  Rollback using the before image RowObjUpd record    <br>
    * Parameters:  &lt;none&gt;<br>
    * Notes:
    */
   public void doUndoRow()
   {
      handle hRowObject = TypeFactory.handle();
      handle hRowObjUpd = TypeFactory.handle();
      
      internalProcedure(new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("RowObject"));
         hRowObject.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("RowObjUpd"));
         hRowObjUpd.assign(ghProp.unwrapBufferField().value());
         
         if (_and(hRowObject.unwrapBuffer().available(), () -> hRowObjUpd.isValid()))
         {
            ErrorManager.silentErrorEnable();
            hRowObjUpd.unwrapBuffer().findFirst(concat("WHERE RowMod = \"\" AND RowNum = ", valueOf(hRowObject.unwrapBuffer().bufferField("RowNum").unwrapBufferField().value())));
            ErrorManager.silentErrorDisable();
            
            if ((hRowObjUpd.unwrapBuffer().available()).booleanValue())
            {
               hRowObject.unwrapBuffer().bufferCopy(hRowObjUpd);
               hRowObjUpd.unwrapBuffer().deleteRecord();
            }
         }
         returnNormal();
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     Does the buffer delete and copy operations to restore the<br>
    *              RowObject temp-table when an Undo occurs.  New RowObject records<br>
    *              (added or copied, RowObject.RowMod = "A"/"C") are deleted, modified <br>
    *              records (RowObject.RowMod = "U") are restored to their original <br>
    *              states and deleted records (RowObjUpd.RowMod = "D") are recreated.  <br>
    *              The RowObjUpd table is emptied.<br>
    * <p>
    * Parameters:  &lt;none&gt;<br>
    * <p>
    * Notes:       Invoked from the event procedure undoTransaction.<br>
    *              doUndoTrans is run on the client side.<br>
    *              This code is currently operating directly on RowObject <br>
    *              as it always FINDs after the looping.<br>
    *              The caller (f.ex undoTransaction) must check the position <br>
    *              before it reopens the query and reposition to it.
    */
   public void doUndoTrans()
   {
      handle hRowObject = TypeFactory.handle();
      handle hRowObjUpd = TypeFactory.handle();
      handle hQuery = TypeFactory.handle();
      integer iLastRowNum = TypeFactory.integer();
      logical lLastRowInBatch = TypeFactory.logical();
      logical lAny = TypeFactory.logical();
      character cRowMod = TypeFactory.character();
      rowid rRowid = TypeFactory.rowid();
      
      internalProcedure(new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("RowObject"));
         hRowObject.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("RowObjUpd"));
         hRowObjUpd.assign(ghProp.unwrapBufferField().value());
         rRowid.assign(hRowObject.unwrapBuffer().rowID());
         ControlFlowOps.invokeIn("doUndoDelete", targetProcedure());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("LastRowNum"));
         iLastRowNum.assign(ghProp.unwrapBufferField().value());
         QueryWrapper.createQuery(hQuery);
         hQuery.unwrapQuery().setBuffers(hRowObject);
         hQuery.unwrapQuery().prepare(concat("FOR EACH ", hRowObject.unwrap().name(), " WHERE RowMod > \"\""));
         hQuery.unwrapQuery().queryOpen();
         hQuery.unwrapQuery().getFirst();
         
         loopLabel9:
         while (_not(hQuery.unwrapQuery().isOffEnd()))
         {
            cRowMod.assign(hRowObject.unwrapBuffer().bufferField("RowMod").unwrapBufferField().value());
            
            /* new record */
            if (_or(isEqual(cRowMod, "A"), () -> isEqual(cRowMod, "C")))
            {
               lAny.assign(true);
               
               /* If a new record is last, we know that we have the original new 
                        record in the batch, because that's the requirement for a new record 
                        to be marked as last. */
               if (_isEqual(hRowObject.unwrapBuffer().bufferField("RowNum").unwrapBufferField().value(), iLastRowNum))
               {
                  lLastRowInBatch.assign(true);
               }
               
               hRowObject.unwrapBuffer().deleteRecord();
            }
            else
            {
               if (_isEqual(cRowMod, "U"))
               {
                  ErrorManager.silentErrorEnable();
                  hRowObjUpd.unwrapBuffer().findUnique(concat("WHERE RowMod = \"\" AND RowNum = ", valueOf(hRowObject.unwrapBuffer().bufferField("RowNum").unwrapBufferField().value())));
                  ErrorManager.silentErrorDisable();
                  
                  if ((hRowObjUpd.unwrapBuffer().available()).booleanValue())
                  {
                     lAny.assign(true);
                     hRowObject.unwrapBuffer().bufferCopy(hRowObjUpd);
                  }
               }
            }
            hQuery.unwrapQuery().getNext();
         }
         
         /* for each RowObject */
         HandleOps.delete(hQuery);
         
         /* We have undone an Added record that was last and know that the real last
              is in the batch */
         if ((lLastRowInBatch).booleanValue())
         {
            ErrorManager.silentErrorEnable();
            hRowObject.unwrapBuffer().findLast();
            ErrorManager.silentErrorDisable();
            
            if ((hRowObject.unwrapBuffer().available()).booleanValue())
            {
               ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
               ghProp.assign(ghProp.unwrapBuffer().bufferField("LastRowNum"));
               ghProp.unwrapBufferField().changeValue(hRowObject.unwrapBuffer().bufferField("RowNum").unwrapBufferField().value());
               ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
               ghProp.assign(ghProp.unwrapBuffer().bufferField("LastDbRowIdent"));
               ghProp.unwrapBufferField().changeValue(new character(""));
               /* not needed if lastrownum is set */
            }
         }
         
         /* lLastrowInbatch */
         if ((hRowObjUpd.isValid()).booleanValue())
         {
            ErrorManager.silentErrorEnable();
            /* For repositioning, locate the first Update or Delete record thats being
                   undone and position there. */
            hRowObjUpd.unwrapBuffer().findFirst(new character("WHERE RowObjUpd.RowMod = \"\" OR RowObjUpd.RowMod = \"D\" USE-INDEX RowNum"));
            ErrorManager.silentErrorDisable();
         }
         
         if (_and(hRowObjUpd.isValid(), () -> hRowObjUpd.unwrapBuffer().available()))
         {
            ErrorManager.silentErrorEnable();
            hRowObject.unwrapBuffer().findUnique(concat("WHERE RowNum = ", valueOf(hRowObjUpd.unwrapBuffer().bufferField("RowNum").unwrapBufferField().value())));
            ErrorManager.silentErrorDisable();
         }
         else
         {
            if ((lAny).booleanValue())
            {
               ErrorManager.silentErrorEnable();
               hRowObject.unwrapBuffer().findLast();
               ErrorManager.silentErrorDisable();
            }
            else
            {
               if (_notUnknown(rRowid))
               {
                  ErrorManager.silentErrorEnable();
                  hRowObject.unwrapBuffer().findByRowID(rRowid);
                  ErrorManager.silentErrorDisable();
               }
 /* just in case this was called with no trans */               else
               {
                  ErrorManager.silentErrorEnable();
                  hRowObject.unwrapBuffer().findFirst();
                  ErrorManager.silentErrorDisable();
               }
 /* this would only happen of no RowObject was avail before this started */;
            }
         }
         
         if (_not(hRowObject.unwrapBuffer().available()))
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("FirstRowNum"));
            ghProp.unwrapBufferField().changeValue(new unknown());
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("LastRowNum"));
            ghProp.unwrapBufferField().changeValue(new unknown());
         }
         
         ControlFlowOps.invokeIn("updateQueryPosition", targetProcedure());
         
         if ((hRowObjUpd.isValid()).booleanValue())
         {
            hRowObjUpd.unwrapBuffer().deleteAll();
         }
         
         returnNormal();
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     Supports a cancelRow by copying the current RowObjUpd record back <br>
    *              to the current RowObject record. <br>
    * Parameters:  <br>
    * Notes:       From 9.1C the rollback of updates is really no longer needed by <br>
    *              the visual objects, the only case where it would happen was <br>
    *              with AutoCommit off, but it would roll back ALL changes, which<br>
    *              was quite confusing if changes had been done in several steps.
    */
   public void doUndoUpdate()
   {
      handle hRowObject = TypeFactory.handle();
      handle hRowObjUpd = TypeFactory.handle();
      handle hDataQuery = TypeFactory.handle();
      rowid rRowObject = TypeFactory.rowid();
      logical lNewMode = TypeFactory.logical();
      
      internalProcedure(new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("RowObject"));
         hRowObject.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("RowObjUpd"));
         hRowObjUpd.assign(ghProp.unwrapBufferField().value());
         
         if ((hRowObject.unwrapBuffer().available()).booleanValue())
         {
            if ((hRowObjUpd.isValid()).booleanValue())
            {
               ErrorManager.silentErrorEnable();
               hRowObjUpd.unwrapBuffer().findUnique(concat("WHERE RowMod = \"\" AND RowNum = ", valueOf(hRowObject.unwrapBuffer().bufferField("RowNum").unwrapBufferField().value())));
               ErrorManager.silentErrorDisable();
            }
            
            if (_and(hRowObjUpd.isValid(), () -> hRowObjUpd.unwrapBuffer().available()))
            {
               hRowObject.unwrapBuffer().bufferCopy(hRowObjUpd);
               hRowObjUpd.unwrapBuffer().deleteRecord();
               NamedEventManager.publish("dataAvailable", targetProcedure(), "I", new Object[]
               {
                  "SAME"
               });
            }
 /* may not be there if never saved */            else
            {
               /* Check new mode to avoid deleting new uncommitted records in updatemode*/
               /* {get Prop var [handle]} */ 
               lNewMode.assign(ControlFlowOps.invokeDynamicFunctionIn(logical.class, "getNewMode", targetProcedure()));
               
               if (_and(lNewMode, () -> or(isEqual(hRowObject.unwrapBuffer().bufferField("RowMod").unwrapBufferField().value(), "A"), () -> isEqual(hRowObject.unwrapBuffer().bufferField("RowMod").unwrapBufferField().value(), "C"))))
               {
                  hRowObject.unwrapBuffer().deleteRecord();
                  /* Tell a browse updateSource to get rid of the insert-row */
                  NamedEventManager.publish("cancelNew", targetProcedure());
                  ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
                  ghProp.assign(ghProp.unwrapBuffer().bufferField("DataHandle"));
                  hDataQuery.assign(ghProp.unwrapBufferField().value());
                  ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
                  ghProp.assign(ghProp.unwrapBuffer().bufferField("CurrentRowid"));
                  rRowObject.assign(ghProp.unwrapBufferField().value());
                  
                  if (_and(hDataQuery.unwrapQuery().isOpen(), () -> notUnknown(rRowObject)))
                  {
                     ErrorManager.silentErrorEnable();
                     hDataQuery.unwrapQuery().queryRepositionByID(rRowObject);
                     ErrorManager.silentErrorDisable();
                     
                     /* Next needed if not browser (Data-Target) */
                     if (_and(not(hRowObject.unwrapBuffer().available()), () -> isEqual(ErrorManager.getErrorText(1), "")))
                     {
                        ErrorManager.silentErrorEnable();
                        hDataQuery.unwrapQuery().getNext();
                        ErrorManager.silentErrorDisable();
                     }
                  }
               }
            }
         }
         /* if avail rowobject */
         returnNormal();
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * ***********************  Function Implementations****************
    */
   public logical getAsynchronousSdo()
   {
      /*------------------------------------------------------------------------------
        Purpose:  
          Notes:  
      ------------------------------------------------------------------------------*/
      logical lAsynchronousSdo_1 = TypeFactory.logical();
      
      return function("getAsynchronousSDO", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("AsynchronousSDO"));
         lAsynchronousSdo_1.assign(ghProp.unwrapBufferField().value());
         returnNormal(lAsynchronousSdo_1);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical getAutoCommit()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Returns a flag indicating whether a Commit happens on every 
                  Record update.
         Params:  <none>
      ------------------------------------------------------------------------------*/
      logical lAuto = TypeFactory.logical();
      
      return function("getAutoCommit", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("AutoCommit"));
         lAuto.assign(ghProp.unwrapBufferField().value());
         returnNormal(lAuto);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical getCheckCurrentChanged()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Returns a flag indicating whether the DataObject code should check
                  whether the database row(s) being updated have been changed since read.
         Params:  <none>
          Notes:  TRUE by default.
      ------------------------------------------------------------------------------*/
      logical lCheck = TypeFactory.logical();
      
      return function("getCheckCurrentChanged", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("CheckCurrentChanged"));
         lCheck.assign(ghProp.unwrapBufferField().value());
         returnNormal(lCheck);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public handle getCommitSource()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Returns the handle of the object's CommitSource.
         Params:  <none>
      ------------------------------------------------------------------------------*/
      handle hSource = TypeFactory.handle();
      
      return function("getCommitSource", handle.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("CommitSource"));
         hSource.assign(ghProp.unwrapBufferField().value());
         returnNormal(hSource);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getCommitSourceEvents()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Returns the list of events this object subscribes to in its
                  Commit-Source
         Params:  <none>
      ------------------------------------------------------------------------------*/
      character cEvents = TypeFactory.character();
      
      return function("getCommitSourceEvents", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("CommitSourceEvents"));
         cEvents.assign(ghProp.unwrapBufferField().value());
         returnNormal(cEvents);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getCommitTarget()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Returns the handle of the object's CommitTarget, in character form.
         Params:  <none>
      ------------------------------------------------------------------------------*/
      character cTarget = TypeFactory.character();
      
      return function("getCommitTarget", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("CommitTarget"));
         cTarget.assign(ghProp.unwrapBufferField().value());
         returnNormal(cTarget);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getCommitTargetEvents()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Returns the list of events this object subscribes to in its
                  Commit-Target
         Params:  <none>
      ------------------------------------------------------------------------------*/
      character cEvents = TypeFactory.character();
      
      return function("getCommitTargetEvents", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("CommitTargetEvents"));
         cEvents.assign(ghProp.unwrapBufferField().value());
         returnNormal(cEvents);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical getCurrentRowModified()
   {
      /*------------------------------------------------------------------------------
        Purpose:     Returns TRUE if any values in the current RowObject row
                     have been modified.  If there is no current Rowobject record,
                     then getCurrentRowModified returns ?.
         
        Parameters:  <none>
      ------------------------------------------------------------------------------*/
      handle hRowObject = TypeFactory.handle();
      handle hRowMod = TypeFactory.handle();
      
      return function("getCurrentRowModified", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("RowObject"));
         hRowObject.assign(ghProp.unwrapBufferField().value());
         
         if (_not(hRowObject.unwrapBuffer().available()))
         {
            returnNormal(new logical());
         }
         else
         {
            hRowMod.assign(hRowObject.unwrapBuffer().bufferField("RowMod"));
            returnNormal(isEqual(hRowMod.unwrapBufferField().value(), "U"));
         }
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public handle getCurrentUpdateSource()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Returns the current updateSource 
          Notes:  This is just set temporarily in updateState before re-publishing 
                  updateState, so that the updateSource/DataTarget can avoid a 
                  republish when it is the original publisher.
      ------------------------------------------------------------------------------*/
      handle hSource = TypeFactory.handle();
      
      return function("getCurrentUpdateSource", handle.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("CurrentUpdateSource"));
         hSource.assign(ghProp.unwrapBufferField().value());
         returnNormal(hSource);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getDataDelimiter()
   {
      /*------------------------------------------------------------------------------
        Purpose: Delimiter for 'open' APIs receiveData and input-output in 
                 updateData and createData  
          Notes:  
      ------------------------------------------------------------------------------*/
      character cDelimiter = TypeFactory.character();
      
      return function("getDataDelimiter", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DataDelimiter"));
         cDelimiter.assign(ghProp.unwrapBufferField().value());
         returnNormal(cDelimiter);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getDataFieldDefs()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Returns the name of the include file in which the field 
                  definitions for this SDO's RowObject table are stored.
         Params:  <none>  
      ------------------------------------------------------------------------------*/
      character cDefs = TypeFactory.character();
      
      return function("getDataFieldDefs", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DataFieldDefs"));
         cDefs.assign(ghProp.unwrapBufferField().value());
         returnNormal(cDefs);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public handle getDataHandle()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Returns the handle to the temp-table query
         Params:  <none>
      ------------------------------------------------------------------------------*/
      handle hData = TypeFactory.handle();
      
      return function("getDataHandle", handle.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DataHandle"));
         hData.assign(ghProp.unwrapBufferField().value());
         returnNormal(hData);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public handle getDataLogicObject()
   {
      /*------------------------------------------------------------------------------
        Purpose: Returns the handle of the logic procedure that contains business
                 logic for the data object. 
          Notes: initializeLogicObject starts the logical object using the 
                 DataLogicProcedure property and stores its handle in this property.
      ------------------------------------------------------------------------------*/
      handle hDataLogicObject = TypeFactory.handle();
      
      return function("getDataLogicObject", handle.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DataLogicObject"));
         hDataLogicObject.assign(ghProp.unwrapBufferField().value());
         returnNormal(hDataLogicObject);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getDataLogicProcedure()
   {
      /*------------------------------------------------------------------------------
        Purpose: Returns the name of the logic procedure that contains  business
                 logic for the data object 
          Notes:  
      ------------------------------------------------------------------------------*/
      character cDataLogicProcedure = TypeFactory.character();
      
      return function("getDataLogicProcedure", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DataLogicProcedure"));
         cDataLogicProcedure.assign(ghProp.unwrapBufferField().value());
         returnNormal(cDataLogicProcedure);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical getDataModified()
   {
      /*------------------------------------------------------------------------------
        Purpose:     Returns TRUE if the current RowObject record is modified, 
                     Returns no if there is no current RowObject.
        Parameters:  <none>  
        Notes:       We need to check updateTargets since this may be called from 
                     the toolbar as a result of the updateSource's  
                     setDataModifed -> publish updateState, BEFORE the updateState
                     reaches us...                       
      ------------------------------------------------------------------------------*/
      logical lDataModified = TypeFactory.logical();
      character cUpdateSource = TypeFactory.character();
      integer iSource = TypeFactory.integer();
      handle hSource = TypeFactory.handle();
      
      return function("getDataModified", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DataModified"));
         lDataModified.assign(ghProp.unwrapBufferField().value());
         
         if (_isEqual(lDataModified, false))
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("UpdateSource"));
            cUpdateSource.assign(ghProp.unwrapBufferField().value());
            
            IntegerExpression resExpr0 = new IntegerExpression()
            {
               public integer execute()
               {
                  return numEntries(cUpdateSource);
               }
            };
            ToClause toClause0 = new ToClause(iSource, 1, resExpr0);
            
            doTo("loopLabel10", toClause0, new Block((Body) () -> 
            {
               hSource.assign(handle.fromString(entry(iSource, cUpdateSource)));
               
               if ((hSource.isValid()).booleanValue())
               {
                  /* {get Prop var [handle]} */
                  lDataModified.assign(ControlFlowOps.invokeDynamicFunctionIn(logical.class, "getDataModified", hSource));
                  
                  if ((lDataModified).booleanValue())
                  {
                     leave("loopLabel10");
                  }
               }
            }));
         }
         returnNormal(lDataModified);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical getDataQueryBrowsed()
   {
      /*------------------------------------------------------------------------------
        Purpose:      Returns TRUE if this SmartDataObject's Query is being browsed 
                      by a SmartDataBrowser.
        
        Parametrers:  <none>
        
        Notes:        This is used to prevent two SmartDataBrowsers from attempting 
                      to browse the same query, which is not allowed because 
                      conflicts would occur.
      ------------------------------------------------------------------------------*/
      logical lBrowsed = TypeFactory.logical();
      
      return function("getDataQueryBrowsed", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DataQueryBrowsed"));
         lBrowsed.assign(ghProp.unwrapBufferField().value());
         returnNormal(lBrowsed);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getDataQueryString()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Returns the string used to prepare the RowObject query  
          Notes:  
      ------------------------------------------------------------------------------*/
      character cQueryString = TypeFactory.character();
      
      return function("getDataQueryString", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DataQueryString"));
         cQueryString.assign(ghProp.unwrapBufferField().value());
         returnNormal(cQueryString);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public handle getDataReadBuffer()
   {
      /*------------------------------------------------------------------------------
        Purpose:  
          Notes:  
      ------------------------------------------------------------------------------*/
      handle hBuffer = TypeFactory.handle();
      
      return function("getDataReadBuffer", handle.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DataReadBuffer"));
         hBuffer.assign(ghProp.unwrapBufferField().value());
         returnNormal(hBuffer);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getDataReadColumns()
   {
      /*------------------------------------------------------------------------------
        Purpose:  
          Notes:  
      ------------------------------------------------------------------------------*/
      character cColumns = TypeFactory.character();
      
      return function("getDataReadColumns", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DataReadColumns"));
         cColumns.assign(ghProp.unwrapBufferField().value());
         returnNormal(cColumns);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getDataReadFormat()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Format for columns passed to receiveData and for output in 
                  updateData and createData 
          Notes:  
      ------------------------------------------------------------------------------*/
      character cFormat = TypeFactory.character();
      
      return function("getDataReadFormat", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DataReadFormat"));
         cFormat.assign(ghProp.unwrapBufferField().value());
         returnNormal(cFormat);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public handle getDataReadHandler()
   {
      /*------------------------------------------------------------------------------
        Purpose:  
          Notes:  
      ------------------------------------------------------------------------------*/
      handle hHandle = TypeFactory.handle();
      
      return function("getDataReadHandler", handle.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DataReadHandler"));
         hHandle.assign(ghProp.unwrapBufferField().value());
         returnNormal(hHandle);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getDataSignature()
   {
      /*------------------------------------------------------------------------------
        Purpose:     Returns a character string value which is a list of integers 
                     corresponding to the datatypes of the fields in the RowObject 
                     Temp-Table, for use in comparing objects for equivalence.
      
        Parameters:  <none>
        
        Notes:       A SmartDataObject and a SmartDataBrowser which have the same 
                     DataSignature will be compatible; the SmartDataObject's query 
                     can be used by the Browser.  The integer values are the same 
                     codes used in the _dtype field in the schema:
                                     1 = CHARACTER
                                     2 = DATE
                                     3 = LOGICAL
                                     4 = INTEGER
                                     5 = DECIMAL
                                     6 = Reserved for FLOAT OR DOUBLE in the future
                                     7 = RECID
                                     8 = RAW
                                     9 = Reserved for IMAGE in the future
                                    10 = HANDLE
                                    13 = ROWID
                                        
      ------------------------------------------------------------------------------*/
      character cSignature = TypeFactory.character("");
      integer iCol = TypeFactory.integer();
      handle hRowObject = TypeFactory.handle();
      character cDataType = TypeFactory.character();
      handle hColumn = TypeFactory.handle();
      
      return function("getDataSignature", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("RowObject"));
         hRowObject.assign(ghProp.unwrapBufferField().value());
         
         if ((hRowObject.isValid()).booleanValue())
         {
            loopLabel11:
            for (iCol.assign(1); _isLessThanOrEqual(iCol, hRowObject.unwrapBuffer().numFields()); iCol.increment())
            {
               hColumn.assign(hRowObject.unwrapBuffer().bufferField(iCol));
               cDataType.assign(hColumn.unwrapCommonField().getDataType());
               /* Note: Float/Double reserved for possible future use. */
               cSignature.assign(concat(cSignature, (_isEqual(cDataType, "CHARACTER") ? new character("1") : _isEqual(cDataType, "DATE") ? new character("2") : _isEqual(cDataType, "LOGICAL") ? new character("3") : _isEqual(cDataType, "INTEGER") ? new character("4") : _isEqual(cDataType, "DECIMAL") ? new character("5") : _or(isEqual(cDataType, "FLOAT"), () -> isEqual(cDataType, "DOUBLE")) ? new character("6") : _isEqual(cDataType, "RECID") ? new character("7") : _isEqual(cDataType, "RAW") ? new character("8") : _isEqual(cDataType, "HANDLE") ? new character("10") : _isEqual(cDataType, "ROWID") ? new character("13") : new character("0"))));
            }
         }
         
         returnNormal(cSignature);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getDbnames()
   {
      /*------------------------------------------------------------------------------
         Purpose:  Returns a comma delimited list of DBNames that corresponds 
                   to the Tables in the Query Objects
      Parameters:  <none>
           Notes: This override is just in case the property is referrenced before
                  the sdo or sbo has copied the property from the server.           
      ------------------------------------------------------------------------------*/
      character cDbnames = TypeFactory.character();
      handle hAsHandle = TypeFactory.handle();
      character cAsDivision = TypeFactory.character();
      character cContAsdiv = TypeFactory.character();
      handle hContainer = TypeFactory.handle();
      logical lQueryObject = TypeFactory.logical();
      
      return function("getDBNames", character.class, new Block((Body) () -> 
      {
         cDbnames.assign(((character) ControlFlowOps.runSuper("getDBNames")));
         
         if (_isUnknown(cDbnames))
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("ContainerSource"));
            hContainer.assign(ghProp.unwrapBufferField().value());
            ErrorManager.silentErrorEnable();
            ghProp.assign(handle.fromString(entry(1, hContainer.unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("queryObject"));
            lQueryObject.assign(ghProp.unwrapBufferField().value());
            ErrorManager.silentErrorDisable();
            
            if ((lQueryObject).booleanValue())
            {
               ErrorManager.silentErrorEnable();
               /* {get Prop var [handle]} */
               cContAsdiv.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getASDivision", hContainer));
               ErrorManager.silentErrorDisable();
            }
            
            if (_isEqual(cContAsdiv, "Client"))
            {
               ControlFlowOps.invokeIn("startServerObject", hContainer);
            }
            else
            {
               /* {get Prop var [handle]} */ 
               cAsDivision.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getASDivision", targetProcedure()));
               
               if (_isEqual(cAsDivision, "Client"))
               {
                  /* {get Prop var [handle]} */ 
                  hAsHandle.assign(ControlFlowOps.invokeDynamicFunctionIn(handle.class, "getASHandle", targetProcedure()));
               }
            }
            
            if (_or(isEqual(cContAsdiv, "client"), () -> isEqual(cAsDivision, "Client")))
            {
               /* Check again as this is should be retrieved at start up fromthe calls 
                       above */
               cDbnames.assign(((character) ControlFlowOps.runSuper("getDBNames")));
               
               /* Just in case something went wrong go and get it */
               if (_isUnknown(cDbnames))
               {
                  /* {get Prop var [handle]} */ 
                  hAsHandle.assign(ControlFlowOps.invokeDynamicFunctionIn(handle.class, "getAshandle", targetProcedure()));
                  
                  if (_and(hAsHandle.isValid(), () -> isNotEqual(hAsHandle, targetProcedure())))
                  {
                     cDbnames.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getDBNames", hAsHandle));
                     /* {set Prop value [handle]} */ 
                     ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "setDBNames", targetProcedure(), "I", cDbnames);
                  }
               }
               /* We may need to unbind if this call did the bind (getASHandle) */
               ControlFlowOps.invokeInWithMode("unbindServer", targetProcedure(), "I", new unknown());
            }
         }
         /* IF DBNames not yet defined locally. */
         returnNormal(cDbnames);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical getDestroyStateless()
   {
      /*------------------------------------------------------------------------------
        Purpose: Defines if the persistent sdo should be destroyed on stateless requests
          Notes: This is only possible to set to false in WebSpeed (default). 
      ------------------------------------------------------------------------------*/
      logical lDestroy = TypeFactory.logical();
      
      return function("getDestroyStateless", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DestroyStateless"));
         lDestroy.assign(ghProp.unwrapBufferField().value());
         returnNormal(lDestroy);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical getDisconnectAppServer()
   {
      /*------------------------------------------------------------------------------
        Purpose: Should the persistent sdo disconnect the AppServer.  
          Notes: This is only used for stateless WebSpeed SDO's that never are destroyed 
      ------------------------------------------------------------------------------*/
      logical lDisconnect = TypeFactory.logical();
      
      return function("getDisconnectAppServer", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DisconnectAppServer"));
         lDisconnect.assign(ghProp.unwrapBufferField().value());
         returnNormal(lDisconnect);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical getDynamicData()
   {
      /*------------------------------------------------------------------------------
        Purpose: Returns true if this dataobject is using dynamic defined temp-tables  
          Notes:  
      ------------------------------------------------------------------------------*/
      logical lDynamic = TypeFactory.logical();
      
      return function("getDynamicData", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DynamicData"));
         lDynamic.assign(ghProp.unwrapBufferField().value());
         returnNormal(lDynamic);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical getFillBatchOnRepos()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Returns (LOGICAL) a flag indicating whether fetchRowIdent should
                  retrieve enough rows to fill a batch of records when repositioning
                  to the end or near the end of the dataset where an entire batch
                  wouldn't be retrieved.
         Params:  <none>
      ------------------------------------------------------------------------------*/
      logical lFillBatch = TypeFactory.logical();
      
      return function("getFillBatchOnRepos", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("fillBatchOnRepos"));
         lFillBatch.assign(ghProp.unwrapBufferField().value());
         returnNormal(lFillBatch);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public handle getFilterSource()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Returns the handle of the object's FilterSource.
         Params:  <none>
      ------------------------------------------------------------------------------*/
      handle hSource = TypeFactory.handle();
      
      return function("getFilterSource", handle.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FilterSource"));
         hSource.assign(ghProp.unwrapBufferField().value());
         returnNormal(hSource);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getFilterWindow()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Returns the Partition name which this object 
                  will run on, if any.
         Params:  none
      ------------------------------------------------------------------------------*/
      character cFilterWindow = TypeFactory.character();
      
      return function("getFilterWindow", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FilterWindow"));
         cFilterWindow.assign(ghProp.unwrapBufferField().value());
         returnNormal(cFilterWindow);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getForeignFieldsContainer()
   {
      /*------------------------------------------------------------------------------
        Purpose:  
          Notes:  
      ------------------------------------------------------------------------------*/
      character cObjectName = TypeFactory.character();
      character cCurrField = TypeFactory.character();
      character cForFields = TypeFactory.character();
      handle hSource = TypeFactory.handle();
      integer iCol = TypeFactory.integer();
      character cObjectFf = TypeFactory.character();
      
      return function("getForeignFieldsContainer", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ObjectName"));
         cObjectName.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ContainerSource"));
         hSource.assign(ghProp.unwrapBufferField().value());
         /* {get Prop var [handle]} */
         cForFields.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getForeignFields", hSource));
         
         loopLabel12:
         for (iCol.assign(1); _isLessThanOrEqual(iCol, numEntries(cForFields)); iCol.assign(plus(iCol, 2)))
         {
            cCurrField.assign(entry(iCol, cForFields));
            
            if (_and(isGreaterThan(numEntries(cCurrField, "."), 1), () -> isEqual(entry(1, cCurrField, "."), cObjectName)))
            {
               cObjectFf.assign(concat(cObjectFf, (_isEqual(cObjectFf, "") ? new character("") : new character(",")), cCurrField, ",", entry(plus(iCol, 1), cForFields)));
            }
         }
         
         /* no FF match our Object Name */
         returnNormal(cObjectFf);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getForeignValues()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Returns the ForeignValues  .
         Params:  none
      ------------------------------------------------------------------------------*/
      character cForeignValues = TypeFactory.character();
      
      return function("getForeignValues", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ForeignValues"));
         cForeignValues.assign(ghProp.unwrapBufferField().value());
         returnNormal(cForeignValues);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical getHasNewRow()
   {
      /*------------------------------------------------------------------------------
        Purpose:  
          Notes:  
      ------------------------------------------------------------------------------*/
      handle hTable = TypeFactory.handle();
      handle hBuffer = TypeFactory.handle();
      logical lHasNewRow = TypeFactory.logical();
      
      return function("getHasNewRow", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("RowObjUpd"));
         hTable.assign(ghProp.unwrapBufferField().value());
         
         if (_not(hTable.isValid()))
         {
            returnNormal(false);
         }
         
         RecordBuffer.create(hBuffer, hTable);
         ErrorManager.silentErrorEnable();
         hBuffer.unwrapBuffer().findFirst(new character("WHERE RowMod = 'A' or RowMod = 'C'"));
         ErrorManager.silentErrorDisable();
         lHasNewRow.assign(hBuffer.unwrapBuffer().available());
         HandleOps.delete(hBuffer);
         returnNormal(lHasNewRow);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getIndexInformation()
   {
      /*------------------------------------------------------------------------------
        Purpose: Return indexinFormation formatted as the 4GL index-information 
                 attribute, but with RowObject column names and chr(1) as index 
                 separator and chr(2) as table separator.  
          Notes: - Intended for internal use by other index info functions, which uses 
                   this as input to indexInformation(). 
                 - Unmapped columns are returned fully qualifed!      
                 - This property can be used as input parameter to indexInformation() 
                   for further refinement.
                 - If the property is ? it calls the indexInformation() in query.p
                   and stores the returned value for future calls. 
                   
                 - If a similar list with database fields are needed use 
                   indexInformation() directly (if connected)             
      ------------------------------------------------------------------------------*/
      character cInfo = TypeFactory.character();
      character cTableIndexes = TypeFactory.character();
      character cTableList = TypeFactory.character();
      character cTable = TypeFactory.character();
      character cIndex = TypeFactory.character();
      character cField = TypeFactory.character();
      character cColumn = TypeFactory.character();
      integer iIndex = TypeFactory.integer();
      integer iTable = TypeFactory.integer();
      integer iField = TypeFactory.integer();
      character cAsDivision = TypeFactory.character();
      character cContAsDiv = TypeFactory.character();
      handle hAppServer = TypeFactory.handle();
      handle hContainer = TypeFactory.handle();
      logical lQueryObject = TypeFactory.logical();
      
      return function("getIndexInformation", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("IndexInformation"));
         cInfo.assign(ghProp.unwrapBufferField().value());
         
         if (_or(isUnknown(cInfo), () -> isEqual(cInfo, "")))
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("ContainerSource"));
            hContainer.assign(ghProp.unwrapBufferField().value());
            ErrorManager.silentErrorEnable();
            ghProp.assign(handle.fromString(entry(1, hContainer.unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("queryObject"));
            lQueryObject.assign(ghProp.unwrapBufferField().value());
            ErrorManager.silentErrorDisable();
            
            if ((lQueryObject).booleanValue())
            {
               ErrorManager.silentErrorEnable();
               /* {get Prop var [handle]} */
               cContAsDiv.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getASDivision", hContainer));
               ErrorManager.silentErrorDisable();
            }
            
            if (_isEqual(cContAsDiv, "Client"))
            {
               ControlFlowOps.invokeIn("startServerObject", hContainer);
            }
            else
            {
               /* {get Prop var [handle]} */ 
               cAsDivision.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getASDivision", targetProcedure()));
               
               if (_isEqual(cAsDivision, "Client"))
               {
                  /* {get Prop var [handle]} */ 
                  hAppServer.assign(ControlFlowOps.invokeDynamicFunctionIn(handle.class, "getASHandle", targetProcedure()));
               }
            }
            
            if (_or(isEqual(cContAsDiv, "client"), () -> isEqual(cAsDivision, "Client")))
            {
               ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
               ghProp.assign(ghProp.unwrapBuffer().bufferField("IndexInformation"));
               cInfo.assign(ghProp.unwrapBufferField().value());
               
               /* It should have been found above, but in case sometihn is wrong let's
                        go and get it */
               if (_isUnknown(cInfo))
               {
                  /* {get Prop var [handle]} */ 
                  hAppServer.assign(ControlFlowOps.invokeDynamicFunctionIn(handle.class, "getASHandle", targetProcedure()));
                  
                  if (_and(hAppServer.isValid(), () -> isNotEqual(hAppServer, targetProcedure())))
                  {
                     /* {get Prop var [handle]} */
                     cInfo.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getIndexInformation", hAppServer));
                  }
               }
               /* We may need to unbind if this call did the bind (getASHandle) */
               ControlFlowOps.invokeInWithMode("unbindServer", targetProcedure(), "I", new unknown());
            }
            else
            {
               cInfo.assign(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "indexInformation", targetProcedure(), "III", "Info", "yes", new unknown()));
               ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
               ghProp.assign(ghProp.unwrapBuffer().bufferField("Tables"));
               cTableList.assign(ghProp.unwrapBufferField().value());
               
               loopLabel13:
               for (iTable.assign(1); _isLessThanOrEqual(iTable, numEntries(cInfo, chr(2))); iTable.increment())
               {
                  cTableIndexes.assign(entry(iTable, cInfo, chr(2)));
                  cTable.assign(entry(iTable, cTableList));
                  
                  loopLabel14:
                  for (iIndex.assign(1); _isLessThanOrEqual(iIndex, numEntries(cTableIndexes, chr(1))); iIndex.increment())
                  {
                     cIndex.assign(entry(iIndex, cTableIndexes, chr(1)));
                     
                     IntegerExpression resExpr1 = new IntegerExpression()
                     {
                        public integer execute()
                        {
                           return numEntries(cIndex);
                        }
                     };
                     ToClause toClause1 = new ToClause(iField, 5, resExpr1, new integer(2));
                     
                     doTo("loopLabel15", toClause1, new Block((Body) () -> 
                     {
                        cField.assign(concat(cTable, ".", entry(iField, cIndex)));
                        /* Is there a mapped RowObject Column? */
                        cColumn /* {fnarg <func-name> <arg> [handle] */ .assign(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "dbColumnDataName", targetProcedure(), "I", cField));
                        
                        if (_notUnknown(cColumn))
                        {
                           cIndex.replaceEntry(iField, cColumn);
                        }
                        else
                        {
                           cIndex.replaceEntry(iField, cField);
                        }
                     }));
                     
                     /* iField = 5 to num-entries by 2 */
                     cTableIndexes.replaceEntry(iIndex, chr(1), cIndex);
                  }
                  
                  /* iIndex = 1 to num indexes in tableindexes */
                  cInfo.replaceEntry(iTable, chr(2), cTableIndexes);
               }
            }
            /* else not 'client' */
            /* Store it for next time */
            /* {set Prop value [handle]} */ 
            ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "setIndexInformation", targetProcedure(), "I", cInfo);
         }
         /* if cInfo = ? or cInfo = '' */
         returnNormal(cInfo);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getInternalEntries()
   {
      return function("getInternalEntries", character.class, new Block((Body) () -> 
      {
         /*------------------------------------------------------------------------------
           Purpose: To pass back internal entries of SDO as internal-entries cannot be
                    accessed for remote proxy procedures.
             Notes:  
         ------------------------------------------------------------------------------*/
         returnNormal(thisProcedure().unwrapProcedure().internalEntries());
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical getIsRowObjectExternal()
   {
      /*------------------------------------------------------------------------------
        Purpose:  
          Notes:  
      ------------------------------------------------------------------------------*/
      logical lVar = TypeFactory.logical();
      
      return function("getIsRowObjectExternal", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("IsRowObjectExternal"));
         lVar.assign(ghProp.unwrapBufferField().value());
         returnNormal(lVar);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical getIsRowObjUpdExternal()
   {
      /*------------------------------------------------------------------------------
        Purpose: Indicates that the rowObjUpd table has been passed from 
                 somewhere else (usually the client)   
          Notes:  
      ------------------------------------------------------------------------------*/
      logical lExternal = TypeFactory.logical();
      
      return function("getIsRowObjUpdExternal", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("IsRowObjUpdExternal"));
         lExternal.assign(ghProp.unwrapBufferField().value());
         returnNormal(lExternal);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getKeyTableId()
   {
      /*------------------------------------------------------------------------------
        Purpose:     Returns the tableid for the KeyFields.
        Parameters:  <none>
        Notes:       This is normally the first enabled table, and is usually only
                     used on the server side to join to comments/auditing etc
                     This is also the dump name of that table
      ------------------------------------------------------------------------------*/
      character cKeyTableId = TypeFactory.character();
      handle hAppServer = TypeFactory.handle();
      character cAsdivision = TypeFactory.character();
      character cContAsDiv = TypeFactory.character();
      handle hContainer = TypeFactory.handle();
      logical lQueryObject = TypeFactory.logical();
      
      return function("getKeyTableId", character.class, new Block((Body) () -> 
      {
         cKeyTableId.assign(((character) ControlFlowOps.runSuper("getKeyTableId")));
         
         if (_isEqual(cKeyTableId, ""))
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("ContainerSource"));
            hContainer.assign(ghProp.unwrapBufferField().value());
            ErrorManager.silentErrorEnable();
            ghProp.assign(handle.fromString(entry(1, hContainer.unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("queryObject"));
            lQueryObject.assign(ghProp.unwrapBufferField().value());
            ErrorManager.silentErrorDisable();
            
            if ((lQueryObject).booleanValue())
            {
               ErrorManager.silentErrorEnable();
               /* {get Prop var [handle]} */
               cContAsDiv.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getASDivision", hContainer));
               ErrorManager.silentErrorDisable();
            }
            
            if (_isEqual(cContAsDiv, "Client"))
            {
               ControlFlowOps.invokeIn("startServerObject", hContainer);
            }
            else
            {
               /* {get Prop var [handle]} */ 
               cAsdivision.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getASDivision", targetProcedure()));
               
               /* If we're on the client then the AppServer haven't return this property 
                        value yet. */
               if (_isEqual(cAsdivision, "Client"))
               {
                  /* This will retrieve all Server properties including KeyTable */
                  /* {get Prop var [handle]} */ 
                  hAppServer.assign(ControlFlowOps.invokeDynamicFunctionIn(handle.class, "getASHandle", targetProcedure()));
               }
            }
            
            if (_or(isEqual(cContAsDiv, "Client"), () -> isEqual(cAsdivision, "Client")))
            {
               cKeyTableId.assign(((character) ControlFlowOps.runSuper("getKeyTableId")));
               
               /* We should have got it now, but if something is wrong let's just go 
                        and get it ourselves */
               if (_isEqual(cKeyTableId, ""))
               {
                  /* {get Prop var [handle]} */ 
                  hAppServer.assign(ControlFlowOps.invokeDynamicFunctionIn(handle.class, "getAsHandle", targetProcedure()));
                  
                  if (_and(hAppServer.isValid(), () -> isNotEqual(hAppServer, targetProcedure())))
                  {
                     cKeyTableId.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getKeyTableId", hAppServer));
                     
                     if (_isNotEqual(cKeyTableId, ""))
                     {
                        /* {set Prop value [handle]} */ 
                        ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "setKeyTableId", targetProcedure(), "I", cKeyTableId);
                     }
                  }
               }
               /* We may need to unbind if this call did the bind (getASHandle) */
               ControlFlowOps.invokeInWithMode("unbindServer", targetProcedure(), "I", new unknown());
            }
         }
         /* END DO IF OpenQuery not yet defined locally. */
         returnNormal(cKeyTableId);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getLastCommitErrorKeys()
   {
      /*------------------------------------------------------------------------------
        Purpose: Returns a list of key values separated by 'DataDelimiter' identifying
                 the records that fail to be committed on the last time data was committed.
                 Blank  - indicates that the last commit was successful, 
      ------------------------------------------------------------------------------*/
      character cLastCommitErrorKeys = TypeFactory.character();
      
      return function("getLastCommitErrorKeys", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("LastCommitErrorKeys"));
         cLastCommitErrorKeys.assign(ghProp.unwrapBufferField().value());
         returnNormal(cLastCommitErrorKeys);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getLastCommitErrorType()
   {
      /*------------------------------------------------------------------------------
        Purpose: Returns the type of error encountered the last time data was committed.
                 Blank  - indicates that the last commit was successful, 
                 unknown - indicates that a commit never has been attempted after run
                 Conflict - Locking conflict
                 Error    - Unspecified 
          Notes: Currently used to identify a Conflict error after having used the 
                 UpdateData API. More types may be added later.   
      ------------------------------------------------------------------------------*/
      character cLastCommitErrorType = TypeFactory.character();
      
      return function("getLastCommitErrorType", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("LastCommitErrorType"));
         cLastCommitErrorType.assign(ghProp.unwrapBufferField().value());
         returnNormal(cLastCommitErrorType);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getManualAddQueryWhere()
   {
      /*------------------------------------------------------------------------------
        Purpose: Retrieve manual calls to addquerywhere so that filter can reapply this
                 when filter is changed, thus ensuring the original query stays intact. 
          Notes: Value is pcwhere + chr(3) + pcbuffer or empty or "?" + chr(3) + pcandor 
                 Note that multiple entries are supported, seperated by chr(4).
      ------------------------------------------------------------------------------*/
      character cString_3 = TypeFactory.character();
      
      return function("getManualAddQueryWhere", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ManualAddQueryWhere"));
         cString_3.assign(ghProp.unwrapBufferField().value());
         returnNormal(cString_3);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getManualAssignQuerySelection()
   {
      /*------------------------------------------------------------------------------
        Purpose: Retrieve manual calls to assignqueryselection so that filter can reapply this
                 when filter is changed, thus ensuring the original query stays intact. 
          Notes: Value is pccolumns + chr(3) + pcvalues + chr(3) + pcoperators 
                 Note that multiple entries are supported, seperated by chr(4).
      ------------------------------------------------------------------------------*/
      character cString_3 = TypeFactory.character();
      
      return function("getManualAssignQuerySelection", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ManualAssignQuerySelection"));
         cString_3.assign(ghProp.unwrapBufferField().value());
         returnNormal(cString_3);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getManualSetQuerySort()
   {
      /*------------------------------------------------------------------------------
        Purpose: Retrieve manual calls to setquerysort so that filter can reapply this
                 when filter is changed, thus ensuring the original query stays intact. 
          Notes: Value is pcsort 
                 Note that multiple entries are supported, seperated by chr(4).
      ------------------------------------------------------------------------------*/
      character cString_3 = TypeFactory.character();
      
      return function("getManualSetQuerySort", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ManualSetQuerySort"));
         cString_3.assign(ghProp.unwrapBufferField().value());
         returnNormal(cString_3);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical getNewMode()
   {
      /*------------------------------------------------------------------------------
        Purpose:     Returns TRUE if the current RowObject record is in new mode, 
                     which is the same as the Object is in NewMode.   
                    - (an add or a copy of an existing record has NOT been saved)                   
                     Returns ? if there is no current RowObject.
        Parameters:  <none>  
        Notes:       The difference from getNewRow is that it returns true for any 
                     saved and uncommitted new record and thus cannot be used
                     to check whether we are in Add Mode.
                     This uses the RowMod field in the Temp-Table to see if the 
                     row's new (just as getNewRow) and in addition checks to see
                     if the RowObjUpd is not valid or not avail, which indicates that 
                     this has not been saved. 
                     We do some double checking if a rowObjUpd is avail to ensure 
                     that this is the right one.                 
      ------------------------------------------------------------------------------*/
      handle hRowObject = TypeFactory.handle();
      handle hRowObjUpd = TypeFactory.handle();
      handle hRowMod = TypeFactory.handle();
      logical lNewMode = TypeFactory.logical();
      
      return function("getNewMode", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("RowObject"));
         hRowObject.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("RowObjUpd"));
         hRowObjUpd.assign(ghProp.unwrapBufferField().value());
         
         if (_or(not(hRowObject.isValid()), () -> not(hRowObject.unwrapBuffer().available())))
         {
            returnNormal(new logical());
         }
         
         /* create a buffer for the upd table (avoid messing with the real one)
              and see if it has a corresponding record */
         if ((hRowObjUpd.isValid()).booleanValue())
         {
            RecordBuffer.create(hRowObjUpd, hRowObjUpd);
            /* find the corresponding rowObjUpd -- buffer that is */
            ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "findRowObjUpd", targetProcedure(), "II", hRowObject, hRowObjUpd);
         }
         
         hRowMod.assign(hRowObject.unwrapBuffer().bufferField("RowMod"));
         lNewMode.assign(and(or(isEqual(hRowMod.unwrapBufferField().value(), "A"), () -> isEqual(hRowMod.unwrapBufferField().value(), "C")), () -> or(not(hRowObjUpd.isValid()), () -> not(hRowObjUpd.unwrapBuffer().available()))));
         ErrorManager.silentErrorEnable();
         /* Delete the locally created buffer (no-error as it may not have been created) */
         HandleOps.delete(hRowObjUpd);
         ErrorManager.silentErrorDisable();
         returnNormal(lNewMode);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical getNewRow()
   {
      /*------------------------------------------------------------------------------
        Purpose:     Returns TRUE if the current RowObject record is new - (an added
                     record or a copy of an existing record has never been written to
                     the database.)  Returns ? if there is no current RowObject.
      
        Parameters:  <none>
        
        Notes:       This uses the RowMod field in the Temp-Table to see if the 
                     row's new.
      ------------------------------------------------------------------------------*/
      handle hRowObject = TypeFactory.handle();
      handle hColumn = TypeFactory.handle();
      
      return function("getNewRow", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("RowObject"));
         hRowObject.assign(ghProp.unwrapBufferField().value());
         
         if (_or(not(hRowObject.isValid()), () -> not(hRowObject.unwrapBuffer().available())))
         {
            returnNormal(false);
         }
         else
         {
            hColumn.assign(hRowObject.unwrapBuffer().bufferField("RowMod"));
            
            if (_or(isEqual(hColumn.unwrapBufferField().value(), "A"), () -> isEqual(hColumn.unwrapBufferField().value(), "C")))
            {
               returnNormal(true);
            }
            else
            {
               returnNormal(false);
            }
         }
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getOpenQuery()
   {
      /*------------------------------------------------------------------------------
        Purpose:     Returns the original design where-clause for the database query.
        Parameters:  <none>
        Notes:       This is normally not modified, and is retrieved at startup by
                     a client-side SDO proxy or dynamic SDO, so if it's defined,
                     just use the local value; otherwise ask the server-side SDO.
      ------------------------------------------------------------------------------*/
      character cQuery = TypeFactory.character();
      handle hAppServer = TypeFactory.handle();
      character cAsdivision = TypeFactory.character();
      handle hContainer = TypeFactory.handle();
      logical lQueryObject = TypeFactory.logical();
      
      return function("getOpenQuery", character.class, new Block((Body) () -> 
      {
         cQuery.assign(((character) ControlFlowOps.runSuper("getOpenQuery")));
         
         if (_isEqual(cQuery, ""))
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("ContainerSource"));
            hContainer.assign(ghProp.unwrapBufferField().value());
            ErrorManager.silentErrorEnable();
            ghProp.assign(handle.fromString(entry(1, hContainer.unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("queryObject"));
            lQueryObject.assign(ghProp.unwrapBufferField().value());
            ErrorManager.silentErrorDisable();
            
            if ((lQueryObject).booleanValue())
            {
               ErrorManager.silentErrorEnable();
               /* {get Prop var [handle]} */
               cAsdivision.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getASDivision", hContainer));
               ErrorManager.silentErrorDisable();
            }
            
            /* If we're on the client inside of a container then the container 
                    haven't return this property from the appserver yet. */
            if (_isEqual(cAsdivision, "Client"))
            {
               ControlFlowOps.invokeIn("startServerObject", hContainer);
            }
            else
            {
               /* {get Prop var [handle]} */ 
               cAsdivision.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getASDivision", targetProcedure()));
               
               /* If we're on the client then the AppServer haven't return this property 
                         value yet. */
               if (_isEqual(cAsdivision, "Client"))
               {
                  /* This will retrieve all Server properties including openQuery */
                  /* {get Prop var [handle]} */ 
                  hAppServer.assign(ControlFlowOps.invokeDynamicFunctionIn(handle.class, "getASHandle", targetProcedure()));
               }
            }
            /* The property should now be available, so try again */
            cQuery.assign(((character) ControlFlowOps.runSuper("getOpenQuery")));
            
            /* We should have got it now, but if something is wrong let's just go 
                    and get it ourselves */
            if (_and(isEqual(cAsdivision, "client"), () -> isEqual(cQuery, "")))
            {
               /* {get Prop var [handle]} */ 
               hAppServer.assign(ControlFlowOps.invokeDynamicFunctionIn(handle.class, "getASHandle", targetProcedure()));
               
               if (_and(hAppServer.isValid(), () -> isNotEqual(hAppServer, targetProcedure())))
               {
                  cQuery.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getOpenQuery", hAppServer));
                  ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
                  ghProp.assign(ghProp.unwrapBuffer().bufferField("BaseQuery"));
                  ghProp.unwrapBufferField().changeValue(cQuery);
               }
            }
            /* We may need to unbind if this call did the bind (getASHandle) */
            ControlFlowOps.invokeInWithMode("unbindServer", targetProcedure(), "I", new unknown());
         }
         /* END DO IF OpenQuery not yet defined locally. */
         returnNormal(cQuery);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public handle getPrimarySdosource()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Returns the handle of the object's PrimarySDOSource.
         Params:  <none>
      ------------------------------------------------------------------------------*/
      handle hSource = TypeFactory.handle();
      
      return function("getPrimarySDOSource", handle.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("PrimarySDOSource"));
         hSource.assign(ghProp.unwrapBufferField().value());
         returnNormal(hSource);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getPromptColumns()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Return a list of field values when prompting for a delete action
          Notes:  
      ------------------------------------------------------------------------------*/
      character cPromptColumns = TypeFactory.character();
      
      return function("getPromptColumns", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("PromptColumns"));
         cPromptColumns.assign(ghProp.unwrapBufferField().value());
         
         if (_isEqual(cPromptColumns, ""))
         {
            cPromptColumns.assign(((gshSessionManager.isValid()).booleanValue() ? new character("(ALL)") : new character("(NONE)")));
         }
         
         returnNormal(cPromptColumns);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical getPromptOnDelete()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Is the user to be prompted before a delete action executes?
          Notes:  
      ------------------------------------------------------------------------------*/
      logical lPrompt = TypeFactory.logical();
      
      return function("getPromptOnDelete", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("PromptOnDelete"));
         lPrompt.assign(ghProp.unwrapBufferField().value());
         returnNormal(lPrompt);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical getQueryContainer()
   {
      /*------------------------------------------------------------------------------
        Purpose: Returns a flag indicating whether our Container is itself a
                 QueryObject. Used to determine whether we're in an SBO which handles
                 the transaction for us. 
         Params: <none>
      ------------------------------------------------------------------------------*/
      logical lQuery = TypeFactory.logical();
      
      return function("getQueryContainer", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryContainer"));
         lQuery.assign(ghProp.unwrapBufferField().value());
         returnNormal(lQuery);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getQueryContext()
   {
      /*------------------------------------------------------------------------------
        Purpose: Returns the queryContext on the client 
      Parameter:  
          Notes: For INTERNAL use only , exists as function so that it can be 
                 accessed in generic calls.     
      ------------------------------------------------------------------------------*/
      character cQueryContext = TypeFactory.character();
      
      return function("getQueryContext", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryContext"));
         cQueryContext.assign(ghProp.unwrapBufferField().value());
         returnNormal(cQueryContext);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical getQueryOpen()
   {
      /*------------------------------------------------------------------------------
          Purpose:  Returns TRUE if the Query is currently open.
       Parameters:  <none>
            Notes:  Overrides query to resolve this on the client when this is 
                    not the server side object.
                 -  The need to check the rowobject query is important for the case 
                    when it is 'client'. 
                 -  From the dataobject's perspective the server side query is less 
                    important as an object state, but the Database query is open a 
                    (little) while before the rowObject query, so it is still checked 
                    in that case. 
       ----------------------------------------------------------------------------*/
      handle hDataQuery = TypeFactory.handle();
      character cAsDivision = TypeFactory.character();
      
      return function("getQueryOpen", logical.class, new Block((Body) () -> 
      {
         /* {get Prop var [handle]} */ 
         cAsDivision.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getAsDivision", targetProcedure()));
         
         if (_isEqual(cAsDivision, "Client"))
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("DataHandle"));
            hDataQuery.assign(ghProp.unwrapBufferField().value());
            returnNormal(_not(hDataQuery.isValid()) ? new logical(false) : hDataQuery.unwrapQuery().isOpen());
         }
         else
         {
            returnNormal((logical) ControlFlowOps.runSuper("getQueryOpen"));
         }
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getQueryRowIdent()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Returns the property which holds a RowIdent to be used to
                  position an SDO query when it if first opened.
          Params: <none>
          Notes:  Generally used to save the position of a query when it is
                  closed so that position can be restored on re-open.
      ------------------------------------------------------------------------------*/
      character cRowIdent = TypeFactory.character();
      
      return function("getQueryRowIdent", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryRowIdent"));
         cRowIdent.assign(ghProp.unwrapBufferField().value());
         returnNormal(cRowIdent);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getQueryWhere()
   {
      /*------------------------------------------------------------------------------
        Purpose:     Returns the current where-clause for the database query.
        Parameters:  <none>
        Notes:       (See getOpenQuery for the original where clause.)
                     The Where clause is stored locally on the client for statless 
                     SDOs. 
                     restartServerObject will use it  
      ------------------------------------------------------------------------------*/
      character cQuery = TypeFactory.character();
      handle hAppServer = TypeFactory.handle();
      character cAsdivision = TypeFactory.character();
      character cOperatingMode = TypeFactory.character();
      logical lAsHasStarted = TypeFactory.logical();
      
      return function("getQueryWhere", character.class, new Block((Body) () -> 
      {
         /* {get Prop var [handle]} */ 
         cAsdivision.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getASDivision", targetProcedure()));
         
         if (_isEqual(cAsdivision, "Client"))
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryContext"));
            cQuery.assign(ghProp.unwrapBufferField().value());
            
            if (_isUnknown(cQuery))
            {
               /* If not Appserver has not started return the default query */
               /* {get Prop var [handle]} */ 
               lAsHasStarted.assign(ControlFlowOps.invokeDynamicFunctionIn(logical.class, "getAsHasStarted", targetProcedure()));
               
               /* This code block has never been in use, but was almost added when the 
                        logic to avoid server hits if not AsHasStarted was added, so it's kept 
                        here for information and possible future considerations:
                         ---------------  
                         All code should currently handle queryWhere = ?, so this is probably
                         not necessary (maybe even wrong since the query is not opened and 
                        the querywhere is unknown )       
                     IF NOT lAsHasStarted THEN
                     DO:
                       {get OpenQuery cQuery}. 
                     END.  */
               /* This should never happen as this is part of the context, but 
                        just in case let's support a direct appserver call */
               if ((lAsHasStarted).booleanValue())
               {
                  /* {get Prop var [handle]} */ 
                  hAppServer.assign(ControlFlowOps.invokeDynamicFunctionIn(handle.class, "getASHandle", targetProcedure()));
                  
                  if (_and(hAppServer.isValid(), () -> isNotEqual(hAppServer, targetProcedure())))
                  {
                     cQuery.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getQueryWhere", hAppServer));
                     /* {get Prop var [handle]} */ 
                     cOperatingMode.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getserverOperatingMode", targetProcedure()));
                     
                     /* We store the query locally for next time for stateless SDOs */
                     if (_isEqual(cOperatingMode, "STATELESS"))
                     {
                        /* unbind if this call did the bind (getASHandle) */
                        ControlFlowOps.invokeInWithMode("unbindServer", targetProcedure(), "I", new unknown());
                     }
                     
                     ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
                     ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryContext"));
                     ghProp.unwrapBufferField().changeValue(cQuery);
                  }
               }
            }
            /* IF QueryWhere not yet defined locally. */
            returnNormal(cQuery);
         }
         else
         {
            returnNormal((character) ControlFlowOps.runSuper("getQueryWhere"));
         }
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical getRebuildOnRepos()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Returns the flag indicating whether the RowObject temp-table
                  should be rebuilt if a fetchLast or other reposition is done
                  which is outside the bounds of the current dataset.
         Params:  <none>
      ------------------------------------------------------------------------------*/
      logical lRebuild = TypeFactory.logical();
      
      return function("getRebuildOnRepos", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("RebuildOnRepos"));
         lRebuild.assign(ghProp.unwrapBufferField().value());
         returnNormal(lRebuild);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getRowIdent()
   {
      /*------------------------------------------------------------------------------
        Purpose:     This is a comma delimited character string containing the ROWID 
                     of the database record(s) that are the source of the RowObject 
                     record.  
        Parameters:  <none>
        Notes:       This RowIdent key can be used to reposition to that row using
                     fetchRowIdent.
      ------------------------------------------------------------------------------*/
      handle hDataQuery = TypeFactory.handle();
      handle hRow = TypeFactory.handle();
      
      return function("getRowIdent", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DataHandle"));
         hDataQuery.assign(ghProp.unwrapBufferField().value());
         
         if ((hDataQuery.isValid()).booleanValue())
         {
            hRow.assign(hDataQuery.unwrapQuery().bufferHandle(1));
            
            if (_and(hRow.isValid(), () -> hRow.unwrapBuffer().available()))
            {
               hRow.assign(hRow.unwrapBuffer().bufferField("RowIdent"));
               returnNormal(hRow.unwrapBufferField().value());
            }
            else
            {
               returnNormal(new character());
            }
         }
         else
         {
            returnNormal(new character());
         }
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public handle getRowObject()
   {
      /*------------------------------------------------------------------------------
        Purpose:     Returns the handle of the RowObject Temp-Table buffer.
        
        Parameters:  <none>
      ------------------------------------------------------------------------------*/
      handle hRowObject = TypeFactory.handle();
      
      return function("getRowObject", handle.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("RowObject"));
         hRowObject.assign(ghProp.unwrapBufferField().value());
         returnNormal(hRowObject);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getRowObjectState()
   {
      /*------------------------------------------------------------------------------
        Purpose:     Retrieves the property value which signals whether there are
                     uncommitted updates in the object.  
       
        Parameters:  <none>
        
        Note:        The two possible return values are: 'NoUpdates' and 'RowUpdated'
      ------------------------------------------------------------------------------*/
      character cState = TypeFactory.character();
      
      return function("getRowObjectState", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("RowObjectState"));
         cState.assign(ghProp.unwrapBufferField().value());
         returnNormal(cState);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public handle getRowObjectTable()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Returns the Temp-Table handle of the RowObject table.
         Params:  none
         Notes:   Note that this is different from the RowObject property,
                  which is the handle of the RowObject Buffer. This function
                  does not use the {get} syntax because the setRowObjectTable
                  function must also do other work.
      ------------------------------------------------------------------------------*/
      handle hTable = TypeFactory.handle();
      
      return function("getRowObjectTable", handle.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("RowObjectTable"));
         hTable.assign(ghProp.unwrapBufferField().value());
         returnNormal(hTable);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public handle getRowObjUpd()
   {
      /*------------------------------------------------------------------------------
        Purpose:     Returns the handle of the RowObjUpd Temp-Table buffer where 
                     Updates are stored.
        
        Parameters:  <none>
      ------------------------------------------------------------------------------*/
      handle hRowObjUpd = TypeFactory.handle();
      
      return function("getRowObjUpd", handle.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("RowObjUpd"));
         hRowObjUpd.assign(ghProp.unwrapBufferField().value());
         returnNormal(hRowObjUpd);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public handle getRowObjUpdTable()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Returns the handle to the RowObjUpd Temp-Table
         Params:  none
      ------------------------------------------------------------------------------*/
      handle hTable = TypeFactory.handle();
      
      return function("getRowObjUpdTable", handle.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("RowObjUpdTable"));
         hTable.assign(ghProp.unwrapBufferField().value());
         returnNormal(hTable);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public integer getRowsToBatch()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Returns the number of rows to be transferred from the database
                  query into the RowObject temp-table at a time.
         Params:  <none>
          Notes:  set to 200 by default. 0 = ALL 
      ------------------------------------------------------------------------------*/
      integer iRows = TypeFactory.integer();
      
      return function("getRowsToBatch", integer.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("RowsToBatch"));
         iRows.assign(ghProp.unwrapBufferField().value());
         returnNormal(iRows);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical getRunDataLogicProxy()
   {
      /*------------------------------------------------------------------------------
        Purpose:  
          Notes:  If TRUE, only the "_cl.r" version of the data logic procedure 
                  will be run. If it cannot be found, the associated Data Object 
                  is not run.
      ------------------------------------------------------------------------------*/
      logical lRunProxy = TypeFactory.logical();
      
      return function("getRunDataLogicProxy", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("RunDataLogicProxy"));
         lRunProxy.assign(ghProp.unwrapBufferField().value());
         returnNormal(lRunProxy);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getSchemaLocation()
   {
      /*------------------------------------------------------------------------------
        Purpose:  The Schema Location decides how the ROwObject is defined in dynamic 
                  SDOs
                  - 'ENT' or Blank - Default (from Entity Cache)
                  - 'DLP' - Use DataLogicProcedure
                  - 'BUF' - Use the buffer in the query  
                  
         Notes: - If using ENTity cache all fields must exist in the entity cache.
                - If using DLP any missing field will be searched for in the buffer   
                - Setting it to 'DLP' or 'BUF', (no difference) for static SDOs avoids
                  getting format and labels from the Entity cache. Static SDOs will 
                  not fail if a field is missing from the entities or an Entity is 
                  missing.
      ------------------------------------------------------------------------------*/
      character cSchemaLocation = TypeFactory.character();
      
      return function("getSchemaLocation", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("SchemaLocation"));
         cSchemaLocation.assign(ghProp.unwrapBufferField().value());
         returnNormal(cSchemaLocation);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical getServerSubmitValidation()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Returns the value of the property which signals whether the
                  column and RowObject Validation procedures done as part of
                  client validation are to be executed on the server side. 
                  If *yes*, normally when the SDO is being run through the 
                  open client interface, then serverCommit will execute 
                  SubmitValidation itself.
          Notes:  Because the set function verifies that the property is not
                  reset from *yes* to *no* (for security purposes), the property 
                  value is always accessed through the set and get functions.
      ------------------------------------------------------------------------------*/
      logical lVal = TypeFactory.logical();
      
      return function("getServerSubmitValidation", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ServerSubmitValidation"));
         lVal.assign(ghProp.unwrapBufferField().value());
         returnNormal(lVal);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical getToggleDataTargets()
   {
      /*------------------------------------------------------------------------------
        Purpose: Returns true if dataTargets should be toggled on/of in LinkState
                 based on the passed 'active' or 'inactive' parameter 
          Notes:  
      ------------------------------------------------------------------------------*/
      logical ltoggle = TypeFactory.logical();
      
      return function("getToggleDataTargets", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("toggleDataTargets"));
         ltoggle.assign(ghProp.unwrapBufferField().value());
         returnNormal(ltoggle);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical getUpdateFromSource()
   {
      /*------------------------------------------------------------------------------
        Purpose: Returns true if this object should be updated as one-to-one 
                 of the datasource updates (one-to-one) 
          Notes:  
      ------------------------------------------------------------------------------*/
      logical lUpdate = TypeFactory.logical();
      
      return function("getUpdateFromSource", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("UpdateFromSource"));
         lUpdate.assign(ghProp.unwrapBufferField().value());
         returnNormal(lUpdate);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getUpdateSource()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Returns the handle of the object's UpdateSource.
         Params:  <none>
      ------------------------------------------------------------------------------*/
      character cSource = TypeFactory.character();
      
      return function("getUpdateSource", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("UpdateSource"));
         cSource.assign(ghProp.unwrapBufferField().value());
         returnNormal(cSource);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical getUseStaticOnFetch()
   {
      /*------------------------------------------------------------------------------
        Purpose: Indicates that server fetch must use the static TT. 
          Notes: This is required when there are code that sets data in the static 
                 RowObject table during the fetch operation. This is true if the 
                 SDO has a default calculated field.
                 It could also need to be set to true if there are overrides in any of
                 the fetch event procedures, like sendRows and transferDbRow.        
      ------------------------------------------------------------------------------*/
      logical lStaticFetch = TypeFactory.logical();
      logical lDestroy = TypeFactory.logical();
      
      return function("getUseStaticOnFetch", logical.class, new Block((Body) () -> 
      {
         if ((matchesList(targetProcedure().unwrapProcedure().internalEntries(), "Data.Calculate")).booleanValue())
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("DestroyStateless"));
            lDestroy.assign(ghProp.unwrapBufferField().value());
            
            /* We assume this is a server-only function so that we don't have */
            /* to check the ASDivision property */
            if ((lDestroy).booleanValue())
            {
               lStaticFetch.assign(true);
            }
         }
         returnNormal(lStaticFetch);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getWordIndexedFields()
   {
      /*------------------------------------------------------------------------------
        Purpose: Returns a comma separated list of RowObject fields that are 
                 mapped to database fields that has a word indexed.   
          Notes: This overrides query.p version completely.  
      ------------------------------------------------------------------------------*/
      character cIndexInfo = TypeFactory.character();
      character cFieldList = TypeFactory.character();
      character cColumnList = TypeFactory.character();
      character cColumn = TypeFactory.character();
      integer i = TypeFactory.integer();
      
      return function("getWordIndexedFields", character.class, new Block((Body) () -> 
      {
         /* The indexInformation contains all indexes for this dataobject */
         /* {get Prop var [handle]} */ 
         cIndexInfo.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getIndexInformation", targetProcedure()));
         
         if (_notUnknown(cIndexInfo))
         {
            /* Get the word indexes from the IndexInformation function */
            cFieldList.assign(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "indexInformation", targetProcedure(), "III", "WORD", "no", cIndexInfo));
            
            loopLabel16:
            for (i.assign(1); _isLessThanOrEqual(i, numEntries(cFieldList, chr(1))); i.increment())
            {
               cColumn.assign(entry(i, cFieldList, chr(1)));
               
               if (_isEqual(indexOf(cColumn, "."), 0))
               {
                  cColumnList.assign(concat(cColumnList, (_isEqual(cColumnList, "") ? new character("") : new character(",")), cColumn));
               }
            }
         }
         /* cinfo <> ? */
         returnNormal(trim(cColumnList, ","));
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setAsynchronousSdo(final logical _lAsynchronousSdo_1)
   {
      logical lAsynchronousSdo_1 = TypeFactory.initInput(_lAsynchronousSdo_1);
      
      return function("setAsynchronousSDO", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("AsynchronousSDO"));
         ghProp.unwrapBufferField().changeValue(lAsynchronousSdo_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setAutoCommit(final logical _plFlag_2)
   {
      logical plFlag_2 = TypeFactory.initInput(_plFlag_2);
      /*------------------------------------------------------------------------------
        Purpose:  Sets the AutoCommit flag on or off; when On, every call to submitRow
                  will result in a Commit.
         Params:  plFlag AS LOGICAL  -- If true, AutoCommit is set On.
      ------------------------------------------------------------------------------*/
      logical lQuery = TypeFactory.logical();
      
      return function("setAutoCommit", logical.class, new Block((Body) () -> 
      {
         /* IF The QueryObject property has been set off, then this object is just
              being used for update of another object's data. AutoCommit must be true
              because there is no temp-table to store uncommitted changes in. */
         if (_not(plFlag_2))
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryObject"));
            lQuery.assign(ghProp.unwrapBufferField().value());
            
            if (_not(lQuery))
            {
               returnNormal(false);
            }
         }
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("AutoCommit"));
         ghProp.unwrapBufferField().changeValue(plFlag_2);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setCheckCurrentChanged(final logical _plCheck_1)
   {
      logical plCheck_1 = TypeFactory.initInput(_plCheck_1);
      
      return function("setCheckCurrentChanged", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("CheckCurrentChanged"));
         ghProp.unwrapBufferField().changeValue(plCheck_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setClientProxyHandle(final character _pcClientProxy_1)
   {
      character pcClientProxy_1 = TypeFactory.initInput(_pcClientProxy_1);
      
      return function("setClientProxyHandle", logical.class, new Block((Body) () -> 
      {
         /*------------------------------------------------------------------------------
           Purpose: To store the character version of the client-side SDO handle
            Params: The string containing the client-side SDO procedure handle
             Notes:  
         ------------------------------------------------------------------------------*/
         /* {set Prop value [handle]} */ 
         ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "setClientProxyHandle", targetProcedure(), "I", pcClientProxy_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setCommitSource(final handle _phObject_3)
   {
      handle phObject_3 = TypeFactory.initInput(_phObject_3);
      
      return function("setCommitSource", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("CommitSource"));
         ghProp.unwrapBufferField().changeValue(phObject_3);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setCommitSourceEvents(final character _pcEvents_2)
   {
      character pcEvents_2 = TypeFactory.initInput(_pcEvents_2);
      
      return function("setCommitSourceEvents", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("CommitSourceEvents"));
         ghProp.unwrapBufferField().changeValue(pcEvents_2);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setCommitTarget(final character _pcObject_3)
   {
      character pcObject_3 = TypeFactory.initInput(_pcObject_3);
      
      return function("setCommitTarget", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("CommitTarget"));
         ghProp.unwrapBufferField().changeValue(pcObject_3);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setCommitTargetEvents(final character _pcEvents_2)
   {
      character pcEvents_2 = TypeFactory.initInput(_pcEvents_2);
      
      return function("setCommitTargetEvents", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("CommitTargetEvents"));
         ghProp.unwrapBufferField().changeValue(pcEvents_2);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setCurrentUpdateSource(final handle _phSource_1)
   {
      handle phSource_1 = TypeFactory.initInput(_phSource_1);
      
      return function("setCurrentUpdateSource", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("CurrentUpdateSource"));
         ghProp.unwrapBufferField().changeValue(phSource_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setDataDelimiter(final character _pcDelimiter_1)
   {
      character pcDelimiter_1 = TypeFactory.initInput(_pcDelimiter_1);
      
      return function("setDataDelimiter", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DataDelimiter"));
         ghProp.unwrapBufferField().changeValue(pcDelimiter_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setDataLogicObject(final handle _phDataLogicObject_1)
   {
      handle phDataLogicObject_1 = TypeFactory.initInput(_phDataLogicObject_1);
      
      return function("setDataLogicObject", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DataLogicObject"));
         ghProp.unwrapBufferField().changeValue(phDataLogicObject_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setDataLogicProcedure(final character _pcDataLogicProcedure_1)
   {
      character pcDataLogicProcedure_1 = TypeFactory.initInput(_pcDataLogicProcedure_1);
      
      return function("setDataLogicProcedure", logical.class, new Block((Body) () -> 
      {
         /*------------------------------------------------------------------------------
           Purpose: Set the name of the logic procedure that contains business logic for 
                    the data object. 
             Notes:  
         ------------------------------------------------------------------------------*/
         if (_isGreaterThan(pcDataLogicProcedure_1, ""))
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("DataLogicProcedure"));
            ghProp.unwrapBufferField().changeValue(pcDataLogicProcedure_1);
            /* Add the datalogic procedure as a super  */
            ControlFlowOps.invokeIn("initializeLogicObject", targetProcedure());
         }
         
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setDataModified(final logical _plDataModified_1)
   {
      logical plDataModified_1 = TypeFactory.initInput(_plDataModified_1);
      
      return function("setDataModified", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DataModified"));
         ghProp.unwrapBufferField().changeValue(plDataModified_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setDataQueryBrowsed(final logical _plBrowsed_1)
   {
      logical plBrowsed_1 = TypeFactory.initInput(_plBrowsed_1);
      /*------------------------------------------------------------------------------
        Purpose:     Sets a property flag indicating that this object's DataQuery
                     is being browsed by a SmartDataBrowser.
       
        Parameters:
          INPUT plBrowsed - True if a SmartDataBrowser uses this SmartDataObject
                            as a Data-Source.
          
        Notes:       This is set by a SmartDataBrowser which is a Data-Target.
                     The property is used to prevent multiple SmartDataBrowsers 
                     from attempting to browse the same query, which is not 
                     allowed.
      ------------------------------------------------------------------------------*/
      logical lAlreadyBrowsed = TypeFactory.logical();
      
      return function("setDataQueryBrowsed", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DataQueryBrowsed"));
         lAlreadyBrowsed.assign(ghProp.unwrapBufferField().value());
         
         if (_and(plBrowsed_1, lAlreadyBrowsed))
         {
            returnNormal(false);
         }
         
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DataQueryBrowsed"));
         ghProp.unwrapBufferField().changeValue(plBrowsed_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setDataQueryString(final character _pcQueryString_1)
   {
      character pcQueryString_1 = TypeFactory.initInput(_pcQueryString_1);
      
      return function("setDataQueryString", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DataQueryString"));
         ghProp.unwrapBufferField().changeValue(pcQueryString_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setDataReadBuffer(final handle _phBuffer_1)
   {
      handle phBuffer_1 = TypeFactory.initInput(_phBuffer_1);
      
      return function("setDataReadBuffer", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DataReadBuffer"));
         ghProp.unwrapBufferField().changeValue(phBuffer_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setDataReadColumns(final character _pcColumns_1)
   {
      character pcColumns_1 = TypeFactory.initInput(_pcColumns_1);
      
      return function("setDataReadColumns", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DataReadColumns"));
         ghProp.unwrapBufferField().changeValue(pcColumns_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setDataReadFormat(final character _pcFormat_1)
   {
      character pcFormat_1 = TypeFactory.initInput(_pcFormat_1);
      
      return function("setDataReadFormat", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DataReadFormat"));
         ghProp.unwrapBufferField().changeValue(pcFormat_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setDataReadHandler(final handle _phHandle_1)
   {
      handle phHandle_1 = TypeFactory.initInput(_phHandle_1);
      
      return function("setDataReadHandler", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DataReadHandler"));
         ghProp.unwrapBufferField().changeValue(phHandle_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setDestroyStateless(final logical _plDestroy_1)
   {
      logical plDestroy_1 = TypeFactory.initInput(_plDestroy_1);
      
      return function("setDestroyStateless", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DestroyStateless"));
         ghProp.unwrapBufferField().changeValue(plDestroy_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setDisconnectAppserver(final logical _plDisconnect_1)
   {
      logical plDisconnect_1 = TypeFactory.initInput(_plDisconnect_1);
      
      return function("setDisconnectAppserver", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DisconnectAppServer"));
         ghProp.unwrapBufferField().changeValue(plDisconnect_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setDynamicData(final logical _plDynamic_1)
   {
      logical plDynamic_1 = TypeFactory.initInput(_plDynamic_1);
      
      return function("setDynamicData", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DynamicData"));
         ghProp.unwrapBufferField().changeValue(plDynamic_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setFillBatchOnRepos(final logical _plFlag_2)
   {
      logical plFlag_2 = TypeFactory.initInput(_plFlag_2);
      
      return function("setFillBatchOnRepos", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("fillBatchOnRepos"));
         ghProp.unwrapBufferField().changeValue(plFlag_2);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setFilterSource(final handle _phObject_3)
   {
      handle phObject_3 = TypeFactory.initInput(_phObject_3);
      
      return function("setFilterSource", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FilterSource"));
         ghProp.unwrapBufferField().changeValue(phObject_3);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setFilterWindow(final character _pcObject_3)
   {
      character pcObject_3 = TypeFactory.initInput(_pcObject_3);
      
      return function("setFilterWindow", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FilterWindow"));
         ghProp.unwrapBufferField().changeValue(pcObject_3);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setForeignValues(final character _pcValues_1)
   {
      character pcValues_1 = TypeFactory.initInput(_pcValues_1);
      
      return function("setForeignValues", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ForeignValues"));
         ghProp.unwrapBufferField().changeValue(pcValues_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setIndexInformation(final character _pcInfo_1)
   {
      character pcInfo_1 = TypeFactory.initInput(_pcInfo_1);
      
      return function("setIndexInformation", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("IndexInformation"));
         ghProp.unwrapBufferField().changeValue(pcInfo_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setIsRowObjectExternal(final logical _plExternal_2)
   {
      logical plExternal_2 = TypeFactory.initInput(_plExternal_2);
      
      return function("setIsRowObjectExternal", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("IsRowObjectExternal"));
         ghProp.unwrapBufferField().changeValue(plExternal_2);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setIsRowObjUpdExternal(final logical _plExternal_2)
   {
      logical plExternal_2 = TypeFactory.initInput(_plExternal_2);
      
      return function("setIsRowObjUpdExternal", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("IsRowObjUpdExternal"));
         ghProp.unwrapBufferField().changeValue(plExternal_2);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setLastCommitErrorKeys(final character _pcLastKeys_1)
   {
      character pcLastKeys_1 = TypeFactory.initInput(_pcLastKeys_1);
      
      return function("setLastCommitErrorKeys", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("LastCommitErrorKeys"));
         ghProp.unwrapBufferField().changeValue(pcLastKeys_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setLastCommitErrorType(final character _pcLastCommitErrorType_1)
   {
      character pcLastCommitErrorType_1 = TypeFactory.initInput(_pcLastCommitErrorType_1);
      
      return function("setLastCommitErrorType", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("LastCommitErrorType"));
         ghProp.unwrapBufferField().changeValue(pcLastCommitErrorType_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setLastDbRowIdent(final character _pcLastDbRowIdent_1)
   {
      character pcLastDbRowIdent_1 = TypeFactory.initInput(_pcLastDbRowIdent_1);
      
      return function("setLastDbRowIdent", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("LastDbRowIdent"));
         ghProp.unwrapBufferField().changeValue(pcLastDbRowIdent_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setManualAddQueryWhere(final character _cString_3)
   {
      character cString_3 = TypeFactory.initInput(_cString_3);
      
      return function("setManualAddQueryWhere", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ManualAddQueryWhere"));
         ghProp.unwrapBufferField().changeValue(cString_3);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setManualAssignQuerySelection(final character _cString_3)
   {
      character cString_3 = TypeFactory.initInput(_cString_3);
      
      return function("setManualAssignQuerySelection", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ManualAssignQuerySelection"));
         ghProp.unwrapBufferField().changeValue(cString_3);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setManualSetQuerySort(final character _cString_3)
   {
      character cString_3 = TypeFactory.initInput(_cString_3);
      
      return function("setManualSetQuerySort", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ManualSetQuerySort"));
         ghProp.unwrapBufferField().changeValue(cString_3);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setPrimarySdosource(final handle _phObject_3)
   {
      handle phObject_3 = TypeFactory.initInput(_phObject_3);
      
      return function("setPrimarySDOSource", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("PrimarySDOSource"));
         ghProp.unwrapBufferField().changeValue(phObject_3);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setPromptColumns(final character _pcPromptColumns_1)
   {
      character pcPromptColumns_1 = TypeFactory.initInput(_pcPromptColumns_1);
      
      return function("setPromptColumns", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("PromptColumns"));
         ghProp.unwrapBufferField().changeValue(pcPromptColumns_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setPromptOnDelete(final logical _plPrompt_1)
   {
      logical plPrompt_1 = TypeFactory.initInput(_plPrompt_1);
      
      return function("setPromptOnDelete", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("PromptOnDelete"));
         ghProp.unwrapBufferField().changeValue(plPrompt_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setQueryContext(final character _pcQueryContext_1)
   {
      character pcQueryContext_1 = TypeFactory.initInput(_pcQueryContext_1);
      
      return function("setQueryContext", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryContext"));
         ghProp.unwrapBufferField().changeValue(pcQueryContext_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setQueryRowIdent(final character _pcRowIdent_1)
   {
      character pcRowIdent_1 = TypeFactory.initInput(_pcRowIdent_1);
      
      return function("setQueryRowIdent", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryRowIdent"));
         ghProp.unwrapBufferField().changeValue(pcRowIdent_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setQueryWhere(final character _pcWhere_1)
   {
      character pcWhere_1 = TypeFactory.initInput(_pcWhere_1);
      /*------------------------------------------------------------------------------
           Purpose:     
        Parameters:  <none>
             Notes:  (See getOpenQuery for the original where clause.)
                     The Where clause is stored locally on the client
                 -   From 9.1D02 this is done also for state-aware objects since 
                     the context always is passed to the server in all data requests.               
       ------------------------------------------------------------------------------*/
      character cQueryWhere = TypeFactory.character();
      character cAsDivision = TypeFactory.character();
      integer iField = TypeFactory.integer();
      character cLocalFields = TypeFactory.character();
      character cForeignFields = TypeFactory.character();
      character cSourceFields = TypeFactory.character();
      character cValues = TypeFactory.character();
      logical lOk = TypeFactory.logical();
      
      return function("setQueryWhere", logical.class, new Block((Body) () -> 
      {
         /* {get Prop var [handle]} */ 
         cAsDivision.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getAsDivision", targetProcedure()));
         
         if (_isEqual(cAsDivision, "CLIENT"))
         {
            cQueryWhere /* {fnarg <func-name> <arg> [handle] */ .assign(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "newQueryWhere", targetProcedure(), "I", pcWhere_1));
            
            /* Store the query locally */
            if (_notUnknown(cQueryWhere))
            {
               ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
               ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryContext"));
               ghProp.unwrapBufferField().changeValue(cQueryWhere);
            }
            /* {set Prop value [handle]} */ 
            
            lOk.assign(notUnknown(cQueryWhere));
         }
         else
         {
            lOk.assign(((logical) ControlFlowOps.runSuper("setQueryWhere", pcWhere_1)));
         }
         
         /* apply foreign field values if they exist (issue 8056) */
         if (_isNotEqual(cAsDivision, "SERVER"))
         {
            /* {get Prop var [handle]} */ 
            cForeignFields.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getForeignFields", targetProcedure()));
            
            if (_and(isGreaterThan(cForeignFields, ""), () -> notUnknown(cForeignFields)))
            {
               ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
               ghProp.assign(ghProp.unwrapBuffer().bufferField("ForeignValues"));
               cValues.assign(ghProp.unwrapBufferField().value());
               
               if (_notUnknown(cValues))
               {
                  loopLabel17:
                  for (iField.assign(1); _isLessThanOrEqual(iField, numEntries(cForeignFields)); iField.assign(plus(iField, 2)))
                  {
                     cLocalFields.assign(concat(cLocalFields, (_isNotEqual(cLocalFields, "") ? new character(",") : new character("")), entry(iField, cForeignFields)));
                     cSourceFields.assign(concat(cSourceFields, (_isNotEqual(cSourceFields, "") ? new character(",") : new character("")), entry(plus(iField, 1), cForeignFields)));
                  }
                  
                  lOk.assign(ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "assignQuerySelection", targetProcedure(), "III", cLocalFields, cValues, ""));
                  
                  if (_isEqual(cAsDivision, "CLIENT"))
                  {
                     cQueryWhere /* {fn <func-name> [handle] */ .assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getQueryString", targetProcedure()));
                     
                     /* this version now includes FF */
                     if (_notUnknown(cQueryWhere))
                     {
                        ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
                        ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryContext"));
                        ghProp.unwrapBufferField().changeValue(cQueryWhere);
                     }
                     /* {set Prop value [handle]} */ 
                  }
               }
            }
         }
         returnNormal(lOk);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setRebuildOnRepos(final logical _plRebuild_1)
   {
      logical plRebuild_1 = TypeFactory.initInput(_plRebuild_1);
      
      return function("setRebuildOnRepos", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("RebuildOnRepos"));
         ghProp.unwrapBufferField().changeValue(plRebuild_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setRowObject(final handle _phRowObject_1)
   {
      handle phRowObject_1 = TypeFactory.initInput(_phRowObject_1);
      
      return function("setRowObject", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("RowObject"));
         ghProp.unwrapBufferField().changeValue(phRowObject_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setRowObjectState(final character _pcState_1)
   {
      character pcState_1 = TypeFactory.initInput(_pcState_1);
      /*------------------------------------------------------------------------------
        Purpose:  Sets the RowObjectState property, which keeps track of whether
                  there are uncommitted updates in the object.
       
        Parameters:
          INPUT pcState - Can only be 'RowUpdated' OR 'NoUpdates'
          Notes:    When we get 'NoUpdates' and we have a Commit Panel; our 
                    datasource have suppressed any 'UpdateComplete' events
                    they have received until we are committed, so we send that
                    updatestate event now.
      ------------------------------------------------------------------------------*/
      logical lCommit = TypeFactory.logical((Boolean) null);
      
      return function("setRowObjectState", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("RowObjectState"));
         ghProp.unwrapBufferField().changeValue(pcState_1);
         NamedEventManager.publish("rowObjectState", targetProcedure(), "I", new Object[]
         {
            pcState_1
         });
         
         if (_isEqual(pcState_1, "NoUpdates"))
         {
            /* {get Prop var [handle]} */ 
            lCommit.assign(ControlFlowOps.invokeDynamicFunctionIn(logical.class, "getAutoCommit", targetProcedure()));
            
            if (_isEqual(lCommit, false))
            {
               NamedEventManager.publish("updateState", targetProcedure(), "I", new Object[]
               {
                  "UpdateComplete"
               });
            }
         }
         /* Tell container */
         NamedEventManager.publish("UpdateActive", targetProcedure(), "I", new Object[]
         {
            isNotEqual(pcState_1, "NoUpdates")
         });
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setRowObjectTable(final handle _phTable_2)
   {
      handle phTable_2 = TypeFactory.initInput(_phTable_2);
      /*------------------------------------------------------------------------------
        Purpose: Sets the property which stores the RowObject temp-table handle.
         Params: phTable AS HANDLE -- temp-table handle
          Notes: This is the handle to the temp-table itself, not its buffer.
                 Supports dynamic SDO (not valid RowObject) by also setting 
                 RowObject and DataHandle if it is unknown or different.
      ------------------------------------------------------------------------------*/
      character cQueryString = TypeFactory.character();
      handle hRowobject = TypeFactory.handle();
      handle hDataHandle = TypeFactory.handle();
      character cDefs = TypeFactory.character();
      
      return function("setRowObjectTable", logical.class, new Block((Body) () -> 
      {
         /* Support non-persistent callers that want to return the TT directly
              The TT will be prepared in 
               initializeObject -> createObjects -> (defineTempTables) or (prepareRowObject). */
         if (_isEqual(phTable_2.unwrapTempTable().prepared(), false))
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("IsRowObjectExternal"));
            ghProp.unwrapBufferField().changeValue(new logical(true));
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("DataFieldDefs"));
            cDefs.assign(ghProp.unwrapBufferField().value());
            
            /* if a static TT recieves an unprepared we prepare the new TT from
                   the old one immediately */
            if (_isGreaterThan(cDefs, ""))
            {
               /* {fnarg <func-name> <arg> [handle] */ 
               ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "prepareRowObject", targetProcedure(), "I", phTable_2);
            }
         }
         else
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("RowObject"));
            hRowobject.assign(ghProp.unwrapBufferField().value());
            
            /* Existing temp-table buffer. */
            /* We ensure that query and buffer handle properties is correct also */
            if (_or(not(hRowobject.isValid()), () -> isNotEqual(hRowobject, phTable_2.unwrapTempTable().defaultBufferHandle())))
            {
               ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
               ghProp.assign(ghProp.unwrapBuffer().bufferField("RowObject"));
               ghProp.unwrapBufferField().changeValue(phTable_2.unwrapTempTable().defaultBufferHandle());
               ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
               ghProp.assign(ghProp.unwrapBuffer().bufferField("DataHandle"));
               hDataHandle.assign(ghProp.unwrapBufferField().value());
               ErrorManager.silentErrorEnable();
               /* Query handle */
               HandleOps.delete(hDataHandle);
               ErrorManager.silentErrorDisable();
               ErrorManager.silentErrorEnable();
               HandleOps.delete(hRowobject);
               ErrorManager.silentErrorDisable();
               QueryWrapper.createQuery(hDataHandle);
               ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
               ghProp.assign(ghProp.unwrapBuffer().bufferField("DataHandle"));
               ghProp.unwrapBufferField().changeValue(hDataHandle);
               /* Something should be done with the ghDataQuery variable in data.i  */
               hDataHandle.unwrapQuery().setBuffers(phTable_2.unwrapTempTable().defaultBufferHandle());
               ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
               ghProp.assign(ghProp.unwrapBuffer().bufferField("DataQueryString"));
               cQueryString.assign(ghProp.unwrapBufferField().value());
               /* Fix european format issues */
               /* {fnarg <func-name> <arg> [handle] */ 
               hDataHandle.unwrapQuery().prepare(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "fixQueryString", targetProcedure(), "I", cQueryString));
            }
         }
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("RowObjectTable"));
         ghProp.unwrapBufferField().changeValue(phTable_2);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setRowObjUpd(final handle _phRowObjUpd_1)
   {
      handle phRowObjUpd_1 = TypeFactory.initInput(_phRowObjUpd_1);
      
      return function("setRowObjUpd", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("RowObjUpd"));
         ghProp.unwrapBufferField().changeValue(phRowObjUpd_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setRowObjUpdTable(final handle _phTable_2)
   {
      handle phTable_2 = TypeFactory.initInput(_phTable_2);
      /*------------------------------------------------------------------------------
          Purpose: Sets the property which stores the RowObject temp-table handle.
           Params: phTable AS HANDLE -- temp-table handle
            Notes: This is the handle to the temp-table itself, not its buffer.
                   Supports dynamic SDO (not valid RowObject) by also setting 
                   RowObject and DataHandle if it is unknown or different.
                -  setting the TT before initialization marks it as external, 
                   IsRowObjUpdExternal, which ensures that it wont be created during 
                   initialization 
      ------------------------------------------------------------------------------*/
      handle hRowobjUpd = TypeFactory.handle();
      logical lInitialized = TypeFactory.logical();
      
      return function("setRowObjUpdTable", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("RowObjUpd"));
         hRowobjUpd.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ObjectInitialized"));
         lInitialized.assign(ghProp.unwrapBufferField().value());
         
         /* We ensure that buffer property is correct also.     
             Note: we used to also delete the old RowObjUpd, but that caused
             problems running locally with an SBO as it corrupted the  
             default-buffer-handle of the caller. 
             The RowObjUpd TT should/is deleted when required so the buffer delete
             is not needed here */
         if (_or(not(hRowobjUpd.isValid()), () -> isNotEqual(hRowobjUpd, phTable_2.unwrapTempTable().defaultBufferHandle())))
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("RowObjUpd"));
            ghProp.unwrapBufferField().changeValue(phTable_2.unwrapTempTable().defaultBufferHandle());
         }
         /* {set Prop value [handle]} */ 
         
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("RowObjUpdTable"));
         ghProp.unwrapBufferField().changeValue(phTable_2);
         
         /* If the TT is set before initialization mark it as external, so 
             it wont be created during initialization */
         if (_not(lInitialized))
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("IsRowObjUpdExternal"));
            ghProp.unwrapBufferField().changeValue(new logical(true));
         }
         /* {set Prop value [handle]} */ 
         
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setRowsToBatch(final integer _piRows_1)
   {
      integer piRows_1 = TypeFactory.initInput(_piRows_1);
      
      return function("setRowsToBatch", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("RowsToBatch"));
         ghProp.unwrapBufferField().changeValue(piRows_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setRunDataLogicProxy(final logical _plRunProxy_1)
   {
      logical plRunProxy_1 = TypeFactory.initInput(_plRunProxy_1);
      
      return function("setRunDataLogicProxy", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("RunDataLogicProxy"));
         ghProp.unwrapBufferField().changeValue(plRunProxy_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setSchemaLocation(final character _pcSchemaLocation_1)
   {
      character pcSchemaLocation_1 = TypeFactory.initInput(_pcSchemaLocation_1);
      
      return function("setSchemaLocation", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("SchemaLocation"));
         ghProp.unwrapBufferField().changeValue(pcSchemaLocation_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setServerSubmitValidation(final logical _plSubmit_1)
   {
      logical plSubmit_1 = TypeFactory.initInput(_plSubmit_1);
      /*------------------------------------------------------------------------------
        Purpose:  Sets the value of the property which signals whether the
                  column and RowObject Validation procedures done as part of
                  client validation should be executed on the server. 
                  It is *no* by default; an SDO which uses client
                  validation and which may be run from the open client interface
                  should set it to *yes*, either in the SDO itself or at runtime.
                  If it is *no* when serverCommit executes, then serverCommit 
                  will execute SubmitValidation itself.
         Params:  plSubmit AS LOGICAL  -- yes if client validation should be executed
                    in the server SDO.
         Notes:   As a security measure, if the property has been set to *yes*,
                  it cannot be reset to *no*.
      ------------------------------------------------------------------------------*/
      logical lCurrentValue = TypeFactory.logical();
      
      return function("setServerSubmitValidation", logical.class, new Block((Body) () -> 
      {
         /* Cannot be set to NO */
         if (_isEqual(plSubmit_1, false))
         {
            /* {get Prop var [handle]} */ 
            lCurrentValue.assign(ControlFlowOps.invokeDynamicFunctionIn(logical.class, "getServerSubmitValidation", targetProcedure()));
            
            if (_isEqual(lCurrentValue, true))
            {
               returnNormal(false);
            }
         }
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ServerSubmitValidation"));
         ghProp.unwrapBufferField().changeValue(plSubmit_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setToggleDataTargets(final logical _plToggleDataTargets_1)
   {
      logical plToggleDataTargets_1 = TypeFactory.initInput(_plToggleDataTargets_1);
      
      return function("setToggleDataTargets", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ToggleDataTargets"));
         ghProp.unwrapBufferField().changeValue(plToggleDataTargets_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setUpdateFromSource(final logical _plUpdateFromSource_1)
   {
      logical plUpdateFromSource_1 = TypeFactory.initInput(_plUpdateFromSource_1);
      
      return function("setUpdateFromSource", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("UpdateFromSource"));
         ghProp.unwrapBufferField().changeValue(plUpdateFromSource_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setUpdateSource(final character _pcObject_3)
   {
      character pcObject_3 = TypeFactory.initInput(_pcObject_3);
      
      return function("setUpdateSource", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("UpdateSource"));
         ghProp.unwrapBufferField().changeValue(pcObject_3);
         returnNormal(true);
      }));
   }
}
