/*------------------------------------------------------------------------
    File        : Field.java
    Purpose     : Super procedure for SmartDataField objects

    Syntax      : adm2/field.p

    Modified    : June 23, 1999 Version 9.1A
    Modified    : 16/11/2001    Mark Davies (MIP)
                  Added functions getSDFFrameHandle to return the frame
                  handle of any SDF frame. This is used to move away
                  from using PRIVATE-DATA in ADM procedures.
                  17 Nov 2001   Peter Judge
                  Added support for SDF's on local fill-ins. The FieldName
                  attribute of SDF's on local fill-in's must be "<Local>".
  ----------------------------------------------------------------------*/

package com.goldencode.hotel.common.adm2;

import com.goldencode.p2j.util.*;
import com.goldencode.util.*;
import com.goldencode.p2j.ui.*;

import static com.goldencode.p2j.util.BlockManager.*;
import static com.goldencode.p2j.util.MathOps.*;
import static com.goldencode.p2j.util.CompareOps.*;
import static com.goldencode.p2j.util.logical.*;
import static com.goldencode.p2j.util.TextOps.*;
import static com.goldencode.p2j.util.ProcedureManager.*;
import static com.goldencode.p2j.util.character.*;

/**
 * Business logic (converted to Java from the 4GL source code
 * in common/adm2/field.p).
 */
public class Field
{
   /*********************************************************************
   * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
   * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
   * below.  All Rights Reserved.                                       *
   *                                                                    *
   * The Initial Developer of the Original Code is PSC.  The Original   *
   * Code is Progress IDE code released to open source December 1, 2000.*
   *                                                                    *
   * The contents of this file are subject to the Possenet Public       *
   * License Version 1.0 (the "License"); you may not use this file     *
   * except in compliance with the License.  A copy of the License is   *
   * available as of the date of this notice at                         *
   * http://www.possenet.org/license.html                               *
   *                                                                    *
   * Software distributed under the License is distributed on an "AS IS"*
   * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
   * should refer to the License for the specific language governing    *
   * rights and limitations under the License.                          *
   *                                                                    *
   * Contributors:                                                      *
   *                                                                    *
   *********************************************************************/   
   /*          This .W file was created with the Progress AppBuilder.      */   
   /*----------------------------------------------------------------------*/   
   /* ***************************  Definitions  ************************** */   
   /* Tell fieldprop.i that this is the Super procedure. */   
   /*********************************************************************
   * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
   * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
   * below.  All Rights Reserved.                                       *
   *                                                                    *
   * The Initial Developer of the Original Code is PSC.  The Original   *
   * Code is Progress IDE code released to open source December 1, 2000.*
   *                                                                    *
   * The contents of this file are subject to the Possenet Public       *
   * License Version 1.0 (the "License"); you may not use this file     *
   * except in compliance with the License.  A copy of the License is   *
   * available as of the date of this notice at                         *
   * http://www.possenet.org/license.html                               *
   *                                                                    *
   * Software distributed under the License is distributed on an "AS IS"*
   * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
   * should refer to the License for the specific language governing    *
   * rights and limitations under the License.                          *
   *                                                                    *
   * Contributors:                                                      *
   *                                                                    *
   *********************************************************************/   
   /*--------------------------------------------------------------------------
       File        : fieldexclcustom.i
       Purpose     : Defines EXCLUDE-xxx preprocessor variables to exclude
                     functions or internal procedures in the super procedure
                     where it is referenced.
       Syntax      : {src/adm2/custom/fieldexclcustom.i}
   
       Modified    : 06/1999
       
       Notes       : Referenced in src/adm2/field.p
     ------------------------------------------------------------------------*/   
   /*********************************************************************
   * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
   * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
   * below.  All Rights Reserved.                                       *
   *                                                                    *
   * The Initial Developer of the Original Code is PSC.  The Original   *
   * Code is Progress IDE code released to open source December 1, 2000.*
   *                                                                    *
   * The contents of this file are subject to the Possenet Public       *
   * License Version 1.0 (the "License"); you may not use this file     *
   * except in compliance with the License.  A copy of the License is   *
   * available as of the date of this notice at                         *
   * http://www.possenet.org/license.html                               *
   *                                                                    *
   * Software distributed under the License is distributed on an "AS IS"*
   * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
   * should refer to the License for the specific language governing    *
   * rights and limitations under the License.                          *
   *                                                                    *
   * Contributors:                                                      *
   *                                                                    *
   *********************************************************************/   
   /*--------------------------------------------------------------------------
     File: launch.i
   
     Description: Run business logic persistent include
   
     Purpose:     This include will be used to run any persistent business logic
                  procedures (PLIPs) instead of the progress run statement.
                  This include wraps the original Astra include
                  af/sup2/afrun2.i which encapsulates calls to launchProcedure 
                  in the Session Manager and physically does the run of the 
                  internal procedure if specified.
   
     Parameters: {&PLIP}            Physical procedure (PLIP) name, e.g. 'ac/app/actaxplipp.p'
                 {&IProc}           Internal procedure name, e.g. 'calculateTax' or
                                    left undefined to just actually start the PLIP
                 {&PList}           Parameter List, e.g. "(INPUT cTaxCode, INPUT tDate, INPUT dCalcAmount, OUTPUT cTaxAmount)"
                                    or left undefined for no parameters
                 {&OnApp}           Run on Appserver flag 'YES', 'NO', or 'APPSERVER', default = 'YES'
                 {&Partition}       Appserver partition, Astra = default, e.g. 'Astra'
                 {&Perm}            Run permanently YES/NO, default = NO (do not let agent kill it)
                 {&AutoKill}        Auto kill plip after run YES/NO, default = NO
                 {&NewInstance}     New instance YES/NO, default = NO (use running instance if any)
                 {&Define-only}     YES = define the variables you need but take no other action
                 {&AutoKillOnError} Auto kill plip if error occurs when running YES/NO, default = NO.
                                    If AutoKill is YES, this value will automatically be YES.             
                 {&PlipRunError}    What to do when an error occurs when the {&IProc} cannot be found
                                    in the {&PLIP}. The following default behaviour occurs when this
                                    parameter is left blank: when running on a client side connection,
                                    the error is messaged, but not when running server side. 
                                    This parameter must contain all of the required logic needed when
                                    the above error occurs, since it is all that executes.
                 {&dynCall}         Run using dynamic call wrapper.  Note this option can only be selected
                                    when running Appserver (&OnApp) = YES, and when {&AutoKill} = YES.  Note when
                                    running like this, parameters are not specified using {&PList}, but are specified
                                    individually, using {&mode1} {&parm1} {&dataType1} {&mode2} {&parm2} {&dataType2}...etc
                                    
                 {&mode1} - {&mode64}          When running using dynamic call wrapper, parameter 1 -> 64 mode : INPUT INPUT-OUTPUT or OUTPUT
                 {&parm1} - {&parm64}          When running using dynamic call wrapper, the name of the variable or field storing parameter 1 -> 64.
                                               When passing constants, put in single quotes, example 'Pass this string in'.
                 {&dataType1} - {&dataType64}  When running using dynamic call wrapper, the data type for parameter 1 -> 64 : BUFFER TABLE-HANDLE CHARACTER DECIMAL etc...
                               
      RULES:
      1. Required logical arguments must be passed in unquoted as YES or NO. Other text
         arguments must be single quoted literals, e.g. 'text' or unquoted variables, 
         and if the literals require spaces, they should be double quoted then single
         quoted, e.g. "'text'".
      2. The only exception to the above rule is for the parameter list, which does not
         support a variable and so only double quotes must be used.
      3. The internal procedure may be ommitted if required so the developer can manually run
         procedures in the PLIP and the PLIP is handle is simply made available in hPlip.
      4. If the onApp parameter is not specified it will default to 'YES' which will run on
         Appserver if Appserver connected, otherwise run locally. Passing in 'APPSERVER' will
         cause the procedure to fail if the appserver is not connected.
      5. If the OnApp parameter is not NO, then a partition may be specified and will be connected
         if not already connected. If no partition is specified it will default to the 'Astra'
         Appserver partition and will run on the gshAstraAppserver handle.
      6. If the Perm parameter is passed in as YES, then the procedure will not be killed when the
         agent disconnects, leaving it running for other connections. This is not the norm.
      7. If AutoKill is set to YES, then following the run of the internal procedure in the persistent
         procedure, the PLIP will be automatically closed. This is useful when running routines on 
         Appserver to ensure the connection does not remain bound.      
      8. If NewInstance is set to YES, then it will not check for an already running instance of the
         PLIP to use and will simply run a new version.
      9. If AutoKill is set to YES, and an error occurs when attempting to run an IP in the procedure,
         the procedure will be automatically closed, for the reasons specified in (7) above.
     10. The dynamic call wrapper can only be used when running on the Appserver, and &autokill has been set to yes.
         In other cases, the persistent procedure handle needs to be available after call has been invoked, which is not
         going to be true in this case.  When using the dynamic call wrapper, specify your parameters as follows:
         &mode1 = INPUT  &parm1 = cCharVarName &dataType1 = CHARACTER
         &mode2 = OUTPUT &parm2 = hTableHandle &dataType2 = TABLE-HANDLE etc.
     
      Following use of the include file, the following variable values will be defined and available:
      hPlip                  Handle of plip run
      lRunErrorStatus        Stored value of the ERROR-STATUS:ERROR after running an IP } needed when
      cRunReturnValue        Stored value of the RETURN-VALUE after running an IP       } AutoKill is YES
      cErrorMessage          Error message text caused by an internal procedure failing to tun in the PLIP
   
      If the run failed for any reason, the handle will be invalid and any errors
      will have been displayed if possible.
   
     History:
     --------
     Created: 11/08/2001     Mark Davies (MIP)
              Combined ICF/Dynamics afrun2.i and aflaunch.i
   
     (v:010001)    Task:                UserRef:    
                   Date:   APR/11/2002  Author:     Mauricio J. dos Santos (MJS) 
                                                    mdsantos@progress.com
     Update Notes: Adapted for WebSpeed by changing SESSION:PARAM = "REMOTE" 
                   to SESSION:CLIENT-TYPE = "WEBSPEED".
   
   ---------------------------------------------------------------------------*/   
   /* Set-up defaults */   
   /* Default to run on Appserver if can */   
   /* Default to Astra Appserver partition */   
   /* Default to not permanent */   
   /* Default to not killed automatically */   
   /* Default to new instance */   
   logical lRunErrorStatus = TypeFactory.logical();

   character cRunReturnValue = TypeFactory.character();

   handle hPlip = TypeFactory.handle();

   character cErrorMessage = TypeFactory.character();

   /* _UIB-CODE-BLOCK-END */   
   /* *********************** Procedure Settings ************************ */   
   /* Settings for THIS-PROCEDURE
      Type: Procedure
      Allow: 
      Frames: 0
      Add Fields to: Neither
      Other Settings: CODE-ONLY COMPILE
    */   
   /* *************************  Create Window  ************************** */   
   /* DESIGN Window definition (used by the UIB) 
     CREATE WINDOW Procedure ASSIGN
            HEIGHT             = 22.67
            WIDTH              = 58.4.
   _* END WINDOW DEFINITION *_
                                                                           */   
   /* ************************* Included-Libraries *********************** */   
   /*********************************************************************
   * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
   * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
   * below.  All Rights Reserved.                                       *
   *                                                                    *
   * The Initial Developer of the Original Code is PSC.  The Original   *
   * Code is Progress IDE code released to open source December 1, 2000.*
   *                                                                    *
   * The contents of this file are subject to the Possenet Public       *
   * License Version 1.0 (the "License"); you may not use this file     *
   * except in compliance with the License.  A copy of the License is   *
   * available as of the date of this notice at                         *
   * http://www.possenet.org/license.html                               *
   *                                                                    *
   * Software distributed under the License is distributed on an "AS IS"*
   * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
   * should refer to the License for the specific language governing    *
   * rights and limitations under the License.                          *
   *                                                                    *
   * Contributors:                                                      *
   *                                                                    *
   *********************************************************************/   
   /*--------------------------------------------------------------------------
       File        : fieldprop.i
       Purpose     : Starts Super procedure field.p and defines SmartDataField
                     properties
       Syntax      : {src/adm2/fieldprop.i}
   
       Description :
   
       Modified    : June 23, 1999 Version 9.1A
       Modified    : 10/25/2001        Mark Davies (MIP)
                     1. Added ADM props fields for DataValue and DisplayedValue
       Modified    : 03/15/2002        Mark Davies (MIP)
                     Added ADM props field for CustomSuperProc
     ------------------------------------------------------------------------*/   
   /*          This .W file was created with the Progress UIB.             */   
   /*----------------------------------------------------------------------*/   
   /* ***************************  Definitions  ************************** */   
   /*********************************************************************
   * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
   * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
   * below.  All Rights Reserved.                                       *
   *                                                                    *
   * The Initial Developer of the Original Code is PSC.  The Original   *
   * Code is Progress IDE code released to open source December 1, 2000.*
   *                                                                    *
   * The contents of this file are subject to the Possenet Public       *
   * License Version 1.0 (the "License"); you may not use this file     *
   * except in compliance with the License.  A copy of the License is   *
   * available as of the date of this notice at                         *
   * http://www.possenet.org/license.html                               *
   *                                                                    *
   * Software distributed under the License is distributed on an "AS IS"*
   * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
   * should refer to the License for the specific language governing    *
   * rights and limitations under the License.                          *
   *                                                                    *
   * Contributors:                                                      *
   *                                                                    *
   *********************************************************************/   
   /*--------------------------------------------------------------------------
       File        : fielddefscustom.i
       Purpose     : Entry point to define:
                       New definitions
                       {&xcInstanceProperties}
                       {&ADM-PROPERTY-DLG}
       Syntax      : {src/adm2/custom/fielddefscustom.i}
   
       Modified    : 06/1999
       
       Notes       : Referenced in {src/adm2/fieldprop.i}
     ------------------------------------------------------------------------*/   
   /* This is the procedure to execute to set InstanceProperties at design time. */   
   /* _UIB-CODE-BLOCK-END */   
   /* ********************  Preprocessor Definitions  ******************** */   
   /* _UIB-PREPROCESSOR-BLOCK-END */   
   /* *********************** Procedure Settings ************************ */   
   /* Settings for THIS-PROCEDURE
      Type: Include
      Allow: 
      Frames: 0
      Add Fields to: Neither
      Other Settings: INCLUDE-ONLY
    */   
   /* *************************  Create Window  ************************** */   
   /* DESIGN Window definition (used by the UIB) 
     CREATE WINDOW Include ASSIGN
            HEIGHT             = 8
            WIDTH              = 60.
   _* END WINDOW DEFINITION *_
                                                                           */   
   /* ***************************  Main Block  *************************** */   
   /* Include the file which defines prototypes for all of the super
        procedure's entry points. Also, start or attach to the super procedure.
        Skip start-super-proc if we *are* the super procedure. 
        And skip including the prototypes if we are *any* super procedure. */   
   /* These preprocessors tell at compile time which properties can be
        retrieved directly from the temp-table. */   
   /* Include the next property file up the chain to get property FIELDs
        defined. */   
   /*********************************************************************
   * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
   * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
   * below.  All Rights Reserved.                                       *
   *                                                                    *
   * The Initial Developer of the Original Code is PSC.  The Original   *
   * Code is Progress IDE code released to open source December 1, 2000.*
   *                                                                    *
   * The contents of this file are subject to the Possenet Public       *
   * License Version 1.0 (the "License"); you may not use this file     *
   * except in compliance with the License.  A copy of the License is   *
   * available as of the date of this notice at                         *
   * http://www.possenet.org/license.html                               *
   *                                                                    *
   * Software distributed under the License is distributed on an "AS IS"*
   * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
   * should refer to the License for the specific language governing    *
   * rights and limitations under the License.                          *
   *                                                                    *
   * Contributors:                                                      *
   *                                                                    *
   *********************************************************************/   
   /*--------------------------------------------------------------------------
       File        : visprop.i
       Purpose     : Starts Super procedure visual.p and defines basic properties
                     for visual SmartObjects in V9
       Syntax      : {src/adm2/visprop.i}
   
       Description :
   
       Modified    : May 19, 1999 Version 9.1A
       Modified    : 10/23/2001          Mark Davies (MIP)
                     Added two new fields for ResizeHorizontal and ResizeVertical
       Modified    : 11/02/2001          Mark Davies (MIP)
                     Replaced properties for FrameMinHeightChars and FrameMinWidthChars with MinHeight and MinWidth
       Modified    : 11/14/2001          Mark Davies (MIP)
                     Renamed property 'DisplayFieldsSecurity' to 'FieldSecurity'
       Modified    : 02/27/2003          Neil Bell (MIP)
                     Added property 'SecuredTokens'                  
     ------------------------------------------------------------------------*/   
   /*          This .W file was created with the Progress UIB.             */   
   /*----------------------------------------------------------------------*/   
   /* ***************************  Definitions  ************************** */   
   /*********************************************************************
   * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
   * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
   * below.  All Rights Reserved.                                       *
   *                                                                    *
   * The Initial Developer of the Original Code is PSC.  The Original   *
   * Code is Progress IDE code released to open source December 1, 2000.*
   *                                                                    *
   * The contents of this file are subject to the Possenet Public       *
   * License Version 1.0 (the "License"); you may not use this file     *
   * except in compliance with the License.  A copy of the License is   *
   * available as of the date of this notice at                         *
   * http://www.possenet.org/license.html                               *
   *                                                                    *
   * Software distributed under the License is distributed on an "AS IS"*
   * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
   * should refer to the License for the specific language governing    *
   * rights and limitations under the License.                          *
   *                                                                    *
   * Contributors:                                                      *
   *                                                                    *
   *********************************************************************/   
   /*--------------------------------------------------------------------------
       File        : visualdefscustom.i
       Purpose     : Entry point to define:
                       New definitions
                       {&xcInstanceProperties}
                       {&ADM-PROPERTY-DLG}
       Syntax      : {src/adm2/custom/visualdefscustom.i}
   
       Modified    : 06/03/1999
       
       Notes       : Referenced in {src/adm2/visprop.i}
     ------------------------------------------------------------------------*/   
   /* This is the procedure to execute to set InstanceProperties at design time. */   
   /* _UIB-CODE-BLOCK-END */   
   /* ********************  Preprocessor Definitions  ******************** */   
   /* _UIB-PREPROCESSOR-BLOCK-END */   
   /* *********************** Procedure Settings ************************ */   
   /* Settings for THIS-PROCEDURE
      Type: Include
      Allow: 
      Frames: 0
      Add Fields to: Neither
      Other Settings: INCLUDE-ONLY
    */   
   /* *************************  Create Window  ************************** */   
   /* DESIGN Window definition (used by the UIB) 
     CREATE WINDOW Include ASSIGN
            HEIGHT             = 8
            WIDTH              = 60.
   _* END WINDOW DEFINITION *_
                                                                           */   
   /* ***************************  Main Block  *************************** */   
   /* Include the file which defines prototypes for all of the super
        procedure's entry points. Also, start or attach to the super procedure.
        Skip start-super-proc if we *are* the super procedure. 
        And skip including the prototypes if we are *any* super procedure. */   
   /* These preprocessors tell at compile time which properties can be
        retrieved directly from the temp-table. */   
   /* This property contains the security for all objects found in the AllFieldHandles Property */    
   /* Include the next property file up the chain (in this case, the top
        of the chain); it starts the property temp-table definition and we
        add our field definitions to it. */   
   /*********************************************************************
   * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
   * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
   * below.  All Rights Reserved.                                       *
   *                                                                    *
   * The Initial Developer of the Original Code is PSC.  The Original   *
   * Code is Progress IDE code released to open source December 1, 2000.*
   *                                                                    *
   * The contents of this file are subject to the Possenet Public       *
   * License Version 1.0 (the "License"); you may not use this file     *
   * except in compliance with the License.  A copy of the License is   *
   * available as of the date of this notice at                         *
   * http://www.possenet.org/license.html                               *
   *                                                                    *
   * Software distributed under the License is distributed on an "AS IS"*
   * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
   * should refer to the License for the specific language governing    *
   * rights and limitations under the License.                          *
   *                                                                    *
   * Contributors:                                                      *
   *                                                                    *
   *********************************************************************/   
   /*--------------------------------------------------------------------------
       File        : smrtprop.i
       Purpose     : Starts smart.p super procedure and defines general
                     SmartObject properties and other values.
       Syntax      : {src/adm2/smrtprop.i}
   
       Description :
   
       Modified    : February 4, 2000 - Version 9.1B
     ------------------------------------------------------------------------*/   
   /*          This .W file was created with the Progress UIB.             */   
   /*----------------------------------------------------------------------*/   
   /* ***************************  Definitions  ************************** */   
   /*********************************************************************
   * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
   * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
   * below.  All Rights Reserved.                                       *
   *                                                                    *
   * The Initial Developer of the Original Code is PSC.  The Original   *
   * Code is Progress IDE code released to open source December 1, 2000.*
   *                                                                    *
   * The contents of this file are subject to the Possenet Public       *
   * License Version 1.0 (the "License"); you may not use this file     *
   * except in compliance with the License.  A copy of the License is   *
   * available as of the date of this notice at                         *
   * http://www.possenet.org/license.html                               *
   *                                                                    *
   * Software distributed under the License is distributed on an "AS IS"*
   * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
   * should refer to the License for the specific language governing    *
   * rights and limitations under the License.                          *
   *                                                                    *
   * Contributors:                                                      *
   *                                                                    *
   *********************************************************************/   
   /*--------------------------------------------------------------------------
       File        : smartdefscustom.i
       Purpose     : Entry point to define:
                       New definitions
                       {&xcInstanceProperties}
                       {&ADM-PROPERTY-DLG}
       Syntax      : {src/adm2/custom/smartdefscustom.i}
   
       Modified    : 06/03/1999
       
       Notes       : Referenced in {src/adm2/smrtprop.i}
     ------------------------------------------------------------------------*/   
   /* Service Managers */   
   /*********************************************************************
   * Copyright (C) 2000-2002 by Progress Software Corporation ("PSC"),  *
   * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
   * below.  All Rights Reserved.                                       *
   *                                                                    *
   * The Initial Developer of the Original Code is PSC.  The Original   *
   * Code is Progress IDE code released to open source December 1, 2000.*
   *                                                                    *
   * The contents of this file are subject to the Possenet Public       *
   * License Version 1.0 (the "License"); you may not use this file     *
   * except in compliance with the License.  A copy of the License is   *
   * available as of the date of this notice at                         *
   * http://www.possenet.org/license.html                               *
   *                                                                    *
   * Software distributed under the License is distributed on an "AS IS"*
   * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
   * should refer to the License for the specific language governing    *
   * rights and limitations under the License.                          *
   *                                                                    *
   * Contributors:                                                      *
   *                                                                    *
   *********************************************************************/   
   /*---------------------------------------------------------------------------------
     File: globals.i
   
     Description:  ICF Global Shared Variables
   
     Purpose:      ICF Global Shared Variables
                   The global shared variables available within the Dynamics environment.
                   
                   NOTE: (Dynamic Only )
                   Shared variables do not span the client / server boundary. On the
                   client these are set-up in afstartupp.p and on the server in the agent
                   as_startup.p
                   Also note if we include the guts of a procedure as an internal
                   procedure for shared client/server code, we sometime need to
                   ensure the shared variables are not redefined.
   
     Parameters:   <none>
   
     History:
     --------
   
     Modified: 11/08/2001 - Mark Davies (MIP)
               Copied code for ICF afglobals.i and added here to allow
               ICF changed adm code to compile. afglobals.i now includes
               this file for definitions.
               Any ADM code will now reference globals.i which excludes any other
               ICF specific includes.
   -----------------------------------------------------------*/   
   /*                   This .W file was created with the Progress UIB.             */   
   /*-------------------------------------------------------------------------------*/   
   /* ***************************  Definitions  ************************** */   
   /* _UIB-CODE-BLOCK-END */   
   /* ********************  Preprocessor Definitions  ******************** */   
   /* _UIB-PREPROCESSOR-BLOCK-END */   
   /* *********************** Procedure Settings ************************ */   
   /* Settings for THIS-PROCEDURE
      Type: Include
      Allow: 
      Frames: 0
      Add Fields to: Neither
      Other Settings: INCLUDE-ONLY
    */   
   /* *************************  Create Window  ************************** */   
   /* DESIGN Window definition (used by the UIB) 
     CREATE WINDOW Include ASSIGN
            HEIGHT             = 5.57
            WIDTH              = 64.4.
   _* END WINDOW DEFINITION *_
                                                                           */   
   /* ***************************  Main Block  *************************** */   
   /* Used whereever Crystal is used and when defining the tt_datasource temp-table 
      to define the maximum columns supported when printing to crystal.
      See browsercustom.p and afcrpplip2p.p for example uses
   */   
   /* Used for filter checks */   
   /* The table delimiter used in assignlist, *ColumnsByTable props */   
   handle gshAstraAppserver /* quick access to all handles on object, e.g. viewer, whether enabled or not */  /* quick access to all fields on object, e.g. viewer, whether enabled or not */ = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshastraappserver", true, new handle());

   /* Handle to Astra Application Server Partition */   
   handle gshSessionManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshsessionmanager", true, new handle());

   /* Handle Astra Session Manager */   
   handle gshRimanager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshrimanager", true, new handle());

   /* Handle Astra Session Manager */   
   handle gshSecurityManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshsecuritymanager", true, new handle());

   /* Handle Astra Security Manager */   
   handle gshProfileManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshprofilemanager", true, new handle());

   /* Handle Astra Profile Manager */   
   handle gshRepositoryManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshrepositorymanager", true, new handle());

   /* Handle Astra Repository Manager */   
   handle gshTranslationManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshtranslationmanager", true, new handle());

   /* Handle Astra Translation Manager */   
   handle gshWebManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshwebmanager", true, new handle());

   /* Handle Astra Web Manager */   
   character gscSessionId = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gscsessionid", true, new character(""));

   /* Unique session id */   
   decimal gsdSessionObj = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gsdsessionobj", true, new decimal(0));

   /* Unique session id */   
   handle gshFinManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshfinmanager", true, new handle());

   /* Handle AstraFin Generic Manager */   
   handle gshGenManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshgenmanager", true, new handle());

   /* Handle AstraFramework Generic Manager */   
   handle gshAgnManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshagnmanager", true, new handle());

   /* Handle AstraGen Generic Manager */   
   decimal gsdTempUniqueId = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gsdtempuniqueid", true, new decimal(0));

   /* A number for unique objects */   
   decimal gsdUserObj = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gsduserobj", true, new decimal(0));

   /* User Object ID */   
   decimal gsdRenderTypeObj = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gsdrendertypeobj", true, new decimal(0));

   /* Render Type Object ID */   
   decimal gsdSessionScopeObj = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gsdsessionscopeobj", true, new decimal(0));

   /* Render Type Object ID */   
   /*********************************************************************
   * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
   * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
   * below.  All Rights Reserved.                                       *
   *                                                                    *
   * The Initial Developer of the Original Code is PSC.  The Original   *
   * Code is Progress IDE code released to open source December 1, 2000.*
   *                                                                    *
   * The contents of this file are subject to the Possenet Public       *
   * License Version 1.0 (the "License"); you may not use this file     *
   * except in compliance with the License.  A copy of the License is   *
   * available as of the date of this notice at                         *
   * http://www.possenet.org/license.html                               *
   *                                                                    *
   * Software distributed under the License is distributed on an "AS IS"*
   * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
   * should refer to the License for the specific language governing    *
   * rights and limitations under the License.                          *
   *                                                                    *
   * Contributors:                                                      *
   *                                                                    *
   *********************************************************************/   
   /*------------------------------------------------------------------------
       File        : globalscustom.i
       Purpose     : The developer can create his own version of this to 
                     allow 'global' definitions to be added to the 
                     framework.   
       Syntax      : {src/adm2/custom/globalscustom.i}
   
       Description :
   
       Created     : 06/03/1999
       Notes       : Referenced in {src/adm2/globals.i}
     ----------------------------------------------------------------------*/   
   /*          This .W file was created with the Progress AppBuilder.      */   
   /*----------------------------------------------------------------------*/   
   /* ***************************  Definitions  ************************** */   
   /* _UIB-CODE-BLOCK-END */   
   /* define the ADM Version and broker handle for all SmartObjects */   
   /* Note that adm-props-from-repository is documented as possible to define
        as a variable for flexible override, so it cannot be used for conditional 
        compile.  */   
   /* adm-props-from-repository is supported to be defined as a variable,
       so it is not used for conditional compile */   
   handle ghProp = TypeFactory.handle();

   /**
    * External procedure (converted to Java from the 4GL source code
    * in common/adm2/field.p).
    */
   public void execute()
   {
      externalProcedure(Field.this, new Block((Init) () -> 
      {
         gsdSessionObj.setPrecision(9);
         gsdTempUniqueId.setPrecision(9);
         gsdUserObj.setPrecision(9);
         gsdRenderTypeObj.setPrecision(9);
         gsdSessionScopeObj.setPrecision(9);
      }, 
      (Body) () -> 
      {
         /* _UIB-CODE-BLOCK-END */
         /* *********************** Procedure Settings ************************ */
         /* Settings for THIS-PROCEDURE
            Type: Method-Library
            Allow: 
            Frames: 0
            Add Fields to: Neither
            Other Settings: INCLUDE-ONLY
          */
         /* *************************  Create Window  ************************** */
         /* DESIGN Window definition (used by the UIB) 
           CREATE WINDOW Method-Library ASSIGN
                  HEIGHT             = 14.86
                  WIDTH              = 57.2.
         _* END WINDOW DEFINITION *_
                                                                                 */
         /* ***************************  Main Block  *************************** */
         /* Include the file which defines prototypes for all of the super
              procedure's entry points. Also, start or attach to the super procedure.
              Skip start-super-proc if we *are* the super procedure. 
              And skip including the prototypes if we are *any* super procedure. */
         /* These preprocessors let the get and set methods know at compile time
             which property values are located in the temp-table and which must
             be accessed through the property functions.
          */
         /* for support of dynamic objects */
         /* physical name - no path */
         /* for support of runtime parameters passed from the menus */
         /* Properties for super procedure management */
         /* The name of the super procedure; can be a CSV list */ 
 /* STATELESS, STATEFUL; the mode that all supers for the object will be launched in */ 
 /* Runtime property storing handles to the object's super procedures, defined in SuperProcedure. */
         /* Layout support for dynamic objects */
         /* contains a layout position that allows an object to be laid out correctly. */
         /* determine if we need to create the prop fields here or from the */
         /* repository (if ICF is running ) */
         /* COMMENT THE FOLLOWING LINE TO USE THE "<class>prop.i" DEFINITIONS */
         /* This temp-table defines all the propertt fields for an object.
             This include file contributes the DEFINE statement header and
             all basic smart object properties. Each other property class include file
             adds its own fields and then the parent object ends the statement.
             Define the fields for smart objects only, not for their super procedures.
          */
         if (_not(handle.fromString(entry(1, thisProcedure().unwrapADMData().getADMData(), chr(1))).isValid()))
         {
         }
         
         /* _UIB-CODE-BLOCK-END */
         /* else if appserver aware (adecomm/appserv.i) include appserver props */
         if (_not(handle.fromString(entry(1, thisProcedure().unwrapADMData().getADMData(), chr(1))).isValid()))
         {
         }
         
         /* _UIB-CODE-BLOCK-END */
         if (_not(handle.fromString(entry(1, thisProcedure().unwrapADMData().getADMData(), chr(1))).isValid()))
         {
         }
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * ***********************  Function Implementations****************
    */
   public character getObjectType()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Returns the Type of the SmartObject, such as "SmartDataObject", etc.
                  For an ADM Super procedure, this function returns "SUPER".
         Params:  <none>
      ------------------------------------------------------------------------------*/
      character cType = TypeFactory.character();
      
      return function("getObjectType", character.class, new Block((Body) () -> 
      {
         cType.assign("SUPER");
         returnNormal(cType);
      }));
   }

   /** *********************  Internal Procedures ********************** */
   /**
    * Purpose:     Special code for initializing SmartDataFields.<br>
    * Parameters:  &lt;none&gt;<br>
    * Notes:       The SmartDataField adds itself to the DisplayedFields property<br>
    *              of its containing SmartDataViewer if the DisplayField logical <br>
    *              property is true, and to the EnabledFields or, if localField, <br>
    *              EnabledObjFlds, if EnableField is true.
    */
   public void initializeObject()
   {
      logical lInitialized = TypeFactory.logical();
      character cField = TypeFactory.character();
      logical lLocal = TypeFactory.logical();
      logical lEnable = TypeFactory.logical();
      logical lDisplay = TypeFactory.logical();
      handle hSource = TypeFactory.handle();
      character cCustomSuperProc = TypeFactory.character();
      character cList = TypeFactory.character();
      logical lDisableOnInit = TypeFactory.logical();
      logical lHideOnInit = TypeFactory.logical();
      character cEnabledObjFlds = TypeFactory.character();
      logical lSecured = TypeFactory.logical();
      logical lTranslated = TypeFactory.logical();
      character cSuperProcedureMode = TypeFactory.character();
      
      internalProcedure(new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ObjectInitialized"));
         lInitialized.assign(ghProp.unwrapBufferField().value());
         
         if ((lInitialized).booleanValue())
         {
            returnNormal("adm-error");
         }
         
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ContainerSource"));
         hSource.assign(ghProp.unwrapBufferField().value());
         
         if ((hSource.isValid()).booleanValue())
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("FieldName"));
            cField.assign(ghProp.unwrapBufferField().value());
            /* {get Prop var [handle]} */ 
            lLocal.assign(ControlFlowOps.invokeDynamicFunctionIn(logical.class, "getLocalField", targetProcedure()));
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("EnableField"));
            lEnable.assign(ghProp.unwrapBufferField().value());
            
            if ((lLocal).booleanValue())
            {
               if ((lEnable).booleanValue())
               {
                  ghProp.assign(handle.fromString(entry(1, hSource.unwrapADMData().getADMData(), chr(1))));
                  ghProp.assign(ghProp.unwrapBuffer().bufferField("EnabledObjFlds"));
                  cList.assign(ghProp.unwrapBufferField().value());
                  
                  if (_isEqual(lookup(cField, cList), 0))
                  {
                     cList.assign(concat((_isGreaterThan(cList, "") ? concat(cList, ",") : new character("")), cField));
                     ghProp.assign(handle.fromString(entry(1, hSource.unwrapADMData().getADMData(), chr(1))));
                     ghProp.assign(ghProp.unwrapBuffer().bufferField("EnabledObjFlds"));
                     ghProp.unwrapBufferField().changeValue(cList);
                  }
               }
            }
            else
            {
               ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
               ghProp.assign(ghProp.unwrapBuffer().bufferField("DisplayField"));
               lDisplay.assign(ghProp.unwrapBufferField().value());
               
               if ((lEnable).booleanValue())
               {
                  /* {get Prop var [handle]} */
                  cList.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getEnabledFields", hSource));
                  
                  if (_isEqual(lookup(cField, cList), 0))
                  {
                     cList.assign(concat((_isGreaterThan(cList, "") ? concat(cList, ",") : new character("")), cField));
                     /* {set Prop value [handle]} */
                     ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "setEnabledFields", hSource, "I", cList);
                  }
               }
               
               if ((lDisplay).booleanValue())
               {
                  /* {get Prop var [handle]} */
                  cList.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getDisplayedFields", hSource));
                  
                  if (_isEqual(lookup(cField, cList), 0))
                  {
                     cList.assign(concat((_isGreaterThan(cList, "") ? concat(cList, ",") : new character("")), cField));
                     /* {set Prop value [handle]} */
                     ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "setDisplayedFields", hSource, "I", cList);
                  }
               }
            }
            /* Start the SDF's super procedure - if specified */
            /* {get Prop var [handle]} */ 
            cCustomSuperProc.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getCustomSuperProc", targetProcedure()));
            
            /* If the super procedure couldn't be found for in the CustomSuperProc
                     property then we should test for it in SuperProcedure */
            if (_or(isEqual(cCustomSuperProc, ""), () -> isUnknown(cCustomSuperProc)))
            {
               ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
               ghProp.assign(ghProp.unwrapBuffer().bufferField("SuperProcedure"));
               cCustomSuperProc.assign(ghProp.unwrapBufferField().value());
            }
            /* {get Prop var [handle]} */ 
            
            if (_and(isNotEqual(cCustomSuperProc, ""), () -> notUnknown(cCustomSuperProc)))
            {
               cSuperProcedureMode.assign("STATEFUL");
               
               /* Is Dynamics running? */
               if ((gshSessionManager.isValid()).booleanValue())
               {
                  hPlip.setUnknown();
                  
                  if ((gshSessionManager.isValid()).booleanValue())
                  {
                     ControlFlowOps.invokeInWithMode("launchProcedure", gshSessionManager, "IIIIIO", cCustomSuperProc, new logical(true), new character("NO"), new character(""), new logical(false), hPlip);
                  }
                  
                  /* Report an error if the PLIP failed to start (or we couldn't get it's handle)
                       * or if the IP we specified doesn't exist in the PLIP.
                       *
                       * Use getInternalEntryExists function because the PLIP may be running on the
                       * AppServer, in which case the :INTERNAL-ENTRIES attribute will be ?.
                       * ----------------------------------------------------------------------- */
                  if (_or(not(hPlip.isValid()), () -> isNotEqual("", "")))
                  {
                     if (_and(gshSessionManager.isValid(), () -> isEqual(new logical(ControlFlowOps.invokeDynamicFunctionInWithMode("getInternalEntryExists", gshSessionManager, "II", hPlip, "")), false)))
                     {
                        hPlip.setUnknown();
                        cErrorMessage.assign(concat("Could not run procedure in PLIP specified. The PLIP ", cCustomSuperProc, " could not be run."));
                        
                        /* We allow for the error generated to be passed back to a calling procedure.
                                 * We may (should) want to do this if the procedure we are running the PLIP
                                 * from is also running on the AppServer partition. If we don't specify this
                                 * parameter, the message will merely be written into the AppServer log, and
                                 * we will not know that it occurred.
                                 *
                                 * showWarningMessages is used in place of showMessages because the former
                                 * doesn't have an input blocking statement in it. The input blocking statment
                                 * causes problems when afrun2.i is run from a function.
                                 * ----------------------------------------------------------------------- */
                        if (_and(not(or(SessionUtils.isRemote(), () -> isEqual(LogicalTerminal.getClientType(), "WEBSPEED"))), () -> gshSessionManager.isValid()))
                        {
                           ControlFlowOps.invokeInWithMode("showWarningMessages", gshSessionManager, "III", cErrorMessage, new character("ERR"), new character("Business Logic Run (afrun2.i)"));
                        }
                     }
                  }
                  
                  /* Can't run */
                  if (_and(hPlip.isValid(), () -> isNotEqual("", "")))
                  {
                     lRunErrorStatus.assign(false);
                     cRunReturnValue.assign("");
                     ErrorManager.silentErrorEnable();
                     ControlFlowOps.invokeIn("", hPlip);
                     ErrorManager.silentErrorDisable();
                     lRunErrorStatus.assign(ErrorManager.isError());
                     cRunReturnValue.assign(trim(ControlFlowOps.getReturnValue()));
                     
                     /* The Autokill raises the ERROR-STATUS and clears the RETURN-VALUE. Set them again so that they are
                              * available to the procedure or function that used afrun2
                              * If killPlip raised ERROR-STATUS, but RUN VALUE() didn't, don't clear ERROR-STATUS, i.e.
                              * ------------------------------------------------------------------------------------------------- */
                     if (_isEqual(ErrorManager.isError(), false))
                     {
                        ErrorManager.setError(lRunErrorStatus);
                     }
                     
                     /* This will override the RETURN-VALUE of killPlip (if it had any), with RETURN-VALUE of RUN VALUE() Statement */
                     if (_and(isNotEqual(cRunReturnValue, ""), () -> gshSessionManager.isValid()))
                     {
                        ControlFlowOps.invokeInWithMode("setReturnValue", gshSessionManager, "I", cRunReturnValue);
                     }
                  }
                  /* PLIP valid &  */
                  /* NOT define-only */
                  /* Add all the super procedures. */
                  ControlFlowOps.invokeDynamicFunctionInWithMode("addAsSuperProcedure", gshSessionManager, "II", hPlip, targetProcedure());
               }
               else
               {
                  OnPhrase[] onPhrase0 = new OnPhrase[]
                  {
                     new OnPhrase(Condition.STOP, Action.RETURN_NORMAL, "blockLabel0", new character("ADM-ERROR"))
                  };
                  
                  doBlock(TransactionType.SUB, "blockLabel0", onPhrase0, new Block((Body) () -> 
                  {
                     ControlFlowOps.invokePersistentSet((cCustomSuperProc).toStringMessage(), hPlip);
                  }));
                  
                  /* run the super procedure */
                  if ((hPlip.isValid()).booleanValue())
                  {
                     targetProcedure().unwrapCommonProcedure().addSuperProcedure(hPlip, ProcedureManager.SEARCH_TARGET);
                  }
               }
               
               /* sesison manger not running */
               /* Store the handle of the super so that it is killed when
                            this object shuts down. The destruction of this super procedure
                            is done by the container (the viewer typically).
                          */
               if (_isNotEqual(cSuperProcedureMode, "STATELESS"))
               {
                  ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
                  ghProp.assign(ghProp.unwrapBuffer().bufferField("SuperProcedureHandle"));
                  ghProp.unwrapBufferField().changeValue(valueOf(hPlip));
               }
               /* {set Prop value [handle]} */ 
            }
         }
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ObjectSecured"));
         lSecured.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ObjectTranslated"));
         lTranslated.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("EnabledObjFlds"));
         cEnabledObjFlds.assign(ghProp.unwrapBufferField().value());
         
         /* not (log = true) to treat unknown and false similar  */
         if (_or(or(not(isEqual(lSecured, true)), () -> not(isEqual(lTranslated, true))), () -> isGreaterThan(cEnabledObjFlds, "")))
         {
            ControlFlowOps.runSuper("initializeObject");
         }
         else
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("ObjectInitialized"));
            ghProp.unwrapBufferField().changeValue(new logical(true));
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("DisableOnInit"));
            lDisableOnInit.assign(ghProp.unwrapBufferField().value());
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("HideOnInit"));
            lHideOnInit.assign(ghProp.unwrapBufferField().value());
            
            if (_and(not(lDisableOnInit), lEnable))
            {
               ControlFlowOps.invokeIn("enableObject", targetProcedure());
            }
            
            if (_not(lHideOnInit))
            {
               ControlFlowOps.invokeIn("viewObject", targetProcedure());
            }
            else
            {
               NamedEventManager.publish("LinkState", targetProcedure(), "I", new Object[]
               {
                  "inactive"
               });
            }
         }
         /* No enabledObjFlds (dynselect etc.. ) */
         returnNormal();
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose: Resize the Field<br>
    * <p>
    * Parameters:  INPUT pidHeight decimal New height of component <br>
    *              INPUT pidWidth decimal New width of component<br>
    * <p>
    * Notes:  The procedure steps through all the widgets on the frame and, resizes<br>
    *         the frame and the widgets (where the programmer wants the widgets to<br>
    *         be resized). If a programmer does not want a widget to be resized,<br>
    *         the string 'NO-RESIZE' must be inserted into the widget's PRIVATE-DATA<br>
    *         <br>
    *        ** NB: No Buttons nor the height or label (LITERAL) of ANY object / widget<br>
    *                 will be adjusted by default. For this teh programmer needs to create<br>
    *                 an override procedure in his SmartDataField write his / her own code,<br>
    *                 seeing that it would be fairly specific to his / her SmartDataField.
    */
   public void resizeObject(final decimal _pidHeight, final decimal _pidWidth)
   {
      decimal pidHeight = TypeFactory.initInput(_pidHeight);
      decimal pidWidth = TypeFactory.initInput(_pidWidth);
      decimal dFrameMinimumHeight = TypeFactory.decimal();
      decimal dFrameMinimumWidth = TypeFactory.decimal();
      handle hChildWidget = TypeFactory.handle();
      handle hFieldGroup = TypeFactory.handle();
      handle hFrame = TypeFactory.handle();
      
      internalProcedure(new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ContainerHandle"));
         hFrame.assign(ghProp.unwrapBufferField().value());
         dFrameMinimumWidth.assign(pidWidth);
         dFrameMinimumHeight.assign(pidHeight);
         /* Set up the variables to step through the widgets */
         hFieldGroup.assign(hFrame);
         hFieldGroup.assign(hFieldGroup.unwrapTree().firstChild());
         hChildWidget.assign(hFieldGroup.unwrapTree().firstChild());
         
         LogicalOp whileClause0 = () -> hChildWidget.isValid();
         
         repeatWhile("loopLabel0", whileClause0, new Block((Body) () -> 
         {
            if (_and(and(and(and(and(HandleOps.canQuery(hChildWidget, "PRIVATE-DATA"), () -> HandleOps.canQuery(hChildWidget, "COLUMN")), () -> HandleOps.canQuery(hChildWidget, "WIDTH")), () -> HandleOps.canQuery(hChildWidget, "TYPE")), () -> isNotEqual(hChildWidget.unwrapType().getResourceType(), "LITERAL")), () -> isNotEqual(hChildWidget.unwrapType().getResourceType(), "BUTTON")))
            {
               if (_and(isNotEqual(indexOf("NO-RESIZE", hChildWidget.unwrap().getPrivateData()), 0), () -> notUnknown(indexOf("NO-RESIZE", hChildWidget.unwrap().getPrivateData()))))
               {
                  /* The non resizable field is wider than the current frame width */
                  if (_isGreaterThan(plus(hChildWidget.unwrapWidget().getColumn(), hChildWidget.unwrapSizeable().getWidthChars()), dFrameMinimumWidth))
                  {
                     dFrameMinimumWidth.assign(plus(hChildWidget.unwrapWidget().getColumn(), hChildWidget.unwrapSizeable().getWidthChars()));
                  }
               }
               
               /* If the field is resizable, make sure we don't make the frame smaller than the furthest field */
               if (_isLessThan(dFrameMinimumWidth, hChildWidget.unwrapWidget().getColumn()))
               {
                  dFrameMinimumWidth.assign(hChildWidget.unwrapWidget().getColumn());
               }
               
               /* The field's position and height is more than the current frame height */
               if (_isGreaterThan(plus(hChildWidget.unwrapWidget().getRow(), decimal.fromLiteral("0.05")), dFrameMinimumHeight))
               {
                  dFrameMinimumHeight.assign(plus(hChildWidget.unwrapWidget().getRow(), decimal.fromLiteral("0.05")));
               }
            }
            hChildWidget.assign(hChildWidget.unwrap().getNextSibling());
         }));
         
         dFrameMinimumHeight.assign(dFrameMinimumHeight);
         hFrame.unwrapWidget().setScrollable(new logical(true));
         hFrame.unwrapSizeable().setWidthChars(dFrameMinimumWidth);
         hFrame.unwrapSizeable().setHeightChars(dFrameMinimumHeight);
         hFrame.unwrapWidget().setVirtWidth(hFrame.unwrapSizeable().getWidthChars());
         hFrame.unwrapWidget().setVirtHeight(hFrame.unwrapSizeable().getHeightChars());
         hFrame.unwrapWidget().setScrollable(new logical(false));
         /* Put the child widget back to the first widget on the frame */
         hChildWidget.assign(hFieldGroup.unwrapTree().firstChild());
         
         LogicalOp whileClause1 = () -> hChildWidget.isValid();
         
         repeatWhile("loopLabel1", whileClause1, new Block((Body) () -> 
         {
            /* Check to see if the relevant properties can be queried and the adjust the width accordingly */
            if (_and(and(and(and(and(HandleOps.canQuery(hChildWidget, "PRIVATE-DATA"), () -> HandleOps.canQuery(hChildWidget, "COLUMN")), () -> HandleOps.canQuery(hChildWidget, "WIDTH")), () -> HandleOps.canQuery(hChildWidget, "TYPE")), () -> isNotEqual(hChildWidget.unwrapType().getResourceType(), "LITERAL")), () -> isNotEqual(hChildWidget.unwrapType().getResourceType(), "BUTTON")))
            {
               /* Check if the object should be resized */
               if (_or(isEqual(indexOf("NO-RESIZE", hChildWidget.unwrap().getPrivateData()), 0), () -> isUnknown(indexOf("NO-RESIZE", hChildWidget.unwrap().getPrivateData()))))
               {
                  /* Will the size for the widget be valid ? */
                  if (_isGreaterThan(minus(dFrameMinimumWidth, hChildWidget.unwrapWidget().getColumn()), 0))
                  {
                     hChildWidget.unwrapSizeable().setWidthChars(minus(dFrameMinimumWidth, hChildWidget.unwrapWidget().getColumn()));
                  }
               }
            }
            hChildWidget.assign(hChildWidget.unwrap().getNextSibling());
         }));
         
         /* REPEAT */
         returnNormal();
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * ***********************  Function Implementations****************
    */
   public character formattedValue(final character _pcValue_5)
   {
      character pcValue_5 = TypeFactory.initInput(_pcValue_5);
      /*------------------------------------------------------------------------------
        Purpose: Return the formatted value of a passed value according to the 
                 SmartSelect/Dynamic Combo/Dynamic Lookup/SDF format. 
      Parameter: pcValue - The value that need to be formatted.      
          Notes: Used internally in order to ensure that unformatted data can be 
                 applied to screen-value (setDataValue) or used as lookup in
                 list-item-pairs in (getDisplayValue)  
                 This code was moved from select.p into field.p to accomodate
                 dynamic lookups and combos
      ------------------------------------------------------------------------------*/
      handle hSelection = TypeFactory.handle();
      character cFormat = TypeFactory.character();
      character cDataType = TypeFactory.character();
      
      return function("formattedValue", character.class, new Block((Body) () -> 
      {
         ErrorManager.silentErrorEnable();
         /* {get Prop var [handle]} */
         hSelection.assign(ControlFlowOps.invokeDynamicFunctionIn("getSelectionHandle", targetProcedure()));
         ErrorManager.silentErrorDisable();
         
         /* For selection lists */
         if (_and(hSelection.isValid(), () -> HandleOps.canQuery(hSelection, "FORMAT")))
         {
            cFormat.assign(hSelection.unwrapCommonField().getFormat());
            cDataType.assign(hSelection.unwrapCommonField().getDataType());
         }
         else
         {
            ErrorManager.silentErrorEnable();
            cFormat.assign(ControlFlowOps.invokeDynamicFunctionIn("getKeyFormat", targetProcedure()));
            ErrorManager.silentErrorDisable();
            ErrorManager.silentErrorEnable();
            cDataType.assign(ControlFlowOps.invokeDynamicFunctionIn("getKeyDataType", targetProcedure()));
            ErrorManager.silentErrorDisable();
         }
         
         /* Invalid Format */
         if (_or(or(isEqual(cFormat, ""), () -> isUnknown(cFormat)), () -> isEqual(cFormat, "?")))
         {
            returnNormal(pcValue_5);
         }
         
         if (_or(or(isEqual(cDataType, ""), () -> isUnknown(cDataType)), () -> isEqual(cDataType, "?")))
         {
            returnNormal(pcValue_5);
         }
         
         switch (StringHelper.safeTrimTrailing((cDataType).toStringMessage()).toUpperCase())
         {
            case "CHARACTER":
               {
                  pcValue_5.assign(rightTrim(valueOf(pcValue_5, cFormat)));
               }
               break;
            case "DATE":
               {
                  pcValue_5.assign(valueOf(new date(pcValue_5), cFormat));
               }
               break;
            case "DECIMAL":
               {
                  pcValue_5.assign(valueOf(new decimal(pcValue_5), cFormat));
               }
               break;
            case "INTEGER":
               {
                  pcValue_5.assign(valueOf(new integer(pcValue_5), cFormat));
               }
               break;
            case "LOGICAL":
               {
                  pcValue_5.assign(entry((matchesList("yes,true", pcValue_5)).booleanValue() ? new integer(1) : new integer(2), cFormat, "/"));
               }
               break;
         }
         
         returnNormal(pcValue_5);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getCustomSuperProc()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Returns the value of a SmartDataField.
         Params:  <none>
      ------------------------------------------------------------------------------*/
      character cCustomSuperProc = TypeFactory.character();
      
      return function("getCustomSuperProc", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("CustomSuperProc"));
         cCustomSuperProc.assign(ghProp.unwrapBufferField().value());
         returnNormal(cCustomSuperProc);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical getDataModified()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Returns true if the SmartDataField value has been
                  changed but not saved, otherwise false.
         Params:  <none>
      ------------------------------------------------------------------------------*/
      logical lModified = TypeFactory.logical();
      
      return function("getDataModified", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DataModified"));
         lModified.assign(ghProp.unwrapBufferField().value());
         returnNormal(lModified);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getDataValue()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Returns the value of a SmartDataField.
         Params:  <none>
      ------------------------------------------------------------------------------*/
      character cDataValue = TypeFactory.character();
      
      return function("getDataValue", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DataValue"));
         cDataValue.assign(ghProp.unwrapBufferField().value());
         returnNormal(cDataValue);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical getDisplayField()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Returns true if the SmartDataField is to be displayed
                  along with other fields in its Container, otherwise false.
         Params:  <none>
          Notes:  This instance property is initialized by the AppBuilder
                  in adm-create-objects.
      ------------------------------------------------------------------------------*/
      logical lDisplay = TypeFactory.logical();
      
      return function("getDisplayField", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DisplayField"));
         lDisplay.assign(ghProp.unwrapBufferField().value());
         returnNormal(lDisplay);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getDisplayValue()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Returns the saved screen/display value of a SmartDataField.
         Params:  <none>
      ------------------------------------------------------------------------------*/
      character cDisplayValue = TypeFactory.character();
      
      return function("getDisplayValue", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DisplayValue"));
         cDisplayValue.assign(ghProp.unwrapBufferField().value());
         returnNormal(cDisplayValue);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical getEnableField()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Returns true if the SmartDataField is to be enabled for user
                  input along with other fields in its Container, otherwise false.
         Params:  <none>
          Notes:  This instance property is initialized by the AppBuilder
                  in adm-create-objects.
      ------------------------------------------------------------------------------*/
      logical lEnable = TypeFactory.logical();
      
      return function("getEnableField", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("EnableField"));
         lEnable.assign(ghProp.unwrapBufferField().value());
         returnNormal(lEnable);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical getFieldEnabled()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Returns true if the SmartDataField is enabled for user
                  input, otherwise false.
         Params:  <none>
          Notes:  This property is set by user-defined SDF procedures
                  enableField and disableField.
      ------------------------------------------------------------------------------*/
      logical lEnabled = TypeFactory.logical();
      
      return function("getFieldEnabled", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FieldEnabled"));
         lEnabled.assign(ghProp.unwrapBufferField().value());
         returnNormal(lEnabled);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getFieldName()
   {
      /*------------------------------------------------------------------------------
        Purpose: Returns the name of the SDO field this object maps to.
         Params: <none>  
      ------------------------------------------------------------------------------*/
      character cField = TypeFactory.character();
      
      return function("getFieldName", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FieldName"));
         cField.assign(ghProp.unwrapBufferField().value());
         returnNormal(cField);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical getKeyFieldValue()
   {
      return function("getKeyFieldValue", logical.class, new Block((Body) () -> 
      {
         /*------------------------------------------------------------------------------
           Purpose: See DataValue  
             Notes: Obsolete, but kept for backwards compatibility with ICF 0.9  
         ------------------------------------------------------------------------------*/
         returnNormal( /* {fn <func-name> [handle] */ ControlFlowOps.invokeDynamicFunctionIn(character.class, "getDataValue", targetProcedure()));
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical getLocalField()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Returns TRUE if the field name begins with "<" (backswards 
                  compatibility where fields were called <Local> to indicate
                  that they were local fields) or it returns the value
                  of the LocalField property.
         Params:  <none>
      ------------------------------------------------------------------------------*/
      character cFieldName = TypeFactory.character();
      logical lLocal = TypeFactory.logical();
      
      return function("getLocalField", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FieldName"));
         cFieldName.assign(ghProp.unwrapBufferField().value());
         
         if (_begins(cFieldName, "<"))
         {
            returnNormal(true);
         }
         
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("LocalField"));
         lLocal.assign(ghProp.unwrapBufferField().value());
         returnNormal(lLocal);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical getSavedScreenValue()
   {
      return function("getSavedScreenValue", logical.class, new Block((Body) () -> 
      {
         /*------------------------------------------------------------------------------
           Purpose: See DisplayValue  
             Notes: Obsolete, but kept for backwards compatibility with ICF 0.9  
         ------------------------------------------------------------------------------*/
         returnNormal( /* {fn <func-name> [handle] */ ControlFlowOps.invokeDynamicFunctionIn(character.class, "getDisplayValue", targetProcedure()));
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public handle getSdfframeHandle()
   {
      /*------------------------------------------------------------------------------
        Purpose:  
          Notes:  
      ------------------------------------------------------------------------------*/
      handle hSource = TypeFactory.handle();
      handle hFrame = TypeFactory.handle();
      
      return function("getSDFFrameHandle", handle.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ContainerSource"));
         hSource.assign(ghProp.unwrapBufferField().value());
         
         if ((hSource.isValid()).booleanValue())
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("ContainerHandle"));
            hFrame.assign(ghProp.unwrapBufferField().value());
            
            if ((hFrame.isValid()).booleanValue())
            {
               returnNormal(hFrame);
            }
         }
         returnNormal(new handle());
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setCustomSuperProc(final character _pcCustomSuperProc_1)
   {
      character pcCustomSuperProc_1 = TypeFactory.initInput(_pcCustomSuperProc_1);
      
      return function("setCustomSuperProc", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("CustomSuperProc"));
         ghProp.unwrapBufferField().changeValue(pcCustomSuperProc_1);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setDataModified(final logical _plModified_1)
   {
      logical plModified_1 = TypeFactory.initInput(_plModified_1);
      /*------------------------------------------------------------------------------
        Purpose:  Sets the DataModified property of the SmartDataField,
                  and uses a standard event to notify the containing
                  SmartDataViewer of this change.
         Params:  plModified AS LOGICAL
      ------------------------------------------------------------------------------*/
      handle hContainer = TypeFactory.handle();
      
      return function("setDataModified", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DataModified"));
         ghProp.unwrapBufferField().changeValue(plModified_1);
         
         if ((plModified_1).booleanValue())
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("ContainerSource"));
            hContainer.assign(ghProp.unwrapBufferField().value());
            
            /* pass on to our parent. */
            if ((hContainer.isValid()).booleanValue())
            {
               ControlFlowOps.invokeInWithMode("fieldModified", hContainer, "I", targetProcedure());
            }
         }
 /* If it's "yes" then... */;
         /* END DO IF plModified */
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setDataValue(final character _pcValue_5)
   {
      character pcValue_5 = TypeFactory.initInput(_pcValue_5);
      
      return function("setDataValue", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DataValue"));
         ghProp.unwrapBufferField().changeValue(pcValue_5);
         returnNormal(false);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setDisplayField(final logical _plDisplay_1)
   {
      logical plDisplay_1 = TypeFactory.initInput(_plDisplay_1);
      
      return function("setDisplayField", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DisplayField"));
         ghProp.unwrapBufferField().changeValue(plDisplay_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setDisplayValue(final character _pcValue_5)
   {
      character pcValue_5 = TypeFactory.initInput(_pcValue_5);
      
      return function("setDisplayValue", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DisplayValue"));
         ghProp.unwrapBufferField().changeValue(pcValue_5);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setEnableField(final logical _plEnable_1)
   {
      logical plEnable_1 = TypeFactory.initInput(_plEnable_1);
      
      return function("setEnableField", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("EnableField"));
         ghProp.unwrapBufferField().changeValue(plEnable_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setFieldEnabled(final logical _plEnabled_1)
   {
      logical plEnabled_1 = TypeFactory.initInput(_plEnabled_1);
      
      return function("setFieldEnabled", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FieldEnabled"));
         ghProp.unwrapBufferField().changeValue(plEnabled_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setFieldName(final character _pcField_1)
   {
      character pcField_1 = TypeFactory.initInput(_pcField_1);
      
      return function("setFieldName", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FieldName"));
         ghProp.unwrapBufferField().changeValue(pcField_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setKeyFieldValue(final character _pcValue_5)
   {
      character pcValue_5 = TypeFactory.initInput(_pcValue_5);
      
      return function("setKeyFieldValue", logical.class, new Block((Body) () -> 
      {
         /*------------------------------------------------------------------------------
           Purpose: See DataValue 
             Notes: Kept for backwards compatibility with ICF 0.9  
         ------------------------------------------------------------------------------*/
         returnNormal( /* {set Prop value [handle]} */ ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "setDataValue", targetProcedure(), "I", pcValue_5));
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setLocalField(final logical _plLocal_1)
   {
      logical plLocal_1 = TypeFactory.initInput(_plLocal_1);
      
      return function("setLocalField", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("LocalField"));
         ghProp.unwrapBufferField().changeValue(plLocal_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setSavedScreenValue(final character _pcValue_5)
   {
      character pcValue_5 = TypeFactory.initInput(_pcValue_5);
      
      return function("setSavedScreenValue", logical.class, new Block((Body) () -> 
      {
         /*------------------------------------------------------------------------------
           Purpose: See DisplayValue 
             Notes: Kept for backwards compatibility with ICF 0.9  
         ------------------------------------------------------------------------------*/
         returnNormal( /* {set Prop value [handle]} */ ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "setDisplayValue", targetProcedure(), "I", pcValue_5));
      }));
   }
}
