/*--------------------------------------------------------------------------
    File        : Filter.java
    Purpose     : Super procedure for filter class.

    Syntax      : RUN start-super-proc("adm2/filter.p":U).

    Modified    : 06/28/1999
    Modified    : 10/24/2001        Mark Davies (MIP)
                  Set new property FilterAvailable to TRUE. This will ensure
                  that the Filter button is enabled on the toolbar.
  ------------------------------------------------------------------------*/

package com.goldencode.hotel.common.adm2;

import com.goldencode.p2j.util.*;
import com.goldencode.p2j.ui.*;
import com.goldencode.hotel.ui.common.adm2.*;
import com.goldencode.util.*;

import static com.goldencode.p2j.util.BlockManager.*;
import static com.goldencode.p2j.util.CompareOps.*;
import static com.goldencode.p2j.util.MathOps.*;
import static com.goldencode.p2j.util.logical.*;
import static com.goldencode.p2j.util.TextOps.*;
import static com.goldencode.p2j.util.ProcedureManager.*;
import static com.goldencode.p2j.util.KeyReader.*;
import static com.goldencode.p2j.util.SelfManager.*;
import static com.goldencode.p2j.util.character.*;
import static com.goldencode.p2j.ui.LogicalTerminal.*;

/**
 * Business logic (converted to Java from the 4GL source code
 * in common/adm2/filter.p).
 */
public class Filter
{
   /* _UIB-CODE-BLOCK-END */   
   /* *********************** Procedure Settings ************************ */   
   /* Settings for THIS-PROCEDURE
      Type: Procedure
      Allow: 
      Frames: 0
      Add Fields to: Neither
      Other Settings: CODE-ONLY COMPILE
    */   
   /* *************************  Create Window  ************************** */   
   /* DESIGN Window definition (used by the UIB) 
     CREATE WINDOW Procedure ASSIGN
            HEIGHT             = 13.67
            WIDTH              = 58.4.
   _* END WINDOW DEFINITION *_
                                                                           */   
   /* ************************* Included-Libraries *********************** */   
   /*********************************************************************
   * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
   * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
   * below.  All Rights Reserved.                                       *
   *                                                                    *
   * The Initial Developer of the Original Code is PSC.  The Original   *
   * Code is Progress IDE code released to open source December 1, 2000.*
   *                                                                    *
   * The contents of this file are subject to the Possenet Public       *
   * License Version 1.0 (the "License"); you may not use this file     *
   * except in compliance with the License.  A copy of the License is   *
   * available as of the date of this notice at                         *
   * http://www.possenet.org/license.html                               *
   *                                                                    *
   * Software distributed under the License is distributed on an "AS IS"*
   * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
   * should refer to the License for the specific language governing    *
   * rights and limitations under the License.                          *
   *                                                                    *
   * Contributors:                                                      *
   *                                                                    *
   *********************************************************************/   
   /*--------------------------------------------------------------------------
       File        : filtprop.i
       Purpose     : Defines basic properties.
       Syntax      : {src/adm2/filtprop.i}
   
       Description :
   
       Modified    : 06/28/1999
     ------------------------------------------------------------------------*/   
   /*          This .W file was created with the Progress UIB.             */   
   /*----------------------------------------------------------------------*/   
   /* ***************************  Definitions  ************************** */   
   /* Custom instance definition file */   
   /*********************************************************************
   * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
   * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
   * below.  All Rights Reserved.                                       *
   *                                                                    *
   * The Initial Developer of the Original Code is PSC.  The Original   *
   * Code is Progress IDE code released to open source December 1, 2000.*
   *                                                                    *
   * The contents of this file are subject to the Possenet Public       *
   * License Version 1.0 (the "License"); you may not use this file     *
   * except in compliance with the License.  A copy of the License is   *
   * available as of the date of this notice at                         *
   * http://www.possenet.org/license.html                               *
   *                                                                    *
   * Software distributed under the License is distributed on an "AS IS"*
   * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
   * should refer to the License for the specific language governing    *
   * rights and limitations under the License.                          *
   *                                                                    *
   * Contributors:                                                      *
   *                                                                    *
   *********************************************************************/   
   /*--------------------------------------------------------------------------
       File        : filterdefscustom.i
       Purpose     : Entry point to define:
                       New definitions
                       {&xcInstanceProperties}
                       {&ADM-PROPERTY-DLG}
       Syntax      : {src/adm2/custom/filterdefscustom.i}
   
       Modified    : 06/28/1999
       
       Notes       : Referenced in {src/adm2/filtprop.i}
     ------------------------------------------------------------------------*/   
   /* Custom instance definition file */   
   /* This is the procedure to execute to set InstanceProperties at design time. */   
   /* _UIB-CODE-BLOCK-END */   
   /* ********************  Preprocessor Definitions  ******************** */   
   /* _UIB-PREPROCESSOR-BLOCK-END */   
   /* *********************** Procedure Settings ************************ */   
   /* Settings for THIS-PROCEDURE
      Type: Include
      Allow: 
      Frames: 0
      Add Fields to: Neither
      Other Settings: INCLUDE-ONLY
    */   
   /* *************************  Create Window  ************************** */   
   /* DESIGN Window definition (used by the UIB) 
     CREATE WINDOW Include ASSIGN
            HEIGHT             = 8
            WIDTH              = 60.
   _* END WINDOW DEFINITION *_
                                                                           */   
   /* ***************************  Main Block  *************************** */   
   /* Include the file which defines prototypes for all of the super
        procedure's entry points. 
        And skip including the prototypes if we are *any* super procedure. */   
   /*********************************************************************
   * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
   * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
   * below.  All Rights Reserved.                                       *
   *                                                                    *
   * The Initial Developer of the Original Code is PSC.  The Original   *
   * Code is Progress IDE code released to open source December 1, 2000.*
   *                                                                    *
   * The contents of this file are subject to the Possenet Public       *
   * License Version 1.0 (the "License"); you may not use this file     *
   * except in compliance with the License.  A copy of the License is   *
   * available as of the date of this notice at                         *
   * http://www.possenet.org/license.html                               *
   *                                                                    *
   * Software distributed under the License is distributed on an "AS IS"*
   * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
   * should refer to the License for the specific language governing    *
   * rights and limitations under the License.                          *
   *                                                                    *
   * Contributors:                                                      *
   *                                                                    *
   *********************************************************************/   
   /*--------------------------------------------------------------------------
       File        : visprop.i
       Purpose     : Starts Super procedure visual.p and defines basic properties
                     for visual SmartObjects in V9
       Syntax      : {src/adm2/visprop.i}
   
       Description :
   
       Modified    : May 19, 1999 Version 9.1A
       Modified    : 10/23/2001          Mark Davies (MIP)
                     Added two new fields for ResizeHorizontal and ResizeVertical
       Modified    : 11/02/2001          Mark Davies (MIP)
                     Replaced properties for FrameMinHeightChars and FrameMinWidthChars with MinHeight and MinWidth
       Modified    : 11/14/2001          Mark Davies (MIP)
                     Renamed property 'DisplayFieldsSecurity' to 'FieldSecurity'
       Modified    : 02/27/2003          Neil Bell (MIP)
                     Added property 'SecuredTokens'                  
     ------------------------------------------------------------------------*/   
   /*          This .W file was created with the Progress UIB.             */   
   /*----------------------------------------------------------------------*/   
   /* ***************************  Definitions  ************************** */   
   /*********************************************************************
   * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
   * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
   * below.  All Rights Reserved.                                       *
   *                                                                    *
   * The Initial Developer of the Original Code is PSC.  The Original   *
   * Code is Progress IDE code released to open source December 1, 2000.*
   *                                                                    *
   * The contents of this file are subject to the Possenet Public       *
   * License Version 1.0 (the "License"); you may not use this file     *
   * except in compliance with the License.  A copy of the License is   *
   * available as of the date of this notice at                         *
   * http://www.possenet.org/license.html                               *
   *                                                                    *
   * Software distributed under the License is distributed on an "AS IS"*
   * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
   * should refer to the License for the specific language governing    *
   * rights and limitations under the License.                          *
   *                                                                    *
   * Contributors:                                                      *
   *                                                                    *
   *********************************************************************/   
   /*--------------------------------------------------------------------------
       File        : visualdefscustom.i
       Purpose     : Entry point to define:
                       New definitions
                       {&xcInstanceProperties}
                       {&ADM-PROPERTY-DLG}
       Syntax      : {src/adm2/custom/visualdefscustom.i}
   
       Modified    : 06/03/1999
       
       Notes       : Referenced in {src/adm2/visprop.i}
     ------------------------------------------------------------------------*/   
   /* This is the procedure to execute to set InstanceProperties at design time. */   
   /* _UIB-CODE-BLOCK-END */   
   /* ********************  Preprocessor Definitions  ******************** */   
   /* _UIB-PREPROCESSOR-BLOCK-END */   
   /* *********************** Procedure Settings ************************ */   
   /* Settings for THIS-PROCEDURE
      Type: Include
      Allow: 
      Frames: 0
      Add Fields to: Neither
      Other Settings: INCLUDE-ONLY
    */   
   /* *************************  Create Window  ************************** */   
   /* DESIGN Window definition (used by the UIB) 
     CREATE WINDOW Include ASSIGN
            HEIGHT             = 8
            WIDTH              = 60.
   _* END WINDOW DEFINITION *_
                                                                           */   
   /* ***************************  Main Block  *************************** */   
   /* Include the file which defines prototypes for all of the super
        procedure's entry points. Also, start or attach to the super procedure.
        Skip start-super-proc if we *are* the super procedure. 
        And skip including the prototypes if we are *any* super procedure. */   
   /* These preprocessors tell at compile time which properties can be
        retrieved directly from the temp-table. */   
   /* This property contains the security for all objects found in the AllFieldHandles Property */    
   /* Include the next property file up the chain (in this case, the top
        of the chain); it starts the property temp-table definition and we
        add our field definitions to it. */   
   /*********************************************************************
   * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
   * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
   * below.  All Rights Reserved.                                       *
   *                                                                    *
   * The Initial Developer of the Original Code is PSC.  The Original   *
   * Code is Progress IDE code released to open source December 1, 2000.*
   *                                                                    *
   * The contents of this file are subject to the Possenet Public       *
   * License Version 1.0 (the "License"); you may not use this file     *
   * except in compliance with the License.  A copy of the License is   *
   * available as of the date of this notice at                         *
   * http://www.possenet.org/license.html                               *
   *                                                                    *
   * Software distributed under the License is distributed on an "AS IS"*
   * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
   * should refer to the License for the specific language governing    *
   * rights and limitations under the License.                          *
   *                                                                    *
   * Contributors:                                                      *
   *                                                                    *
   *********************************************************************/   
   /*--------------------------------------------------------------------------
       File        : smrtprop.i
       Purpose     : Starts smart.p super procedure and defines general
                     SmartObject properties and other values.
       Syntax      : {src/adm2/smrtprop.i}
   
       Description :
   
       Modified    : February 4, 2000 - Version 9.1B
     ------------------------------------------------------------------------*/   
   /*          This .W file was created with the Progress UIB.             */   
   /*----------------------------------------------------------------------*/   
   /* ***************************  Definitions  ************************** */   
   /*********************************************************************
   * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
   * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
   * below.  All Rights Reserved.                                       *
   *                                                                    *
   * The Initial Developer of the Original Code is PSC.  The Original   *
   * Code is Progress IDE code released to open source December 1, 2000.*
   *                                                                    *
   * The contents of this file are subject to the Possenet Public       *
   * License Version 1.0 (the "License"); you may not use this file     *
   * except in compliance with the License.  A copy of the License is   *
   * available as of the date of this notice at                         *
   * http://www.possenet.org/license.html                               *
   *                                                                    *
   * Software distributed under the License is distributed on an "AS IS"*
   * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
   * should refer to the License for the specific language governing    *
   * rights and limitations under the License.                          *
   *                                                                    *
   * Contributors:                                                      *
   *                                                                    *
   *********************************************************************/   
   /*--------------------------------------------------------------------------
       File        : smartdefscustom.i
       Purpose     : Entry point to define:
                       New definitions
                       {&xcInstanceProperties}
                       {&ADM-PROPERTY-DLG}
       Syntax      : {src/adm2/custom/smartdefscustom.i}
   
       Modified    : 06/03/1999
       
       Notes       : Referenced in {src/adm2/smrtprop.i}
     ------------------------------------------------------------------------*/   
   /* Service Managers */   
   /*********************************************************************
   * Copyright (C) 2000-2002 by Progress Software Corporation ("PSC"),  *
   * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
   * below.  All Rights Reserved.                                       *
   *                                                                    *
   * The Initial Developer of the Original Code is PSC.  The Original   *
   * Code is Progress IDE code released to open source December 1, 2000.*
   *                                                                    *
   * The contents of this file are subject to the Possenet Public       *
   * License Version 1.0 (the "License"); you may not use this file     *
   * except in compliance with the License.  A copy of the License is   *
   * available as of the date of this notice at                         *
   * http://www.possenet.org/license.html                               *
   *                                                                    *
   * Software distributed under the License is distributed on an "AS IS"*
   * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
   * should refer to the License for the specific language governing    *
   * rights and limitations under the License.                          *
   *                                                                    *
   * Contributors:                                                      *
   *                                                                    *
   *********************************************************************/   
   /*---------------------------------------------------------------------------------
     File: globals.i
   
     Description:  ICF Global Shared Variables
   
     Purpose:      ICF Global Shared Variables
                   The global shared variables available within the Dynamics environment.
                   
                   NOTE: (Dynamic Only )
                   Shared variables do not span the client / server boundary. On the
                   client these are set-up in afstartupp.p and on the server in the agent
                   as_startup.p
                   Also note if we include the guts of a procedure as an internal
                   procedure for shared client/server code, we sometime need to
                   ensure the shared variables are not redefined.
   
     Parameters:   <none>
   
     History:
     --------
   
     Modified: 11/08/2001 - Mark Davies (MIP)
               Copied code for ICF afglobals.i and added here to allow
               ICF changed adm code to compile. afglobals.i now includes
               this file for definitions.
               Any ADM code will now reference globals.i which excludes any other
               ICF specific includes.
   -----------------------------------------------------------*/   
   /*                   This .W file was created with the Progress UIB.             */   
   /*-------------------------------------------------------------------------------*/   
   /* ***************************  Definitions  ************************** */   
   /* _UIB-CODE-BLOCK-END */   
   /* ********************  Preprocessor Definitions  ******************** */   
   /* _UIB-PREPROCESSOR-BLOCK-END */   
   /* *********************** Procedure Settings ************************ */   
   /* Settings for THIS-PROCEDURE
      Type: Include
      Allow: 
      Frames: 0
      Add Fields to: Neither
      Other Settings: INCLUDE-ONLY
    */   
   /* *************************  Create Window  ************************** */   
   /* DESIGN Window definition (used by the UIB) 
     CREATE WINDOW Include ASSIGN
            HEIGHT             = 5.57
            WIDTH              = 64.4.
   _* END WINDOW DEFINITION *_
                                                                           */   
   /* ***************************  Main Block  *************************** */   
   /* Used whereever Crystal is used and when defining the tt_datasource temp-table 
      to define the maximum columns supported when printing to crystal.
      See browsercustom.p and afcrpplip2p.p for example uses
   */   
   /* Used for filter checks */   
   /* The table delimiter used in assignlist, *ColumnsByTable props */   
   handle gshAstraAppserver /* quick access to all handles on object, e.g. viewer, whether enabled or not */  /* quick access to all fields on object, e.g. viewer, whether enabled or not */ = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshastraappserver", true, new handle());

   /* Handle to Astra Application Server Partition */   
   handle gshSessionManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshsessionmanager", true, new handle());

   /* Handle Astra Session Manager */   
   handle gshRimanager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshrimanager", true, new handle());

   /* Handle Astra Session Manager */   
   handle gshSecurityManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshsecuritymanager", true, new handle());

   /* Handle Astra Security Manager */   
   handle gshProfileManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshprofilemanager", true, new handle());

   /* Handle Astra Profile Manager */   
   handle gshRepositoryManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshrepositorymanager", true, new handle());

   /* Handle Astra Repository Manager */   
   handle gshTranslationManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshtranslationmanager", true, new handle());

   /* Handle Astra Translation Manager */   
   handle gshWebManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshwebmanager", true, new handle());

   /* Handle Astra Web Manager */   
   character gscSessionId = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gscsessionid", true, new character(""));

   /* Unique session id */   
   decimal gsdSessionObj = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gsdsessionobj", true, new decimal(0));

   /* Unique session id */   
   handle gshFinManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshfinmanager", true, new handle());

   /* Handle AstraFin Generic Manager */   
   handle gshGenManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshgenmanager", true, new handle());

   /* Handle AstraFramework Generic Manager */   
   handle gshAgnManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshagnmanager", true, new handle());

   /* Handle AstraGen Generic Manager */   
   decimal gsdTempUniqueId = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gsdtempuniqueid", true, new decimal(0));

   /* A number for unique objects */   
   decimal gsdUserObj = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gsduserobj", true, new decimal(0));

   /* User Object ID */   
   decimal gsdRenderTypeObj = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gsdrendertypeobj", true, new decimal(0));

   /* Render Type Object ID */   
   decimal gsdSessionScopeObj = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gsdsessionscopeobj", true, new decimal(0));

   /* Render Type Object ID */   
   /*********************************************************************
   * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
   * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
   * below.  All Rights Reserved.                                       *
   *                                                                    *
   * The Initial Developer of the Original Code is PSC.  The Original   *
   * Code is Progress IDE code released to open source December 1, 2000.*
   *                                                                    *
   * The contents of this file are subject to the Possenet Public       *
   * License Version 1.0 (the "License"); you may not use this file     *
   * except in compliance with the License.  A copy of the License is   *
   * available as of the date of this notice at                         *
   * http://www.possenet.org/license.html                               *
   *                                                                    *
   * Software distributed under the License is distributed on an "AS IS"*
   * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
   * should refer to the License for the specific language governing    *
   * rights and limitations under the License.                          *
   *                                                                    *
   * Contributors:                                                      *
   *                                                                    *
   *********************************************************************/   
   /*------------------------------------------------------------------------
       File        : globalscustom.i
       Purpose     : The developer can create his own version of this to 
                     allow 'global' definitions to be added to the 
                     framework.   
       Syntax      : {src/adm2/custom/globalscustom.i}
   
       Description :
   
       Created     : 06/03/1999
       Notes       : Referenced in {src/adm2/globals.i}
     ----------------------------------------------------------------------*/   
   /*          This .W file was created with the Progress AppBuilder.      */   
   /*----------------------------------------------------------------------*/   
   /* ***************************  Definitions  ************************** */   
   /* _UIB-CODE-BLOCK-END */   
   /* define the ADM Version and broker handle for all SmartObjects */   
   /* Note that adm-props-from-repository is documented as possible to define
        as a variable for flexible override, so it cannot be used for conditional 
        compile.  */   
   /* adm-props-from-repository is supported to be defined as a variable,
       so it is not used for conditional compile */   
   handle ghProp = TypeFactory.handle();

   FilterX xFrame = null;

   /**
    * External procedure (converted to Java from the 4GL source code
    * in common/adm2/filter.p).
    */
   public void execute()
   {
      externalProcedure(Filter.this, new Block((Init) () -> 
      {
         gsdSessionObj.setPrecision(9);
         gsdTempUniqueId.setPrecision(9);
         gsdUserObj.setPrecision(9);
         gsdRenderTypeObj.setPrecision(9);
         gsdSessionScopeObj.setPrecision(9);
      }, 
      (Body) () -> 
      {
         /* _UIB-CODE-BLOCK-END */
         /* *********************** Procedure Settings ************************ */
         /* Settings for THIS-PROCEDURE
            Type: Method-Library
            Allow: 
            Frames: 0
            Add Fields to: Neither
            Other Settings: INCLUDE-ONLY
          */
         /* *************************  Create Window  ************************** */
         /* DESIGN Window definition (used by the UIB) 
           CREATE WINDOW Method-Library ASSIGN
                  HEIGHT             = 14.86
                  WIDTH              = 57.2.
         _* END WINDOW DEFINITION *_
                                                                                 */
         /* ***************************  Main Block  *************************** */
         /* Include the file which defines prototypes for all of the super
              procedure's entry points. Also, start or attach to the super procedure.
              Skip start-super-proc if we *are* the super procedure. 
              And skip including the prototypes if we are *any* super procedure. */
         /* These preprocessors let the get and set methods know at compile time
             which property values are located in the temp-table and which must
             be accessed through the property functions.
          */
         /* for support of dynamic objects */
         /* physical name - no path */
         /* for support of runtime parameters passed from the menus */
         /* Properties for super procedure management */
         /* The name of the super procedure; can be a CSV list */ 
 /* STATELESS, STATEFUL; the mode that all supers for the object will be launched in */ 
 /* Runtime property storing handles to the object's super procedures, defined in SuperProcedure. */
         /* Layout support for dynamic objects */
         /* contains a layout position that allows an object to be laid out correctly. */
         /* determine if we need to create the prop fields here or from the */
         /* repository (if ICF is running ) */
         /* COMMENT THE FOLLOWING LINE TO USE THE "<class>prop.i" DEFINITIONS */
         /* This temp-table defines all the propertt fields for an object.
             This include file contributes the DEFINE statement header and
             all basic smart object properties. Each other property class include file
             adds its own fields and then the parent object ends the statement.
             Define the fields for smart objects only, not for their super procedures.
          */
         if (_not(handle.fromString(entry(1, thisProcedure().unwrapADMData().getADMData(), chr(1))).isValid()))
         {
         }
         
         /* _UIB-CODE-BLOCK-END */
         /* else if appserver aware (adecomm/appserv.i) include appserver props */
         if (_not(handle.fromString(entry(1, thisProcedure().unwrapADMData().getADMData(), chr(1))).isValid()))
         {
         }
         
         /* _UIB-CODE-BLOCK-END */
         if (_not(handle.fromString(entry(1, thisProcedure().unwrapADMData().getADMData(), chr(1))).isValid()))
         {
         }
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * ***********************  Function Implementations****************
    */
   public character getObjectType()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Returns the Type of the SmartObject, such as "SmartDataObject", etc.
                  For an ADM Super procedure, this function returns "SUPER".
         Params:  <none>
      ------------------------------------------------------------------------------*/
      character cType = TypeFactory.character();
      
      return function("getObjectType", character.class, new Block((Body) () -> 
      {
         cType.assign("SUPER");
         returnNormal(cType);
      }));
   }

   /** *********************  Internal Procedures ********************** */
   /**
    * Purpose: Apply filter criteria to the filter-target     <br>
    * Parameters:  <br>
    * Notes:   The procedure parses the dynamic filter fields and builds a list<br>
    *          of fields, operators and values to pass to the query. <br>
    *          The fields are only added to the list if the field or the operator <br>
    *          is MODIFIED. (We may need a better mechanism to identify  <br>
    *          actually blanks vlaues that needs to be added to the query)<br>
    *          If columnQuerySelection returns any criteria for the field with an<br>
    *          operator that are not in the list to add it will be <br>
    *          added to a list of fields and operators to remove form the query. <br>
    *          The operator for the field is either<br>
    *          - In a separate widget. (combo-box or radio-set).<br>
    *          - If wildcard in the value we use MATCHES or BEGINS.<br>
    *          - GE and LE if there is a range field with a second value.<br>
    *          - First blank separated entry in the field if OperatorStyle is set <br>
    *            to "inline"<br>
    *          - Implicit specified by the Operator property.
    */
   public void applyFilter()
   {
      integer iField = TypeFactory.integer();
      handle hField = TypeFactory.handle();
      character enabledHandles = TypeFactory.character();
      character cFieldNames = TypeFactory.character();
      character cFieldValues = TypeFactory.character();
      character cField = TypeFactory.character();
      character cValue = TypeFactory.character();
      character cOperator = TypeFactory.character();
      character cRemoveFields = TypeFactory.character();
      handle hOperator = TypeFactory.handle();
      handle hRangeField = TypeFactory.handle();
      character cOperatorStyle = TypeFactory.character();
      character cOperatorHandles = TypeFactory.character();
      character cDefaultOperator = TypeFactory.character();
      character cFieldOperators = TypeFactory.character();
      character cOldSelection = TypeFactory.character();
      character cOldOperator = TypeFactory.character();
      character cOldValue = TypeFactory.character();
      character cRemoveoperators = TypeFactory.character();
      logical lUseBegins = TypeFactory.logical();
      logical lRange = TypeFactory.logical();
      integer iWildCard = TypeFactory.integer();
      integer iWildCardChar = TypeFactory.integer();
      handle hFilterTarget = TypeFactory.handle();
      character cFilterTarget = TypeFactory.character();
      integer iValue = TypeFactory.integer();
      logical lCancel = TypeFactory.logical();
      handle hdl = TypeFactory.handle();
      logical lFirst = TypeFactory.logical();
      character xcInlineOpList = TypeFactory.character(">=,>,<,<=");
      character xcRangeOperator = TypeFactory.character("<=");
      
      internalProcedure(new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("EnabledHandles"));
         enabledHandles.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("OperatorHandles"));
         cOperatorHandles.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("OperatorStyle"));
         cOperatorStyle.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("Operator"));
         cDefaultOperator.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("useBegins"));
         lUseBegins.assign(ghProp.unwrapBufferField().value());
         /* {get Prop var [handle]} */ 
         cFilterTarget.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getFilterTarget", targetProcedure()));
         /* What happened to the intention to support multiple filter targets ?*/
         hFilterTarget.assign(handle.fromString(cFilterTarget));
         
         if (_not(hFilterTarget.isValid()))
         {
            returnNormal();
         }
         
         /* Is there any unsaved or uncommitted data? */
         ControlFlowOps.invokeInWithMode("confirmContinue", hFilterTarget, "U", lCancel);
         
         if ((lCancel).booleanValue())
         {
            returnNormal();
         }
         
         IntegerExpression resExpr0 = new IntegerExpression()
         {
            public integer execute()
            {
               return numEntries(enabledHandles);
            }
         };
         ToClause toClause0 = new ToClause(iField, 1, resExpr0);
         
         doTo("loopLabel0", toClause0, new Block((Body) () -> 
         {
            hField.assign(handle.fromString(entry(iField, enabledHandles)));
            hOperator.setUnknown();
            hRangeField.setUnknown();
            cField.assign(concat((_isEqual(numEntries(hField.unwrap().name(), "."), 1) ? new character("RowObject.") : new character("")), hField.unwrap().name()));
            cOperator.assign("");
            
            /* Chweck if we have a true operator or a range field */
            if (_isNotEqual(cOperatorHandles, ""))
            {
               hdl.assign(handle.fromString(entry(iField, cOperatorHandles)));
               
               if ((hdl.isValid()).booleanValue())
               {
                  if (_isEqual(hdl.unwrapType().getResourceType(), "FILL-IN"))
                  {
                     hRangeField.assign(hdl);
                  }
                  else
                  {
                     hOperator.assign(hdl);
                  }
               }
            }
            
            /* if cOperatorhandle <> '' */
            if (_or(hField.unwrapWidget().isModified(), () -> and(hOperator.isValid(), () -> hOperator.unwrapWidget().isModified())))
            {
               cValue.assign(hField.unwrapWidget().inputValue());
               cOperator.assign(cDefaultOperator);
               iWildCardChar.assign(indexOf(cValue, "."));
               iWildCard.assign(indexOf(cValue, "*"));
               iWildCard.assign(_and(lUseBegins, () -> isEqual(iWildCard, length(cValue))) ? new integer(0) : iWildCard);
               iWildCard.assign(_and(isGreaterThan(iWildCard, 0), () -> isGreaterThan(iWildCardChar, 0)) ? int64.minimum(iWildCard, iWildCardChar) : int64.maximum(iWildCard, iWildCardChar));
               
               /* A valid hOperator means that the screen-value holds the operator */
               if ((hOperator.isValid()).booleanValue())
               {
                  cOperator.assign(hOperator.unwrapWidget().getScreenValue());
               }
               else
               {
                  if (_isEqual(hField.unwrapType().getResourceType(), "EDITOR"))
                  {
                     cOperator.assign("CONTAINS");
                  }
                  else
                  {
                     if (_isGreaterThan(iWildCard, 0))
                     {
                        cOperator.assign(_and(isEqual(iWildCardChar, 0), () -> isEqual(iWildCard, length(cValue))) ? new character("BEGINS") : new character("MATCHES"));
                        cValue.assign(_isEqual(cOperator, "BEGINS") ? substring(cValue, 1, minus(iWildCard, 1)) : cValue);
                     }
                     else
                     {
                        if ((hRangeField.isValid()).booleanValue())
                        {
                           cOperator.assign(">=");
                        }
                        else
                        {
                           if (_isEqual(hField.unwrapCommonField().getDataType(), "CHARACTER"))
                           {
                              if (_and(and(isNotEqual(cValue, ""), () -> isEqual(cOperatorStyle, "INLINE")), () -> matchesList(xcInlineOpList, entry(1, cValue, " "))))
                              {
                                 cOperator.assign(entry(1, cValue, " "));
                                 cValue.assign(substring(cValue, plus(length(cOperator), 2)));
                              }
                              else
                              {
                                 if (_and(lUseBegins, () -> isEqual( /* {fnarg <func-name> <arg> [handle] */ ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "columnDataType", targetProcedure(), "I", hField.unwrap().name()), "CHARACTER")))
                                 {
                                    cOperator.assign("BEGINS");
                                 }
                              }
                           }
                        }
                     }
                  }
               }
               /* else if hField:data-type = CHAR */
               /* This is not needed to remove formats as we are using INPUT-VALUE,
                        but it's useful for data-type checking when inline */
               cValue.assign(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "dataValue", targetProcedure(), "II", hField.unwrap().name(), cValue));
            }
            /* Should we add the rangefield screen-value  
                     Note that blank is not supported */
            lRange.assign(and(and(hRangeField.isValid(), () -> hRangeField.unwrapWidget().isModified()), () -> isNotEqual(hRangeField.unwrapWidget().getScreenValue(), "")));
            /* Find all old criteria for this field and add them to the list of 
                   field/operators to remove.
                   Currently we don't care about checking if the value already is 
                   in the query. */
            cOldSelection /* {fnarg <func-name> <arg> [handle] */.assign(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "columnQuerySelection", hFilterTarget, "I", cField));
            
            if (_isNotEqual(cOldSelection, ""))
            {
               loopLabel1:
               for (iValue.assign(1); _isLessThanOrEqual(iValue, numEntries(cOldSelection, chr(1))); iValue.assign(plus(iValue, 2)))
               {
                  cOldOperator.assign(entry(iValue, cOldSelection, chr(1)));
                  cOldValue.assign(entry(plus(iValue, 1), cOldSelection, chr(1)));
                  
                  if (_or(isNotEqual(cOldOperator, cOperator), () -> and(isEqual(cOldOperator, xcRangeOperator), not(lRange))))
                  {
                     lFirst.assign(isEqual(cRemoveFields, ""));
                     cRemoveFields.assign(((lFirst).booleanValue() ? cField : concat(cRemoveFields, ",", cField)));
                     cRemoveoperators.assign(((lFirst).booleanValue() ? cOldOperator : concat(cRemoveoperators, ",", cOldOperator)));
                  }
               }
            }
            
            /* if cOldSelection <> "":U then DO iValue = 1 TO NUM-ENTRIES */
            /* if coperator is set we have a value to add to the list */
            if (_isNotEqual(cOperator, ""))
            {
               /*  We may have blank entries so we cannot do 'if  = '' and 
                           this also is a problem when trying to trim, so we use a logical */
               lFirst.assign(isEqual(cFieldNames, ""));
               cFieldNames.assign(((lFirst).booleanValue() ? cField : concat(cFieldNames, ",", cField)));
               cFieldOperators.assign(((lFirst).booleanValue() ? cOperator : concat(cFieldOperators, ",", cOperator)));
               cFieldValues.assign(concat(((lFirst).booleanValue() ? new character("") : concat(cFieldValues, chr(1))), (_isUnknown(cValue) ? new character("?") : cValue)));
            }
            
            if ((lRange).booleanValue())
            {
               /*  We may gave blank entries so we cannot do 'if  = '' and 
                          this also is a problem when trying to trim, so we use a logical */
               lFirst.assign(isEqual(cFieldNames, ""));
               cFieldNames.assign(((lFirst).booleanValue() ? cField : concat(cFieldNames, ",", cField)));
               cFieldOperators.assign(((lFirst).booleanValue() ? cOperator : concat(cFieldOperators, ",", "<=")));
               cFieldValues.assign(concat(((lFirst).booleanValue() ? new character("") : concat(cFieldValues, chr(1))), (_isUnknown(hRangeField.unwrapWidget().getScreenValue()) ? new character("?") : hRangeField.unwrapWidget().getScreenValue())));
            }
         }));
         
         /*  do iField = 1 to .. */
         if ((new logical( /* {fn <func-name> [handle] */ ControlFlowOps.invokeDynamicFunctionIn(logical.class, "anyMessage", targetProcedure()))).booleanValue())
         {
            cField /* {fn <func-name> [handle] */ .assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "showDataMessages", targetProcedure()));
            ControlFlowOps.invokeInWithMode("applyEntry", targetProcedure(), "I", cField);
         }
         else
         {
            if (_isNotEqual(cRemoveFields, ""))
            {
               ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "removeQuerySelection", hFilterTarget, "II", cRemoveFields, cRemoveoperators);
            }
            
            if (_isNotEqual(cFieldNames, ""))
            {
               ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "assignQuerySelection", hFilterTarget, "III", cFieldNames, cFieldValues, cFieldOperators);
            }
            
            ControlFlowOps.invokeDynamicFunctionIn(logical.class, "openQuery", hFilterTarget);
         }
         returnNormal();
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose: Set all filter fields to blank and unmodified    <br>
    * Parameters:  &lt;none&gt;<br>
    * Notes:
    */
   public void blankFields()
   {
      handle hFrameField = TypeFactory.handle();
      handle hOperator = TypeFactory.handle();
      handle hdl = TypeFactory.handle();
      handle hRangeField = TypeFactory.handle();
      integer iValue = TypeFactory.integer();
      character cEnabledHandles = TypeFactory.character();
      character cOperatorHandles = TypeFactory.character();
      
      internalProcedure(new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("EnabledHandles"));
         cEnabledHandles.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("OperatorHandles"));
         cOperatorHandles.assign(ghProp.unwrapBufferField().value());
         
         IntegerExpression resExpr1 = new IntegerExpression()
         {
            public integer execute()
            {
               return numEntries(cEnabledHandles);
            }
         };
         ToClause toClause1 = new ToClause(iValue, 1, resExpr1);
         
         doTo("loopLabel2", toClause1, new Block((Body) () -> 
         {
            hFrameField.assign(handle.fromString(entry(iValue, cEnabledHandles)));
            hRangeField.setUnknown();
            hOperator.setUnknown();
            
            if (_isNotEqual(cOperatorHandles, ""))
            {
               hdl.assign(handle.fromString(entry(iValue, cOperatorHandles)));
               
               if ((hdl.isValid()).booleanValue())
               {
                  if (_isEqual(hdl.unwrapType().getResourceType(), "FILL-IN"))
                  {
                     hRangeField.assign(hdl);
                  }
                  else
                  {
                     hOperator.assign(hdl);
                  }
               }
            }
            /* if cOperatorhandle <> '' */
            ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "blankField", targetProcedure(), "III", hFrameField, hRangeField, hOperator);
         }));
         
         returnNormal();
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose: This procedure is subscribed to the filter-target in order to <br>
    *          make sure that the filter reflects ForeignFields and values.     <br>
    * Parameters: pcRelative - NOT USED <br>
    * Notes:   This is necessary in order not to override the Foreignfield <br>
    *          criteria in the query
    */
   public void dataAvailable(final character _pcRelative)
   {
      character pcRelative = TypeFactory.initInput(_pcRelative);
      character cField = TypeFactory.character();
      handle hField = TypeFactory.handle();
      character cForeignFields = TypeFactory.character();
      character cForeignValues = TypeFactory.character();
      handle hFilterTarget = TypeFactory.handle();
      character cFilterTarget = TypeFactory.character();
      character cDisplayedFields = TypeFactory.character();
      character cFieldHandles = TypeFactory.character();
      integer i = TypeFactory.integer();
      integer cnt = TypeFactory.integer();
      integer iField = TypeFactory.integer();
      
      internalProcedure(new Block((Body) () -> 
      {
         /* {get Prop var [handle]} */ 
         cFilterTarget.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getFilterTarget", targetProcedure()));
         /* Proc. handle of SDO. */
         hFilterTarget.assign(handle.fromString(cFilterTarget));
         
         if ((hFilterTarget.isValid()).booleanValue())
         {
            /* {get Prop var [handle]} */
            cForeignFields.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getForeignFields", hFilterTarget));
            /* {get Prop var [handle]} */
            cForeignValues.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getForeignValues", hFilterTarget));
            
            if (_isNotEqual(cForeignValues, ""))
            {
               ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
               ghProp.assign(ghProp.unwrapBuffer().bufferField("FieldHandles"));
               cFieldHandles.assign(ghProp.unwrapBufferField().value());
               ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
               ghProp.assign(ghProp.unwrapBuffer().bufferField("DisplayedFields"));
               cDisplayedFields.assign(ghProp.unwrapBufferField().value());
               
               loopLabel3:
               for (i.assign(2); _isLessThanOrEqual(i, numEntries(cForeignFields)); i.assign(plus(i, 2)))
               {
                  cField.assign(entry(i, cForeignFields));
                  cnt.assign(plus(cnt, 1));
                  iField.assign(lookup(cField, cDisplayedFields));
                  
                  if (_isNotEqual(iField, 0))
                  {
                     hField.assign(handle.fromString(entry(iField, cFieldHandles)));
                     hField.unwrapWidget().setScreenValue(entry(cnt, cForeignValues, chr(1)));
                  }
               }
            }
         }
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     Disables fields in the ENABLED-FIELDS list<br>
    * Parameters:  &lt;none&gt; <br>
    * Notes:
    */
   public void disableFields()
   {
      character cEnableFields = TypeFactory.character();
      /* List of handles. */
      integer iField = TypeFactory.integer();
      handle hField = TypeFactory.handle();
      handle hOperator = TypeFactory.handle();
      character cOperatorHandles = TypeFactory.character();
      logical lEnabled = TypeFactory.logical();
      
      internalProcedure(new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FieldsEnabled"));
         lEnabled.assign(ghProp.unwrapBufferField().value());
         
         if ((lEnabled).booleanValue())
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("EnabledHandles"));
            cEnableFields.assign(ghProp.unwrapBufferField().value());
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("operatorHandles"));
            cOperatorHandles.assign(ghProp.unwrapBufferField().value());
            
            loopLabel4:
            for (iField.assign(1); _isLessThanOrEqual(iField, numEntries(cEnableFields)); iField.increment())
            {
               hField.assign(handle.fromString(entry(iField, cEnableFields)));
               hOperator.assign(handle.fromString(entry(iField, cOperatorHandles)));
               
               if (_not(hField.unwrapWidget().isHidden()))
               {
                  hField.unwrapWidget().setReadOnly(new logical(true));
                  
                  if ((hOperator.isValid()).booleanValue())
                  {
                     if (_isEqual(hOperator.unwrapType().getResourceType(), "FILL-IN"))
                     {
                        hOperator.unwrapWidget().setReadOnly(new logical(true));
                     }
                     else
                     {
                        hOperator.unwrapSensitive().setSensitive(new logical(false));
                     }
                  }
               }
 /* Skip fields hidden for multi-layout etc. */;
            }
            
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("fieldsEnabled"));
            ghProp.unwrapBufferField().changeValue(new logical(false));
         }
         returnNormal();
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     Enables fields in the ENABLED-FIELDS list<br>
    * Parameters:  &lt;none&gt;
    */
   public void enableFields()
   {
      character cEnableFields = TypeFactory.character();
      /* List of handles. */
      integer iField = TypeFactory.integer();
      handle hField = TypeFactory.handle();
      handle hOperator = TypeFactory.handle();
      character cOperatorHandles = TypeFactory.character();
      logical lEnabled = TypeFactory.logical();
      
      internalProcedure(new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FieldsEnabled"));
         lEnabled.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("EnabledHandles"));
         cEnableFields.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("operatorHandles"));
         cOperatorHandles.assign(ghProp.unwrapBufferField().value());
         
         loopLabel5:
         for (iField.assign(1); _isLessThanOrEqual(iField, numEntries(cEnableFields)); iField.increment())
         {
            hField.assign(handle.fromString(entry(iField, cEnableFields)));
            hOperator.assign(handle.fromString(entry(iField, cOperatorHandles)));
            
            if (_not(hField.unwrapWidget().isHidden()))
            {
               hField.unwrapSensitive().setSensitive(new logical(true));
               hField.unwrapWidget().setReadOnly(new logical(false));
               
               if ((hOperator.isValid()).booleanValue())
               {
                  if (_isEqual(hOperator.unwrapType().getResourceType(), "FILL-IN"))
                  {
                     hOperator.unwrapWidget().setReadOnly(new logical(false));
                  }
                  
                  hOperator.unwrapSensitive().setSensitive(new logical(true));
               }
            }
 /* Skip fields hidden for multi-layout */;
         }
         
         /* If the list of enabled field handles isn't set, it may because this
              object hasn't been fully initialized yet. So don't set the enabled
              flag because we may be through here again. */
         if (_isNotEqual(cEnableFields, ""))
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("FieldsEnabled"));
            ghProp.unwrapBufferField().changeValue(new logical(true));
         }
         /* {set Prop value [handle]} */ 
         
         returnNormal();
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     Filter object initialization code.<br>
    * Parameters:  &lt;none&gt;<br>
    * Note:        Dynamic widgets are created for the list of fields specified <br>
    *              in DisplayedFields.<br>
    *              The deleteObjects() function is called in order to clean up <br>
    *              to be able to reflect changes  in the  Instance Property dialog<br>
    *            - WordIndexedFields property is NEVER checked again.<br>
    *              Operator objects will assume that CONTAINS is allowed if <br>
    *              the filter field object type is EDITOR. <br>
    *              UseContains will also only work together with EDITOR fields. <br>
    *            - RANGE will NOT be used for wordindexed fields unless <br>
    *              UseContains is FALSE.
    */
   public void initializeObject()
   {
      handle hFilterTarget = TypeFactory.handle();
      character cFilterTarget = TypeFactory.character();
      character cFieldHandles = TypeFactory.character();
      character cLabelHandles = TypeFactory.character();
      character cDataColumns = TypeFactory.character();
      character cOperatorHandles = TypeFactory.character();
      character cOperatorStyle = TypeFactory.character();
      decimal dHeight = TypeFactory.decimal();
      decimal dFieldColumn = TypeFactory.decimal();
      integer iNumSkipped = TypeFactory.integer();
      integer iFieldSpacingPxl = TypeFactory.integer();
      integer iFieldSeparatorPxl = TypeFactory.integer();
      decimal dDefaultWidth = TypeFactory.decimal();
      decimal dDefaultCharWidth = TypeFactory.decimal();
      decimal dWidth = TypeFactory.decimal();
      decimal dRow = TypeFactory.decimal();
      integer iEditorLines = TypeFactory.integer();
      decimal dMinHeight = TypeFactory.decimal();
      character cOperatorViewAs = TypeFactory.character();
      character cOperatorShortValues = TypeFactory.character();
      character cOperatorLongValues = TypeFactory.character();
      character cOperator = TypeFactory.character();
      character cEnabledHandles = TypeFactory.character();
      character cDisplayedFields = TypeFactory.character();
      character cUibmode = TypeFactory.character();
      integer iField = TypeFactory.integer();
      character cField = TypeFactory.character();
      character cLabel = TypeFactory.character();
      character cToolTip = TypeFactory.character();
      integer iHelpId = TypeFactory.integer();
      character cOpValues = TypeFactory.character();
      handle hLabel = TypeFactory.handle();
      handle hOperator = TypeFactory.handle();
      character cFormat = TypeFactory.character();
      character cDataType = TypeFactory.character();
      handle hContainerHandle = TypeFactory.handle();
      handle hField = TypeFactory.handle();
      logical lWordIdx = TypeFactory.logical();
      logical lUseContains = TypeFactory.logical();
      character cWordIdxFields = TypeFactory.character();
      character cForeignFields = TypeFactory.character();
      decimal dOperatorWidth = TypeFactory.decimal();
      integer iOp = TypeFactory.integer();
      logical lHidden = TypeFactory.logical();
      
      internalProcedure(new Block((Body) () -> 
      {
         ControlFlowOps.runSuper("initializeObject");
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("UIBmode"));
         cUibmode.assign(ghProp.unwrapBufferField().value());
         
         if (_begins(cUibmode, "DESIGN"))
         {
            /* {fn <func-name> [handle] */ 
            ControlFlowOps.invokeDynamicFunctionIn(logical.class, "deleteObjects", targetProcedure());
         }
         
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FieldHandles"));
         cFieldHandles.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FieldSpacingPXL"));
         iFieldSpacingPxl.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FieldSeparatorPXL"));
         iFieldSeparatorPxl.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DefaultEditorLines"));
         iEditorLines.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DefaultCharWidth"));
         dDefaultCharWidth.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DefaultWidth"));
         dDefaultWidth.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DefaultHeight"));
         dHeight.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FieldColumn"));
         dFieldColumn.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("Operator"));
         cOperator.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("UseContains"));
         lUseContains.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("OperatorShortValues"));
         cOperatorShortValues.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("OperatorLongValues"));
         cOperatorLongValues.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("OperatorViewAs"));
         cOperatorViewAs.assign(ghProp.unwrapBufferField().value());
         
         if (_isEqual(cFieldHandles, ""))
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("DisplayedFields"));
            cDisplayedFields.assign(ghProp.unwrapBufferField().value());
            /* {get Prop var [handle]} */ 
            cFilterTarget.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getFilterTarget", targetProcedure()));
            
            /* Proc. handle of SDO. */
            if (_isNotEqual(cFilterTarget, ""))
            {
               /* use the last one for now */
               hFilterTarget.assign(handle.fromString(entry(numEntries(cFilterTarget), cFilterTarget)));
            }
            
            if ((hFilterTarget.isValid()).booleanValue())
            {
               /* {get Prop var [handle]} */
               cDataColumns.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getDataColumns", hFilterTarget));
               /* {get Prop var [handle]} */
               cWordIdxFields.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getWordIndexedFields", hFilterTarget));
               
               /* If Displayed fields is blank, take all fields from the SDO. */
               if (_isEqual(cDisplayedFields, ""))
               {
                  ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
                  ghProp.assign(ghProp.unwrapBuffer().bufferField("DisplayedFields"));
                  ghProp.unwrapBufferField().changeValue(cDataColumns);
               }
               /* {set Prop value [handle]} */ 
               
               /* Set the property. */
               /* {get Prop var [handle]} */
               cForeignFields.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getForeignFields", hFilterTarget));
               /* {set Prop value [handle]} */
               ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "setFilterAvailable", hFilterTarget, "I", true);
            }
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("ContainerHandle"));
            hContainerHandle.assign(ghProp.unwrapBufferField().value());
            
            /* Parse non-dynamic widgets to decide the minimumm size of the frame.
                   The dynamic ones were deleted by the deleteObject call above */
            if (_begins(cUibmode, "DESIGN"))
            {
               hField.assign(hContainerHandle.unwrapTree().firstChild());
               hField.assign(hField.unwrapTree().firstChild());
               
               loopLabel6:
               while ((hField.isValid()).booleanValue())
               {
                  dMinHeight.assign(decimal.maximum(dMinHeight, minus(plus(hField.unwrapWidget().getRow(), hField.unwrapSizeable().getHeightChars()), 1)));
                  hField.assign(hField.unwrap().getNextSibling());
               }
            }
            
            lHidden.assign(hContainerHandle.unwrapWidget().isHidden());
            hContainerHandle.unwrapWidget().setHidden(new logical(true));
            dRow.assign(1);
            cOpValues.assign(_isEqual(cOperatorViewAs, "combo-box") ? cOperatorLongValues : cOperatorShortValues);
            
            IntegerExpression resExpr2 = new IntegerExpression()
            {
               public integer execute()
               {
                  return numEntries(cDisplayedFields);
               }
            };
            ToClause toClause2 = new ToClause(iField, 1, resExpr2);
            
            doTo("loopLabel7", toClause2, new Block((Body) () -> 
            {
               cField.assign(entry(iField, cDisplayedFields));
               
               /* Skip fields that are not in the filter-target.
                       Note that for now, we do not resize the frame in run-time, so the frame 
                       will not be adjusted accordingly. */
               if (_and(hFilterTarget.isValid(), () -> not(matchesList(cDataColumns, cField))))
               {
                  iNumSkipped.assign(plus(iNumSkipped, 1));
                  next("loopLabel7");
               }
               
               dRow.assign(plus(dRow, divide(iFieldSpacingPxl, SessionUtils.getPixelsPerRow())));
               cLabel /* {fnarg <func-name> <arg> [handle] */ .assign(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "columnLabel", targetProcedure(), "I", cField));
               cFormat /* {fnarg <func-name> <arg> [handle] */ .assign(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "columnFormat", targetProcedure(), "I", cField));
               cToolTip /* {fnarg <func-name> <arg> [handle] */ .assign(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "columnToolTip", targetProcedure(), "I", cField));
               iHelpId /* {fnarg <func-name> <arg> [handle] */ .assign(ControlFlowOps.invokeDynamicFunctionInWithMode(integer.class, "columnHelpId", targetProcedure(), "I", cField));
               cFormat /* {fnarg <func-name> <arg> [handle] */ .assign(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "columnFormat", targetProcedure(), "I", cField));
               cDataType /* {fnarg <func-name> <arg> [handle] */ .assign(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "columnDataType", targetProcedure(), "I", cField));
               dWidth /* {fnarg <func-name> <arg> [handle] */ .assign(ControlFlowOps.invokeDynamicFunctionInWithMode(decimal.class, "columnWidth", targetProcedure(), "I", cField));
               cOperatorStyle /* {fnarg <func-name> <arg> [handle] */ .assign(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "columnOperatorStyle", targetProcedure(), "I", cField));
               /* createOperator will remove begins if not char and
                                                                  contains if not editor,
                                     We don't care about UseContains when Explicit */
               lWordIdx.assign(and(matchesList(cWordIdxFields, cField), () -> or(isEqual(cOperatorStyle, "EXPLICIT"), lUseContains)));
               hOperator.setUnknown();
               
               /* If we are gong to use a combo-box calculate width from data
                        unless we already have done so. */
               if (_and(and(isEqual(dOperatorWidth, 0), () -> isEqual(cOperatorStyle, "EXPLICIT")), () -> isEqual(cOperatorViewAs, "combo-box")))
               {
                  loopLabel8:
                  for (iOp.assign(1); _isLessThanOrEqual(iOp, numEntries(cOpValues)); iOp.assign(plus(iOp, 2)))
                  {
                     dOperatorWidth.assign(decimal.maximum(dOperatorWidth, FontTable.getTextWidthChars(entry(iOp, cOpValues), hContainerHandle.unwrapWidget().getFont())));
                  }
                  
                  dOperatorWidth.assign(plus(dOperatorWidth, 5));
               }
               
               hField.assign(ControlFlowOps.invokeDynamicFunctionInWithMode(handle.class, "createField", targetProcedure(), "IIIIIIIIIIII", hContainerHandle, cField, _isEqual(cOperatorStyle, "INLINE") ? new character( /*datatype */"CHARACTER") : cDataType, (lWordIdx).booleanValue() ? new character( /* viewas */"editor") : new character("fill-in"), _isEqual(cOperatorStyle, "INLINE") ? new character( /* format */"x(256)") : cFormat, not(begins(cUibmode, "DESIGN")), cToolTip, iHelpId, dRow, dFieldColumn, _and(lWordIdx, () -> isGreaterThan(iEditorLines, 1)) ? new decimal(iEditorLines) : dHeight, dWidth));
               hLabel.assign(ControlFlowOps.invokeDynamicFunctionInWithMode(handle.class, "createLabel", targetProcedure(), "II", hField, cLabel));
               
               /* Turns MODIFIED off on delete and backspace if the result is blank */
               if (_isEqual(cDataType, "CHARACTER"))
               {
                  registerTriggerPersistentIn(new EventList(new String[]
                  {
                     "delete-character",
                     "backspace"
                  }, hField), "removeSpace", targetProcedure());
               }
               
               if (_and(isNotEqual(cForeignFields, ""), () -> matchesList(cForeignFields, cField)))
               {
                  hField.unwrapWidget().setReadOnly(new logical(true));
               }
               else
               {
                  if (_or(isEqual(cOperatorStyle, "EXPLICIT"), () -> and(isEqual(cOperatorStyle, "RANGE"), () -> isNotEqual(hField.unwrapType().getResourceType(), "EDITOR"))))
                  {
                     hOperator.assign(ControlFlowOps.invokeDynamicFunctionInWithMode(handle.class, "createOperator", targetProcedure(), "IIIII", hField, _isEqual(cOperatorStyle, "RANGE") ? new character("") : cOperatorViewAs, cOpValues, plus(plus(dFieldColumn, decimal.maximum(dWidth, dDefaultCharWidth, dDefaultWidth)), divide(iFieldSeparatorPxl, SessionUtils.getPixelsPerRow())), dOperatorWidth));
                  }
                  
                  /* YES enabled-->*/ 
                  cOperatorHandles.assign(concat(cOperatorHandles, (_isNotEqual(cEnabledHandles, "") ? new character(",") : new character("")), (_isUnknown(hOperator) ? new character("") : valueOf(hOperator))));
                  cEnabledHandles.assign(concat(cEnabledHandles, (_isNotEqual(cEnabledHandles, "") ? new character(",") : new character("")), valueOf(hField)));
               }
               dRow.assign(plus(dRow, hField.unwrapSizeable().getHeightChars()));
               cFieldHandles.assign(concat(cFieldHandles, (_isNotEqual(cFieldHandles, "") ? new character(",") : new character("")), valueOf(hField)));
               cLabelHandles.assign(concat(cLabelHandles, (_isNotEqual(cLabelHandles, "") ? new character(",") : new character("")), valueOf(hLabel)));
            }));
            
            /* END DO iField */
            if (_isEqual(cUibmode, "DESIGN"))
            {
               hContainerHandle.unwrapSizeable().setHeightChars(decimal.maximum(dMinHeight, minus(plus(dHeight, dRow), 1)));
            }
            
            /* if cUIBmode begins 'design' */
            if (_not(lHidden))
            {
               ErrorManager.silentErrorEnable();
               hContainerHandle.unwrapWidget().setHidden(new logical(false));
               ErrorManager.silentErrorDisable();
            }
            
            if (_and(isEqual(cUibmode, "DESIGN"), () -> isEqual(ErrorManager.getErrorNumber(1), 6491)))
            {
               messageBox(new Object[]
               {
                  "The attempt to resize the frame to fit the filter fields failed.",
                  new SkipEntity(true),
                  "This will typically occur when the frame's container is to small.",
                  new SkipEntity(1),
                  "You must manually resize the container and the filter frame to make all filter fields visible."
               }, ALERT_INFO, BTN_OK, (String) null);
            }
            
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("LabelHandles"));
            ghProp.unwrapBufferField().changeValue(cLabelHandles);
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("OperatorHandles"));
            ghProp.unwrapBufferField().changeValue(cOperatorHandles);
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("FieldsEnabled"));
            ghProp.unwrapBufferField().changeValue(new logical(true));
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("EnabledHandles"));
            ghProp.unwrapBufferField().changeValue(cEnabledHandles);
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("DisplayedFields"));
            ghProp.unwrapBufferField().changeValue(cDisplayedFields);
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("EnabledFields"));
            ghProp.unwrapBufferField().changeValue(cDisplayedFields);
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("FieldHandles"));
            ghProp.unwrapBufferField().changeValue(cFieldHandles);
         }
         /* if cEnabledHandles = ""  */
         ControlFlowOps.invokeIn("ResetFields", targetProcedure());
         returnNormal();
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:  Backspace and delete sets the field to unmodifed which will keep <br>
    *           it out of the selection criteria.  <br>
    * Parameters:  <br>
    * Notes:    Start persistent in initializeObject.<br>
    *           Note that filter.i has the following trigger anywhere of frame: <br>
    *           on value-changed &gt; u10 &gt; unblankFillin trigger to reset format<br>
    *           when all fields have been blanked (emptied)
    */
   public void removeSpace()
   {
      internalProcedure(new Block((Body) () -> 
      {
         /* Make sure that backspace and delete always makes the field UNModifed
              if it is blank after apply. */
         if ((matchesList("backspace,delete-character", KeyReader.getLastFunction())).booleanValue())
         {
            apply(lastKey());
            
            if (_isEqual(self().unwrapWidget().getScreenValue(), ""))
            {
               self().unwrapWidget().setModified(new logical(false));
            }
            
            returnConsume();
         }
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose: Reset all the fields to the previosly applied filter values  <br>
    * Parameters: <br>
    * Notes:  The field values and operator values are retrieved form the actual <br>
    *         filter-targets query.
    */
   public void resetFields()
   {
      integer iField = TypeFactory.integer();
      integer iValue = TypeFactory.integer();
      handle hField = TypeFactory.handle();
      character cField = TypeFactory.character();
      character cFormat = TypeFactory.character();
      character cEnabledHandles = TypeFactory.character();
      character cDefaultOperator = TypeFactory.character();
      handle hFilterTarget = TypeFactory.handle();
      character cFilterTarget = TypeFactory.character();
      handle hOperator = TypeFactory.handle();
      character cOperator = TypeFactory.character();
      character cOperatorStyle = TypeFactory.character();
      character cValue = TypeFactory.character();
      handle hRangeField = TypeFactory.handle();
      character cOperatorHandles = TypeFactory.character();
      handle hdl = TypeFactory.handle();
      character cSelection = TypeFactory.character();
      character xcInlineOpList = TypeFactory.character(">=,>,<,<=");
      character xcRangeOperator = TypeFactory.character("<=");
      logical lUseBegins = TypeFactory.logical();
      logical lHi = TypeFactory.logical();
      logical lLo = TypeFactory.logical();
      
      internalProcedure(new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("EnabledHandles"));
         cEnabledHandles.assign(ghProp.unwrapBufferField().value());
         /* {get Prop var [handle]} */ 
         cFilterTarget.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getFilterTarget", targetProcedure()));
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("OperatorHandles"));
         cOperatorHandles.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("Operator"));
         cDefaultOperator.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("OperatorStyle"));
         cOperatorStyle.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("useBegins"));
         lUseBegins.assign(ghProp.unwrapBufferField().value());
         /* We obviously only support one filter target*/
         hFilterTarget.assign(handle.fromString(cFilterTarget));
         
         if (_not(hFilterTarget.isValid()))
         {
            ControlFlowOps.invokeIn("blankFields", targetProcedure());
            returnNormal();
         }
         
         IntegerExpression resExpr3 = new IntegerExpression()
         {
            public integer execute()
            {
               return numEntries(cEnabledHandles);
            }
         };
         ToClause toClause3 = new ToClause(iField, 1, resExpr3);
         
         doTo("loopLabel9", toClause3, new Block((Body) () -> 
         {
            hField.assign(handle.fromString(entry(iField, cEnabledHandles)));
            cFormat.assign(_isNotEqual(cOperatorStyle, "Inline") ? ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "ColumnFormat", targetProcedure(), "I", hField.unwrap().name()) : new character("x(256)"));
            hOperator.setUnknown();
            hRangeField.setUnknown();
            cField.assign(concat((_isEqual(numEntries(hField.unwrap().name(), "."), 1) ? new character("RowObject.") : new character("")), hField.unwrap().name()));
            cOperator.assign("");
            lHi.assign(false);
            lLo.assign(false);
            
            if ((HandleOps.canSet(hField, "FORMAT")).booleanValue())
            {
               hField.unwrapCommonField().setFormat(cFormat);
            }
            
            if (_isNotEqual(cOperatorHandles, ""))
            {
               hdl.assign(handle.fromString(entry(iField, cOperatorHandles)));
               
               if ((hdl.isValid()).booleanValue())
               {
                  if (_isEqual(hdl.unwrapType().getResourceType(), "FILL-IN"))
                  {
                     hRangeField.assign(hdl);
                     hRangeField.unwrapCommonField().setFormat(cFormat);
                  }
                  else
                  {
                     hOperator.assign(hdl);
                  }
               }
            }
            /* if cOperatorhandle <> '' */
            cSelection /* {fnarg <func-name> <arg> [handle] */.assign(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "columnQuerySelection", hFilterTarget, "I", cField));
            
            if (_isNotEqual(cSelection, ""))
            {
               loopLabel10:
               for (iValue.assign(1); _isLessThanOrEqual(iValue, numEntries(cSelection, chr(1))); iValue.assign(plus(iValue, 2)))
               {
                  cOperator.assign(entry(iValue, cSelection, chr(1)));
                  cValue.assign(entry(plus(iValue, 1), cSelection, chr(1)));
                  
                  if ((hOperator.isValid()).booleanValue())
                  {
                     ErrorManager.silentErrorEnable();
                     hField.unwrapWidget().setScreenValue(cValue);
                     ErrorManager.silentErrorDisable();
                     ErrorManager.silentErrorEnable();
                     hOperator.unwrapWidget().setScreenValue(cOperator);
                     ErrorManager.silentErrorDisable();
                  }
                  else
                  {
                     if (_and(hRangeField.isValid(), () -> isEqual(cOperator, xcRangeOperator)))
                     {
                        hRangeField.unwrapWidget().setScreenValue(cValue);
                        lHi.assign(true);
                     }
                     else
                     {
                        if (_and(hRangeField.isValid(), () -> isEqual(cOperator, ">=")))
                        {
                           hField.unwrapWidget().setScreenValue(cValue);
                           lLo.assign(true);
                        }
                        else
                        {
                           if (_and(isEqual(cOperator, "BEGINS"), not(lUseBegins)))
                           {
                              hField.unwrapWidget().setScreenValue(concat(cValue, "*"));
                           }
                           else
                           {
                              if (_and(isNotEqual(cOperator, cDefaultOperator), () -> matchesList(xcInlineOpList, cOperator)))
                              {
                                 hField.unwrapWidget().setScreenValue(concat(cOperator, " ", cValue));
                              }
                              else
                              {
                                 hField.unwrapWidget().setScreenValue(cValue);
                              }
                           }
                        }
                     }
                  }
               }
            }
            else
            {
               ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "blankField", targetProcedure(), "III", hField, hRangeField, hOperator);
            }
            
            /* Blank range fields in case only one of them was set */
            if (_and(isNotEqual(cSelection, ""), () -> hRangeField.isValid()))
            {
               if (_not(lLo))
               {
                  /* {fnarg <func-name> <arg> [handle] */ 
                  ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "BlankFillIn", targetProcedure(), "I", hField);
               }
               
               if (_not(lHi))
               {
                  /* {fnarg <func-name> <arg> [handle] */ 
                  ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "BlankFillIn", targetProcedure(), "I", hRangeField);
               }
            }
         }));
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    *   Purpose:  Reset the format of a blank fill-in of data-type LOGICAL<br>
    * Parameters: INPUT phField - the handle of a field      <br>
    *     Notes: The blankFields changes the format of fill-ins in order to make it <br>
    *            appear blank. <br>
    *            This procedure is defined  <br>
    *             ON ANY-PRINTABLE PERSISTENT trigger when the field is blanked
    */
   public void unBlankLogical(final handle _phField_4)
   {
      handle phField_4 = UndoableFactory.initInput(_phField_4);
      character cFormat = TypeFactory.character();
      character cFalse = TypeFactory.character();
      character cTrue = TypeFactory.character();
      character cKey = TypeFactory.character();
      
      internalProcedure(new Block((Body) () -> 
      {
         cFormat /* {fnarg <func-name> <arg> [handle] */ .assign(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "columnFormat", targetProcedure(), "I", phField_4.unwrap().name()));
         cFalse.assign(entry(2, cFormat, "/"));
         cTrue.assign(entry(1, cFormat, "/"));
         cKey.assign(keyFunction(lastKey()));
         
         if (_and(or(begins(cFalse, cKey), () -> begins(cTrue, cKey)), () -> isNotEqual(phField_4.unwrapCommonField().getFormat(), cFormat)))
         {
            phField_4.unwrapCommonField().setFormat(cFormat);
         }
         
         if (_isNotEqual(cKey, ""))
         {
            apply(phField_4, lastKey());
         }
         
         returnConsume();
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * ***********************  Function Implementations****************
    */
   public logical assignColumnFormat(final character _pcColumn_18, final character _pcFormat_2)
   {
      character pcColumn_18 = TypeFactory.initInput(_pcColumn_18);
      character pcFormat_2 = TypeFactory.initInput(_pcFormat_2);
      /*------------------------------------------------------------------------------
        Purpose: Override the default format of a column. 
        Parameters - pcColumn - Column in the Filter Target
                     pcFormat - Overridden format. 
          Notes: The value will be stored with all overidden formats in the 
                 FieldFormats property.
                  The private insertFieldProperty function will add the field to the 
                 correct postion in the internal list.
                 The columnFormat(pccolumn) function uses the private fieldProperty 
                 function to retrieve it.  
      ------------------------------------------------------------------------------*/
      character cFieldFormats = TypeFactory.character();
      
      return function("assignColumnFormat", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FieldFormats"));
         cFieldFormats.assign(ghProp.unwrapBufferField().value());
         cFieldFormats.assign(insertFieldProperty(cFieldFormats, pcColumn_18, pcFormat_2));
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FieldFormats"));
         ghProp.unwrapBufferField().changeValue(cFieldFormats);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical assignColumnHelpId(final character _pcColumn_18, final integer _piHelpId_1)
   {
      character pcColumn_18 = TypeFactory.initInput(_pcColumn_18);
      integer piHelpId_1 = TypeFactory.initInput(_piHelpId_1);
      /*------------------------------------------------------------------------------
        Purpose: Store HelpId of a column. 
        Parameters - pcColumn - Column in the Filter Target
                     pcFormat - HelpId. 
          Notes: The value will be stored with all defined HelpIds in the 
                 internal FieldHelpIds property.
                 The private insertFieldProperty function will add the field to the 
                 correct postion in the internal list.
                 The private fieldProperty function must be used to retrieve it.  
      ------------------------------------------------------------------------------*/
      character cFieldHelpIds = TypeFactory.character();
      
      return function("assignColumnHelpId", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FieldHelpIds"));
         cFieldHelpIds.assign(ghProp.unwrapBufferField().value());
         cFieldHelpIds.assign(insertFieldProperty(cFieldHelpIds, pcColumn_18, new character(valueOf(piHelpId_1))));
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FieldHelpIds"));
         ghProp.unwrapBufferField().changeValue(cFieldHelpIds);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical assignColumnLabel(final character _pcColumn_18, final character _pcLabel_2)
   {
      character pcColumn_18 = TypeFactory.initInput(_pcColumn_18);
      character pcLabel_2 = TypeFactory.initInput(_pcLabel_2);
      /*------------------------------------------------------------------------------
        Purpose: Override the default Label of a column. 
        Parameters - pcColumn - Column in the Filter Target
                     pcFormat - Overridden Label. 
          Notes: - The value will be stored with all overidden labels in the 
                   internal label Property.
                 - The private insertFieldProperty function will add the field to the 
                   correct postion in the internal list.
                 - The private fieldProperty function must be used to retrieve it.  
      ------------------------------------------------------------------------------*/
      character cFieldLabels = TypeFactory.character();
      
      return function("assignColumnLabel", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FieldLabels"));
         cFieldLabels.assign(ghProp.unwrapBufferField().value());
         cFieldLabels.assign(insertFieldProperty(cFieldLabels, pcColumn_18, pcLabel_2));
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FieldLabels"));
         ghProp.unwrapBufferField().changeValue(cFieldLabels);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical assignColumnOperatorStyle(final character _pcColumn_18, final character _pcStyle_1)
   {
      character pcColumn_18 = TypeFactory.initInput(_pcColumn_18);
      character pcStyle_1 = TypeFactory.initInput(_pcStyle_1);
      /*------------------------------------------------------------------------------
        Purpose: Override default operatorStyle of a column. 
        Parameters - pcColumn - Column in the Filter Target
                     pcFormat - Overridden OperatorStyle.  
          Notes: The value will be stored with all other fields where this is overridden 
                 in the internal FieldOperatorStyles property.
                 The private insertFieldProperty function will add the field to the 
                 correct postion in the internal list.
                 The private fieldProperty function must be used to retrieve it.  
      ------------------------------------------------------------------------------*/
      character cFieldOperatorStyles = TypeFactory.character();
      
      return function("assignColumnOperatorStyle", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FieldOperatorStyles"));
         cFieldOperatorStyles.assign(ghProp.unwrapBufferField().value());
         cFieldOperatorStyles.assign(insertFieldProperty(cFieldOperatorStyles, pcColumn_18, pcStyle_1));
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FieldOperatorStyles"));
         ghProp.unwrapBufferField().changeValue(cFieldOperatorStyles);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical assignColumnTooltip(final character _pcColumn_18, final character _pcTooltip_2)
   {
      character pcColumn_18 = TypeFactory.initInput(_pcColumn_18);
      character pcTooltip_2 = TypeFactory.initInput(_pcTooltip_2);
      /*------------------------------------------------------------------------------
        Purpose: Store Tooltip of a column. 
        Parameters - pcColumn - Column in the Filter Target
                     pcFormat - Tooltip. 
          Notes: The value will be stored with all defined ToolTips in the 
                 FieldToolTips property.
                 The private insertFieldProperty function will add the field to the 
                 correct postion in the internal list.
                 The private fieldProperty function must be used to retrieve it.  
      ------------------------------------------------------------------------------*/
      character cToolTips = TypeFactory.character();
      
      return function("assignColumnTooltip", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FieldToolTips"));
         cToolTips.assign(ghProp.unwrapBufferField().value());
         cToolTips.assign(insertFieldProperty(cToolTips, pcColumn_18, pcTooltip_2));
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FieldToolTips"));
         ghProp.unwrapBufferField().changeValue(cToolTips);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical assignColumnWidth(final character _pcColumn_18, final decimal _pdWidth_3)
   {
      character pcColumn_18 = TypeFactory.initInput(_pcColumn_18);
      decimal pdWidth_3 = TypeFactory.initInput(_pdWidth_3);
      /*------------------------------------------------------------------------------
        Purpose: Override default Width of a column.  
        Parameters - pcColumn - Column in the Filter Target
                     pcFormat - Overridden width. 
          Notes: The default width is stored in the DefaultWidth or DefaultCharWidth 
                 Properties. 
                 The value will be stored with all overidden widths in the 
                 internal FieldWidths property.
                 The private insertFieldProperty function will add the field to the 
                 correct postion in the internal list.
                 The private fieldProperty function must be used to retrieve it.            
      ------------------------------------------------------------------------------*/
      character cFieldWidths = TypeFactory.character();
      
      return function("assignColumnWidth", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FieldWidths"));
         cFieldWidths.assign(ghProp.unwrapBufferField().value());
         cFieldWidths.assign(insertFieldProperty(cFieldWidths, pcColumn_18, new character(valueOf(pdWidth_3))));
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FieldWidths"));
         ghProp.unwrapBufferField().changeValue(cFieldWidths);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical blankField(final handle _phField_4, final handle _phRangeField_1, final handle _phOperator_1)
   {
      handle phField_4 = TypeFactory.initInput(_phField_4);
      handle phRangeField_1 = TypeFactory.initInput(_phRangeField_1);
      handle phOperator_1 = TypeFactory.initInput(_phOperator_1);
      /*------------------------------------------------------------------------------
        Purpose: Blank a field including its operator or rangefield and 
                 sets MODIFIED eq false.
      Parameters:
          INPUT phField       -  handle of the actual filter field.
          INPUT phRangeField  -  handle of the optional range field.  
          INPUT phOperator    -  handle of the optional operator. 
                      
          Notes: A special blankFillIn function takes care of making all fill-ins            
                 really appear as blank.    
                 Used internally in blankFields and resetFields.
      ------------------------------------------------------------------------------*/
      character cDefaultOperator = TypeFactory.character();
      
      return function("blankField", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("Operator"));
         cDefaultOperator.assign(ghProp.unwrapBufferField().value());
         
         if (_isEqual(cDefaultOperator, ""))
         {
            cDefaultOperator.assign(">=");
         }
         
         if (_isEqual(phField_4.unwrapType().getResourceType(), "FILL-IN"))
         {
            /* {fnarg <func-name> <arg> [handle] */ 
            ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "blankFillIn", targetProcedure(), "I", phField_4);
         }
         else
         {
            phField_4.unwrapWidget().setScreenValue(new character(""));
         }
         
         if ((phRangeField_1.isValid()).booleanValue())
         {
            /* {fnarg <func-name> <arg> [handle] */ 
            ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "blankFillIn", targetProcedure(), "I", phRangeField_1);
            phRangeField_1.unwrapWidget().setModified(new logical(false));
         }
         
         if ((phOperator_1.isValid()).booleanValue())
         {
            ErrorManager.silentErrorEnable();
            phOperator_1.unwrapWidget().setScreenValue(_isEqual(phField_4.unwrapType().getResourceType(), "EDITOR") ? new character("CONTAINS") : _isEqual(phField_4.unwrapCommonField().getDataType(), "CHARACTER") ? new character("BEGINS") : cDefaultOperator);
            ErrorManager.silentErrorDisable();
            phOperator_1.unwrapWidget().setModified(new logical(false));
         }
         
         phField_4.unwrapWidget().setModified(new logical(false));
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical blankFillIn(final handle _phFillIn_1)
   {
      handle phFillIn_1 = TypeFactory.initInput(_phFillIn_1);
      
      return function("blankFillIn", logical.class, new Block((Body) () -> 
      {
         /*------------------------------------------------------------------------------
           Purpose: Makes all types of fill-in appear as blank by changing their format.  
         Parameters: INPUT phFillIn - The handle of an actual fill-in 
             Notes:  Called internally from blankFields - blankField() 
         ------------------------------------------------------------------------------*/
         phFillIn_1.unwrapWidget().setScreenValue(_isNotEqual(phFillIn_1.unwrapCommonField().getDataType(), "LOGICAL") ? new character("") : new character("no"));
         
         switch (StringHelper.safeTrimTrailing((phFillIn_1.unwrapCommonField().getDataType()).toStringMessage()))
         {
            case "character":
               {
                  phFillIn_1.unwrapCommonField().setFormat(new character("x"));
               }
               break;
            case "DECIMAL":
               {
                  phFillIn_1.unwrapCommonField().setFormat(new character("ZZZ"));
               }
               break;
            case "INTEGER":
               {
                  phFillIn_1.unwrapCommonField().setFormat(new character("Z"));
               }
               break;
            case "LOGICAL":
               {
                  phFillIn_1.unwrapCommonField().setFormat(concat(entry(1, phFillIn_1.unwrapCommonField().getFormat(), "/"), "/"));
                  registerTriggerPersistentIn(new EventList("ANY-PRINTABLE", phFillIn_1), "unBlankLogical", targetProcedure(), phFillIn_1);
               }
               break;
         }
         
         phFillIn_1.unwrapWidget().setModified(new logical(false));
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character columnDataType(final character _pcColumn_18)
   {
      character pcColumn_18 = TypeFactory.initInput(_pcColumn_18);
      /*------------------------------------------------------------------------------
        Purpose: Return the datatype of a column
      Parameters: INPUT pcColumn - The name of the column in the filter-target.    
          Notes: When the object is opened at design time there's no target available
                 so the field will always appear as character.
                  
      ------------------------------------------------------------------------------*/
      handle hFilterTarget = UndoableFactory.handle();
      character cDataType = UndoableFactory.character();
      character cOperatorStyle = TypeFactory.character();
      
      return function("columnDataType", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("OperatorStyle"));
         cOperatorStyle.assign(ghProp.unwrapBufferField().value());
         hFilterTarget /* {fnarg <func-name> <arg> [handle] */ .assign(ControlFlowOps.invokeDynamicFunctionInWithMode(handle.class, "columnFilterTarget", targetProcedure(), "I", pcColumn_18));
         
         if ((hFilterTarget.isValid()).booleanValue())
         {
            cDataType /* {fnarg <func-name> <arg> [handle] */.assign(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "columnDataType", hFilterTarget, "I", pcColumn_18));
         }
         else
         {
            cDataType.assign("CHARACTER");
         }
         
         returnNormal(cDataType);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public handle columnFilterTarget(final character _pcColumn_18)
   {
      character pcColumn_18 = TypeFactory.initInput(_pcColumn_18);
      /*------------------------------------------------------------------------------
        Purpose: Return the filter target of a specific column  
      Parameters: INPUT pcColumn - The name of the column in the filter-target.    
          Notes: This currently returns the same value for all fields, but may  be 
                 changed in order to support more than one filter-target.
                 This function should subsequently always be used to find the 
                 filter-target of a specific field.    
      -----------------------------------------------------------------------------*/
      character cFilterTarget = TypeFactory.character();
      
      return function("columnFilterTarget", handle.class, new Block((Body) () -> 
      {
         /* {get Prop var [handle]} */ 
         cFilterTarget.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getFilterTarget", targetProcedure()));
         
         if (_isNotEqual(cFilterTarget, ""))
         {
            returnNormal(handle.fromString(entry(1, cFilterTarget)));
         }
         
         returnNormal(new handle());
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character columnFormat(final character _pcColumn_18)
   {
      character pcColumn_18 = TypeFactory.initInput(_pcColumn_18);
      /*------------------------------------------------------------------------------
        Purpose: Return the Format of a column. 
      Parameters: INPUT pcColumn - The name of the column in the filter-target.    
          Notes: If the format is overridden it will be found in the CHR(1) separated
                 FieldFormats property.   
                 otherwise if there's a valid filter-target it will return the format 
                 returned from the filter-target.
                 The last resort is a hardcoded {&charformat} 
                 (Which is the case at open in design-time)      
      ------------------------------------------------------------------------------*/
      character cFieldFormats = TypeFactory.character();
      handle hFilterTarget = TypeFactory.handle();
      character cFormat = TypeFactory.character();
      character cOperatorStyle = TypeFactory.character();
      
      return function("columnFormat", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FieldFormats"));
         cFieldFormats.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("OperatorStyle"));
         cOperatorStyle.assign(ghProp.unwrapBufferField().value());
         cFormat.assign(fieldProperty(cFieldFormats, pcColumn_18));
         
         /* no override, find datasource format */
         if (_isEqual(cFormat, ""))
         {
            hFilterTarget /* {fnarg <func-name> <arg> [handle] */ .assign(ControlFlowOps.invokeDynamicFunctionInWithMode(handle.class, "columnFilterTarget", targetProcedure(), "I", pcColumn_18));
            
            if ((hFilterTarget.isValid()).booleanValue())
            {
               cFormat.assign(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "columnFormat", hFilterTarget, "I", pcColumn_18));
            }
            else
            {
               cFormat.assign("x(256)");
            }
         }
         returnNormal(cFormat);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public integer columnHelpId(final character _pcColumn_18)
   {
      character pcColumn_18 = TypeFactory.initInput(_pcColumn_18);
      /*------------------------------------------------------------------------------
        Purpose: Return the HelpId of a column. 
      Parameters: INPUT pcColumn - The name of the column in the filter-target.    
          Notes: The actual value is stored in the FieldHelpIds property.    
      ------------------------------------------------------------------------------*/
      character cFieldHelpIds = TypeFactory.character();
      
      return function("columnHelpId", integer.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FieldHelpIds"));
         cFieldHelpIds.assign(ghProp.unwrapBufferField().value());
         returnNormal(new integer(fieldProperty(cFieldHelpIds, pcColumn_18)));
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character columnLabel(final character _pcColumn_18)
   {
      character pcColumn_18 = TypeFactory.initInput(_pcColumn_18);
      /*------------------------------------------------------------------------------
        Purpose: Return the Label of a column. 
      Parameters: INPUT pcColumn - The name of the column in the filter-target.    
          Notes: If the label is overridden it will be found in the CHR(1) separated
                 FieldLabels property.   
                 otherwise if there is a valid filter-target it will return the label 
                 returned from the filter-target.
                 The last resort is blank. (Which is the case at open in design-time)       
      ------------------------------------------------------------------------------*/
      character cFieldLabels = TypeFactory.character();
      handle hFilterTarget = TypeFactory.handle();
      character cLabel = TypeFactory.character(null);
      
      return function("columnLabel", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FieldLabels"));
         cFieldLabels.assign(ghProp.unwrapBufferField().value());
         
         if (_isEqual(fieldLookup(pcColumn_18, cFieldLabels), 0))
         {
            cLabel.setUnknown();
         }
         else
         {
            cLabel.assign(fieldProperty(cFieldLabels, pcColumn_18));
         }
         
         /* no override, find datasource Label */
         if (_or(isEqual(cLabel, "?"), () -> isUnknown(cLabel)))
         {
            hFilterTarget /* {fnarg <func-name> <arg> [handle] */ .assign(ControlFlowOps.invokeDynamicFunctionInWithMode(handle.class, "columnFilterTarget", targetProcedure(), "I", pcColumn_18));
            
            if ((hFilterTarget.isValid()).booleanValue())
            {
               cLabel.assign(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "columnLabel", hFilterTarget, "I", pcColumn_18));
            }
            else
            {
               cLabel.assign("");
            }
         }
         returnNormal(cLabel);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical columnLabelDefault(final character _pcColumn_18)
   {
      character pcColumn_18 = TypeFactory.initInput(_pcColumn_18);
      /*------------------------------------------------------------------------------
        Purpose: Return true if the Label is overridden. 
      Parameters: INPUT pcColumn - The name of the column in the filter-target.    
          Notes: Only used in design-time because the columnLabel function returns
                 the overidden value.  
      ---------------------------------------------------------------------------*/
      character cFieldLabels = TypeFactory.character();
      character cLabel = TypeFactory.character();
      
      return function("columnLabelDefault", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FieldLabels"));
         cFieldLabels.assign(ghProp.unwrapBufferField().value());
         
         if (_isEqual(fieldLookup(pcColumn_18, cFieldLabels), 0))
         {
            returnNormal(true);
         }
         
         cLabel.assign(fieldProperty(cFieldLabels, pcColumn_18));
         returnNormal(or(isEqual(cLabel, "?"), () -> isUnknown(cLabel)));
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character columnOperatorStyle(final character _pcColumn_18)
   {
      character pcColumn_18 = TypeFactory.initInput(_pcColumn_18);
      /*------------------------------------------------------------------------------
        Purpose: Returns the operator style of a field, will return the OperatorStyle
                 property if not defined for the column.
      Parameters: INPUT pcColumn - The name of the column in the filter-target.                 
          Notes: Current restrictions:
                  - If default is "INLINE" it always returns the OperatorStyle prop.
                  - "EXPLICIT" is only allowed if default style is "RANGE".    
      ------------------------------------------------------------------------------*/
      character cFieldOperatorStyles = TypeFactory.character();
      character cOperatorStyle = TypeFactory.character();
      character cStyle = TypeFactory.character();
      
      return function("columnOperatorStyle", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FieldOperatorStyles"));
         cFieldOperatorStyles.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("OperatorStyle"));
         cOperatorStyle.assign(ghProp.unwrapBufferField().value());
         
         /* no override of inline */
         if (_isEqual(cOperatorStyle, "InLine"))
         {
            returnNormal(cOperatorStyle);
         }
         
         cStyle.assign(fieldProperty(cFieldOperatorStyles, pcColumn_18));
         
         if (_isNotEqual(cStyle, ""))
         {
            if (_isEqual(cOperatorStyle, "RANGE"))
            {
               if (_isEqual(cStyle, "EXPLICIT"))
               {
                  returnNormal(cStyle);
               }
            }
            else
            {
               returnNormal(cStyle);
            }
         }
         /* dwidth = 0 */
         returnNormal(cOperatorStyle);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical columnStyleDefault(final character _pcColumn_18)
   {
      character pcColumn_18 = TypeFactory.initInput(_pcColumn_18);
      /*------------------------------------------------------------------------------
        Purpose: Return true if the column's Style is overridden. 
      Parameters: INPUT pcColumn - The name of the column in the filter-target.    
          Notes: Only used in design-time because the columnOperatorStyle function 
                 returns the overidden value.  
      ---------------------------------------------------------------------------*/
      character cStyle = TypeFactory.character();
      character cFieldOperatorStyles = TypeFactory.character();
      
      return function("columnStyleDefault", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FieldOperatorStyles"));
         cFieldOperatorStyles.assign(ghProp.unwrapBufferField().value());
         cStyle.assign(fieldProperty(cFieldOperatorStyles, pcColumn_18));
         returnNormal(isEqual(cStyle, ""));
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character columnTooltip(final character _pcColumn_18)
   {
      character pcColumn_18 = TypeFactory.initInput(_pcColumn_18);
      /*------------------------------------------------------------------------------
        Purpose: Return the Toooltip of a column. 
      Parameters: INPUT pcColumn - The name of the column in the filter-target.    
          Notes: The actual value is stored in the FieldTooltips property.    
      ------------------------------------------------------------------------------*/
      character cFieldTooltips = TypeFactory.character();
      
      return function("columnTooltip", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FieldTooltips"));
         cFieldTooltips.assign(ghProp.unwrapBufferField().value());
         returnNormal(fieldProperty(cFieldTooltips, pcColumn_18));
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public decimal columnWidth(final character _pcColumn_18)
   {
      character pcColumn_18 = TypeFactory.initInput(_pcColumn_18);
      /*------------------------------------------------------------------------------
        Purpose: Returns the width of a field 
      Parameters: INPUT pcColumn - The name of the column in the filter-target.    
           Notes: Checks the internal FieldWidths property to see if the width is 
                  overridden. If not it returns the default withs from the 
                  DefaultCharWidth or DefaultWidth properties. 
                  The field widget is NOT used because this value needs to
                  be read before/while the object is created.  
      ------------------------------------------------------------------------------*/
      character cFieldWidths = TypeFactory.character();
      handle hFilterTarget = TypeFactory.handle();
      decimal dWidth = TypeFactory.decimal();
      
      return function("columnWidth", decimal.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FieldWidths"));
         cFieldWidths.assign(ghProp.unwrapBufferField().value());
         dWidth.assign(new decimal(fieldProperty(cFieldWidths, pcColumn_18)));
         
         if (_or(isEqual(dWidth, 0), () -> isUnknown(dWidth)))
         {
            hFilterTarget /* {fnarg <func-name> <arg> [handle] */ .assign(ControlFlowOps.invokeDynamicFunctionInWithMode(handle.class, "columnFilterTarget", targetProcedure(), "I", pcColumn_18));
            
            if (_isEqual( /* {fnarg <func-name> <arg> [handle] */ ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "columnDataType", targetProcedure(), "I", pcColumn_18), "CHARACTER"))
            {
               ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
               ghProp.assign(ghProp.unwrapBuffer().bufferField("DefaultCharWidth"));
               dWidth.assign(ghProp.unwrapBufferField().value());
            }
            else
            {
               ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
               ghProp.assign(ghProp.unwrapBuffer().bufferField("DefaultWidth"));
               dWidth.assign(ghProp.unwrapBufferField().value());
            }
            /* {get Prop var [handle]} */ 
            /* {get Prop var [handle]} */ 
         }
         /* dwidth = 0 */
         returnNormal(dWidth);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical columnWidthDefault(final character _pcColumn_18)
   {
      character pcColumn_18 = TypeFactory.initInput(_pcColumn_18);
      /*------------------------------------------------------------------------------
        Purpose: Return true if the Width is overridden. 
      Parameters: INPUT pcColumn - The name of the column in the filter-target.    
          Notes: Only used in design-time because the columnWidth function returns
                 the overidden value.  
      ---------------------------------------------------------------------------*/
      character cFieldWidths = TypeFactory.character();
      character cWidth = TypeFactory.character();
      
      return function("columnWidthDefault", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FieldWidths"));
         cFieldWidths.assign(ghProp.unwrapBufferField().value());
         cWidth.assign(fieldProperty(cFieldWidths, pcColumn_18));
         returnNormal(or(isEqual(cWidth, ""), () -> isEqual(cWidth, "?")));
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public handle createField(final handle _phFrame_1, final character _pcName_1, final character _pcDataType_1, final character _pcViewAs_1, final character _pcFormat_2, final logical _plEnable_1, final character _pcTooltip_2, final integer _piHelpid_1, final decimal _pdRow_1, final decimal _pdCol_2, final decimal _pdHeight_1, final decimal _pdWidth_3)
   {
      handle phFrame_1 = TypeFactory.initInput(_phFrame_1);
      character pcName_1 = TypeFactory.initInput(_pcName_1);
      character pcDataType_1 = TypeFactory.initInput(_pcDataType_1);
      character pcViewAs_1 = TypeFactory.initInput(_pcViewAs_1);
      character pcFormat_2 = TypeFactory.initInput(_pcFormat_2);
      logical plEnable_1 = TypeFactory.initInput(_plEnable_1);
      character pcTooltip_2 = TypeFactory.initInput(_pcTooltip_2);
      integer piHelpid_1 = TypeFactory.initInput(_piHelpid_1);
      decimal pdRow_1 = TypeFactory.initInput(_pdRow_1);
      decimal pdCol_2 = TypeFactory.initInput(_pdCol_2);
      decimal pdHeight_1 = TypeFactory.initInput(_pdHeight_1);
      decimal pdWidth_3 = TypeFactory.initInput(_pdWidth_3);
      handle hField = TypeFactory.handle();
      
      return function("createField", handle.class, new Block((Body) () -> 
      {
         DynamicWidgetFactory.create((pcViewAs_1).toStringMessage(), hField);
         hField.unwrapWidget().setFrameHandle(phFrame_1);
         hField.unwrapWidget().setRow(pdRow_1);
         hField.unwrapWidget().setColumn(pdCol_2);
         hField.unwrap().name(pcName_1);
         hField.unwrapSizeable().setWidthChars(pdWidth_3);
         hField.unwrapWidget().setTooltip(pcTooltip_2);
         hField.unwrapWidget().setContextHelpId(piHelpid_1);
         hField.unwrapWidget().setHidden(new logical(false));
         hField.unwrapSensitive().setSensitive(new logical(true));
         hField.unwrapWidget().setReadOnly(not(plEnable_1));
         hField.unwrapWidget().setTabStop(plEnable_1);
         
         /* Height >= 2 will be used to set INNER-LINES */
         if (_isEqual(pcViewAs_1, "editor"))
         {
            hField.unwrapInnerLines().setInnerLines(_isGreaterThanOrEqual(pdHeight_1, 2) ? new integer(pdHeight_1) : new integer(1));
            /* Scroll left and rigth in one line editor */
            hField.unwrapEditor().setWordWrap(isGreaterThanOrEqual(pdHeight_1, 2));
            hField.unwrapScrollbarVerticalElement().setScrollbarVertical(isGreaterThanOrEqual(pdHeight_1, 2));
            hField.unwrapSizeable().setHeightChars(_isLessThan(pdHeight_1, 2) ? pdHeight_1 : hField.unwrapSizeable().getHeightChars());
         }
         else
         {
            hField.unwrapSizeable().setHeightChars(pdHeight_1);
            hField.unwrapCommonField().setDataType(pcDataType_1);
            hField.unwrapCommonField().setFormat(pcFormat_2);
         }
         
         returnNormal(hField);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public handle createLabel(final handle _phField_4, final character _pcLabel_2)
   {
      handle phField_4 = TypeFactory.initInput(_phField_4);
      character pcLabel_2 = TypeFactory.initInput(_pcLabel_2);
      /*------------------------------------------------------------------------------
        Purpose: Create the label for the filter field
      Parameters: INPUT phField - The handle of the actual filter field
                  INPUT pcLabel - the text to use as label
          Notes: PRIVATE  
      ------------------------------------------------------------------------------*/
      handle hLabel = TypeFactory.handle();
      handle hFrame = TypeFactory.handle();
      character cUibmode = TypeFactory.character();
      integer iLabelLength = TypeFactory.integer();
      
      return function("createLabel", handle.class, new Block((Body) () -> 
      {
         hFrame.assign(phField_4.unwrapWidget().getFrameHandle());
         iLabelLength.assign(FontTable.getTextWidthPixels(concat(pcLabel_2, ": "), hFrame.unwrapWidget().getFont()));
         
         if (_isLessThan(minus(phField_4.unwrapXY().getX(), iLabelLength), 0))
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("UIBMode"));
            cUibmode.assign(ghProp.unwrapBufferField().value());
            
            if (_isEqual(cUibmode, "DESIGN"))
            {
               messageBox(new Object[]
               {
                  "The label for field",
                  phField_4.unwrap().name(),
                  "was truncated!",
                  new SkipEntity(true)
               }, ALERT_INFO, BTN_OK, (String) null);
            }
         }
         /* X lt 0 */
         DynamicWidgetFactory.createText(hLabel);
         hLabel.unwrapWidget().setFrameHandle(hFrame);
         hLabel.unwrapXY().setX(new integer(int64.maximum(new int64(0), minus(phField_4.unwrapXY().getX(), iLabelLength))));
         hLabel.unwrapWidget().setRow(phField_4.unwrapWidget().getRow());
         hLabel.unwrapWidget().setHidden(new logical(false));
         hLabel.unwrapSizeable().setWidthPixels(new integer(int64.minimum(phField_4.unwrapXY().getX(), iLabelLength)));
         hLabel.unwrapCommonField().setFormat(concat("x(", valueOf(plus(length(pcLabel_2), 1)), ")"));
         hLabel.unwrapWidget().setScreenValue(concat(pcLabel_2, ":"));
         hLabel.unwrapSizeable().setHeightPixels(SessionUtils.getPixelsPerRow());
         returnNormal(hLabel);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public handle createOperator(final handle _phField_4, final character _pcType_1, final character _pcValues_1, final decimal _pdCol_2, final decimal _pdWidth_3)
   {
      handle phField_4 = TypeFactory.initInput(_phField_4);
      character pcType_1 = TypeFactory.initInput(_pcType_1);
      character pcValues_1 = TypeFactory.initInput(_pcValues_1);
      decimal pdCol_2 = TypeFactory.initInput(_pdCol_2);
      decimal pdWidth_3 = TypeFactory.initInput(_pdWidth_3);
      /*------------------------------------------------------------------------------
        Purpose: Create the widget to use as operator or fill-in for the range option   
      Parameters: INPUT phField - The handle of the actual filter field
                  INPUT pcType  - Widget-type 
                                  - "radio-set", "combo-box" 
                                  - otherwise a fill-in will be created
                  INPUT pcValues - List of operator values for the combo-box 
                                   or radio-set        
                  INPUT pdCol    - decimal column position
                  INPUT pdWidth  - decimal width                                      
          Notes:  
      ------------------------------------------------------------------------------*/
      handle hOperator = TypeFactory.handle();
      
      return function("createOperator", handle.class, new Block((Body) () -> 
      {
         if (_isEqual(pcType_1, "Radio-set"))
         {
            DynamicWidgetFactory.createRadioSet(hOperator);
            hOperator.unwrapWidget().setFrameHandle(phField_4.unwrapWidget().getFrameHandle());
            hOperator.unwrapWidget().setRow(phField_4.unwrapWidget().getRow());
            hOperator.unwrapWidget().setColumn(plus(phField_4.unwrapWidget().getColumn(), 21));
            hOperator.unwrapRadioSet().setRadioButtons(pcValues_1);
            hOperator.unwrapOrientation().setHorizontal(new logical(true));
            hOperator.unwrapSizeable().setHeightChars(phField_4.unwrapSizeable().getHeightChars());
            hOperator.unwrapWidget().setHidden(new logical(false));
            hOperator.unwrapSensitive().setSensitive(not(phField_4.unwrapWidget().isReadOnly()));
         }
         else
         {
            if (_isEqual(pcType_1, "Combo-box"))
            {
               DynamicWidgetFactory.createComboBox(hOperator);
               hOperator.unwrapWidget().setFrameHandle(phField_4.unwrapWidget().getFrameHandle());
               hOperator.unwrapWidget().setRow(phField_4.unwrapWidget().getRow());
               hOperator.unwrapWidget().setColumn(pdCol_2);
               hOperator.unwrapSizeable().setWidthChars(pdWidth_3);
               hOperator.unwrapCommonListWidget().setListItemPairs(pcValues_1);
               hOperator.unwrapWidget().setHidden(new logical(false));
               hOperator.unwrapSensitive().setSensitive(not(phField_4.unwrapWidget().isReadOnly()));
            }
            else
            {
               DynamicWidgetFactory.createFillIn(hOperator);
               hOperator.unwrapWidget().setFrameHandle(phField_4.unwrapWidget().getFrameHandle());
               hOperator.unwrapWidget().setRow(phField_4.unwrapWidget().getRow());
               hOperator.unwrapWidget().setColumn(pdCol_2);
               hOperator.unwrap().name(phField_4.unwrap().name());
               hOperator.unwrapSizeable().setHeightChars(phField_4.unwrapSizeable().getHeightChars());
               hOperator.unwrapSizeable().setWidthChars(phField_4.unwrapSizeable().getWidthChars());
               hOperator.unwrapCommonField().setDataType(phField_4.unwrapCommonField().getDataType());
               hOperator.unwrapCommonField().setFormat(phField_4.unwrapCommonField().getFormat());
               hOperator.unwrapWidget().setHidden(new logical(false));
               hOperator.unwrapSensitive().setSensitive(new logical(true));
               hOperator.unwrapWidget().setReadOnly(phField_4.unwrapWidget().isReadOnly());
               hOperator.unwrapWidget().setTabStop(phField_4.unwrapWidget().isTabStop());
            }
         }
         
         /* Remove BEGINS from list if not character and CONTAINS if not editor */
         if (_isNotEqual(hOperator.unwrapType().getResourceType(), "Fill-in"))
         {
            if (_isNotEqual(phField_4.unwrapCommonField().getDataType(), "CHARACTER"))
            {
               hOperator.unwrapCommonListWidget().delete("BEGINS");
            }
            
            if (_isNotEqual(phField_4.unwrapType().getResourceType(), "EDITOR"))
            {
               hOperator.unwrapCommonListWidget().delete("CONTAINS");
            }
         }
         /* hoperator <> fill-in */
         returnNormal(hOperator);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character dataValue(final character _pcColumn_18, final character _pcValue_2)
   {
      character pcColumn_18 = TypeFactory.initInput(_pcColumn_18);
      character pcValue_2 = TypeFactory.initInput(_pcValue_2);
      /*------------------------------------------------------------------------------
         Purpose: validate and return the true datavalue of a column   
      Parameters: INPUT pcColumn - The column's name in the data-source.
                  INPUT pcValue  - formatted screen-Value.
           Notes:  
      ------------------------------------------------------------------------------*/
      character ch = TypeFactory.character();
      date da = TypeFactory.date();
      decimal de = TypeFactory.decimal();
      integer i = TypeFactory.integer();
      logical lo = TypeFactory.logical();
      character cValue = TypeFactory.character();
      character cDataType = TypeFactory.character();
      character cFormat = TypeFactory.character();
      
      return function("dataValue", character.class, new Block((Pre) () -> 
      {
         xFrame = GenericFrame.createFrame(FilterX.class, "x");
      }, 
      (Body) () -> 
      {
         xFrame.openScope();
         cDataType.assign(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "columnDataType", targetProcedure(), "I", pcColumn_18));
         cFormat.assign(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "columnFormat", targetProcedure(), "I", pcColumn_18));
         
         switch (StringHelper.safeTrimTrailing((cDataType).toStringMessage()).toUpperCase())
         {
            case "CHARACTER":
               {
                  ErrorManager.silentErrorEnable();
                  xFrame.widgetCh().setFormat(cFormat);
                  xFrame.widgetCh().setScreenValue(pcValue_2);
                  cValue.assign(xFrame.getCh());
                  ErrorManager.silentErrorDisable();
               }
               break;
            case "DATE":
               {
                  ErrorManager.silentErrorEnable();
                  xFrame.widgetDa().setFormat(cFormat);
                  xFrame.widgetDa().setScreenValue(pcValue_2);
                  cValue.assign(valueOf(xFrame.getDa()));
                  ErrorManager.silentErrorDisable();
               }
               break;
            case "DECIMAL":
               {
                  ErrorManager.silentErrorEnable();
                  xFrame.widgetDe().setFormat(cFormat);
                  xFrame.widgetDe().setScreenValue(pcValue_2);
                  cValue.assign(valueOf(xFrame.getDe()));
                  ErrorManager.silentErrorDisable();
               }
               break;
            case "INTEGER":
               {
                  ErrorManager.silentErrorEnable();
                  xFrame.widgeti().setFormat(cFormat);
                  xFrame.widgeti().setScreenValue(pcValue_2);
                  cValue.assign(valueOf(xFrame.getI()));
                  ErrorManager.silentErrorDisable();
               }
               break;
            case "LOGICAL":
               {
                  ErrorManager.silentErrorEnable();
                  xFrame.widgetLo().setFormat(cFormat);
                  xFrame.widgetLo().setScreenValue(pcValue_2);
                  cValue.assign(valueOf(xFrame.isLo()));
                  ErrorManager.silentErrorDisable();
               }
               break;
         }
         
         /* cDatatype  */
         /* Only return the first error-message */
         if (_isNotEqual(ErrorManager.getErrorText(1), ""))
         {
            ControlFlowOps.invokeInWithMode("AddMessage", targetProcedure(), "III", ErrorManager.getErrorText(1), pcColumn_18, new unknown());
         }
         
         returnNormal(cValue);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical deleteObjects()
   {
      /*------------------------------------------------------------------------------
        Purpose: Delete all dynamic widgets.  
          Notes: Called at design-time every time the Instance Properties are changed  
      ------------------------------------------------------------------------------*/
      character cFieldHandles = TypeFactory.character();
      character cLabelHandles = TypeFactory.character();
      character cOperatorHandles = TypeFactory.character();
      integer i = TypeFactory.integer();
      handle hdl = TypeFactory.handle();
      
      return function("deleteObjects", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FieldHandles"));
         cFieldHandles.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("LabelHandles"));
         cLabelHandles.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("OperatorHandles"));
         cOperatorHandles.assign(ghProp.unwrapBufferField().value());
         
         loopLabel11:
         for (i.assign(1); _isLessThanOrEqual(i, numEntries(cFieldHandles)); i.increment())
         {
            hdl.assign(handle.fromString(entry(i, cFieldHandles)));
            
            if (_and(hdl.isValid(), () -> hdl.unwrapDynamicResource().dynamic()))
            {
               HandleOps.delete(hdl);
            }
            
            hdl.assign(handle.fromString(entry(i, cLabelHandles)));
            
            if (_and(hdl.isValid(), () -> hdl.unwrapDynamicResource().dynamic()))
            {
               HandleOps.delete(hdl);
            }
            
            if (_and(isNotEqual(cOperatorHandles, ""), () -> isGreaterThanOrEqual(numEntries(cOperatorHandles), i)))
            {
               hdl.assign(handle.fromString(entry(i, cOperatorHandles)));
               
               if ((hdl.isValid()).booleanValue())
               {
                  HandleOps.delete(hdl);
               }
            }
         }
         
         cFieldHandles.assign("");
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FieldHandles"));
         ghProp.unwrapBufferField().changeValue(cFieldHandles);
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("LabelHandles"));
         ghProp.unwrapBufferField().changeValue(cFieldHandles);
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("OperatorHandles"));
         ghProp.unwrapBufferField().changeValue(cFieldHandles);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   private integer fieldLookup(final character _pcField_3, final character _pcList_3)
   {
      character pcField_3 = TypeFactory.initInput(_pcField_3);
      character pcList_3 = TypeFactory.initInput(_pcList_3);
      /*------------------------------------------------------------------------------
        Purpose: Find the LOOKUP position of the corresponding value to the field 
                 in a chr(1) list of fields and values.   
      Parameters:  INPUT pcFields - fieldname 
                   INPUT pcList   - CHR(1) separated list with fieldnames and values. 
          Notes: PRIVATE used for all defined and/or overridden column properties
      ------------------------------------------------------------------------------*/
      integer iLookUp = TypeFactory.integer();
      
      return function("fieldLookup", integer.class, new Block((Body) () -> 
      {
         loopLabel12:
         while (true)
         {
            iLookUp.assign(lookup(pcField_3, pcList_3, chr(1)));
            
            if (_or(isEqual(iLookUp, 0), () -> isUnknown(iLookUp)))
            {
               break loopLabel12;
            }
            
            if (_isEqual(modulo(iLookUp, 2), 1))
            {
               break loopLabel12;
            }
            
            /* Remove this entry because its data and not a field */
            pcList_3.replaceEntry(iLookUp, chr(1), "");
         }
         
         returnNormal(iLookUp);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   private character fieldProperty(final character _pcList_3, final character _pcField_3)
   {
      character pcList_3 = TypeFactory.initInput(_pcList_3);
      character pcField_3 = TypeFactory.initInput(_pcField_3);
      /*------------------------------------------------------------------------------
        Purpose: Find the actual value of the corresponding value to the field in a 
                 chr(1) list of fields and values.   
      Parameters:  INPUT pcFields - fieldname 
                   INPUT pcList   - CHR(1) separated list with fieldnames and values. 
          Notes: PRIVATE used for all defined and/or overridden column properties
      ------------------------------------------------------------------------------*/
      integer iLookUp = TypeFactory.integer();
      
      return function("fieldProperty", character.class, new Block((Body) () -> 
      {
         iLookUp.assign(fieldLookup(pcField_3, pcList_3));
         
         if (_isGreaterThan(iLookUp, 0))
         {
            returnNormal(entry(plus(iLookUp, 1), pcList_3, chr(1)));
         }
         
         returnNormal("");
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getDataObject()
   {
      /*------------------------------------------------------------------------------
        Purpose: Returns the filter used at design time  
          Notes:  
      ------------------------------------------------------------------------------*/
      character cDataObject = TypeFactory.character();
      
      return function("getDataObject", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DataObject"));
         cDataObject.assign(ghProp.unwrapBufferField().value());
         returnNormal(cDataObject);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public decimal getDefaultCharWidth()
   {
      /*------------------------------------------------------------------------------
        Purpose: Defalt width for character fields. 
          Notes:  
      ------------------------------------------------------------------------------*/
      decimal dDefaultCharWidth = TypeFactory.decimal();
      
      return function("getDefaultCharWidth", decimal.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DefaultCharWidth"));
         dDefaultCharWidth.assign(ghProp.unwrapBufferField().value());
         returnNormal(dDefaultCharWidth);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public decimal getDefaultEditorLines()
   {
      /*------------------------------------------------------------------------------
        Purpose: Defalt Inner-lines for editors. 
          Notes: Word indexed field will automatically use editor. 
      ------------------------------------------------------------------------------*/
      decimal iLines = TypeFactory.decimal();
      
      return function("getDefaultEditorLines", decimal.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DefaultEditorLines"));
         iLines.assign(ghProp.unwrapBufferField().value());
         returnNormal(iLines);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getDefaultLogical()
   {
      /*------------------------------------------------------------------------------
        Purpose: Not used 
          Notes:  
      ------------------------------------------------------------------------------*/
      character cDefaultLogical = TypeFactory.character();
      
      return function("getDefaultLogical", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DefaultLogical"));
         cDefaultLogical.assign(ghProp.unwrapBufferField().value());
         returnNormal(cDefaultLogical);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public decimal getDefaultWidth()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Defualt width for fields that are non-char 
          Notes:  
      ------------------------------------------------------------------------------*/
      decimal dDefaultWidth = TypeFactory.decimal();
      
      return function("getDefaultWidth", decimal.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DefaultWidth"));
         dDefaultWidth.assign(ghProp.unwrapBufferField().value());
         returnNormal(dDefaultWidth);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getDisplayedFields()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Complete list of fields in the filter.
          Notes:  
      ------------------------------------------------------------------------------*/
      character cDisplayedFields = TypeFactory.character();
      
      return function("getDisplayedFields", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DisplayedFields"));
         cDisplayedFields.assign(ghProp.unwrapBufferField().value());
         returnNormal(cDisplayedFields);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getEnabledFields()
   {
      /*------------------------------------------------------------------------------
        Purpose: 
          Notes:  
      ------------------------------------------------------------------------------*/
      character cEnabledFields = TypeFactory.character();
      
      return function("getEnabledFields", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("EnabledFields"));
         cEnabledFields.assign(ghProp.unwrapBufferField().value());
         returnNormal(cEnabledFields);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public decimal getFieldColumn()
   {
      /*------------------------------------------------------------------------------
        Purpose:    Column of left-most field.   
          Notes:    
      ------------------------------------------------------------------------------*/
      decimal dFieldColumn = TypeFactory.decimal();
      
      return function("getFieldColumn", decimal.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FieldColumn"));
         dFieldColumn.assign(ghProp.unwrapBufferField().value());
         returnNormal(dFieldColumn);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getFieldFormats()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Internal override of formats for fields
          Notes:  
      ------------------------------------------------------------------------------*/
      character cFieldFormats = TypeFactory.character();
      
      return function("getFieldFormats", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FieldFormats"));
         cFieldFormats.assign(ghProp.unwrapBufferField().value());
         returnNormal(cFieldFormats);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getFieldHelpIds()
   {
      /*------------------------------------------------------------------------------
        Purpose:    Internal list of properties for the fields 
          Notes:    CHR(1) separated list of names and values
      ------------------------------------------------------------------------------*/
      character cFieldHelpIds = TypeFactory.character();
      
      return function("getFieldHelpIds", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FieldHelpIds"));
         cFieldHelpIds.assign(ghProp.unwrapBufferField().value());
         returnNormal(cFieldHelpIds);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getFieldLabels()
   {
      /*------------------------------------------------------------------------------
        Purpose:    Internal list of properties for the fields 
          Notes:    CHR(1) separated list of names and values
      ------------------------------------------------------------------------------*/
      character cFieldLabels = TypeFactory.character();
      
      return function("getFieldLabels", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FieldLabels"));
         cFieldLabels.assign(ghProp.unwrapBufferField().value());
         returnNormal(cFieldLabels);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getFieldOperatorStyles()
   {
      /*------------------------------------------------------------------------------
        Purpose:    Internal list of properties for the fields 
          Notes:    CHR(1) separated list of names and values
      ------------------------------------------------------------------------------*/
      character cFieldOperatorStyles = TypeFactory.character();
      
      return function("getFieldOperatorStyles", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FieldOperatorStyles"));
         cFieldOperatorStyles.assign(ghProp.unwrapBufferField().value());
         returnNormal(cFieldOperatorStyles);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getFieldToolTips()
   {
      /*------------------------------------------------------------------------------
        Purpose:    Internal list of properties for the fields 
          Notes:    CHR(1) separated list of names and values
      ------------------------------------------------------------------------------*/
      character cFieldToolTips = TypeFactory.character();
      
      return function("getFieldToolTips", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FieldToolTips"));
         cFieldToolTips.assign(ghProp.unwrapBufferField().value());
         returnNormal(cFieldToolTips);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getFieldWidths()
   {
      /*------------------------------------------------------------------------------
        Purpose:    Internal list of properties for the fields 
          Notes:    CHR(1) separated list of names and values
      ------------------------------------------------------------------------------*/
      character cFieldWidths = TypeFactory.character();
      
      return function("getFieldWidths", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FieldWidths"));
         cFieldWidths.assign(ghProp.unwrapBufferField().value());
         returnNormal(cFieldWidths);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getFilterTarget()
   {
      /*------------------------------------------------------------------------------
        Purpose: The linked filter object. Currently supports only one.
          Notes: USE columnFilterTarget for a column  
      ------------------------------------------------------------------------------*/
      character cFilterTarget = TypeFactory.character();
      
      return function("getFilterTarget", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FilterTarget"));
         cFilterTarget.assign(ghProp.unwrapBufferField().value());
         returnNormal(cFilterTarget);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getFilterTargetEvents()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Returns a comma-separated list of the events this object wants 
                  to subscribe to in its FilterTarget
         Params:  <none>
      ------------------------------------------------------------------------------*/
      character cEvents = TypeFactory.character();
      
      return function("getFilterTargetEvents", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FilterTargetEvents"));
         cEvents.assign(ghProp.unwrapBufferField().value());
         returnNormal(cEvents);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getOperator()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Returns the default operator when OperatorStyle eq "Implicit"
          Notes:  
      ------------------------------------------------------------------------------*/
      character cOperator = TypeFactory.character();
      
      return function("getOperator", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("Operator"));
         cOperator.assign(ghProp.unwrapBufferField().value());
         returnNormal(cOperator);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getOperatorLongValues()
   {
      /*------------------------------------------------------------------------------
        Purpose: A list of operators and long text    
          Notes:  
      ------------------------------------------------------------------------------*/
      character cOperatorLongValues = TypeFactory.character();
      
      return function("getOperatorLongValues", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("OperatorLongValues"));
         cOperatorLongValues.assign(ghProp.unwrapBufferField().value());
         returnNormal(cOperatorLongValues);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getOperatorStyle()
   {
      /*------------------------------------------------------------------------------
        Purpose: Returns operatorStyle  
          Notes: Four valid values.
                 - "Explicit"  - specify operator in a separate widget
                 - "Implicit"  - Use the Operator and UseBegins property 
                 - "Range"     - Use two fill-ins and specify GE and LE values
                 - "Inline"    - Type the operator in the field 
                                  (Defualt Equals or BEGINS if UseBegins eq true)     
      ------------------------------------------------------------------------------*/
      character cOperatorStyle = TypeFactory.character();
      
      return function("getOperatorStyle", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("OperatorStyle"));
         cOperatorStyle.assign(ghProp.unwrapBufferField().value());
         returnNormal(cOperatorStyle);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getOperatorViewAs()
   {
      /*------------------------------------------------------------------------------
        Purpose: Returns the view-as type used to define the operator when 
                 OperatorStyle equals "Explicit".  
          Notes:  
      ------------------------------------------------------------------------------*/
      character cOperatorViewAs = TypeFactory.character();
      
      return function("getOperatorViewAs", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("OperatorViewAs"));
         cOperatorViewAs.assign(ghProp.unwrapBufferField().value());
         returnNormal(cOperatorViewAs);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical getUseBegins()
   {
      /*------------------------------------------------------------------------------
        Purpose: Returns true when BEGINS is supposed to be used as operator 
                 for character values.  
          Notes: Is NOT used for OperatorStyle "RANGE" or "EXPLICIT"  
      ------------------------------------------------------------------------------*/
      logical lUseBegins = TypeFactory.logical();
      
      return function("getUseBegins", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("UseBegins"));
         lUseBegins.assign(ghProp.unwrapBufferField().value());
         returnNormal(lUseBegins);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical getUseContains()
   {
      /*------------------------------------------------------------------------------
        Purpose: Returns true when Contains is supposed to be used as operator 
                 for character values.  
          Notes: Is NOT used for OperatorStyle "EXPLICIT"
                  (It will always be in the list of available operators) 
      ------------------------------------------------------------------------------*/
      logical lUseContains = TypeFactory.logical();
      
      return function("getUseContains", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("UseContains"));
         lUseContains.assign(ghProp.unwrapBufferField().value());
         returnNormal(lUseContains);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getViewAsFields()
   {
      /*------------------------------------------------------------------------------
        Purpose: not in use  
          Notes:   
      ------------------------------------------------------------------------------*/
      character cViewAsFields = TypeFactory.character();
      
      return function("getViewAsFields", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ViewAsFields"));
         cViewAsFields.assign(ghProp.unwrapBufferField().value());
         returnNormal(cViewAsFields);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getVisualBlank()
   {
      /*------------------------------------------------------------------------------
        Purpose: Return the value that are used to visualize that we actually are 
                 searching for BLANK values.   
          Notes: Toggles on and off with space-bar and off with back-space.   
      ------------------------------------------------------------------------------*/
      character cVisual = TypeFactory.character();
      
      return function("getVisualBlank", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("VisualBlank"));
         cVisual.assign(ghProp.unwrapBufferField().value());
         returnNormal(cVisual);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   private character insertFieldProperty(final character _pcList_3, final character _pcField_3, final character _pcValue_2)
   {
      character pcList_3 = TypeFactory.initInput(_pcList_3);
      character pcField_3 = TypeFactory.initInput(_pcField_3);
      character pcValue_2 = TypeFactory.initInput(_pcValue_2);
      /*------------------------------------------------------------------------------
        Purpose: Insert or add a  fields property to the CHR(1) separated internal 
                 property that holdes fields and values.   
      Parameters: INPUT pcList   - CHR(1) separated list with fieldnames and values. 
                  INPUT pcFields - fieldname
                  INPUT pcValue  - Value of the field property 
         Notes:  PRIVATE
      ------------------------------------------------------------------------------*/
      integer iLookup = TypeFactory.integer();
      
      return function("insertFieldProperty", character.class, new Block((Body) () -> 
      {
         if (_isUnknown(pcList_3))
         {
            pcList_3.assign("");
         }
         
         iLookup.assign(fieldLookup(pcField_3, pcList_3));
         
         if (_isGreaterThan(iLookup, 0))
         {
            pcList_3.replaceEntry(plus(iLookup, 1), chr(1), _isUnknown(pcValue_2) ? new character("?") : pcValue_2);
         }
         else
         {
            if (_and(and(isNotEqual(pcValue_2, ""), () -> isNotEqual(pcValue_2, "?")), () -> notUnknown(pcValue_2)))
            {
               pcList_3.assign(concat(pcList_3, (_isEqual(pcList_3, "") ? new character("") : chr(1)), pcField_3, chr(1), pcValue_2));
            }
         }
         returnNormal(pcList_3);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setDataObject(final character _pcDataObject_1)
   {
      character pcDataObject_1 = TypeFactory.initInput(_pcDataObject_1);
      
      return function("setDataObject", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DataObject"));
         ghProp.unwrapBufferField().changeValue(pcDataObject_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setDefaultCharWidth(final decimal _pdDefaultCharWidth_1)
   {
      decimal pdDefaultCharWidth_1 = TypeFactory.initInput(_pdDefaultCharWidth_1);
      
      return function("setDefaultCharWidth", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DefaultCharWidth"));
         ghProp.unwrapBufferField().changeValue(pdDefaultCharWidth_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setDefaultEditorLines(final integer _piLines_1)
   {
      integer piLines_1 = TypeFactory.initInput(_piLines_1);
      
      return function("setDefaultEditorLines", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DefaultEditorLines"));
         ghProp.unwrapBufferField().changeValue(piLines_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setDefaultLogical(final character _pcDefaultLogical_1)
   {
      character pcDefaultLogical_1 = TypeFactory.initInput(_pcDefaultLogical_1);
      
      return function("setDefaultLogical", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DefaultLogical"));
         ghProp.unwrapBufferField().changeValue(pcDefaultLogical_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setDefaultWidth(final decimal _pdDefaultWidth_1)
   {
      decimal pdDefaultWidth_1 = TypeFactory.initInput(_pdDefaultWidth_1);
      
      return function("setDefaultWidth", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DefaultWidth"));
         ghProp.unwrapBufferField().changeValue(pdDefaultWidth_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setDisplayedFields(final character _pcDisplayedFields_1)
   {
      character pcDisplayedFields_1 = TypeFactory.initInput(_pcDisplayedFields_1);
      
      return function("setDisplayedFields", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DisplayedFields"));
         ghProp.unwrapBufferField().changeValue(pcDisplayedFields_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setEnabledFields(final character _pcEnabledFields_1)
   {
      character pcEnabledFields_1 = TypeFactory.initInput(_pcEnabledFields_1);
      
      return function("setEnabledFields", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("EnabledFields"));
         ghProp.unwrapBufferField().changeValue(pcEnabledFields_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setFieldColumn(final decimal _pcFieldColumn_1)
   {
      decimal pcFieldColumn_1 = TypeFactory.initInput(_pcFieldColumn_1);
      
      return function("setFieldColumn", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FieldColumn"));
         ghProp.unwrapBufferField().changeValue(pcFieldColumn_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setFieldFormats(final character _pcFieldFormats_1)
   {
      character pcFieldFormats_1 = TypeFactory.initInput(_pcFieldFormats_1);
      
      return function("setFieldFormats", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FieldFormats"));
         ghProp.unwrapBufferField().changeValue(pcFieldFormats_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setFieldHelpIds(final character _pcFieldHelpIds_1)
   {
      character pcFieldHelpIds_1 = TypeFactory.initInput(_pcFieldHelpIds_1);
      
      return function("setFieldHelpIds", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FieldHelpIds"));
         ghProp.unwrapBufferField().changeValue(pcFieldHelpIds_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setFieldLabels(final character _pcFieldLabels_1)
   {
      character pcFieldLabels_1 = TypeFactory.initInput(_pcFieldLabels_1);
      
      return function("setFieldLabels", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FieldLabels"));
         ghProp.unwrapBufferField().changeValue(pcFieldLabels_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setFieldOperatorStyles(final character _pcFieldOperatorStyles_1)
   {
      character pcFieldOperatorStyles_1 = TypeFactory.initInput(_pcFieldOperatorStyles_1);
      
      return function("setFieldOperatorStyles", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FieldOperatorStyles"));
         ghProp.unwrapBufferField().changeValue(pcFieldOperatorStyles_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setFieldToolTips(final character _pcFieldToolTips_1)
   {
      character pcFieldToolTips_1 = TypeFactory.initInput(_pcFieldToolTips_1);
      
      return function("setFieldToolTips", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FieldToolTips"));
         ghProp.unwrapBufferField().changeValue(pcFieldToolTips_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setFieldWidths(final character _pcFieldWidths_1)
   {
      character pcFieldWidths_1 = TypeFactory.initInput(_pcFieldWidths_1);
      
      return function("setFieldWidths", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FieldWidths"));
         ghProp.unwrapBufferField().changeValue(pcFieldWidths_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setFilterTarget(final character _pcTarget_1)
   {
      character pcTarget_1 = TypeFactory.initInput(_pcTarget_1);
      
      return function("setFilterTarget", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FilterTarget"));
         ghProp.unwrapBufferField().changeValue(pcTarget_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setFilterTargetEvents(final character _pcEvents_1)
   {
      character pcEvents_1 = TypeFactory.initInput(_pcEvents_1);
      
      return function("setFilterTargetEvents", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FilterTargetEvents"));
         ghProp.unwrapBufferField().changeValue(pcEvents_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setOperator(final character _pcOperator_1)
   {
      character pcOperator_1 = TypeFactory.initInput(_pcOperator_1);
      
      return function("setOperator", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("Operator"));
         ghProp.unwrapBufferField().changeValue(pcOperator_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setOperatorStyle(final character _pcOperatorStyle_1)
   {
      character pcOperatorStyle_1 = TypeFactory.initInput(_pcOperatorStyle_1);
      
      return function("setOperatorStyle", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("OperatorStyle"));
         ghProp.unwrapBufferField().changeValue(pcOperatorStyle_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setOperatorViewAs(final character _pcOperatorViewAs_1)
   {
      character pcOperatorViewAs_1 = TypeFactory.initInput(_pcOperatorViewAs_1);
      
      return function("setOperatorViewAs", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("OperatorViewAs"));
         ghProp.unwrapBufferField().changeValue(pcOperatorViewAs_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setUseBegins(final character _plUseBegins_1)
   {
      character plUseBegins_1 = TypeFactory.initInput(_plUseBegins_1);
      
      return function("setUseBegins", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("UseBegins"));
         ghProp.unwrapBufferField().changeValue(plUseBegins_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setUseContains(final character _plUseContains_1)
   {
      character plUseContains_1 = TypeFactory.initInput(_plUseContains_1);
      
      return function("setUseContains", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("UseContains"));
         ghProp.unwrapBufferField().changeValue(plUseContains_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setViewAsFields(final character _pcViewAsFields_1)
   {
      character pcViewAsFields_1 = TypeFactory.initInput(_pcViewAsFields_1);
      
      return function("setViewAsFields", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ViewAsFields"));
         ghProp.unwrapBufferField().changeValue(pcViewAsFields_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setVisualBlank(final character _pcVisual_1)
   {
      character pcVisual_1 = TypeFactory.initInput(_pcVisual_1);
      
      return function("setVisualBlank", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("VisualBlank"));
         ghProp.unwrapBufferField().changeValue(pcVisual_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character showDataMessages()
   {
      /*------------------------------------------------------------------------------
        Purpose:   Returns the name of the field (if any) from the first
                   error message, to allow the caller to use it to position the 
                   cursor.
         Params:   <none>.   
         Notes:    Invokes fetchMessages() to retrieve all Data-related messages
                   (normally database update-related error messages), and
                   displays them in a alert-box of type error.
                   This function expects to receive back a single string 
                   from fetchMessages with one or more messages delimited by CHR(3),
                   and within each message the message text, Fieldname (or blank) +
                   a Tablename (or blank), delimited by CHR(4) if present.
      ------------------------------------------------------------------------------*/
      character cMessages = TypeFactory.character();
      integer iMsg = TypeFactory.integer();
      integer iMsgCnt = TypeFactory.integer();
      character cMessage = TypeFactory.character();
      character cFirstField = TypeFactory.character();
      character cField = TypeFactory.character();
      character cTable = TypeFactory.character();
      character cText = TypeFactory.character("");
      
      return function("showDataMessages", character.class, new Block((Body) () -> 
      {
         cMessages.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "fetchMessages", targetProcedure()));
         iMsgCnt.assign(numEntries(cMessages, chr(3)));
         
         ToClause toClause4 = new ToClause(iMsg, 1, iMsgCnt);
         
         doTo("loopLabel13", toClause4, new Block((Body) () -> 
         {
            cMessage.assign(entry(iMsg, cMessages, chr(3)));
            cField.assign(_isGreaterThan(numEntries(cMessage, chr(4)), 1) ? entry(2, cMessage, chr(4)) : new character(""));
            cTable.assign(_isGreaterThan(numEntries(cMessage, chr(4)), 2) ? entry(3, cMessage, chr(4)) : new character(""));
            cText.assign(concat(cText, (_isNotEqual(cField, "") ? ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "messageNumber", targetProcedure(), "I", 10) : new character("")), cField, "   ", (_isNotEqual(cTable, "") ? ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "messageNumber", targetProcedure(), "I", 11) : new character("")), cTable, (_or(isNotEqual(cField, ""), () -> isNotEqual(cTable, "")) ? new character("\n") : new character("")), "  ", entry(1, cMessage, chr(4)), "\n"));
            
            /* Return the field name from the first error message so the caller can
                   use it to position the cursor. */
            if (_isEqual(iMsg, 1))
            {
               cFirstField.assign(cField);
            }
         }));
         
         /* END DO iMsg */
         if (_isNotEqual(cText, ""))
         {
            messageBox(cText, ALERT_ERROR, BTN_OK, (String) null);
         }
         
         returnNormal(cFirstField);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical unBlankFillin(final handle _phField_4)
   {
      handle phField_4 = TypeFactory.initInput(_phField_4);
      /*------------------------------------------------------------------------------
        Purpose: Reset the format of a blank fill-in
      Parameters: INPUT phField - the handle of a field      
          Notes: The blankFields changes the format of fill-ins in order to make it 
                 appear blank. 
                 This function is called as soon as the user starts editing the field 
                 VALUE-CHANGED -> "U10" triggers in filter.i makes this happen.         
      ------------------------------------------------------------------------------*/
      character cFormat = TypeFactory.character();
      character cDataType = TypeFactory.character();
      integer iPeriod = TypeFactory.integer();
      character cDecPoint = TypeFactory.character();
      
      return function("unBlankFillin", logical.class, new Block((Body) () -> 
      {
         cDataType /* {fnarg <func-name> <arg> [handle] */ .assign(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "columnDataType", targetProcedure(), "I", phField_4.unwrap().name()));
         /* The objects data-type may be different than the field's real data-type 
              (OperatorStyle = "inline") */
         cFormat.assign(_and(isNotEqual(cDataType, phField_4.unwrapCommonField().getDataType()), () -> isEqual(phField_4.unwrapCommonField().getDataType(), "CHARACTER")) ? new character("x(256)") : ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "columnFormat", targetProcedure(), "I", phField_4.unwrap().name()));
         
         if (_isNotEqual(phField_4.unwrapCommonField().getFormat(), cFormat))
         {
            phField_4.unwrapCommonField().setFormat(cFormat);
            
            if (_isNotEqual(phField_4.unwrapCommonField().getDataType(), "DECIMAL"))
            {
               if (_isNotEqual(phField_4.unwrapWidget().getScreenValue(), ""))
               {
                  apply(phField_4, "END");
               }
               else
               {
                  /* apply 'END' will not work if screen-value = '' 
                                So we apply lastkey which is in this case is spacebar */
                  apply(lastKey());
               }
            }
            else
            {
               cDecPoint.assign(_isEqual(NumberType.getNumericFormat(), "AMERICAN") ? new character(".") : new character(","));
               iPeriod.assign(indexOf(phField_4.unwrapWidget().getScreenValue(), cDecPoint));
               
               if (_isGreaterThan(iPeriod, 0))
               {
                  phField_4.unwrapWidget().setCursorOffset(iPeriod);
               }
               else
               {
                  apply(phField_4, "END");
               }
            }
         }
         returnNormal(true);
      }));
   }
}
