package com.goldencode.hotel.dmo._meta.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import com.goldencode.p2j.persist.*;
import com.goldencode.p2j.util.*;
import com.goldencode.hotel.dmo._meta.*;

/**
 * Data Model Object corresponding with table <code>_meta.meta_db</code> (previously
 * <code>_meta._db</code>).
 */
@LegacyTable(name = "_db", table = "meta_db")
@LegacyIndexes(
{
   @LegacyIndex(name = "_database", converted = "meta_db_database", primary = true, unique = true, components = 
   {
      @LegacyIndexComponent(fieldName = "_db-name")
   })
})
public class MetaDbImpl
implements Serializable,
           Persistable,
           MetaDb
{
   /** Surrogate primary key */
   private Long id = null;

   /** Db-name */
   @LegacyField(fieldId = 1, name = "_db-name", format = "x(12)", initial = "?", label = "Db-name", mandatory = true)
   private final character dbName;

   /** Db-addr */
   @LegacyField(fieldId = 2, name = "_db-addr", format = "x(60)", initial = "?", label = "Db-addr")
   private final character dbAddr;

   /** Db-type */
   @LegacyField(fieldId = 3, name = "_db-type", format = "x(12)", initial = "?", label = "Db-type", mandatory = true)
   private final character dbType;

   /** Db-comm */
   @LegacyField(fieldId = 4, name = "_db-comm", format = "x(12)", initial = "?", label = "Db-comm", mandatory = true)
   private final character dbComm;

   /** Db-revision */
   @LegacyField(fieldId = 5, name = "_db-revision", format = "->>>>>>>>>9", initial = "?", label = "Db-revision")
   private final integer dbRevision;

   /** Db-slave */
   @LegacyField(fieldId = 8, name = "_db-slave", format = "yes/no", initial = "?", label = "Db-slave", mandatory = true)
   private final logical dbSlave;

   /** Db-local */
   @LegacyField(fieldId = 9, name = "_db-local", format = "yes/no", initial = "false", label = "Db-local", mandatory = true)
   private final logical dbLocal;

   /** Data member */
   @LegacyField(fieldId = 13, name = "_db-xl-name", format = "x(20)", initial = "?")
   private final character dbXlName;

   /** Data member */
   @LegacyField(fieldId = 15, name = "_db-coll-name", format = "x(20)", initial = "?")
   private final character dbCollName;

   /** User-Misc */
   @LegacyField(fieldId = 16, name = "_user-misc", format = "x(10)", initial = "?", label = "User-Misc")
   private final character userMisc;

   /**
    * Db guid; This is the current globally unique identifier (GUID) for the database to support
    * the aggregation of data across multiple databases. This field is used in all audit related
    * tables to identify which database the audit information is for. All other fields are then
    * only unique within a specific database. Note that the database GUID may change, e.g. through
    * back / restore and copy operations to new databases so careful consideration is required
    * when using it as the foreign key to related tables as joins could be broken / need to be
    * fixed, depending on the nature of the related data. The other way to join from the _db table
    * is via the recid, which never changes but is also not globally unique - so which to use will
    * depend on the related data.
    */
   @LegacyField(fieldId = 17, name = "_db-guid", format = "X(28)", label = "Db guid", caseSensitive = true)
   private final character dbGuid;

   /** List of composite properties; 2 elements of type {@link MetaDbImpl.Composite2 Composite2} */
   private List<Composite2> composite2 = new ArrayList<>(2);

   /** List of composite properties; 8 elements of type {@link MetaDbImpl.Composite8 Composite8} */
   private List<Composite8> composite8 = new ArrayList<>(8);

   /** List of composite properties; 10 elements of type {@link MetaDbImpl.Composite10 Composite10} */
   private List<Composite10> composite10 = new ArrayList<>(10);

   /**
    * Default constructor.
    */
   public MetaDbImpl()
   {
      dbName = new character("?");
      dbAddr = new character("?");
      dbType = new character("?");
      dbComm = new character("?");
      dbRevision = new integer();
      dbSlave = new logical();
      dbLocal = new logical(false);
      dbXlName = new character("?");
      dbCollName = new character("?");
      userMisc = new character("?");
      dbGuid = new character("", true);
      
      // Populate Composite2 list.
      for (int i = 0; i < 2; i++)
      {
         composite2.add(new Composite2());
      }
      
      // Populate Composite8 list.
      for (int i = 0; i < 8; i++)
      {
         composite8.add(new Composite8());
      }
      
      // Populate Composite10 list.
      for (int i = 0; i < 10; i++)
      {
         composite10.add(new Composite10());
      }
   }

   /**
    * Make a deep copy of the current DMO instance and return it.
    * 
    * @return  A new instance of this class whose data is identical to that of this instance.
    */
   public Undoable deepCopy()
   {
      Undoable copy = new MetaDbImpl();
      copy.assign(this);
      return copy;
   }

   /**
    * Modify the contained data of the instance to copy the instance data from the given, backup
    * instance.
    * 
    * @param   old
    *          The backup instance from which to copy data.
    */
   public void assign(Undoable old)
   {
      MetaDbImpl from = (MetaDbImpl) old;
      setId(from.id);
      setDbName(from.dbName);
      setDbAddr(from.dbAddr);
      setDbType(from.dbType);
      setDbComm(from.dbComm);
      setDbRevision(from.dbRevision);
      setDbSlave(from.dbSlave);
      setDbLocal(from.dbLocal);
      setDbXlName(from.dbXlName);
      setDbCollName(from.dbCollName);
      setUserMisc(from.userMisc);
      setDbGuid(from.dbGuid);
      
      // Assign Composite2 properties.
      for (int i = 0; i < 2; i++)
      {
         setDbXlate(i, from.composite2.get(i).dbXlate);
      }
      
      // Assign Composite8 properties.
      for (int i = 0; i < 8; i++)
      {
         setDbMisc1(i, from.composite8.get(i).dbMisc1);
         setDbMisc2(i, from.composite8.get(i).dbMisc2);
         setDbRes1(i, from.composite8.get(i).dbRes1);
         setDbRes2(i, from.composite8.get(i).dbRes2);
      }
      
      // Assign Composite10 properties.
      for (int i = 0; i < 10; i++)
      {
         setDbCollate(i, from.composite10.get(i).dbCollate);
      }
   }

   /**
    * Getter: Surrogate primary key
    * 
    * @return  Primary key
    */
   public Long getId()
   {
      return id;
   }

   /**
    * Setter: Surrogate primary key
    * 
    * @param   id
    *          Primary key
    */
   public void setId(Long id)
   {
      this.id = id;
   }

   /**
    * Getter: Db-name
    * 
    * @return  Db-name
    */
   public character getDbName()
   {
      return (new character(dbName));
   }

   /**
    * Setter: Db-name
    * 
    * @param   dbName
    *          Db-name
    */
   public void setDbName(Text dbName)
   {
      this.dbName.assign(dbName, true);
   }

   /**
    * Getter: Db-addr
    * 
    * @return  Db-addr
    */
   public character getDbAddr()
   {
      return (new character(dbAddr));
   }

   /**
    * Setter: Db-addr
    * 
    * @param   dbAddr
    *          Db-addr
    */
   public void setDbAddr(Text dbAddr)
   {
      this.dbAddr.assign(dbAddr, true);
   }

   /**
    * Getter: Db-type
    * 
    * @return  Db-type
    */
   public character getDbType()
   {
      return (new character(dbType));
   }

   /**
    * Setter: Db-type
    * 
    * @param   dbType
    *          Db-type
    */
   public void setDbType(Text dbType)
   {
      this.dbType.assign(dbType, true);
   }

   /**
    * Getter: Db-comm
    * 
    * @return  Db-comm
    */
   public character getDbComm()
   {
      return (new character(dbComm));
   }

   /**
    * Setter: Db-comm
    * 
    * @param   dbComm
    *          Db-comm
    */
   public void setDbComm(Text dbComm)
   {
      this.dbComm.assign(dbComm, true);
   }

   /**
    * Getter: Db-revision
    * 
    * @return  Db-revision
    */
   public integer getDbRevision()
   {
      return (new integer(dbRevision));
   }

   /**
    * Setter: Db-revision
    * 
    * @param   dbRevision
    *          Db-revision
    */
   public void setDbRevision(NumberType dbRevision)
   {
      this.dbRevision.assign(dbRevision, true);
   }

   /**
    * Indexed Getter: dbMisc1
    * 
    * @param   index
    *          Index of element (where <code>0 &lt;= <em>index</em> &lt; 8</code>).
    * 
    * @return  dbMisc1
    */
   public integer getDbMisc1(int index)
   {
      return composite8.get(index).getDbMisc1();
   }

   /**
    * Indexed Setter: dbMisc1
    * 
    * @param   index
    *          Index of element (where <code>0 &lt;= <em>index</em> &lt; 8</code>).
    * @param   element
    *          dbMisc1
    */
   public void setDbMisc1(int index, NumberType element)
   {
      composite8.get(index).setDbMisc1(element);
   }

   /**
    * Indexed Getter: dbMisc2
    * 
    * @param   index
    *          Index of element (where <code>0 &lt;= <em>index</em> &lt; 8</code>).
    * 
    * @return  dbMisc2
    */
   public character getDbMisc2(int index)
   {
      return composite8.get(index).getDbMisc2();
   }

   /**
    * Indexed Setter: dbMisc2
    * 
    * @param   index
    *          Index of element (where <code>0 &lt;= <em>index</em> &lt; 8</code>).
    * @param   element
    *          dbMisc2
    */
   public void setDbMisc2(int index, Text element)
   {
      composite8.get(index).setDbMisc2(element);
   }

   /**
    * Getter: Db-slave
    * 
    * @return  Db-slave
    */
   public logical isDbSlave()
   {
      return (new logical(dbSlave));
   }

   /**
    * Setter: Db-slave
    * 
    * @param   dbSlave
    *          Db-slave
    */
   public void setDbSlave(logical dbSlave)
   {
      this.dbSlave.assign(dbSlave, true);
   }

   /**
    * Getter: Db-local
    * 
    * @return  Db-local
    */
   public logical isDbLocal()
   {
      return (new logical(dbLocal));
   }

   /**
    * Setter: Db-local
    * 
    * @param   dbLocal
    *          Db-local
    */
   public void setDbLocal(logical dbLocal)
   {
      this.dbLocal.assign(dbLocal, true);
   }

   /**
    * Indexed Getter: dbRes1
    * 
    * @param   index
    *          Index of element (where <code>0 &lt;= <em>index</em> &lt; 8</code>).
    * 
    * @return  dbRes1
    */
   public integer getDbRes1(int index)
   {
      return composite8.get(index).getDbRes1();
   }

   /**
    * Indexed Setter: dbRes1
    * 
    * @param   index
    *          Index of element (where <code>0 &lt;= <em>index</em> &lt; 8</code>).
    * @param   element
    *          dbRes1
    */
   public void setDbRes1(int index, NumberType element)
   {
      composite8.get(index).setDbRes1(element);
   }

   /**
    * Indexed Getter: dbRes2
    * 
    * @param   index
    *          Index of element (where <code>0 &lt;= <em>index</em> &lt; 8</code>).
    * 
    * @return  dbRes2
    */
   public character getDbRes2(int index)
   {
      return composite8.get(index).getDbRes2();
   }

   /**
    * Indexed Setter: dbRes2
    * 
    * @param   index
    *          Index of element (where <code>0 &lt;= <em>index</em> &lt; 8</code>).
    * @param   element
    *          dbRes2
    */
   public void setDbRes2(int index, Text element)
   {
      composite8.get(index).setDbRes2(element);
   }

   /**
    * Indexed Getter: dbXlate
    * 
    * @param   index
    *          Index of element (where <code>0 &lt;= <em>index</em> &lt; 2</code>).
    * 
    * @return  dbXlate
    */
   public raw getDbXlate(int index)
   {
      return composite2.get(index).getDbXlate();
   }

   /**
    * Indexed Setter: dbXlate
    * 
    * @param   index
    *          Index of element (where <code>0 &lt;= <em>index</em> &lt; 2</code>).
    * @param   element
    *          dbXlate
    */
   public void setDbXlate(int index, BinaryData element)
   {
      composite2.get(index).setDbXlate(element);
   }

   /**
    * Getter: dbXlName
    * 
    * @return  dbXlName
    */
   public character getDbXlName()
   {
      return (new character(dbXlName));
   }

   /**
    * Setter: dbXlName
    * 
    * @param   dbXlName
    *          dbXlName
    */
   public void setDbXlName(Text dbXlName)
   {
      this.dbXlName.assign(dbXlName, true);
   }

   /**
    * Indexed Getter: dbCollate
    * 
    * @param   index
    *          Index of element (where <code>0 &lt;= <em>index</em> &lt; 10</code>).
    * 
    * @return  dbCollate
    */
   public raw getDbCollate(int index)
   {
      return composite10.get(index).getDbCollate();
   }

   /**
    * Indexed Setter: dbCollate
    * 
    * @param   index
    *          Index of element (where <code>0 &lt;= <em>index</em> &lt; 10</code>).
    * @param   element
    *          dbCollate
    */
   public void setDbCollate(int index, BinaryData element)
   {
      composite10.get(index).setDbCollate(element);
   }

   /**
    * Getter: dbCollName
    * 
    * @return  dbCollName
    */
   public character getDbCollName()
   {
      return (new character(dbCollName));
   }

   /**
    * Setter: dbCollName
    * 
    * @param   dbCollName
    *          dbCollName
    */
   public void setDbCollName(Text dbCollName)
   {
      this.dbCollName.assign(dbCollName, true);
   }

   /**
    * Getter: User-Misc
    * 
    * @return  User-Misc
    */
   public character getUserMisc()
   {
      return (new character(userMisc));
   }

   /**
    * Setter: User-Misc
    * 
    * @param   userMisc
    *          User-Misc
    */
   public void setUserMisc(Text userMisc)
   {
      this.userMisc.assign(userMisc, true);
   }

   /**
    * Getter: This is the current globally unique identifier (GUID) for the database to support
    * the aggregation of data across multiple databases. This field is used in all audit related
    * tables to identify which database the audit information is for. All other fields are then
    * only unique within a specific database. Note that the database GUID may change, e.g. through
    * back / restore and copy operations to new databases so careful consideration is required
    * when using it as the foreign key to related tables as joins could be broken / need to be
    * fixed, depending on the nature of the related data. The other way to join from the _db table
    * is via the recid, which never changes but is also not globally unique - so which to use will
    * depend on the related data.
    * 
    * @return  Db guid
    */
   public character getDbGuid()
   {
      return (new character(dbGuid));
   }

   /**
    * Setter: This is the current globally unique identifier (GUID) for the database to support
    * the aggregation of data across multiple databases. This field is used in all audit related
    * tables to identify which database the audit information is for. All other fields are then
    * only unique within a specific database. Note that the database GUID may change, e.g. through
    * back / restore and copy operations to new databases so careful consideration is required
    * when using it as the foreign key to related tables as joins could be broken / need to be
    * fixed, depending on the nature of the related data. The other way to join from the _db table
    * is via the recid, which never changes but is also not globally unique - so which to use will
    * depend on the related data.
    * 
    * @param   dbGuid
    *          Db guid
    */
   public void setDbGuid(Text dbGuid)
   {
      this.dbGuid.assign(dbGuid, true);
   }

   /**
    * Inner class which composites properties of like extent. All properties migrated from the
    * legacy table <code>_meta._db</code> of extent 2 are data members of this class.
    */
   static class Composite2
   implements Serializable
   {
      /** Data member */
      @LegacyField(fieldId = 12, name = "_db-xlate", format = "x(70)", initial = "?", extent = 2)
      private final raw dbXlate;

      /**
       * Default constructor.
       */
      public Composite2()
      {
         dbXlate = new raw();
      }

      /**
       * Getter: dbXlate
       * 
       * @return  dbXlate
       */
      public raw getDbXlate()
      {
         return (new raw(dbXlate));
      }

      /**
       * Setter: dbXlate
       * 
       * @param   dbXlate
       *          dbXlate
       */
      public void setDbXlate(BinaryData dbXlate)
      {
         this.dbXlate.assign(dbXlate, true);
      }
   }

   /**
    * Inner class which composites properties of like extent. All properties migrated from the
    * legacy table <code>_meta._db</code> of extent 8 are data members of this class.
    */
   static class Composite8
   implements Serializable
   {
      /** Data member */
      @LegacyField(fieldId = 6, name = "_db-misc1", format = ">>>>9", initial = "?", extent = 8)
      private final integer dbMisc1;

      /** Data member */
      @LegacyField(fieldId = 7, name = "_db-misc2", format = "x(8)", initial = "?", extent = 8)
      private final character dbMisc2;

      /** Data member */
      @LegacyField(fieldId = 10, name = "_db-res1", format = ">>>9", initial = "?", extent = 8)
      private final integer dbRes1;

      /** Data member */
      @LegacyField(fieldId = 11, name = "_db-res2", format = "x(12)", initial = "?", extent = 8)
      private final character dbRes2;

      /**
       * Default constructor.
       */
      public Composite8()
      {
         dbMisc1 = new integer();
         dbMisc2 = new character("?");
         dbRes1 = new integer();
         dbRes2 = new character("?");
      }

      /**
       * Getter: dbMisc1
       * 
       * @return  dbMisc1
       */
      public integer getDbMisc1()
      {
         return (new integer(dbMisc1));
      }

      /**
       * Setter: dbMisc1
       * 
       * @param   dbMisc1
       *          dbMisc1
       */
      public void setDbMisc1(NumberType dbMisc1)
      {
         this.dbMisc1.assign(dbMisc1, true);
      }

      /**
       * Getter: dbMisc2
       * 
       * @return  dbMisc2
       */
      public character getDbMisc2()
      {
         return (new character(dbMisc2));
      }

      /**
       * Setter: dbMisc2
       * 
       * @param   dbMisc2
       *          dbMisc2
       */
      public void setDbMisc2(Text dbMisc2)
      {
         this.dbMisc2.assign(dbMisc2, true);
      }

      /**
       * Getter: dbRes1
       * 
       * @return  dbRes1
       */
      public integer getDbRes1()
      {
         return (new integer(dbRes1));
      }

      /**
       * Setter: dbRes1
       * 
       * @param   dbRes1
       *          dbRes1
       */
      public void setDbRes1(NumberType dbRes1)
      {
         this.dbRes1.assign(dbRes1, true);
      }

      /**
       * Getter: dbRes2
       * 
       * @return  dbRes2
       */
      public character getDbRes2()
      {
         return (new character(dbRes2));
      }

      /**
       * Setter: dbRes2
       * 
       * @param   dbRes2
       *          dbRes2
       */
      public void setDbRes2(Text dbRes2)
      {
         this.dbRes2.assign(dbRes2, true);
      }
   }

   /**
    * Inner class which composites properties of like extent. All properties migrated from the
    * legacy table <code>_meta._db</code> of extent 10 are data members of this class.
    */
   static class Composite10
   implements Serializable
   {
      /** Data member */
      @LegacyField(fieldId = 14, name = "_db-collate", format = "x(70)", initial = "?", extent = 10)
      private final raw dbCollate;

      /**
       * Default constructor.
       */
      public Composite10()
      {
         dbCollate = new raw();
      }

      /**
       * Getter: dbCollate
       * 
       * @return  dbCollate
       */
      public raw getDbCollate()
      {
         return (new raw(dbCollate));
      }

      /**
       * Setter: dbCollate
       * 
       * @param   dbCollate
       *          dbCollate
       */
      public void setDbCollate(BinaryData dbCollate)
      {
         this.dbCollate.assign(dbCollate, true);
      }
   }
}
