package com.goldencode.hotel.dmo.hotel.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import com.goldencode.p2j.persist.*;
import com.goldencode.p2j.util.*;
import com.goldencode.hotel.dmo.hotel.*;

/**
 * Data Model Object corresponding with table <code>hotel.guest_id_type</code> (previously
 * <code>hotel.guest-id-type</code>).
 * <p>
 * Possible types of guests' identity documents
 */
@LegacyTable(name = "guest-id-type", table = "guest_id_type")
@LegacyIndexes(
{
   @LegacyIndex(name = "id-type", converted = "guest_id_type_id_type", primary = true, unique = true, components = 
   {
      @LegacyIndexComponent(fieldName = "id-type")
   })
})
public class GuestIdTypeImpl
implements Serializable,
           Persistable,
           GuestIdType
{
   /** Surrogate primary key */
   private Long id = null;

   /** ID type */
   @LegacyField(fieldId = 1, name = "id-type", format = ">9", initial = "0", columnLabel = "ID type", label = "ID type")
   private final integer idType;

   /** Description */
   @LegacyField(fieldId = 2, name = "description", format = "x(20)", columnLabel = "Description", label = "Description")
   private final character description;

   /**
    * Default constructor.
    */
   public GuestIdTypeImpl()
   {
      idType = new integer(0);
      description = new character("");
   }

   /**
    * Make a deep copy of the current DMO instance and return it.
    * 
    * @return  A new instance of this class whose data is identical to that of this instance.
    */
   public Undoable deepCopy()
   {
      Undoable copy = new GuestIdTypeImpl();
      copy.assign(this);
      return copy;
   }

   /**
    * Modify the contained data of the instance to copy the instance data from the given, backup
    * instance.
    * 
    * @param   old
    *          The backup instance from which to copy data.
    */
   public void assign(Undoable old)
   {
      GuestIdTypeImpl from = (GuestIdTypeImpl) old;
      setId(from.id);
      setIdType(from.idType);
      setDescription(from.description);
   }

   /**
    * Getter: Surrogate primary key
    * 
    * @return  Primary key
    */
   public Long getId()
   {
      return id;
   }

   /**
    * Setter: Surrogate primary key
    * 
    * @param   id
    *          Primary key
    */
   public void setId(Long id)
   {
      this.id = id;
   }

   /**
    * Getter: ID type
    * 
    * @return  ID type
    */
   public integer getIdType()
   {
      return (new integer(idType));
   }

   /**
    * Setter: ID type
    * 
    * @param   idType
    *          ID type
    */
   public void setIdType(NumberType idType)
   {
      this.idType.assign(idType, true);
   }

   /**
    * Getter: Description
    * 
    * @return  Description
    */
   public character getDescription()
   {
      return (new character(description));
   }

   /**
    * Setter: Description
    * 
    * @param   description
    *          Description
    */
   public void setDescription(Text description)
   {
      this.description.assign(description, true);
   }
}
