package com.goldencode.hotel.dmo.hotel.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import com.goldencode.p2j.persist.*;
import com.goldencode.p2j.util.*;
import com.goldencode.hotel.dmo.hotel.*;

/**
 * Data Model Object corresponding with table <code>hotel.guest</code> (previously
 * <code>hotel.guest</code>).
 * <p>
 * Guest staying in a room
 */
@LegacyTable(name = "guest", table = "guest")
@LegacyIndexes(
{
   @LegacyIndex(name = "stay-order", converted = "guest_stay_order", primary = true, unique = true, components = 
   {
      @LegacyIndexComponent(fieldName = "stay-id"),
      @LegacyIndexComponent(fieldName = "order")
   })
})
public class GuestImpl
implements Serializable,
           Persistable,
           Guest
{
   /** Surrogate primary key */
   private Long id = null;

   /** Stay ID */
   @LegacyField(fieldId = 1, name = "stay-id", format = "->,>>>,>>9", initial = "0", columnLabel = "Stay ID", label = "Stay ID")
   private final integer stayId;

   /** Display Order */
   @LegacyField(fieldId = 2, name = "order", format = "->,>>>,>>9", initial = "1", columnLabel = "Display Order", label = "Display Order")
   private final integer order;

   /** First Name */
   @LegacyField(fieldId = 3, name = "first-name", format = "x(20)", columnLabel = "First Name", label = "First Name")
   private final character firstName;

   /** Last Name */
   @LegacyField(fieldId = 4, name = "last-name", format = "x(20)", columnLabel = "Last Name", label = "Last Name")
   private final character lastName;

   /** ID Type */
   @LegacyField(fieldId = 5, name = "guest-id-type", format = "->,>>>,>>9", initial = "0", columnLabel = "ID Type", label = "ID Type")
   private final integer guestIdType;

   /** ID Number */
   @LegacyField(fieldId = 6, name = "guest-id-num", format = "x(20)", columnLabel = "ID Number", label = "ID Number")
   private final character guestIdNum;

   /** Phone Number */
   @LegacyField(fieldId = 7, name = "phone", format = "x(20)", columnLabel = "Phone Number", label = "Phone Number")
   private final character phone;

   /** Date of Birth */
   @LegacyField(fieldId = 8, name = "birth-date", format = "99/99/99", initial = "?", columnLabel = "Date of Birth", label = "Date of Birth")
   private final date birthDate;

   /** Country */
   @LegacyField(fieldId = 9, name = "country", format = "x(20)", columnLabel = "Country", label = "Country")
   private final character country;

   /** City */
   @LegacyField(fieldId = 10, name = "city", format = "x(20)", columnLabel = "City", label = "City")
   private final character city;

   /** Address */
   @LegacyField(fieldId = 11, name = "address", format = "x(60)", columnLabel = "Address", label = "Address")
   private final character address;

   /**
    * Default constructor.
    */
   public GuestImpl()
   {
      stayId = new integer(0);
      order = new integer(1);
      firstName = new character("");
      lastName = new character("");
      guestIdType = new integer(0);
      guestIdNum = new character("");
      phone = new character("");
      birthDate = new date();
      country = new character("");
      city = new character("");
      address = new character("");
   }

   /**
    * Make a deep copy of the current DMO instance and return it.
    * 
    * @return  A new instance of this class whose data is identical to that of this instance.
    */
   public Undoable deepCopy()
   {
      Undoable copy = new GuestImpl();
      copy.assign(this);
      return copy;
   }

   /**
    * Modify the contained data of the instance to copy the instance data from the given, backup
    * instance.
    * 
    * @param   old
    *          The backup instance from which to copy data.
    */
   public void assign(Undoable old)
   {
      GuestImpl from = (GuestImpl) old;
      setId(from.id);
      setStayId(from.stayId);
      setOrder(from.order);
      setFirstName(from.firstName);
      setLastName(from.lastName);
      setGuestIdType(from.guestIdType);
      setGuestIdNum(from.guestIdNum);
      setPhone(from.phone);
      setBirthDate(from.birthDate);
      setCountry(from.country);
      setCity(from.city);
      setAddress(from.address);
   }

   /**
    * Getter: Surrogate primary key
    * 
    * @return  Primary key
    */
   public Long getId()
   {
      return id;
   }

   /**
    * Setter: Surrogate primary key
    * 
    * @param   id
    *          Primary key
    */
   public void setId(Long id)
   {
      this.id = id;
   }

   /**
    * Getter: Stay ID
    * 
    * @return  Stay ID
    */
   public integer getStayId()
   {
      return (new integer(stayId));
   }

   /**
    * Setter: Stay ID
    * 
    * @param   stayId
    *          Stay ID
    */
   public void setStayId(NumberType stayId)
   {
      this.stayId.assign(stayId, true);
   }

   /**
    * Getter: Display Order
    * 
    * @return  Display Order
    */
   public integer getOrder()
   {
      return (new integer(order));
   }

   /**
    * Setter: Display Order
    * 
    * @param   order
    *          Display Order
    */
   public void setOrder(NumberType order)
   {
      this.order.assign(order, true);
   }

   /**
    * Getter: First Name
    * 
    * @return  First Name
    */
   public character getFirstName()
   {
      return (new character(firstName));
   }

   /**
    * Setter: First Name
    * 
    * @param   firstName
    *          First Name
    */
   public void setFirstName(Text firstName)
   {
      this.firstName.assign(firstName, true);
   }

   /**
    * Getter: Last Name
    * 
    * @return  Last Name
    */
   public character getLastName()
   {
      return (new character(lastName));
   }

   /**
    * Setter: Last Name
    * 
    * @param   lastName
    *          Last Name
    */
   public void setLastName(Text lastName)
   {
      this.lastName.assign(lastName, true);
   }

   /**
    * Getter: ID Type
    * 
    * @return  ID Type
    */
   public integer getGuestIdType()
   {
      return (new integer(guestIdType));
   }

   /**
    * Setter: ID Type
    * 
    * @param   guestIdType
    *          ID Type
    */
   public void setGuestIdType(NumberType guestIdType)
   {
      this.guestIdType.assign(guestIdType, true);
   }

   /**
    * Getter: ID Number
    * 
    * @return  ID Number
    */
   public character getGuestIdNum()
   {
      return (new character(guestIdNum));
   }

   /**
    * Setter: ID Number
    * 
    * @param   guestIdNum
    *          ID Number
    */
   public void setGuestIdNum(Text guestIdNum)
   {
      this.guestIdNum.assign(guestIdNum, true);
   }

   /**
    * Getter: Phone Number
    * 
    * @return  Phone Number
    */
   public character getPhone()
   {
      return (new character(phone));
   }

   /**
    * Setter: Phone Number
    * 
    * @param   phone
    *          Phone Number
    */
   public void setPhone(Text phone)
   {
      this.phone.assign(phone, true);
   }

   /**
    * Getter: Date of Birth
    * 
    * @return  Date of Birth
    */
   public date getBirthDate()
   {
      return (new date(birthDate));
   }

   /**
    * Setter: Date of Birth
    * 
    * @param   birthDate
    *          Date of Birth
    */
   public void setBirthDate(date birthDate)
   {
      this.birthDate.assign(birthDate, true);
   }

   /**
    * Getter: Country
    * 
    * @return  Country
    */
   public character getCountry()
   {
      return (new character(country));
   }

   /**
    * Setter: Country
    * 
    * @param   country
    *          Country
    */
   public void setCountry(Text country)
   {
      this.country.assign(country, true);
   }

   /**
    * Getter: City
    * 
    * @return  City
    */
   public character getCity()
   {
      return (new character(city));
   }

   /**
    * Setter: City
    * 
    * @param   city
    *          City
    */
   public void setCity(Text city)
   {
      this.city.assign(city, true);
   }

   /**
    * Getter: Address
    * 
    * @return  Address
    */
   public character getAddress()
   {
      return (new character(address));
   }

   /**
    * Setter: Address
    * 
    * @param   address
    *          Address
    */
   public void setAddress(Text address)
   {
      this.address.assign(address, true);
   }
}
