package com.goldencode.hotel.dmo.hotel.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import com.goldencode.p2j.persist.*;
import com.goldencode.p2j.util.*;
import com.goldencode.hotel.dmo.hotel.*;

/**
 * Data Model Object corresponding with table <code>hotel.rate</code> (previously
 * <code>hotel.rate</code>).
 * <p>
 * Rates for rooms
 */
@LegacyTable(name = "rate", table = "rate")
@LegacyIndexes(
{
   @LegacyIndex(name = "room-type-date", converted = "rate_room_type_date", primary = true, components = 
   {
      @LegacyIndexComponent(fieldName = "room-type"),
      @LegacyIndexComponent(fieldName = "start-date"),
      @LegacyIndexComponent(fieldName = "end-date")
   })
})
public class RateImpl
implements Serializable,
           Persistable,
           Rate
{
   /** Surrogate primary key */
   private Long id = null;

   /** Room Type */
   @LegacyField(fieldId = 1, name = "room-type", format = ">9", initial = "0", columnLabel = "Room Type", label = "Room Type")
   private final integer roomType;

   /** Start Date; Date when rate starts. */
   @LegacyField(fieldId = 2, name = "start-date", format = "99/99/99", initial = "?", columnLabel = "Start Date", label = "Start Date")
   private final date startDate;

   /** End Date; Date when rate ends (inclusive). */
   @LegacyField(fieldId = 3, name = "end-date", format = "99/99/99", initial = "?", columnLabel = "End Date", label = "End Date")
   private final date endDate;

   /** Rate */
   @LegacyField(fieldId = 4, name = "rate", format = ">>>,>>9.99", initial = "0", columnLabel = "Rate", label = "Rate")
   private final decimal rate;

   /**
    * Default constructor.
    */
   public RateImpl()
   {
      roomType = new integer(0);
      startDate = new date();
      endDate = new date();
      rate = new decimal(0, 2);
   }

   /**
    * Make a deep copy of the current DMO instance and return it.
    * 
    * @return  A new instance of this class whose data is identical to that of this instance.
    */
   public Undoable deepCopy()
   {
      Undoable copy = new RateImpl();
      copy.assign(this);
      return copy;
   }

   /**
    * Modify the contained data of the instance to copy the instance data from the given, backup
    * instance.
    * 
    * @param   old
    *          The backup instance from which to copy data.
    */
   public void assign(Undoable old)
   {
      RateImpl from = (RateImpl) old;
      setId(from.id);
      setRoomType(from.roomType);
      setStartDate(from.startDate);
      setEndDate(from.endDate);
      setRate(from.rate);
   }

   /**
    * Getter: Surrogate primary key
    * 
    * @return  Primary key
    */
   public Long getId()
   {
      return id;
   }

   /**
    * Setter: Surrogate primary key
    * 
    * @param   id
    *          Primary key
    */
   public void setId(Long id)
   {
      this.id = id;
   }

   /**
    * Getter: Room Type
    * 
    * @return  Room Type
    */
   public integer getRoomType()
   {
      return (new integer(roomType));
   }

   /**
    * Setter: Room Type
    * 
    * @param   roomType
    *          Room Type
    */
   public void setRoomType(NumberType roomType)
   {
      this.roomType.assign(roomType, true);
   }

   /**
    * Getter: Date when rate starts.
    * 
    * @return  Start Date
    */
   public date getStartDate()
   {
      return (new date(startDate));
   }

   /**
    * Setter: Date when rate starts.
    * 
    * @param   startDate
    *          Start Date
    */
   public void setStartDate(date startDate)
   {
      this.startDate.assign(startDate, true);
   }

   /**
    * Getter: Date when rate ends (inclusive).
    * 
    * @return  End Date
    */
   public date getEndDate()
   {
      return (new date(endDate));
   }

   /**
    * Setter: Date when rate ends (inclusive).
    * 
    * @param   endDate
    *          End Date
    */
   public void setEndDate(date endDate)
   {
      this.endDate.assign(endDate, true);
   }

   /**
    * Getter: Rate
    * 
    * @return  Rate
    */
   public decimal getRate()
   {
      return (new decimal(rate));
   }

   /**
    * Setter: Rate
    * 
    * @param   rate
    *          Rate
    */
   public void setRate(NumberType rate)
   {
      this.rate.assign(rate, true);
   }
}
