package com.goldencode.hotel.dmo.hotel.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import com.goldencode.p2j.persist.*;
import com.goldencode.p2j.util.*;
import com.goldencode.hotel.dmo.hotel.*;

/**
 * Data Model Object corresponding with table <code>hotel.reservation</code> (previously
 * <code>hotel.reservation</code>).
 * <p>
 * Room reservations
 */
@LegacyTable(name = "reservation", table = "reservation")
@LegacyIndexes(
{
   @LegacyIndex(name = "room-num-date", converted = "reservation_room_num_date", primary = true, components = 
   {
      @LegacyIndexComponent(fieldName = "room-num"),
      @LegacyIndexComponent(fieldName = "start-date"),
      @LegacyIndexComponent(fieldName = "end-date")
   }),
   @LegacyIndex(name = "guest", converted = "reservation_guest", word = true, components = 
   {
      @LegacyIndexComponent(fieldName = "guest")
   })
})
public class ReservationImpl
implements Serializable,
           Persistable,
           Reservation
{
   /** Surrogate primary key */
   private Long id = null;

   /** Room Number */
   @LegacyField(fieldId = 1, name = "room-num", format = ">>>>9", initial = "0", columnLabel = "Room Number", label = "Room Number")
   private final integer roomNum;

   /** Check-in Date */
   @LegacyField(fieldId = 2, name = "start-date", format = "99/99/99", initial = "?", columnLabel = "Check-in Date", label = "Check-in Date")
   private final date startDate;

   /** Check-out Date */
   @LegacyField(fieldId = 3, name = "end-date", format = "99/99/99", initial = "?", columnLabel = "Check-out Date", label = "Check-out Date")
   private final date endDate;

   /** Guest */
   @LegacyField(fieldId = 4, name = "guest", format = "x(20)", columnLabel = "Guest", label = "Guest")
   private final character guest;

   /** Number of Guests */
   @LegacyField(fieldId = 5, name = "num-guests", format = ">9", initial = "1", columnLabel = "Number of Guests", label = "Number of Guests")
   private final integer numGuests;

   /** Price */
   @LegacyField(fieldId = 6, name = "price", format = ">>>,>>9.99", initial = "0", columnLabel = "Price", label = "Price")
   private final decimal price;

   /**
    * Default constructor.
    */
   public ReservationImpl()
   {
      roomNum = new integer(0);
      startDate = new date();
      endDate = new date();
      guest = new character("");
      numGuests = new integer(1);
      price = new decimal(0, 2);
   }

   /**
    * Make a deep copy of the current DMO instance and return it.
    * 
    * @return  A new instance of this class whose data is identical to that of this instance.
    */
   public Undoable deepCopy()
   {
      Undoable copy = new ReservationImpl();
      copy.assign(this);
      return copy;
   }

   /**
    * Modify the contained data of the instance to copy the instance data from the given, backup
    * instance.
    * 
    * @param   old
    *          The backup instance from which to copy data.
    */
   public void assign(Undoable old)
   {
      ReservationImpl from = (ReservationImpl) old;
      setId(from.id);
      setRoomNum(from.roomNum);
      setStartDate(from.startDate);
      setEndDate(from.endDate);
      setGuest(from.guest);
      setNumGuests(from.numGuests);
      setPrice(from.price);
   }

   /**
    * Getter: Surrogate primary key
    * 
    * @return  Primary key
    */
   public Long getId()
   {
      return id;
   }

   /**
    * Setter: Surrogate primary key
    * 
    * @param   id
    *          Primary key
    */
   public void setId(Long id)
   {
      this.id = id;
   }

   /**
    * Getter: Room Number
    * 
    * @return  Room Number
    */
   public integer getRoomNum()
   {
      return (new integer(roomNum));
   }

   /**
    * Setter: Room Number
    * 
    * @param   roomNum
    *          Room Number
    */
   public void setRoomNum(NumberType roomNum)
   {
      this.roomNum.assign(roomNum, true);
   }

   /**
    * Getter: Check-in Date
    * 
    * @return  Check-in Date
    */
   public date getStartDate()
   {
      return (new date(startDate));
   }

   /**
    * Setter: Check-in Date
    * 
    * @param   startDate
    *          Check-in Date
    */
   public void setStartDate(date startDate)
   {
      this.startDate.assign(startDate, true);
   }

   /**
    * Getter: Check-out Date
    * 
    * @return  Check-out Date
    */
   public date getEndDate()
   {
      return (new date(endDate));
   }

   /**
    * Setter: Check-out Date
    * 
    * @param   endDate
    *          Check-out Date
    */
   public void setEndDate(date endDate)
   {
      this.endDate.assign(endDate, true);
   }

   /**
    * Getter: Guest
    * 
    * @return  Guest
    */
   public character getGuest()
   {
      return (new character(guest));
   }

   /**
    * Setter: Guest
    * 
    * @param   guest
    *          Guest
    */
   public void setGuest(Text guest)
   {
      this.guest.assign(guest, true);
   }

   /**
    * Getter: Number of Guests
    * 
    * @return  Number of Guests
    */
   public integer getNumGuests()
   {
      return (new integer(numGuests));
   }

   /**
    * Setter: Number of Guests
    * 
    * @param   numGuests
    *          Number of Guests
    */
   public void setNumGuests(NumberType numGuests)
   {
      this.numGuests.assign(numGuests, true);
   }

   /**
    * Getter: Price
    * 
    * @return  Price
    */
   public decimal getPrice()
   {
      return (new decimal(price));
   }

   /**
    * Setter: Price
    * 
    * @param   price
    *          Price
    */
   public void setPrice(NumberType price)
   {
      this.price.assign(price, true);
   }
}
