package com.goldencode.hotel.dmo.hotel.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import com.goldencode.p2j.persist.*;
import com.goldencode.p2j.util.*;
import com.goldencode.hotel.dmo.hotel.*;

/**
 * Data Model Object corresponding with table <code>hotel.service</code> (previously
 * <code>hotel.service</code>).
 * <p>
 * Additional services
 */
@LegacyTable(name = "service", table = "service")
@LegacyIndexes(
{
   @LegacyIndex(name = "stay-charge", converted = "service_stay_charge", primary = true, components = 
   {
      @LegacyIndexComponent(fieldName = "stay-id"),
      @LegacyIndexComponent(fieldName = "service-type"),
      @LegacyIndexComponent(fieldName = "service-date")
   })
})
public class ServiceImpl
implements Serializable,
           Persistable,
           Service
{
   /** Surrogate primary key */
   private Long id = null;

   /** Stay ID */
   @LegacyField(fieldId = 1, name = "stay-id", format = "->,>>>,>>9", initial = "0", columnLabel = "Stay ID", label = "Stay ID")
   private final integer stayId;

   /** Charge Amount */
   @LegacyField(fieldId = 2, name = "amount", format = "->>,>>9.99", initial = "0", columnLabel = "Charge Amount", label = "Charge Amount")
   private final decimal amount;

   /** Date */
   @LegacyField(fieldId = 3, name = "service-date", format = "99/99/99", initial = "?", columnLabel = "Date", label = "Date")
   private final date serviceDate;

   /** Service Type */
   @LegacyField(fieldId = 4, name = "service-type", format = "->,>>>,>>9", initial = "0", columnLabel = "Service Type", label = "Service Type")
   private final integer serviceType;

   /**
    * Default constructor.
    */
   public ServiceImpl()
   {
      stayId = new integer(0);
      amount = new decimal(0, 2);
      serviceDate = new date();
      serviceType = new integer(0);
   }

   /**
    * Make a deep copy of the current DMO instance and return it.
    * 
    * @return  A new instance of this class whose data is identical to that of this instance.
    */
   public Undoable deepCopy()
   {
      Undoable copy = new ServiceImpl();
      copy.assign(this);
      return copy;
   }

   /**
    * Modify the contained data of the instance to copy the instance data from the given, backup
    * instance.
    * 
    * @param   old
    *          The backup instance from which to copy data.
    */
   public void assign(Undoable old)
   {
      ServiceImpl from = (ServiceImpl) old;
      setId(from.id);
      setStayId(from.stayId);
      setAmount(from.amount);
      setServiceDate(from.serviceDate);
      setServiceType(from.serviceType);
   }

   /**
    * Getter: Surrogate primary key
    * 
    * @return  Primary key
    */
   public Long getId()
   {
      return id;
   }

   /**
    * Setter: Surrogate primary key
    * 
    * @param   id
    *          Primary key
    */
   public void setId(Long id)
   {
      this.id = id;
   }

   /**
    * Getter: Stay ID
    * 
    * @return  Stay ID
    */
   public integer getStayId()
   {
      return (new integer(stayId));
   }

   /**
    * Setter: Stay ID
    * 
    * @param   stayId
    *          Stay ID
    */
   public void setStayId(NumberType stayId)
   {
      this.stayId.assign(stayId, true);
   }

   /**
    * Getter: Charge Amount
    * 
    * @return  Charge Amount
    */
   public decimal getAmount()
   {
      return (new decimal(amount));
   }

   /**
    * Setter: Charge Amount
    * 
    * @param   amount
    *          Charge Amount
    */
   public void setAmount(NumberType amount)
   {
      this.amount.assign(amount, true);
   }

   /**
    * Getter: Date
    * 
    * @return  Date
    */
   public date getServiceDate()
   {
      return (new date(serviceDate));
   }

   /**
    * Setter: Date
    * 
    * @param   serviceDate
    *          Date
    */
   public void setServiceDate(date serviceDate)
   {
      this.serviceDate.assign(serviceDate, true);
   }

   /**
    * Getter: Service Type
    * 
    * @return  Service Type
    */
   public integer getServiceType()
   {
      return (new integer(serviceType));
   }

   /**
    * Setter: Service Type
    * 
    * @param   serviceType
    *          Service Type
    */
   public void setServiceType(NumberType serviceType)
   {
      this.serviceType.assign(serviceType, true);
   }
}
