package com.goldencode.hotel.dmo.hotel.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import com.goldencode.p2j.persist.*;
import com.goldencode.p2j.util.*;
import com.goldencode.hotel.dmo.hotel.*;

/**
 * Data Model Object corresponding with table <code>hotel.service_type</code> (previously
 * <code>hotel.service-type</code>).
 * <p>
 * Additional service types
 */
@LegacyTable(name = "service-type", table = "service_type")
@LegacyIndexes(
{
   @LegacyIndex(name = "charge-type", converted = "service_type_charge_type", primary = true, unique = true, components = 
   {
      @LegacyIndexComponent(fieldName = "service-type")
   })
})
public class ServiceTypeImpl
implements Serializable,
           Persistable,
           ServiceType
{
   /** Surrogate primary key */
   private Long id = null;

   /** Service Type */
   @LegacyField(fieldId = 1, name = "service-type", format = ">>9", initial = "0", columnLabel = "Service Type", label = "Service Type")
   private final integer serviceType;

   /** Description */
   @LegacyField(fieldId = 2, name = "description", format = "x(20)", columnLabel = "Description", label = "Description")
   private final character description;

   /** Default Charge Amount */
   @LegacyField(fieldId = 3, name = "default-amount", format = "->>,>>9.99", initial = "0", columnLabel = "Default Charge Amount", label = "Default Charge Amount")
   private final decimal defaultAmount;

   /**
    * Default constructor.
    */
   public ServiceTypeImpl()
   {
      serviceType = new integer(0);
      description = new character("");
      defaultAmount = new decimal(0, 2);
   }

   /**
    * Make a deep copy of the current DMO instance and return it.
    * 
    * @return  A new instance of this class whose data is identical to that of this instance.
    */
   public Undoable deepCopy()
   {
      Undoable copy = new ServiceTypeImpl();
      copy.assign(this);
      return copy;
   }

   /**
    * Modify the contained data of the instance to copy the instance data from the given, backup
    * instance.
    * 
    * @param   old
    *          The backup instance from which to copy data.
    */
   public void assign(Undoable old)
   {
      ServiceTypeImpl from = (ServiceTypeImpl) old;
      setId(from.id);
      setServiceType(from.serviceType);
      setDescription(from.description);
      setDefaultAmount(from.defaultAmount);
   }

   /**
    * Getter: Surrogate primary key
    * 
    * @return  Primary key
    */
   public Long getId()
   {
      return id;
   }

   /**
    * Setter: Surrogate primary key
    * 
    * @param   id
    *          Primary key
    */
   public void setId(Long id)
   {
      this.id = id;
   }

   /**
    * Getter: Service Type
    * 
    * @return  Service Type
    */
   public integer getServiceType()
   {
      return (new integer(serviceType));
   }

   /**
    * Setter: Service Type
    * 
    * @param   serviceType
    *          Service Type
    */
   public void setServiceType(NumberType serviceType)
   {
      this.serviceType.assign(serviceType, true);
   }

   /**
    * Getter: Description
    * 
    * @return  Description
    */
   public character getDescription()
   {
      return (new character(description));
   }

   /**
    * Setter: Description
    * 
    * @param   description
    *          Description
    */
   public void setDescription(Text description)
   {
      this.description.assign(description, true);
   }

   /**
    * Getter: Default Charge Amount
    * 
    * @return  Default Charge Amount
    */
   public decimal getDefaultAmount()
   {
      return (new decimal(defaultAmount));
   }

   /**
    * Setter: Default Charge Amount
    * 
    * @param   defaultAmount
    *          Default Charge Amount
    */
   public void setDefaultAmount(NumberType defaultAmount)
   {
      this.defaultAmount.assign(defaultAmount, true);
   }
}
