package com.goldencode.hotel.dmo.hotel.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import com.goldencode.p2j.persist.*;
import com.goldencode.p2j.util.*;
import com.goldencode.hotel.dmo.hotel.*;

/**
 * Data Model Object corresponding with table <code>hotel.stay</code> (previously
 * <code>hotel.stay</code>).
 * <p>
 * Staying of some people for a period of time in specifc room
 */
@LegacyTable(name = "stay", table = "stay")
@LegacyIndexes(
{
   @LegacyIndex(name = "stay-id", converted = "stay_stay_id", primary = true, unique = true, components = 
   {
      @LegacyIndexComponent(fieldName = "stay-id")
   })
})
public class StayImpl
implements Serializable,
           Persistable,
           Stay
{
   /** Surrogate primary key */
   private Long id = null;

   /** Stay ID */
   @LegacyField(fieldId = 1, name = "stay-id", format = "->,>>>,>>9", initial = "0", columnLabel = "Stay ID", label = "Stay ID")
   private final integer stayId;

   /** Room Number */
   @LegacyField(fieldId = 2, name = "room-num", format = ">>>>9", initial = "0", columnLabel = "Room Number", label = "Room Number")
   private final integer roomNum;

   /** Check-in Date */
   @LegacyField(fieldId = 3, name = "start-date", format = "99/99/99", initial = "?", columnLabel = "Check-in Date", label = "Check-in Date")
   private final date startDate;

   /** Check-out Date */
   @LegacyField(fieldId = 4, name = "end-date", format = "99/99/99", initial = "?", columnLabel = "Check-out Date", label = "Check-out Date")
   private final date endDate;

   /** Checked Out */
   @LegacyField(fieldId = 5, name = "checked-out", format = "yes/no", initial = "false", columnLabel = "Checked Out", label = "Checked Out")
   private final logical checkedOut;

   /** Number of Guests */
   @LegacyField(fieldId = 6, name = "num-guests", format = ">9", initial = "0", columnLabel = "Number of Guests", label = "Number of Guests")
   private final integer numGuests;

   /** Price */
   @LegacyField(fieldId = 7, name = "price", format = ">>>,>>>,>>9", initial = "0", columnLabel = "Price", label = "Price")
   private final integer price;

   /**
    * Default constructor.
    */
   public StayImpl()
   {
      stayId = new integer(0);
      roomNum = new integer(0);
      startDate = new date();
      endDate = new date();
      checkedOut = new logical(false);
      numGuests = new integer(0);
      price = new integer(0);
   }

   /**
    * Make a deep copy of the current DMO instance and return it.
    * 
    * @return  A new instance of this class whose data is identical to that of this instance.
    */
   public Undoable deepCopy()
   {
      Undoable copy = new StayImpl();
      copy.assign(this);
      return copy;
   }

   /**
    * Modify the contained data of the instance to copy the instance data from the given, backup
    * instance.
    * 
    * @param   old
    *          The backup instance from which to copy data.
    */
   public void assign(Undoable old)
   {
      StayImpl from = (StayImpl) old;
      setId(from.id);
      setStayId(from.stayId);
      setRoomNum(from.roomNum);
      setStartDate(from.startDate);
      setEndDate(from.endDate);
      setCheckedOut(from.checkedOut);
      setNumGuests(from.numGuests);
      setPrice(from.price);
   }

   /**
    * Getter: Surrogate primary key
    * 
    * @return  Primary key
    */
   public Long getId()
   {
      return id;
   }

   /**
    * Setter: Surrogate primary key
    * 
    * @param   id
    *          Primary key
    */
   public void setId(Long id)
   {
      this.id = id;
   }

   /**
    * Getter: Stay ID
    * 
    * @return  Stay ID
    */
   public integer getStayId()
   {
      return (new integer(stayId));
   }

   /**
    * Setter: Stay ID
    * 
    * @param   stayId
    *          Stay ID
    */
   public void setStayId(NumberType stayId)
   {
      this.stayId.assign(stayId, true);
   }

   /**
    * Getter: Room Number
    * 
    * @return  Room Number
    */
   public integer getRoomNum()
   {
      return (new integer(roomNum));
   }

   /**
    * Setter: Room Number
    * 
    * @param   roomNum
    *          Room Number
    */
   public void setRoomNum(NumberType roomNum)
   {
      this.roomNum.assign(roomNum, true);
   }

   /**
    * Getter: Check-in Date
    * 
    * @return  Check-in Date
    */
   public date getStartDate()
   {
      return (new date(startDate));
   }

   /**
    * Setter: Check-in Date
    * 
    * @param   startDate
    *          Check-in Date
    */
   public void setStartDate(date startDate)
   {
      this.startDate.assign(startDate, true);
   }

   /**
    * Getter: Check-out Date
    * 
    * @return  Check-out Date
    */
   public date getEndDate()
   {
      return (new date(endDate));
   }

   /**
    * Setter: Check-out Date
    * 
    * @param   endDate
    *          Check-out Date
    */
   public void setEndDate(date endDate)
   {
      this.endDate.assign(endDate, true);
   }

   /**
    * Getter: Checked Out
    * 
    * @return  Checked Out
    */
   public logical isCheckedOut()
   {
      return (new logical(checkedOut));
   }

   /**
    * Setter: Checked Out
    * 
    * @param   checkedOut
    *          Checked Out
    */
   public void setCheckedOut(logical checkedOut)
   {
      this.checkedOut.assign(checkedOut, true);
   }

   /**
    * Getter: Number of Guests
    * 
    * @return  Number of Guests
    */
   public integer getNumGuests()
   {
      return (new integer(numGuests));
   }

   /**
    * Setter: Number of Guests
    * 
    * @param   numGuests
    *          Number of Guests
    */
   public void setNumGuests(NumberType numGuests)
   {
      this.numGuests.assign(numGuests, true);
   }

   /**
    * Getter: Price
    * 
    * @return  Price
    */
   public integer getPrice()
   {
      return (new integer(price));
   }

   /**
    * Setter: Price
    * 
    * @param   price
    *          Price
    */
   public void setPrice(NumberType price)
   {
      this.price.assign(price, true);
   }
}
