/*--------------------------------------------------------------------------
    File        : Query.java
    Purpose     : Super procedure for ADM objects which retrieve data from
                  database tables.

    Syntax      : adm2/query.p

    Modified    : IZ 1569 : Foreign fields requires field to exist in target
                  SDO RowObject.
                  Fix : Renamed function columnHandle to dbColumnHandle and
                  updated all references to it here.
    Modified    : IZ 4050 Gikas A. Gikas
  ------------------------------------------------------------------------*/

package com.goldencode.hotel.common.adm2;

import com.goldencode.p2j.util.*;
import com.goldencode.p2j.persist.*;
import com.goldencode.p2j.ui.*;
import com.goldencode.util.*;
import com.goldencode.p2j.persist.lock.*;

import static com.goldencode.p2j.util.BlockManager.*;
import static com.goldencode.p2j.util.CompareOps.*;
import static com.goldencode.p2j.util.MathOps.*;
import static com.goldencode.p2j.util.logical.*;
import static com.goldencode.p2j.util.TextOps.*;
import static com.goldencode.p2j.util.ProcedureManager.*;
import static com.goldencode.p2j.util.ArrayAssigner.*;
import static com.goldencode.p2j.util.character.*;
import static com.goldencode.p2j.ui.LogicalTerminal.*;

/**
 * Business logic (converted to Java from the 4GL source code
 * in common/adm2/query.p).
 */
public class Query
{
   /*********************************************************************
   * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
   * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
   * below.  All Rights Reserved.                                       *
   *                                                                    *
   * The Initial Developer of the Original Code is PSC.  The Original   *
   * Code is Progress IDE code released to open source December 1, 2000.*
   *                                                                    *
   * The contents of this file are subject to the Possenet Public       *
   * License Version 1.0 (the "License"); you may not use this file     *
   * except in compliance with the License.  A copy of the License is   *
   * available as of the date of this notice at                         *
   * http://www.possenet.org/license.html                               *
   *                                                                    *
   * Software distributed under the License is distributed on an "AS IS"*
   * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
   * should refer to the License for the specific language governing    *
   * rights and limitations under the License.                          *
   *                                                                    *
   * Contributors:                                                      *
   *                                                                    *
   *********************************************************************/   
   /*          This .W file was created with the Progress UIB.             */   
   /*----------------------------------------------------------------------*/   
   /* ***************************  Definitions  ************************** */   
   /* Tell qryprop.i that this is the query super procedure. */   
   /*********************************************************************
   * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
   * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
   * below.  All Rights Reserved.                                       *
   *                                                                    *
   * The Initial Developer of the Original Code is PSC.  The Original   *
   * Code is Progress IDE code released to open source December 1, 2000.*
   *                                                                    *
   * The contents of this file are subject to the Possenet Public       *
   * License Version 1.0 (the "License"); you may not use this file     *
   * except in compliance with the License.  A copy of the License is   *
   * available as of the date of this notice at                         *
   * http://www.possenet.org/license.html                               *
   *                                                                    *
   * Software distributed under the License is distributed on an "AS IS"*
   * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
   * should refer to the License for the specific language governing    *
   * rights and limitations under the License.                          *
   *                                                                    *
   * Contributors:                                                      *
   *                                                                    *
   *********************************************************************/   
   /*--------------------------------------------------------------------------
       File        : queryexclcustom.i
       Purpose     : Defines EXCLUDE-xxx preprocessor variables to exclude
                     functions or internal procedures in the super procedure
                     where it is referenced.
       Syntax      : {src/adm2/custom/queryexclcustom.i}
   
       Modified    : 06/03/1999
       
       Notes       : Referenced in src/adm2/query.p
     ------------------------------------------------------------------------*/   
   /* allow for override of prefix for calculation functions */   
   /* This variable is needed at least temporarily in 9.1B/C so that a called
      fn can tell who the actual source was. */   
   handle ghTargetProcedure = TypeFactory.handle();

   /* _UIB-CODE-BLOCK-END */   
   /* *********************** Procedure Settings ************************ */   
   /* Settings for THIS-PROCEDURE
      Type: Procedure
      Allow: 
      Frames: 0
      Add Fields to: Neither
      Other Settings: CODE-ONLY COMPILE
    */   
   /* *************************  Create Window  ************************** */   
   /* DESIGN Window definition (used by the UIB) 
     CREATE WINDOW Procedure ASSIGN
            HEIGHT             = 21.91
            WIDTH              = 58.8.
   _* END WINDOW DEFINITION *_
                                                                           */   
   /* ************************* Included-Libraries *********************** */   
   /*********************************************************************
   * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
   * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
   * below.  All Rights Reserved.                                       *
   *                                                                    *
   * The Initial Developer of the Original Code is PSC.  The Original   *
   * Code is Progress IDE code released to open source December 1, 2000.*
   *                                                                    *
   * The contents of this file are subject to the Possenet Public       *
   * License Version 1.0 (the "License"); you may not use this file     *
   * except in compliance with the License.  A copy of the License is   *
   * available as of the date of this notice at                         *
   * http://www.possenet.org/license.html                               *
   *                                                                    *
   * Software distributed under the License is distributed on an "AS IS"*
   * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
   * should refer to the License for the specific language governing    *
   * rights and limitations under the License.                          *
   *                                                                    *
   * Contributors:                                                      *
   *                                                                    *
   *********************************************************************/   
   /*--------------------------------------------------------------------------
       File        : qryprop.i
       Purpose     : Starts query.p Super procedure and defines Basic Query Object
                     properties.
       Syntax      : {src/adm2/qryprop.i}
   
       Description :
   
       Modified    : May 15, 2000 -- Version 9.1B
       Modified    : 10/24/2001            Mark Davies (MIP0
                     Added new property FilterAvailable.
     ------------------------------------------------------------------------*/   
   /*          This .W file was created with the Progress UIB.             */   
   /*----------------------------------------------------------------------*/   
   /* ***************************  Definitions  ************************** */   
   /*********************************************************************
   * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
   * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
   * below.  All Rights Reserved.                                       *
   *                                                                    *
   * The Initial Developer of the Original Code is PSC.  The Original   *
   * Code is Progress IDE code released to open source December 1, 2000.*
   *                                                                    *
   * The contents of this file are subject to the Possenet Public       *
   * License Version 1.0 (the "License"); you may not use this file     *
   * except in compliance with the License.  A copy of the License is   *
   * available as of the date of this notice at                         *
   * http://www.possenet.org/license.html                               *
   *                                                                    *
   * Software distributed under the License is distributed on an "AS IS"*
   * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
   * should refer to the License for the specific language governing    *
   * rights and limitations under the License.                          *
   *                                                                    *
   * Contributors:                                                      *
   *                                                                    *
   *********************************************************************/   
   /*--------------------------------------------------------------------------
       File        : querydefscustom.i
       Purpose     : Entry point to define:
                       New definitions
                       {&xcInstanceProperties}
                       {&ADM-PROPERTY-DLG}
       Syntax      : {src/adm2/custom/querydefscustom.i}
   
       Modified    : 06/03/1999
       
       Notes       : Referenced in {src/adm2/qryprop.i}
     ------------------------------------------------------------------------*/   
   /* This variable designates the maximal length of any database 
      table index within the Rocket. If the total length of index 
      fields exceeds this limit, PROGRESS raises a run-time error. */   
   integer xiRocketIndexLimit = TypeFactory.integer((long) 188);

   /* _UIB-CODE-BLOCK-END */   
   /* ********************  Preprocessor Definitions  ******************** */   
   /* _UIB-PREPROCESSOR-BLOCK-END */   
   /* *********************** Procedure Settings ************************ */   
   /* Settings for THIS-PROCEDURE
      Type: Include
      Allow: 
      Frames: 0
      Add Fields to: Neither
      Other Settings: INCLUDE-ONLY
    */   
   /* *************************  Create Window  ************************** */   
   /* DESIGN Window definition (used by the UIB) 
     CREATE WINDOW Include ASSIGN
            HEIGHT             = 4.57
            WIDTH              = 42.4.
   _* END WINDOW DEFINITION *_
                                                                           */   
   /* ***************************  Main Block  *************************** */   
   /* Include the file which defines prototypes for all of the super
        procedure's entry points. Also, start or attach to the super procedure.
        Skip start-super-proc if we *are* the super procedure. 
        And skip including the prototypes if we are *any* super procedure. */   
   /* Preprocessors to define properties that can be retrieved
       directly from the property temp-table. */   
   /* if appserver aware add appserver properties */   
   /*********************************************************************
   * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
   * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
   * below.  All Rights Reserved.                                       *
   *                                                                    *
   * The Initial Developer of the Original Code is PSC.  The Original   *
   * Code is Progress IDE code released to open source December 1, 2000.*
   *                                                                    *
   * The contents of this file are subject to the Possenet Public       *
   * License Version 1.0 (the "License"); you may not use this file     *
   * except in compliance with the License.  A copy of the License is   *
   * available as of the date of this notice at                         *
   * http://www.possenet.org/license.html                               *
   *                                                                    *
   * Software distributed under the License is distributed on an "AS IS"*
   * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
   * should refer to the License for the specific language governing    *
   * rights and limitations under the License.                          *
   *                                                                    *
   * Contributors:                                                      *
   *                                                                    *
   *********************************************************************/   
   /*--------------------------------------------------------------------------
       File        : smrtprop.i
       Purpose     : Starts smart.p super procedure and defines general
                     SmartObject properties and other values.
       Syntax      : {src/adm2/smrtprop.i}
   
       Description :
   
       Modified    : February 4, 2000 - Version 9.1B
     ------------------------------------------------------------------------*/   
   /*          This .W file was created with the Progress UIB.             */   
   /*----------------------------------------------------------------------*/   
   /* ***************************  Definitions  ************************** */   
   /*********************************************************************
   * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
   * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
   * below.  All Rights Reserved.                                       *
   *                                                                    *
   * The Initial Developer of the Original Code is PSC.  The Original   *
   * Code is Progress IDE code released to open source December 1, 2000.*
   *                                                                    *
   * The contents of this file are subject to the Possenet Public       *
   * License Version 1.0 (the "License"); you may not use this file     *
   * except in compliance with the License.  A copy of the License is   *
   * available as of the date of this notice at                         *
   * http://www.possenet.org/license.html                               *
   *                                                                    *
   * Software distributed under the License is distributed on an "AS IS"*
   * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
   * should refer to the License for the specific language governing    *
   * rights and limitations under the License.                          *
   *                                                                    *
   * Contributors:                                                      *
   *                                                                    *
   *********************************************************************/   
   /*--------------------------------------------------------------------------
       File        : smartdefscustom.i
       Purpose     : Entry point to define:
                       New definitions
                       {&xcInstanceProperties}
                       {&ADM-PROPERTY-DLG}
       Syntax      : {src/adm2/custom/smartdefscustom.i}
   
       Modified    : 06/03/1999
       
       Notes       : Referenced in {src/adm2/smrtprop.i}
     ------------------------------------------------------------------------*/   
   /* Service Managers */   
   /*********************************************************************
   * Copyright (C) 2000-2002 by Progress Software Corporation ("PSC"),  *
   * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
   * below.  All Rights Reserved.                                       *
   *                                                                    *
   * The Initial Developer of the Original Code is PSC.  The Original   *
   * Code is Progress IDE code released to open source December 1, 2000.*
   *                                                                    *
   * The contents of this file are subject to the Possenet Public       *
   * License Version 1.0 (the "License"); you may not use this file     *
   * except in compliance with the License.  A copy of the License is   *
   * available as of the date of this notice at                         *
   * http://www.possenet.org/license.html                               *
   *                                                                    *
   * Software distributed under the License is distributed on an "AS IS"*
   * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
   * should refer to the License for the specific language governing    *
   * rights and limitations under the License.                          *
   *                                                                    *
   * Contributors:                                                      *
   *                                                                    *
   *********************************************************************/   
   /*---------------------------------------------------------------------------------
     File: globals.i
   
     Description:  ICF Global Shared Variables
   
     Purpose:      ICF Global Shared Variables
                   The global shared variables available within the Dynamics environment.
                   
                   NOTE: (Dynamic Only )
                   Shared variables do not span the client / server boundary. On the
                   client these are set-up in afstartupp.p and on the server in the agent
                   as_startup.p
                   Also note if we include the guts of a procedure as an internal
                   procedure for shared client/server code, we sometime need to
                   ensure the shared variables are not redefined.
   
     Parameters:   <none>
   
     History:
     --------
   
     Modified: 11/08/2001 - Mark Davies (MIP)
               Copied code for ICF afglobals.i and added here to allow
               ICF changed adm code to compile. afglobals.i now includes
               this file for definitions.
               Any ADM code will now reference globals.i which excludes any other
               ICF specific includes.
   -----------------------------------------------------------*/   
   /*                   This .W file was created with the Progress UIB.             */   
   /*-------------------------------------------------------------------------------*/   
   /* ***************************  Definitions  ************************** */   
   /* _UIB-CODE-BLOCK-END */   
   /* ********************  Preprocessor Definitions  ******************** */   
   /* _UIB-PREPROCESSOR-BLOCK-END */   
   /* *********************** Procedure Settings ************************ */   
   /* Settings for THIS-PROCEDURE
      Type: Include
      Allow: 
      Frames: 0
      Add Fields to: Neither
      Other Settings: INCLUDE-ONLY
    */   
   /* *************************  Create Window  ************************** */   
   /* DESIGN Window definition (used by the UIB) 
     CREATE WINDOW Include ASSIGN
            HEIGHT             = 5.57
            WIDTH              = 64.4.
   _* END WINDOW DEFINITION *_
                                                                           */   
   /* ***************************  Main Block  *************************** */   
   /* Used whereever Crystal is used and when defining the tt_datasource temp-table 
      to define the maximum columns supported when printing to crystal.
      See browsercustom.p and afcrpplip2p.p for example uses
   */   
   /* Used for filter checks */   
   /* The table delimiter used in assignlist, *ColumnsByTable props */   
   handle gshAstraAppserver = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshastraappserver", true, new handle());

   /* Handle to Astra Application Server Partition */   
   handle gshSessionManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshsessionmanager", true, new handle());

   /* Handle Astra Session Manager */   
   handle gshRimanager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshrimanager", true, new handle());

   /* Handle Astra Session Manager */   
   handle gshSecurityManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshsecuritymanager", true, new handle());

   /* Handle Astra Security Manager */   
   handle gshProfileManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshprofilemanager", true, new handle());

   /* Handle Astra Profile Manager */   
   handle gshRepositoryManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshrepositorymanager", true, new handle());

   /* Handle Astra Repository Manager */   
   handle gshTranslationManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshtranslationmanager", true, new handle());

   /* Handle Astra Translation Manager */   
   handle gshWebManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshwebmanager", true, new handle());

   /* Handle Astra Web Manager */   
   character gscSessionId = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gscsessionid", true, new character(""));

   /* Unique session id */   
   decimal gsdSessionObj = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gsdsessionobj", true, new decimal(0));

   /* Unique session id */   
   handle gshFinManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshfinmanager", true, new handle());

   /* Handle AstraFin Generic Manager */   
   handle gshGenManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshgenmanager", true, new handle());

   /* Handle AstraFramework Generic Manager */   
   handle gshAgnManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshagnmanager", true, new handle());

   /* Handle AstraGen Generic Manager */   
   decimal gsdTempUniqueId = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gsdtempuniqueid", true, new decimal(0));

   /* A number for unique objects */   
   decimal gsdUserObj = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gsduserobj", true, new decimal(0));

   /* User Object ID */   
   decimal gsdRenderTypeObj = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gsdrendertypeobj", true, new decimal(0));

   /* Render Type Object ID */   
   decimal gsdSessionScopeObj = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gsdsessionscopeobj", true, new decimal(0));

   /* Render Type Object ID */   
   /*********************************************************************
   * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
   * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
   * below.  All Rights Reserved.                                       *
   *                                                                    *
   * The Initial Developer of the Original Code is PSC.  The Original   *
   * Code is Progress IDE code released to open source December 1, 2000.*
   *                                                                    *
   * The contents of this file are subject to the Possenet Public       *
   * License Version 1.0 (the "License"); you may not use this file     *
   * except in compliance with the License.  A copy of the License is   *
   * available as of the date of this notice at                         *
   * http://www.possenet.org/license.html                               *
   *                                                                    *
   * Software distributed under the License is distributed on an "AS IS"*
   * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
   * should refer to the License for the specific language governing    *
   * rights and limitations under the License.                          *
   *                                                                    *
   * Contributors:                                                      *
   *                                                                    *
   *********************************************************************/   
   /*------------------------------------------------------------------------
       File        : globalscustom.i
       Purpose     : The developer can create his own version of this to 
                     allow 'global' definitions to be added to the 
                     framework.   
       Syntax      : {src/adm2/custom/globalscustom.i}
   
       Description :
   
       Created     : 06/03/1999
       Notes       : Referenced in {src/adm2/globals.i}
     ----------------------------------------------------------------------*/   
   /*          This .W file was created with the Progress AppBuilder.      */   
   /*----------------------------------------------------------------------*/   
   /* ***************************  Definitions  ************************** */   
   /* _UIB-CODE-BLOCK-END */   
   /* define the ADM Version and broker handle for all SmartObjects */   
   /* Note that adm-props-from-repository is documented as possible to define
        as a variable for flexible override, so it cannot be used for conditional 
        compile.  */   
   /* adm-props-from-repository is supported to be defined as a variable,
       so it is not used for conditional compile */   
   handle ghProp = TypeFactory.handle();

   /**
    * External procedure (converted to Java from the 4GL source code
    * in common/adm2/query.p).
    */
   public void execute()
   {
      externalProcedure(Query.this, new Block((Init) () -> 
      {
         gsdSessionObj.setPrecision(9);
         gsdTempUniqueId.setPrecision(9);
         gsdUserObj.setPrecision(9);
         gsdRenderTypeObj.setPrecision(9);
         gsdSessionScopeObj.setPrecision(9);
      }, 
      (Body) () -> 
      {
         /* _UIB-CODE-BLOCK-END */
         /* *********************** Procedure Settings ************************ */
         /* Settings for THIS-PROCEDURE
            Type: Method-Library
            Allow: 
            Frames: 0
            Add Fields to: Neither
            Other Settings: INCLUDE-ONLY
          */
         /* *************************  Create Window  ************************** */
         /* DESIGN Window definition (used by the UIB) 
           CREATE WINDOW Method-Library ASSIGN
                  HEIGHT             = 14.86
                  WIDTH              = 57.2.
         _* END WINDOW DEFINITION *_
                                                                                 */
         /* ***************************  Main Block  *************************** */
         /* Include the file which defines prototypes for all of the super
              procedure's entry points. Also, start or attach to the super procedure.
              Skip start-super-proc if we *are* the super procedure. 
              And skip including the prototypes if we are *any* super procedure. */
         /* These preprocessors let the get and set methods know at compile time
             which property values are located in the temp-table and which must
             be accessed through the property functions.
          */
         /* for support of dynamic objects */
         /* physical name - no path */
         /* for support of runtime parameters passed from the menus */
         /* Properties for super procedure management */
         /* The name of the super procedure; can be a CSV list */ 
 /* STATELESS, STATEFUL; the mode that all supers for the object will be launched in */ 
 /* Runtime property storing handles to the object's super procedures, defined in SuperProcedure. */
         /* Layout support for dynamic objects */
         /* contains a layout position that allows an object to be laid out correctly. */
         /* determine if we need to create the prop fields here or from the */
         /* repository (if ICF is running ) */
         /* COMMENT THE FOLLOWING LINE TO USE THE "<class>prop.i" DEFINITIONS */
         /* This temp-table defines all the propertt fields for an object.
             This include file contributes the DEFINE statement header and
             all basic smart object properties. Each other property class include file
             adds its own fields and then the parent object ends the statement.
             Define the fields for smart objects only, not for their super procedures.
          */
         if (_not(handle.fromString(entry(1, thisProcedure().unwrapADMData().getADMData(), chr(1))).isValid()))
         {
         }
         
         /* _UIB-CODE-BLOCK-END */
         if (_not(handle.fromString(entry(1, thisProcedure().unwrapADMData().getADMData(), chr(1))).isValid()))
         {
         }
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character whereClauseBuffer(final character _pcWhere_6)
   {
      character pcWhere_6 = TypeFactory.initInput(_pcWhere_6);
      
      return function("whereClauseBuffer", character.class, new Block((Body) () -> 
      {
         /*------------------------------------------------------------------------------
           Purpose:     Returns the buffername of a where clause expression. 
                        This function avoids problems with leading or double blanks in 
                        where clauses.
           Parameters:
             pcWhere - Complete where clause for ONE table with or without the FOR 
                       keyword. The buffername must be the second token in the
                       where clause as in "EACH order OF Customer" or if "FOR" is
                       specified, the third token as in "FOR EACH order".
           
           Notes:      Used internally in query.p.
         ------------------------------------------------------------------------------*/
         pcWhere_6.assign(leftTrim(pcWhere_6));
         
         loopLabel0:
         while (_isGreaterThan(indexOf(pcWhere_6, "  "), 0))
         {
            pcWhere_6.assign(replaceAll(pcWhere_6, "  ", " "));
         }
         
         /* Get rid of potential line break characters */
         pcWhere_6.assign(replaceAll(pcWhere_6, chr(10), ""));
         returnNormal((_isGreaterThan(numEntries(pcWhere_6, " "), 1) ? entry(_begins(pcWhere_6, "FOR ") ? new integer(3) : new integer(2), pcWhere_6, " ") : new character("")));
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * ***********************  Function Implementations****************
    */
   public character getObjectType()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Returns the Type of the SmartObject, such as "SmartDataObject", etc.
                  For an ADM Super procedure, this function returns "SUPER".
         Params:  <none>
      ------------------------------------------------------------------------------*/
      character cType = TypeFactory.character();
      
      return function("getObjectType", character.class, new Block((Body) () -> 
      {
         cType.assign("SUPER");
         returnNormal(cType);
      }));
   }

   /** *********************  Internal Procedures ********************** */
   /**
    * Purpose:     Copies modified values from the RowObject row to the database<br>
    *              records (which for an Update have already been retrieved <br>
    *              and locked).<br>
    *              If the RowMod field is "A" for Add or "C" for Copy, the procedure<br>
    *              creates the database record(s) first and then does the assign.<br>
    * <p>
    * Parameters:<br>
    *   INPUT phRowObjUpd - handle of the buffer to copy from.<br>
    * <p>
    * Notes:       The procedure copies over only those fields whose values were<br>
    *              actually modified, unless this is a copied record, in which<br>
    *              case we want to save all the fields that were enabled for update.
    */
   public void assignDbrow(final handle _phRowObjUpd_1)
   {
      handle phRowObjUpd_1 = TypeFactory.initInput(_phRowObjUpd_1);
      handle hRowBefore = TypeFactory.handle();
      character cUpdColumnsByTable = TypeFactory.character();
      handle hRowUpdFld = TypeFactory.handle();
      handle hRowOldFld = TypeFactory.handle();
      handle hDbfield = TypeFactory.handle();
      integer iField = TypeFactory.integer();
      integer iTable_1 = TypeFactory.integer();
      character cFromField = TypeFactory.character();
      character cToField = TypeFactory.character();
      character cTables = TypeFactory.character();
      character cBuffers = TypeFactory.character();
      handle hBuffer = TypeFactory.handle();
      character cAssignList = TypeFactory.character();
      integer iEntry = TypeFactory.integer();
      integer iBracket = TypeFactory.integer();
      integer iExtent = TypeFactory.integer();
      handle hRowMod = TypeFactory.handle();
      character cRowIdent = TypeFactory.character();
      logical lCreate = TypeFactory.logical();
      character cRowid = TypeFactory.character();
      character cExclude = TypeFactory.character();
      character cUpdColumns = TypeFactory.character();
      character cColumnsByTable = TypeFactory.character();
      character cColumns = TypeFactory.character();
      logical lChanged = TypeFactory.logical();
      character cNotChanged = TypeFactory.character();
      character cAssignFields = TypeFactory.character();
      character cUseAssignFields = TypeFactory.character();
      character cDataColumns = TypeFactory.character();
      logical lAmbiguous = TypeFactory.logical();
      
      internalProcedure(new Block((Body) () -> 
      {
         /* get the 'before' record if this is an Update... */
         if (_isEqual(phRowObjUpd_1.unwrapBuffer().bufferField("RowMod").unwrapBufferField().value(), "U"))
         {
            RecordBuffer.create(hRowBefore, phRowObjUpd_1);
            hRowBefore.unwrapBuffer().findFirst(concat("WHERE RowMod = \"\" AND RowNum = ", valueOf(phRowObjUpd_1.unwrapBuffer().bufferField("RowNum").unwrapBufferField().value())));
         }
         else
         {
            /* if new create a rowObject and compare with this to identify
                          which fields have been actively changed  */
            /* {get Prop var [handle]} */ 
            hRowBefore.assign(ControlFlowOps.invokeDynamicFunctionIn(handle.class, "getRowObject", targetProcedure()));
            RecordBuffer.create(hRowBefore, hRowBefore);
            hRowBefore.unwrapBuffer().bufferCreate();
         }
         
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("UpdatableColumnsByTable"));
         cUpdColumnsByTable.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DataColumnsByTable"));
         cColumnsByTable.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DataColumns"));
         cDataColumns.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("BufferHandles"));
         cBuffers.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("Tables"));
         cTables.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("AssignList"));
         cAssignList.assign(ghProp.unwrapBufferField().value());
         hRowMod.assign(phRowObjUpd_1.unwrapBuffer().bufferField("RowMod"));
         
         IntegerExpression resExpr0 = new IntegerExpression()
         {
            public integer execute()
            {
               return numEntries(cBuffers);
            }
         };
         ToClause toClause0 = new ToClause(iTable_1, 1, resExpr0);
         
         doTo("loopLabel1", toClause0, new Block((Body) () -> 
         {
            hBuffer.assign(handle.fromString(entry(iTable_1, cBuffers)));
            cExclude /* {fnarg <func-name> <arg> [handle] */ .assign(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "excludeColumns", targetProcedure(), "I", iTable_1));
            cAssignFields.assign(entry(iTable_1, cAssignList, ";"));
            cUpdColumns.assign(entry(iTable_1, cUpdColumnsByTable, ";"));
            cColumns.assign(entry(iTable_1, cColumnsByTable, ";"));
            cNotChanged.assign("");
            
            /* This table has enabled fields, so update it. */
            if (_isNotEqual(cUpdColumns, ""))
            {
               /* If this is an Add or a Copy then create the DB records first. */
               if (_or(isEqual(hRowMod.unwrapBufferField().value(), "A"), () -> isEqual(hRowMod.unwrapBufferField().value(), "C")))
               {
                  OnPhrase[] onPhrase0 = new OnPhrase[]
                  {
                     new OnPhrase(Condition.ERROR, Action.LEAVE, "blockLabel0")
                  };
                  
                  doBlock(TransactionType.SUB, "blockLabel0", onPhrase0, new Block((Body) () -> 
                  {
                     lCreate.assign(hBuffer.unwrapBuffer().bufferCreate());
                  }));
                  
                  /* Probably not very likely that anyone would code trigger errors in a 
                             create trigger (?), as no data has yet been assigned, but we do 
                             support passing a return-value from the trigger as a message to the 
                             client, but an ERROR is required to undo the create, so the trigger
                             would need to have the following error handling. 
                             RETURN ERROR cMessages + "my-error-message". 
                             Regular errors is more likely (sequences out of sequence...eh.)
                             and will be brought to the client if the caller calls 
                             addMessage with unknown as the indicator to use error-status */
                  if (_not(lCreate))
                  {
                     /* Prepend a potential return-value and append the table as the last 
                                  CHR(3) entry */
                     returnNormal(concat(ControlFlowOps.getReturnValue(), (_isEqual(ControlFlowOps.getReturnValue(), "") ? new character("") : chr(3)), entry(iTable_1, cTables)));
                  }
               }
               
               loopLabel2:
               for (iField.assign(1); _isLessThanOrEqual(iField, numEntries(cColumns)); iField.increment())
               {
                  cFromField.assign(entry(iField, cColumns));
                  
                  if (_isGreaterThan(lookup(cFromField, cUpdColumns), 0))
                  {
                     hRowUpdFld.assign(phRowObjUpd_1.unwrapBuffer().bufferField(cFromField));
                     hRowOldFld.assign(hRowBefore.unwrapBuffer().bufferField(cFromField));
                     lChanged.assign(_isEqual(hRowUpdFld.unwrapCommonField().getDataType(), "CHARACTER") ? compare(new character(hRowUpdFld.unwrapBufferField().value()), "<>", new character(hRowOldFld.unwrapBufferField().value()), "case-sensitive") : isNotEqual(hRowUpdFld.unwrapBufferField().value(), hRowOldFld.unwrapBufferField().value()));
                  }
                  else
                  {
                     lChanged.assign(false);
                  }
                  
                  /* The fromfield will be replaced by ToField if different when 
                               looping through the assignFields */
                  if (_not(lChanged))
                  {
                     cNotChanged.assign(concat(cNotChanged, (_isEqual(cNotChanged, "") ? new character("") : new character(",")), cFromField));
                  }
               }
               
               /* Loop through the field mappings in order to identify fields that 
                        must be assigned directly (extents) and in order to rename entries 
                        that are in the notchanged list and remove them from the assinglist.
                        We need two variables; the UseAssignFields is being changed while 
                        AssignFields need to remain unchanged in order to be looped */
               cUseAssignFields.assign(cAssignFields);
               
               loopLabel3:
               for (iField.assign(1); _isLessThanOrEqual(iField, numEntries(cAssignFields)); iField.assign(plus(iField, 2)))
               {
                  cFromField.assign(entry(iField, cAssignFields));
                  cToField.assign(entry(plus(iField, 1), cAssignFields));
                  /* Is the FromField unchanged */
                  iEntry.assign(lookup(cFromField, cNotChanged));
                  /* is this a mapping to an extent */
                  iBracket.assign(indexOf(cToField, "["));
                  ErrorManager.silentErrorEnable();
                  lAmbiguous.assign(and(isGreaterThan(lookup(cToField, cDataColumns), 0), () -> hBuffer.unwrapBuffer().bufferField(cFromField).isValid()));
                  ErrorManager.silentErrorDisable();
                  
                  /* If not changed or if assigned directly below then remove the fields 
                             from the assignlist that will be used in buffer-copy. */
                  if (_or(or(isGreaterThan(iEntry, 0), () -> isGreaterThan(iBracket, 0)), lAmbiguous))
                  {
                     cUseAssignFields.assign(trim(replaceAll(concat(",", cUseAssignFields, ","), concat(",", cFromField, ",", cToField, ","), ","), ","));
                  }
                  
                  /* If bracket then remove the bracketfrom the Tofield for inclusion 
                             in buffer-copy params or for buffer-field(<ToField>) and find the 
                             extent for buffer-value  */
                  if (_isGreaterThan(iBracket, 0))
                  {
                     iExtent.assign(new integer(substring(cToField, plus(iBracket, 1), minus(minus(indexOf(cToField, "]"), iBracket), 1))));
                     cToField.assign(substring(cToField, 1, minus(iBracket, 1)));
                  }
                  else
                  {
                     iExtent.assign(0);
                  }
                  
                  /* If Not Changed or if assigned directly below then add fields 
                             to the not changed list */
                  if (_or(or(isGreaterThan(iEntry, 0), () -> isGreaterThan(iBracket, 0)), lAmbiguous))
                  {
                     /* We need both fields in the exclude list to ensure that the tofield
                                  is not assigned from another rowobject field with the same name 
                                  and to ensure that the fromfield is not assigned to a db field with 
                                  the same name  (The assignFields could in fact always be in the 
                                  excludeList, since the buffer-copy assignList always ignores 
                                  anything in the exclude list, but since we need to remove arrays 
                                  we may as well do it this way)
                                  The check to avoid duplicate entries here is likely to be overkill 
                                  as buffer-copy probably handles multiple entries with less overhead*/
                     if (_isEqual(lookup(cToField, cNotChanged), 0))
                     {
                        /* This will also add extent fields to the exclude list, but that is 
                                       actually a safe guard even if the RowObject does not support them 
                                       as some may have defined the RowObject using LIKE to be able to 
                                       use one SDO against a large table. */
                        cNotChanged.assign(concat(cNotChanged, (_isEqual(cNotChanged, "") ? new character("") : new character(",")), cToField));
                     }
                     
                     /* The fromfield is of course in the list if entry <> 0 */
                     if (_and(isEqual(iEntry, 0), () -> isEqual(lookup(cFromField, cNotChanged), 0)))
                     {
                        cNotChanged.assign(concat(cNotChanged, (_isEqual(cNotChanged, "") ? new character("") : new character(",")), cFromField));
                     }
                  }
                  
                  /* if changed and extent or ambiguous mapping then do an explicit 
                             assign of the ToField from the FromField */
                  if (_and(isEqual(iEntry, 0), () -> or(isGreaterThan(iBracket, 0), lAmbiguous)))
                  {
                     ErrorManager.silentErrorEnable();
                     hRowUpdFld.assign(phRowObjUpd_1.unwrapBuffer().bufferField(cFromField));
                     hDbfield.assign(hBuffer.unwrapBuffer().bufferField(cToField));
                     hDbfield.unwrapBufferField().changeValue(iExtent, hRowUpdFld.unwrapBufferField().value());
                     ErrorManager.silentErrorDisable();
                     
                     if ((ErrorManager.isError()).booleanValue())
                     {
 /* in case something's bad */
                        /* Prepend a potential return-value and append the table as the last 
                                     CHR(3) entry */
                        returnNormal(concat(ControlFlowOps.getReturnValue(), (_isEqual(ControlFlowOps.getReturnValue(), "") ? new character("") : chr(3)), entry(iTable_1, cTables)));
                     }
                  }
               }
               
               ErrorManager.silentErrorEnable();
               /* iField = 1 to num-entries(assignFields) */
               hBuffer.unwrapBuffer().bufferCopy(phRowObjUpd_1, concat(cNotChanged, (_and(isNotEqual(cNotChanged, ""), () -> isNotEqual(cExclude, "")) ? new character(",") : new character("")), cExclude), cUseAssignFields);
               ErrorManager.silentErrorDisable();
               
               if ((ErrorManager.isError()).booleanValue())
               {
 /* in case something's bad */
                  /* Prepend a potential return-value and append the table as the last 
                                       CHR(3) entry */
                  returnNormal(concat(ControlFlowOps.getReturnValue(), (_isEqual(ControlFlowOps.getReturnValue(), "") ? new character("") : chr(3)), entry(iTable_1, cTables)));
               }
            }
            
            /* cUpdColumns <> '':u */
            if (_or(isEqual(hRowMod.unwrapBufferField().value(), "A"), () -> isEqual(hRowMod.unwrapBufferField().value(), "C")))
            {
               ErrorManager.silentErrorEnable();
               /* We reference the ROWID of the new row only AFTER the field values have 
                        been assigned. This is necessary because DataServers create the
                        record as soon as the ROWID is referenced rather than at the end of the 
                        transaction and this may cause errors if for example the fields are 
                        not allowed to have NULL values in the DataServer have not yet been 
                        assigned. */
               /* For DataServers, we need to capture errors when creating the record, 
                          like duplicate key errors, because they will happen when we 
                          reference ROWID */
               cRowid.assign(valueOf(hBuffer.unwrapBuffer().rowID()));
               ErrorManager.silentErrorDisable();
               
               /* If the ROWID for the current buffer is blank, a DataServer 
                          record was not created for some reason. */
               if (_isEqual(cRowid, ""))
               {
                  returnNormal(entry(iTable_1, cTables));
               }
               
               cRowIdent.assign(_isEqual(iTable_1, 1) ? cRowid : concat(cRowIdent, ",", ((hBuffer.unwrapBuffer().available()).booleanValue() ? cRowid : new character(""))));
            }
         }));
         
         /* DO iTable */
         if (_or(isEqual(hRowMod.unwrapBufferField().value(), "A"), () -> isEqual(hRowMod.unwrapBufferField().value(), "C")))
         {
            phRowObjUpd_1.unwrapBuffer().bufferField("RowIdentIdx").unwrapBufferField().changeValue(substring(cRowIdent, 1, xiRocketIndexLimit));
            phRowObjUpd_1.unwrapBuffer().bufferField("RowIdent").unwrapBufferField().changeValue(cRowIdent);
            hRowBefore.unwrapBuffer().deleteRecord();
         }
         
         ErrorManager.silentErrorEnable();
         HandleOps.delete(hRowBefore);
         ErrorManager.silentErrorDisable();
         returnNormal();
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     <br>
    * Parameters:  &lt;none&gt;<br>
    * Notes:
    */
   public void batchAvailable(final character _pcMode)
   {
      character pcMode = TypeFactory.initInput(_pcMode);
      
      internalProcedure(new Block());
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     To group a sequence of SDO service requests into a single request<br>
    *              and thereby minimize network messaging and improve performance.<br>
    * <p>
    * Parameters:<br>
    *   INPUT pcServices - A character string containing a CHR(1) delimitted list of<br>
    *                      SDO internal procedures and functions to be executed.  <br>
    *                      Each entry consists of a CHR(2) delimitted list of INPUT<br>
    *                      PARAMETER values with the first entry being the NAME of<br>
    *                      the procedure or function to be executed.<br>
    *   OUTPUT pcValues  - A character string containing a CHR(1) delimitted list of<br>
    *                      CHR(2) delimitted strings of output values that result <br>
    *                      from the execution of the services listed in pcServices<br>
    *                      (above).  There is a one-to-one correspondence between<br>
    *                      the CHR(1) delimitted list of Services in pcServices <br>
    *                      (above) and the CHR(1) delimitted list in pcValues.  <br>
    *                      Procedures with no output paramters have a NULL entry.<br>
    *                      The return value of functions appear as the first entry <br>
    *                      of the corresponding CHR(2) delimitted list, followed<br>
    *                      by any output parameters.<br>
    * <p>
    * Notes:       As shipped, batchServices only supports a limited list of services.<br>
    *              This list consists of all "get" and "set" functions that don't<br>
    *              require processing other than getting and setting the property in<br>
    *              the ADMprops temp-table record as well as columnProps,<br>
    *              initializeObject, openQuery and synchronizeProperties.<br>
    *              Developers can extend the list by overriding batchServices with a<br>
    *              local version that has an extended list then running SUPER.
    */
   public void batchServices(final character _pcServices, final character pcValues_4)
   {
      character pcServices = TypeFactory.initInput(_pcServices);
      integer iService = TypeFactory.integer();
      integer numServices = TypeFactory.integer();
      character cRequest = TypeFactory.character();
      character cService = TypeFactory.character();
      character cDataType = TypeFactory.character();
      character cValue = TypeFactory.character();
      
      internalProcedure(new Block((Init) () -> 
      {
         TypeFactory.initOutput(pcValues_4);
      }, 
      (Body) () -> 
      {
         numServices.assign(numEntries(pcServices, chr(1)));
         
         ToClause toClause1 = new ToClause(iService, 1, numServices);
         
         doTo("loopLabel4", toClause1, new Block((Body) () -> 
         {
            cRequest.assign(entry(iService, pcServices, chr(1)));
            cService.assign(entry(1, cRequest, chr(2)));
            
            switch (StringHelper.safeTrimTrailing((cService).toStringMessage()).toUpperCase())
            {
               case "COLUMNPROPS":
                  {
                     pcValues_4.assign(concat(pcValues_4, columnProps(new character(entry(2, cRequest, chr(2))), new character(entry(3, cRequest, chr(2))))));
                  }
                  break;
               case "INITIALIZEOBJECT":
                  {
                     ControlFlowOps.invokeIn("initializeObject", targetProcedure());
                  }
                  break;
               case "OPENQUERY":
                  {
                     /* {fn <func-name> [handle] */ 
                     pcValues_4.assign(concat(pcValues_4, valueOf(ControlFlowOps.invokeDynamicFunctionIn(logical.class, "openQuery", targetProcedure()))));
                  }
                  break;
               case "SYNCHRONIZEPROPERTIES":
                  {
                     ControlFlowOps.invokeInWithMode("synchronizeProperties", targetProcedure(), "IO", entry(2, cRequest, chr(2)), cValue);
                     pcValues_4.assign(concat(pcValues_4, cValue));
                  }
                  break;
               default:
                  {
                     /* See if it is a get- or set- property function */
                     if (_or(and(begins(cService, "get"), () -> isEqual(numEntries(cRequest, chr(2)), 1)), () -> and(begins(cService, "set"), () -> isEqual(numEntries(cRequest, chr(2)), 2))))
                     {
                        /* Get the datatype of the property */
                        cDataType.assign(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "propertyType", targetProcedure(), "I", substring(cService, 4)));
                        
                        if (_isUnknown(cDataType))
                        {
 /* It wasn't found */
                           /* {fnarg <func-name> <arg> [handle] */ 
                           ControlFlowOps.invokeInWithMode("addMessage", targetProcedure(), "III", substitute(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "messageNumber", targetProcedure(), "I", 60), cService), new unknown(), new unknown());
                        }
                        else
                        {
                           if (_begins(cService, "s"))
                           {
                              switch (StringHelper.safeTrimTrailing((cDataType).toStringMessage()).toUpperCase())
                              {
                                 case "CHARACTER":
                                    {
                                       pcValues_4.assign(concat(pcValues_4, valueOf(ControlFlowOps.invokeDynamicFunctionInWithMode(cService, targetProcedure(), "I", entry(2, cRequest, chr(2))))));
                                    }
                                    break;
                                 case "INTEGER":
                                    {
                                       pcValues_4.assign(concat(pcValues_4, valueOf(ControlFlowOps.invokeDynamicFunctionInWithMode(cService, targetProcedure(), "I", new integer(entry(2, cRequest, chr(2)))))));
                                    }
                                    break;
                                 case "LOGICAL":
                                    {
                                       pcValues_4.assign(concat(pcValues_4, valueOf(ControlFlowOps.invokeDynamicFunctionInWithMode(cService, targetProcedure(), "I", isEqual(entry(2, cRequest, chr(2)), "yes")))));
                                    }
                                    break;
                                 case "DECIMAL":
                                    {
                                       pcValues_4.assign(concat(pcValues_4, valueOf(ControlFlowOps.invokeDynamicFunctionInWithMode(cService, targetProcedure(), "I", new decimal(entry(2, cRequest, chr(2)))))));
                                    }
                                    break;
                              }
                           }
                           else
                           {
                               /* getProperty case */pcValues_4.assign(concat(pcValues_4, valueOf(ControlFlowOps.invokeDynamicFunctionIn(cService, targetProcedure()))));
                           }
                        }
                     }
                     else
                     {
 /* Not a get or set function */
                        /* {fnarg <func-name> <arg> [handle] */ 
                        ControlFlowOps.invokeInWithMode("addMessage", targetProcedure(), "III", substitute(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "messageNumber", targetProcedure(), "I", 60), cService), new unknown(), new unknown());
                     }
                  }
                  break;
            }
            
            /* CASE on cService */
            if (_isNotEqual(iService, numServices))
            {
               pcValues_4.assign(concat(pcValues_4, chr(1)));
            }
         }));
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     Perform a BUFFER-COPY of a database buffer to a row object buffer.<br>
    *              In particular, if an assign-list is used to map individual array<br>
    *              elements from the database buffer to the row object buffer, this<br>
    *              procedure will ensure that the values are copied properly.<br>
    * Parameters:  INPUT phRowObj AS HANDLE - Handle to the row object buffer that<br>
    *                is to be the target of the BUFFER-COPY.<br>
    *              INPUT phBuffer AS HANDLE - Handle to the database buffer that is<br>
    *                to be the source of the BUFFER-COPY.<br>
    *              INPUT pcExcludes AS CHARACTER - Comma-delimited list of fields to be<br>
    *                excluded from the BUFFER-COPY.<br>
    *              INPUT pcAssigns AS CHARACTER - Comma-delimited list of field pairs to<br>
    *                be individually copied; The field pairs are mappings of fields <br>
    *                from the target/source buffers where the field names differ.<br>
    * Notes:       The primary purpose of this procedure is to detect when individual<br>
    *              array fields are referenced in the assign-list (pcAssigns) from <br>
    *              the database buffer and to ensure that they are copied properly.<br>
    *              It is assumed that the database buffer is always the source of the<br>
    *              BUFFER-COPY, so that the second field in the assign-list field pair<br>
    *              is always where the individual array reference will be found <br>
    *              (e.g., pcAssigns = "ROfld1,DBfld[1]").
    */
   public void bufferCopyDbtoRo(final handle _phRowObj, final handle _phBuffer_1, final character _pcExcludes_1, final character _pcAssigns_1)
   {
      handle phRowObj = TypeFactory.initInput(_phRowObj);
      handle phBuffer_1 = TypeFactory.initInput(_phBuffer_1);
      character pcExcludes_1 = TypeFactory.initInput(_pcExcludes_1);
      character pcAssigns_1 = TypeFactory.initInput(_pcAssigns_1);
      handle hField = TypeFactory.handle();
      handle hField2 = TypeFactory.handle();
      character cTemp = TypeFactory.character();
      integer iCtr = TypeFactory.integer();
      integer iExt = TypeFactory.integer();
      integer iPos = TypeFactory.integer();
      
      internalProcedure(new Block((Body) () -> 
      {
         /* If the assign-list (cAssigns) contains any individual array fields, we
                  can not use BUFFER-COPY.  We must assign each field (pair) manually. */
         if (_isNotEqual(indexOf(pcAssigns_1, "["), 0))
         {
            /* Copy everything but the assign-list fields. */
            phRowObj.unwrapBuffer().bufferCopy(phBuffer_1, pcExcludes_1);
            
            loopLabel5:
            for (iCtr.assign(2); _isLessThanOrEqual(iCtr, numEntries(pcAssigns_1)); iCtr.assign(plus(iCtr, 2)))
            {
               cTemp.assign(entry(iCtr, pcAssigns_1));
               /* DB field */
               iPos.assign(indexOf(cTemp, "["));
               
               /* Position of left bracket */
               if (_isGreaterThan(iPos, 0))
               {
                  iExt.assign(new integer(entry(1, substring(cTemp, plus(iPos, 1)), "]")));
                  cTemp.assign(substring(cTemp, 1, minus(iPos, 1)));
               }
 /* Get extent value and base name for array field */               else
               {
                  iExt.assign(0);
               }
               
               hField.assign(phRowObj.unwrapBuffer().bufferField(entry(minus(iCtr, 1), pcAssigns_1)));
               hField2.assign(phBuffer_1.unwrapBuffer().bufferField(cTemp));
               hField.unwrapBufferField().changeValue(hField2.unwrapBufferField().value(iExt));
            }
         }
         else
         {
            phRowObj.unwrapBuffer().bufferCopy(phBuffer_1, pcExcludes_1, pcAssigns_1);
         }
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     Does a buffer-compare of the current RowObjUpd row to the<br>
    *              database records it is derived from. If any fields have been<br>
    *              changed the the name of the first table with changed fields<br>
    *              is returned in the RETURN-VALUE of the procedure.<br>
    * <p>
    * Parameters:  &lt;none&gt;<br>
    * <p>
    * Notes:       The dynamic buffer-compare does not return the names of fields<br>
    *              which were changed, so this information is not available to put<br>
    *              into a message. The procedure returns the first table name which<br>
    *              has fields which don't match.
    */
   public void compareDbrow()
   {
      handle hBuffer = TypeFactory.handle();
      character cBuffers = TypeFactory.character();
      integer iTable_1 = TypeFactory.integer();
      handle hRowObjUpd = TypeFactory.handle();
      character cAssigns = TypeFactory.character();
      character cRowIdent = TypeFactory.character();
      character cExclude = TypeFactory.character();
      logical lSame = TypeFactory.logical();
      
      internalProcedure(new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("BufferHandles"));
         cBuffers.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("AssignList"));
         cAssigns.assign(ghProp.unwrapBufferField().value());
         /* List of fields whose names were changed */
         /* {get Prop var [handle]} */ 
         hRowObjUpd.assign(ControlFlowOps.invokeDynamicFunctionIn(handle.class, "getRowObjUpd", targetProcedure()));
         cRowIdent.assign(hRowObjUpd.unwrapBuffer().bufferField("RowIdent").unwrapBufferField().value());
         
         IntegerExpression resExpr1 = new IntegerExpression()
         {
            public integer execute()
            {
               return numEntries(cBuffers);
            }
         };
         ToClause toClause2 = new ToClause(iTable_1, 1, resExpr1);
         
         doTo("loopLabel6", toClause2, new Block((Body) () -> 
         {
            if (_isNotEqual(entry(iTable_1, cRowIdent), ""))
            {
               hBuffer.assign(handle.fromString(entry(iTable_1, cBuffers)));
               /* don't compare with field form other tables that have the same 
                          name, but are not used */
               cExclude /* {fnarg <func-name> <arg> [handle] */ .assign(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "excludeColumns", targetProcedure(), "I", iTable_1));
               
               if (_isEqual(indexOf(cAssigns, "["), 0))
               {
                  lSame.assign(hBuffer.unwrapBuffer().bufferCompare(hRowObjUpd, new character("case-sensitive"), cExclude, entry(iTable_1, cAssigns, ";")));
               }
               else
               {
                   /* If array elements are present, we must assign them manually. */lSame.assign(bufferCompareDbtoRo(hRowObjUpd, hBuffer, cExclude, new character(entry(iTable_1, cAssigns, ";"))));
               }
               
               if (_not(lSame))
               {
                  ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
                  ghProp.assign(ghProp.unwrapBuffer().bufferField("Tables"));
                  cBuffers.assign(ghProp.unwrapBufferField().value());
                  returnNormal(entry(iTable_1, cBuffers));
               }
            }
 /* allow for outer join w/ no rec */;
         }));
         
         /* END DO iTable */
         returnNormal();
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    *   Purpose: Checks the state of all data-targets too se if it's ok to commit.<br>
    *            It's not OK to commit if a data-target is Modified or the <br>
    *            in these case the I-O parameter should return cancel = TRUE. <br>
    *            The visual objects (visual.p) will, however, offer the user the <br>
    *            opportunity to save/cancelRecord in order to be able to commit.   <br>
    *                   <br>
    * Parameters: INPUT-OUTPUT  pioCancel (logical) <br>
    *                  Will return true if it's NOT ok to commit.  <br>
    *   Notes:
    */
   public void confirmCommit(final logical pioCancel)
   {
      internalProcedure(new Block((Body) () -> 
      {
         /* don't ask data-targets if already cancelled */
         if (_not(pioCancel))
         {
            NamedEventManager.publish("confirmCommit", targetProcedure(), "U", new Object[]
            {
               pioCancel
            });
         }
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    *   Purpose: Checks the state of all data-targets too se if it's ok to continue.<br>
    *            It's not OK to continue if a data-target is Modified or the <br>
    *            RowUpdateState equals 'rowUpdated', in these case the I-O parameter <br>
    *            should return cancel = TRUE. The visual objects (visual.p) <br>
    *            will, however, offer the user the opportunity to save/cancelRecord or <br>
    *            commit/undo in order to be able to continue.   <br>
    *                   <br>
    * Parameters: INPUT-OUTPUT  pioCancel (logical) <br>
    *                  Will return true if it's NOT ok to continue.  <br>
    *   Notes:   This method should be called from any method that may change the <br>
    *            result set somewhere in the data-source chain like openQuery or <br>
    *            navigation actions.<br>
    *            Currently called from the filter-source to see if new criteria can <br>
    *            be applied.        <br>
    *            (Currently the Navigation actions are disabled whenever a state<br>
    *             that may disallow continuation is set to true, but if a less modal <br>
    *             dialog could be achieved by calling this from fetch* methods.)
    */
   public void confirmContinue(final logical pioCancel)
   {
      internalProcedure(new Block((Body) () -> 
      {
         /* don't ask data-targets if already cancelled */
         if (_not(pioCancel))
         {
            NamedEventManager.publish("confirmContinue", targetProcedure(), "U", new Object[]
            {
               pioCancel
            });
         }
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    *   Purpose: Checks the state of all data-targets too se if it's ok to Undo.<br>
    *            It's not OK to undo if a data-target is Modified or in AddMode.  <br>
    *            In these case the I-O parameter should return cancel = TRUE. <br>
    *            The visual objects (visual.p) will, however, warn the user <br>
    *            that unsaved changes will be cancelled.   <br>
    *                   <br>
    * Parameters: INPUT-OUTPUT  pioCancel (logical) <br>
    *                  Will return true if it's NOT ok to commit.  <br>
    *   Notes:
    */
   public void confirmUndo(final logical pioCancel)
   {
      internalProcedure(new Block((Body) () -> 
      {
         /* don't ask data-targets if already cancelled */
         if (_not(pioCancel))
         {
            NamedEventManager.publish("confirmUndo", targetProcedure(), "U", new Object[]
            {
               pioCancel
            });
         }
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     <br>
    * Parameters:  &lt;none&gt;<br>
    * Notes:
    */
   public void createObjects()
   {
      integer iLoop = TypeFactory.integer();
      handle hBuffer = TypeFactory.handle();
      handle hQuery = TypeFactory.handle();
      character cBufferHandles = TypeFactory.character();
      character cDbnames = TypeFactory.character();
      logical lStatic = TypeFactory.logical();
      character cTables = TypeFactory.character();
      character cPhysicalTables = TypeFactory.character();
      character cTable = TypeFactory.character();
      character cPhysical = TypeFactory.character();
      character cQueryContext = TypeFactory.character();
      logical lNewQuery = TypeFactory.logical();
      
      internalProcedure(new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("Tables"));
         cTables.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("PhysicalTables"));
         cPhysicalTables.assign(ghProp.unwrapBufferField().value());
         
         if (_isEqual(cTables, ""))
         {
            returnNormal();
         }
         
         if (_isEqual(cPhysicalTables, ""))
         {
            cPhysicalTables.assign(cTables);
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("PhysicalTables"));
            ghProp.unwrapBufferField().changeValue(cPhysicalTables);
         }
         
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryHandle"));
         hQuery.assign(ghProp.unwrapBufferField().value());
         
         if ((hQuery.isValid()).booleanValue())
         {
            lStatic.assign(isGreaterThan(lookup("STATIC", hQuery.unwrapADMData().getADMData(), chr(1)), 0));
         }
         
         if (_not(lStatic))
         {
            ErrorManager.silentErrorEnable();
            HandleOps.delete(hQuery);
            ErrorManager.silentErrorDisable();
            QueryWrapper.createQuery(hQuery);
            lNewQuery.assign(true);
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryHandle"));
            ghProp.unwrapBufferField().changeValue(hQuery);
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("BufferHandles"));
            cBufferHandles.assign(ghProp.unwrapBufferField().value());
            
            loopLabel7:
            for (iLoop.assign(1); _isLessThanOrEqual(iLoop, numEntries(cBufferHandles)); iLoop.increment())
            {
               hBuffer.assign(handle.fromString(entry(iLoop, cBufferHandles)));
               HandleOps.delete(hBuffer);
            }
            
            IntegerExpression resExpr2 = new IntegerExpression()
            {
               public integer execute()
               {
                  return numEntries(cTables);
               }
            };
            ToClause toClause3 = new ToClause(iLoop, 1, resExpr2);
            
            doTo("loopLabel8", toClause3, new Block((Body) () -> 
            {
               cTable.assign(entry(iLoop, cTables));
               cPhysical.assign(entry(iLoop, cPhysicalTables));
               hBuffer.assign(ControlFlowOps.invokeDynamicFunctionInWithMode(handle.class, "createBuffer", targetProcedure(), "II", entry(numEntries(cTable, "."), cTable, "."), cPhysical));
               
               if (_isEqual(iLoop, 1))
               {
                  ErrorManager.silentErrorEnable();
                  hQuery.unwrapQuery().setBuffers(hBuffer);
                  ErrorManager.silentErrorDisable();
                  
                  /* Static query */
                  if (_isEqual(ErrorManager.getErrorNumber(1), 7317))
                  {
                     HandleOps.delete(hBuffer);
                     leave("loopLabel8");
                  }
               }
               else
               {
                  hQuery.unwrapQuery().addBuffer(hBuffer);
               }
            }));
         }
         cBufferHandles.setUnknown();
         
         loopLabel9:
         for (iLoop.assign(1); _isLessThanOrEqual(iLoop, hQuery.unwrapQuery().numBuffers()); iLoop.increment())
         {
            hBuffer.assign(hQuery.unwrapQuery().bufferHandle(iLoop));
            cBufferHandles.assign(concat((_isEqual(iLoop, 1) ? new character("") : concat(cBufferHandles, ",")), valueOf(hBuffer)));
            cDbnames.assign(concat(cDbnames, (_isEqual(iLoop, 1) ? new character("") : new character(",")), _notUnknown(hBuffer.unwrapDatabaseInfo().getDbName()) ? hBuffer.unwrapDatabaseInfo().getDbName() : new character("?")));
         }
         
         /* iTable = 1 to num-buffers */
         /* {set Prop value [handle]} */ 
         ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "setDBNames", targetProcedure(), "I", cDbnames);
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("BufferHandles"));
         ghProp.unwrapBufferField().changeValue(cBufferHandles);
         
         /* setQueryWhere will store data in QueryContext if called early (context) */
         if (_isGreaterThan(hQuery.unwrapQuery().numBuffers(), 0))
         {
            /* {get Prop var [handle]} */ 
            cQueryContext.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getQueryContext", targetProcedure()));
            
            if (_or(isGreaterThan(cQueryContext, ""), lNewQuery))
            {
               /* {fnarg <func-name> <arg> [handle] */ 
               ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "prepareQuery", targetProcedure(), "I", cQueryContext);
               /* {set Prop value [handle]} */ 
               ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "setQueryContext", targetProcedure(), "I", "");
            }
         }
         returnNormal();
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     dataAvailable is an event procedure that generates a dependent <br>
    *              query dynamically based on the ForeignFields property.  This <br>
    *              event occurs when a Data-Source publishes "dataAvailable" because<br>
    *              it has new data available and a dependant object need to react.<br>
    * <p>
    * Parameters:<br>
    *   INPUT pcRelative - a flag decribing the newly available record.<br>
    * <p>
    *       SAME           - The current record is being resent because it has been <br>
    *                         updated.  This procedure ignores this type.<br>
    *       VALUE-CHANGED - A target SmartDataObject has changed the position.  <br>
    *                         This procedure needs to set QueryPosition property <br>
    *                         then change pcRelative to "DIFFERENT" before passing <br>
    *                         it along to other target procedures so as to appear as<br>
    *                         if the change occured in this procedure.<br>
    *        RESET        - Request to reset statuses and foreignfields and refresh<br>
    *                       visual objects and panels in all objects in the data-link <br>
    *                       chain. This was introduced in 9.1C because in some cases <br>
    *                       'same' does too little and 'different' too much. <br>
    *                       It achieves the same as 'different' except that queries <br>
    *                       will only be reopened if foreign values has changed.   <br>
    *        DIFFERENT    - Request to reapply foreign Fields and reopen the query.<br>
    *        TRANSFER     - Signal to call the open client fetchFirstBatch API. <br>
    *                       Called internally from a parent that has a ReadHandler <br>
    *                       defined that receives records as they are transferred <br>
    *                       from the database.<br>
    *  <br>
    *        <br>
    *        FIRST        - <br>
    *        NEXT         -<br>
    *        PREV         -<br>
    *        LAST         -<br>
    * <p>
    * Notes:       This is a special version of DataAvailable for SmartDataObjects<br>
    *              with a dependency on another SmartDataObject. So the code is <br>
    *              different from that for a Viewer, for example. If there are no <br>
    *              ForeignFields, then it is being run because of a reposition in <br>
    *              one of its targets (a SmartDataBrowser, normally), in which <br>
    *              case it just passes on the event to other targets.
    */
   public void dataAvailable(final character _pcRelative)
   {
      character pcRelative = TypeFactory.initInput(_pcRelative);
      character cForeignFields = TypeFactory.character();
      character cSourceFields = TypeFactory.character("");
      character cLocalFields = TypeFactory.character("");
      character cValues = TypeFactory.character();
      handle hDataSource = TypeFactory.handle();
      integer iField = TypeFactory.integer();
      character cQuery = TypeFactory.character();
      character cWhere = TypeFactory.character();
      handle hQuery = TypeFactory.handle();
      character cPosition = TypeFactory.character();
      integer iCurrent = TypeFactory.integer();
      character cNewPos = TypeFactory.character();
      logical lNewSource = TypeFactory.logical();
      logical lNewModeSource = TypeFactory.logical();
      character cAsdivision = TypeFactory.character();
      character cCurrentValues = TypeFactory.character();
      logical lFetched = TypeFactory.logical();
      logical lNewMode = TypeFactory.logical();
      logical lOneToOne = TypeFactory.logical();
      handle hSourceRow = TypeFactory.handle();
      integer iCount = TypeFactory.integer();
      handle hNewQuery = TypeFactory.handle();
      character cNewBatchInfo = TypeFactory.character();
      
      internalProcedure(new Block((Body) () -> 
      {
         /* If the same row is being resent because of an update, we don't want to react 
             to that, so just skip it if it's not a different row. */
         if (_isEqual(pcRelative, "SAME"))
         {
            returnNormal();
         }
         
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryHandle"));
         hQuery.assign(ghProp.unwrapBufferField().value());
         /* Points to the database query */
         /* {get Prop var [handle]} */ 
         cForeignFields.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getForeignFields", targetProcedure()));
         
         /* Get the foreign fields, which map the Data Source's RowObject fields
              to this object's database fields and prepare the Query
              using that value. setQueryWhere will open the query. */
         if (_or(isEqual(cForeignFields, ""), () -> isEqual(pcRelative, "VALUE-CHANGED")))
         {
            /* If there were no ForeignFields, a Browser has repositioned the query.
                   Just pass along the event to other targets. Set QueryPosition
                   property first if that has changed. */
            /* {get Prop var [handle]} */ 
            cPosition.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getQueryPosition", targetProcedure()));
            /* current First/Last etc. before repos */
            iCurrent.assign(hQuery.unwrapQuery().currentRow());
            
            /* *new* current row. */
            if (_and(isEqual(cPosition, "FirstRecord"), () -> isNotEqual( /* moved away from first */iCurrent, 1)))
            {
               cNewPos.assign(_isEqual(pcRelative, "Last") ? new character("LastRecord") : new character("NotFirstOrLast"));
            }
            else
            {
               if (_isEqual(cPosition, "NotFirstOrLast"))
               {
                   /* moved to first/last */cNewPos.assign(_isEqual(iCurrent, 1) ? new character("FirstRecord") : _isEqual(pcRelative, "Last") ? new character("LastRecord") : new character(""));
               }
               else
               {
                  if (_and(and(isEqual(cPosition, "LastRecord"), () -> isNotEqual( /* moved away from last */pcRelative, "Last")), () -> isNotEqual(pcRelative, "Next")))
                  {
                     cNewPos.assign(_isEqual(iCurrent, 1) ? new character("FirstRecord") : new character("NotFirstOrLast"));
                  }
               }
            }
            
            if (_isNotEqual(cNewPos, ""))
            {
               /* {set Prop value [handle]} */ 
                /* We changed relative position...*/ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "setQueryPosition", targetProcedure(), "I", cNewPos);
            }
            
            if (_isEqual(pcRelative, "VALUE-CHANGED"))
            {
               pcRelative.assign("DIFFERENT");
            }
            
            /* Now let other targets (if any) know about the new position. */
            NamedEventManager.publish("dataAvailable", targetProcedure(), "I", new Object[]
            {
               pcRelative
            });
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("NewBatchInfo"));
            ghProp.unwrapBufferField().changeValue(new character(""));
            returnNormal();
         }
         /* END DO IF FF = "" */
         /* {get Prop var [handle]} */ 
         hDataSource.assign(ControlFlowOps.invokeDynamicFunctionIn(handle.class, "getDataSource", targetProcedure()));
         
         if ((hDataSource.isValid()).booleanValue())
         {
            ghTargetProcedure.assign(targetProcedure());
            /* NewRow is true also when saved an uncommitted */
            /* {get Prop var [handle]} */
            lNewSource.assign(ControlFlowOps.invokeDynamicFunctionIn(logical.class, "getNewRow", hDataSource));
            ghTargetProcedure.assign(targetProcedure());
            /* just in case ..*/
            /* NewMode is only true when in addmode (closeQuery below) */
            /* {get Prop var [handle]} */
            lNewModeSource.assign(ControlFlowOps.invokeDynamicFunctionIn(logical.class, "getNewMode", hDataSource));
            ghTargetProcedure.setUnknown();
         }
         
         loopLabel10:
         for (iField.assign(1); _isLessThanOrEqual(iField, numEntries(cForeignFields)); iField.assign(plus(iField, 2)))
         {
            cLocalFields.assign(concat(cLocalFields, (_isNotEqual(cLocalFields, "") ? new character(",") : new character("")), entry(iField, cForeignFields)));
            cSourceFields.assign(concat(cSourceFields, (_isNotEqual(cSourceFields, "") ? new character(",") : new character("")), entry(plus(iField, 1), cForeignFields)));
         }
         
         ghTargetProcedure.assign(targetProcedure());
         ErrorManager.silentErrorEnable();
         cValues /* {fnarg <func-name> <arg> [handle] */.assign(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "colValues", hDataSource, "I", cSourceFields));
         ErrorManager.silentErrorDisable();
         ghTargetProcedure.setUnknown();
         
         /* Throw away the RowIdent entry returned by colValues*/
         if (_notUnknown(cValues))
         {
            cValues.assign(substring(cValues, plus(indexOf(cValues, chr(1)), 1)));
         }
         
         /* NOT (lNewModeSoure = FALSE) to include ? when source closed */
         /* cValues = ? indicated No row available in the Source.  */
         /* If we are in new mode then we cannot close the query 
              This may happen when a commit fails in an SBO (one to one particularly)  */
         /* {get Prop var [handle]} */ 
         lNewMode.assign(ControlFlowOps.invokeDynamicFunctionIn(logical.class, "getNewMode", targetProcedure()));
         
         if (_or(and(not(lNewMode), () -> not(isEqual(lNewModeSource, false))), () -> isUnknown(cValues)))
         {
            /* {fn <func-name> [handle] */ 
            ControlFlowOps.invokeDynamicFunctionIn(logical.class, "closeQuery", targetProcedure());
         }
         
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ForeignValues"));
         cCurrentValues.assign(ghProp.unwrapBufferField().value());
         
         /* If an outside object manages the temp-tables it can set this to true 
              to avoid reopening the query here (sbo.p fetchContainedRows)
              We only check this if we are sure the foreignfield are the same  */
         if (_isEqual(cCurrentValues, cValues))
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("DataIsFetched"));
            lFetched.assign(ghProp.unwrapBufferField().value());
         }
         /* {get Prop var [handle]} */ 
         
         /* Save FF values for later querying, except when 'RESET'
              RESET indicates that ForeignValues has NOT changed, but it may also be 
              from a DataContainer during initialization when openOnInit is false,
              so we must NOT set the value. The data.p override uses hasForeignKeyChanged
              to check whether a 'RESET' can be passed up here, so it must match with 
              the values that id used to open the query. If new source we need to store 
              it as in this case foreignvalues has changed */
         /* child of previously initialized dataSource (This may need another look) */
         /* child of previously initialized dataSource (This may need another look) */
         if (_or(or(isNotEqual(pcRelative, "RESET"), lNewSource), () -> and(hDataSource.isValid(), () -> new logical(ControlFlowOps.invokeDynamicFunctionIn(logical.class, "getObjectInitialized", hDataSource)))))
         {
            /* {get Prop var [handle]} */ 
            lOneToOne.assign(ControlFlowOps.invokeDynamicFunctionIn(logical.class, "getUpdateFromSource", targetProcedure()));
            /* {get Prop var [handle]} */ 
            cAsdivision.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getASDivision", targetProcedure()));
            
            /* If this is a 1-to-1 SDO, get the RowObject of its DataSource and retrieve all */
            /* related records by creating a temporary join to the parent's RowObject */
            if (_and(lOneToOne, () -> isNotEqual(cAsdivision, "CLIENT")))
            {
               /* {get Prop var [handle]} */
               hSourceRow.assign(ControlFlowOps.invokeDynamicFunctionIn(handle.class, "getRowObject", hDataSource));
               ghProp.assign(handle.fromString(entry(1, hDataSource.unwrapADMData().getADMData(), chr(1))));
               ghProp.assign(ghProp.unwrapBuffer().bufferField("NewBatchInfo"));
               cNewBatchInfo.assign(ghProp.unwrapBufferField().value());
               RecordBuffer.create(hSourceRow, hSourceRow, "SourceRow");
               
               loopLabel11:
               for (iCount.assign(1); _isLessThanOrEqual(iCount, numEntries(cLocalFields)); iCount.increment())
               {
                  cWhere.assign(concat((_isGreaterThan(iCount, 1) ? concat(cWhere, " AND ") : new character("")), entry(iCount, cLocalFields), " = SourceRow.", entry(iCount, cSourceFields)));
               }
               
               if (_and(and(isGreaterThan(cNewBatchInfo, ""), () -> notUnknown(cNewBatchInfo)), () -> isNotEqual(cNewBatchInfo, "ALL")))
               {
                  cQuery.assign(concat("FOR EACH SourceRow WHERE ", cNewBatchInfo, ", "));
               }
               else
               {
                  cQuery.assign("FOR EACH SourceRow, ");
               }
               
               cQuery.assign(concat(cQuery, replaceAll(hQuery.unwrapQuery().prepareString(), "FOR EACH ", "FIRST ")));
               cQuery.assign(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "newWhereClause", targetProcedure(), "IIII", hQuery.unwrapQuery().bufferHandle(1).unwrap().name(), cWhere, cQuery, "AND"));
               /*           cQuery = REPLACE(cQuery, " WHERE ":U, " OUTER-JOIN WHERE ":U).  */
               /* set up a temporary 1-to-1 query */
               QueryWrapper.createQuery(hNewQuery);
               hNewQuery.unwrapQuery().addBuffer(hSourceRow);
               
               loopLabel12:
               for (iCount.assign(1); _isLessThanOrEqual(iCount, hQuery.unwrapQuery().numBuffers()); iCount.increment())
               {
                  hNewQuery.unwrapQuery().addBuffer(hQuery.unwrapQuery().bufferHandle(iCount));
               }
               
               hNewQuery.unwrapQuery().prepare(cQuery);
               ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
               ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryHandle"));
               ghProp.unwrapBufferField().changeValue(hNewQuery);
            }
            else
            {
               ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
               ghProp.assign(ghProp.unwrapBuffer().bufferField("ForeignValues"));
               ghProp.unwrapBufferField().changeValue(cValues);
               ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "assignQuerySelection", targetProcedure(), "III", cLocalFields, cValues, "");
            }
         }
         
         if (_isEqual(pcRelative, "TRANSFER"))
         {
            ControlFlowOps.invokeIn("fetchFirstBatch", targetProcedure());
         }
         else
         {
            if (_or(or(or(isEqual(pcRelative, "RESET"), () -> isEqual(lFetched, true)), lNewSource), () -> isUnknown(cValues)))
            {
               ControlFlowOps.invokeIn("updateQueryPosition", targetProcedure());
               /* From 9.1C we republish 'reset' also when lFetched = true,
                      in 9.1B we did not republish in this case as 'reset' did not exist.*/
               NamedEventManager.publish("DataAvailable", targetProcedure(), "I", new Object[]
               {
                  "RESET"
               });
               ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
               ghProp.assign(ghProp.unwrapBuffer().bufferField("NewBatchInfo"));
               ghProp.unwrapBufferField().changeValue(new character(""));
            }
            else
            {
               /* {fn <func-name> [handle] */ 
               ControlFlowOps.invokeDynamicFunctionIn(logical.class, "openQuery", targetProcedure());
            }
         }
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryHandle"));
         ghProp.unwrapBufferField().changeValue(hQuery);
         ErrorManager.silentErrorEnable();
         HandleOps.delete(hSourceRow);
         ErrorManager.silentErrorDisable();
         ErrorManager.silentErrorEnable();
         HandleOps.delete(hNewQuery);
         ErrorManager.silentErrorDisable();
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DataIsFetched"));
         ghProp.unwrapBufferField().changeValue(new logical(false));
         returnNormal();
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     <br>
    * Parameters:  &lt;none&gt;<br>
    * Notes:
    */
   public void destroyObject()
   {
      handle hQuery = TypeFactory.handle();
      integer iBuffer = TypeFactory.integer();
      
      internalProcedure(new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryHandle"));
         hQuery.assign(ghProp.unwrapBufferField().value());
         
         /* dynamic */
         if (_and(hQuery.isValid(), () -> isEqual(hQuery.unwrap().name(), "")))
         {
            loopLabel13:
            for (iBuffer.assign(1); _isLessThanOrEqual(iBuffer, hQuery.unwrapQuery().numBuffers()); iBuffer.increment())
            {
               ErrorManager.silentErrorEnable();
               HandleOps.delete(hQuery.unwrapQuery().bufferHandle(iBuffer));
               ErrorManager.silentErrorDisable();
            }
            
            ErrorManager.silentErrorEnable();
            HandleOps.delete(hQuery);
            ErrorManager.silentErrorDisable();
         }
         
         ControlFlowOps.runSuper("destroyObject");
         returnNormal();
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     <br>
    * Parameters:  &lt;none&gt;<br>
    * Notes:
    */
   public void fetchCurrentBatch()
   {
      integer iRowsReturned = TypeFactory.integer();
      logical lFillBatch = TypeFactory.logical();
      handle hDataQuery = TypeFactory.handle();
      
      internalProcedure(new Block((Body) () -> 
      {
         /* {set Prop value [handle]} */ 
         ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "setFetchOnOpen", targetProcedure(), "I", "");
         /* close query to avoid openQuery wiping out all batchprops */
         /* {get Prop var [handle]} */ 
         hDataQuery.assign(ControlFlowOps.invokeDynamicFunctionIn(handle.class, "getDataHandle", targetProcedure()));
         hDataQuery.unwrapQuery().queryClose();
         /* {fn <func-name> [handle] */ 
         ControlFlowOps.invokeDynamicFunctionIn(logical.class, "openQuery", targetProcedure());
         /* {fnarg <func-name> <arg> [handle] */ 
         ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "emptyRowObject", targetProcedure(), "I", "");
         /* {get Prop var [handle]} */ 
         lFillBatch.assign(ControlFlowOps.invokeDynamicFunctionIn(logical.class, "getFillBatchOnRepos", targetProcedure()));
         ControlFlowOps.invokeInWithMode("transferRows", targetProcedure(), "IIIIIO", new logical(false), new character("Current"), new character(""), lFillBatch, new unknown(), iRowsReturned);
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     Retrieves with EXCLUSIVE-LOCK the database records associated <br>
    *              with the current RowObjUpd record, as identified by a list of <br>
    *              RowIds in the RowIdent field of that record.<br>
    *              IF the RowMod field is "D" for Delete, deletes the DB record(s).<br>
    * <p>
    * Parameters:  &lt;none&gt;<br>
    * <p>
    * Notes:       If one or more of the database records cannot be locked (for any<br>
    *              reason - most likely because it is locked by another user, but<br>
    *              perhaps because it has been deleted), the table name of the <br>
    *              first unlockable record is returned as the return value.
    */
   public void fetchDbrowForUpdate()
   {
      integer iTable_1 = TypeFactory.integer();
      handle hRowObjUpd = TypeFactory.handle();
      rowid rRowid = TypeFactory.rowid();
      handle hBuffer = TypeFactory.handle();
      character cBuffers = TypeFactory.character();
      character cTables = TypeFactory.character();
      handle hRowIdent = TypeFactory.handle();
      character cRowIdent = TypeFactory.character();
      handle hRowMod = TypeFactory.handle();
      logical lDelSuccess = TypeFactory.logical();
      handle hQuery = TypeFactory.handle();
      character cUpdTbls = TypeFactory.character();
      integer iDelCnt = TypeFactory.integer();
      logical lFirstRecDeleted = TypeFactory.logical();
      logical lSchemaValidate = TypeFactory.logical();
      handle hLogicObject = TypeFactory.handle();
      character cDelMessage = TypeFactory.character();
      character cAsDivision = TypeFactory.character();
      rowid[] rRowidList = TypeFactory.rowidExtent(18);
      
      internalProcedure(new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("BufferHandles"));
         cBuffers.assign(ghProp.unwrapBufferField().value());
         /* DB Buffer handles. */
         /* {get Prop var [handle]} */ 
         hRowObjUpd.assign(ControlFlowOps.invokeDynamicFunctionIn(handle.class, "getRowObjUpd", targetProcedure()));
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("Tables"));
         cTables.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("UpdatableColumnsByTable"));
         cUpdTbls.assign(ghProp.unwrapBufferField().value());
         hRowIdent.assign(hRowObjUpd.unwrapBuffer().bufferField("RowIdent"));
         cRowIdent.assign(hRowIdent.unwrapBufferField().value());
         hRowMod.assign(hRowObjUpd.unwrapBuffer().bufferField("RowMod"));
         
         loopLabel14:
         for (iTable_1.assign(1); _isLessThanOrEqual(iTable_1, numEntries(cRowIdent)); iTable_1.increment())
         {
            hBuffer.assign(handle.fromString(entry(iTable_1, cBuffers)));
            
            if (_isNotEqual(entry(iTable_1, cRowIdent), ""))
            {
               rRowid.assign(new rowid(entry(iTable_1, cRowIdent)));
               
               if (_not(hBuffer.unwrapBuffer().findByRowID(rRowid, LockType.EXCLUSIVE_NO_WAIT)))
               {
                  if (_and(isEqual(hRowMod.unwrapBufferField().value(), "D"), () -> isGreaterThan(iTable_1, 1)))
                  {
                     continue loopLabel14;
                  }
                  else
                  {
                     returnNormal(entry(iTable_1, cTables));
                  }
               }
               
               if (_isEqual(hRowMod.unwrapBufferField().value(), "D"))
               {
                  /* Table has no enabled fields, so it's not updatable
                               and can't be deleted. */
                  if (_isEqual(entry(iTable_1, cUpdTbls, ";"), ""))
                  {
                     continue loopLabel14;
                  }
                  else
                  {
                     String[] enclBlocks0 = new String[]
                     {
                        "loopLabel14"
                     };
                     OnPhrase[] onPhrase1 = new OnPhrase[]
                     {
                        new OnPhrase(Condition.ERROR, Action.LEAVE, "blockLabel1")
                     };
                     
                     doBlock(TransactionType.SUB, enclBlocks0, "blockLabel1", onPhrase1, new Block((Body) () -> 
                     {
                        ErrorManager.silentErrorEnable();
                        lDelSuccess.assign(hBuffer.unwrapBuffer().deleteRecord());
                        ErrorManager.silentErrorDisable();
                        /* The above statement does not support schema delete validation,
                                     so check for error 7347. */
                        lSchemaValidate.assign(isEqual(ErrorManager.getErrorNumber(1), 7347));
                        
                        /* The DO ON ERROR will ensure that the RETURN below sends message 
                                     7374 back to the user if no static delete function is found. 
                                     (There will also be an error message HERE (server log) about no 
                                      existing functions, but no-error is intentionally avoided on the 
                                      function calls to ensure that the error status from the delete 
                                     (for example the val msg) is intact */
                        if ((lSchemaValidate).booleanValue())
                        {
                           OnPhrase[] onPhrase2 = new OnPhrase[]
                           {
                              new OnPhrase(Condition.ERROR, Action.LEAVE, "blockLabel2")
                           };
                           
                           doBlock(TransactionType.SUB, "blockLabel2", onPhrase2, new Block((Body) () -> 
                           {
                              /* {get Prop var [handle]} */ 
                              hLogicObject.assign(ControlFlowOps.invokeDynamicFunctionIn(handle.class, "getDataLogicObject", targetProcedure()));
                              
                              if ((hLogicObject.isValid()).booleanValue())
                              {
                                 /* Run Delete<Table>static in Logic Procedure */
                                 lDelSuccess.assign(ControlFlowOps.invokeDynamicFunctionInWithMode(concat("delete", entry(iTable_1, cTables), "Static"), targetProcedure(), "I", rRowid));
                              }
                              else
                              {
                                  /* Run Delete<Table>static in Logic Procedure */lDelSuccess.assign(ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "deleteRecordStatic", targetProcedure(), "I", iTable_1));
                              }
                           }));
                        }
                     }));
                     
                     if (deferredLeave("loopLabel14"))
                        break loopLabel14;
                     if (deferredNext("loopLabel14"))
                        continue loopLabel14;
                  }
                  
                  if (_not(isEqual(lDelSuccess, true)))
                  {
                     cDelMessage.assign(_isNotEqual(ControlFlowOps.getReturnValue(), "") ? ControlFlowOps.getReturnValue() : ErrorManager.getErrorText(1));
                     /* Signal that Delete failed. Support return-value from triggers */
                     returnNormal(concat(entry(iTable_1, cTables), (_isNotEqual(cDelMessage, "") ? concat(",", cDelMessage) : new character("")), ",Delete"));
                  }
                  else
                  {
                     if (_isEqual(iTable_1, 1))
                     {
                        /* If you delete the first record in a join, then you must
                                       always delete the result list entry and remove the row
                                       from the query. */
                        lFirstRecDeleted.assign(true);
                     }
                     
                     iDelCnt.assign(plus(iDelCnt, 1));
                  }
               }
            }
 /* allow for outer-join w/ no rec*/            else
            {
               ErrorManager.silentErrorEnable();
               hBuffer.unwrapBuffer().bufferRelease();
               ErrorManager.silentErrorDisable();
            }
         }
         
         /* END DO iTable */
         /* sendRows currently does not reopen the query when running locally, so 
              we do a delete-result-list-entry so transferRows does not find this 
              row in the result list when determining where to start retrieving records 
              for the next batch. */
         /* {get Prop var [handle]} */ 
         cAsDivision.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getAsDivision", targetProcedure()));
         
         if (_isEqual(cAsDivision, ""))
         {
            /* Only delete result list entry if all records in the row are deleted,
                 or if the first record in the row is deleted. */
            if (_or(lFirstRecDeleted, () -> isEqual(iDelCnt, numEntries(cRowIdent))))
            {
               ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
               ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryHandle"));
               hQuery.assign(ghProp.unwrapBufferField().value());
               
               loopLabel15:
               for (iTable_1.assign(1); _isLessThanOrEqual(iTable_1, numEntries(cRowIdent)); iTable_1.increment())
               {
                  ErrorManager.silentErrorEnable();
                  assignSingle(rRowidList, iTable_1, new rowid(entry(iTable_1, cRowIdent)));
                  ErrorManager.silentErrorDisable();
               }
               
               if ((hQuery.unwrapQuery().queryRepositionByID(subscript(rRowidList, 1), subscript(rRowidList, 2), subscript(rRowidList, 3), subscript(rRowidList, 4), subscript(rRowidList, 5), subscript(rRowidList, 6), subscript(rRowidList, 7), subscript(rRowidList, 8), subscript(rRowidList, 9), subscript(rRowidList, 10), subscript(rRowidList, 11), subscript(rRowidList, 12), subscript(rRowidList, 13), subscript(rRowidList, 14), subscript(rRowidList, 15), subscript(rRowidList, 16), subscript(rRowidList, 17), subscript(rRowidList, 18))).booleanValue())
               {
                  hQuery.unwrapResultListHandler().deleteResultListEntry();
               }
            }
         }
         returnNormal();
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:      Repositions the database query to the first row.<br>
    * <p>
    * Parameterss:  &lt;none&gt;<br>
    * <p>
    * Notes:        Requests rows to be transferred from the database query<br>
    *               if the RowObject query is empty.
    */
   public void fetchFirst()
   {
      handle hQuery = TypeFactory.handle();
      handle hBuffer = TypeFactory.handle();
      character cRowIdent = TypeFactory.character();
      logical lSkipRowIdent = TypeFactory.logical();
      rowid[] rRowIds = TypeFactory.rowidExtent(19);
      integer iRow = TypeFactory.integer();
      logical lOk = TypeFactory.logical();
      
      internalProcedure(new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryHandle"));
         hQuery.assign(ghProp.unwrapBufferField().value());
         hBuffer.assign(hQuery.unwrapQuery().bufferHandle(1));
         
         if (_not(hQuery.unwrapQuery().isOpen()))
         {
             /* NOTE: This points to the database Query */hQuery.unwrapQuery().queryOpen();
         }
         
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryRowIdent"));
         cRowIdent.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryRowIdent"));
         ghProp.unwrapBufferField().changeValue(new unknown());
         
         /* Immediately reset once we have the value. */
         if (_and(notUnknown(cRowIdent), () -> isNotEqual(cRowIdent, "")))
         {
            /* {get Prop var [handle]} */ 
            lSkipRowIdent.assign(ControlFlowOps.invokeDynamicFunctionIn("getSkipQueryRowIdent", targetProcedure()));
            /* {set Prop value [handle]} */ 
            ControlFlowOps.invokeDynamicFunctionInWithMode("setSkipQueryRowIdent", targetProcedure(), "I", false);
            /* Immediately reset  */
            assignUnknown(rRowIds);
            
            loopLabel16:
            for (iRow.assign(1); _isLessThanOrEqual(iRow, numEntries(cRowIdent)); iRow.increment())
            {
               assignSingle(rRowIds, iRow, new rowid(entry(iRow, cRowIdent)));
            }
            
            /* END DO iRow */
            lOk.assign(hQuery.unwrapQuery().queryRepositionByID(subscript(rRowIds, 1), subscript(rRowIds, 2), subscript(rRowIds, 3), subscript(rRowIds, 4), subscript(rRowIds, 5), subscript(rRowIds, 6), subscript(rRowIds, 7), subscript(rRowIds, 8), subscript(rRowIds, 9), subscript(rRowIds, 10), subscript(rRowIds, 11), subscript(rRowIds, 12), subscript(rRowIds, 13), subscript(rRowIds, 14), subscript(rRowIds, 15), subscript(rRowIds, 16), subscript(rRowIds, 17), subscript(rRowIds, 18), subscript(rRowIds, 19)));
            
            if ((lOk).booleanValue())
            {
               hQuery.unwrapQuery().getNext();
               
               if ((lSkipRowIdent).booleanValue())
               {
                  hQuery.unwrapQuery().getNext();
               }
            }
         }
         else
         {
            ErrorManager.silentErrorEnable();
            hQuery.unwrapQuery().getFirst();
            ErrorManager.silentErrorDisable();
         }
         
         if ((hBuffer.unwrapBuffer().available()).booleanValue())
         {
            /* {set Prop value [handle]} */ 
             /*  ...unless no records satisfy the query. */ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "setQueryPosition", targetProcedure(), "I", "FirstRecord");
         }
         else
         {
            /* {set Prop value [handle]} */ 
            ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "setQueryPosition", targetProcedure(), "I", "NoRecordAvailable");
         }
         
         NamedEventManager.publish("dataAvailable", targetProcedure(), "I", new Object[]
         {
            "FIRST"
         });
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("NewBatchInfo"));
         ghProp.unwrapBufferField().changeValue(new character(""));
         returnNormal();
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     <br>
    * Parameters:  &lt;none&gt;<br>
    * Notes:
    */
   public void fetchFirstBatch()
   {
      integer iRowsReturned = TypeFactory.integer();
      
      internalProcedure(new Block((Body) () -> 
      {
         /* {set Prop value [handle]} */ 
         ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "setFetchOnOpen", targetProcedure(), "I", "");
         /* {fn <func-name> [handle] */ 
         ControlFlowOps.invokeDynamicFunctionIn(logical.class, "openQuery", targetProcedure());
         ControlFlowOps.invokeInWithMode("transferRows", targetProcedure(), "IIIIIO", new logical(false), new character("FIRST"), new character(""), new logical(false), new unknown(), iRowsReturned);
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     Repositions the database query to the last row.<br>
    * <p>
    * Parameters:  &lt;none&gt;
    */
   public void fetchLast()
   {
      handle hQuery = TypeFactory.handle();
      handle hBuffer = TypeFactory.handle();
      logical lHidden = TypeFactory.logical();
      
      internalProcedure(new Block((Body) () -> 
      {
         /* IF this object is currently hidden, then just return. This effectively
                makes the link inactive. */
         /* {get Prop var [handle]} */ 
         lHidden.assign(ControlFlowOps.invokeDynamicFunctionIn(logical.class, "getObjectHidden", targetProcedure()));
         
         if ((lHidden).booleanValue())
         {
            returnNormal();
         }
         
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryHandle"));
         hQuery.assign(ghProp.unwrapBufferField().value());
         
         if (_not(hQuery.unwrapQuery().isOpen()))
         {
            returnNormal();
         }
         
         hBuffer.assign(hQuery.unwrapQuery().bufferHandle(1));
         hQuery.unwrapQuery().getLast();
         
         if ((hBuffer.unwrapBuffer().available()).booleanValue())
         {
            /* {set Prop value [handle]} */ 
            ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "setQueryPosition", targetProcedure(), "I", "LastRecord");
         }
         
         /* Signal row change in any case. */
         NamedEventManager.publish("dataAvailable", targetProcedure(), "I", new Object[]
         {
            "LAST"
         });
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("NewBatchInfo"));
         ghProp.unwrapBufferField().changeValue(new character(""));
         returnNormal();
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     <br>
    * Parameters:  &lt;none&gt;<br>
    * Notes:
    */
   public void fetchLastBatch()
   {
      integer iRowsReturned = TypeFactory.integer();
      
      internalProcedure(new Block((Body) () -> 
      {
         /* {set Prop value [handle]} */ 
         ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "setFetchOnOpen", targetProcedure(), "I", "");
         /* {fn <func-name> [handle] */ 
         ControlFlowOps.invokeDynamicFunctionIn(logical.class, "openQuery", targetProcedure());
         ControlFlowOps.invokeInWithMode("transferRows", targetProcedure(), "IIIIIO", new logical(false), new character("LAST"), new character(""), new logical(false), new unknown(), iRowsReturned);
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     Repositions the database query to the next row.<br>
    * <p>
    * Parameters:  &lt;none&gt;
    */
   public void fetchNext()
   {
      handle hQuery = TypeFactory.handle();
      handle hBuffer = TypeFactory.handle();
      logical lHidden = TypeFactory.logical();
      integer iCnt = TypeFactory.integer();
      character cLastDbRowId = TypeFactory.character();
      character cQueryPos = TypeFactory.character();
      character cRow = TypeFactory.character();
      
      internalProcedure(new Block((Body) () -> 
      {
         /* IF this object is currently hidden, then just return. This effectively
                makes the link inactive. */
         /* {get Prop var [handle]} */ 
         lHidden.assign(ControlFlowOps.invokeDynamicFunctionIn(logical.class, "getObjectHidden", targetProcedure()));
         
         if ((lHidden).booleanValue())
         {
            returnNormal();
         }
         
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryHandle"));
         hQuery.assign(ghProp.unwrapBufferField().value());
         
         if (_not(hQuery.unwrapQuery().isOpen()))
         {
            returnNormal();
         }
         
         hBuffer.assign(hQuery.unwrapQuery().bufferHandle(1));
         /* {get Prop var [handle]} */ 
         cQueryPos.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getQueryPosition", targetProcedure()));
         
         /* Return if already at end. */
         if (_isEqual(cQueryPos, "LastRecord"))
         {
            returnNormal();
         }
         
         hQuery.unwrapQuery().getNext();
         
         if (_not(hBuffer.unwrapBuffer().available()))
         {
            /* No data so reget the last row. */
            hQuery.unwrapQuery().getLast();
            /* {set Prop value [handle]} */ 
            ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "setQueryPosition", targetProcedure(), "I", "LastRecord");
         }
         else
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("LastDbRowIdent"));
            cLastDbRowId.assign(ghProp.unwrapBufferField().value());
            
            loopLabel17:
            for (iCnt.assign(1); _isLessThanOrEqual(iCnt, hQuery.unwrapQuery().numBuffers()); iCnt.increment())
            {
               hBuffer.assign(hQuery.unwrapQuery().bufferHandle(iCnt));
               cRow.assign(concat(cRow, (_isNotEqual(cRow, "") ? new character(",") : new character("")), valueOf(hBuffer.unwrapBuffer().rowID())));
            }
            
            /* do iCnt 1 to num-buffers */
            if (_isEqual(cLastDbRowId, cRow))
            {
               /* {set Prop value [handle]} */ 
               ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "setQueryPosition", targetProcedure(), "I", "LastRecord");
            }
            else
            {
               if (_isEqual(cQueryPos, "FirstRecord"))
               {
                  /* {set Prop value [handle]} */ 
                  ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "setQueryPosition", targetProcedure(), "I", "NotFirstOrLast");
               }
            }
         }
         /* else do */
         /* Signal row change in any case. */
         NamedEventManager.publish("dataAvailable", targetProcedure(), "I", new Object[]
         {
            "NEXT"
         });
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("NewBatchInfo"));
         ghProp.unwrapBufferField().changeValue(new character(""));
         returnNormal();
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     <br>
    * Parameters:  &lt;none&gt;<br>
    * Notes:
    */
   public void fetchNextBatch()
   {
      integer iRowsReturned = TypeFactory.integer();
      logical lFillBatch = TypeFactory.logical();
      handle hDataQuery = TypeFactory.handle();
      
      internalProcedure(new Block((Body) () -> 
      {
         /* {set Prop value [handle]} */ 
         ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "setFetchOnOpen", targetProcedure(), "I", "");
         /* {get Prop var [handle]} */ 
         hDataQuery.assign(ControlFlowOps.invokeDynamicFunctionIn(handle.class, "getDataHandle", targetProcedure()));
         /* close query to avoid openQuery wiping out all batchprops */
         hDataQuery.unwrapQuery().queryClose();
         /* {fn <func-name> [handle] */ 
         ControlFlowOps.invokeDynamicFunctionIn(logical.class, "openQuery", targetProcedure());
         /* {fnarg <func-name> <arg> [handle] */ 
         ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "emptyRowObject", targetProcedure(), "I", "");
         /* {get Prop var [handle]} */ 
         lFillBatch.assign(ControlFlowOps.invokeDynamicFunctionIn(logical.class, "getFillBatchOnRepos", targetProcedure()));
         ControlFlowOps.invokeInWithMode("transferRows", targetProcedure(), "IIIIIO", new logical(false), new character("Next"), new character(""), lFillBatch, new unknown(), iRowsReturned);
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     Repositions the database query to the previous row.<br>
    * <p>
    * Parameters:  &lt;none&gt;
    */
   public void fetchPrev()
   {
      handle hQuery = TypeFactory.handle();
      handle hBuffer = TypeFactory.handle();
      logical lHidden = TypeFactory.logical();
      character cQueryPos = TypeFactory.character();
      
      internalProcedure(new Block((Body) () -> 
      {
         /* IF this object is currently hidden, then just return. This effectively
                makes the link inactive. */
         /* {get Prop var [handle]} */ 
         lHidden.assign(ControlFlowOps.invokeDynamicFunctionIn(logical.class, "getObjectHidden", targetProcedure()));
         
         if ((lHidden).booleanValue())
         {
            returnNormal();
         }
         
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryHandle"));
         hQuery.assign(ghProp.unwrapBufferField().value());
         hBuffer.assign(hQuery.unwrapQuery().bufferHandle(1));
         
         if (_not(hQuery.unwrapQuery().isOpen()))
         {
            returnNormal();
         }
         
         hQuery.unwrapQuery().getPrevious();
         
         /* If we have moved to the first record or away from the last, then
                signal a change of record position state. */
         if ((hQuery.unwrapQuery().isOffEnd()).booleanValue())
         {
            hQuery.unwrapQuery().getFirst();
            /* {set Prop value [handle]} */ 
            ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "setQueryPosition", targetProcedure(), "I", "FirstRecord");
         }
         else
         {
            /* {get Prop var [handle]} */ 
            cQueryPos.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getQueryPosition", targetProcedure()));
            
            if (_isEqual(cQueryPos, "LastRecord"))
            {
               /* {set Prop value [handle]} */ 
               ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "setQueryPosition", targetProcedure(), "I", "NotFirstOrLast");
            }
            
            if (_isEqual(hQuery.unwrapQuery().currentRow(), 1))
            {
               /* {set Prop value [handle]} */ 
               ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "setQueryPosition", targetProcedure(), "I", "FirstRecord");
            }
         }
         /* ELSE DO IF NOT OFF-END */
         /* Signal row change in any case. */
         NamedEventManager.publish("dataAvailable", targetProcedure(), "I", new Object[]
         {
            "PREV"
         });
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("NewBatchInfo"));
         ghProp.unwrapBufferField().changeValue(new character(""));
         returnNormal();
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     <br>
    * Parameters:  &lt;none&gt;<br>
    * Notes:
    */
   public void fetchPrevBatch()
   {
      integer iRowsReturned = TypeFactory.integer();
      logical lFillBatch = TypeFactory.logical();
      handle hDataQuery = TypeFactory.handle();
      
      internalProcedure(new Block((Body) () -> 
      {
         /* {set Prop value [handle]} */ 
         ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "setFetchOnOpen", targetProcedure(), "I", "");
         /* {get Prop var [handle]} */ 
         hDataQuery.assign(ControlFlowOps.invokeDynamicFunctionIn(handle.class, "getDataHandle", targetProcedure()));
         /* close query to avoid openQuery wiping out all batchprops */
         hDataQuery.unwrapQuery().queryClose();
         /* {fn <func-name> [handle] */ 
         ControlFlowOps.invokeDynamicFunctionIn(logical.class, "openQuery", targetProcedure());
         /* {fnarg <func-name> <arg> [handle] */ 
         ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "emptyRowObject", targetProcedure(), "I", "");
         /* {get Prop var [handle]} */ 
         lFillBatch.assign(ControlFlowOps.invokeDynamicFunctionIn(logical.class, "getFillBatchOnRepos", targetProcedure()));
         ControlFlowOps.invokeInWithMode("transferRows", targetProcedure(), "IIIIIO", new logical(false), new character("Prev"), new character(""), lFillBatch, new unknown(), iRowsReturned);
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     Adds the Filter link between itself and a Filter container.  <br>
    *              Called from startFilter after the Filter container is <br>
    *              contructed.  <br>
    * Parameters:  phFilterContainer AS HANDLE - handle of the Filter container<br>
    * Notes:       The code to add the Filter link has been separated from <br>
    *              startFilter so that filterContainerHandler can be overridden<br>
    *              to add other links between this object and the Filter container.
    */
   public void filterContainerHandler(final handle _phFilterContainer)
   {
      handle phFilterContainer = TypeFactory.initInput(_phFilterContainer);
      
      internalProcedure(new Block((Body) () -> 
      {
         ControlFlowOps.invokeInWithMode("addLink", targetProcedure(), "III", phFilterContainer, new character("Filter"), targetProcedure());
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    *   Purpose:    Retrieve entityDetails from the Repository and set properties<br>
    *               from it.    <br>
    *   Parameters: &lt;none&gt; <br>
    *   Notes:      This is currently only called if asdivision &lt;&gt; 'client'. <br>
    *               'Client' proxys get these properties from the server as first <br>
    *               time props.<br>
    * Note Date: 2002/07/07
    */
   public void initializeEntityDetails()
   {
      character cKeyTableEntityFields = TypeFactory.character();
      character cKeyTableEntityValues = TypeFactory.character();
      character cKeyTableEntityMnemonic = TypeFactory.character();
      character cKeyTableEntityObjField = TypeFactory.character();
      logical lHasObjectField = TypeFactory.logical();
      integer iLookup = TypeFactory.integer();
      logical lAuditEnabled = TypeFactory.logical();
      logical lFetchHasAudit = TypeFactory.logical();
      logical lFetchHasComment = TypeFactory.logical();
      logical lFetchAutoComment = TypeFactory.logical();
      logical lHasAudit = TypeFactory.logical();
      logical lHasComment = TypeFactory.logical();
      logical lHasAutoComment = TypeFactory.logical();
      character cEntityFields = TypeFactory.character();
      character cKeyTable = TypeFactory.character();
      /*********************************************************************
      * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
      * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
      * below.  All Rights Reserved.                                       *
      *                                                                    *
      * The Initial Developer of the Original Code is PSC.  The Original   *
      * Code is Progress IDE code released to open source December 1, 2000.*
      *                                                                    *
      * The contents of this file are subject to the Possenet Public       *
      * License Version 1.0 (the "License"); you may not use this file     *
      * except in compliance with the License.  A copy of the License is   *
      * available as of the date of this notice at                         *
      * http://www.possenet.org/license.html                               *
      *                                                                    *
      * Software distributed under the License is distributed on an "AS IS"*
      * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
      * should refer to the License for the specific language governing    *
      * rights and limitations under the License.                          *
      *                                                                    *
      * Contributors:                                                      *
      *                                                                    *
      *********************************************************************/
      /* af/sup2/afcheckerr.i - to check for errors and handle them.
         This include file takes the following arguments:
         {&display-error}     YES = Display the error if not in a transaction
         {&return-only}       YES = Do not pass on the error but do a return
         {&return-error}      YES = Pass on error - defaults to this if in a transaction
         {&no-return}         YES = Do not return when finished - let programmer decide what to do
         {&errors-not-zero}   YES = Build messages even if ERROR-STATUS:ERROR = NO
         {&ignore-errorlist}  YES = do not use get-message errors at all (use with care)
         {&define-only}       YES = define the variables you need but take no other action
      
         If they are displaying an error message, then these arguements are also available:
         {&message-type}      Type of message, e.g. 'MES','INF','WAR','ERR','HAL','QUE', default = 'ERR'
         {&message-buttons}   Comma list of buttons, default is 'OK'
         {&button-default}    Default button, default is 'OK'
         {&button-cancel}     Cancel button, default is 'OK'
         {&message-title}     Title, default is ''
         {&display-empty}     YES = display message dialog even if empty, default = YES
      
         And if message type is QUE for Question then these arguments are also available:
         {&message-datatype}  Datatype, default is 'Character'
         {&message-format}    Format of question, default = 'CHR(35)'
         {&default-answer}    Default answer, default is empty
         Also, for questions, the other defaults change to message-buttons = 'OK,CANCEL', and
         button-default = 'OK', and button-cancel = 'CANCEL'.
      
         RULES:
         1. Required logical arguments must be passed in unquoted as YES or NO. Other text
            arguments must be single quoted literals, e.g. 'text' or unquoted variables, 
            and if the literals require spaces, they should be double quoted then single
            quoted, e.g. "'text'".
         2. All arguments may be left blank - in which cased the error will not be displayed and will be passed on with a return error.
         3. If everything is left blank except display-error = YES, then the error will be
            displayed using the Astra dialog and a standard return statement done.
         4. If in a transaction, the error will be passed on regardless, unless they say no-return = YES.
         5. If they display the error, the default is to do a return only as already displayed error.
         6. If they do not display the error, the default is to do a return error to pass errr on.
         7. If return-only is set to YES, this will override any other return settings.
      
         Following the include file if they said not to return, then the following variable
         values will be available for the programmer:
         cMessageList           CHR(3) delimited list of error messages
         cMessageButton         Button Pressed if message displayed
         cMessageAnswer         Answer if question dialog used
      */
      /* set up defaults if arguements not passed in */
      /* Default to use get-message list of no return-value */
      /* Default to actually do a return */
      /* Default to not return error if displaying errors */
      /* Default to not return error if displaying errors */
      /* Default to return only when display errors */
      /* Default to message type of Error */
      /* Default buttons to 'OK' for normal show messages */
      /* Default buttons is 'OK' for normal show messages */
      /* Cancel buttons is 'OK' for normal show messages */
      /* Default to display an empty dialog */
      /* Default to not merely defining variables */
      integer iErrorLoop = TypeFactory.integer();
      character cMessageList = TypeFactory.character();
      character cMessageButton = TypeFactory.character();
      character cMessageAnswer = TypeFactory.character();
      
      internalProcedure(new Block((Body) () -> 
      {
         if ((gshGenManager.isValid()).booleanValue())
         {
            /* {get Prop var [handle]} */ 
            cKeyTable.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getEnabledTables", targetProcedure()));
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("FetchHasComment"));
            lFetchHasComment.assign(ghProp.unwrapBufferField().value());
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("FetchHasAudit"));
            lFetchHasAudit.assign(ghProp.unwrapBufferField().value());
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("FetchAutoComment"));
            lFetchAutoComment.assign(ghProp.unwrapBufferField().value());
            /* First enabled .. */
            cKeyTable.assign(entry(1, cKeyTable));
            
            /* if no enabled use the first table */
            if (_isEqual(cKeyTable, ""))
            {
               cKeyTable.assign(entry(1, ControlFlowOps.invokeDynamicFunctionIn(character.class, "getTables", targetProcedure())));
            }
            
            if (_isNotEqual(cKeyTable, ""))
            {
               cKeyTable.assign(concat(cKeyTable, ",entity_mnemonic,table_has_object_field,entity_object_field,entity_key_field", ((lFetchHasAudit).booleanValue() ? new character(",HasAudit") : new character("")), ((lFetchHasComment).booleanValue() ? new character(",HasComment") : new character("")), ((lFetchAutoComment).booleanValue() ? new character(",HasAutoComment") : new character("")), ",auditing_enabled"));
               ErrorManager.silentErrorEnable();
               ControlFlowOps.invokeInWithMode("getEntityDetail", gshGenManager, "IOO", cKeyTable, cKeyTableEntityFields, cKeyTableEntityValues);
               ErrorManager.silentErrorDisable();
               
               if (_or(ErrorManager.isError(), () -> isNotEqual(ControlFlowOps.getReturnValue(), "")))
               {
                  cMessageList.assign("");
                  cMessageButton.assign("");
                  cMessageAnswer.assign("");
                  
                  if (_or(ErrorManager.isError(), () -> isNotEqual(ControlFlowOps.getReturnValue(), "")))
                  {
                     cMessageList.assign(_notUnknown(ControlFlowOps.getReturnValue()) ? ControlFlowOps.getReturnValue() : new character(""));
                     
                     if (_isEqual(cMessageList, ""))
                     {
                        loopLabel18:
                        for (iErrorLoop.assign(1); _isLessThanOrEqual(iErrorLoop, ErrorManager.numErrors()); iErrorLoop.increment())
                        {
                           cMessageList.assign(concat(cMessageList, (_isEqual(cMessageList, "") ? new character("") : chr(3)), ErrorManager.getErrorText(iErrorLoop), chr(4), chr(4)));
                        }
                     }
                     
                     ErrorManager.silentErrorEnable();
                     cMessageList.assign(cMessageList);
                     ErrorManager.silentErrorDisable();
                     
                     /* display error if they said to do so and not in a transaction */
                     if (_not(TransactionManager.isTransactionActive()))
                     {
                        /* use ShowMessages */
                        if ((gshSessionManager.isValid()).booleanValue())
                        {
                           ControlFlowOps.invokeInWithMode("showMessages", gshSessionManager, "IIIIIIIIO", cMessageList, new character("ERR"), new character("OK"), new character("OK"), new character("OK"), new character(""), new logical(true), new unknown(), cMessageButton);
                        }
                        else
                        {
                           if (_isNotEqual(cMessageList, ""))
                           {
                              messageBox(replaceAll(cMessageList, chr(3), chr(13)), ALERT_MESSAGE, BTN_OK, "");
                           }
                        }
                     }
                     
                     /* now deal with what they wanted us to do afterwards */
                     /* if in a transaction, cannot display error so pass it on unless they said do nothing */
                     if (TransactionManager.isTransaction())
                     {
                        returnError(cMessageList);
                     }
                     
                     returnNormal();
                     
                     /*Cancel any wait state if error occurs*/
                     if (_isNotEqual(LogicalTerminal.getWaitState(), ""))
                     {
                        if ((LogicalTerminal.setWaitState("")).booleanValue())
                        {
                           processEvents();
                        }
                     }
                  }
               }
               
               if (_isNotEqual(cKeyTableEntityFields, ""))
               {
                  iLookup.assign(lookup("entity_mnemonic", cKeyTableEntityFields, chr(1)));
                  
                  if (_isGreaterThan(iLookup, 0))
                  {
                     cKeyTableEntityMnemonic.assign(entry(iLookup, cKeyTableEntityValues, chr(1)));
                  }
                  
                  iLookup.assign(lookup("table_has_object_field", cKeyTableEntityFields, chr(1)));
                  
                  if (_isGreaterThan(iLookup, 0))
                  {
                     lHasObjectField.assign(matchesList("TRUE,YES", entry(iLookup, cKeyTableEntityValues, chr(1))));
                  }
                  
                  iLookup.assign(lookup("auditing_enabled", cKeyTableEntityFields, chr(1)));
                  
                  if (_isGreaterThan(iLookup, 0))
                  {
                     lAuditEnabled.assign(matchesList("TRUE,YES", entry(iLookup, cKeyTableEntityValues, chr(1))));
                  }
                  
                  if ((lHasObjectField).booleanValue())
                  {
                     if (_isNotEqual(lookup("entity_object_field", cKeyTableEntityFields, chr(1)), 0))
                     {
                        cKeyTableEntityObjField.assign(entry(lookup("entity_object_field", cKeyTableEntityFields, chr(1)), cKeyTableEntityValues, chr(1)));
                     }
                     else
                     {
                        if (_isNotEqual(lookup("entity_key_field", cKeyTableEntityFields, chr(1)), 0))
                        {
                           cKeyTableEntityObjField.assign(entry(lookup("entity_key_field", cKeyTableEntityFields, chr(1)), cKeyTableEntityValues, chr(1)));
                        }
                     }
                  }
                  iLookup.assign((lFetchHasAudit).booleanValue() ? lookup("HasAudit", cKeyTableEntityFields, chr(1)) : new integer(0));
                  
                  if (_isGreaterThan(iLookup, 0))
                  {
                     lHasAudit.assign(matchesList("TRUE,YES", entry(iLookup, cKeyTableEntityValues, chr(1))));
                  }
                  
                  iLookup.assign((lFetchHasComment).booleanValue() ? lookup("HasComment", cKeyTableEntityFields, chr(1)) : new integer(0));
                  
                  if (_isGreaterThan(iLookup, 0))
                  {
                     lHasComment.assign(matchesList("TRUE,YES", entry(iLookup, cKeyTableEntityValues, chr(1))));
                  }
                  
                  iLookup.assign((lFetchAutoComment).booleanValue() ? lookup("HasAutoComment", cKeyTableEntityFields, chr(1)) : new integer(0));
                  
                  if (_isGreaterThan(iLookup, 0))
                  {
                     lHasAutoComment.assign(matchesList("TRUE,YES", entry(iLookup, cKeyTableEntityValues, chr(1))));
                  }
                  
                  cEntityFields.assign(concat(((lHasAudit).booleanValue() ? new character("HasAudit,") : new character("")), ((lHasComment).booleanValue() ? new character("HasComment,") : new character("")), ((lHasAutoComment).booleanValue() ? new character("AutoComment") : new character(""))));
                  /* {set Prop value [handle]} */ 
                  ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "setKeyTableId", targetProcedure(), "I", cKeyTableEntityMnemonic);
                  /* {set Prop value [handle]} */ 
                  ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "setKeyFields", targetProcedure(), "I", cKeyTableEntityObjField);
                  ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
                  ghProp.assign(ghProp.unwrapBuffer().bufferField("AuditEnabled"));
                  ghProp.unwrapBufferField().changeValue(lAuditEnabled);
               }
            }
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("EntityFields"));
            ghProp.unwrapBufferField().changeValue(cEntityFields);
         }
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     Initialization code for query objects.<br>
    * <p>
    * Parameters:  &lt;none&gt;<br>
    * <p>
    * Notes:       The query is not opened if we have a DataSource. This is because<br>
    *              a DataSource means the object is dependent on data from another<br>
    *              object. In this case the Data-Target (this procedure) waits<br>
    *              until the dataAvailable event tells it to open its query. <br>
    *              Instead, this procedure RUNs dataAvailable to see if its <br>
    *              DataSource is already running with a row available for it;<br>
    *              in that case the dataAvailable will open the query.
    */
   public void initializeObject()
   {
      handle hDataSource = TypeFactory.handle();
      handle hContainerSrc = TypeFactory.handle();
      logical lQueryContainer = TypeFactory.logical();
      character cUibmode = TypeFactory.character();
      logical lOpenOnInit = TypeFactory.logical();
      logical lQuery = TypeFactory.logical();
      character cAsDivision = TypeFactory.character();
      logical lIsFetchPending = TypeFactory.logical();
      logical lHideOnInit = TypeFactory.logical();
      character cEntityFields = TypeFactory.character();
      
      internalProcedure(new Block((Body) () -> 
      {
         /* Don't open query in design mode  */
         /* {get Prop var [handle]} */ 
         cAsDivision.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getAsDivision", targetProcedure()));
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("HideOnInit"));
         lHideOnInit.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("EntityFields"));
         cEntityFields.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("UIBMode"));
         cUibmode.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("OpenOnInit"));
         lOpenOnInit.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ContainerSource"));
         hContainerSrc.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ObjectInitialized"));
         ghProp.unwrapBufferField().changeValue(new logical(true));
         
         /**
           RUN SUPER.
           **/
         if (_not(lHideOnInit))
         {
            /* The Objecthidden is set and used also for non-visual objects */
            /* {set Prop value [handle]} */ 
            ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "setObjectHidden", targetProcedure(), "I", false);
            NamedEventManager.publish("LinkState", targetProcedure(), "I", new Object[]
            {
               "active"
            });
         }
         else
         {
            /* The Objecthidden is set and used also for non-visual objects */
            /* {set Prop value [handle]} */ 
            ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "setObjectHidden", targetProcedure(), "I", true);
            NamedEventManager.publish("LinkState", targetProcedure(), "I", new Object[]
            {
               "inactive"
            });
         }
         
         /* retrieve entitydetails on the first call on server */
         if (_and(and(gshGenManager.isValid(), () -> isNotEqual(cAsDivision, "CLIENT")), () -> isUnknown(cEntityFields)))
         {
            ControlFlowOps.invokeIn("initializeEntityDetails", targetProcedure());
         }
         
         if ((hContainerSrc.isValid()).booleanValue())
         {
            if (_isEqual(cAsDivision, "CLIENT"))
            {
               lIsFetchPending /* {fn <func-name> [handle] */.assign(ControlFlowOps.invokeDynamicFunctionIn(logical.class, "IsFetchPending", hContainerSrc));
            }
            
            ghProp.assign(handle.fromString(entry(1, hContainerSrc.unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryObject"));
            lQueryContainer.assign(ghProp.unwrapBufferField().value());
         }
         
         if (_not(begins(cUibmode, "Design")))
         {
            /* {get Prop var [handle]} */ 
            hDataSource.assign(ControlFlowOps.invokeDynamicFunctionIn(handle.class, "getDataSource", targetProcedure()));
            
            /* Also, don't open the query or run dataavailable if we're inside a 
                  container such as an SBO; in that case it gets our data for us. */
            if (_and(not(isEqual(lQueryContainer, true)), not(lIsFetchPending)))
            {
               ErrorManager.silentErrorEnable();
               ghProp.assign(handle.fromString(entry(1, hDataSource.unwrapADMData().getADMData(), chr(1))));
               ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryObject"));
               lQuery.assign(ghProp.unwrapBufferField().value());
               ErrorManager.silentErrorDisable();
               
               if ((lOpenOnInit).booleanValue())
               {
                  if (_or(isUnknown(hDataSource), not(lQuery)))
                  {
                     /* {fn <func-name> [handle] */ 
                     ControlFlowOps.invokeDynamicFunctionIn(logical.class, "openQuery", targetProcedure());
                  }
                  else
                  {
                     ControlFlowOps.invokeInWithMode("dataAvailable", targetProcedure(), "I", new unknown());
                  }
               }
            }
            else
            {
               if (_and(and(hDataSource.isValid(), () -> new logical(ControlFlowOps.invokeDynamicFunctionIn(logical.class, "getObjectInitialized", hDataSource))), () -> isEqual(cAsDivision, "CLIENT")))
               {
                  /* child of previously initialized dataSource. */
                  ControlFlowOps.invokeInWithMode("dataAvailable", targetProcedure(), "I", new character("RESET"));
               }
            }
         }
         returnNormal();
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:    Obtain one entry of column, operator and value to add to a query <br>
    *             for a specific buffer from comma separated lists of columns<br>
    *             buffers and values    <br>
    * Parameters: pcBuffer          - Buffer name to check against <br>
    *                                 - 'RowObject' means SDO query which means that <br>
    *                                    all columns will be returned with the <br>
    *                                    rowObject name. (see NOTES below) <br>
    *             piColumn          - Entry to lookup in the lists<br>
    *             pcColumnList      - Comma separated list of columns <br>
    *                                 (see assignQuerySelection )<br>
    *             pcValueList       - CHR(1) separated list of values <br>
    *             pcOperatorList    - Comma separate list of operators <br>
    *                                 or single operator for all columns             <br>
    *                                 The single operator can have an optional <br>
    *                                 string operator 'ge/BEGINS <br>
    *             output pcColumn   - Column name, resolved with correct <br>
    *                                 qualification. <br>
    *             output pcOperator - Operator. <br>
    *             output pcValue    - Value (with quotes) <br>
    *                               - or '?' as ?                         <br>
    * Notes:  The order of the 3 input lists are similar to various query <br>
    *         manipulation methods, while the single similar output parameters is in<br>
    *         the order they are supposed to be used.. <br>
    *      -  The caller is responsible to ensure that entries that is found in a <br>
    *         buffer is not checked again against another table as the same <br>
    *         unqualifed column may be found in several tables.<br>
    *      -  Unqualified columns are ALWAYS assumed to be database references also<br>
    *         when buffer is RowObject. (This is due to the fact that this is used <br>
    *         from findRowWhere -&gt; findRowObjectWhere -&gt; and we do not want to keep <br>
    *         changing qualifiers through the calls, so we follow the rule of the <br>
    *         outer ..)                      <br>
    *         <br>
    * PRIVATE - Not sufficiently self contained, since the table resolution is not <br>
    *           embedded here. 'salesrep' would be resolved both for order and <br>
    *           customer. This could be fixed by checking all tables in one call, <br>
    *           but this would be even more expensive <br>
    *           (only for the unqualified fields not on the SDO though)
    */
   public void obtainExpressionEntries(final character _pcBuffer_6, final integer _piColumn, final character _pcColumnList_1, final character _pcValueList, final character _pcOperatorList, final character pcColumn_8, final character pcOperator, final character pcValue)
   {
      character pcBuffer_6 = TypeFactory.initInput(_pcBuffer_6);
      integer piColumn = TypeFactory.initInput(_piColumn);
      character pcColumnList_1 = TypeFactory.initInput(_pcColumnList_1);
      character pcValueList = TypeFactory.initInput(_pcValueList);
      character pcOperatorList = TypeFactory.initInput(_pcOperatorList);
      character cStringOperator = TypeFactory.character();
      character cQuote = TypeFactory.character();
      
      internalProcedure(new Block((Init) () -> 
      {
         TypeFactory.initOutput(pcColumn_8);
         TypeFactory.initOutput(pcOperator);
         TypeFactory.initOutput(pcValue);
      }, 
      (Body) () -> 
      {
         pcColumn_8.assign(entry(piColumn, pcColumnList_1));
         
         /* if buffer 'RowObject' we change all column references to RowObject names,
              but unqualified columns are assumed to be db reference    */
         if (_isEqual(pcBuffer_6, "RowObject"))
         {
            if (_not(begins(pcColumn_8, "RowObject.")))
            {
               pcColumn_8.assign(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "dbColumnDataName", targetProcedure(), "I", pcColumn_8));
            }
         }
         else
         {
            /* Convert rowObject reference to db reference */
            if (_begins(pcColumn_8, "RowObject."))
            {
               pcColumn_8.assign(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "columnDBColumn", targetProcedure(), "I", entry(2, pcColumn_8, ".")));
            }
            
            /* If unqualified add buffer to column and check if it is a match. 
                  We want to return the column as found in list, otherwise we could just 
                  have added pcBuffer and used the check below.    
                  NOTE: columnTable only checks server if qualified*/
            if (_isEqual(numEntries(pcColumn_8, "."), 1))
            {
               /* May need server call if not in SDO! */
               if (_isNotEqual( /* {fnarg <func-name> <arg> [handle] */ ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "columnTable", targetProcedure(), "I", concat(pcBuffer_6, ".", pcColumn_8)), pcBuffer_6))
               {
                  pcColumn_8.assign("");
               }
            }
            else
            {
               if (_not(begins(pcColumn_8, concat(pcBuffer_6, "."))))
               {
                  /* If the column db qualification does not match the query's 
                          check to see if it is the correct table after all */
                  if (_isNotEqual(minus(numEntries(pcColumn_8, "."), 1), numEntries(pcBuffer_6, ".")))
                  {
                     /* May go to server if this column is not in the rowobject! */
                     if (_isNotEqual( /* {fnarg <func-name> <arg> [handle] */ ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "columnTable", targetProcedure(), "I", pcColumn_8), pcBuffer_6))
                     {
                        pcColumn_8.assign("");
                     }
                  }
                  else
                  {
                      /* wrong buffer */pcColumn_8.assign("");
                  }
               }
               else
               {
                  if (_isEqual( /* {fnarg <func-name> <arg> [handle] */ ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "dbColumnDataName", targetProcedure(), "I", pcColumn_8), ""))
                  {
                     /* Server call! */
                     if (_isNotEqual( /* {fnarg <func-name> <arg> [handle] */ ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "columnTable", targetProcedure(), "I", pcColumn_8), pcBuffer_6))
                     {
                        pcColumn_8.assign("");
                     }
                  }
               }
            }
         }
         
         if (_isGreaterThan(pcColumn_8, ""))
         {
            pcValue.assign((_notUnknown(pcValueList) ? entry(piColumn, pcValueList, chr(1)) : new character("?")));
            /* Add  to single quotes, as we use single quote around the value) */
            pcValue.assign((_isNotEqual(pcValue, "") ? replaceAll(pcValue, "'", "~'") : new character(" ")));
            /* Get the operator for this valuelist. 
                     Support '',? and '/begins' as default */
            pcOperator.assign(_isGreaterThan(numEntries(pcOperatorList), 1) ? entry(piColumn, pcOperatorList) : _or(or(begins(pcOperatorList, "/"), () -> isUnknown(pcOperatorList)), () -> isEqual(pcOperatorList, "")) ? new character("=") : entry(1, pcOperatorList, "/"));
            /* Look for optional string operator if only one entry in operator */
            cStringOperator.assign(_isEqual(numEntries(pcOperatorList, "/"), 2) ? entry(2, pcOperatorList, "/") : new character(" "));
            pcOperator.assign(_isEqual(cStringOperator, "") ? pcOperator : _isEqual( /* {fnarg <func-name> <arg> [handle] */ ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "columnDataType", targetProcedure(), "I", pcColumn_8), "CHARACTER") ? cStringOperator : pcOperator);
            /* We are quoting ALL values except ? to ensure that decimals behave in both 
                     american and european format and to avoid having to check the datatype
                     unless absolutely necessary (only if stringoperator is defined).  
                     Unknown values must not be quoted for characters, we do not support '?' 
                     unquoted works for all types (Matches and begins does not compile
                     with ? so they are quoted) */
            cQuote.assign((_and(and(isEqual(pcValue, "?"), () -> isNotEqual(pcOperator, "BEGINS")), () -> isNotEqual(pcOperator, "MATCHES")) ? new character("") : new character("'")));
            /* From 9.1B the quotes are included in the value to avoid problems
                     when replacing unquoted ? to a quoted value */
            pcValue.assign(concat(cQuote, pcValue, cQuote));
         }
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     Re-reads the current database record(s) to get any field values<br>
    *              which may have been changed by update triggers, and repopulates <br>
    *              the RowObjUpd row with those values.<br>
    * <p>
    * Parameters:<br>
    *   INPUT phRowObjUpd - handle of the Update temp-table buffer to be used. Note <br>
    *                       that this is required because the caller uses two <br>
    *                       different buffers in some cases.
    */
   public void refetchDbrow(final handle _phRowObjUpd_1)
   {
      handle phRowObjUpd_1 = TypeFactory.initInput(_phRowObjUpd_1);
      character cBuffers = TypeFactory.character();
      handle hBuffer = TypeFactory.handle();
      integer iTable_1 = TypeFactory.integer();
      rowid rRowid = TypeFactory.rowid();
      handle hRowIdent = TypeFactory.handle();
      character cRowIdent = TypeFactory.character();
      character cAssigns = TypeFactory.character();
      character cExclude = TypeFactory.character();
      logical lFound = TypeFactory.logical();
      handle hRowObject = TypeFactory.handle();
      character cColumns = TypeFactory.character();
      integer iColumn = TypeFactory.integer();
      handle hColumn = TypeFactory.handle();
      integer iCalc = TypeFactory.integer();
      character cCalcCol = TypeFactory.character();
      character cCalculatedColumns = TypeFactory.character();
      rowid rRorowid = TypeFactory.rowid();
      
      internalProcedure(new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("BufferHandles"));
         cBuffers.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("AssignList"));
         cAssigns.assign(ghProp.unwrapBufferField().value());
         hRowIdent.assign(phRowObjUpd_1.unwrapBuffer().bufferField("RowIdent"));
         cRowIdent.assign(hRowIdent.unwrapBufferField().value());
         
         if (_isGreaterThan(numEntries(cRowIdent), 1))
         {
            /* Refresh rowids for 'readonly' tables in the join */
            cRowIdent.assign(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "refreshRowident", targetProcedure(), "I", cRowIdent));
            hRowIdent.unwrapBufferField().changeValue(cRowIdent);
         }
         
         IntegerExpression resExpr3 = new IntegerExpression()
         {
            public integer execute()
            {
               return numEntries(cBuffers);
            }
         };
         ToClause toClause4 = new ToClause(iTable_1, 1, resExpr3);
         
         doTo("loopLabel19", toClause4, new Block((Body) () -> 
         {
            if (_isNotEqual(entry(iTable_1, cRowIdent), ""))
            {
               hBuffer.assign(handle.fromString(entry(iTable_1, cBuffers)));
               rRowid.assign(new rowid(entry(iTable_1, cRowIdent)));
               /* 9.1D06 - Due to a core issue, FIND-BY-ROWID will *not* fire WRITE triggers. */
               /* Releasing the buffer will properly fire any WRITE triggers and will return */
               /* false if any trigger returned an error */
               lFound.assign(hBuffer.unwrapBuffer().bufferRelease());
               
               if ((lFound).booleanValue())
               {
                  lFound.assign(hBuffer.unwrapBuffer().findByRowID(rRowid, LockType.SHARE_NO_WAIT));
               }
               
               /* find-by-rowid will fire triggers when called from the transaction block,
                        if the trigger returns ERROR find-by-rowid will return false
                        A return-value will be returned to the user thru adm messaging */
               if (_not(lFound))
               {
                  returnError(((hBuffer.unwrapBuffer().locked()).booleanValue() ? ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "messageNumber", targetProcedure(), "I", 18) : ControlFlowOps.getReturnValue()));
               }
               
               cExclude /* {fnarg <func-name> <arg> [handle] */ .assign(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "excludeColumns", targetProcedure(), "I", iTable_1));
               
               if (_isEqual(indexOf(cAssigns, "["), 0))
               {
                  phRowObjUpd_1.unwrapBuffer().bufferCopy(hBuffer, cExclude /* exclude-list */, entry(iTable_1, cAssigns, ";"));
               }
               else
               {
 /* If array elements are present, we must assign them manually. */
                  ControlFlowOps.invokeWithMode("BufferCopyDBToRO", "IIII", phRowObjUpd_1, hBuffer, cExclude, entry(iTable_1, cAssigns, ";"));
               }
               
               /* If there are calculated fields, make sure they are updated */
               /* {get Prop var [handle]} */ 
               cCalculatedColumns.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getCalculatedColumns", targetProcedure()));
               
               if (_isGreaterThan(cCalculatedColumns, ""))
               {
                  /* {get Prop var [handle]} */ 
                  hRowObject.assign(ControlFlowOps.invokeDynamicFunctionIn(handle.class, "getRowObject", targetProcedure()));
                  /* Buffer handle for RowObject table */
                  rRorowid.assign(hRowObject.unwrapBuffer().rowID());
                  hRowObject.unwrapBuffer().bufferCreate();
                  /* Create "dummy" RowObject record */
                  hRowObject.unwrapBuffer().bufferCopy(phRowObjUpd_1);
                  ErrorManager.silentErrorEnable();
                  /* Copy RowObjUpd to RowObject */
                  ControlFlowOps.invokeIn("Data.Calculate", targetProcedure());
                  ErrorManager.silentErrorDisable();
                  
                  /* Updates RowObject */
                  if (_isEqual(ErrorManager.getErrorNumber(1), 6456))
                  {
                     IntegerExpression resExpr4 = new IntegerExpression()
                     {
                        public integer execute()
                        {
                           return numEntries(cCalculatedColumns);
                        }
                     };
                     ToClause toClause5 = new ToClause(iCalc, 1, resExpr4);
                     
                     doTo("loopLabel20", toClause5, new Block((Body) () -> 
                     {
                        cCalcCol.assign(entry(iCalc, cCalculatedColumns));
                        ErrorManager.silentErrorEnable();
                        hRowObject.unwrapBuffer().bufferField(cCalcCol).unwrapBufferField().changeValue(ControlFlowOps.invokeDynamicFunctionIn(concat("Calculate", cCalcCol), targetProcedure()));
                        ErrorManager.silentErrorDisable();
                     }));
                  }
                  
                  phRowObjUpd_1.unwrapBuffer().bufferCopy(hRowObject);
                  /* Copy updated RowObject to RowObjUpd */
                  hRowObject.unwrapBuffer().deleteRecord();
                  
                  /* Clean up */
                  if (_and(notUnknown(rRorowid), () -> isNotEqual(rRorowid, hRowObject.unwrapBuffer().rowID())))
                  {
                     hRowObject.unwrapBuffer().findByRowID(rRorowid);
                  }
               }
            }
            else
            {
               ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
               ghProp.assign(ghProp.unwrapBuffer().bufferField("DataColumnsByTable"));
               cColumns.assign(ghProp.unwrapBufferField().value());
               cColumns.assign(entry(iTable_1, cColumns, ";"));
               
               loopLabel21:
               for (iColumn.assign(1); _isLessThanOrEqual(iColumn, numEntries(cColumns)); iColumn.increment())
               {
                  hColumn.assign(phRowObjUpd_1.unwrapBuffer().bufferField(entry(iColumn, cColumns)));
                  hColumn.unwrapBufferField().changeValue(new character(""));
               }
            }
         }));
         
         /* END DO iTable */
         returnNormal();
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     Releases the database records following an update.<br>
    * <p>
    * Parameters:  &lt;none&gt;
    */
   public void releaseDbrow()
   {
      character cBuffers = TypeFactory.character();
      handle hBuffer = TypeFactory.handle();
      integer iTable_1 = TypeFactory.integer();
      
      internalProcedure(new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("BufferHandles"));
         cBuffers.assign(ghProp.unwrapBufferField().value());
         
         loopLabel22:
         for (iTable_1.assign(1); _isLessThanOrEqual(iTable_1, numEntries(cBuffers)); iTable_1.increment())
         {
            hBuffer.assign(handle.fromString(entry(iTable_1, cBuffers)));
            
            if ((hBuffer.unwrapBuffer().available()).booleanValue())
            {
               hBuffer.unwrapBuffer().bufferRelease();
            }
         }
         
         /* END DO iTable */
         returnNormal();
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     View/Start the filter-source<br>
    * Parameters:  &lt;none&gt;<br>
    * Notes:
    */
   public void startFilter()
   {
      handle hFilterSource = TypeFactory.handle();
      handle hWindow = TypeFactory.handle();
      logical lHide = TypeFactory.logical();
      handle hFilterContainer = TypeFactory.handle();
      handle hMyContainer = TypeFactory.handle();
      character cFilterWindow = TypeFactory.character();
      
      internalProcedure(new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FilterSource"));
         hFilterSource.assign(ghProp.unwrapBufferField().value());
         
         if ((hFilterSource.isValid()).booleanValue())
         {
            ghProp.assign(handle.fromString(entry(1, hFilterSource.unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("ContainerSource"));
            hFilterContainer.assign(ghProp.unwrapBufferField().value());
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("ContainerSource"));
            hMyContainer.assign(ghProp.unwrapBufferField().value());
            
            if (_isNotEqual(hMyContainer, hFilterContainer))
            {
               ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
               ghProp.assign(ghProp.unwrapBuffer().bufferField("FilterWindow"));
               ghProp.unwrapBufferField().changeValue(hFilterContainer.unwrapFile().getFileName());
               ghProp.assign(handle.fromString(entry(1, hFilterContainer.unwrapADMData().getADMData(), chr(1))));
               ghProp.assign(ghProp.unwrapBuffer().bufferField("HideOnInit"));
               lHide.assign(ghProp.unwrapBufferField().value());
               
               /* Workaround to make it visible if it's hideoninit */
               if ((lHide).booleanValue())
               {
                  ControlFlowOps.invokeIn("destroyObject", hFilterContainer);
               }
            }
         }
         
         if (_not(hFilterContainer.isValid()))
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("FilterWindow"));
            cFilterWindow.assign(ghProp.unwrapBufferField().value());
            
            if (_isNotEqual(cFilterWindow, ""))
            {
               ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
               ghProp.assign(ghProp.unwrapBuffer().bufferField("ContainerSource"));
               hMyContainer.assign(ghProp.unwrapBufferField().value());
               ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
               ghProp.assign(ghProp.unwrapBuffer().bufferField("ContainerHandle"));
               hWindow.assign(ghProp.unwrapBufferField().value());
               ControlFlowOps.invokeInWithMode("constructObject", hMyContainer, "IIIO", cFilterWindow, hWindow, concat("HideOnInit", chr(4), "no", chr(3), "DisableOnInit", chr(4), "no", chr(3), "ObjectLayout", chr(4)), hFilterContainer);
               /* filterContainerHandler adds the Filter link between this object
                        and the Filter container */
               ControlFlowOps.invokeInWithMode("filterContainerHandler", targetProcedure(), "I", hFilterContainer);
               ControlFlowOps.invokeIn("initializeObject", hFilterContainer);
            }
         }
         ControlFlowOps.invokeIn("viewObject", hFilterContainer);
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     This procedure buffer-copies the database record(s) for the<br>
    *              current query row into the RowObject temp-table.<br>
    *              <br>
    * Parameters:<br>
    *   INPUT pcRowIdent - comma-separated list consisting of the RowId of the <br>
    *                      RowObject record followed by the RowIds of the<br>
    *                      database record(s); if unknown or blank, then the <br>
    *                      database records have already been retrieved.<br>
    *   INPUT piRowNum   - integer value to assign to the RowNum<br>
    *                      field in the RowObject temp-table record.<br>
    *    <br>
    * Notes:       This procedure replaces the transferRowFromDB function from V9.0.<br>
    *              It can be localized to perform additional custom operations<br>
    *              each time a row is tramsferred from the database to the <br>
    *              RowObject table.
    */
   public void transferDbrow(final character _pcRowIdent, final integer _piRowNum)
   {
      character pcRowIdent = TypeFactory.initInput(_pcRowIdent);
      integer piRowNum = TypeFactory.initInput(_piRowNum);
      integer iTable_1 = TypeFactory.integer();
      handle hBuffer = TypeFactory.handle();
      character cBuffers = TypeFactory.character();
      handle hRowObject = TypeFactory.handle();
      rowid rRowid = TypeFactory.rowid();
      handle hField = TypeFactory.handle();
      character cAssigns = TypeFactory.character();
      character cExclude = TypeFactory.character();
      character cColumns = TypeFactory.character();
      handle hColumn = TypeFactory.handle();
      integer iColumn = TypeFactory.integer();
      integer iLoop = TypeFactory.integer();
      character cRowUserProp = TypeFactory.character();
      integer iCalc = TypeFactory.integer();
      character cCalcCol = TypeFactory.character();
      character cCalculatedColumns = TypeFactory.character();
      /* END DO iTable */
      /*********************************************************************
      * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
      * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
      * below.  All Rights Reserved.                                       *
      *                                                                    *
      * The Initial Developer of the Original Code is PSC.  The Original   *
      * Code is Progress IDE code released to open source December 1, 2000.*
      *                                                                    *
      * The contents of this file are subject to the Possenet Public       *
      * License Version 1.0 (the "License"); you may not use this file     *
      * except in compliance with the License.  A copy of the License is   *
      * available as of the date of this notice at                         *
      * http://www.possenet.org/license.html                               *
      *                                                                    *
      * Software distributed under the License is distributed on an "AS IS"*
      * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
      * should refer to the License for the specific language governing    *
      * rights and limitations under the License.                          *
      *                                                                    *
      * Contributors:                                                      *
      *                                                                    *
      *********************************************************************/
      /*--------------------------------------------------------------------------
          File        : src/adm2/entityfields.i 
          Purpose     : Populate the entityfields 
                        This is separated out as an include because it is
                        used by both transferDbRow and transferRows. 
                        transferRows is a performance version. We DO NOT want 
                        to put this into an EXTRA procedure as this adds an
                        unacceptable performance cost. 
                        
          Notes:        We may separate this into a internal procedure if the call 
                        does NOT need to call any more procedures and move all the
                        logic in getRecordUserProp into it.
                        (Or move all of this into getRecordUserProp if it can do 
                         direct access to the sdo props)  
                         
        Parameters:  The variables that need to be defined on the outside of this 
                     need to be defined as the following preprocessors:   
              &RowObject    - Rowobject handle - already populated
              &RowUserProp  - Char variable that is in effect the returned value
              &entityfields - Optional char var for entityfields                         
                              If NOT defined this include will do the get
                              If defined we assume that the get is done.
                             (the latter allows transferRows to only do the get once)
        ------------------------------------------------------------------------*/
      /*          This .W file was created with the Progress UIB.             */
      /*----------------------------------------------------------------------*/
      character cKeyTableId = TypeFactory.character();
      character cKeyFields = TypeFactory.character();
      character cKeyValue = TypeFactory.character();
      integer iKeyLoop = TypeFactory.integer();
      handle hKeyField = TypeFactory.handle();
      logical lFetchHasAudit = TypeFactory.logical();
      logical lFetchHasComment = TypeFactory.logical();
      logical lFetchAutoComment = TypeFactory.logical();
      integer iHasAudit = TypeFactory.integer();
      integer iHasComment = TypeFactory.integer();
      integer iAutoComment = TypeFactory.integer();
      character cEntityFields = TypeFactory.character();
      
      internalProcedure(new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("BufferHandles"));
         cBuffers.assign(ghProp.unwrapBufferField().value());
         
         /* List of DB Buffer Handles */
         if (_not(begins(EnvironmentOps.getSourceName(2), "transferDBRow")))
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("SkiptransferDBRow"));
            ghProp.unwrapBufferField().changeValue(new logical(true));
         }
         /* {set Prop value [handle]} */ 
         
         /* {get Prop var [handle]} */ 
         hRowObject.assign(ControlFlowOps.invokeDynamicFunctionIn(handle.class, "getRowObject", targetProcedure()));
         
         /* Buffer handle for the RowObject table. */
         if (_isUnknown(pcRowIdent))
         {
            pcRowIdent.assign("");
         }
         
         if (_isNotEqual(pcRowIdent, ""))
         {
            loopLabel23:
            for (iTable_1.assign(2); _isLessThanOrEqual(iTable_1, numEntries(pcRowIdent)); iTable_1.increment())
            {
               hBuffer.assign(handle.fromString(entry(minus(iTable_1, 1), cBuffers)));
               rRowid.assign(new rowid(entry(iTable_1, pcRowIdent)));
               ErrorManager.silentErrorEnable();
               hBuffer.unwrapBuffer().findByRowID(rRowid);
               ErrorManager.silentErrorDisable();
            }
         }
         /* In this case (called from doCreateUpdate, e.g.), we need to
                re-retrieve a row that is no longer in the RowObject table.
                The RowIds if the db records are passed in to us. 
                Note that the first entry in pcRowIdent is ignored because it
                the RowId of the RowObject record itself. */
         
         /* END DO iTable */
         /* Buffer-Copy the database record(s) into the RowObject buffer. */
         pcRowIdent.assign("");
         hField.assign(hRowObject.unwrapBuffer().bufferField("RowNum"));
         hField.unwrapBufferField().changeValue(piRowNum);
         hField.assign(hRowObject.unwrapBuffer().bufferField("RowMod"));
         hField.unwrapBufferField().changeValue(new character(""));
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("AssignList"));
         cAssigns.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DataColumnsByTable"));
         cColumns.assign(ghProp.unwrapBufferField().value());
         
         IntegerExpression resExpr5 = new IntegerExpression()
         {
            public integer execute()
            {
               return numEntries(cBuffers);
            }
         };
         ToClause toClause6 = new ToClause(iTable_1, 1, resExpr5);
         
         doTo("loopLabel24", toClause6, new Block((Body) () -> 
         {
            if (_isNotEqual(iTable_1, 1))
            {
               pcRowIdent.assign(concat(pcRowIdent, ","));
            }
            
            hBuffer.assign(handle.fromString(entry(iTable_1, cBuffers)));
            
            /* Allow for an outer join without a rec in every buffer. */
            if ((hBuffer.unwrapBuffer().available()).booleanValue())
            {
               /* Exclude all fieldnames from all other tables to prevent a potential 
                        overwrite from an unintentional table. */
               cExclude /* {fnarg <func-name> <arg> [handle] */ .assign(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "excludeColumns", targetProcedure(), "I", iTable_1));
               
               /*
                     _* Remove the columns from the iTable table from the excludelist *_
                     IF NUM-ENTRIES(cBuffers) > 1 THEN
                     ASSIGN
                       ENTRY(iTable,cColumns,{&adm-tabledelimiter}) = "":U
                       cExclude = TRIM(REPLACE(cExclude,{&adm-tabledelimiter},",":U),",":U). 
                        */
               if (_isEqual(indexOf(cAssigns, "["), 0))
               {
                  hRowObject.unwrapBuffer().bufferCopy(hBuffer, cExclude, entry(iTable_1, cAssigns, ";"));
               }
               else
               {
 /* If array elements are present, we must assign them manually. */
                  ControlFlowOps.invokeWithMode("BufferCopyDBToRO", "IIII", hRowObject, hBuffer, cExclude, entry(iTable_1, cAssigns, ";"));
               }
               
               pcRowIdent.assign(concat(pcRowIdent, valueOf(hBuffer.unwrapBuffer().rowID())));
            }
            else
            {
               ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
               ghProp.assign(ghProp.unwrapBuffer().bufferField("DataColumnsByTable"));
               cColumns.assign(ghProp.unwrapBufferField().value());
               cColumns.assign(entry(iTable_1, cColumns, ";"));
               
               loopLabel25:
               for (iColumn.assign(1); _isLessThanOrEqual(iColumn, numEntries(cColumns)); iColumn.increment())
               {
                  hColumn.assign(hRowObject.unwrapBuffer().bufferField(entry(iColumn, cColumns)));
                  hColumn.unwrapBufferField().changeValue(new character(""));
               }
            }
         }));
         
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("EntityFields"));
         cEntityFields.assign(ghProp.unwrapBufferField().value());
         
         /* - Update of the RowUserProp field  */
         if (_and(isGreaterThan(cEntityFields, ""), () -> gshGenManager.isValid()))
         {
            /* {get Prop var [handle]} */ 
            cKeyTableId.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getKeyTableId", targetProcedure()));
            /* Key Table Entity Mnemonic / _dump-name */
            /* {get Prop var [handle]} */ 
            cKeyFields.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getKeyFields", targetProcedure()));
            cKeyValue.assign("");
            
            loopLabel26:
            for (iKeyLoop.assign(1); _isLessThanOrEqual(iKeyLoop, numEntries(cKeyFields)); iKeyLoop.increment())
            {
               ErrorManager.silentErrorEnable();
               hKeyField.assign(hRowObject.unwrapBuffer().bufferField(entry(iKeyLoop, cKeyFields)));
               ErrorManager.silentErrorDisable();
               
               if ((hKeyField.isValid()).booleanValue())
               {
                  if (_isNotEqual(cKeyValue, ""))
                  {
                     cKeyValue.assign(concat(cKeyValue, chr(1)));
                  }
                  
                  cKeyValue.assign(concat(cKeyValue, hKeyField.unwrapBufferField().value()));
               }
            }
            
            iHasAudit.assign(lookup("HasAudit", cEntityFields));
            iHasComment.assign(lookup("HasComment", cEntityFields));
            iAutoComment.assign(lookup("AutoComment", cEntityFields));
            
            if (_isGreaterThan(iHasAudit, 0))
            {
               ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
               ghProp.assign(ghProp.unwrapBuffer().bufferField("FetchHasAudit"));
               lFetchHasAudit.assign(ghProp.unwrapBufferField().value());
               
               if (_not(isEqual(lFetchHasAudit, true)))
               {
                  cEntityFields.replaceEntry(iHasAudit, "");
               }
            }
            
            if (_isGreaterThan(iHasComment, 0))
            {
               ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
               ghProp.assign(ghProp.unwrapBuffer().bufferField("FetchHasComment"));
               lFetchHasComment.assign(ghProp.unwrapBufferField().value());
               
               if (_not(isEqual(lFetchHasComment, true)))
               {
                  cEntityFields.replaceEntry(iHasComment, "");
               }
            }
            
            if (_isGreaterThan(iAutoComment, 0))
            {
               ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
               ghProp.assign(ghProp.unwrapBuffer().bufferField("FetchAutoComment"));
               lFetchAutoComment.assign(ghProp.unwrapBufferField().value());
               
               if (_not(isEqual(lFetchAutoComment, true)))
               {
                  cEntityFields.replaceEntry(iAutoComment, "");
               }
            }
            
            /* Comments were changed to use CHR(2) instead of CHR(1) due
                 to clashes with the ADM-PROPS delimiter of CHR(1). Now testing
                for both CHR(1) and CHR(2) for backward compatibility. */
            if (_isNotEqual(trim(cEntityFields, ","), ""))
            {
               if (_isNotEqual(cKeyValue, ""))
               {
                  ErrorManager.silentErrorEnable();
                  ControlFlowOps.invokeInWithMode("getRecordUserProp", gshGenManager, "IIIO", cKeyTableId, cEntityFields, cKeyValue, cRowUserProp);
                  ErrorManager.silentErrorDisable();
               }
               
               if (_isGreaterThan(numEntries(cKeyValue, chr(1)), 1))
               {
                  ErrorManager.silentErrorEnable();
                  ControlFlowOps.invokeInWithMode("getRecordUserProp", gshGenManager, "IIIO", cKeyTableId, cEntityFields, replaceAll(cKeyValue, chr(1), chr(2)), cRowUserProp);
                  ErrorManager.silentErrorDisable();
               }
            }
         }
         ErrorManager.silentErrorEnable();
         hField.assign(hRowObject.unwrapBuffer().bufferField("RowIdentIdx"));
         hField.unwrapBufferField().changeValue(substring(pcRowIdent, 1, xiRocketIndexLimit));
         hField.assign(hRowObject.unwrapBuffer().bufferField("RowIdent"));
         hField.unwrapBufferField().changeValue(pcRowIdent);
         hField.assign(hRowObject.unwrapBuffer().bufferField("RowUserProp"));
         hField.unwrapBufferField().changeValue(cRowUserProp);
         ErrorManager.silentErrorDisable();
         ErrorManager.silentErrorEnable();
         ControlFlowOps.invokeIn("Data.Calculate", targetProcedure());
         ErrorManager.silentErrorDisable();
         
         if (_isEqual(ErrorManager.getErrorNumber(1), 6456))
         {
            /* {get Prop var [handle]} */ 
            cCalculatedColumns.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getCalculatedColumns", targetProcedure()));
            
            IntegerExpression resExpr6 = new IntegerExpression()
            {
               public integer execute()
               {
                  return numEntries(cCalculatedColumns);
               }
            };
            ToClause toClause7 = new ToClause(iCalc, 1, resExpr6);
            
            doTo("loopLabel27", toClause7, new Block((Body) () -> 
            {
               cCalcCol.assign(entry(iCalc, cCalculatedColumns));
               ErrorManager.silentErrorEnable();
               hRowObject.unwrapBuffer().bufferField(cCalcCol).unwrapBufferField().changeValue(ControlFlowOps.invokeDynamicFunctionIn(concat("Calculate", cCalcCol), targetProcedure()));
               ErrorManager.silentErrorDisable();
            }));
         }
         
         ErrorManager.silentErrorEnable();
         /* Erase everything from error-status from the run no-error   
              TRUE NO-ERROR  also works... but    */
         iLoop.assign(iLoop);
         ErrorManager.silentErrorDisable();
         returnNormal();
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     <br>
    * Parameters:  <br>
    *  plAppend   - True indicates that we are appending to a resultset. <br>
    *               This flag ensures that we only changes the batch property <br>
    *               in the end we are appending. Note that we may append to our <br>
    *               own local RowObject TT or to a resultset on the client<br>
    *             - This is NOT the direct opposite of RebuildOnRepos. Prev and <br>
    *               next requests for a GUI client are for example always appending.<br>
    *               A FIRST request should never be appending.           <br>
    *  pcMode     - The mode defines how this new batch is added to an existing <br>
    *               batch. <br>
    *               - 'FIRST' - first batch.                                  <br>
    *               - 'LAST'  - last batch. <br>
    *               - 'NEXT'  - next batch.<br>
    *               - 'PREV'  - previous batch<br>
    *               - 'REPOSITION[-FROM]'- Reposition use passed Rowident or expression, <br>
    *                  - sendRows with rowident and plNext=NO<br>
    *               - 'REPOSITION-AFTER'- Reposition use passed Rowident or expression, <br>
    *                  - sendRows with rowident and plNext=YES <br>
    *               - 'REPOSITION-TO'  - Reposition use passed Rowident or expression.<br>
    *                  - sendRows with rowident and plNext=NO and negative rowstoreturn<br>
    *               - 'REPOSITION-BEFORE' - Reposition use passed Rowident or expression.<br>
    *                  - sendRows with rowident and plNext=YES and negative rowstoreturn<br>
    *               - 'REFRESH' indicates a refresh of current data.                                                                <br>
    *  pcPosition  - This indicates which new record to position to in the new<br>
    *                batch of data for REFRESH and REPOSITION.                    <br>
    *                - RowNum;Rowident for pcPosition "REFRESH"<br>
    *                - Rowident or a Query String for "REPOSITION"                                  <br>
    *  plFillBatch  - TRUE indicates that the batch should be filled with records<br>
    *                 tot he number specified in piRowstoReturn.  <br>
    *                 This applies to 'prev', 'next' or 'reposition' request when <br>
    *                 plAppend is false. A 'prev' will behave almost as a 'first' <br>
    *                 while the others almost as a 'last', the position will <br>
    *                 be on the actual 'new' record.          <br>
    *  pirowsToReturn - Number of requested records                                                   <br>
    *                 - ?  Use RowsToBatch property<br>
    *                 - 0  Read all records from current    <br>
    *  OUTPUT pirowsReturned - The number of records actually added to the RowObject<br>
    *                          Temp-table.<br>
    *                                <br>
    *     Notes: Although Last and FirstResultRow currently is read here to <br>
    *            find the starting rownumber   <br>
    *            The database query must be positioned BEFORE the first record that<br>
    *             is to be read. We (currently) move forward in the query also when <br>
    *             pcMode is 'BOTTOM' (last or previous request). <br>
    *           - If pcMode &lt;&gt; 'Top' we ensure that the RowNum is high, also when <br>
    *             plAppend is false, since future 'prev' requests may need to <br>
    *             append record with lower Row Numbers.    <br>
    *           - The 'TOP' or 'BOTTOM' also decides when to publish <br>
    *             DataAvailable('TRANSFER') when TransferChildrenForAll is false.
    */
   public void transferRows(final logical _plAppend, final character _pcMode, final character _pcPosition, final logical _plFillBatch, final integer _piRowsToReturn, final integer piRowsReturned)
   {
      logical plAppend = TypeFactory.initInput(_plAppend);
      character pcMode = TypeFactory.initInput(_pcMode);
      character pcPosition = TypeFactory.initInput(_pcPosition);
      logical plFillBatch = TypeFactory.initInput(_plFillBatch);
      integer piRowsToReturn = TypeFactory.initInput(_piRowsToReturn);
      handle hRowObject = TypeFactory.handle();
      character cLastResult = TypeFactory.character();
      character cFirstResult = TypeFactory.character();
      character cToColumn = TypeFactory.character();
      character cToVaLue = TypeFactory.character();
      handle hQuery = TypeFactory.handle();
      integer iBuffer = TypeFactory.integer();
      handle hFirstBuffer = TypeFactory.handle();
      character cToPosition = TypeFactory.character();
      logical lRefresh = TypeFactory.logical();
      logical lFromTop = TypeFactory.logical();
      character cStartRow = TypeFactory.character();
      rowid[] rRowidList = TypeFactory.rowidExtent(18);
      logical lPos = TypeFactory.logical();
      logical lFound = TypeFactory.logical();
      
      internalProcedure(new Block((Init) () -> 
      {
         TypeFactory.initOutput(piRowsReturned);
      }, 
      (Body) () -> 
      {
         if (_isUnknown(piRowsToReturn))
         {
            /* {get Prop var [handle]} */ 
            piRowsToReturn.assign(ControlFlowOps.invokeDynamicFunctionIn(integer.class, "getRowsToBatch", targetProcedure()));
         }
         
         /* Ensure plFillbatch only is set when applicable to simplify logic below */
         if ((plAppend).booleanValue())
         {
            plFillBatch.assign(false);
            
            /* Maybe we can support these later, but it need some more work 
                  ('reposition' is the same as 'reposition-from')  */
            if (_and(begins(pcMode, "Reposition-"), () -> isNotEqual(pcPosition, "Reposition-from")))
            {
               /* {fnarg <func-name> <arg> [handle] */ 
               message(substitute(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "messageNumber", targetProcedure(), "I", 68), character.quoter(pcMode)));
               returnNormal();
            }
         }
         /* {get Prop var [handle]} */ 
         hRowObject.assign(ControlFlowOps.invokeDynamicFunctionIn(handle.class, "getRowObject", targetProcedure()));
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryHandle"));
         hQuery.assign(ghProp.unwrapBufferField().value());
         hFirstBuffer.assign(hQuery.unwrapQuery().bufferHandle(1));
         /* Historically there has been problems with the buffer being available 
             and not in synch with the query. This problem was in sendRows and may 
             not be present here, but serious testing is required to ensure this 
             including dataServers.  */
         hFirstBuffer.unwrapBuffer().bufferRelease();
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("LastResultRow"));
         cLastResult.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FirstResultRow"));
         cFirstResult.assign(ghProp.unwrapBufferField().value());
         
         /* Appending last is a big unlimited next... */
         if (_and(plAppend, () -> isEqual(pcMode, "Last")))
         {
            pcMode.assign("Next");
            piRowsToReturn.assign(0);
         }
         else
         {
            if (_and(isEqual(pcMode, "Prev"), () -> isUnknown(cFirstResult)))
            {
               pcMode.assign("Last");
            }
         }
         
         switch (StringHelper.safeTrimTrailing((pcMode).toStringMessage()).toUpperCase())
         {
            case "FIRST":
               {
                  hQuery.unwrapQuery().getFirst();
                  hQuery.unwrapQuery().queryBackward(1);
                  lFromTop.assign(true);
               }
               break;
            case "LAST":
               {
                  hQuery.unwrapQuery().getLast();
                  hQuery.unwrapQuery().queryBackward(piRowsToReturn);
                  lFromTop.assign(false);
               }
               break;
            case "NEXT":
               {
                  lFromTop.assign(true);
                  cStartRow.assign(_isUnknown(cLastResult) ? new character() : entry(2, cLastResult, ";"));
               }
               break;
            case "PREV":
               {
                  lFromTop.assign(false);
                  cStartRow.assign(_isUnknown(cFirstResult) ? new character() : entry(2, cFirstResult, ";"));
               }
               break;
            case "REFRESH":
               {
                  lFromTop.assign(true);
                  lRefresh.assign(true);
                  cStartRow.assign(entry(2, pcPosition, ";"));
                  cToVaLue.assign(entry(1, pcPosition, ";"));
                  cToColumn.assign("RowNum");
               }
               break;
            case "CURRENT":
               {
                  lFromTop.assign(true);
                  lRefresh.assign(true);
                  cStartRow.assign(_isUnknown(cFirstResult) ? new character() : entry(2, cFirstResult, ";"));
                  cToColumn.assign("Rowident");
                  cToVaLue.assign(entry(2, cLastResult, ";"));
                  plFillBatch.assign(_isGreaterThan(cToVaLue, "") ? new logical(false) : plFillBatch);
                  piRowsToReturn.assign(_isGreaterThan(cToVaLue, "") ? new integer(0) : piRowsToReturn);
               }
               break;
            case "REPOSITION":
            case "REPOSITION-FROM":
            case "REPOSITION-AFTER":
            case "REPOSITION-TO":
            case "REPOSITION-BEFORE":
               {
                  if (_isGreaterThan(indexOf(pcPosition, " "), 0))
                  {
                     pcPosition.assign(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "firstRowIds", targetProcedure(), "I", pcPosition));
                     
                     /* no record found */
                     if (_isUnknown(pcPosition))
                     {
                        returnNormal();
                     }
                  }
                  lFromTop.assign(matchesList("Reposition,Reposition-from,Reposition-after", pcMode));
                  
                  if (_not(plAppend))
                  {
                     cStartRow.assign(pcPosition);
                  }
                  else
                  {
                     cStartRow.assign(_isUnknown(cLastResult) ? new character() : entry(2, cLastResult, ";"));
                     cToVaLue.assign(pcPosition);
                     
                     /* If appending and pcPosition is a rowident we do a direct repos here 
                               to ensure that the record actually will be encountered in the batch
                               (Otherwise we would read the whole batch) */
                     if (_isEqual(indexOf(pcPosition, " "), 0))
                     {
                        loopLabel28:
                        for (iBuffer.assign(1); _isLessThanOrEqual(iBuffer, numEntries(cToVaLue)); iBuffer.increment())
                        {
                           ErrorManager.silentErrorEnable();
                           assignSingle(rRowidList, iBuffer, new rowid(entry(iBuffer, cToVaLue)));
                           ErrorManager.silentErrorDisable();
                        }
                        
                        ErrorManager.silentErrorEnable();
                        lFound.assign(hQuery.unwrapQuery().queryRepositionByID(subscript(rRowidList, 1), subscript(rRowidList, 2), subscript(rRowidList, 3), subscript(rRowidList, 4), subscript(rRowidList, 5), subscript(rRowidList, 6), subscript(rRowidList, 7), subscript(rRowidList, 8), subscript(rRowidList, 9), subscript(rRowidList, 10), subscript(rRowidList, 11), subscript(rRowidList, 12), subscript(rRowidList, 13), subscript(rRowidList, 14), subscript(rRowidList, 15), subscript(rRowidList, 16), subscript(rRowidList, 17), subscript(rRowidList, 18)));
                        ErrorManager.silentErrorDisable();
                        
                        if (_not(lFound))
                        {
                           returnNormal();
                        }
                        
                        /* if no startrow we have to go back to the first/top */
                        if (_isUnknown(cStartRow))
                        {
                           hQuery.unwrapQuery().getFirst();
                           hQuery.unwrapQuery().queryBackward(1);
                        }
                     }
                  }
               }
               break;
         }
         
         if (_isGreaterThan(cStartRow, ""))
         {
            loopLabel29:
            for (iBuffer.assign(1); _isLessThanOrEqual(iBuffer, numEntries(cStartRow)); iBuffer.increment())
            {
               ErrorManager.silentErrorEnable();
               assignSingle(rRowidList, iBuffer, new rowid(entry(iBuffer, cStartRow)));
               ErrorManager.silentErrorDisable();
            }
            
            ErrorManager.silentErrorEnable();
            lFound.assign(hQuery.unwrapQuery().queryRepositionByID(subscript(rRowidList, 1), subscript(rRowidList, 2), subscript(rRowidList, 3), subscript(rRowidList, 4), subscript(rRowidList, 5), subscript(rRowidList, 6), subscript(rRowidList, 7), subscript(rRowidList, 8), subscript(rRowidList, 9), subscript(rRowidList, 10), subscript(rRowidList, 11), subscript(rRowidList, 12), subscript(rRowidList, 13), subscript(rRowidList, 14), subscript(rRowidList, 15), subscript(rRowidList, 16), subscript(rRowidList, 17), subscript(rRowidList, 18)));
            ErrorManager.silentErrorDisable();
            
            if ((lFound).booleanValue())
            {
               if ((lFromTop).booleanValue())
               {
                  /* Append search is the same as NEXT batch, the actual search is the 
                            to value (also note that we only support one mode for append .
                                     see error message at top)*/
                  /* Append search is the same as NEXT batch, the actual search is the 
                            to value (also note that we only support one mode for append .
                                     see error message at top)*/
                  if (_or(or(isEqual(pcMode, "NEXT"), () -> isEqual(pcMode, "REPOSITION-AFTER")), () -> and(begins(pcMode, "REPOSITION"), plAppend)))
                  {
                     hQuery.unwrapQuery().queryForward(1);
                  }
                  
                  /* Check if we need to start higher to fill the batch */
                  if ((plFillBatch).booleanValue())
                  {
                     lPos.assign(hQuery.unwrapQuery().queryForward(piRowsToReturn));
                     
                     /* We do not have enough records to fill the batch, so set TOvalue*/
                     if (_not(lPos))
                     {
                        if (_not(lRefresh))
                        {
                            /* for REFRESH, ToValue is already set to RowNum */cToVaLue.assign(cStartRow);
                        }
                        
                        if (_or(isEqual(pcMode, "NEXT"), () -> isEqual(pcMode, "REPOSITION-AFTER")))
                        {
                           cToPosition.assign("AFTER");
                        }
                     }
                     hQuery.unwrapQuery().queryBackward(piRowsToReturn);
                  }
               }
               else
               {
                  if (_or(isEqual(pcMode, "PREV"), () -> isEqual(pcMode, "REPOSITION-BEFORE")))
                  {
                     hQuery.unwrapQuery().queryBackward(piRowsToReturn);
                     cToVaLue.assign(cStartRow);
                     cToPosition.assign("BEFORE");
                  }
                  else
                  {
                     if (_isEqual(pcMode, "REPOSITION-TO"))
                     {
                        hQuery.unwrapQuery().queryBackward(minus(piRowsToReturn, 1));
                        cToVaLue.assign(cStartRow);
                     }
                  }
               }
            }
            else
            {
               ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
               ghProp.assign(ghProp.unwrapBuffer().bufferField("PositionForClient"));
               ghProp.unwrapBufferField().changeValue(new unknown());
               returnNormal();
            }
         }
         
         if (_and(isGreaterThan(cToVaLue, ""), () -> isEqual(cToColumn, "")))
         {
            cToColumn.assign("Rowident");
         }
         
         /* open and position */
         ControlFlowOps.invokeInWithMode("transferRowsFromDB", targetProcedure(), "IIIIIIIIO", plAppend, lFromTop, cToColumn, cToVaLue, cToPosition, plFillBatch, new logical(true), piRowsToReturn, piRowsReturned);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public void transferRowsFromDb(final logical _plAppend, final logical _plFromTop, final character _pcColumns_4, final character _pcValues_4, final character _pcPosition, final logical _plFillBatch, final logical _plOpen, final integer _piRowsToReturn, final integer piRowsReturned)
   {
      logical plAppend = TypeFactory.initInput(_plAppend);
      logical plFromTop = TypeFactory.initInput(_plFromTop);
      character pcColumns_4 = TypeFactory.initInput(_pcColumns_4);
      character pcValues_4 = TypeFactory.initInput(_pcValues_4);
      character pcPosition = TypeFactory.initInput(_pcPosition);
      logical plFillBatch = TypeFactory.initInput(_plFillBatch);
      logical plOpen = TypeFactory.initInput(_plOpen);
      integer piRowsToReturn = TypeFactory.initInput(_piRowsToReturn);
      logical lAppendSearch = TypeFactory.logical();
      character cToRowident = TypeFactory.character();
      logical lRefresh = TypeFactory.logical();
      handle hRowIdent = TypeFactory.handle();
      handle hRowIdentIdx = TypeFactory.handle();
      handle hRowNum = TypeFactory.handle();
      handle hRowUserProp = TypeFactory.handle();
      handle hRowObject = TypeFactory.handle();
      logical lFirst = TypeFactory.logical();
      handle hQuery = TypeFactory.handle();
      character cLastResult = TypeFactory.character();
      character cFirstResult = TypeFactory.character();
      logical lSkipApi = TypeFactory.logical();
      logical lStop = TypeFactory.logical();
      integer iLoopCount = TypeFactory.integer();
      integer iRownum = TypeFactory.integer();
      handle[] hBuffer = TypeFactory.handleExtent(18);
      character[] cExclude = TypeFactory.characterExtent(18);
      character[] cAssignArray = TypeFactory.characterExtent(18);
      character[] cColumns = TypeFactory.characterExtent(18);
      character cAssigns = TypeFactory.character();
      character cRowident = TypeFactory.character();
      integer iBuffer = TypeFactory.integer();
      character cEntityFields = TypeFactory.character();
      character cRowUserProp = TypeFactory.character();
      logical lNoCalc = TypeFactory.logical();
      logical lFieldCalc = TypeFactory.logical();
      handle hDataSource = TypeFactory.handle();
      character cNewFirstResult = TypeFactory.character();
      character cNewLastResult = TypeFactory.character();
      character cLastDbrowident = TypeFactory.character();
      handle hReadHandler = TypeFactory.handle();
      character cReadFormat = TypeFactory.character();
      character cReadcolumns = TypeFactory.character();
      handle hReadBuffer = TypeFactory.handle();
      logical lChildrenForAll = TypeFactory.logical();
      character cDelimiter = TypeFactory.character();
      integer iRowPos = TypeFactory.integer();
      rowid rRowid = TypeFactory.rowid();
      handle hDataQuery = TypeFactory.handle();
      character cAsDivision = TypeFactory.character();
      character cCalculatedColumns = TypeFactory.character();
      integer iCalc = TypeFactory.integer();
      character cCalcCol = TypeFactory.character();
      integer iCol = TypeFactory.integer();
      character cColumnsByTable = TypeFactory.character();
      handle hColumn = TypeFactory.handle();
      logical lQueryContainer = TypeFactory.logical();
      handle hContainer = TypeFactory.handle();
      character cContainerName = TypeFactory.character();
      character cUniqueObjectName = TypeFactory.character();
      character cBufferHandles = TypeFactory.character();
      logical lOneToOne = TypeFactory.logical();
      handle hFirstBuffer = TypeFactory.handle();
      character cNewBatchInfo = TypeFactory.character();
      logical lLastFound = TypeFactory.logical();
      /*********************************************************************
      * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
      * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
      * below.  All Rights Reserved.                                       *
      *                                                                    *
      * The Initial Developer of the Original Code is PSC.  The Original   *
      * Code is Progress IDE code released to open source December 1, 2000.*
      *                                                                    *
      * The contents of this file are subject to the Possenet Public       *
      * License Version 1.0 (the "License"); you may not use this file     *
      * except in compliance with the License.  A copy of the License is   *
      * available as of the date of this notice at                         *
      * http://www.possenet.org/license.html                               *
      *                                                                    *
      * Software distributed under the License is distributed on an "AS IS"*
      * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
      * should refer to the License for the specific language governing    *
      * rights and limitations under the License.                          *
      *                                                                    *
      * Contributors:                                                      *
      *                                                                    *
      *********************************************************************/
      /*--------------------------------------------------------------------------
          File        : src/adm2/entityfields.i 
          Purpose     : Populate the entityfields 
                        This is separated out as an include because it is
                        used by both transferDbRow and transferRows. 
                        transferRows is a performance version. We DO NOT want 
                        to put this into an EXTRA procedure as this adds an
                        unacceptable performance cost. 
                        
          Notes:        We may separate this into a internal procedure if the call 
                        does NOT need to call any more procedures and move all the
                        logic in getRecordUserProp into it.
                        (Or move all of this into getRecordUserProp if it can do 
                         direct access to the sdo props)  
                         
        Parameters:  The variables that need to be defined on the outside of this 
                     need to be defined as the following preprocessors:   
              &RowObject    - Rowobject handle - already populated
              &RowUserProp  - Char variable that is in effect the returned value
              &entityfields - Optional char var for entityfields                         
                              If NOT defined this include will do the get
                              If defined we assume that the get is done.
                             (the latter allows transferRows to only do the get once)
        ------------------------------------------------------------------------*/
      /*          This .W file was created with the Progress UIB.             */
      /*----------------------------------------------------------------------*/
      character cKeyTableId = TypeFactory.character();
      character cKeyFields = TypeFactory.character();
      character cKeyValue = TypeFactory.character();
      integer iKeyLoop = TypeFactory.integer();
      handle hKeyField = TypeFactory.handle();
      logical lFetchHasAudit = TypeFactory.logical();
      logical lFetchHasComment = TypeFactory.logical();
      logical lFetchAutoComment = TypeFactory.logical();
      integer iHasAudit = TypeFactory.integer();
      integer iHasComment = TypeFactory.integer();
      integer iAutoComment = TypeFactory.integer();
      
      internalProcedure(new Block((Init) () -> 
      {
         TypeFactory.initOutput(piRowsReturned);
      }, 
      (Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryHandle"));
         hQuery.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("BufferHandles"));
         cBufferHandles.assign(ghProp.unwrapBufferField().value());
         /* {get Prop var [handle]} */ 
         hRowObject.assign(ControlFlowOps.invokeDynamicFunctionIn(handle.class, "getRowObject", targetProcedure()));
         /* {get Prop var [handle]} */ 
         cCalculatedColumns.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getCalculatedColumns", targetProcedure()));
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("EntityFields"));
         cEntityFields.assign(ghProp.unwrapBufferField().value());
         /* {get Prop var [handle]} */ 
         lOneToOne.assign(ControlFlowOps.invokeDynamicFunctionIn(logical.class, "getUpdateFromSource", targetProcedure()));
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("AssignList"));
         cAssigns.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("PositionForClient"));
         ghProp.unwrapBufferField().changeValue(new unknown());
         assignSingle( /* Search for Rownum is a refresh..*/hBuffer, 1, handle.fromString(entry(1, cBufferHandles)));
         assignSingle(cExclude, 1 /* {fnarg <func-name> <arg> [handle] */ , ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "excludeColumns", targetProcedure(), "I", 1));
         assignSingle(cAssignArray, 1, entry(1, cAssigns, ";"));
         hRowIdent.assign(hRowObject.unwrapBuffer().bufferField("RowIdent"));
         hRowIdentIdx.assign(hRowObject.unwrapBuffer().bufferField("RowIdentIdx"));
         hRowNum.assign(hRowObject.unwrapBuffer().bufferField("RowNum"));
         hRowUserProp.assign(hRowObject.unwrapBuffer().bufferField("RowUserProp"));
         lRefresh.assign(matchesList(pcColumns_4, "RowNum"));
         /* appending from top start counting FROM the search! */
         lAppendSearch.assign(and(and(and(plAppend, plFromTop), () -> isGreaterThan(pcColumns_4, "")), not(lRefresh)));
         cToRowident.assign(_isGreaterThan(lookup("Rowident", pcColumns_4), 0) ? entry(lookup("Rowident", pcColumns_4), pcValues_4, chr(1)) : new character(""));
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DataColumnsByTable"));
         cColumnsByTable.assign(ghProp.unwrapBufferField().value());
         
         IntegerExpression resExpr7 = new IntegerExpression()
         {
            public integer execute()
            {
               return numEntries(cBufferHandles);
            }
         };
         ToClause toClause8 = new ToClause(iBuffer, 2, resExpr7);
         
         doTo("loopLabel30", toClause8, new Block((Body) () -> 
         {
            assignSingle(cAssignArray, iBuffer, entry(iBuffer, cAssigns, ";"));
            assignSingle(hBuffer, iBuffer, handle.fromString(entry(iBuffer, cBufferHandles)));
            assignSingle(cExclude, iBuffer /* {fnarg <func-name> <arg> [handle] */ , ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "excludeColumns", targetProcedure(), "I", iBuffer));
            assignSingle(cColumns, iBuffer, entry(iBuffer, cColumnsByTable, ";"));
         }));
         
         /* {get Prop var [handle]} */ 
         hReadHandler.assign(ControlFlowOps.invokeDynamicFunctionIn(handle.class, "getDataReadHandler", targetProcedure()));
         
         if ((hReadHandler.isValid()).booleanValue())
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("ObjectName"));
            cUniqueObjectName.assign(ghProp.unwrapBufferField().value());
            /* {get Prop var [handle]} */ 
            lQueryContainer.assign(ControlFlowOps.invokeDynamicFunctionIn(logical.class, "getQueryContainer", targetProcedure()));
            
            if ((lQueryContainer).booleanValue())
            {
               ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
               ghProp.assign(ghProp.unwrapBuffer().bufferField("ContainerSource"));
               hContainer.assign(ghProp.unwrapBufferField().value());
               ghProp.assign(handle.fromString(entry(1, hContainer.unwrapADMData().getADMData(), chr(1))));
               ghProp.assign(ghProp.unwrapBuffer().bufferField("ObjectName"));
               cContainerName.assign(ghProp.unwrapBufferField().value());
               cUniqueObjectName.assign(concat(cContainerName, ".", cUniqueObjectName));
            }
            
            /* {get Prop var [handle]} */ 
            cReadcolumns.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getDataReadColumns", targetProcedure()));
            /* {get Prop var [handle]} */ 
            cReadFormat.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getDataReadFormat", targetProcedure()));
            /* {get Prop var [handle]} */ 
            hReadBuffer.assign(ControlFlowOps.invokeDynamicFunctionIn(handle.class, "getDataReadBuffer", targetProcedure()));
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("TransferChildrenForAll"));
            lChildrenForAll.assign(ghProp.unwrapBufferField().value());
            /* {get Prop var [handle]} */ 
            cDelimiter.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getDataDelimiter", targetProcedure()));
            
            if ((hReadBuffer.isValid()).booleanValue())
            {
               hReadBuffer.unwrapBuffer().bufferCreate();
            }
         }
         /* Always check if we are starting on the first record of the dataset. */
         lFirst.assign(not(hQuery.unwrapQuery().queryBackward(1)));
         
         if (_not(lFirst))
         {
            hQuery.unwrapQuery().queryForward(1);
         }
         
         /* ready, Go! */
         hQuery.unwrapQuery().getNext();
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("LastResultRow"));
         cLastResult.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FirstResultRow"));
         cFirstResult.assign(ghProp.unwrapBufferField().value());
         
         /* If this SDO is a 1-to-1 participant the DB query has been joined */
         /* with the RowObject table of its DataSource SDO. We need a reference to */
         /* that buffer (to support possible future outer-join of the child) . */
         if ((lOneToOne).booleanValue())
         {
            hFirstBuffer.assign(hQuery.unwrapQuery().bufferHandle(1));
         }
         else
         {
            hFirstBuffer.assign(subscript(hBuffer, 1));
            /* Save the NewBatchInfo property for use by 1-to-1 clients */
            /* The default is 'ALL'. NewBatchInfo is set to blank after it has been used */
            /* to signify that the RowObject TT for parent and child(ren) are synchronized */
            cNewBatchInfo.assign("ALL");
            
            if ((plFromTop).booleanValue())
            {
               if (_notUnknown(cLastResult))
               {
                  cNewBatchInfo.assign(concat("RowNum > '", entry(1, cLastResult, ";"), "'"));
               }
            }
            else
            {
               if (_notUnknown(cFirstResult))
               {
                  cNewBatchInfo.assign(concat("RowNum < '", entry(1, cFirstResult, ";"), "'"));
               }
            }
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("NewBatchInfo"));
            ghProp.unwrapBufferField().changeValue(cNewBatchInfo);
            
            if ((lRefresh).booleanValue())
            {
               iRownum.assign(minus(new integer(entry(lookup("RowNum", pcColumns_4), pcValues_4, chr(1))), 1));
            }
            else
            {
               /* Start at 0, no matter what, if we know we are at the top */
               if ((lFirst).booleanValue())
               {
                  iRownum.assign(0);
               }
               else
               {
                  if (_and(plFromTop, not(plAppend)))
                  {
                     iRownum.assign(2000000);
                  }
                  else
                  {
                     if ((plFromTop).booleanValue())
                     {
                        iRownum.assign(_isUnknown(cLastResult) ? new integer(0) : new integer(entry(1, cLastResult, ";")));
                     }
                     else
                     {
                                                /* AT top (not from top), ensure no negative numbers and
                                     substract rowsToReturn.. */iRownum.assign(int64.maximum(new int64(0), _isUnknown(cFirstResult) ? new int64(2000000) : minus(new integer(entry(1, cFirstResult, ";")), plus(piRowsToReturn, 1))));
                     }
                  }
               }
               
               if (_and(plAppend, () -> subscript(hBuffer, 1).unwrapBuffer().available()))
               {
                  if ((plFromTop).booleanValue())
                  {
                     cLastResult.setUnknown();
                  }
                  else
                  {
                     cFirstResult.setUnknown();
                  }
               }
               else
               {
                  if (_not(plAppend))
                  {
                     cFirstResult.setUnknown();
                     cLastResult.setUnknown();
                     
                     /* If we are not appending then ensure that First and last Row is reset
                               so they do not have the values from a previous batch */
                     if ((subscript(hBuffer, 1).unwrapBuffer().available()).booleanValue())
                     {
                        ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
                        ghProp.assign(ghProp.unwrapBuffer().bufferField("FirstRowNum"));
                        ghProp.unwrapBufferField().changeValue(new unknown());
                        ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
                        ghProp.assign(ghProp.unwrapBuffer().bufferField("LastRowNum"));
                        ghProp.unwrapBufferField().changeValue(new unknown());
                     }
                     else
                     {
                        ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
                        ghProp.assign(ghProp.unwrapBuffer().bufferField("FirstRowNum"));
                        ghProp.unwrapBufferField().changeValue(new integer(0));
                        ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
                        ghProp.assign(ghProp.unwrapBuffer().bufferField("LastRowNum"));
                        ghProp.unwrapBufferField().changeValue(new integer(0));
                     }
                  }
               }
            }
         }
         /* NOT 1-to-1 */
         /* {get Prop var [handle]} */ 
         cAsDivision.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getASDivision", targetProcedure()));
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("SkipTransferDBRow"));
         lSkipApi.assign(ghProp.unwrapBufferField().value());
         
         LogicalOp whileClause0 = () -> and(and(hFirstBuffer.unwrapBuffer().available(), not(lStop)), () -> or(isLessThan(iLoopCount, piRowsToReturn), () -> isEqual(piRowsToReturn, 0)));
         
         doWhile("loopLabel31", whileClause0, new Block((Body) () -> 
         {
            hRowObject.unwrapBuffer().bufferCreate();
            piRowsReturned.assign(plus(piRowsReturned, 1));
            iLoopCount.assign(_not(lAppendSearch) ? plus(iLoopCount, 1) : new int64(0));
            
            /* start on one when appendsearch is off */
            /* Use the parent's RowObject RowNum to avoid conflicts */
            if ((lOneToOne).booleanValue())
            {
               iRownum.assign(hFirstBuffer.unwrapBuffer().bufferField("RowNum").unwrapBufferField().value());
            }
            else
            {
               iRownum.assign(plus(iRownum, 1));
            }
            
            if ((lSkipApi).booleanValue())
            {
               /* testing indicates that it is more efficient to check cRowident 
                       below in one place */
               if ((subscript(hBuffer, 1).unwrapBuffer().available()).booleanValue())
               {
                  if (_isEqual(indexOf(subscript(cAssignArray, 1), "["), 0))
                  {
                     hRowObject.unwrapBuffer().bufferCopy(subscript(hBuffer, 1), subscript(cExclude, 1), subscript(cAssignArray, 1));
                  }
                  else
                  {
                     ControlFlowOps.invokeWithMode("BufferCopyDBToRO", "IIII", hRowObject, subscript(hBuffer, 1), subscript(cExclude, 1), subscript(cAssignArray, 1));
                  }
               }
               else
               {
                  loopLabel32:
                  for (iCol.assign(1); _isLessThanOrEqual(iCol, numEntries(subscript(cColumns, 1))); iCol.increment())
                  {
                     hColumn.assign(hRowObject.unwrapBuffer().bufferField(entry(iCol, subscript(cColumns, 1))));
                     hColumn.unwrapBufferField().changeValue(new character(""));
                  }
               }
               cRowident.assign((subscript(hBuffer, 1).unwrapBuffer().available()).booleanValue() ? valueOf(subscript(hBuffer, 1).unwrapBuffer().rowID()) : new character(""));
               
               IntegerExpression resExpr8 = new IntegerExpression()
               {
                  public integer execute()
                  {
                     return numEntries(cBufferHandles);
                  }
               };
               ToClause toClause9 = new ToClause(iBuffer, 2, resExpr8);
               
               doTo("loopLabel33", toClause9, new Block((Body) () -> 
               {
                  if ((subscript(hBuffer, iBuffer).unwrapBuffer().available()).booleanValue())
                  {
                     if (_isEqual(indexOf(subscript(cAssignArray, iBuffer), "["), 0))
                     {
                        hRowObject.unwrapBuffer().bufferCopy(subscript(hBuffer, iBuffer), subscript(cExclude, iBuffer), subscript(cAssignArray, iBuffer));
                     }
                     else
                     {
                        ControlFlowOps.invokeWithMode("BufferCopyDBToRO", "IIII", hRowObject, subscript(hBuffer, iBuffer), subscript(cExclude, iBuffer), subscript(cAssignArray, iBuffer));
                     }
                  }
                  else
                  {
                     loopLabel34:
                     for (iCol.assign(1); _isLessThanOrEqual(iCol, numEntries(subscript(cColumns, iBuffer))); iCol.increment())
                     {
                        hColumn.assign(hRowObject.unwrapBuffer().bufferField(entry(iCol, subscript(cColumns, iBuffer))));
                        hColumn.unwrapBufferField().changeValue(new character(""));
                     }
                  }
                  cRowident.assign(concat(cRowident, ",", (subscript(hBuffer, iBuffer).unwrapBuffer().available()).booleanValue() ? valueOf(subscript(hBuffer, iBuffer).unwrapBuffer().rowID()) : new character("")));
               }));
               
               /* - Update of the RowUserProp field  */
               if (_and(isGreaterThan(cEntityFields, ""), () -> gshGenManager.isValid()))
               {
                  /* {get Prop var [handle]} */ 
                  cKeyTableId.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getKeyTableId", targetProcedure()));
                  /* Key Table Entity Mnemonic / _dump-name */
                  /* {get Prop var [handle]} */ 
                  cKeyFields.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getKeyFields", targetProcedure()));
                  cKeyValue.assign("");
                  
                  loopLabel35:
                  for (iKeyLoop.assign(1); _isLessThanOrEqual(iKeyLoop, numEntries(cKeyFields)); iKeyLoop.increment())
                  {
                     ErrorManager.silentErrorEnable();
                     hKeyField.assign(hRowObject.unwrapBuffer().bufferField(entry(iKeyLoop, cKeyFields)));
                     ErrorManager.silentErrorDisable();
                     
                     if ((hKeyField.isValid()).booleanValue())
                     {
                        if (_isNotEqual(cKeyValue, ""))
                        {
                           cKeyValue.assign(concat(cKeyValue, chr(1)));
                        }
                        
                        cKeyValue.assign(concat(cKeyValue, hKeyField.unwrapBufferField().value()));
                     }
                  }
                  
                  iHasAudit.assign(lookup("HasAudit", cEntityFields));
                  iHasComment.assign(lookup("HasComment", cEntityFields));
                  iAutoComment.assign(lookup("AutoComment", cEntityFields));
                  
                  if (_isGreaterThan(iHasAudit, 0))
                  {
                     ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
                     ghProp.assign(ghProp.unwrapBuffer().bufferField("FetchHasAudit"));
                     lFetchHasAudit.assign(ghProp.unwrapBufferField().value());
                     
                     if (_not(isEqual(lFetchHasAudit, true)))
                     {
                        cEntityFields.replaceEntry(iHasAudit, "");
                     }
                  }
                  
                  if (_isGreaterThan(iHasComment, 0))
                  {
                     ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
                     ghProp.assign(ghProp.unwrapBuffer().bufferField("FetchHasComment"));
                     lFetchHasComment.assign(ghProp.unwrapBufferField().value());
                     
                     if (_not(isEqual(lFetchHasComment, true)))
                     {
                        cEntityFields.replaceEntry(iHasComment, "");
                     }
                  }
                  
                  if (_isGreaterThan(iAutoComment, 0))
                  {
                     ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
                     ghProp.assign(ghProp.unwrapBuffer().bufferField("FetchAutoComment"));
                     lFetchAutoComment.assign(ghProp.unwrapBufferField().value());
                     
                     if (_not(isEqual(lFetchAutoComment, true)))
                     {
                        cEntityFields.replaceEntry(iAutoComment, "");
                     }
                  }
                  
                  /* Comments were changed to use CHR(2) instead of CHR(1) due
                       to clashes with the ADM-PROPS delimiter of CHR(1). Now testing
                      for both CHR(1) and CHR(2) for backward compatibility. */
                  if (_isNotEqual(trim(cEntityFields, ","), ""))
                  {
                     if (_isNotEqual(cKeyValue, ""))
                     {
                        ErrorManager.silentErrorEnable();
                        ControlFlowOps.invokeInWithMode("getRecordUserProp", gshGenManager, "IIIO", cKeyTableId, cEntityFields, cKeyValue, cRowUserProp);
                        ErrorManager.silentErrorDisable();
                     }
                     
                     if (_isGreaterThan(numEntries(cKeyValue, chr(1)), 1))
                     {
                        ErrorManager.silentErrorEnable();
                        ControlFlowOps.invokeInWithMode("getRecordUserProp", gshGenManager, "IIIO", cKeyTableId, cEntityFields, replaceAll(cKeyValue, chr(1), chr(2)), cRowUserProp);
                        ErrorManager.silentErrorDisable();
                     }
                  }
               }
               hRowIdent.unwrapBufferField().changeValue(cRowident);
               hRowIdentIdx.unwrapBufferField().changeValue(substring(cRowident, 1, xiRocketIndexLimit));
               hRowNum.unwrapBufferField().changeValue(iRownum);
               hRowUserProp.unwrapBufferField().changeValue(cRowUserProp);
               
               if (_not(lNoCalc))
               {
                  if (_not(lFieldCalc))
                  {
                     ErrorManager.silentErrorEnable();
                     ControlFlowOps.invokeIn("Data.Calculate", targetProcedure());
                     ErrorManager.silentErrorDisable();
                     lNoCalc.assign(isEqual(ErrorManager.getErrorNumber(1), 6456));
                  }
                  
                  if (_and(or(lNoCalc, lFieldCalc), () -> isGreaterThan(cCalculatedColumns, "")))
                  {
                     IntegerExpression resExpr9 = new IntegerExpression()
                     {
                        public integer execute()
                        {
                           return numEntries(cCalculatedColumns);
                        }
                     };
                     ToClause toClause10 = new ToClause(iCalc, 1, resExpr9);
                     
                     doTo("loopLabel36", toClause10, new Block((Body) () -> 
                     {
                        cCalcCol.assign(entry(iCalc, cCalculatedColumns));
                        ErrorManager.silentErrorEnable();
                        hRowObject.unwrapBuffer().bufferField(cCalcCol).unwrapBufferField().changeValue(ControlFlowOps.invokeDynamicFunctionIn(concat("Calculate", cCalcCol), targetProcedure()));
                        ErrorManager.silentErrorDisable();
                        lNoCalc.assign(false);
                        lFieldCalc.assign(true);
                     }));
                  }
               }
            }
            else
            {
               ControlFlowOps.invokeInWithMode("transferDBRow", targetProcedure(), "II", new character("" /* Signal to use the current db row */), iRownum);
               cRowident.assign(hRowIdent.unwrapBufferField().value());
               ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
               ghProp.assign(ghProp.unwrapBuffer().bufferField("SkipTransferDBRow"));
               lSkipApi.assign(ghProp.unwrapBufferField().value());
            }
            
            /* If are the child (0) in a 1-TO-0 relationship we don't want to go here */
            if (_and(isEqual(cToRowident, cRowident), () -> isNotEqual(cRowident, "")) /* Skip if in a 1-TO-0 child */ )
            {
               if (_and(not(plFillBatch), not(lAppendSearch)))
               {
                  if (_isEqual(pcPosition, "BEFORE"))
                  {
                     hRowObject.unwrapBuffer().deleteRecord();
                     rRowid.assign(hRowObject.unwrapBuffer().rowID());
                     piRowsReturned.assign(minus(piRowsReturned, 1));
                     iRownum.assign(minus(iRownum, 1));
                     iRowPos.assign(iRownum);
                     ErrorManager.silentErrorEnable();
                     hRowObject.unwrapBuffer().findFirst(concat("WHERE RowNum = ", valueOf(iRownum)));
                     ErrorManager.silentErrorDisable();
                     leave("loopLabel31");
                  }
                  else
                  {
                     if (_isEqual(pcPosition, "AFTER"))
                     {
                        iRowPos.assign(plus(iRownum, 1));
                        iLoopCount.assign(minus(piRowsToReturn, 1));
                     }
                     else
                     {
                        rRowid.assign(hRowObject.unwrapBuffer().rowID());
                        lLastFound.assign(true);
                     }
                  }
               }
               else
               {
                  if (_isEqual(pcPosition, "BEFORE"))
                  {
                     iRowPos.assign(minus(iRownum, 1));
                  }
                  else
                  {
                     if (_isEqual(pcPosition, "AFTER"))
                     {
                        iRowPos.assign(plus(iRownum, 1));
                     }
                     else
                     {
                        rRowid.assign(hRowObject.unwrapBuffer().rowID());
                     }
                  }
               }
               
               /* Start counting, include current */
               if ((lAppendSearch).booleanValue())
               {
                  iLoopCount.assign(1);
                  lAppendSearch.assign(false);
               }
            }
            
            if ((hReadHandler.isValid()).booleanValue())
            {
               if ((hReadBuffer.isValid()).booleanValue())
               {
                  hReadBuffer.unwrapBuffer().bufferCopy(hRowObject);
                  ControlFlowOps.invokeInWithMode("receiveBuffer", hReadHandler, "II", cUniqueObjectName, hReadBuffer);
               }
               else
               {
                  ControlFlowOps.invokeInWithMode("receivedata", hReadHandler, "III", cUniqueObjectName, cReadcolumns, ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "colStringValues", targetProcedure(), "III", cReadcolumns, cReadFormat, cDelimiter));
               }
               
               if (_or(isEqual(ControlFlowOps.getReturnValue(), "adm-error"), () -> ErrorManager.isError()))
               {
                  lStop.assign(true);
                  /* {get Prop var [handle]} */ 
                  hDataSource.assign(ControlFlowOps.invokeDynamicFunctionIn(handle.class, "getDataSource", targetProcedure()));
                  
                  if ((hDataSource.isValid()).booleanValue())
                  {
                     ControlFlowOps.invokeInWithMode("removeLink", targetProcedure(), "III", hDataSource, new character("Data"), targetProcedure());
                  }
               }
               
               if ((lChildrenForAll).booleanValue())
               {
                  NamedEventManager.publish("DataAvailable", targetProcedure(), "I", new Object[]
                  {
                     "TRANSFER"
                  });
               }
            }
            
            /* if valid readhandler */
            if (_isEqual(cNewFirstResult, ""))
            {
                              /*  .if...else rRowid. usually ?, but NOT in the case where
                                 toRowident was also the FIRST record in this loop  */rRowid.assign(_and(plFromTop, () -> isEqual(cToRowident, "")) ? hRowObject.unwrapBuffer().rowID() : rRowid);
               cNewFirstResult.assign(concat(valueOf(iRownum), ";", hRowIdent.unwrapBufferField().value()));
               
               /* If we have encountered the first record in the dataset, we set 
                       all first props disregarding whther we are forward or appending */
               if ((lFirst).booleanValue())
               {
                  ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
                  ghProp.assign(ghProp.unwrapBuffer().bufferField("FirstResultRow"));
                  ghProp.unwrapBufferField().changeValue(cNewFirstResult);
                  ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
                  ghProp.assign(ghProp.unwrapBuffer().bufferField("FirstRowNum"));
                  ghProp.unwrapBufferField().changeValue(iRownum);
               }
               else
               {
                  if (_isUnknown(cFirstResult))
                  {
                     ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
                     ghProp.assign(ghProp.unwrapBuffer().bufferField("FirstResultRow"));
                     ghProp.unwrapBufferField().changeValue(cNewFirstResult);
                  }
                  /* {set Prop value [handle]} */ 
               }
            }
            
            if ((lLastFound).booleanValue())
            {
               leave("loopLabel31");
            }
            
            if (_and(not(lStop), () -> or(isLessThan(iLoopCount, piRowsToReturn), () -> isEqual(piRowsToReturn, 0))))
            {
               hQuery.unwrapQuery().getNext();
            }
         }));
         
         /* do while */
         if (_isGreaterThan(piRowsReturned, 0))
         {
            /* if positioning is not indicated by now then we want to position to the 
                    last row */
            rRowid.assign(_and(isUnknown(rRowid), () -> isEqual(iRowPos, 0)) ? hRowObject.unwrapBuffer().rowID() : rRowid);
            cNewLastResult.assign(concat(valueOf(iRownum), ";", hRowIdent.unwrapBufferField().value()));
            
            /* If we are off end then set all batch properties disregarding whther we
                  are appending or forward  */
            if ((hQuery.unwrapQuery().isOffEnd()).booleanValue())
            {
               ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
               ghProp.assign(ghProp.unwrapBuffer().bufferField("LastResultRow"));
               ghProp.unwrapBufferField().changeValue(cNewLastResult);
               ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
               ghProp.assign(ghProp.unwrapBuffer().bufferField("LastRowNum"));
               ghProp.unwrapBufferField().changeValue(iRownum);
               hQuery.unwrapQuery().getPrevious();
            }
            else
            {
               /*  set lastResultRoe unless we are appending data to the beginning of 
                        existing data (cLastRow is set above depending on old value) */
               if (_isUnknown(cLastResult))
               {
                  ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
                  ghProp.assign(ghProp.unwrapBuffer().bufferField("LastResultRow"));
                  ghProp.unwrapBufferField().changeValue(cNewLastResult);
               }
               /* {set Prop value [handle]} */ 
               
               ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
               ghProp.assign(ghProp.unwrapBuffer().bufferField("LastDbRowIdent"));
               cLastDbrowident.assign(ghProp.unwrapBufferField().value());
               
               /* if no lastDbrowIdent then look ahead */
               if (_isEqual(cLastDbrowident, ""))
               {
                  hQuery.unwrapQuery().queryForward(1);
                  
                  /* We only get here if we are NOT off end */
                  if ((hQuery.unwrapQuery().isOffEnd()).booleanValue())
                  {
                     ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
                     ghProp.assign(ghProp.unwrapBuffer().bufferField("LastRowNum"));
                     ghProp.unwrapBufferField().changeValue(iRownum);
                  }
                  /* {set Prop value [handle]} */ 
                  
                  hQuery.unwrapQuery().getPrevious();
               }
               else
               {
                  if (_isEqual(cLastDbrowident, entry(2, cNewLastResult, ";")))
                  {
                     ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
                     ghProp.assign(ghProp.unwrapBuffer().bufferField("LastRowNum"));
                     ghProp.unwrapBufferField().changeValue(iRownum);
                  }
                  /* {set Prop value [handle]} */ 
               }
            }
         }
         
         if (_isGreaterThan(iRowPos, 0))
         {
            ErrorManager.silentErrorEnable();
            hRowObject.unwrapBuffer().findFirst(concat("WHERE RowNum = ", valueOf(iRowPos)));
            ErrorManager.silentErrorDisable();
            rRowid.assign(hRowObject.unwrapBuffer().rowID());
         }
         
         if ((plOpen).booleanValue())
         {
            /* {get Prop var [handle]} */ 
            hDataQuery.assign(ControlFlowOps.invokeDynamicFunctionIn(handle.class, "getDataHandle", targetProcedure()));
            
            if (_notUnknown(rRowid))
            {
               hDataQuery.unwrapQuery().queryOpen();
               hDataQuery.unwrapQuery().queryRepositionByID(rRowid);
               
               if (_not(hRowObject.unwrapBuffer().available()))
               {
                  hDataQuery.unwrapQuery().getNext();
               }
               
               if (_isEqual(cAsDivision, "SERVER"))
               {
                  ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
                  ghProp.assign(ghProp.unwrapBuffer().bufferField("PositionForClient"));
                  ghProp.unwrapBufferField().changeValue(hRowIdent.unwrapBufferField().value());
               }
               /* {set Prop value [handle]} */ 
               
               if (_and(not(lChildrenForAll), () -> hReadHandler.isValid()))
               {
                  NamedEventManager.publish("DataAvailable", targetProcedure(), "I", new Object[]
                  {
                     "TRANSFER"
                  });
               }
            }
         }
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * ***********************  Function Implementations****************
    */
   public logical addNotFoundMessage(final character _pcFields_1, final character _pcValues_4, final character _pcOperators_5)
   {
      character pcFields_1 = TypeFactory.initInput(_pcFields_1);
      character pcValues_4 = TypeFactory.initInput(_pcValues_4);
      character pcOperators_5 = TypeFactory.initInput(_pcOperators_5);
      /*------------------------------------------------------------------------------
        Purpose: Add an error message for record not found based on keys and values 
                 in the same format as findRowWhere and other query functions...  
        
        pcFields   - Column names (Comma separated)                    
        pcValues   - corresponding Values (CHR(1) separated)
       pcOperators - FUTURE not currently added to message 
                      Operator - one for all columns
                                      - blank - defaults to (EQ)  
                                      - Use slash to define alternative string operator
                                        EQ/BEGINS etc..
                                    - comma separated for each column/value        
          Notes: The message is for unique finds.  
      ------------------------------------------------------------------------------*/
      character cTables = TypeFactory.character();
      character cKeyerror = TypeFactory.character();
      integer iLoop = TypeFactory.integer();
      
      return function("addNotFoundMessage", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("Tables"));
         cTables.assign(ghProp.unwrapBufferField().value());
         
         IntegerExpression resExpr10 = new IntegerExpression()
         {
            public integer execute()
            {
               return numEntries(pcFields_1);
            }
         };
         ToClause toClause11 = new ToClause(iLoop, 1, resExpr10);
         
         doTo("loopLabel37", toClause11, new Block((Body) () -> 
         {
            cKeyerror.assign(concat(cKeyerror, " ", ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "columnLabel", targetProcedure(), "I", entry(iLoop, pcFields_1)), " ", _isGreaterThanOrEqual(numEntries(pcValues_4, chr(1)), iLoop) ? entry(iLoop, pcValues_4, chr(1)) : new character(" ")));
         }));
         
         /* do iloop = 1 to num-entries(cKeyFields) */
         /* MESSAGE 29: &1 was not found with &2 */
         /* {fnarg <func-name> <arg> [handle] */ 
         ControlFlowOps.invokeInWithMode("addMessage", targetProcedure(), "III", substitute(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "messageNumber", targetProcedure(), "I", 29), cTables, cKeyerror), new unknown(), new unknown());
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical addQueryWhere(final character _pcWhere_6, final character _pcBuffer_6, final character _pcAndOr_5)
   {
      character pcWhere_6 = TypeFactory.initInput(_pcWhere_6);
      character pcBuffer_6 = TypeFactory.initInput(_pcBuffer_6);
      character pcAndOr_5 = TypeFactory.initInput(_pcAndOr_5);
      character cQueryString = TypeFactory.character();
      
      return function("addQueryWhere", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryString"));
         cQueryString.assign(ghProp.unwrapBufferField().value());
         
         /* If no QueryString find the current query */
         if (_or(isEqual(cQueryString, ""), () -> isUnknown(cQueryString)))
         {
            /* {get Prop var [handle]} */ 
            cQueryString.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getQueryWhere", targetProcedure()));
            
            /* If no current Query find the defined base query */
            if (_or(isEqual(cQueryString, ""), () -> isUnknown(cQueryString)))
            {
               /* {get Prop var [handle]} */ 
               cQueryString.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getOpenQuery", targetProcedure()));
            }
         }
         /* cQueryString = "":U */
         cQueryString.assign(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "newQueryValidate", targetProcedure(), "IIII", cQueryString, pcWhere_6, pcBuffer_6, pcAndOr_5));
         
         if (_notUnknown(cQueryString))
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryString"));
            ghProp.unwrapBufferField().changeValue(cQueryString);
         }
         /* {set Prop value [handle]} */ 
         
         returnNormal(notUnknown(cQueryString));
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical assignQuerySelection(final character _pcColumns_4, final character _pcValues_4, final character _pcOperators_5)
   {
      character pcColumns_4 = TypeFactory.initInput(_pcColumns_4);
      character pcValues_4 = TypeFactory.initInput(_pcValues_4);
      character pcOperators_5 = TypeFactory.initInput(_pcOperators_5);
      /*------------------------------------------------------------------------------   
         Purpose: Assigns selection criteria to the query and distributes the 
                  column/value pairs to the corresponding buffer's where-clause. 
                  Each buffer's expression will always be embedded in parenthesis.
         Parameters: 
           pcColumns   - Column names (Comma separated) 
                         
                         Fieldname of a table in the query in the form of 
                         TBL.FLDNM or DB.TBL.FLDNM (only if qualified with db is specified),
                         (RowObject.FLDNM should be used for SDO's)  
                         If the fieldname isn't qualified it checks the tables in 
                         the TABLES property and assumes the first with a match.
                         
           pcValues    - corresponding Values (CHR(1) separated)
           pcOperators - Operator - one for all columns
                                    - blank - defaults to (EQ)  
                                    - Use slash to define alternative string operator
                                      EQ/BEGINS etc..
                                  - comma separated for each column/value       
         Notes:   This procedure is designed to run on the client and to be 
                  called several times to build up the the query's where clause 
                  (storing intermediate results in the QueryString property) before 
                  it is finally used in a Query-Prepare method. 
                  openQuery takes care of the preparation of the QueryString property.
                  The QueryColumns property is used to ensure that each column and 
                  operator only will be added once to the QueryString. The property is 
                  also used to store the offset and length of the corresponding values.
      ------------------------------------------------------------------------------*/
      character cQueryString = TypeFactory.character();
      character cBufferList = TypeFactory.character();
      character cBuffer = TypeFactory.character();
      /* We need the columns name and the parts */
      character cColumn = TypeFactory.character();
      character cColumnName = TypeFactory.character();
      integer iBuffer = TypeFactory.integer();
      integer iColumn = TypeFactory.integer();
      character cUsedNums = TypeFactory.character();
      /* Used to builds the column/value string expression */
      character cBufWhere = TypeFactory.character();
      character cValue = TypeFactory.character();
      character cOperator = TypeFactory.character();
      character cAndOr = TypeFactory.character();
      /* Used to store and maintain offset and length */
      integer iValLength = TypeFactory.integer();
      integer iValPos = TypeFactory.integer();
      integer iExpPos = TypeFactory.integer();
      integer iPos = TypeFactory.integer();
      integer iDiff = TypeFactory.integer();
      character cQueryColumns = TypeFactory.character();
      character cQueryBufCols = TypeFactory.character();
      character cQueryColOp = TypeFactory.character();
      character cChangedValues = TypeFactory.character();
      character cChangedList = TypeFactory.character();
      integer iOldEntries = TypeFactory.integer();
      integer iLowestChanged = TypeFactory.integer();
      integer iBufPos = TypeFactory.integer();
      integer iWhereBufPos = TypeFactory.integer();
      
      return function("assignQuerySelection", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("Tables"));
         cBufferList.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryString"));
         cQueryString.assign(ghProp.unwrapBufferField().value());
         
         /* If no QueryString find the current query */
         if (_or(isEqual(cQueryString, ""), () -> isUnknown(cQueryString)))
         {
            /* {get Prop var [handle]} */ 
            cQueryString.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getQueryWhere", targetProcedure()));
            
            /* If no current Query find the defined base query */
            if (_or(isEqual(cQueryString, ""), () -> isUnknown(cQueryString)))
            {
               /* {get Prop var [handle]} */ 
               cQueryString.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getOpenQuery", targetProcedure()));
            }
         }
         
         /* cQueryString = "":U */
         if (_or(isEqual(cQueryString, ""), () -> isUnknown(cQueryString)))
         {
            returnNormal(false);
         }
         
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryColumns"));
         cQueryColumns.assign(ghProp.unwrapBufferField().value());
         /* cQueryColumns has the form of:
                 BufName1:columns_of_buf1:BufName2:columns_of_buf2...
               
                 Each columns_of_buf has the form of:
                 ColumnName.Operator,ValuePosition,ValueLength
                 
                 The Operator is one of: ">=", "<=","<", ">", "=", "BEGINS", etc.
                 The ValuePosition refers to the character position of the value
                 in an expression: ColumnName Opr Value
                 (The quote is considered part of the value)
                 The length of the value is the number of characters in the string
                 that represents the value (including quotes) */
         cAndOr.assign("AND");
         
         IntegerExpression resExpr11 = new IntegerExpression()
         {
            public integer execute()
            {
               return numEntries(cBufferList);
            }
         };
         ToClause toClause12 = new ToClause(iBuffer, 1, resExpr11);
         
         doTo("loopLabel38", toClause12, new Block((Body) () -> 
         {
            cBufWhere.assign("");
            cBuffer.assign(entry(iBuffer, cBufferList));
            iBufPos.assign(lookup(cBuffer, cQueryColumns, ":"));
            cQueryBufCols.assign(_isGreaterThan(iBufPos, 0) ? entry(plus(iBufPos, 1), cQueryColumns, ":") : new character(""));
            iOldEntries.assign(divide(numEntries(cQueryBufCols), 3));
            cChangedValues.assign(fill(chr(1), minus(iOldEntries, 1)));
            cChangedList.assign("");
            iLowestChanged.assign(0);
            
            IntegerExpression resExpr12 = new IntegerExpression()
            {
               public integer execute()
               {
                  return numEntries(pcColumns_4);
               }
            };
            ToClause toClause13 = new ToClause(iColumn, 1, resExpr12);
            
            doTo("columnLoop", toClause13, new Block((Body) () -> 
            {
               if ((matchesList(cUsedNums, valueOf(iColumn))).booleanValue())
               {
                  next("columnLoop");
               }
               
               ControlFlowOps.invokeInWithMode("obtainExpressionEntries", targetProcedure(), "IIIIIOOO", cBuffer, iColumn, pcColumns_4, pcValues_4, pcOperators_5, cColumn, cOperator, cValue);
               
               if (_isEqual(cColumn, ""))
               {
                  next("columnLoop");
               }
               
               cUsedNums.assign(concat(cUsedNums, (_isEqual(cUsedNums, "") ? new character("") : new character(",")), valueOf(iColumn)));
               /* The Column and operator are unique entries so we must mak sure that  
                          that blank or different styles doesn't get misinterpreted  */
               cQueryColOp.assign(cOperator);
               cQueryColOp.assign(trim(_isEqual(cQueryColOp, "GE") ? new character(">=") : _isEqual(cQueryColOp, "LE") ? new character("<=") : _isEqual(cQueryColOp, "LT") ? new character("<") : _isEqual(cQueryColOp, "GT") ? new character(">") : _isEqual(cQueryColOp, "EQ") ? new character("=") : cQueryColOp));
               cColumnName.assign(entry(numEntries(cColumn, "."), cColumn, "."));
               /* Have the column and operator been added to the querystring
                          (by this function) */
               iPos.assign(lookup(concat(cColumnName, ".", cQueryColOp), cQueryBufCols));
               
               /* If the column + operator was found in the list
                        we build a list of the new values to use when we insert the data
                        into the QueryString further down.
                        We also build a list of the changed numbers, to check if any change 
                        has occured. (The list of new values cannot be checked because any 
                        data may be new data and we don't know the old value) */
               if (_isGreaterThan(iPos, 0))
               {
                  cChangedValues.replaceEntry(new integer(plus(divide(minus(iPos, 1), 3), 1)), chr(1), cValue);
                  iLowestChanged.assign(int64.minimum(iPos, _isEqual(iLowestChanged, 0) ? iPos : iLowestChanged));
                  cChangedList.assign(concat(cChangedList, (_isEqual(cChangedList, "") ? new character("") : new character(",")), valueOf(new integer(plus(divide(minus(iPos, 1), 3), 1)))));
               }
               else
               {
                  cBufWhere.assign(concat(cBufWhere, (_isEqual(cBufWhere, "") ? new character("") : concat(" ", cAndOr, " ")), cColumn, " ", cOperator, " ", cValue));
                  /* Calculate the temporary offset of this columns value. 
                              We (Who are we?) will justify it after the expression has been 
                              added to the whereclause, because even if we know the buffer's 
                              position, the expression may or may not need and/where */
                  iValPos.assign(minus(length(cBufWhere), length(cValue)));
                  /* Store the ColumName and operator with period as delimiter and 
                                add the position and length as separate entries*/
                  cQueryBufCols.assign(concat(cQueryBufCols, (_isNotEqual(cQueryBufCols, "") ? new character(",") : new character("")), cColumnName, ".", cQueryColOp, ",", valueOf(iValPos), ",", valueOf(length(cValue))));
                  /* Ensure that the list used to log changes have correct number of
                                entries (Probably only necessary if the SAME column and operator
                                         appears a second time in the same call, which is unlikely)
                                         */
                  cChangedValues.assign(concat(cChangedValues, chr(1)));
               }
            }));
            
            iWhereBufPos.assign(indexOf(concat(cQueryString, " "), concat(" ", cBuffer, " ")));
            iPos.assign(indexOf(cQueryString, concat(" ", cBuffer, ",")));
            iWhereBufPos.assign(plus((_and(isGreaterThan(iWhereBufPos, 0), () -> isGreaterThan(iPos, 0)) ? int64.minimum(iPos, iWhereBufPos) : int64.maximum(iPos, iWhereBufPos)), 1));
            iDiff.assign(0);
            
            /* We have a new expression */
            if (_isNotEqual(cBufWhere, ""))
            {
               cQueryString.assign(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "newWhereClause", targetProcedure(), "IIII", cBuffer, cBufWhere, cQueryString, "AND"));
               /* get the offset of the new expression */
               iExpPos.assign(indexOf(cQueryString, cBufWhere, iWhereBufPos));
               
               loopLabel39:
               for (iColumn.assign(minus(multiply(plus(iOldEntries, 1), 3), 2)); _isLessThanOrEqual(iColumn, numEntries(cQueryBufCols)); iColumn.assign(plus(iColumn, 3)))
               {
                  cQueryBufCols.replaceEntry(plus(iColumn, 1), valueOf(plus(new integer(entry(plus(iColumn, 1), cQueryBufCols)), minus(iExpPos, iWhereBufPos))));
               }
            }
            
            /* if cbufwhere <> '' do */
            if (_isGreaterThan(iLowestChanged, 0))
            {
               loopLabel40:
               for (iColumn.assign(iLowestChanged); _isLessThanOrEqual(iColumn, numEntries(cQueryBufCols)); iColumn.assign(plus(iColumn, 3)))
               {
                  iValPos.assign(new integer(entry(plus(iColumn, 1), cQueryBufCols)));
                  iValLength.assign(new integer(entry(plus(iColumn, 2), cQueryBufCols)));
                  iValPos.assign(plus(iValPos, iDiff));
                  
                  if ((matchesList(cChangedList, valueOf(new integer(plus(divide(minus(iColumn, 1), 3), 1))))).booleanValue())
                  {
                     cValue.assign(entry(new integer(plus(divide(minus(iColumn, 1), 3), 1)), cChangedValues, chr(1)));
                     cQueryString.replace(cValue, plus(iValPos, iWhereBufPos), iValLength);
                     iDiff.assign(plus(iDiff, minus(length(cValue), iValLength)));
                     iValLength.assign(length(cValue));
                  }
                  
                  cQueryBufCols.replaceEntry(plus(iColumn, 1), valueOf(iValPos));
                  cQueryBufCols.replaceEntry(plus(iColumn, 2), valueOf(iValLength));
               }
            }
            
            /* else if ilowestchanged do icolumn = ilowestChanged to num-entries */
            /* If the buffer has no entry in QueryColumns we append the new entry 
                   The order in Querycolumns is NOT dependent of the order in the query */
            if (_isNotEqual(cQueryBufCols, ""))
            {
               if (_isEqual(iBufPos, 0))
               {
                  cQueryColumns.assign(concat(cQueryColumns, (_isEqual(cQueryColumns, "") ? new character("") : new character(":")), cBuffer, ":", cQueryBufCols));
               }
               else
               {
 /* There is already a entry for this buffer */
                  cQueryColumns.replaceEntry(plus(iBufPos, 1), ":", cQueryBufCols);
               }
            }
         }));
         
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryColumns"));
         ghProp.unwrapBufferField().changeValue(cQueryColumns);
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryString"));
         ghProp.unwrapBufferField().changeValue(cQueryString);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical bufferCompareDbtoRo(final handle _phRowObjUpd_1, final handle _phBuffer_1, final character _pcExcludes_1, final character _pcAssigns_1)
   {
      handle phRowObjUpd_1 = TypeFactory.initInput(_phRowObjUpd_1);
      handle phBuffer_1 = TypeFactory.initInput(_phBuffer_1);
      character pcExcludes_1 = TypeFactory.initInput(_pcExcludes_1);
      character pcAssigns_1 = TypeFactory.initInput(_pcAssigns_1);
      /*------------------------------------------------------------------------------
        Purpose:    Perform a BUFFER-COMPARE of a database buffer to a RowObjectUpd 
                    buffer.  In particular, if an assign-list is used to map 
                    individual array elements from the database buffer to the 
                    RowObjectUpd buffer, this procedure will ensure that the values 
                    are compared correctly.
        Parameters: INPUT phRowObjUpd AS HANDLE - Handle to the RowObjectUpd buffer
                    INPUT phBuffer    AS HANDLE - Handle to the database buffer
                    INPUT pcExcludes  AS CHAR   - Comma-delimited list of fields to 
                                                  be excluded from the BUFFER-COMPARE
                    INPUT pcAssigns   AS CHAR   - Comma-delimited list of field pairs
                                                  to be individually compared; The 
                                                  field pairs are mappings of fields
                                                  from the target/source buffers where
                                                  the field names differ.
          Notes:    The primary purpose of this procedure is to detect when 
                    individual array fields are referenced in the assign-list 
                    (pcAssigns) from the database buffer and the ensure that they
                    are compared properly.  
      ------------------------------------------------------------------------------*/
      character cTemp = TypeFactory.character();
      handle hField = TypeFactory.handle();
      handle hField2 = TypeFactory.handle();
      integer iCtr = TypeFactory.integer();
      integer iExt = TypeFactory.integer();
      integer iPos = TypeFactory.integer();
      logical lSame = TypeFactory.logical();
      
      return function("bufferCompareDBToRO", logical.class, new Block((Body) () -> 
      {
         if (_isNotEqual(indexOf(pcAssigns_1, "["), 0))
         {
            lSame.assign(phBuffer_1.unwrapBuffer().bufferCompare(phRowObjUpd_1, new character("case-sensitive"), pcExcludes_1));
            
            if ((lSame).booleanValue())
            {
               compareAssigns:
               for (iCtr.assign(2); _isLessThanOrEqual(iCtr, numEntries(pcAssigns_1)); iCtr.assign(plus(iCtr, 2)))
               {
                  cTemp.assign(entry(iCtr, pcAssigns_1));
                  iPos.assign(indexOf(cTemp, "["));
                  
                  if (_isGreaterThan(iPos, 0))
                  {
                     iExt.assign(new integer(entry(1, substring(cTemp, plus(iPos, 1)), "]")));
                     cTemp.assign(substring(cTemp, 1, minus(iPos, 1)));
                  }
                  else
                  {
                     iExt.assign(0);
                  }
                  
                  hField.assign(phRowObjUpd_1.unwrapBuffer().bufferField(entry(minus(iCtr, 1), pcAssigns_1)));
                  hField2.assign(phBuffer_1.unwrapBuffer().bufferField(cTemp));
                  lSame.assign(isEqual(hField.unwrapBufferField().value(), hField2.unwrapBufferField().value(iExt)));
                  
                  if (_not(lSame))
                  {
                     break compareAssigns;
                  }
               }
            }
         }
         else
         {
            lSame.assign(phBuffer_1.unwrapBuffer().bufferCompare(phRowObjUpd_1, new character("case-sensitive"), pcExcludes_1, pcAssigns_1));
         }
         returnNormal(lSame);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character bufferWhereClause(final character _pcBuffer_6, final character _pcWhere_6)
   {
      character pcBuffer_6 = TypeFactory.initInput(_pcBuffer_6);
      character pcWhere_6 = TypeFactory.initInput(_pcWhere_6);
      /*------------------------------------------------------------------------------
        Purpose:     Returns the complete query where clause for a specified buffer
                     INCLUDING leading and trailing blanks.
                     EXCLUDING commas and period.                            
        Parameters:  pcBuffer     - Buffer. See notes
                     pcWhere      - A complete query:prepare-string.
                                  - ? use the current query 
                                    1. QueryString, 2 QueryWhere 3 OpenQuery.    
        Notes:       This is supported as a 'utility function' that doesn't use any 
                     properties. 
                     if target-procedure = super the passed buffer's qualification 
                     MUST match the query's. 
                     If target-procedure <> super the buffer will be corrected 
                     IF it exists in the object's query.  
                  -  RETURNs the expression immediately when found. 
                     RETURNs '' at bottom if nothing is found. 
      ------------------------------------------------------------------------------*/
      integer iComma = TypeFactory.integer();
      integer iCount = TypeFactory.integer();
      integer iStart = TypeFactory.integer();
      character cString = TypeFactory.character();
      character cFoundWhere = TypeFactory.character();
      character cNextWhere = TypeFactory.character();
      character cTargetType = TypeFactory.character();
      character cBuffer = TypeFactory.character();
      
      return function("bufferWhereClause", character.class, new Block((Body) () -> 
      {
         /* If unkown value is passed used the existing query string */
         if (_isUnknown(pcWhere_6))
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryString"));
            pcWhere_6.assign(ghProp.unwrapBufferField().value());
            
            /* If no QueryString find the current query */
            if (_or(isEqual(pcWhere_6, ""), () -> isUnknown(pcWhere_6)))
            {
               /* {get Prop var [handle]} */ 
               pcWhere_6.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getQueryWhere", targetProcedure()));
               
               /* If no current Query find the defined base query */
               if (_or(isEqual(pcWhere_6, ""), () -> isUnknown(pcWhere_6)))
               {
                  /* {get Prop var [handle]} */ 
                  pcWhere_6.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getOpenQuery", targetProcedure()));
               }
            }
         }
         cString.assign(rightTrim(pcWhere_6, " "));
         iStart.assign(1);
         
         /* Keep our promises and ensure that trailing blanks BEFORE the period are 
             returned, but remove the period and trailing blanks AFTER it. 
             If the length of right-trim with blank and blank + period is the same 
             then there is no period, so just use the passed pcWhere as is. 
             (Otherwise the remaining period is right-trimmed with comma further down)*/
         if (_isEqual(length(cString), length(rightTrim(pcWhere_6, ". "))))
         {
            cString.assign(pcWhere_6);
         }
         
         /* This is the guts of what used to be in newQueryWhere, which used to be 
             called without IN TARGET-PROCEDURE... so if target is a super we just keep 
             the old requirement of correct qual, otherwise we try to resolve it,
             but we continue the search also if it is not found in the current query 
             in order to support this and newQueryWhere as utilities for any buffer and 
             query.  */
         cTargetType.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getObjectType", targetProcedure()));
         
         if (_isNotEqual(cTargetType, "SUPER"))
         {
            cBuffer /* {fnarg <func-name> <arg> [handle] */ .assign(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "resolveBuffer", targetProcedure(), "I", pcBuffer_6));
            
            if (_and(isNotEqual(cBuffer, ""), () -> notUnknown(cBuffer)))
            {
               pcBuffer_6.assign(cBuffer);
            }
         }
         
         doWhile("loopLabel41", () -> new logical(true), new Block((Body) () -> 
         {
            iComma.assign(indexOf(cString, ","));
            
            /* If a comma was found we split the string into cFoundWhere and cNextwhere */
            if (_isNotEqual(iComma, 0))
            {
               cFoundWhere.assign(concat(cFoundWhere, substring(cString, 1, iComma)));
               cNextWhere.assign(substring(cString, plus(iComma, 1)));
               iCount.assign(plus(iCount, iComma));
            }
            else
            {
               /* cFoundWhere is blank if this is the first time or if we have moved on 
                       to the next buffer's where clause
                       If cFoundwhere is not blank the last comma that was used to split 
                       the string into cFoundwhere and cNextwhere was not a join, so we set 
                       them together again.  */
               cFoundWhere.assign(_isEqual(cFoundWhere, "") ? cString : concat(cFoundWhere, cNextWhere));
            }
            
            /* We have a complete table whereclause if there are no more commas
                  or the next whereclause starts with each,first or last */
            if (_or(isEqual(iComma, 0), () -> matchesList("EACH,FIRST,LAST", entry(1, trim(cNextWhere), " "))))
            {
               cFoundWhere.assign(rightTrim(cFoundWhere, ",."));
               
               if (_isEqual( /* {fnarg <func-name> <arg> [handle] */ ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "whereClauseBuffer", targetProcedure(), "I", cFoundWhere), pcBuffer_6))
               {
                  returnNormal(cFoundWhere);
               }
               else
               {
                  cFoundWhere.assign("");
                  iStart.assign(plus(iCount, 1));
               }
               /* We're moving on to the next whereclause so reset cFoundwhere */
               
               /* No table found and we are at the end so we need to get out of here */
               if (_isEqual(iComma, 0))
               {
                  leave("loopLabel41");
               }
            }
            /* if iComma = 0 or can-do(EACH,FIRST,LAST */
            cString.assign(cNextWhere);
         }));
         
         /* do while true. */
         returnNormal("");
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical closeQuery()
   {
      /*------------------------------------------------------------------------------
        Purpose:     Closes the database query.
      
        Parameters:  <none>
        
        Notes:       closeQuery should not be executed on the client side of a 
                     split SmartDataObject because the database query only exists
                     on the server side.  No harm will be done, but nothing will
                     be closed.
      ------------------------------------------------------------------------------*/
      handle hQuery = TypeFactory.handle();
      
      return function("closeQuery", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryHandle"));
         hQuery.assign(ghProp.unwrapBufferField().value());
         
         if ((hQuery.isValid()).booleanValue())
         {
            hQuery.unwrapQuery().queryClose();
            returnNormal(true);
         }
         else
         {
            returnNormal(false);
         }
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character columnDataType(final character _pcColumn_8)
   {
      character pcColumn_8 = TypeFactory.initInput(_pcColumn_8);
      /*------------------------------------------------------------------------------
        Purpose:     Returns the data-type of a database column.  
         Parameters:  pcColumn - Database fieldname.  
                     Can be in the form of: DB.TBL.FLDNM, TBL.FLDNM or FLDNM.  
                     (If not qualifed the FIRST ref in query will be used)
        Notes:       Is capable of processing a pcColumn specified with brackets.
      ------------------------------------------------------------------------------*/
      handle hField = TypeFactory.handle();
      
      return function("columnDataType", character.class, new Block((Body) () -> 
      {
         hField /* {fnarg <func-name> <arg> [handle] */ .assign(ControlFlowOps.invokeDynamicFunctionInWithMode(handle.class, "dbColumnHandle", targetProcedure(), "I", pcColumn_8));
         returnNormal((hField.isValid()).booleanValue() ? hField.unwrapCommonField().getDataType() : new character());
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character columnDbColumn(final character _pcColumn_8)
   {
      character pcColumn_8 = TypeFactory.initInput(_pcColumn_8);
      /*----------------------------------------------------------------------------
        Purpose:   Returns the qualified database name ([DB.]TBL.FLDNM) mapped to 
                   the RowObject column specified in pcColumn
        Parameter: pcColumn - Rowobject name to look up
      -----------------------------------------------------------------------------*/
      character cName = TypeFactory.character();
      character cAssignList = TypeFactory.character();
      character cDataColumnsByTable = TypeFactory.character();
      character cTables = TypeFactory.character();
      integer iTable_1 = TypeFactory.integer();
      character cAssign = TypeFactory.character();
      
      return function("columnDbColumn", character.class, new Block((Body) () -> 
      {
         if (_begins(pcColumn_8, "RowObject."))
         {
            pcColumn_8.assign(entry(2, pcColumn_8, "."));
         }
         
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("Tables"));
         cTables.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("AssignList"));
         cAssignList.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DataColumnsByTable"));
         cDataColumnsByTable.assign(ghProp.unwrapBufferField().value());
         
         IntegerExpression resExpr13 = new IntegerExpression()
         {
            public integer execute()
            {
               return numEntries(cTables);
            }
         };
         ToClause toClause14 = new ToClause(iTable_1, 1, resExpr13);
         
         doTo("loopLabel42", toClause14, new Block((Body) () -> 
         {
            cAssign.assign(entry(iTable_1, cAssignList, ";"));
            
            /*Flds for one table */
            if (_isNotEqual(cAssign, ""))
            {
               /* If the RowObject name is different from dbname it's in the assignList*/
               cName.assign(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "mappedEntry", targetProcedure(), "IIII", pcColumn_8, cAssign, true, "," /* delimiter */));
               
               if (_notUnknown(cName))
               {
                  returnNormal(concat(entry(iTable_1, cTables), ".", cName));
               }
            }
            
            /* cAssign <> '' */
            if ((matchesList(entry(iTable_1, cDataColumnsByTable, ";"), pcColumn_8)).booleanValue())
            {
               returnNormal(concat(entry(iTable_1, cTables), ".", pcColumn_8));
            }
         }));
         
         /* END DO iTable... */
         returnNormal("");
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character columnPhysicalColumn(final character _pcColumn_8)
   {
      character pcColumn_8 = TypeFactory.initInput(_pcColumn_8);
      /*----------------------------------------------------------------------------
        Purpose:   Returns the qualified physical name ([DB.]TBL.FLDNM) mapped to 
                   the RowObject column specified in pcColumn
        Parameter: pcColumn - Rowobject name to look up
      -----------------------------------------------------------------------------*/
      character cPhysicalTables = TypeFactory.character();
      character cTables = TypeFactory.character();
      character cBufferColumn = TypeFactory.character();
      character cTableName = TypeFactory.character();
      character cPhysTableName = TypeFactory.character();
      character cPhysicalColumn = TypeFactory.character();
      integer iTable_1 = TypeFactory.integer();
      
      return function("columnPhysicalColumn", character.class, new Block((Body) () -> 
      {
         /* Resolve the buffer column first */
         cBufferColumn /* {fnarg <func-name> <arg> [handle] */ .assign(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "columnDbColumn", targetProcedure(), "I", pcColumn_8));
         
         if (_isGreaterThan(cBufferColumn, ""))
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("PhysicalTables"));
            cPhysicalTables.assign(ghProp.unwrapBufferField().value());
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("Tables"));
            cTables.assign(ghProp.unwrapBufferField().value());
            cTableName.assign(substring(cBufferColumn, 1, minus(lastIndexOf(cBufferColumn, "."), 1)));
            iTable_1.assign(lookup(cTableName, cTables));
            cPhysTableName.assign(entry(iTable_1, cPhysicalTables));
            cPhysicalColumn.assign(replaceAll(cBufferColumn, concat(cTableName, "."), concat(cPhysTableName, ".")));
         }
         
         returnNormal(cPhysicalColumn);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character columnPhysicalTable(final character _pcColumn_8)
   {
      character pcColumn_8 = TypeFactory.initInput(_pcColumn_8);
      /*------------------------------------------------------------------------------
        Purpose:     Returns [dbname.]table of a rowobject or database column.  
        Parameters:  pcColumn - Database fieldname.  
                     Can be in the form of: DB.TBL.FLDNM, TBL.FLDNM or FLDNM.  
                     (If not qualifed the FIRST ref in query will be used)
        Notes:     - Use to ensure and fix a column reference according to 
                     the query's use of database qualification. 
                     (See dbColumnHandle for rules). 
                   - Supports a pcColumn specified with brackets.              
      ------------------------------------------------------------------------------*/
      character cPhysicalTables = TypeFactory.character();
      character cTables = TypeFactory.character();
      character cTableName = TypeFactory.character();
      character cPhysicalTableName = TypeFactory.character();
      integer iTable_1 = TypeFactory.integer();
      
      return function("columnPhysicalTable", character.class, new Block((Body) () -> 
      {
         /* Resolve the table from the passed column */
         cTableName /* {fnarg <func-name> <arg> [handle] */ .assign(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "columnTable", targetProcedure(), "I", pcColumn_8));
         
         if (_isGreaterThan(cTableName, ""))
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("PhysicalTables"));
            cPhysicalTables.assign(ghProp.unwrapBufferField().value());
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("Tables"));
            cTables.assign(ghProp.unwrapBufferField().value());
            
            if (_isGreaterThan(cPhysicalTables, ""))
            {
               iTable_1.assign(lookup(cTableName, cTables));
               cPhysicalTableName.assign(entry(iTable_1, cPhysicalTables));
            }
            else
            {
               cPhysicalTableName.assign(cTableName);
            }
         }
         returnNormal(cPhysicalTableName);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character columnProps(final character _pcColList_1, final character _pcPropList_1)
   {
      character pcColList_1 = TypeFactory.initInput(_pcColList_1);
      character pcPropList_1 = TypeFactory.initInput(_pcPropList_1);
      /*------------------------------------------------------------------------------
        Purpose:     Returns a list of all requested properties for all the requested
                     columns of the RowObject table.  If the pcColList parameter is 
                     "*", then all the columns in the RowObject Column List.
        
        Parameters:
          INPUT pcColList  - comma-seperated list of column names for which properties
                             are to be returned.  "*" indicates all of the columns in
                             the DataObject.
          INPUT pcPropList - comma-separated list of properties to be returned.
                             Valid list items are: DataType, Extent, Format, Help,
                             Initial, Label, Modified, PrivateData,
                             QuerySelection, ReadOnly, StringValue,ValExp, ValMsg
                             and Width.
        
        Notes:       Returns a character string of <col1Props> CHR(3) <col2Props> ...
                     where each <colnProps> is of the form:
                        <columnName> CHR(4) Prop1Value> CHR(4) Prop2Value ... 
      ------------------------------------------------------------------------------*/
      integer iCol = TypeFactory.integer();
      integer iProp = TypeFactory.integer();
      integer iColCount = TypeFactory.integer();
      integer iPropCount = TypeFactory.integer();
      character cColumn = TypeFactory.character();
      character cProp = TypeFactory.character();
      character cColProps = TypeFactory.character("");
      character cPropValue = TypeFactory.character();
      
      return function("columnProps", character.class, new Block((Body) () -> 
      {
         if (_isEqual(pcColList_1, "*"))
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("DataColumns"));
            pcColList_1.assign(ghProp.unwrapBufferField().value());
         }
         /* {get Prop var [handle]} */ 
         
         iColCount.assign(numEntries(pcColList_1));
         iPropCount.assign(numEntries(pcPropList_1));
         
         loopLabel43:
         for (iCol.assign(1); _isLessThanOrEqual(iCol, iColCount); iCol.increment())
         {
            cColumn.assign(trim(entry(iCol, pcColList_1)));
            cColProps.assign(concat(cColProps, cColumn, chr(4)));
            
            ToClause toClause15 = new ToClause(iProp, 1, iPropCount);
            
            doTo("loopLabel44", toClause15, new Block((Body) () -> 
            {
               cProp.assign(trim(entry(iProp, pcPropList_1)));
               ErrorManager.silentErrorEnable();
               /* Note that for now at least we suppress errors and leave it up
                      to the caller to make sure that all column names and properties 
                      are valid.*/
               cPropValue.assign(valueOf(ControlFlowOps.invokeDynamicFunctionInWithMode(concat("column", cProp), targetProcedure(), "I", cColumn)));
               ErrorManager.silentErrorDisable();
               cColProps.assign(concat(cColProps, (_isUnknown(cPropValue) ? new character("") : cPropValue), (_isLessThan(iProp, iPropCount) ? chr(4) : new character(""))));
            }));
            
            /* END DO iProp */
            cColProps.assign(concat(cColProps, (_isLessThan(iCol, iColCount) ? chr(3) : new character(""))));
         }
         
         /* END DO iCol  */
         returnNormal(cColProps);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character columnQuerySelection(final character _pcColumn_8)
   {
      character pcColumn_8 = TypeFactory.initInput(_pcColumn_8);
      /*------------------------------------------------------------------------------
        Purpose:     Returns a CHR(1) separated string with ALL operators and values 
                     that has been added to the Query for this column using the 
                     assignQuerySelection method. 
                     
                     Example: If the query contains 'custnum > 5 and custnum < 9' 
                              this function will return (chr(1) is shown as '|'): 
                              '>|5|<|9'    
                                    
        Parameters:
         INPUT pcColumn - Fieldname of a table in the query in the form of 
                          TBL.FLDNM or DB.TBL.FLDNM (only if qualified with db),
                          If the fieldname isn't qualified it checks the tables in 
                          the TABLES property and assumes the first with a match.
       
        Notes:       The data returned reflects the QueryString/QueryColumns properties, 
                     which is maintained by the assignQuerySelection. These values
                     may not have been used in an openQuery yet.
      ------------------------------------------------------------------------------*/
      character cQueryString = TypeFactory.character();
      character cBuffer = TypeFactory.character();
      character cBufferList = TypeFactory.character();
      character cColumn = TypeFactory.character();
      integer iBuffer = TypeFactory.integer();
      character cValue = TypeFactory.character();
      character cOperator = TypeFactory.character();
      /* Used to store and maintain offset and length */
      integer iValLength = TypeFactory.integer();
      integer iValPos = TypeFactory.integer();
      character cString = TypeFactory.character();
      character cQueryColumns = TypeFactory.character();
      character cQueryBufCols = TypeFactory.character();
      integer iBufPos = TypeFactory.integer();
      integer iColPos = TypeFactory.integer();
      integer iPos = TypeFactory.integer();
      integer iNumEnt = TypeFactory.integer();
      character cSelection = TypeFactory.character();
      integer iWhereBufPos = TypeFactory.integer();
      
      return function("columnQuerySelection", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryString"));
         cQueryString.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryColumns"));
         cQueryColumns.assign(ghProp.unwrapBufferField().value());
         
         /* If the properties are blank we return immediately with blank */
         if (_or(isEqual(cQueryString, ""), () -> isEqual(cQueryColumns, "")))
         {
            returnNormal("");
         }
         
         iNumEnt.assign(numEntries(pcColumn_8, "."));
         
         /* If the column is qualified add the buffer part of it to the bufferlist
              which then will be the only entry in the list for the loop below */
         if (_isGreaterThan(iNumEnt, 1))
         {
            cBufferList.assign(substring(pcColumn_8, 1, minus(lastIndexOf(pcColumn_8, "."), 1)));
            cBufferList /* {fnarg <func-name> <arg> [handle] */ .assign(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "resolveBuffer", targetProcedure(), "I", cBufferList));
            cColumn.assign(entry(iNumEnt, pcColumn_8, "."));
         }
         else
         {
            cColumn.assign(pcColumn_8);
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("Tables"));
            cBufferList.assign(ghProp.unwrapBufferField().value());
         }
         
         loopLabel45:
         for (iBuffer.assign(1); _isLessThanOrEqual(iBuffer, numEntries(cBufferList)); iBuffer.increment())
         {
            cBuffer.assign(entry(iBuffer, cBufferList));
            iBufPos.assign(lookup(cBuffer, cQueryColumns, ":"));
            cQueryBufCols.assign(_isGreaterThan(iBufPos, 0) ? entry(plus(iBufPos, 1), cQueryColumns, ":") : new character(""));
            iWhereBufPos.assign(indexOf(concat(cQueryString, " "), concat(" ", cBuffer, " ")));
            iPos.assign(indexOf(cQueryString, concat(" ", cBuffer, ",")));
            iWhereBufPos.assign(plus((_and(isGreaterThan(iWhereBufPos, 0), () -> isGreaterThan(iPos, 0)) ? int64.minimum(iPos, iWhereBufPos) : int64.maximum(iPos, iWhereBufPos)), 1));
            iColPos.assign(1);
            
            loopLabel46:
            while (_isGreaterThan(iColPos, 0))
            {
               /* Add comma to cQueryBufCols to find first entry and search for ,column.*/
               iColPos.assign(indexOf(concat(",", cQueryBufCols), concat(",", cColumn, "."), iColPos));
               
               if (_isGreaterThan(iColPos, 0))
               {
                  iColPos.assign(plus(plus(iColPos, length(cColumn)), 1));
                  cString.assign(substring(cQueryBufCols, iColPos));
                  cOperator.assign(entry(1, cString));
                  iValPos.assign(new integer(entry(2, cString)));
                  iValLength.assign(new integer(entry(3, cString)));
                  cValue.assign(substring(cQueryString, plus(iValPos, iWhereBufPos), iValLength));
                  /* replace escaped single quotes */
                  cValue.assign((replaceAll(cValue, "~'", "'")));
                  /* From 9.1B the quote is included in the pos/length */
                  cValue.assign(_not(begins(cValue, "'")) ? cValue : substring(cValue, 2, minus(length(cValue), 2)));
                  cSelection.assign(concat(cSelection, chr(1), cOperator, chr(1), cValue));
               }
            }
            
            /* do while icolpos > 0 */
            if (_isNotEqual(cSelection, ""))
            {
               break loopLabel45;
            }
         }
         
         /* do ibuffer = 1 to num-entries */
         returnNormal(leftTrim(cSelection, chr(1)));
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character columnTable(final character _pcColumn_8)
   {
      character pcColumn_8 = TypeFactory.initInput(_pcColumn_8);
      /*------------------------------------------------------------------------------
        Purpose:     Returns [dbname.]table or buffer of a database column.  
        Parameters:  pcColumn - Database fieldname.  
                     Can be in the form of: DB.TBL.FLDNM, TBL.FLDNM or FLDNM.  
                     (If not qualifed the FIRST ref in query will be used)
        Notes:     - Use to ensure and fix a column reference according to 
                     the query's use of database qualification. 
                     (See dbColumnHandle for rules). 
                   - Data.p overrides this for unqualified (RowObject) columns, but
                     calls this for qualified fields.   
                   - Is capable of processing a pcColumn specified with brackets.              
      ------------------------------------------------------------------------------*/
      handle hField = TypeFactory.handle();
      logical lUseDbQual = TypeFactory.logical();
      
      return function("columnTable", character.class, new Block((Body) () -> 
      {
         /* {get Prop var [handle]} */ 
         lUseDbQual.assign(ControlFlowOps.invokeDynamicFunctionIn(logical.class, "getUseDbQualifier", targetProcedure()));
         hField /* {fnarg <func-name> <arg> [handle] */ .assign(ControlFlowOps.invokeDynamicFunctionInWithMode(handle.class, "dbColumnHandle", targetProcedure(), "I", pcColumn_8));
         returnNormal((hField.isValid()).booleanValue() ? concat(((lUseDbQual).booleanValue() ? concat(hField.unwrapDatabaseInfo().getDbName(), ".") : new character("")), hField.unwrapDatabaseInfo().getTable()) : new character(""));
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character columnValMsg(final character _pcColumn_8)
   {
      character pcColumn_8 = TypeFactory.initInput(_pcColumn_8);
      /*------------------------------------------------------------------------------
        Purpose:     Returns the validation message of a database column.  
        Parameters:  pcColumn - Database fieldname.  
                     Can be in the form of: DB.TBL.FLDNM, TBL.FLDNM or FLDNM.  
                     (If not qualifed the FIRST ref in query will be used)
        Notes:       Is capable of processing a pcColumn specified with brackets.
      ------------------------------------------------------------------------------*/
      handle hField = TypeFactory.handle();
      
      return function("columnValMsg", character.class, new Block((Body) () -> 
      {
         hField /* {fnarg <func-name> <arg> [handle] */ .assign(ControlFlowOps.invokeDynamicFunctionInWithMode(handle.class, "dbColumnHandle", targetProcedure(), "I", pcColumn_8));
         returnNormal((hField.isValid()).booleanValue() ? hField.unwrapBufferField().validateMessage() : new character());
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character colValues(final character _pcViewColList_1)
   {
      character pcViewColList_1 = TypeFactory.initInput(_pcViewColList_1);
      /*------------------------------------------------------------------------------
        Purpose:     Formats into character strings (using the field format 
                     specification) a row of values from the current row of the 
                     database Query for the specified column list.
      
        Parameters:
          pcViewColList- comma-separated list of column names whose values are to be
                         returned.
        
        Notes:       Passes back a CHR(1)-separated list of formatted values preceded
                     by the RowIdent code (a comma-separated list) as the first value 
                     in all cases.
      ------------------------------------------------------------------------------*/
      character cColValues = TypeFactory.character("");
      integer iCol = TypeFactory.integer();
      handle hColumn = TypeFactory.handle();
      integer iColCount = TypeFactory.integer();
      handle hBuffer = TypeFactory.handle();
      handle hQuery = TypeFactory.handle();
      integer iTable_1 = TypeFactory.integer();
      
      return function("colValues", character.class, new Block((Body) () -> 
      {
         iColCount.assign(numEntries(pcViewColList_1));
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryHandle"));
         hQuery.assign(ghProp.unwrapBufferField().value());
         hBuffer.assign(hQuery.unwrapQuery().bufferHandle(1));
         
         if ((hBuffer.unwrapBuffer().available()).booleanValue())
         {
            loopLabel47:
            for (iTable_1.assign(1); _isLessThanOrEqual(iTable_1, hQuery.unwrapQuery().numBuffers()); iTable_1.increment())
            {
               hBuffer.assign(hQuery.unwrapQuery().bufferHandle(iTable_1));
               cColValues.assign(concat(cColValues, ",", valueOf(hBuffer.unwrapBuffer().rowID())));
            }
            
            columnLoop:
            for (iCol.assign(1); _isLessThanOrEqual(iCol, iColCount); iCol.increment())
            {
               cColValues.assign(concat(cColValues, chr(1)));
               
               loopLabel48:
               for (iTable_1.assign(1); _isLessThanOrEqual(iTable_1, hQuery.unwrapQuery().numBuffers()); iTable_1.increment())
               {
                  /* Search buffers until the column found. */
                  hBuffer.assign(hQuery.unwrapQuery().bufferHandle(iTable_1));
                  ErrorManager.silentErrorEnable();
                  hColumn.assign(hBuffer.unwrapBuffer().bufferField(entry(iCol, pcViewColList_1)));
                  ErrorManager.silentErrorDisable();
                  
                  if ((hColumn.isValid()).booleanValue())
                  {
                     cColValues.assign(concat(cColValues, rightTrim(valueOf(hColumn.unwrapBufferField().value()))));
                     continue columnLoop;
                  }
               }
            }
            
            /* END DO iCol            */
            returnNormal(cColValues);
         }
         else
         {
            returnNormal(new character());
         }
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public handle createBuffer(final character _pcBuffer_6, final character _pcPhysicalName_1)
   {
      character pcBuffer_6 = TypeFactory.initInput(_pcBuffer_6);
      character pcPhysicalName_1 = TypeFactory.initInput(_pcPhysicalName_1);
      /*------------------------------------------------------------------------------
        Purpose: Create a buffer  
          Notes: Called by create objects to create buffers for the query 
                 Overridden by the data logic procedure for temp-tables that 
                 are used in the query      
      ------------------------------------------------------------------------------*/
      handle hBuffer = TypeFactory.handle();
      
      return function("createBuffer", handle.class, new Block((Body) () -> 
      {
         ErrorManager.silentErrorEnable();
         RecordBuffer.create(hBuffer, pcPhysicalName_1, pcBuffer_6);
         ErrorManager.silentErrorDisable();
         returnNormal(hBuffer);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character dbColumnDataName(final character _pcDbColumn_1)
   {
      character pcDbColumn_1 = TypeFactory.initInput(_pcDbColumn_1);
      /*------------------------------------------------------------------------------
        Purpose:   Returns the RowObject fieldname of a database fieldname  
        Parameter: pcDbColumn - Qualified database fieldname (In the form of either
                                DB.TBL.FLDNM or TBL.FLDNM or FLDNM.)
        Notes:     Returns '' if not mapped to a RowObject
                   An unqualified oolumn are resolved as the first match in tables
      ------------------------------------------------------------------------------*/
      character cName = TypeFactory.character();
      character cAssignList = TypeFactory.character();
      character cDataColumnsByTable = TypeFactory.character();
      character cTableColumns = TypeFactory.character();
      character cAssign = TypeFactory.character();
      character cTables = TypeFactory.character();
      character cTable = TypeFactory.character();
      integer iTable_1 = TypeFactory.integer();
      integer iIterate = TypeFactory.integer();
      character cColumn = TypeFactory.character();
      logical lNoQual = TypeFactory.logical();
      
      return function("dbColumnDataName", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("Tables"));
         cTables.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DataColumnsByTable"));
         cDataColumnsByTable.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("AssignList"));
         cAssignList.assign(ghProp.unwrapBufferField().value());
         
         if (_isGreaterThan(numEntries(pcDbColumn_1, "."), 1))
         {
            cTable.assign(substring(pcDbColumn_1, 1, minus(lastIndexOf(pcDbColumn_1, "."), 1)));
            cTable /* {fnarg <func-name> <arg> [handle] */ .assign(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "resolveBuffer", targetProcedure(), "I", cTable));
            /* ensure correct qualification*/
            iTable_1.assign(lookup(cTable, cTables));
            /* Get the table num  */
            cColumn.assign(entry(numEntries(pcDbColumn_1, "."), pcDbColumn_1, "."));
            iIterate.assign(iTable_1);
         }
         else
         {
            lNoQual.assign(true);
            iTable_1.assign(1);
            cColumn.assign(pcDbColumn_1);
            iIterate.assign(numEntries(cTables));
         }
         
         /* iTable is set to 1 if not qualified or to the actual tablenumber if 
               qualified above. NOTE: LEAVE of loop at bottom if lNoQual = false  */
         if (_isGreaterThan(iTable_1, 0))
         {
            ToClause toClause16 = new ToClause(iTable_1, iTable_1, iIterate);
            
            doTo("loopLabel49", toClause16, new Block((Body) () -> 
            {
               cAssign.assign(entry(iTable_1, cAssignList, ";"));
               cTableColumns.assign(entry(iTable_1, cDataColumnsByTable, ";"));
               
               /* This function would work without this statement as this would 
                       have been resolved below.. This gives a slight perfromance gain
                       for the unmapped cases which are asssumed to be more common */
               if (_and(matchesList(cTableColumns, cColumn), () -> or(isEqual(cAssign, ""), () -> isEqual(matchesList(cAssign, cColumn), false))))
               {
                  returnNormal(cColumn);
               }
               
               /* Check to see if this column is on left side in the Assign List pairs, 
                       meaning that it has been given a new name in the RowObject table. */
               cName.assign(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "mappedEntry", targetProcedure(), "IIII", cColumn, cAssign, false, "," /* delimiter */));
               
               if (_notUnknown(cName))
               {
                  returnNormal(cName);
               }
               
               /* If the column is in this table's part of DataColumnsByTable,
                       and it isn't a RowObject column that is mapped to another dbfield
                       the RowObject column name is the same as the database field name. */
               if (_and(matchesList(cTableColumns, cColumn), () -> isUnknown(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "mappedEntry", targetProcedure(), "IIII", cColumn, cAssign, true, ","))))
               {
                  returnNormal(cColumn);
               }
               
               /* if qualified then we only look in one entry */
               if (_not(lNoQual))
               {
                  returnNormal("");
               }
            }));
         }
         /* END DO iTable */
         returnNormal("");
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public handle dbColumnHandle(final character _pcColumn_8)
   {
      character pcColumn_8 = TypeFactory.initInput(_pcColumn_8);
      /*------------------------------------------------------------------------------
        Purpose:     Returns the handle of a database column.
        Parameters: 
          pcColumn - Database fieldname.  Can be in the form of: DB.TBL.FLDNM,
                     TBL.FLDNM or FLDNM. Find first field if not qualified. 
        Notes:       Is capable of processing a pcColumn specified with brackets.
                     Used by columnDataType, ColumnValMsg, and ColumnTable.
      ------------------------------------------------------------------------------*/
      integer i = TypeFactory.integer();
      character cDbName = TypeFactory.character();
      character cBuffer = TypeFactory.character();
      character cFieldName = TypeFactory.character();
      handle hBuffer = TypeFactory.handle();
      handle hField = TypeFactory.handle();
      handle hQuery = TypeFactory.handle();
      integer iLbracket = TypeFactory.integer();
      
      return function("dbColumnHandle", handle.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryHandle"));
         hQuery.assign(ghProp.unwrapBufferField().value());
         
         if (_not(hQuery.isValid()))
         {
            returnNormal(new handle());
         }
         
         /* Sanity check */
         if (_isEqual(pcColumn_8, ""))
         {
            returnNormal(new handle());
         }
         
         cDbName.assign(_isEqual(numEntries(pcColumn_8, "."), 3) ? entry(1, pcColumn_8, ".") : new character());
         cBuffer.assign(_isGreaterThan(numEntries(pcColumn_8, "."), 1) ? entry(minus(numEntries(pcColumn_8, "."), 1), pcColumn_8, ".") : new character());
         cFieldName.assign(entry(numEntries(pcColumn_8, "."), pcColumn_8, "."));
         iLbracket.assign(lastIndexOf(cFieldName, "["));
         /* Remove extent bracket from fieldname */
         cFieldName.assign(_isGreaterThan(iLbracket, 0) ? substring(cFieldName, 1, minus(iLbracket, 1)) : cFieldName);
         
         loopLabel50:
         for (i.assign(1); _isLessThanOrEqual(i, hQuery.unwrapQuery().numBuffers()); i.increment())
         {
            hBuffer.assign(hQuery.unwrapQuery().bufferHandle(i));
            
            if (_and(notUnknown(cBuffer), () -> isNotEqual(hBuffer.unwrap().name(), cBuffer)))
            {
               continue loopLabel50;
            }
            
            if (_and(notUnknown(cDbName), () -> isNotEqual(hBuffer.unwrapDatabaseInfo().getDbName(), cDbName)))
            {
               continue loopLabel50;
            }
            
            ErrorManager.silentErrorEnable();
            hField.assign(hBuffer.unwrapBuffer().bufferField(cFieldName));
            ErrorManager.silentErrorDisable();
            
            if ((hField.isValid()).booleanValue())
            {
               returnNormal(hField);
            }
         }
         
         /* do i = 1 to hQuery:num-buffers */
         returnNormal(new handle());
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical defineDataObject(final character _pcTableList_1, final character _pcBaseQuery_1, final character _pcColumnList_1, final character _pcUpdatableColumns_1)
   {
      character pcTableList_1 = TypeFactory.initInput(_pcTableList_1);
      character pcBaseQuery_1 = TypeFactory.initInput(_pcBaseQuery_1);
      character pcColumnList_1 = TypeFactory.initInput(_pcColumnList_1);
      character pcUpdatableColumns_1 = TypeFactory.initInput(_pcUpdatableColumns_1);
      /*------------------------------------------------------------------------------
        Purpose: This is a convenience function which will let the user define the dynamic 
                 data object properties. The properties defined by this function are:
                 Tables,BaseQuery,DataColumns,DataColumsByTable,UpdatableColumnsByTable,AssignList
                   
                 Parameters:
                 1. pcTableList is a comma separated list of tables. If the table 
                    names are database qualified then the column list also needs to 
                    be database qualified.
                 
                 2. pcBaseQuery is the query for the dynamic SDO
                    
                 3. ColumnList is a comma separated list of qualified columns in the 
                    format: TableName.ColumnName[.RemanedColmnName] 
                    Where
                      TableName: would be the database table name
                      ColumnName: would be the field name
                      RenamedColumnName: Would be the renamed field name. This is optional.
                 
                 4. UpdatableColumns is comma separated list of logical values with 
                    following special cases i.e "YES" or "NO" or "YES,NO,YES,NO"
                    
                    If the value is "YES" then all the columns are updatable.
                    If the value is "NO" then all the columns are not updatable.
                    If the value id "YES,NO,YES,YES" then the individual columns will 
                    have the specified value
          Notes:  
      ------------------------------------------------------------------------------*/
      character cTableList = TypeFactory.character();
      character cPhysicalTableList = TypeFactory.character();
      integer iNumEntry = TypeFactory.integer();
      character cColumnName = TypeFactory.character();
      character cFieldName = TypeFactory.character();
      character cTableName = TypeFactory.character();
      character cPhysicalTableName = TypeFactory.character();
      integer iBufferEntries = TypeFactory.integer();
      character cAssignName = TypeFactory.character();
      logical lUpdatable = TypeFactory.logical();
      character cUpdatable = TypeFactory.character();
      integer iDbqual = TypeFactory.integer();
      integer iArrayPos = TypeFactory.integer();
      character cDataColumns = TypeFactory.character();
      character cUpdatableDataColumns = TypeFactory.character();
      character cDataColumnsByTable = TypeFactory.character();
      character cUpdatableDataColumnsByTable = TypeFactory.character();
      character cAssignColumnsByTable = TypeFactory.character();
      logical lDatabaseQualifiedTableNames = TypeFactory.logical();
      character[] caDataColumns = TypeFactory.characterExtent(18);
      character[] caUpdatableDataColumns = TypeFactory.characterExtent(18);
      character[] caAssignColumns = TypeFactory.characterExtent(18);
      
      return function("defineDataObject", logical.class, new Block((Body) () -> 
      {
         /* Check if the table name is database qualified */
         cTableName.assign(entry(1, pcTableList_1, ","));
         
         if (_isGreaterThan(numEntries(cTableName, "."), 1))
         {
            lDatabaseQualifiedTableNames.assign(true);
         }
         else
         {
            lDatabaseQualifiedTableNames.assign(false);
         }
         
         if ((lDatabaseQualifiedTableNames).booleanValue())
         {
            iDbqual.assign(1);
         }
         else
         {
            iDbqual.assign(0);
         }
         
         loopLabel51:
         for (iNumEntry.assign(1); _isLessThanOrEqual(iNumEntry, numEntries(pcTableList_1, ",")); iNumEntry.increment())
         {
            cTableName.assign(entry(iNumEntry, pcTableList_1));
            
            if (_isGreaterThan(numEntries(cTableName, " "), 1))
            {
               loopLabel52:
               while (_isGreaterThan(indexOf("  ", cTableName), 0))
               {
                  cTableName.assign(replaceAll(cTableName, "  ", " "));
               }
               
               iBufferEntries.assign(numEntries(cTableName, " "));
               
               if (_not(or(isEqual(iBufferEntries, 2), () -> and(isEqual(iBufferEntries, 3), () -> isEqual(entry(2, cTableName, " "), "FOR")))))
               {
                  messageBox(new Object[]
                  {
                     concat("The table reference '", entry(1, pcTableList_1), new character("' passed to defineDataObject is not valid.")),
                     new SkipEntity(true),
                     "To define a buffer use syntax 'BufferName [FOR] TableName'"
                  }, ALERT_INFO, BTN_OK, (String) null);
                  returnNormal(false);
               }
               
               cPhysicalTableName.assign(entry(iBufferEntries, cTableName, " "));
               cTableName.assign(entry(1, cTableName, " "));
            }
            else
            {
               cPhysicalTableName.assign(cTableName);
            }
            cTableList.assign(concat(cTableList, (_isEqual(iNumEntry, 1) ? new character("") : new character(",")), cTableName));
            cPhysicalTableList.assign(concat(cPhysicalTableList, (_isEqual(iNumEntry, 1) ? new character("") : new character(",")), cPhysicalTableName));
         }
         
         loopLabel53:
         for (iNumEntry.assign(1); _isLessThanOrEqual(iNumEntry, numEntries(pcColumnList_1, ",")); iNumEntry.increment())
         {
            cColumnName.setUnknown();
            cTableName.setUnknown();
            cFieldName.setUnknown();
            cFieldName.setUnknown();
            lUpdatable.assign(false);
            cUpdatable.assign(pcUpdatableColumns_1);
            ErrorManager.setError(new logical(false));
            ErrorManager.silentErrorEnable();
            /* Set the table, field, assignName values */
            cColumnName.assign(entry(iNumEntry, pcColumnList_1, ","));
            ErrorManager.silentErrorDisable();
            
            if (_or(or(ErrorManager.isError(), () -> isEqual(cColumnName, "")), () -> isUnknown(cColumnName)))
            {
               returnNormal(false);
            }
            
            ErrorManager.silentErrorEnable();
            cTableName.assign(concat((_isEqual(iDbqual, 1) ? concat(entry(1, cColumnName, "."), ".") : new character("")), entry(plus(iDbqual, 1), cColumnName, ".")));
            cFieldName.assign(entry(plus(iDbqual, 2), cColumnName, "."));
            ErrorManager.silentErrorDisable();
            
            if (((ErrorManager.isError())).booleanValue())
            {
               returnNormal(false);
            }
            
            ErrorManager.silentErrorEnable();
            cAssignName.assign(entry(plus(iDbqual, 3), cColumnName, "."));
            ErrorManager.silentErrorDisable();
            ErrorManager.setError(new logical(false));
            
            if (_isGreaterThan(cAssignName, ""))
            {
               cColumnName.assign(cAssignName);
            }
            else
            {
               cColumnName.assign(cFieldName);
            }
            
            /* Make sure that the table name or the field name is not blank or null */
            if (_or(isEqual(cTableName, ""), () -> isUnknown(cTableName)))
            {
               returnNormal(false);
            }
            
            if (_or(isEqual(cColumnName, ""), () -> isUnknown(cColumnName)))
            {
               returnNormal(false);
            }
            
            /* Find out if the column is updatable */
            if (_isGreaterThan(numEntries(cUpdatable, ","), 1))
            {
               ErrorManager.silentErrorEnable();
               cUpdatable.assign(entry(iNumEntry, pcUpdatableColumns_1, ","));
               ErrorManager.silentErrorDisable();
            }
            
            if (_isEqual(trim(cUpdatable), "YES"))
            {
               lUpdatable.assign(true);
            }
            
            if (((ErrorManager.isError())).booleanValue())
            {
               returnNormal(false);
            }
            
            iArrayPos.assign(lookup(cTableName, cTableList, ","));
            
            if (_isLessThanOrEqual(iArrayPos, 0))
            {
               returnNormal(false);
            }
            
            assignSingle(caDataColumns, iArrayPos, concat(subscript(caDataColumns, iArrayPos), ",", cColumnName));
            cDataColumns.assign(concat(cDataColumns, ",", cColumnName));
            
            if ((lUpdatable).booleanValue())
            {
               assignSingle(caUpdatableDataColumns, iArrayPos, concat(subscript(caUpdatableDataColumns, iArrayPos), ",", cColumnName));
               cUpdatableDataColumns.assign(concat(cUpdatableDataColumns, ",", cColumnName));
            }
            
            if (_isGreaterThan(cAssignName, ""))
            {
               assignSingle(caAssignColumns, iArrayPos, concat(subscript(caAssignColumns, iArrayPos), ",", cAssignName, ",", cFieldName));
            }
         }
         
         loopLabel54:
         for (iNumEntry.assign(1); _isLessThanOrEqual(iNumEntry, numEntries(cTableList, ",")); iNumEntry.increment())
         {
            if (_isEqual(iNumEntry, 1))
            {
               cDataColumnsByTable.assign(trim(subscript(caDataColumns, iNumEntry), ","));
               cUpdatableDataColumnsByTable.assign(trim(subscript(caUpdatableDataColumns, iNumEntry), ","));
               cAssignColumnsByTable.assign(trim(subscript(caAssignColumns, iNumEntry), ","));
            }
            else
            {
               cDataColumnsByTable.assign(concat(cDataColumnsByTable, ";", trim(subscript(caDataColumns, iNumEntry), ",")));
               cUpdatableDataColumnsByTable.assign(concat(cUpdatableDataColumnsByTable, ";", trim(subscript(caUpdatableDataColumns, iNumEntry), ",")));
               cAssignColumnsByTable.assign(concat(cAssignColumnsByTable, ";", trim(subscript(caAssignColumns, iNumEntry), ",")));
            }
         }
         
         cDataColumns.assign(trim(cDataColumns, ","));
         cUpdatableDataColumns.assign(trim(cUpdatableDataColumns, ","));
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("Tables"));
         ghProp.unwrapBufferField().changeValue(cTableList);
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("PhysicalTables"));
         ghProp.unwrapBufferField().changeValue(cPhysicalTableList);
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DataColumnsByTable"));
         ghProp.unwrapBufferField().changeValue(cDataColumnsByTable);
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("UpdatableColumnsByTable"));
         ghProp.unwrapBufferField().changeValue(cUpdatableDataColumnsByTable);
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("AssignList"));
         ghProp.unwrapBufferField().changeValue(cAssignColumnsByTable);
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("BaseQuery"));
         ghProp.unwrapBufferField().changeValue(pcBaseQuery_1);
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DataColumns"));
         ghProp.unwrapBufferField().changeValue(cDataColumns);
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("Tables"));
         cTableList.assign(ghProp.unwrapBufferField().value());
         returnNormal(isGreaterThan(cTableList, ""));
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character excludeColumns(final integer _iTable_1)
   {
      integer iTable_1 = TypeFactory.initInput(_iTable_1);
      /*-----------------------------------------------------------------------------
        Purpose: Returns a comma separated list of columns that should be used
                 as the exclude parameter for BUFFER-COPY and BUFFER-COMPARE from
                 the database.
      Parameter: The number of the table in the query. 
          Notes: This protects the Target from being unintentionally overwritten 
                 by unmapped not used columns with the same name, but are supposed 
                 to be referencing a previously copyed or compared table or not at 
                 all. We simply remove all the columns that represents the table 
                 that we are copying comparing to from the {&adm-tabledelimiter} separated 
                 DataColumnsByTable. 
                 DataColumnsByTable stores calculated fields in the last entry, 
                 so we always exclude those.  
                 Used by refetchDbRow, transferDbRow and compareDbRow 
      -----------------------------------------------------------------------------*/
      character cExclude = TypeFactory.character();
      
      return function("excludeColumns", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DataColumnsByTable"));
         cExclude.assign(ghProp.unwrapBufferField().value());
         /* Remove the columns from the iTable table from the excludelist */
         cExclude.replaceEntry(iTable_1, ";", "");
         returnNormal(trim(replaceAll(cExclude, ";", ","), ","));
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   private character firstBufferName(final character _pcExpression_4)
   {
      character pcExpression_4 = TypeFactory.initInput(_pcExpression_4);
      /*------------------------------------------------------------------------------
        Purpose:     Returns the first buffer reference in a where-clause expression.
        Parameters:  pcExpression - a string expression
        Notes:       In order to be recognized as a buffer name, it must contain at
                     least one ".", otherwise the return value is ?.
      ------------------------------------------------------------------------------*/
      integer i = TypeFactory.integer();
      character cWord = TypeFactory.character();
      integer iQuoteStart = TypeFactory.integer();
      integer iQuoteEnd = TypeFactory.integer();
      character cExpression = TypeFactory.character();
      integer iNumeric = TypeFactory.integer();
      
      return function("firstBufferName", character.class, new Block((Body) () -> 
      {
         if (_isEqual(numEntries(pcExpression_4, "."), 1))
         {
            returnNormal(new character());
         }
         
         /* Remove quoted words from this to avoid problems with periods inside quotes*/
         cExpression.assign(replaceAll(pcExpression_4, "'", "\""));
         
         loopLabel55:
         while (_isGreaterThan(indexOf(cExpression, "\""), 0))
         {
            iQuoteStart.assign(indexOf(cExpression, "\""));
            iQuoteEnd.assign(indexOf(substring(cExpression, plus(iQuoteStart, 1)), "\""));
            
            if (_and(isGreaterThan(iQuoteStart, 0), () -> isGreaterThan(iQuoteEnd, 0)))
            {
               cExpression.replace("<>", iQuoteStart, plus(iQuoteEnd, 1));
            }
            else
            {
               break loopLabel55;
            }
         }
         
         cExpression.assign(replaceAll(cExpression, "(", " "));
         cExpression.assign(replaceAll(cExpression, ")", " "));
         cExpression.assign(replaceAll(cExpression, "=", " "));
         cExpression.assign(replaceAll(cExpression, ">=", " "));
         cExpression.assign(replaceAll(cExpression, "<=", " "));
         cExpression.assign(replaceAll(cExpression, ">", " "));
         cExpression.assign(replaceAll(cExpression, "<", " "));
         
         loopLabel56:
         for (i.assign(1); _isLessThanOrEqual(i, numEntries(cExpression, " ")); i.increment())
         {
            cWord.assign(entry(i, cExpression, " "));
            
            if (_isGreaterThanOrEqual(numEntries(cWord, "."), 2))
            {
               ErrorManager.silentErrorEnable();
               /* This '.' might be a decimal delimiter or decimal sign, 
                       if the first entry is integer it cannot be a buffer name ... */
               iNumeric.assign(new integer(entry(1, cWord)));
               ErrorManager.silentErrorDisable();
               
               if (_not(ErrorManager.isError()))
               {
                  continue loopLabel56;
               }
            }
            
            if (_isEqual(numEntries(cWord, "."), 2))
            {
               returnNormal(entry(1, cWord, "."));
            }
            
            if (_isEqual(numEntries(cWord, "."), 3))
            {
               returnNormal(concat(entry(1, cWord, "."), ".", entry(2, cWord, ".")));
            }
         }
         
         /* i = 1 to num-entries cExpression  */
         returnNormal(new character());
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character firstRowIds(final character _pcQueryString_4)
   {
      character pcQueryString_4 = TypeFactory.initInput(_pcQueryString_4);
      /*------------------------------------------------------------------------------
         Purpose:   Returns the ROWID (converted to a character string) of the first 
                    database query row satisfying the passed query prepare string.   
      Parameters:
          pcWhere - A complete query where clause that matches the database query's 
                    buffers.
           Notes:   Used by rowidwhere, findRow and findRowWhere    
      ------------------------------------------------------------------------------*/
      handle hQuery = TypeFactory.handle();
      handle hBuffer = TypeFactory.handle();
      character cBuffer = TypeFactory.character();
      character cBufferList = TypeFactory.character();
      handle hRowQuery = TypeFactory.handle();
      logical lOk = TypeFactory.logical();
      integer i = TypeFactory.integer();
      character cRowIds = TypeFactory.character();
      
      return function("firstRowIds", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryHandle"));
         hQuery.assign(ghProp.unwrapBufferField().value());
         QueryWrapper.createQuery(hRowQuery);
         
         loopLabel57:
         for (i.assign(1); _isLessThanOrEqual(i, hQuery.unwrapQuery().numBuffers()); i.increment())
         {
            hBuffer.assign(hQuery.unwrapQuery().bufferHandle(i));
            cBuffer.assign(hBuffer.unwrap().name());
            RecordBuffer.create(hBuffer, hBuffer, cBuffer);
            hRowQuery.unwrapQuery().addBuffer(hBuffer);
            cBufferList.assign(concat(cBufferList, (_isEqual(i, 1) ? new character("") : new character(",")), valueOf(hBuffer)));
         }
         
         /* do i = 1 to */
         pcQueryString_4 /* {fnarg <func-name> <arg> [handle] */ .assign(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "fixQueryString", targetProcedure(), "I", pcQueryString_4));
         ErrorManager.silentErrorEnable();
         lOk.assign(hRowQuery.unwrapQuery().prepare(pcQueryString_4));
         ErrorManager.silentErrorDisable();
         
         if ((lOk).booleanValue())
         {
            lOk.assign(hRowQuery.unwrapQuery().queryOpen());
         }
         
         if ((lOk).booleanValue())
         {
            lOk.assign(hRowQuery.unwrapQuery().getFirst());
         }
         
         loopLabel58:
         for (i.assign(1); _isLessThanOrEqual(i, numEntries(cBufferList)); i.increment())
         {
            hBuffer.assign(handle.fromString(entry(i, cBufferList)));
            cRowIds.assign(concat(cRowIds, (_isEqual(i, 1) ? new character("") : new character(",")), (_and(lOk, () -> hBuffer.unwrapBuffer().available()) ? valueOf(hBuffer.unwrapBuffer().rowID()) : new character(""))));
            HandleOps.delete(hBuffer);
         }
         
         /* do i = 1 to hRowQuery:num-buffers */
         HandleOps.delete(hRowQuery);
         returnNormal((lOk).booleanValue() ? cRowIds : new character());
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public handle getTargetProcedure()
   {
      return function("getTargetProcedure", handle.class, new Block((Body) () -> 
      {
         /*------------------------------------------------------------------------------
           Purpose:  Temporary fn to return the source-procedure's target-procedure
                     to a function such as colValues in an SBO who needs to know who
                     the *real* caller object is.
             Notes:  
         ------------------------------------------------------------------------------*/
         returnNormal(ghTargetProcedure);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character indexInformation(final character _pcQuery_3, final logical _plUseTableSep_1, final character _pcIndexInfo_1)
   {
      character pcQuery_3 = TypeFactory.initInput(_pcQuery_3);
      logical plUseTableSep_1 = TypeFactory.initInput(_plUseTableSep_1);
      character pcIndexInfo_1 = TypeFactory.initInput(_pcIndexInfo_1);
      /*------------------------------------------------------------------------------
         Purpose: Return index Information for all buffers in the query.
                  Each index is separated with chr(1).
                  Field information is either qualifed with db and table 
                  or the chr(2) is used as table separator.
                  
      Parameters: pcQuery - What information? 
                      - 'All'             All indexed fields 
                      - 'Standard' or ''  All indexed fields excluding word indexes    
                      - 'Word'            Word Indexed 
                      - 'Unique'          Unique indexes 
                      - 'NonUnique'       Non Unique indexes 
                      - 'Primary'         Primary index
                      - 'Info'            All info (meaningless if pcIndexInfo <> ?)               
                 
                 plUseTableSep - Use table separator.  
                 
                      - Yes   Use table separator 
                      - No    Don't use table separator 
                             (if pcIndexinfo = ? fieldnames will be qualifed
                              otherwise they will remain as in pcInddexInfo)
                
                pcIndexInfo - Query or previously retrieved info. Enables the 
                               function to be used with no database connection.    
                      - ? use query - if plUseTableSep = yes the field will be 
                                      returned qualified.     
                      -  Index info in EXACT SAME FORMAT as returned from this 
                         function earlier with indexInformation('info',yes,?).
                         
                         See Notes below for delimiters. 
           Notes: Returned delimiters
                  - qualifed      - semicolon is index separator 
                  - non-qualifed  - chr(1) as index separator 
                                    chr(2) as table separator        
      ------------------------------------------------------------------------------*/
      handle hQuery = TypeFactory.handle();
      handle hBuff = TypeFactory.handle();
      integer iBuff = TypeFactory.integer();
      integer iIdx = TypeFactory.integer();
      integer iField = TypeFactory.integer();
      character cInfo = TypeFactory.character();
      character cIndexInfo = TypeFactory.character();
      character cIndexString = TypeFactory.character();
      logical lFound = TypeFactory.logical();
      logical lFirstIdx = TypeFactory.logical();
      character cTblDlm = TypeFactory.character();
      character cIdxDlm = TypeFactory.character();
      character cField = TypeFactory.character();
      character cFieldList = TypeFactory.character();
      integer iNumBuffers = TypeFactory.integer();
      logical lUseDbqual = TypeFactory.logical();
      
      return function("indexInformation", character.class, new Block((Body) () -> 
      {
         /* We only use query if no previouisly processed info is passed */
         if (_isUnknown(pcIndexInfo_1))
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryHandle"));
            hQuery.assign(ghProp.unwrapBufferField().value());
            
            if (_not(hQuery.isValid()))
            {
               returnNormal(new character());
            }
         }
         cTblDlm.assign(chr(2));
         cIdxDlm.assign(chr(1));
         iNumBuffers.assign((hQuery.isValid()).booleanValue() ? hQuery.unwrapQuery().numBuffers() : numEntries(pcIndexInfo_1, cTblDlm));
         
         /* If no table separator find qualifier rule */
         if (_not(plUseTableSep_1))
         {
            /* {get Prop var [handle]} */ 
            lUseDbqual.assign(ControlFlowOps.invokeDynamicFunctionIn(logical.class, "getUseDBQualifier", targetProcedure()));
         }
         
         loopLabel59:
         for (iBuff.assign(1); _isLessThanOrEqual(iBuff, iNumBuffers); iBuff.increment())
         {
            if (_notUnknown(pcIndexInfo_1))
            {
               cIndexString.assign(entry(iBuff, pcIndexInfo_1, cTblDlm));
            }
            else
            {
               hBuff.assign(hQuery.unwrapQuery().bufferHandle(iBuff));
            }
            
            iIdx.assign(0);
            lFirstIdx.assign(true);
            
            indexBlock:
            while (true)
            {
               iIdx.assign(plus(iIdx, 1));
               cIndexInfo.assign(_isUnknown(pcIndexInfo_1) ? hBuff.unwrapIndexInformation().indexInformation(iIdx) : _isGreaterThanOrEqual(numEntries(cIndexString, cIdxDlm), iIdx) ? entry(iIdx, cIndexString, cIdxDlm) : new character());
               
               if (_isUnknown(cIndexInfo))
               {
                  break indexBlock;
               }
               
               /* No trick .. this is DEFAULT 
                                            Explicit for the Progress illiterate */
               switch (StringHelper.safeTrimTrailing((pcQuery_3).toStringMessage()).toUpperCase())
               {
                  case "STANDARD":
                  case "":
                     {
                        lFound.assign(isEqual(entry(4, cIndexInfo), "0"));
                     }
                     break;
                  case "INFO":
                  case "ALL":
                     {
                        lFound.assign(true);
                     }
                     break;
                  case "WORD":
                     {
                        lFound.assign(isEqual(entry(4, cIndexInfo), "1"));
                     }
                     break;
                  case "UNIQUE":
                     {
                        lFound.assign(isEqual(entry(3, cIndexInfo), "1"));
                     }
                     break;
                  case "NONUNIQUE":
                     {
                        lFound.assign(and(isEqual(entry(3, cIndexInfo), "0"), () -> isEqual(entry(4, cIndexInfo), "0")));
                     }
                     break;
                  case "PRIMARY":
                     {
                        lFound.assign(isEqual(entry(2, cIndexInfo), "1"));
                     }
                     break;
                  default:
                     {
                        /* Design time error */
                        messageBox(new Object[]
                        {
                           "ADM Error:",
                           "Function indexInformation() does not understand",
                           concat(new character("parameter '"), pcQuery_3),
                           "'"
                        }, ALERT_ERROR, BTN_OK, (String) null);
                        returnNormal(new character());
                     }
               }
               
               /* pcQuery */
               /* If pcQuery includes this index then .... */
               if ((lFound).booleanValue())
               {
                  cFieldList.assign("");
                  
                  /* if 'info' and use table separator we have all we need.
                             Otherwise we loop through each field and refine the data */
                  if (_or(isNotEqual(pcQuery_3, "info"), not(plUseTableSep_1)))
                  {
                     loopLabel60:
                     for (iField.assign(5); _isLessThanOrEqual(iField, numEntries(cIndexInfo)); iField.assign(plus(iField, 2)))
                     {
                        /* If no table separator and the buffer is valid 
                                     we qualify the field */
                        cField.assign(concat((_and(not(plUseTableSep_1), () -> hBuff.isValid()) ? concat(((lUseDbqual).booleanValue() ? concat(hBuff.unwrapDatabaseInfo().getDbName(), ".") : new character("")), hBuff.unwrap().name(), ".") : new character("")), entry(iField, cIndexInfo)));
                        
                        /* if 'info' just replace the field with the qualifed one */
                        if (_isEqual(pcQuery_3, "Info"))
                        {
                           cIndexInfo.replaceEntry(iField, cField);
                        }
                        else
                        {
                           cFieldList.assign(concat(cFieldList, (_isEqual(cFieldList, "") ? new character("") : new character(",")), cField));
                        }
                     }
                  }
                  /* if 'info' just apppend all index info  */
                  /* don't add index delimiter for first index after 
                                        the table separator or when empty */
                  cInfo.assign(concat(cInfo, (_or(and(plUseTableSep_1, lFirstIdx), () -> isEqual(cInfo, "")) ? new character("") : cIdxDlm), (_isEqual(pcQuery_3, "Info") ? cIndexInfo : cFieldList)));
                  lFirstIdx.assign(false);
               }
            }
            
            /* do while true */
            /* If no field qualifier add table delimiter unless this is the last buffer */
            if (_and(plUseTableSep_1, () -> isLessThan(iBuff, iNumBuffers)))
            {
               cInfo.assign(concat(cInfo, cTblDlm));
            }
         }
         
         /* do ibuff = 1 to num-buffers */
         returnNormal(trim(cInfo, cTblDlm));
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character insertExpression(final character _pcWhere_6, final character _pcExpression_4, final character _pcAndOr_5)
   {
      character pcWhere_6 = TypeFactory.initInput(_pcWhere_6);
      character pcExpression_4 = TypeFactory.initInput(_pcExpression_4);
      character pcAndOr_5 = TypeFactory.initInput(_pcAndOr_5);
      /*------------------------------------------------------------------------------
       Purpose:     Inserts an expression into ONE buffer's where-clause.
       Parameters:  
            pcWhere      - Complete where clause with or without the FOR keyword,
                           but without any comma before or after.
            pcExpression - New expression OR OF phrase (Existing OF phrase is replaced)
            pcAndOr      - Specifies what operator is used to add the new expression 
                           to existing ones.
                           - AND (default) 
                           - OR         
       Notes:     - The new expression is embedded in parenthesis, but no parentheses
                    are placed around the existing one.  
                  - Lock keywords must be unabbreviated or without -lock (i.e. SHARE
                    or EXCLUSIVE.)   
                  - Any keyword in comments may cause problems.              
      ------------------------------------------------------------------------------*/
      character cTable = TypeFactory.character();
      character cRelTable = TypeFactory.character();
      character cJoinTable = TypeFactory.character();
      character cWhereOrAnd = TypeFactory.character();
      integer iTblPos = TypeFactory.integer();
      integer iWherePos = TypeFactory.integer();
      logical lWhere = TypeFactory.logical();
      integer iOfPos = TypeFactory.integer();
      integer iRelTblPos = TypeFactory.integer();
      integer iInsertPos = TypeFactory.integer();
      integer iUseIdxPos = TypeFactory.integer();
      integer iOuterPos = TypeFactory.integer();
      integer iLockPos = TypeFactory.integer();
      integer iByPos = TypeFactory.integer();
      integer iIdxRePos = TypeFactory.integer();
      
      return function("insertExpression", character.class, new Block((Body) () -> 
      {
         /* Get rid of potential line break characters (query builder -> repository)*/
         pcWhere_6.assign(replaceAll(pcWhere_6, chr(10), " "));
         cTable /* {fnarg <func-name> <arg> [handle] */ .assign(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "whereClauseBuffer", targetProcedure(), "I", pcWhere_6));
         iTblPos.assign(plus(indexOf(pcWhere_6, cTable), length(cTable)));
         iWherePos.assign(plus(indexOf(pcWhere_6, " WHERE "), 6));
         iByPos.assign(indexOf(pcWhere_6, " BY "));
         iUseIdxPos.assign(indexOf(pcWhere_6, " USE-INDEX "));
         iIdxRePos.assign(indexOf(concat(pcWhere_6, " "), " INDEXED-REPOSITION "));
         iOuterPos.assign(indexOf(concat(pcWhere_6, " "), " OUTER-JOIN "));
         iLockPos.assign(int64.maximum(indexOf(concat(pcWhere_6, " "), " NO-LOCK "), indexOf(concat(pcWhere_6, " "), " SHARE-LOCK "), indexOf(concat(pcWhere_6, " "), " EXCLUSIVE-LOCK "), indexOf(concat(pcWhere_6, " "), " SHARE "), indexOf(concat(pcWhere_6, " "), " EXCLUSIVE ")));
         iInsertPos.assign(plus(length(pcWhere_6), 1));
         /* We must insert before the leftmoust keyword,
                                unless the keyword is Before the WHERE keyword */
         iInsertPos.assign(int64.minimum((_isGreaterThan(iLockPos, iWherePos) ? iLockPos : iInsertPos), (_isGreaterThan(iOuterPos, iWherePos) ? iOuterPos : iInsertPos), (_isGreaterThan(iUseIdxPos, iWherePos) ? iUseIdxPos : iInsertPos), (_isGreaterThan(iIdxRePos, iWherePos) ? iIdxRePos : iInsertPos), (_isGreaterThan(iByPos, iWherePos) ? iByPos : iInsertPos)));
         lWhere.assign(isGreaterThan(indexOf(pcWhere_6, " WHERE "), 0));
         cWhereOrAnd.assign((_not(lWhere) ? new character(" WHERE ") : _or(isEqual(pcAndOr_5, ""), () -> isUnknown(pcAndOr_5)) ? new character(" AND ") : concat(" ", pcAndOr_5, " ")));
         iOfPos.assign(indexOf(pcWhere_6, " OF "));
         
         if (_begins(leftTrim(pcExpression_4), "OF "))
         {
            /* If there is an OF in both the join and existing query we replace the 
                   table unless they are the same */
            if (_isGreaterThan(iOfPos, 0))
            {
               /* Find the table in the old join */
               cRelTable.assign(entry(1, leftTrim(substring(pcWhere_6, plus(iOfPos, 4))), " "));
               /* Find the table in the new join */
               cJoinTable.assign(substring(leftTrim(pcExpression_4), 3));
               
               if (_isNotEqual(cJoinTable, cRelTable))
               {
                  iRelTblPos.assign(plus(indexOf(concat(pcWhere_6, " "), concat(" ", cRelTable, " ")), 1));
                  pcWhere_6.replace(cJoinTable, iRelTblPos, length(cRelTable));
               }
            }
            else
            {
               pcWhere_6.replace(concat(" ", pcExpression_4), iTblPos, 0);
            }
         }
         else
         {
            pcWhere_6.replace(concat(cWhereOrAnd, "(", pcExpression_4, ")"), iInsertPos, 0);
         }
         returnNormal(rightTrim(pcWhere_6));
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character newQuerySort(final character _pcQuery_3, final character _pcSort_1, final logical _plDbcolumns_1)
   {
      character pcQuery_3 = TypeFactory.initInput(_pcQuery_3);
      character pcSort_1 = TypeFactory.initInput(_pcSort_1);
      logical plDbcolumns_1 = TypeFactory.initInput(_plDbcolumns_1);
      /*------------------------------------------------------------------------------
        Purpose   :  Insert sort criteria (BY phrase) in a QueryString.
        Parameters:
          pcQuery    - Query to add sort to (current sort will be replaced)        
          pcSort     - new sort expression.
                       - [BY ] column [DESCENDING | TOGGLE ] [ BY ... ]  
                       - TOGGLE sort option specifies that the column should be 
                         sorted opposite of the current sort. The option can be 
                         specified for any column in the sort.      
                       - Use RowObject as qualifier to specify data object's column 
                         names.
          plDbColumns - YES - the query is a db query 
                              columns qualifed with 'RowObject.' will be renamed to 
                              the mapped db field.
                        NO  - columns not qualifed with 'RowObject.' will be renamed 
                              to the mapped RowObject column.  
                                            
             Notes: Unqualified columns are always resolved as db fields (searching 
                    in query order) also when plDBColumns is false.                                                                    
      ------------------------------------------------------------------------------*/
      character cSortEntries = TypeFactory.character();
      character cSortEntry = TypeFactory.character();
      character cSortColumn = TypeFactory.character();
      character cSortOption = TypeFactory.character();
      character cOldSort = TypeFactory.character();
      character cOldSortColumn = TypeFactory.character();
      character cOldSortEntry = TypeFactory.character();
      character cNewSort = TypeFactory.character();
      integer iByPos = TypeFactory.integer();
      integer iIdxPos = TypeFactory.integer();
      integer iLength = TypeFactory.integer();
      integer iColumn = TypeFactory.integer();
      logical lToggled = TypeFactory.logical();
      logical lSameColumns = TypeFactory.logical();
      logical lOldDescending = TypeFactory.logical();
      
      return function("newQuerySort", character.class, new Block((Body) () -> 
      {
         if (_isEqual(pcQuery_3, ""))
         {
            returnNormal("");
         }
         
          /* remove first BY if passed */pcSort_1.assign(_begins(leftTrim(pcSort_1), "BY ") ? trim(substring(leftTrim(pcSort_1), 3)) : trim(pcSort_1));
         /* change 'BY' to comma for looping purpose */
         cSortEntries.assign(replaceAll(pcSort_1, " BY ", ","));
         cOldSort /* {fnarg <func-name> <arg> [handle] */ .assign(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "sortExpression", targetProcedure(), "I", pcQuery_3));
         /* change 'BY' to comma and remove first BY 
                               for looping purpose */
         cOldSort.assign(_isNotEqual(cOldSort, "") ? replaceAll(substring(cOldSort, 3), " BY ", ",") : new character(""));
         /* set to FALSE immediately if new sort is blank and
                              old sort is not.  
                              otherwise the check below will decide whether old sort 
                              is different from the new */
         lSameColumns.assign(not(and(isEqual(pcSort_1, ""), () -> isNotEqual(cOldSort, ""))));
         
         IntegerExpression resExpr14 = new IntegerExpression()
         {
            public integer execute()
            {
               return numEntries(cSortEntries);
            }
         };
         ToClause toClause17 = new ToClause(iColumn, 1, resExpr14);
         
         doTo("loopLabel61", toClause17, new Block((Body) () -> 
         {
            cSortEntry.assign(entry(iColumn, cSortEntries));
            cSortColumn.assign(trim(entry(1, cSortEntry, " ")));
            cSortOption.assign(trim(substring(cSortEntry, plus(length(cSortColumn), 1))));
            cSortColumn.assign(_begins(cSortColumn, "RowObject.") ? (plDbcolumns_1).booleanValue() ? ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "columnDbColumn", targetProcedure(), "I", cSortColumn) : entry(2, cSortColumn, ".") : _not(plDbcolumns_1) ? ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "dbColumnDataName", targetProcedure(), "I", cSortColumn) : cSortColumn);
            
            /* Calculated if plDbColumns or unmapped if not plDbcolumns */
            if (_isEqual(cSortColumn, ""))
            {
               returnNormal(new character());
            }
            
            /* Keep track of whether the old sort criteria is the same as the new.
                   This is a requirement for 'toggle' and is also used to avoid 
                   server hit if the same sort criteria. (the query stays unchanged also 
                   if qualifcations does not match and stops resortQuery from resorting)
                   We do not need to check this any more if an option already is toggled.*/
            if (_and(lSameColumns, not(lToggled)))
            {
               cOldSortEntry.assign((_isGreaterThanOrEqual(numEntries(cOldSort), iColumn) ? trim(entry(iColumn, cOldSort)) : new character("")));
               cOldSortColumn.assign(entry(1, cOldSortEntry, " "));
               lOldDescending.assign(isGreaterThan(indexOf(concat(cOldSortEntry, " "), " DESCENDING "), 0));
               
               if (_isEqual(cOldSortColumn, ""))
               {
                  lSameColumns.assign(false);
               }
               else
               {
                  if (_isEqual(cSortColumn, cOldSortColumn))
                  {
                     lSameColumns.assign(true);
                  }
                  else
                  {
                     if (_and(and(plDbcolumns_1, () -> isNotEqual(entry(numEntries(cSortColumn, "."), cSortColumn, "."), entry(numEntries(cOldSortColumn, "."), cOldSortColumn, "."))), () -> isEqual( /* {fnarg <func-name> <arg> [handle] */ ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "dbColumnDataName", targetProcedure(), "I", cOldSortColumn), ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "dbColumnDataName", targetProcedure(), "I", cSortColumn)) /* {fnarg <func-name> <arg> [handle] */))
                     {
                        lSameColumns.assign(true);
                     }
                     else
                     {
                        lSameColumns.assign(false);
                     }
                  }
               }
            }
            
            /* same (still) and not toggled (yet) */
            /* If sort option is toggle then swap descending/blank */
            if (_isEqual(cSortOption, "Toggle"))
            {
               /* if already toggled or not lSameColumns (See above) ignore toggle */
               if (_or(lToggled, not(lSameColumns)))
               {
                  cSortOption.assign("");
               }
               else
               {
                  if ((lOldDescending).booleanValue())
                  {
                     cSortOption.assign("");
                  }
                  else
                  {
                     cSortOption.assign("DESCENDING");
                  }
                  
                  /* We only support one toggling. */
                  lToggled.assign(true);
               }
            }
            
            /* sortoption = toggle */
            /* if not toggled we must also include the sort option in the check
                   of same (if same and not toggled we don't apply any sort at all) */
            if (_and(not(lToggled), () -> isNotEqual(lOldDescending, isEqual(cSortOption, "DESCENDING"))))
            {
               lSameColumns.assign(false);
            }
            
            cNewSort.assign(trim(concat(cNewSort, " BY ", cSortColumn, " ", cSortOption)));
         }));
         
         /* loop through BY clauses */
         /* Skip sort if Same as old unless a sort option was toggled */
         if (_or(not(lSameColumns), lToggled))
         {
             /* check for  indexed-reposition  */iIdxPos.assign(indexOf(concat(rightTrim(pcQuery_3, ". "), " "), " INDEXED-REPOSITION "));
            /* If no INDEX-REPOSITION is found, set the iLength (where to end insert)
                     to the end of where-clause. (right-trim periods and blanks to find 
                     the true end of the expression) Otherwise iLength is the position of 
                     INDEX-REPOSITION. */
            iLength.assign((_isEqual(iIdxPos, 0) ? plus(length(rightTrim(pcQuery_3, ". ")), 1) : new int64(iIdxPos)));
            /* Any By ? */
            iByPos.assign(indexOf(pcQuery_3, " BY "));
            /* Now find where we should start the insert; 
                     We might have both a BY and an INDEXED-REPOSITION or only one of them 
                     or none. So we make sure we use the MINIMUM of whichever of those 
                     unless they are 0. */
            iByPos.assign(int64.minimum(_isEqual(iByPos, 0) ? iLength : iByPos, _isEqual(iIdxPos, 0) ? iLength : iIdxPos));
            pcQuery_3.replace(_isNotEqual(cNewSort, "") ? concat(" ", cNewSort) : new character(""), iByPos, minus(iLength, iByPos));
         }
         
         returnNormal(pcQuery_3);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character newQueryString(final character _pcColumns_4, final character _pcValues_4, final character _pcOperators_5, final character _pcQueryString_4, final character _pcAndOr_5)
   {
      character pcColumns_4 = TypeFactory.initInput(_pcColumns_4);
      character pcValues_4 = TypeFactory.initInput(_pcValues_4);
      character pcOperators_5 = TypeFactory.initInput(_pcOperators_5);
      character pcQueryString_4 = TypeFactory.initInput(_pcQueryString_4);
      character pcAndOr_5 = TypeFactory.initInput(_pcAndOr_5);
      /*------------------------------------------------------------------------------   
         Purpose: Returns a new query string to the passed query. 
                  The tables in the passed query must match getTables().  
                  Adds column/value pairs to the corresponding buffer's where-clause. 
                  Each buffer's expression will always be embedded in parenthesis.
         Parameters: 
           pcColumns   - Column names (Comma separated)                    
                         Fieldname of a table in the query in the form of 
                         TBL.FLDNM or DB.TBL.FLDNM),
                         (RowObject.FLDNM should be used for SDO's)  
                         If the fieldname isn't qualified it checks the tables in 
                         the TABLES property and assumes the first with a match.
                         
           pcValues    - corresponding Values (CHR(1) separated)
           pcOperators - Operator - one for all columns
                                    - blank - defaults to (EQ)  
                                    - Use slash to define alternative string operator
                                      EQ/BEGINS etc..
                                  - comma separated for each column/value
           pcQueryString - A complete querystring matching the queries tables.
                           MUST be qualifed correctly.
                           ? - use the existing query  
           pcAndOr       - AND or OR decides how the new expression is appended to 
                           the passed query (for each buffer!).                                               
         Notes:  This is basically the same logic as assignQuerySelection, but 
                 without the replace functionality ... 
                 (It should not have been duplicated, but... )     
      ------------------------------------------------------------------------------*/
      character cBufferList = TypeFactory.character();
      character cBuffer = TypeFactory.character();
      /* We need the columns name and the parts */
      character cColumn = TypeFactory.character();
      character cOperator = TypeFactory.character();
      character cValue = TypeFactory.character();
      integer iColumn = TypeFactory.integer();
      integer iBuffer = TypeFactory.integer();
      character cUsedNums = TypeFactory.character();
      /* Used to builds the column/value string expression */
      character cBufWhere = TypeFactory.character();
      
      return function("newQueryString", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("Tables"));
         cBufferList.assign(ghProp.unwrapBufferField().value());
         
         /* If unkown value is passed used the existing query string */
         if (_isUnknown(pcQueryString_4))
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryString"));
            pcQueryString_4.assign(ghProp.unwrapBufferField().value());
            
            /* If no QueryString find the current query */
            if (_or(isEqual(pcQueryString_4, ""), () -> isUnknown(pcQueryString_4)))
            {
               /* {get Prop var [handle]} */ 
               pcQueryString_4.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getQueryWhere", targetProcedure()));
               
               /* If no current Query find the defined base query */
               if (_or(isEqual(pcQueryString_4, ""), () -> isUnknown(pcQueryString_4)))
               {
                  /* {get Prop var [handle]} */ 
                  pcQueryString_4.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getOpenQuery", targetProcedure()));
               }
            }
         }
         
         /* pcQueryString = ? */
         if (_or(isEqual(pcAndOr_5, ""), () -> isUnknown(pcAndOr_5)))
         {
            pcAndOr_5.assign("AND");
         }
         
         IntegerExpression resExpr15 = new IntegerExpression()
         {
            public integer execute()
            {
               return numEntries(cBufferList);
            }
         };
         ToClause toClause18 = new ToClause(iBuffer, 1, resExpr15);
         
         doTo("loopLabel62", toClause18, new Block((Body) () -> 
         {
            cBufWhere.assign("");
            cBuffer.assign(entry(iBuffer, cBufferList));
            
            IntegerExpression resExpr16 = new IntegerExpression()
            {
               public integer execute()
               {
                  return numEntries(pcColumns_4);
               }
            };
            ToClause toClause19 = new ToClause(iColumn, 1, resExpr16);
            
            doTo("columnLoop", toClause19, new Block((Body) () -> 
            {
               if ((matchesList(cUsedNums, valueOf(iColumn))).booleanValue())
               {
                  next("columnLoop");
               }
               
               /* get the column, operator and correctly quoted value from the lists 
                        if it maps to this buffer. */
               ControlFlowOps.invokeInWithMode("obtainExpressionEntries", targetProcedure(), "IIIIIOOO", cBuffer, iColumn, pcColumns_4, pcValues_4, pcOperators_5, cColumn, cOperator, cValue);
               
               if (_isEqual(cColumn, ""))
               {
                  next("columnLoop");
               }
               
               cUsedNums.assign(concat(cUsedNums, (_isEqual(cUsedNums, "") ? new character("") : new character(",")), valueOf(iColumn)));
               cBufWhere.assign(concat(cBufWhere, (_isEqual(cBufWhere, "") ? new character("") : new character(" AND ")), cColumn, " ", cOperator, " ", cValue));
            }));
            
            /* do iColumn = 1 to num-entries(pColumns) */
            /* We have a new expression */
            if (_isNotEqual(cBufWhere, ""))
            {
               pcQueryString_4.assign(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "newWhereClause", targetProcedure(), "IIII", cBuffer, cBufWhere, pcQueryString_4, pcAndOr_5));
            }
         }));
         
         /* do iBuffer = 1 to hQuery:num-buffers */
         returnNormal(pcQueryString_4);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character newQueryValidate(final character _pcQueryString_4, final character _pcExpression_4, final character _pcBuffer_6, final character _pcAndOr_5)
   {
      character pcQueryString_4 = TypeFactory.initInput(_pcQueryString_4);
      character pcExpression_4 = TypeFactory.initInput(_pcExpression_4);
      character pcBuffer_6 = TypeFactory.initInput(_pcBuffer_6);
      character pcAndOr_5 = TypeFactory.initInput(_pcAndOr_5);
      /*------------------------------------------------------------------------------
        Purpose: Inserts a new expression to the passed prepare string if the 
                 buffer reference, usually extracted from the expression, is valid.
                 (RETURNS ? if error. After an error message has been shown).  
      Parameter:  pcQueryString - Complete query string that the expression is
                                  going to be added to.
                  pcExpression  - The new expression to add to the query.
                                  The FIRST field reference is used as buffer 
                                  reference unless pcBuffer is defined. 
                  pcBuffer      - Optional buffer reference. 
                  pcAndOr       - Operator used to append if querystring already
                                  has an expression. 
                                  default to '='      
          Notes: The function avoids duplicate logic in add- and setQueryWhere and 
                 acts as a wrapper for newWhereClause with some significant
                 additions and differences. 
                  - The buffer reference is usually extracted from the new expression.
                  - pcBuffer is optional and thus not the first parameter.
                  - The buffer reference is validated and qualification does not
                    need to match the original query.
                  - Displays error message and returns ? if the buffer is unknown or
                    ambiguous. This will probably be changed to use addmessage, when
                    we have a more complete error/message service. The function is 
                    used in setQuerywhere and addQueryWhere and will almost certainly
                    happen on the client.(data.p setQueryWhere -> newQueryWhere->this)           
                  
                  If no buffer is found in the expression or passed the last 
                  entry in the Tables property is used.
                  
      Author's note: Here's how the order of the parameters were decided, in case 
                     someone (including myself) wonders...   
                   - The parameters are the same as newWhereClause, but pcBuffer is
                     optional (very optional), so it's seems wrong to have it first.
                   - The parameters use and meaning are more similar to addQueryWhere 
                     with one additional (pcQueryString), so the additional one was 
                     added first and the rest follows as in addQueryWhere.                                                                                                        
      ------------------------------------------------------------------------------*/
      character cBuffer = TypeFactory.character();
      character cTables = TypeFactory.character();
      character cError = TypeFactory.character();
      logical lExpression = TypeFactory.logical();
      
      return function("newQueryValidate", character.class, new Block((Body) () -> 
      {
         /* Unless buffer is defined, use the first buffer reference in the expression*/
         if (_or(isUnknown(pcBuffer_6), () -> isEqual(pcBuffer_6, "")))
         {
            pcBuffer_6 /* {fnarg <func-name> <arg> [handle] */ .assign(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "firstBufferName", targetProcedure(), "I", pcExpression_4));
            lExpression.assign(true);
         }
         
         /* Show expression if error */
         /* If no buffer passed or found above use the last of the object's Tables */
         if (_isUnknown(pcBuffer_6))
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("Tables"));
            cTables.assign(ghProp.unwrapBufferField().value());
            cBuffer.assign(entry(numEntries(cTables), cTables));
         }
         else
         {
            /* if passed buffer or found in expression 
                          Ensure correct qualification */
            cBuffer /* {fnarg <func-name> <arg> [handle] */ .assign(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "resolveBuffer", targetProcedure(), "I", pcBuffer_6));
            
            if (_or(isEqual(cBuffer, ""), () -> isUnknown(cBuffer)))
            {
               /* show expression if the buffer was extracted from it */
               /* show expression if the buffer was extracted from it */
               cError.assign(concat(valueOf(30), ",", pcBuffer_6 /* this is the buffer that was unresolved */, ",", ((lExpression).booleanValue() ? concat(chr(10), "(", pcExpression_4, ")") : new character(""))));
               /* {fnarg <func-name> <arg> [handle] */ 
               ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "showMessage", targetProcedure(), "I", cError);
               returnNormal(new character());
            }
         }
         /* else (a buffer ref is found or passed) */
         /* Add the expression to the query */
         returnNormal(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "newWhereClause", targetProcedure(), "IIII", cBuffer, pcExpression_4, pcQueryString_4, pcAndOr_5));
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character newQueryWhere(final character _pcWhere_6)
   {
      character pcWhere_6 = TypeFactory.initInput(_pcWhere_6);
      /*------------------------------------------------------------------------------
        Purpose: Returns a new query 
      Parameter: pcWhere - see setQueryWhere               
          Notes: FOR INTERNAL USE ONLY 
                 This functions exists only to be able to have the same logic in 
                 setQueryWhere in data.p and query.p.     
                 Returns unknown if the buffer reference is invalid. 
      ------------------------------------------------------------------------------*/
      character cQueryText = TypeFactory.character();
      character cQuery = TypeFactory.character();
      character cBlank = TypeFactory.character();
      
      return function("newQueryWhere", character.class, new Block((Body) () -> 
      {
         pcWhere_6.assign(leftTrim(pcWhere_6));
         
         if (_or(begins(pcWhere_6, "FOR "), () -> begins(pcWhere_6, "PRESELECT ")))
         {
            cQuery.assign(pcWhere_6);
         }
         else
         {
            /* {get Prop var [handle]} */ 
            cQueryText.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getOpenQuery", targetProcedure()));
            
            if (_and(and(isNotEqual(pcWhere_6, ""), () -> notUnknown(pcWhere_6)), () -> isNotEqual(pcWhere_6, "?")))
            {
               /* Add the passed where clause to the existing one. Shows error and 
                        returns ? if buffer in expression is unknown/ambiguous*/
               cQuery.assign(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "newQueryValidate", targetProcedure(), "IIII", cQueryText, pcWhere_6, new unknown(), new unknown()));
            }
            else
            {
                /* If they passed a blank argument, just use the original query. */cQuery.assign(cQueryText);
            }
         }
         
         /* else do (no "FOR ") */
         if (_notUnknown(cQuery))
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryString"));
            ghProp.unwrapBufferField().changeValue(cBlank);
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryColumns"));
            ghProp.unwrapBufferField().changeValue(cBlank);
         }
         
         returnNormal(cQuery);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character newWhereClause(final character _pcBuffer_6, final character _pcExpression_4, final character _pcWhere_6, final character _pcAndOr_5)
   {
      character pcBuffer_6 = TypeFactory.initInput(_pcBuffer_6);
      character pcExpression_4 = TypeFactory.initInput(_pcExpression_4);
      character pcWhere_6 = TypeFactory.initInput(_pcWhere_6);
      character pcAndOr_5 = TypeFactory.initInput(_pcAndOr_5);
      /*------------------------------------------------------------------------------
        Purpose:     Inserts a new expression to query's prepare string for a 
                     specified buffer.
        Parameters:  pcBuffer     - Buffer.  
                     pcExpression - The new expression. 
                     pcWhere      - The current query prepare string.
                     pcAndOr      - Specifies what operator is used to add the new
                                    expression to existing expression(s)
                                    - AND (default) 
                                    - OR                                                
        Notes:       This is supported as a 'utility function' that doesn't use any 
                     properties. However, if target-procedure = super the passed 
                     buffer's qualification MUST match the query's. 
                     If target-procedure <> super the buffer will be corrected IF 
                     it exists in the object's query, otherwise it needs to match 
      ------------------------------------------------------------------------------*/
      integer iStart = TypeFactory.integer();
      integer iLength = TypeFactory.integer();
      character cBufferWhere = TypeFactory.character();
      
      return function("newWhereClause", character.class, new Block((Body) () -> 
      {
         /* fix European decimal format issues with in query string */
         pcWhere_6 /* {fnarg <func-name> <arg> [handle] */ .assign(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "fixQueryString", targetProcedure(), "I", pcWhere_6));
         /* Find the buffer's 'expression-entry' in the query */
         cBufferWhere.assign(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "bufferWhereClause", targetProcedure(), "II", pcBuffer_6, pcWhere_6));
         
         /* if we found it, replace it with itself with the new expression inserted */
         if (_isNotEqual(cBufferWhere, ""))
         {
            iStart.assign(indexOf(pcWhere_6, cBufferWhere));
            iLength.assign(length(cBufferWhere));
            pcWhere_6.replace(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "insertExpression", targetProcedure(), "III", cBufferWhere, pcExpression_4, pcAndOr_5), iStart, iLength);
         }
         
         returnNormal(pcWhere_6);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical openQuery()
   {
      /*------------------------------------------------------------------------------
        Purpose:    General logic for opening the database query either for a 
                    SmartDataObject or for another query object.
        Parameters: <none>
        Notes:      The QueryString will be used to prepareQuery if not blank 
      ------------------------------------------------------------------------------*/
      handle hQuery = TypeFactory.handle();
      logical lCheckLast = TypeFactory.logical();
      character cQueryString = TypeFactory.character();
      logical lError = TypeFactory.logical();
      integer iCnt = TypeFactory.integer();
      handle hBuffer = TypeFactory.handle();
      character cRow = TypeFactory.character();
      logical lReturn = TypeFactory.logical();
      character cFetchOnOpen = TypeFactory.character();
      character cFindFunc = TypeFactory.character();
      
      return function("openQuery", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryHandle"));
         hQuery.assign(ghProp.unwrapBufferField().value());
         hQuery.unwrapQuery().queryClose();
         
         /* UpdateFromSource SDOs (1-to-1) depend on their data source to manage batching. */
         /* In doing so, the child's DB query is temporarily replaced with a join to the  */
         /* parent, so all batching properties as well as the current query string are reset */
         if (_and(new logical( /* {fn <func-name> [handle] */ ControlFlowOps.invokeDynamicFunctionIn(logical.class, "getUpdateFromSource", targetProcedure())), () -> isNotEqual( /* {fn <func-name> [handle] */ ControlFlowOps.invokeDynamicFunctionIn(character.class, "getASDivision", targetProcedure()), "CLIENT")))
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryString"));
            ghProp.unwrapBufferField().changeValue(new character(""));
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("FirstRowNum"));
            ghProp.unwrapBufferField().changeValue(new unknown());
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("LastRowNum"));
            ghProp.unwrapBufferField().changeValue(new unknown());
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("FirstResultRow"));
            ghProp.unwrapBufferField().changeValue(new unknown());
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("LastResultRow"));
            ghProp.unwrapBufferField().changeValue(new unknown());
            /* {set Prop value [handle]} */ 
            ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "setRowsToBatch", targetProcedure(), "I", 0);
         }
         
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryString"));
         cQueryString.assign(ghProp.unwrapBufferField().value());
         
         if (_isNotEqual(cQueryString, ""))
         {
            lError.assign(not(new logical( /* {fnarg <func-name> <arg> [handle] */ ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "prepareQuery", targetProcedure(), "I", cQueryString))));
         }
         else
         {
            if (_isUnknown(hQuery.unwrapQuery().prepareString()))
            {
               lError.assign(not(new logical( /* {fnarg <func-name> <arg> [handle] */ ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "prepareQuery", targetProcedure(), "I", ""))));
            }
         }
         
         if (_or(lError, () -> not(hQuery.unwrapQuery().queryOpen())))
         {
            lReturn.assign(false);
         }
         else
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("CheckLastOnOpen"));
            lCheckLast.assign(ghProp.unwrapBufferField().value());
            
            if ((lCheckLast).booleanValue())
            {
               hQuery.unwrapQuery().getLast();
               
               loopLabel63:
               for (iCnt.assign(1); _isLessThanOrEqual(iCnt, hQuery.unwrapQuery().numBuffers()); iCnt.increment())
               {
                  hBuffer.assign(hQuery.unwrapQuery().bufferHandle(iCnt));
                  cRow.assign(concat(cRow, (_isNotEqual(cRow, "") ? new character(",") : new character("")), valueOf(hBuffer.unwrapBuffer().rowID())));
                  ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
                  ghProp.assign(ghProp.unwrapBuffer().bufferField("LastDbRowIdent"));
                  ghProp.unwrapBufferField().changeValue(cRow);
               }
            }
            else
            {
               ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
               ghProp.assign(ghProp.unwrapBuffer().bufferField("LastDbRowIdent"));
               ghProp.unwrapBufferField().changeValue(cRow);
            }
            /* {set Prop value [handle]} */ 
            
            /* blank*/
            /* {get Prop var [handle]} */ 
            cFetchOnOpen.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getfetchOnOpen", targetProcedure()));
            
            if (_isNotEqual(cFetchOnOpen, ""))
            {
               if (_isGreaterThan(lookup(cFetchOnOpen, "FIRST,LAST"), 0))
               {
                  ControlFlowOps.invokeIn((concat("fetch", cFetchOnOpen)).toStringMessage(), targetProcedure());
               }
               else
               {
                  cFindFunc.assign(entry(1, cFetchOnOpen, chr(2)));
                  
                  switch (StringHelper.safeTrimTrailing((cFindFunc).toStringMessage()).toUpperCase())
                  {
                     case "FINDROWFROMOBJECT":
                        {
                           ControlFlowOps.invokeDynamicFunctionInWithMode(cFindFunc, targetProcedure(), "III", entry(2, cFetchOnOpen, chr(2)), entry(3, cFetchOnOpen, chr(2)), entry(4, cFetchOnOpen, chr(2)));
                        }
                        break;
                  }
               }
            }
            lReturn.assign(true);
         }
         /* END DO IF QUERY-OPEN */
         /* Some visual objects that shows more than one record may need to know 
              that the query changed, this cannot be detected through the ordinary
              publish "dataAvailable" from the navigation methods. 
              The SmartSelect populates its list on this event and OCX objects
              like lists and Tree-views may also need to subscribe to this event. */
         NamedEventManager.publish("queryOpened", targetProcedure());
         returnNormal(lReturn);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical prepareQuery(final character _pcQuery_3)
   {
      character pcQuery_3 = TypeFactory.initInput(_pcQuery_3);
      /*------------------------------------------------------------------------------
           Purpose: Prepare the database query either for a SmartDataObject or 
                    for another query object.              
        Parameters: pcQuery  - Complete query expression 
             Notes: This should normally not be called directly. There are basically 
                    3 ways that it is used (2 from openQuery): 
                    1: addQueryWhere, assignQuerywhere manipulates a client side
                       property that openQuery will check and use as input to this 
                       method.                
                    2: setQueryWhere will call this method (if on server). It will
                       also blank properties used by 1 in order to ensure that 
                       openQuery does NOT call this in that case.
                    3. openQuery will also call it if he query is unprepared when 
                       it is called. This function uses the OpenQuery - BaseQuery 
                       if called with blank or ? parameter. 
         Note date: 2002/07/02                        
      ------------------------------------------------------------------------------*/
      handle hQuery = TypeFactory.handle();
      
      return function("prepareQuery", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryHandle"));
         hQuery.assign(ghProp.unwrapBufferField().value());
         
         if (_or(isEqual(pcQuery_3, ""), () -> isUnknown(pcQuery_3)))
         {
            /* {get Prop var [handle]} */ 
            pcQuery_3.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getOpenQuery", targetProcedure()));
         }
         
         /* Fix European decimal format issues with in query string */
         pcQuery_3 /* {fnarg <func-name> <arg> [handle] */ .assign(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "fixQueryString", targetProcedure(), "I", pcQuery_3));
         returnNormal(hQuery.unwrapQuery().prepare(pcQuery_3));
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character refreshRowident(final character _pcRowident_1)
   {
      character pcRowident_1 = TypeFactory.initInput(_pcRowident_1);
      /*------------------------------------------------------------------------------
         Purpose: Returns a refreshed list of rowids for the readonly tables 
                  in the current query.
      Parameters: pcRowident - RowIdent of a row in  the current query.  
           Notes: Used internally by refetchDbRow in order to make changes to 
                  foreignfields reflected in correctly joined tables.  
                  
                  The refresh applies to read-only tables that are at the 
                  end of the join (no updatable tables further down in the join). 
                  The first Rowid is of course NEVER refreshed.  
                   
                  It is assuming that SDOs are used correctly,            
                  - Only one copy of the record that's being updated in the query,
                    which means one-to-one updates if many updatable records.   
                  
                  There are several changes that still require an open query to be
                  reflected.             
                  - Changes in a child record that changes or makes a parent 
                    relationship invalid is NOT reflected and will still require 
                    a complete open query.
                  
                  - Changes that makes an inner-join invalid will blank out Rowids 
                    for ALL read-only tables, even if some of them still was valid. 
                    Note that if the query was reopened this row would disappear from 
                    the query. Normally an application would of course validate that 
                    no invalid keys are entered, but a value could also be changed so 
                    that it was filtered away.   
                     
                  - Two updatable tables will not reflect changes in relationships
                    to each other.                 
      ------------------------------------------------------------------------------*/
      character cColumns = TypeFactory.character();
      integer iTable_1 = TypeFactory.integer();
      integer iFirstRotable = TypeFactory.integer();
      character cTables = TypeFactory.character();
      character cBufferQuery = TypeFactory.character();
      character cQueryWhere = TypeFactory.character();
      character cQuery = TypeFactory.character();
      integer iPos = TypeFactory.integer();
      character cRowident = TypeFactory.character();
      
      return function("refreshRowident", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("UpdatableColumnsByTable"));
         cColumns.assign(ghProp.unwrapBufferField().value());
         
         /* Nothing to refresh if only one table */
         if (_isEqual(numEntries(cColumns, ";"), 1))
         {
            returnNormal(pcRowident_1);
         }
         
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("Tables"));
         cTables.assign(ghProp.unwrapBufferField().value());
         
         loopLabel64:
         for (iTable_1.assign(numEntries(cTables)); _isGreaterThanOrEqual(iTable_1, 2); iTable_1.decrement())
         {
            if (_isNotEqual(entry(iTable_1, cColumns, ";"), ""))
            {
               break loopLabel64;
            }
            
            iFirstRotable.assign(iTable_1);
         }
         
         /* No readonly table found  */
         if (_isEqual(iFirstRotable, 0))
         {
            returnNormal(pcRowident_1);
         }
         
         /* Build a query that uniquely identifies all non-refreshable tables from
             the passed Rowident */
         cQuery.assign("FOR ");
         
         loopLabel65:
         for (iTable_1.assign(1); _isLessThanOrEqual(iTable_1, minus(iFirstRotable, 1)); iTable_1.increment())
         {
            cQuery.assign(concat(cQuery, "EACH ", entry(iTable_1, cTables), " WHERE ROWID(", entry(iTable_1, cTables), ") = to-rowid('", entry(iTable_1, pcRowident_1), "'),"));
         }
         
         /* do iTable = 1 to iFirstROTable -1 */
         /* {get Prop var [handle]} */ 
         cQueryWhere.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getQueryWhere", targetProcedure()));
         /* Find the WhereClause of the first RO table */
         cBufferQuery.assign(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "bufferWhereClause", targetProcedure(), "II", entry(iFirstRotable, cTables), cQueryWhere));
         /* Use it to find where to 'break' the query */
         iPos.assign(indexOf(cQueryWhere, cBufferQuery));
         /* Add the query built above that uniquely identifies non-refreshable tables 
               in front of the where clauses for the read only tables(s) */
         cQuery.assign(concat(cQuery, substring(cQueryWhere, iPos)));
         /* The refreshed rowids is the first row for this query */
         cRowident /* {fnarg <func-name> <arg> [handle] */ .assign(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "firstRowids", targetProcedure(), "I", cQuery));
         
         /* The query did not find any records; This may have many reasons,
             one of them being a change of a keyfield that makes an inner join fail, 
             we ALWAYS return non-refreshable rowidents, so we just blank the readonly.*/
         if (_isUnknown(cRowident))
         {
            loopLabel66:
            for (iTable_1.assign(iFirstRotable); _isLessThanOrEqual(iTable_1, numEntries(pcRowident_1)); iTable_1.increment())
            {
               pcRowident_1.replaceEntry(iTable_1, "");
            }
            
            returnNormal(pcRowident_1);
         }
         
         returnNormal(replaceAll(cRowident, "?", ""));
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical removeForeignKey()
   {
      /*------------------------------------------------------------------------------
        Purpose: Remove the ForeignKey from the query string. 
          Notes: The ForeignKey consists of ForeignKeys and ForeignValues. 
      ------------------------------------------------------------------------------*/
      character cLocalFields = TypeFactory.character();
      character cForeignFields = TypeFactory.character();
      character cForeignValues = TypeFactory.character();
      integer iField = TypeFactory.integer();
      handle hContainer = TypeFactory.handle();
      
      return function("removeForeignKey", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ForeignValues"));
         cForeignValues.assign(ghProp.unwrapBufferField().value());
         /* {get Prop var [handle]} */ 
         cForeignFields.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getForeignFields", targetProcedure()));
         
         if (_isEqual(cForeignFields, ""))
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("ContainerSource"));
            hContainer.assign(ghProp.unwrapBufferField().value());
            ErrorManager.silentErrorEnable();
            /* {get Prop var [handle]} */
            cForeignFields.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getForeignFields", hContainer));
            ErrorManager.silentErrorDisable();
            
            if (_or(isEqual(cForeignFields, ""), () -> isUnknown(cForeignFields)))
            {
               returnNormal(false);
            }
         }
         
         loopLabel67:
         for (iField.assign(1); _isLessThanOrEqual(iField, numEntries(cForeignFields)); iField.assign(plus(iField, 2)))
         {
            cLocalFields.assign(concat(cLocalFields, (_isNotEqual(cLocalFields, "") ? new character(",") : new character("")), entry(iField, cForeignFields)));
         }
         
         /*  DO iField -- find list of local part of Foreign Fields */
         returnNormal(ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "removeQuerySelection", ((hContainer.isValid()).booleanValue() ? hContainer : targetProcedure()), "II", cLocalFields, "="));
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical removeQuerySelection(final character _pcColumns_4, final character _pcOperators_5)
   {
      character pcColumns_4 = TypeFactory.initInput(_pcColumns_4);
      character pcOperators_5 = TypeFactory.initInput(_pcOperators_5);
      /*------------------------------------------------------------------------------
        Purpose:     Remove field expression(s) for specified column(s) and operator(s) 
                     added by assignQuerySelection from the query. 
        Parameters: 
           pcColumns   - Column names (Comma separated)                    
                         Fieldname of a table in the query in the form of 
                         TBL.FLDNM or DB.TBL.FLDNM (only if qualified with db is specified),
                         (RowObject.FLDNM should be used for SDO's)  
                         If the fieldname isn't qualified it checks the tables in 
                         the TABLES property and assumes the first with a match.              
           pcOperators - Operator - one for all columns
                                    - blank - defaults to (EQ)  
                                    - Use slash to define alternative string operator
                                      EQ/BEGINS etc..
                                  - comma separated for each column/value
      
        Notes:       This procedure modifies the QueryString property and is designed 
                     to run on the client and called several times before QueryString
                     is used in a QUERY-PREPARE method to modify the database query.
                     
                     openQuery will prepare the query using the QueryString property.
                     
                     The removal of the actual field expression is done by the help 
                     of the position and length stored in the QueryColumns property. 
      ------------------------------------------------------------------------------*/
      character cQueryString = TypeFactory.character();
      character cBufferList = TypeFactory.character();
      character cBuffer = TypeFactory.character();
      /* We need the columns name and the parts */
      character cColumn = TypeFactory.character();
      character cColumnName = TypeFactory.character();
      integer iBuffer = TypeFactory.integer();
      integer iColumn = TypeFactory.integer();
      character cUsedNums = TypeFactory.character();
      character cOperator = TypeFactory.character();
      character cStringOp = TypeFactory.character();
      /* Used to store and maintain offset and length */
      integer iValLength = TypeFactory.integer();
      integer iValPos = TypeFactory.integer();
      integer iPos = TypeFactory.integer();
      integer iDiff = TypeFactory.integer();
      character cQueryColumns = TypeFactory.character();
      character cQueryBufCols = TypeFactory.character();
      character cNewCols = TypeFactory.character();
      character cQueryColOp = TypeFactory.character();
      character cRemoveList = TypeFactory.character();
      integer iOldEntries = TypeFactory.integer();
      integer iLowestChanged = TypeFactory.integer();
      integer iBufPos = TypeFactory.integer();
      integer iWhereBufPos = TypeFactory.integer();
      character cQuerySplit1 = TypeFactory.character();
      character cQuerySplit2 = TypeFactory.character();
      integer iNumWords = TypeFactory.integer();
      character cWord = TypeFactory.character();
      
      return function("removeQuerySelection", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("Tables"));
         cBufferList.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryString"));
         cQueryString.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryColumns"));
         cQueryColumns.assign(ghProp.unwrapBufferField().value());
         
         /* If no QueryString or QueryColumns just return now */
         if (_or(isEqual(cQueryString, ""), () -> isEqual(cQueryColumns, "")))
         {
            returnNormal(false);
         }
         
         loopLabel68:
         for (iBuffer.assign(1); _isLessThanOrEqual(iBuffer, numEntries(cBufferList)); iBuffer.increment())
         {
            cBuffer.assign(entry(iBuffer, cBufferList));
            iBufPos.assign(lookup(cBuffer, cQueryColumns, ":"));
            cQueryBufCols.assign(_isGreaterThan(iBufPos, 0) ? entry(plus(iBufPos, 1), cQueryColumns, ":") : new character(""));
            iOldEntries.assign(divide(numEntries(cQueryBufCols), 3));
            cRemoveList.assign("");
            iLowestChanged.assign(0);
            
            IntegerExpression resExpr17 = new IntegerExpression()
            {
               public integer execute()
               {
                  return numEntries(pcColumns_4);
               }
            };
            ToClause toClause20 = new ToClause(iColumn, 1, resExpr17);
            
            doTo("columnLoop", toClause20, new Block((Body) () -> 
            {
               if ((matchesList(cUsedNums, valueOf(iColumn))).booleanValue())
               {
                  next("columnLoop");
               }
               
               cColumn.assign(entry(iColumn, pcColumns_4));
               /* Get the operator for this valuelist. 
                        Make sure we handle '',? and '/begins' as default */
               cOperator.assign(_or(or(isEqual(pcOperators_5, ""), () -> begins(pcOperators_5, "/")), () -> isUnknown(pcOperators_5)) ? new character("=") : _isEqual(numEntries(pcOperators_5), 1) ? entry(1, pcOperators_5, "/") : entry(iColumn, pcOperators_5));
               
               /* Convert rowObject reference to db reference */
               if (_begins(cColumn, "RowObject."))
               {
                  ErrorManager.silentErrorEnable();
                  cColumn.assign(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "columnDBColumn", targetProcedure(), "I", entry(2, cColumn, ".")));
                  ErrorManager.silentErrorDisable();
               }
               else
               {
                  if (_isEqual(indexOf(cColumn, "."), 0))
                  {
                     cColumn.assign(concat(cBuffer, ".", cColumn));
                  }
               }
               
               /* Wrong buffer */
               if (_not(begins(cColumn, concat(cBuffer, "."))))
               {
                  /* If the column db qualification does not match the query's we do 
                             an additionl check to see if it is the correct table after all */
                  if (_isNotEqual(minus(numEntries(cColumn, "."), numEntries(cBuffer, ".")), 1))
                  {
                     if (_isNotEqual( /* {fnarg <func-name> <arg> [handle] */ ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "columnTable", targetProcedure(), "I", cColumn), cBuffer))
                     {
                        next("columnLoop");
                     }
                  }
                  else
                  {
                     next("columnLoop");
                  }
               }
               cColumnName.assign(entry(numEntries(cColumn, "."), cColumn, "."));
               cQueryColOp.assign(cOperator);
               cQueryColOp.assign(trim(_isEqual(cQueryColOp, "GE") ? new character(">=") : _isEqual(cQueryColOp, "LE") ? new character("<=") : _isEqual(cQueryColOp, "LT") ? new character("<") : _isEqual(cQueryColOp, "GT") ? new character(">") : _isEqual(cQueryColOp, "EQ") ? new character("=") : cQueryColOp));
               cStringOp.assign(_and(isEqual(numEntries(pcOperators_5), 1), () -> isEqual(numEntries(pcOperators_5, "/"), 2)) ? entry(2, pcOperators_5, "/") : new character(""));
               
               /* First check if this was a string with stringoperator */
               if (_isNotEqual(cStringOp, ""))
               {
                  iPos.assign(lookup(concat(cColumnName, ".", cStringOp), cQueryBufCols));
               }
               else
               {
                  iPos.assign(0);
               }
               
               if (_isEqual(iPos, 0))
               {
                  iPos.assign(lookup(concat(cColumnName, ".", cQueryColOp), cQueryBufCols));
               }
               
               /* If the column + operator was found in the list
                          we build a list of the columns to remove from the QueryString further 
                          down.
                          We also build a list of the numbers to remove. */
               if (_isGreaterThan(iPos, 0))
               {
                  iLowestChanged.assign(int64.minimum(iPos, _isEqual(iLowestChanged, 0) ? iPos : iLowestChanged));
                  cRemoveList.assign(concat(cRemoveList, (_isEqual(cRemoveList, "") ? new character("") : new character(",")), valueOf(new integer(plus(divide(minus(iPos, 1), 3), 1)))));
                  cUsedNums.assign(concat(cUsedNums, (_isEqual(cUsedNums, "") ? new character("") : new character(",")), valueOf(iColumn)));
               }
            }));
            
            iWhereBufPos.assign(indexOf(concat(cQueryString, " "), concat(" ", cBuffer, " ")));
            iPos.assign(indexOf(cQueryString, concat(" ", cBuffer, ",")));
            iWhereBufPos.assign(plus((_and(isGreaterThan(iWhereBufPos, 0), () -> isGreaterThan(iPos, 0)) ? int64.minimum(iPos, iWhereBufPos) : int64.maximum(iPos, iWhereBufPos)), 1));
            iDiff.assign(0);
            cNewCols.assign("");
            
            if (_isGreaterThan(iLowestChanged, 0))
            {
               loopLabel69:
               for (iColumn.assign(1); _isLessThanOrEqual(iColumn, minus(numEntries(cQueryBufCols), 2)); iColumn.assign(plus(iColumn, 3)))
               {
                  iValPos.assign(new integer(entry(plus(iColumn, 1), cQueryBufCols)));
                  iValLength.assign(new integer(entry(plus(iColumn, 2), cQueryBufCols)));
                  iValPos.assign(minus(iValPos, iDiff));
                  
                  /* Remove value, operator, columnname and eventual AND in the
                           parenthesized expression. If it's the last expression in the parenthesis
                           we also remove it and any AND/OR after or WHERE/AND/OR before. */
                  if ((matchesList(cRemoveList, valueOf(new integer(plus(divide(minus(iColumn, 1), 3), 1))))).booleanValue())
                  {
                     /*Split query in two, remove value quotes and blanks */
                     cQuerySplit1.assign(rightTrim(substring(cQueryString, 1, minus(plus(iValPos, iWhereBufPos), 1)), " '\""));
                     cQuerySplit2.assign(leftTrim(substring(cQueryString, plus(plus(iValPos, iWhereBufPos), iValLength)), " '\""));
                     /* Count words in left part */
                     iNumWords.assign(numEntries(cQuerySplit1, " "));
                     /* Remove Operator */
                     cQuerySplit1.replaceEntry(iNumWords, " ", "");
                     cQuerySplit1.assign(rightTrim(cQuerySplit1, " "));
                     /* find columnname */
                     cWord.assign(entry(minus(iNumWords, 1), cQuerySplit1, " "));
                     
                     /* if columnname has parenthesis this is the beginning of an exp */
                     if (_begins(cWord, "("))
                     {
                        /* We are removing the first column/value pair in the parenthesis*/
                        if (_begins(cQuerySplit2, "AND"))
                        {
                           /* Remove the previous word (columnname), but keep parenthesis */
                           cQuerySplit1.replaceEntry(minus(iNumWords, 1), " ", "(");
                           /* Remove AND in right part */
                           cQuerySplit2.replaceEntry(1, " ", "");
                           cQuerySplit2.assign(leftTrim(cQuerySplit2, " "));
                        }
                        else
                        {
                           if (_begins(cQuerySplit2, ") "))
                           {
                              /* Remove columnname and parenthesises around the expression */
                              cQuerySplit1.replaceEntry(minus(iNumWords, 1), " ", "");
                              cQuerySplit1.assign(rightTrim(cQuerySplit1, " "));
                              cQuerySplit2.assign(leftTrim(substring(cQuerySplit2, 2), " "));
                              cWord.assign(entry(minus(iNumWords, 2), cQuerySplit1, " "));
                              
                              /* Now remove AND or OR used to join an eventual expression */
                              if (_or(begins(cQuerySplit2, "AND "), () -> begins(cQuerySplit2, "OR ")))
                              {
                                 cQuerySplit2.replaceEntry(1, " ", "");
                              }
                              else
                              {
                                 if ((matchesList("WHERE,AND,OR", cWord)).booleanValue())
                                 {
                                    cQuerySplit1.replaceEntry(minus(iNumWords, 2), " ", "");
                                 }
                                 else
                                 {
                                    cQuerySplit2.assign(concat(" ", cQuerySplit2));
                                 }
                              }
                           }
                        }
                     }
                     else
                     {
                        /* Now remove the prev word (columnname) */
                        cQuerySplit1.replaceEntry(minus(iNumWords, 1), " ", "");
                        /* Remove blanks in order to find the prev word */
                        cQuerySplit1.assign(rightTrim(cQuerySplit1, " "));
                        /* Now remove AND */
                        cQuerySplit1.replaceEntry(minus(iNumWords, 2), " ", "");
                        
                        /* If we removed the last column/value pair,
                                      we leave no space between the parenthesis */
                        if (_begins(cQuerySplit2, ")"))
                        {
                           cQuerySplit1.assign(rightTrim(cQuerySplit1));
                        }
                     }
                     /* Keep track of shrinkage */
                     iDiff.assign(plus(iDiff, minus(length(cQueryString), length(concat(cQuerySplit1, cQuerySplit2)))));
                     cQueryString.assign(concat(cQuerySplit1, cQuerySplit2));
                  }
                  else
                  {
                      /* not removed, store the position adjusted for shrinkage  */cNewCols.assign(concat(cNewCols, (_isEqual(cNewCols, "") ? new character("") : new character(",")), entry(iColumn, cQueryBufCols), ",", valueOf(iValPos), ",", (entry(plus(iColumn, 2), cQueryBufCols))));
                  }
               }
            }
            
            /* else if ilowestchanged do icolumn = ilowestChanged to num-entries */
            if (_isNotEqual(cNewCols, ""))
            {
               cQueryColumns.replaceEntry(plus(iBufPos, 1), ":", cNewCols);
            }
            else
            {
               if (_isGreaterThan(iLowestChanged, 0))
               {
                  cQueryColumns.replaceEntry(iBufPos, ":", "");
                  cQueryColumns.replaceEntry(plus(iBufPos, 1), ":", "");
                  cQueryColumns.assign(trim(replaceAll(cQueryColumns, ":::", ":"), ":"));
               }
            }
         }
         
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryColumns"));
         ghProp.unwrapBufferField().changeValue(cQueryColumns);
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryString"));
         ghProp.unwrapBufferField().changeValue(cQueryString);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character resolveBuffer(final character _pcBuffer_6)
   {
      character pcBuffer_6 = TypeFactory.initInput(_pcBuffer_6);
      /*------------------------------------------------------------------------------
        Purpose: Resolves the correct qualified buffer name of the passed buffer 
                 reference.  
      Parameter: Buffer name, qualifed or unqualified.          
          Notes: Returns blank if the buffer cannot be resolved in the SDO. 
                 Returns unknown if the table reference is ambiguous, (More than 
                 one table in the SDO matches the unqualifed input parameter).
               - Used internally (columnTable and others) to resolve cases where the 
                 passed column name's qualification is different from the object's.
               - There's no reference to the query handle in order to resolve this
                 on the client.     
      ------------------------------------------------------------------------------*/
      character cTableList = TypeFactory.character();
      logical lUseDbqualifier = TypeFactory.logical();
      character cDbtable = TypeFactory.character();
      character cDbnames = TypeFactory.character();
      integer iTable_1 = TypeFactory.integer();
      integer iEntry = TypeFactory.integer();
      
      return function("resolveBuffer", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("Tables"));
         cTableList.assign(ghProp.unwrapBufferField().value());
         
         /* if the table part matches Tables, just return it */
         if ((matchesList(cTableList, pcBuffer_6)).booleanValue())
         {
            returnNormal(pcBuffer_6);
         }
         
         /* {get Prop var [handle]} */ 
         lUseDbqualifier.assign(ControlFlowOps.invokeDynamicFunctionIn(logical.class, "getUseDBQualifier", targetProcedure()));
         /* {get Prop var [handle]} */ 
         cDbnames.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getDBNames", targetProcedure()));
         
         /* If DB Qualifier in parameter, but not in object */
         if (_and(isEqual(numEntries(pcBuffer_6, "."), 2), not(lUseDbqualifier)))
         {
            loopLabel70:
            for (iTable_1.assign(1); _isLessThanOrEqual(iTable_1, numEntries(cTableList)); iTable_1.increment())
            {
               cDbtable.assign(concat(entry(iTable_1, cDbnames), ".", entry(iTable_1, cTableList)));
               
               /* We found a Match, set cTable and leave the loop  */
               if (_isEqual(pcBuffer_6, cDbtable))
               {
                  returnNormal(entry(iTable_1, cTableList));
               }
            }
         }
         else
         {
            if (_and(isEqual(numEntries(pcBuffer_6, "."), 1), lUseDbqualifier))
            {
               loopLabel71:
               for (iTable_1.assign(1); _isLessThanOrEqual(iTable_1, numEntries(cTableList)); iTable_1.increment())
               {
                  cDbtable.assign(concat(entry(iTable_1, cDbnames), ".", pcBuffer_6));
                  
                  if (_isEqual(cDbtable, entry(iTable_1, cTableList)))
                  {
                     /* We already found an entry so return ? to signal amibiguity. */
                     if (_isNotEqual(iEntry, 0))
                     {
                        returnNormal(new character());
                     }
                     else
                     {
                        iEntry.assign(iTable_1);
                     }
                  }
               }
               
               /* Do iTable = 1 to num-entries(cTables) */
               if (_isNotEqual(iEntry, 0))
               {
                  returnNormal(entry(iEntry, cTableList));
               }
            }
         }
         /* else if .. no DBQual parameter, but DBqual in object  */
         /* We only get here if we're not able to resolve the table */
         returnNormal("");
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character rowidWhere(final character _pcWhere_6)
   {
      character pcWhere_6 = TypeFactory.initInput(_pcWhere_6);
      /*------------------------------------------------------------------------------
        Purpose:    Returns the ROWID (converted to a character string) of the first 
                    database query row satisfying the where clause.  In the case of a 
                    join, only the rowid of the first table in the join will be 
                    returned and the expression in pcWhere can only reference that 
                    table.
       
        Parameters:
          pcWhere - The where clause to apply to the database query to fetch the
                    first record whose ROWID will be returned.
                  -   
        
        Notes:      The ROWID is returned as a string both in anticipation of it
                    being used as an argument to fetchRowIdent, and also to allow
                    this function to be invoked from outside Progress.
      ------------------------------------------------------------------------------*/
      character cTables = TypeFactory.character();
      character cQueryWhere = TypeFactory.character();
      character cBuffer = TypeFactory.character();
      
      return function("rowidWhere", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("Tables"));
         cTables.assign(ghProp.unwrapBufferField().value());
         /* {get Prop var [handle]} */ 
         cQueryWhere.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getQueryWhere", targetProcedure()));
         cBuffer.assign(entry(1, cTables));
         cQueryWhere.assign(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "newWhereClause", targetProcedure(), "IIII", cBuffer, pcWhere_6, cQueryWhere, ""));
         returnNormal(entry(1, ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "firstRowIds", targetProcedure(), "I", cQueryWhere)));
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character rowidWhereCols(final character _pcColumns_4, final character _pcValues_4, final character _pcOperators_5)
   {
      character pcColumns_4 = TypeFactory.initInput(_pcColumns_4);
      character pcValues_4 = TypeFactory.initInput(_pcValues_4);
      character pcOperators_5 = TypeFactory.initInput(_pcOperators_5);
      /*------------------------------------------------------------------------------   
           Purpose: Returns a list of rowids    
                    Adds column/value pairs to the corresponding buffer's where-clause. 
                    Each buffer's expression will always be embedded in parenthesis.
           Parameters: 
             pcColumns   - Column names (Comma separated)                    
                           Fieldname of a table in the query in the form of 
                           TBL.FLDNM or DB.TBL.FLDNM (only if qualified with db is specified),
                           (RowObject.FLDNM should be used for SDO's)  
                           If the fieldname isn't qualified it checks the tables in 
                           the TABLES property and assumes the first with a match.
      
             pcValues    - corresponding Values (CHR(1) separated)
             pcOperators - Operator - one for all columns
                                      - blank - defaults to (EQ)  
                                      - Use slash to define alternative string operator
                                        EQ/BEGINS etc..
                                    - comma separated for each column/value
      ---------------------------------------------------------------------------*/
      character cQueryString = TypeFactory.character();
      character cRowids = TypeFactory.character();
      
      return function("rowidWhereCols", character.class, new Block((Body) () -> 
      {
         cQueryString.assign(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "newQueryString", targetProcedure(), "IIIII", pcColumns_4, pcValues_4, pcOperators_5, new unknown(), new unknown()));
         cRowids.assign(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "firstRowIds", targetProcedure(), "I", cQueryString));
         returnNormal(cRowids);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character sortExpression(final character _pcQueryString_4)
   {
      character pcQueryString_4 = TypeFactory.initInput(_pcQueryString_4);
      /*------------------------------------------------------------------------------
        Purpose: Returns the sort expression of the passed querystring 
          Notes: Includes the first BY also (getQuerySort does not) and 
                 removes extra spaces.
      ------------------------------------------------------------------------------*/
      integer iByPos = TypeFactory.integer();
      character cExpression = TypeFactory.character();
      
      return function("sortExpression", character.class, new Block((Body) () -> 
      {
         /* Any BY ? */
         iByPos.assign(indexOf(concat(pcQueryString_4, " "), " BY "));
         
         if (_isGreaterThan(iByPos, 0))
         {
            /* Trim away blanks and period and remove indexed-reposition */
            cExpression.assign(replaceAll(trim(substring(pcQueryString_4, iByPos), " ."), " INDEXED-REPOSITION", ""));
            
            loopLabel72:
            while (_isGreaterThan(indexOf(cExpression, "  "), 0))
            {
               cExpression.assign(replaceAll(cExpression, "  ", " "));
            }
         }
         
         returnNormal(cExpression);
      }));
   }
}
