/*--------------------------------------------------------------------------
    File        : Visual.java
    Purpose     : Code common to all objects with a visualization, including
                  Viewers, Browsers, Windows, and Frames

    Syntax      : adm2/visual.p

    Modified    : May 19, 1999 Version 9.1A
    Modified    : 11/14/2001          Mark Davies (MIP)
                  Renamed property 'DisplayFieldsSecurity' to 'FieldSecurity'
  ------------------------------------------------------------------------*/

package com.goldencode.hotel.common.adm2;

import com.goldencode.p2j.util.*;
import com.goldencode.p2j.ui.*;
import com.goldencode.hotel.ui.common.adm2.*;
import com.goldencode.util.*;

import static com.goldencode.p2j.util.BlockManager.*;
import static com.goldencode.p2j.util.CompareOps.*;
import static com.goldencode.p2j.util.MathOps.*;
import static com.goldencode.p2j.util.logical.*;
import static com.goldencode.p2j.util.TextOps.*;
import static com.goldencode.p2j.util.ProcedureManager.*;
import static com.goldencode.p2j.ui.LogicalTerminal.*;
import static com.goldencode.p2j.util.character.*;

/**
 * Business logic (converted to Java from the 4GL source code
 * in common/adm2/visual.p).
 */
public class Visual
{
   /*********************************************************************
   * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
   * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
   * below.  All Rights Reserved.                                       *
   *                                                                    *
   * The Initial Developer of the Original Code is PSC.  The Original   *
   * Code is Progress IDE code released to open source December 1, 2000.*
   *                                                                    *
   * The contents of this file are subject to the Possenet Public       *
   * License Version 1.0 (the "License"); you may not use this file     *
   * except in compliance with the License.  A copy of the License is   *
   * available as of the date of this notice at                         *
   * http://www.possenet.org/license.html                               *
   *                                                                    *
   * Software distributed under the License is distributed on an "AS IS"*
   * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
   * should refer to the License for the specific language governing    *
   * rights and limitations under the License.                          *
   *                                                                    *
   * Contributors:                                                      *
   *                                                                    *
   *********************************************************************/   
   /*          This .W file was created with the Progress UIB.             */   
   /*----------------------------------------------------------------------*/   
   /* ***************************  Definitions  ************************** */   
   /* Tell visattr.i that this is the Super procedure. */   
   /*********************************************************************
   * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
   * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
   * below.  All Rights Reserved.                                       *
   *                                                                    *
   * The Initial Developer of the Original Code is PSC.  The Original   *
   * Code is Progress IDE code released to open source December 1, 2000.*
   *                                                                    *
   * The contents of this file are subject to the Possenet Public       *
   * License Version 1.0 (the "License"); you may not use this file     *
   * except in compliance with the License.  A copy of the License is   *
   * available as of the date of this notice at                         *
   * http://www.possenet.org/license.html                               *
   *                                                                    *
   * Software distributed under the License is distributed on an "AS IS"*
   * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
   * should refer to the License for the specific language governing    *
   * rights and limitations under the License.                          *
   *                                                                    *
   * Contributors:                                                      *
   *                                                                    *
   *********************************************************************/   
   /*--------------------------------------------------------------------------
       File        : visualexclcustom.i
       Purpose     : Defines EXCLUDE-xxx preprocessor variables to exclude
                     functions or internal procedures in the super procedure
                     where it is referenced.
       Syntax      : {src/adm2/custom/visualexclcustom.i}
   
       Modified    : 06/03/1999
       
       Notes       : Referenced in src/adm2/visual.p
     ------------------------------------------------------------------------*/   
   integer giColor3Dface = TypeFactory.integer();

   integer giColor3DhighLight = TypeFactory.integer();

   integer giColor3Dshadow = TypeFactory.integer();

   /* _UIB-CODE-BLOCK-END */   
   /* *********************** Procedure Settings ************************ */   
   /* Settings for THIS-PROCEDURE
      Type: Procedure
      Allow: 
      Frames: 0
      Add Fields to: Neither
      Other Settings: CODE-ONLY COMPILE
    */   
   /* *************************  Create Window  ************************** */   
   /* DESIGN Window definition (used by the UIB) 
     CREATE WINDOW Procedure ASSIGN
            HEIGHT             = 20.71
            WIDTH              = 64.2.
   _* END WINDOW DEFINITION *_
                                                                           */   
   /* ************************* Included-Libraries *********************** */   
   /*********************************************************************
   * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
   * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
   * below.  All Rights Reserved.                                       *
   *                                                                    *
   * The Initial Developer of the Original Code is PSC.  The Original   *
   * Code is Progress IDE code released to open source December 1, 2000.*
   *                                                                    *
   * The contents of this file are subject to the Possenet Public       *
   * License Version 1.0 (the "License"); you may not use this file     *
   * except in compliance with the License.  A copy of the License is   *
   * available as of the date of this notice at                         *
   * http://www.possenet.org/license.html                               *
   *                                                                    *
   * Software distributed under the License is distributed on an "AS IS"*
   * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
   * should refer to the License for the specific language governing    *
   * rights and limitations under the License.                          *
   *                                                                    *
   * Contributors:                                                      *
   *                                                                    *
   *********************************************************************/   
   /*--------------------------------------------------------------------------
       File        : visprop.i
       Purpose     : Starts Super procedure visual.p and defines basic properties
                     for visual SmartObjects in V9
       Syntax      : {src/adm2/visprop.i}
   
       Description :
   
       Modified    : May 19, 1999 Version 9.1A
       Modified    : 10/23/2001          Mark Davies (MIP)
                     Added two new fields for ResizeHorizontal and ResizeVertical
       Modified    : 11/02/2001          Mark Davies (MIP)
                     Replaced properties for FrameMinHeightChars and FrameMinWidthChars with MinHeight and MinWidth
       Modified    : 11/14/2001          Mark Davies (MIP)
                     Renamed property 'DisplayFieldsSecurity' to 'FieldSecurity'
       Modified    : 02/27/2003          Neil Bell (MIP)
                     Added property 'SecuredTokens'                  
     ------------------------------------------------------------------------*/   
   /*          This .W file was created with the Progress UIB.             */   
   /*----------------------------------------------------------------------*/   
   /* ***************************  Definitions  ************************** */   
   /*********************************************************************
   * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
   * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
   * below.  All Rights Reserved.                                       *
   *                                                                    *
   * The Initial Developer of the Original Code is PSC.  The Original   *
   * Code is Progress IDE code released to open source December 1, 2000.*
   *                                                                    *
   * The contents of this file are subject to the Possenet Public       *
   * License Version 1.0 (the "License"); you may not use this file     *
   * except in compliance with the License.  A copy of the License is   *
   * available as of the date of this notice at                         *
   * http://www.possenet.org/license.html                               *
   *                                                                    *
   * Software distributed under the License is distributed on an "AS IS"*
   * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
   * should refer to the License for the specific language governing    *
   * rights and limitations under the License.                          *
   *                                                                    *
   * Contributors:                                                      *
   *                                                                    *
   *********************************************************************/   
   /*--------------------------------------------------------------------------
       File        : visualdefscustom.i
       Purpose     : Entry point to define:
                       New definitions
                       {&xcInstanceProperties}
                       {&ADM-PROPERTY-DLG}
       Syntax      : {src/adm2/custom/visualdefscustom.i}
   
       Modified    : 06/03/1999
       
       Notes       : Referenced in {src/adm2/visprop.i}
     ------------------------------------------------------------------------*/   
   /* This is the procedure to execute to set InstanceProperties at design time. */   
   /* _UIB-CODE-BLOCK-END */   
   /* ********************  Preprocessor Definitions  ******************** */   
   /* _UIB-PREPROCESSOR-BLOCK-END */   
   /* *********************** Procedure Settings ************************ */   
   /* Settings for THIS-PROCEDURE
      Type: Include
      Allow: 
      Frames: 0
      Add Fields to: Neither
      Other Settings: INCLUDE-ONLY
    */   
   /* *************************  Create Window  ************************** */   
   /* DESIGN Window definition (used by the UIB) 
     CREATE WINDOW Include ASSIGN
            HEIGHT             = 8
            WIDTH              = 60.
   _* END WINDOW DEFINITION *_
                                                                           */   
   /* ***************************  Main Block  *************************** */   
   /* Include the file which defines prototypes for all of the super
        procedure's entry points. Also, start or attach to the super procedure.
        Skip start-super-proc if we *are* the super procedure. 
        And skip including the prototypes if we are *any* super procedure. */   
   /* These preprocessors tell at compile time which properties can be
        retrieved directly from the temp-table. */   
   /* This property contains the security for all objects found in the AllFieldHandles Property */    
   /* Include the next property file up the chain (in this case, the top
        of the chain); it starts the property temp-table definition and we
        add our field definitions to it. */   
   /*********************************************************************
   * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
   * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
   * below.  All Rights Reserved.                                       *
   *                                                                    *
   * The Initial Developer of the Original Code is PSC.  The Original   *
   * Code is Progress IDE code released to open source December 1, 2000.*
   *                                                                    *
   * The contents of this file are subject to the Possenet Public       *
   * License Version 1.0 (the "License"); you may not use this file     *
   * except in compliance with the License.  A copy of the License is   *
   * available as of the date of this notice at                         *
   * http://www.possenet.org/license.html                               *
   *                                                                    *
   * Software distributed under the License is distributed on an "AS IS"*
   * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
   * should refer to the License for the specific language governing    *
   * rights and limitations under the License.                          *
   *                                                                    *
   * Contributors:                                                      *
   *                                                                    *
   *********************************************************************/   
   /*--------------------------------------------------------------------------
       File        : smrtprop.i
       Purpose     : Starts smart.p super procedure and defines general
                     SmartObject properties and other values.
       Syntax      : {src/adm2/smrtprop.i}
   
       Description :
   
       Modified    : February 4, 2000 - Version 9.1B
     ------------------------------------------------------------------------*/   
   /*          This .W file was created with the Progress UIB.             */   
   /*----------------------------------------------------------------------*/   
   /* ***************************  Definitions  ************************** */   
   /*********************************************************************
   * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
   * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
   * below.  All Rights Reserved.                                       *
   *                                                                    *
   * The Initial Developer of the Original Code is PSC.  The Original   *
   * Code is Progress IDE code released to open source December 1, 2000.*
   *                                                                    *
   * The contents of this file are subject to the Possenet Public       *
   * License Version 1.0 (the "License"); you may not use this file     *
   * except in compliance with the License.  A copy of the License is   *
   * available as of the date of this notice at                         *
   * http://www.possenet.org/license.html                               *
   *                                                                    *
   * Software distributed under the License is distributed on an "AS IS"*
   * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
   * should refer to the License for the specific language governing    *
   * rights and limitations under the License.                          *
   *                                                                    *
   * Contributors:                                                      *
   *                                                                    *
   *********************************************************************/   
   /*--------------------------------------------------------------------------
       File        : smartdefscustom.i
       Purpose     : Entry point to define:
                       New definitions
                       {&xcInstanceProperties}
                       {&ADM-PROPERTY-DLG}
       Syntax      : {src/adm2/custom/smartdefscustom.i}
   
       Modified    : 06/03/1999
       
       Notes       : Referenced in {src/adm2/smrtprop.i}
     ------------------------------------------------------------------------*/   
   /* Service Managers */   
   /*********************************************************************
   * Copyright (C) 2000-2002 by Progress Software Corporation ("PSC"),  *
   * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
   * below.  All Rights Reserved.                                       *
   *                                                                    *
   * The Initial Developer of the Original Code is PSC.  The Original   *
   * Code is Progress IDE code released to open source December 1, 2000.*
   *                                                                    *
   * The contents of this file are subject to the Possenet Public       *
   * License Version 1.0 (the "License"); you may not use this file     *
   * except in compliance with the License.  A copy of the License is   *
   * available as of the date of this notice at                         *
   * http://www.possenet.org/license.html                               *
   *                                                                    *
   * Software distributed under the License is distributed on an "AS IS"*
   * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
   * should refer to the License for the specific language governing    *
   * rights and limitations under the License.                          *
   *                                                                    *
   * Contributors:                                                      *
   *                                                                    *
   *********************************************************************/   
   /*---------------------------------------------------------------------------------
     File: globals.i
   
     Description:  ICF Global Shared Variables
   
     Purpose:      ICF Global Shared Variables
                   The global shared variables available within the Dynamics environment.
                   
                   NOTE: (Dynamic Only )
                   Shared variables do not span the client / server boundary. On the
                   client these are set-up in afstartupp.p and on the server in the agent
                   as_startup.p
                   Also note if we include the guts of a procedure as an internal
                   procedure for shared client/server code, we sometime need to
                   ensure the shared variables are not redefined.
   
     Parameters:   <none>
   
     History:
     --------
   
     Modified: 11/08/2001 - Mark Davies (MIP)
               Copied code for ICF afglobals.i and added here to allow
               ICF changed adm code to compile. afglobals.i now includes
               this file for definitions.
               Any ADM code will now reference globals.i which excludes any other
               ICF specific includes.
   -----------------------------------------------------------*/   
   /*                   This .W file was created with the Progress UIB.             */   
   /*-------------------------------------------------------------------------------*/   
   /* ***************************  Definitions  ************************** */   
   /* _UIB-CODE-BLOCK-END */   
   /* ********************  Preprocessor Definitions  ******************** */   
   /* _UIB-PREPROCESSOR-BLOCK-END */   
   /* *********************** Procedure Settings ************************ */   
   /* Settings for THIS-PROCEDURE
      Type: Include
      Allow: 
      Frames: 0
      Add Fields to: Neither
      Other Settings: INCLUDE-ONLY
    */   
   /* *************************  Create Window  ************************** */   
   /* DESIGN Window definition (used by the UIB) 
     CREATE WINDOW Include ASSIGN
            HEIGHT             = 5.57
            WIDTH              = 64.4.
   _* END WINDOW DEFINITION *_
                                                                           */   
   /* ***************************  Main Block  *************************** */   
   /* Used whereever Crystal is used and when defining the tt_datasource temp-table 
      to define the maximum columns supported when printing to crystal.
      See browsercustom.p and afcrpplip2p.p for example uses
   */   
   /* Used for filter checks */   
   /* The table delimiter used in assignlist, *ColumnsByTable props */   
   handle gshAstraAppserver /* quick access to all handles on object, e.g. viewer, whether enabled or not */  /* quick access to all fields on object, e.g. viewer, whether enabled or not */ = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshastraappserver", true, new handle());

   /* Handle to Astra Application Server Partition */   
   handle gshSessionManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshsessionmanager", true, new handle());

   /* Handle Astra Session Manager */   
   handle gshRimanager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshrimanager", true, new handle());

   /* Handle Astra Session Manager */   
   handle gshSecurityManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshsecuritymanager", true, new handle());

   /* Handle Astra Security Manager */   
   handle gshProfileManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshprofilemanager", true, new handle());

   /* Handle Astra Profile Manager */   
   handle gshRepositoryManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshrepositorymanager", true, new handle());

   /* Handle Astra Repository Manager */   
   handle gshTranslationManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshtranslationmanager", true, new handle());

   /* Handle Astra Translation Manager */   
   handle gshWebManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshwebmanager", true, new handle());

   /* Handle Astra Web Manager */   
   character gscSessionId = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gscsessionid", true, new character(""));

   /* Unique session id */   
   decimal gsdSessionObj = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gsdsessionobj", true, new decimal(0));

   /* Unique session id */   
   handle gshFinManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshfinmanager", true, new handle());

   /* Handle AstraFin Generic Manager */   
   handle gshGenManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshgenmanager", true, new handle());

   /* Handle AstraFramework Generic Manager */   
   handle gshAgnManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshagnmanager", true, new handle());

   /* Handle AstraGen Generic Manager */   
   decimal gsdTempUniqueId = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gsdtempuniqueid", true, new decimal(0));

   /* A number for unique objects */   
   decimal gsdUserObj = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gsduserobj", true, new decimal(0));

   /* User Object ID */   
   decimal gsdRenderTypeObj = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gsdrendertypeobj", true, new decimal(0));

   /* Render Type Object ID */   
   decimal gsdSessionScopeObj = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gsdsessionscopeobj", true, new decimal(0));

   /* Render Type Object ID */   
   /*********************************************************************
   * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
   * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
   * below.  All Rights Reserved.                                       *
   *                                                                    *
   * The Initial Developer of the Original Code is PSC.  The Original   *
   * Code is Progress IDE code released to open source December 1, 2000.*
   *                                                                    *
   * The contents of this file are subject to the Possenet Public       *
   * License Version 1.0 (the "License"); you may not use this file     *
   * except in compliance with the License.  A copy of the License is   *
   * available as of the date of this notice at                         *
   * http://www.possenet.org/license.html                               *
   *                                                                    *
   * Software distributed under the License is distributed on an "AS IS"*
   * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
   * should refer to the License for the specific language governing    *
   * rights and limitations under the License.                          *
   *                                                                    *
   * Contributors:                                                      *
   *                                                                    *
   *********************************************************************/   
   /*------------------------------------------------------------------------
       File        : globalscustom.i
       Purpose     : The developer can create his own version of this to 
                     allow 'global' definitions to be added to the 
                     framework.   
       Syntax      : {src/adm2/custom/globalscustom.i}
   
       Description :
   
       Created     : 06/03/1999
       Notes       : Referenced in {src/adm2/globals.i}
     ----------------------------------------------------------------------*/   
   /*          This .W file was created with the Progress AppBuilder.      */   
   /*----------------------------------------------------------------------*/   
   /* ***************************  Definitions  ************************** */   
   /* _UIB-CODE-BLOCK-END */   
   /* define the ADM Version and broker handle for all SmartObjects */   
   /* Note that adm-props-from-repository is documented as possible to define
        as a variable for flexible override, so it cannot be used for conditional 
        compile.  */   
   /* adm-props-from-repository is supported to be defined as a variable,
       so it is not used for conditional compile */   
   handle ghProp = TypeFactory.handle();

   VisualTestframe testframeFrame = null;

   /**
    * External procedure (converted to Java from the 4GL source code
    * in common/adm2/visual.p).
    */
   public void execute()
   {
      externalProcedure(Visual.this, new Block((Init) () -> 
      {
         gsdSessionObj.setPrecision(9);
         gsdTempUniqueId.setPrecision(9);
         gsdUserObj.setPrecision(9);
         gsdRenderTypeObj.setPrecision(9);
         gsdSessionScopeObj.setPrecision(9);
      }, 
      (Body) () -> 
      {
         /* _UIB-CODE-BLOCK-END */
         /* *********************** Procedure Settings ************************ */
         /* Settings for THIS-PROCEDURE
            Type: Method-Library
            Allow: 
            Frames: 0
            Add Fields to: Neither
            Other Settings: INCLUDE-ONLY
          */
         /* *************************  Create Window  ************************** */
         /* DESIGN Window definition (used by the UIB) 
           CREATE WINDOW Method-Library ASSIGN
                  HEIGHT             = 14.86
                  WIDTH              = 57.2.
         _* END WINDOW DEFINITION *_
                                                                                 */
         /* ***************************  Main Block  *************************** */
         /* Include the file which defines prototypes for all of the super
              procedure's entry points. Also, start or attach to the super procedure.
              Skip start-super-proc if we *are* the super procedure. 
              And skip including the prototypes if we are *any* super procedure. */
         /* These preprocessors let the get and set methods know at compile time
             which property values are located in the temp-table and which must
             be accessed through the property functions.
          */
         /* for support of dynamic objects */
         /* physical name - no path */
         /* for support of runtime parameters passed from the menus */
         /* Properties for super procedure management */
         /* The name of the super procedure; can be a CSV list */ 
 /* STATELESS, STATEFUL; the mode that all supers for the object will be launched in */ 
 /* Runtime property storing handles to the object's super procedures, defined in SuperProcedure. */
         /* Layout support for dynamic objects */
         /* contains a layout position that allows an object to be laid out correctly. */
         /* determine if we need to create the prop fields here or from the */
         /* repository (if ICF is running ) */
         /* COMMENT THE FOLLOWING LINE TO USE THE "<class>prop.i" DEFINITIONS */
         /* This temp-table defines all the propertt fields for an object.
             This include file contributes the DEFINE statement header and
             all basic smart object properties. Each other property class include file
             adds its own fields and then the parent object ends the statement.
             Define the fields for smart objects only, not for their super procedures.
          */
         if (_not(handle.fromString(entry(1, thisProcedure().unwrapADMData().getADMData(), chr(1))).isValid()))
         {
         }
         
         /* _UIB-CODE-BLOCK-END */
         /* else if appserver aware (adecomm/appserv.i) include appserver props */
         if (_not(handle.fromString(entry(1, thisProcedure().unwrapADMData().getADMData(), chr(1))).isValid()))
         {
         }
         
         /* _UIB-CODE-BLOCK-END */
         /* _UIB-CODE-BLOCK-END */
         /* ***************************  Main Block  *************************** */
         ControlFlowOps.invokeIn("initWinColors", targetProcedure());
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * ***********************  Function Implementations****************
    */
   public character getObjectType()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Returns the Type of the SmartObject, such as "SmartDataObject", etc.
                  For an ADM Super procedure, this function returns "SUPER".
         Params:  <none>
      ------------------------------------------------------------------------------*/
      character cType = TypeFactory.character();
      
      return function("getObjectType", character.class, new Block((Body) () -> 
      {
         cType.assign("SUPER");
         returnNormal(cType);
      }));
   }

   /** *********************  Internal Procedures ********************** */
   /**
    * Purpose:     Apply the value of the Layout Attribute. This changes the<br>
    *              object display to an alternate layout when multiple layouts<br>
    *              have been defined for the object.<br>
    * Parameters:  &lt;none&gt;<br>
    * Notes:
    */
   public void applyLayout()
   {
      character cLayout = TypeFactory.character();
      character cLayoutVar = TypeFactory.character();
      
      internalProcedure(new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ObjectLayout"));
         cLayout.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("LayoutVariable"));
         cLayoutVar.assign(ghProp.unwrapBufferField().value());
         
         if (_isUnknown(cLayout))
         {
            cLayout.assign("");
         }
         
         /* A blank cLayout means return to the Default-Layout. If there is
              no default-layout, then return to the Master Layout. */
         if (_isEqual(cLayout, ""))
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("defaultLayout"));
            cLayout.assign(ghProp.unwrapBufferField().value());
            
            if (_isUnknown(cLayout))
            {
               cLayout.assign("");
            }
            
            if (_isEqual(cLayout, ""))
            {
               cLayout.assign("Master Layout");
            }
         }
         
         /* IF cLayout eq ... */
         /* Does the layout REALLY need changing, or is it already in use? */
         if (_isNotEqual(cLayoutVar, cLayout))
         {
            /* Always change layouts by FIRST resetting to the Master Layout. 
                   (assuming that the layout isn't already the Master.) */
            if (_isNotEqual(cLayoutVar, "Master Layout"))
            {
               ControlFlowOps.invokeInWithMode((concat(cLayoutVar, "s")).toStringMessage(), targetProcedure(), "I", new character("Master Layout"));
            }
            
            /* Now change to the desired layout. */
            if (_isNotEqual(cLayout, "Master Layout"))
            {
               ControlFlowOps.invokeInWithMode((concat(cLayoutVar, "s")).toStringMessage(), targetProcedure(), "I", cLayout);
            }
         }
         /* IF...LAYOUT...ne cLayout... */
         returnNormal();
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     <br>
    * Parameters:  &lt;none&gt;<br>
    * Notes:
    */
   public void createObjects()
   {
      logical lUseRepository = TypeFactory.logical();
      
      internalProcedure(new Block((Body) () -> 
      {
         ErrorManager.silentErrorEnable();
         /* Added NO-ERROR to super call because certain objects do not have a super
                for createObjects after this point in the stack (it might be called earlier).
                Toolbars are an example of this.
              */
         ControlFlowOps.runSuper("createObjects");
         ErrorManager.silentErrorDisable();
         /* Set the UI Events for this object */
         /* {get Prop var [handle]} */ 
         lUseRepository.assign(ControlFlowOps.invokeDynamicFunctionIn(logical.class, "getUseRepository", targetProcedure()));
         
         if ((lUseRepository).booleanValue())
         {
            ErrorManager.silentErrorEnable();
            /* {fn <func-name> [handle] */ 
            ControlFlowOps.invokeDynamicFunctionIn(logical.class, "createUiEvents", targetProcedure());
            ErrorManager.silentErrorDisable();
         }
         
         ErrorManager.setError(new logical(false));
         returnNormal();
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     <br>
    * Parameters:  &lt;none&gt;<br>
    * Notes:
    */
   public void destroyObject()
   {
      internalProcedure(new Block((Body) () -> 
      {
         /* {fn <func-name> [handle] */ 
         ControlFlowOps.invokeDynamicFunctionIn(logical.class, "destroyPopups", targetProcedure());
         ErrorManager.silentErrorEnable();
         ControlFlowOps.runSuper("destroyObject");
         ErrorManager.silentErrorDisable();
         
         if (_or(ErrorManager.isError(), () -> isEqual(ControlFlowOps.getReturnValue(), "ADM-ERROR")))
         {
            returnError("ADM-ERROR");
         }
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * <p>
    * Purpose:     Disables all enabled objects in the frame.<br>
    *              Note that this includes RowObject fields if any.<br>
    * Parameters:  &lt;none&gt;<br>
    * Notes:
    */
   public void disableObject()
   {
      character cEnabledObjHdls = TypeFactory.character();
      integer iField = TypeFactory.integer();
      handle hField = TypeFactory.handle();
      logical lObjectHidden = TypeFactory.logical();
      handle hPopup = TypeFactory.handle();
      
      internalProcedure(new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("EnabledObjHdls"));
         cEnabledObjHdls.assign(ghProp.unwrapBufferField().value());
         
         if (_isNotEqual(cEnabledObjHdls, ""))
         {
            IntegerExpression resExpr0 = new IntegerExpression()
            {
               public integer execute()
               {
                  return numEntries(cEnabledObjHdls);
               }
            };
            ToClause toClause0 = new ToClause(iField, 1, resExpr0);
            
            doTo("loopLabel0", toClause0, new Block((Body) () -> 
            {
               hField.assign(handle.fromString(entry(iField, cEnabledObjHdls)));
               
               /* Cater for local SDF's */
               if (_isEqual(hField.unwrapType().getResourceType(), "PROCEDURE"))
               {
                  /* {get Prop var [handle]} */ 
                  lObjectHidden.assign(ControlFlowOps.invokeDynamicFunctionIn(logical.class, "getObjectHidden", targetProcedure()));
                  
                  if (_not(lObjectHidden))
                  {
                     ControlFlowOps.invokeIn("DisableField", hField);
                  }
               }
               else
               {
 /* Skip fields hidden for multi-layout etc.*/
                  if (_and(HandleOps.canSet(hField, "HIDDEN"), () -> not(hField.unwrapWidget().isHidden())))
                  {
                     hPopup /* {fnarg <func-name> <arg> [handle] */ .assign(ControlFlowOps.invokeDynamicFunctionInWithMode(handle.class, "popupHandle", targetProcedure(), "I", hField));
                     ErrorManager.silentErrorEnable();
                     hField.unwrapSensitive().setSensitive(new logical(false));
                     hPopup.unwrapSensitive().setSensitive(new logical(false));
                     ErrorManager.silentErrorDisable();
                  }
               }
            }));
         }
         ErrorManager.silentErrorEnable();
         /* END DO iField */
         ControlFlowOps.invokeInWithMode("disableFields", targetProcedure(), "I", new character("ALL"));
         ErrorManager.silentErrorDisable();
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ObjectEnabled"));
         ghProp.unwrapBufferField().changeValue(new logical(false));
         returnNormal();
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * <p>
    * Purpose:    Enable an object - all components except RowObject fields,<br>
    *             which are enabled using EnableFields().<br>
    * Parameters:  &lt;none&gt;<br>
    * Notes:
    */
   public void enableObject()
   {
      character cAllFieldHandles = TypeFactory.character();
      character cEnabledObjHdls = TypeFactory.character();
      character cSecuredFields = TypeFactory.character();
      integer iField = TypeFactory.integer();
      integer iFieldPos = TypeFactory.integer();
      handle hField = TypeFactory.handle();
      logical lObjectHidden = TypeFactory.logical();
      handle hPopup = TypeFactory.handle();
      
      internalProcedure(new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FieldSecurity"));
         cSecuredFields.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("AllFieldHandles"));
         cAllFieldHandles.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("EnabledObjHdls"));
         cEnabledObjHdls.assign(ghProp.unwrapBufferField().value());
         
         if (_isNotEqual(cEnabledObjHdls, ""))
         {
            IntegerExpression resExpr1 = new IntegerExpression()
            {
               public integer execute()
               {
                  return numEntries(cEnabledObjHdls);
               }
            };
            ToClause toClause1 = new ToClause(iField, 1, resExpr1);
            
            doTo("loopLabel1", toClause1, new Block((Body) () -> 
            {
               hField.assign(handle.fromString(entry(iField, cEnabledObjHdls)));
               iFieldPos.assign(0);
               iFieldPos.assign(lookup(valueOf(hField), cAllFieldHandles));
               
               if ((hField.isValid()).booleanValue())
               {
                  /* Cater for local SDF's */
                  if (_isEqual(hField.unwrapType().getResourceType(), "PROCEDURE"))
                  {
                     /* {get Prop var [handle]} */ 
                     lObjectHidden.assign(ControlFlowOps.invokeDynamicFunctionIn(logical.class, "getObjectHidden", targetProcedure()));
                     
                     if (_and(not(lObjectHidden), () -> or(or(and(and(isNotEqual(iFieldPos, 0), () -> isGreaterThanOrEqual(numEntries(cSecuredFields), iFieldPos)), () -> isEqual(entry(iFieldPos, cSecuredFields), "")), () -> isEqual(iFieldPos, 0)), () -> isEqual(cSecuredFields, ""))))
                     {
 /* Check Security */
                        ControlFlowOps.invokeIn("enableField", hField);
                     }
                     else
                     {
                        if (_isGreaterThanOrEqual(numEntries(cSecuredFields), iFieldPos))
                        {
                           if (_isEqual(entry(iFieldPos, cSecuredFields), "Hidden"))
                           {
                              ErrorManager.silentErrorEnable();
                              ControlFlowOps.invokeIn("hideObject", hField);
                              ErrorManager.silentErrorDisable();
                           }
                           else
                           {
                              if (_isNotEqual(entry(iFieldPos, cSecuredFields), ""))
                              {
                                 ErrorManager.silentErrorEnable();
                                 ControlFlowOps.invokeIn("disableField", hField);
                                 ErrorManager.silentErrorDisable();
                              }
                           }
                        }
                     }
                  }
                  else
                  {
                     if (_and(and(HandleOps.canSet(hField, "HIDDEN"), () -> not(hField.unwrapWidget().isHidden())), () -> or(or(and(and(isNotEqual(iFieldPos, 0), () -> isGreaterThanOrEqual(numEntries(cSecuredFields), iFieldPos)), () -> isEqual(entry(iFieldPos, cSecuredFields), "")), () -> isEqual(iFieldPos, 0)), () -> isEqual(cSecuredFields, ""))))
                     {
                        hField.unwrapSensitive().setSensitive(new logical(true));
                        
                        /* don't enable if read-only (can-query is ok as only fields can 
                                     have popups) */
                        if (_and(HandleOps.canQuery(hField, "read-only"), () -> not(hField.unwrapWidget().isReadOnly())))
                        {
                           ErrorManager.silentErrorEnable();
                           hPopup /* {fnarg <func-name> <arg> [handle] */ .assign(ControlFlowOps.invokeDynamicFunctionInWithMode(handle.class, "popupHandle", targetProcedure(), "I", hField));
                           hPopup.unwrapSensitive().setSensitive(new logical(true));
                           ErrorManager.silentErrorDisable();
                        }
                     }
                  }
               }
            }));
         }
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ObjectEnabled"));
         ghProp.unwrapBufferField().changeValue(new logical(true));
         ErrorManager.silentErrorEnable();
         /* We also run enable_UI from here. */
         ControlFlowOps.invokeIn("enable_UI", targetProcedure());
         ErrorManager.silentErrorDisable();
         returnNormal();
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:  Initialization code specific to visualizations.<br>
    *  Params:  &lt;none&gt;<br>
    *   Notes:  Enables and views the object depending on the values of <br>
    *           DisableOnInit and HideOnInit instance properties.
    */
   public void initializeObject()
   {
      logical lHideOnInit = TypeFactory.logical();
      logical lDisableOnInit = TypeFactory.logical();
      character cLayout = TypeFactory.character();
      handle hContainer = TypeFactory.handle();
      handle hParent = TypeFactory.handle();
      character cEnabledObjFlds = TypeFactory.character();
      character cEnabledObjHdls = TypeFactory.character();
      handle hFrameField = TypeFactory.handle();
      character cResult = TypeFactory.character();
      character cAllFieldHandles = TypeFactory.character();
      character cAllFieldNames = TypeFactory.character();
      handle hContainerSource = TypeFactory.handle();
      integer iSdfloop = TypeFactory.integer();
      character cContainerTargets = TypeFactory.character();
      handle hSdfhandle = TypeFactory.handle();
      handle hSdfframeHandle = TypeFactory.handle();
      handle hFrameProc = TypeFactory.handle();
      character cFieldName = TypeFactory.character();
      character cFrameHandles = TypeFactory.character();
      handle hFrame = TypeFactory.handle();
      integer iSdf = TypeFactory.integer();
      logical lAllFieldsSet = TypeFactory.logical();
      logical lPopupsInFields = TypeFactory.logical();
      
      internalProcedure(new Block((Body) () -> 
      {
         ControlFlowOps.runSuper("initializeObject");
         
         if (_isEqual(ControlFlowOps.getReturnValue(), "ADM-ERROR"))
         {
            returnNormal("ADM-ERROR");
         }
         
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("AllFieldHandles"));
         cAllFieldHandles.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("AllFieldNames"));
         cAllFieldNames.assign(ghProp.unwrapBufferField().value());
         
         if (_and(isNotEqual(cAllFieldHandles, ""), () -> isNotEqual(cAllFieldNames, "")))
         {
            lAllFieldsSet.assign(true);
         }
         
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("UIBMode"));
         cResult.assign(ghProp.unwrapBufferField().value());
         
         if (_begins(cResult, "Design"))
         {
            ControlFlowOps.invokeIn("viewObject", targetProcedure());
            returnNormal();
         }
         
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ContainerHandle"));
         hContainer.assign(ghProp.unwrapBufferField().value());
         
         if (_not(hContainer.isValid()))
         {
            returnNormal();
         }
         
         /* If no container then skip all visual initialization. */
         hParent.assign(hContainer);
         
         if (_isEqual(hContainer.unwrapType().getResourceType(), "Window"))
         {
            /* {get Prop var [handle]} */ 
            hFrame.assign(ControlFlowOps.invokeDynamicFunctionIn(handle.class, "getWindowFrameHandle", targetProcedure()));
         }
         else
         {
            hFrame.assign(hContainer);
         }
         
         loopLabel2:
         while (_and(hParent.unwrapWidget().getParentHandle().isValid(), () -> isNotEqual(hParent.unwrapType().getResourceType(), "WINDOW")))
         {
            hParent.assign(hParent.unwrapWidget().getParentHandle());
         }
         
         if (_and(hParent.isValid(), () -> isEqual(hParent.unwrapType().getResourceType(), "WINDOW")))
         {
            targetProcedure().unwrapProcedure().setCurrentWindow(hParent);
         }
         
         ErrorManager.silentErrorEnable();
         /* Build a list of frame handles corresponding to containertargets list to use 
              later when checking for SmartObjects  */
         cContainerTargets.assign(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "linkHandles", targetProcedure(), "I", "Container-Target"));
         ErrorManager.silentErrorDisable();
         
         if (_isNotEqual(cContainerTargets, ""))
         {
            cFrameHandles.assign(fill(",", minus(numEntries(cContainerTargets), 1)));
            
            loopLabel3:
            for (iSdfloop.assign(1); _isLessThanOrEqual(iSdfloop, numEntries(cContainerTargets)); iSdfloop.increment())
            {
               hSdfhandle.assign(handle.fromString(entry(iSdfloop, cContainerTargets)));
               hSdfframeHandle.setUnknown();
               
               if ((hSdfhandle.isValid()).booleanValue())
               {
                  ghProp.assign(handle.fromString(entry(1, hSdfhandle.unwrapADMData().getADMData(), chr(1))));
                  ghProp.assign(ghProp.unwrapBuffer().bufferField("ContainerHandle"));
                  hSdfframeHandle.assign(ghProp.unwrapBufferField().value());
               }
               
               /* if the containerHandle is a child of this object's frame then add it to the list */
               cFrameHandles.replaceEntry(iSdfloop, _and(and(hSdfframeHandle.isValid(), () -> isEqual(hSdfframeHandle.unwrapType().getResourceType(), "FRAME")), () -> isEqual(hFrame, hSdfframeHandle.unwrapWidget().getFrameHandle())) ? valueOf(hSdfframeHandle) : new character("?"));
            }
         }
         
         /* end if ccontainertargets <> ""*/
         /* Build a list of the handles of ENABLED-OBJECTS, for enable/disableObject.
              These are *non*-db fields. */
         if (_and(hFrame.isValid(), () -> notUnknown(hFrame.unwrapTree().firstChild())))
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("EnabledObjFlds"));
            cEnabledObjFlds.assign(ghProp.unwrapBufferField().value());
            cEnabledObjHdls.assign(fill(",", minus(numEntries(cEnabledObjFlds), 1)));
            hFrameField.assign(hFrame.unwrapTree().firstChild());
             /* Field Group */hFrameField.assign(hFrameField.unwrapTree().firstChild());
            
            LogicalOp whileClause0 = () -> hFrameField.isValid();
            
            doWhile("loopLabel4", whileClause0, new Block((Body) () -> 
            {
               if (_isGreaterThan(lookup(hFrameField.unwrap().name(), cEnabledObjFlds), 0))
               {
                  if (_isNotEqual(lookup(hFrameField.unwrapType().getResourceType(), "FILL-IN,RADIO-SET,EDITOR,COMBO-BOX,SELECTION-LIST,SLIDER,TOGGLE-BOX,BROWSE,BUTTON"), 0))
                  {
                     cEnabledObjHdls.replaceEntry(lookup(hFrameField.unwrap().name(), cEnabledObjFlds), valueOf(hFrameField));
                  }
                  else
                  {
                     cEnabledObjHdls.assign(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "deleteEntry", targetProcedure(), "III", lookup(hFrameField.unwrap().name(), cEnabledObjFlds), cEnabledObjHdls, ","));
                     cEnabledObjFlds.assign(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "deleteEntry", targetProcedure(), "III", lookup(hFrameField.unwrap().name(), cEnabledObjFlds), cEnabledObjFlds, ","));
                  }
               }
               cFieldName.assign("");
               hFrameProc.setUnknown();
               
               if (_and(isEqual(hFrameField.unwrapType().getResourceType(), "FRAME"), () -> isNotEqual(cContainerTargets, "")))
               {
                  /* If this is a SmartObject (one of the frames we found in the container targets) 
                             then we need to use the proc handle and field name or objectname in the lists */
                  iSdf.assign(lookup(valueOf(hFrameField), cFrameHandles));
                  
                  if (_isGreaterThan(iSdf, 0))
                  {
                     hFrameProc.assign(handle.fromString(entry(iSdf, cContainerTargets)));
                     
                     if ((hFrameProc.isValid()).booleanValue())
                     {
                        ErrorManager.silentErrorEnable();
                        cFieldName.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getFieldName", hFrameProc));
                        ErrorManager.silentErrorDisable();
                        
                        if (_isUnknown(cFieldName))
                        {
                           ErrorManager.silentErrorEnable();
                           cFieldName.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getObjectName", hFrameProc));
                           ErrorManager.silentErrorDisable();
                        }
                        
                        ErrorManager.silentErrorEnable();
                        /* SDFs are currently not in the object list, but ...  */
                        cEnabledObjHdls.replaceEntry(lookup(cFieldName, cEnabledObjFlds), valueOf(hFrameProc));
                        ErrorManager.silentErrorDisable();
                        ErrorManager.silentErrorEnable();
                        cEnabledObjFlds.replaceEntry(lookup(cFieldName, cEnabledObjFlds), cFieldName);
                        ErrorManager.silentErrorDisable();
                     }
                  }
               }
               else
               {
                  hFrameProc.setUnknown();
               }
               
               if (_not(lAllFieldsSet))
               {
                  cAllFieldHandles.assign(concat(cAllFieldHandles, (_isNotEqual(cAllFieldHandles, "") ? new character(",") : new character("")), (_isUnknown(hFrameProc) ? valueOf(hFrameField) : valueOf(hFrameProc))));
                  cAllFieldNames.assign(concat(cAllFieldNames, (_isNotEqual(cAllFieldNames, "") ? new character(",") : new character("")), (_isNotEqual(cFieldName, "") ? cFieldName : _and(HandleOps.canQuery(hFrameField, "name"), () -> notUnknown(hFrameField.unwrap().name())) ? hFrameField.unwrap().name() : new character("?"))));
               }
               
               hFrameField.assign(hFrameField.unwrap().getNextSibling());
            }));
            
            cEnabledObjHdls.assign(trim(cEnabledObjHdls, ","));
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("EnabledObjHdls"));
            ghProp.unwrapBufferField().changeValue(cEnabledObjHdls);
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("EnabledObjFlds"));
            ghProp.unwrapBufferField().changeValue(cEnabledObjFlds);
            
            /* we might have deleted text fields*/
            if (_not(lAllFieldsSet))
            {
               ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
               ghProp.assign(ghProp.unwrapBuffer().bufferField("AllFieldHandles"));
               ghProp.unwrapBufferField().changeValue(cAllFieldHandles);
               ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
               ghProp.assign(ghProp.unwrapBuffer().bufferField("AllFieldNames"));
               ghProp.unwrapBufferField().changeValue(cAllFieldNames);
            }
         }
         
         /* if valid-handle(container) */
         if (_isNotEqual(hContainer.unwrapType().getResourceType(), "window"))
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("DisableOnInit"));
            lDisableOnInit.assign(ghProp.unwrapBufferField().value());
            
            if (_not(lDisableOnInit))
            {
               ControlFlowOps.invokeIn("enableObject", targetProcedure());
            }
         }
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("LayoutVariable"));
         cLayout.assign(ghProp.unwrapBufferField().value());
         
         if (_isNotEqual(cLayout, ""))
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("ObjectLayout"));
            cLayout.assign(ghProp.unwrapBufferField().value());
            
            if (_isNotEqual(cLayout, ""))
            {
               ControlFlowOps.invokeIn("applyLayout", targetProcedure());
            }
            else
            {
               ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
               ghProp.assign(ghProp.unwrapBuffer().bufferField("defaultLayout"));
               cLayout.assign(ghProp.unwrapBufferField().value());
               
               if (_isNotEqual(cLayout, ""))
               {
                  ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
                  ghProp.assign(ghProp.unwrapBuffer().bufferField("ObjectLayout"));
                  ghProp.unwrapBufferField().changeValue(cLayout);
                  ControlFlowOps.invokeIn("applyLayout", targetProcedure());
               }
            }
         }
         
         /* END DO IF LayoutVariable */
         if (_isNotEqual(hContainer.unwrapType().getResourceType(), "window"))
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("HideOnInit"));
            lHideOnInit.assign(ghProp.unwrapBufferField().value());
            
            if (_not(lHideOnInit))
            {
               ControlFlowOps.invokeIn("viewObject", targetProcedure());
            }
            else
            {
               NamedEventManager.publish("LinkState", targetProcedure(), "I", new Object[]
               {
                  "inactive"
               });
            }
         }
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ContainerSource"));
         hContainerSource.assign(ghProp.unwrapBufferField().value());
         
         if (_not(hContainerSource.isValid()))
         {
            hContainerSource.assign(targetProcedure());
         }
         
         if (_isEqual(hContainer.unwrapType().getResourceType(), "window"))
         {
            hContainer.assign(hContainer.unwrapTree().firstChild());
         }
         
         if (_and(hContainer.isValid(), () -> gshSessionManager.isValid()))
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("PopupButtonsInFields"));
            lPopupsInFields.assign(ghProp.unwrapBufferField().value());
            
            if (_isUnknown(lPopupsInFields))
            {
               lPopupsInFields.assign(false);
            }
            
            /* Now do the widget walk */
            ControlFlowOps.invokeInWithMode("widgetWalk", gshSessionManager, "IIIII", hContainerSource, targetProcedure(), hContainer, new character("setup"), lPopupsInFields);
         }
         returnNormal();
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:  Defines 3D colors dynamically    <br>
    * Parameters:  &lt;none&gt;<br>
    * Notes:    These are class properties so this is run from the main block
    */
   public void initWinColors()
   {
      internalProcedure(new Block((Body) () -> 
      {
         giColor3Dface /* {fnarg <func-name> <arg> [handle] */ .assign(ControlFlowOps.invokeDynamicFunctionInWithMode(integer.class, "createDynamicColor", targetProcedure(), "I", "Color3DFace"));
         
         if (_isUnknown(giColor3Dface))
         {
            giColor3Dface.assign(8);
         }
         
         /* default to Grey */
         giColor3DhighLight /* {fnarg <func-name> <arg> [handle] */ .assign(ControlFlowOps.invokeDynamicFunctionInWithMode(integer.class, "createDynamicColor", targetProcedure(), "I", "Color3DHighLight"));
         
         if (_isUnknown(giColor3DhighLight))
         {
            giColor3DhighLight.assign(15);
         }
         
         /* default to White */
         giColor3Dshadow /* {fnarg <func-name> <arg> [handle] */ .assign(ControlFlowOps.invokeDynamicFunctionInWithMode(integer.class, "createDynamicColor", targetProcedure(), "I", "Color3DShadow"));
         
         if (_isUnknown(giColor3Dshadow))
         {
            giColor3Dshadow.assign(7);
         }
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     Locates a widget and retuns its handle and the handle of its<br>
    *              TARGET-PROCEDURE<br>
    * Parameters:  pcWidget AS CHARACTER<br>
    *              phWidget AS HANDLE<br>
    *              phTarget AS HANDLE<br>
    * Notes:       Support for locating widgets by instance name and
    */
   public void locateWidget(final character _pcWidget, final handle phWidget, final handle phTarget)
   {
      character pcWidget = TypeFactory.initInput(_pcWidget);
      character cDataObjectType = TypeFactory.character();
      character cDataSourceNames = TypeFactory.character();
      character cDataTargets = TypeFactory.character();
      character cDisplayedFields = TypeFactory.character();
      character cField = TypeFactory.character();
      character cInstanceName = TypeFactory.character();
      character cNameWoinstance = TypeFactory.character();
      character cObjectName = TypeFactory.character();
      character cObjectType = TypeFactory.character();
      character cSdofield = TypeFactory.character();
      character cSdoqualifier = TypeFactory.character();
      character cTargets = TypeFactory.character();
      character cTargetObjectType = TypeFactory.character();
      character cUpdateSources = TypeFactory.character();
      character cUpdateTargetNames = TypeFactory.character();
      handle hContainer = TypeFactory.handle();
      handle hDataTarget = TypeFactory.handle();
      handle hTarget = TypeFactory.handle();
      handle hUpdateSource = TypeFactory.handle();
      integer iDataTarget = TypeFactory.integer();
      integer iNumObjects = TypeFactory.integer();
      integer iTarget = TypeFactory.integer();
      logical lBrowsed = TypeFactory.logical();
      logical lBrowseQualified = TypeFactory.logical();
      logical lQueryObject = TypeFactory.logical();
      logical lTargetQueryObject = TypeFactory.logical();
      
      internalProcedure(new Block((Init) () -> 
      {
         TypeFactory.initOutput(phWidget);
         TypeFactory.initOutput(phTarget);
      }, 
      (Body) () -> 
      {
         if (_isGreaterThan(numEntries(pcWidget, "."), 1))
         {
            cInstanceName.assign(entry(1, pcWidget, "."));
            cNameWoinstance.assign(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "deleteEntry", targetProcedure(), "III", 1, pcWidget, "."));
         }
         
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ObjectType"));
         cObjectType.assign(ghProp.unwrapBufferField().value());
         
         if (_isNotEqual(cObjectType, "SmartDataViewer"))
         {
            ErrorManager.silentErrorEnable();
            cTargets.assign(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "linkHandles", targetProcedure(), "I", "Container-Target"));
            ErrorManager.silentErrorDisable();
         }
         
         if (_or(isUnknown(cTargets), () -> isEqual(cTargets, "")))
         {
            ErrorManager.silentErrorEnable();
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("ContainerSource"));
            hContainer.assign(ghProp.unwrapBufferField().value());
            ErrorManager.silentErrorDisable();
            ErrorManager.silentErrorEnable();
            cTargets.assign(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "linkHandles", hContainer, "I", "Container-Target"));
            ErrorManager.silentErrorDisable();
         }
         
         IntegerExpression resExpr2 = new IntegerExpression()
         {
            public integer execute()
            {
               return numEntries(cTargets);
            }
         };
         ToClause toClause2 = new ToClause(iTarget, 1, resExpr2);
         
         doTo("loopLabel5", toClause2, new Block((Body) () -> 
         {
            hTarget.assign(handle.fromString(entry(iTarget, cTargets)));
            ghProp.assign(handle.fromString(entry(1, hTarget.unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("ObjectName"));
            cObjectName.assign(ghProp.unwrapBufferField().value());
            
            if (_isEqual(cObjectName, cInstanceName))
            {
               ghProp.assign(handle.fromString(entry(1, hTarget.unwrapADMData().getADMData(), chr(1))));
               ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryObject"));
               lQueryObject.assign(ghProp.unwrapBufferField().value());
               
               if ((lQueryObject).booleanValue())
               {
                  ghProp.assign(handle.fromString(entry(1, hTarget.unwrapADMData().getADMData(), chr(1))));
                  ghProp.assign(ghProp.unwrapBuffer().bufferField("ObjectType"));
                  cDataObjectType.assign(ghProp.unwrapBufferField().value());
                  ghProp.assign(handle.fromString(entry(1, hTarget.unwrapADMData().getADMData(), chr(1))));
                  ghProp.assign(ghProp.unwrapBuffer().bufferField("DataTarget"));
                  cDataTargets.assign(ghProp.unwrapBufferField().value());
                  /* {get Prop var [handle]} */
                  lBrowsed.assign(ControlFlowOps.invokeDynamicFunctionIn(logical.class, "getDataQueryBrowsed", hTarget));
                  
                  if (_and(isGreaterThan(numEntries(cNameWoinstance, "."), 1), () -> isEqual(entry(1, cNameWoinstance, "."), "Browse")))
                  {
                     lBrowseQualified.assign(true);
                     cField.assign(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "deleteEntry", targetProcedure(), "III", 1, cNameWoinstance, "."));
                  }
                  else
                  {
                     cField.assign(cNameWoinstance);
                  }
                  
                  if (_and(lBrowseQualified, not(lBrowsed)))
                  {
                     returnNormal();
                  }
                  
                  if (_isEqual(cDataObjectType, "SmartDataObject"))
                  {
                     /* If not qualfied with "Browse", then attempt to locate widget in 
                                  the SDO's update source. */
                     if (_not(lBrowseQualified))
                     {
                        /* {get Prop var [handle]} */
                        hUpdateSource.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getUpdateSource", hTarget));
                        ErrorManager.silentErrorEnable();
                        phWidget.assign(ControlFlowOps.invokeDynamicFunctionInWithMode(handle.class, "internalWidgetHandle", hUpdateSource, "II", cField, "ALL"));
                        phTarget.assign(hUpdateSource);
                        ErrorManager.silentErrorDisable();
                        
                        if (_notUnknown(phWidget))
                        {
                           returnNormal();
                        }
                     }
                     
                     IntegerExpression resExpr3 = new IntegerExpression()
                     {
                        public integer execute()
                        {
                           return numEntries(cDataTargets);
                        }
                     };
                     ToClause toClause3 = new ToClause(iDataTarget, 1, resExpr3);
                     
                     doTo("loopLabel6", toClause3, new Block((Body) () -> 
                     {
                        ErrorManager.silentErrorEnable();
                        hDataTarget.assign(handle.fromString(entry(iDataTarget, cDataTargets)));
                        ErrorManager.silentErrorDisable();
                        ghProp.assign(handle.fromString(entry(1, hDataTarget.unwrapADMData().getADMData(), chr(1))));
                        ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryObject"));
                        lTargetQueryObject.assign(ghProp.unwrapBufferField().value());
                        ghProp.assign(handle.fromString(entry(1, hDataTarget.unwrapADMData().getADMData(), chr(1))));
                        ghProp.assign(ghProp.unwrapBuffer().bufferField("ObjectType"));
                        cTargetObjectType.assign(ghProp.unwrapBufferField().value());
                        
                        if (_and(and(hDataTarget.isValid(), () -> isNotEqual(hDataTarget, hTarget)), not(lTargetQueryObject)))
                        {
                           if (_or(and(lBrowseQualified, () -> isEqual(cTargetObjectType, "SmartDataBrowser")), not(lBrowseQualified)))
                           {
                              ErrorManager.silentErrorEnable();
                              phWidget.assign(ControlFlowOps.invokeDynamicFunctionInWithMode(handle.class, "internalWidgetHandle", hDataTarget, "II", cField, "ALL"));
                              phTarget.assign(hDataTarget);
                              ErrorManager.silentErrorDisable();
                              
                              if (_notUnknown(phWidget))
                              {
                                 returnNormal();
                              }
                           }
                        }
                     }));
                  }
                  else
                  {
                     /* SBO */
                     if (_isGreaterThan(numEntries(cField, "."), 1))
                     {
                        cSdoqualifier.assign(entry(1, cField, "."));
                        cSdofield.assign(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "deleteEntry", targetProcedure(), "III", 1, cField, "."));
                     }
                     else
                     {
                        returnNormal();
                     }
                     
                     /* SDO qualification is required for SBOs */
                     /* If it is not qualified with "Browse", then it attempts to 
                                  locate the widget in the update sources of the SBO where the 
                                  update target name of those update sources matches the SDO qualifier. */
                     if (_not(lBrowseQualified))
                     {
                        /* {get Prop var [handle]} */
                        cUpdateSources.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getUpdateSource", hTarget));
                        
                        IntegerExpression resExpr4 = new IntegerExpression()
                        {
                           public integer execute()
                           {
                              return numEntries(cUpdateSources);
                           }
                        };
                        ToClause toClause4 = new ToClause(iNumObjects, 1, resExpr4);
                        
                        doTo("loopLabel7", toClause4, new Block((Body) () -> 
                        {
                           hUpdateSource.assign(handle.fromString(entry(iNumObjects, cUpdateSources)));
                           /* {get Prop var [handle]} */
                           cUpdateTargetNames.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getUpdateTargetNames", hUpdateSource));
                           
                           if (_isGreaterThan(lookup(cSdoqualifier, cUpdateTargetNames), 0))
                           {
                              /* If the fields are qualified with SDO name in the visual object 
                                                 then search for them with the qualified name (cField). */
                              /* {get Prop var [handle]} */
                              cDisplayedFields.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getDisplayedFields", hUpdateSource));
                              
                              if (_isGreaterThan(indexOf(entry(1, cDisplayedFields), "."), 0))
                              {
                                 cSdofield.assign(cField);
                              }
                              
                              ErrorManager.silentErrorEnable();
                              phWidget.assign(ControlFlowOps.invokeDynamicFunctionInWithMode(handle.class, "internalWidgetHandle", hUpdateSource, "II", cSdofield, "ALL"));
                              phTarget.assign(hUpdateSource);
                              ErrorManager.silentErrorDisable();
                              
                              if (_notUnknown(phWidget))
                              {
                                 returnNormal();
                              }
                           }
                        }));
                     }
                     ghProp.assign(handle.fromString(entry(1, hTarget.unwrapADMData().getADMData(), chr(1))));
                     ghProp.assign(ghProp.unwrapBuffer().bufferField("DataTarget"));
                     cDataTargets.assign(ghProp.unwrapBufferField().value());
                     
                     IntegerExpression resExpr5 = new IntegerExpression()
                     {
                        public integer execute()
                        {
                           return numEntries(cDataTargets);
                        }
                     };
                     ToClause toClause5 = new ToClause(iNumObjects, 1, resExpr5);
                     
                     doTo("loopLabel8", toClause5, new Block((Body) () -> 
                     {
                        hDataTarget.assign(handle.fromString(entry(iNumObjects, cDataTargets)));
                        ghProp.assign(handle.fromString(entry(1, hDataTarget.unwrapADMData().getADMData(), chr(1))));
                        ghProp.assign(ghProp.unwrapBuffer().bufferField("DataSourceNames"));
                        cDataSourceNames.assign(ghProp.unwrapBufferField().value());
                        ghProp.assign(handle.fromString(entry(1, hDataTarget.unwrapADMData().getADMData(), chr(1))));
                        ghProp.assign(ghProp.unwrapBuffer().bufferField("ObjectType"));
                        cTargetObjectType.assign(ghProp.unwrapBufferField().value());
                        
                        if (_and(isGreaterThan(lookup(cSdoqualifier, cDataSourceNames), 0), () -> or(and(lBrowseQualified, () -> isEqual(cTargetObjectType, "SmartDataBrowser")), not(lBrowseQualified))))
                        {
                           /* If the fields are qualified with SDO name in the visual object 
                                            then search for them with the qualified name (cField). */
                           /* {get Prop var [handle]} */
                           cDisplayedFields.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getDisplayedFields", hDataTarget));
                           
                           if (_isGreaterThan(indexOf(entry(1, cDisplayedFields), "."), 0))
                           {
                              cSdofield.assign(cField);
                           }
                           
                           ErrorManager.silentErrorEnable();
                           phWidget.assign(ControlFlowOps.invokeDynamicFunctionInWithMode(handle.class, "internalWidgetHandle", hDataTarget, "II", cSdofield, "ALL"));
                           phTarget.assign(hDataTarget);
                           ErrorManager.silentErrorDisable();
                           
                           if (_notUnknown(phWidget))
                           {
                              returnNormal();
                           }
                        }
                     }));
                  }
               }
               else
               {
                  ErrorManager.silentErrorEnable();
                  phWidget.assign(ControlFlowOps.invokeDynamicFunctionInWithMode(handle.class, "internalWidgetHandle", hTarget, "II", cNameWoinstance, "ALL"));
                  phTarget.assign(hTarget);
                  ErrorManager.silentErrorDisable();
                  
                  if (_notUnknown(phWidget))
                  {
                     returnNormal();
                  }
               }
            }
         }));
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     Generic procedure to process keyboard and mouse actions for<br>
    *              visual objects.<br>
    * Parameters:  input name of action<br>
    * Notes:       Typically a generic event will be put in visualcustom.i that<br>
    *              will run this procedure in order to do the work. It can not do the<br>
    *              work nornmally itself as container handles, etc. are not valid<br>
    *              yet. Also this keeps the code size down.
    */
   public void processAction(final character _pcAction)
   {
      character pcAction = TypeFactory.initInput(_pcAction);
      handle hContainerSource = TypeFactory.handle();
      
      internalProcedure(new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("containersource"));
         hContainerSource.assign(ghProp.unwrapBufferField().value());
         
         if (_not(hContainerSource.isValid()))
         {
            hContainerSource.assign(targetProcedure());
         }
         
         switch (StringHelper.safeTrimTrailing((pcAction).toStringMessage()).toUpperCase())
         {
            case "CTRL-PAGE-UP":
               {
                  NamedEventManager.publish("selectPrevTab", hContainerSource);
               }
               break;
            case "CTRL-PAGE-DOWN":
               {
                  NamedEventManager.publish("selectNextTab", hContainerSource);
               }
               break;
         }
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     Procedure called when a UI event is fired. This procedure will<br>
    *              process the request according to the relevant parameters.<br>
    * Parameters:  pcActionType     - RUN/PUBLISH<br>
    *              pcEventAction    - the name of the procedure to run. This must be a <br>
    *                                 procedure - not a function.<br>
    *              pcActionTarget   - SELF,CONTAINER,ANYWHERE, ManagerCode<br>
    *              pcEventParameter - a parameter to pass into the event procedure.<br>
    * Notes:      Although this API should be used wiht a Dynamcis Repository <br>
    *                connected, it may be possible to use without one.
    */
   public void processEventProcedure(final character _pcActionType, final character _pcEventAction, final character _pcActionTarget, final character _pcEventParameter)
   {
      character pcActionType = TypeFactory.initInput(_pcActionType);
      character pcEventAction = TypeFactory.initInput(_pcEventAction);
      character pcActionTarget = TypeFactory.initInput(_pcActionTarget);
      character pcEventParameter = TypeFactory.initInput(_pcEventParameter);
      handle hActionTarget = TypeFactory.handle();
      
      internalProcedure(new Block((Body) () -> 
      {
         switch (StringHelper.safeTrimTrailing((pcActionTarget).toStringMessage()).toUpperCase())
         {
            case "SELF":
               {
                  hActionTarget.assign(targetProcedure());
               }
               break;
            case "CONTAINER":
               {
                  ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
                  ghProp.assign(ghProp.unwrapBuffer().bufferField("ContainerSource"));
                  hActionTarget.assign(ghProp.unwrapBufferField().value());
               }
               break;
 /* {get Prop var [handle]} */             case "ANYWHERE":
               {
                  hActionTarget.setUnknown();
               }
               break;
            case "AS":
               {
                  hActionTarget.assign(gshAstraAppserver);
               }
               break;
            case "GM":
               {
                  hActionTarget.assign(gshGenManager);
               }
               break;
            case "SM":
               {
                  hActionTarget.assign(gshSessionManager);
               }
               break;
            case "SEM":
               {
                  hActionTarget.assign(gshSecurityManager);
               }
               break;
            case "PM":
               {
                  hActionTarget.assign(gshProfileManager);
               }
               break;
            case "RM":
               {
                  hActionTarget.assign(gshRepositoryManager);
               }
               break;
            case "TM":
               {
                  hActionTarget.assign(gshTranslationManager);
               }
               break;
            default:
               {
                  ErrorManager.silentErrorEnable();
                  hActionTarget.assign(ControlFlowOps.invokeDynamicFunctionWithMode("getManagerHandle", "I", pcActionTarget));
                  ErrorManager.silentErrorDisable();
               }
               break;
         }
         
         /* action target */
         if ((hActionTarget.isValid()).booleanValue())
         {
            if (_isEqual(pcActionType, "RUN"))
            {
               if (_isEqual(pcEventParameter, ""))
               {
                  if (_isEqual(pcActionTarget, "AS"))
                  {
                     ErrorManager.silentErrorEnable();
                     ControlFlowOps.invokeRemote((pcEventAction).toStringMessage(), (handle) null, hActionTarget, false);
                     ErrorManager.silentErrorDisable();
                  }
                  else
                  {
                     ErrorManager.silentErrorEnable();
                     ControlFlowOps.invokeIn((pcEventAction).toStringMessage(), hActionTarget);
                     ErrorManager.silentErrorDisable();
                  }
               }
               else
               {
                  if (_isEqual(pcActionTarget, "AS"))
                  {
                     ErrorManager.silentErrorEnable();
                     ControlFlowOps.invokeRemoteWithMode((pcEventAction).toStringMessage(), (handle) null, hActionTarget, false, "I", pcEventParameter);
                     ErrorManager.silentErrorDisable();
                  }
                  else
                  {
                     ErrorManager.silentErrorEnable();
                     ControlFlowOps.invokeInWithMode((pcEventAction).toStringMessage(), hActionTarget, "I", pcEventParameter);
                     ErrorManager.silentErrorDisable();
                  }
               }
            }
            else
            {
               if (_isEqual(pcActionTarget, "ANYWHERE"))
               {
                  if (_isEqual(pcEventParameter, ""))
                  {
                     NamedEventManager.publish(pcEventAction, thisProcedure());
                  }
                  else
                  {
                     NamedEventManager.publish(pcEventAction, thisProcedure(), "I", new Object[]
                     {
                        pcEventParameter
                     });
                  }
               }
               else
               {
                  if ((hActionTarget.isValid()).booleanValue())
                  {
                     if (_isEqual(pcEventParameter, ""))
                     {
                        NamedEventManager.publish(pcEventAction, hActionTarget);
                     }
                     else
                     {
                        NamedEventManager.publish(pcEventAction, hActionTarget, "I", new Object[]
                        {
                           pcEventParameter
                        });
                     }
                  }
               }
            }
         }
         ErrorManager.setError(new logical(false));
         returnNormal();
      }));
   }

   /** processEventProcedure */
   /**
    * ***********************  Function Implementations****************
    */
   public logical assignFocusedWidget(final character _pcName_8)
   {
      character pcName_8 = TypeFactory.initInput(_pcName_8);
      handle hField = TypeFactory.handle();
      handle hTarget = TypeFactory.handle();
      logical lInvalid = TypeFactory.logical();
      
      return function("assignFocusedWidget", logical.class, new Block((Body) () -> 
      {
         ControlFlowOps.invokeInWithMode("locateWidget", targetProcedure(), "IOO", pcName_8, hField, hTarget);
         
         if ((hField.isValid()).booleanValue())
         {
            if ((HandleOps.canQuery(hField, "FILE-NAME")).booleanValue())
            {
               lInvalid.assign(true);
            }
            else
            {
               apply(hField, "ENTRY");
            }
         }
         else
         {
            lInvalid.assign(true);
         }
         /* Return FALSE if field was invalid or not sensitive. */
         returnNormal(not(lInvalid));
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical assignWidgetValue(final character _pcName_8, final character _pcValue_4)
   {
      character pcName_8 = TypeFactory.initInput(_pcName_8);
      character pcValue_4 = TypeFactory.initInput(_pcValue_4);
      handle hField = TypeFactory.handle();
      handle hTarget = TypeFactory.handle();
      logical lInvalid = TypeFactory.logical();
      
      return function("assignWidgetValue", logical.class, new Block((Body) () -> 
      {
         ControlFlowOps.invokeInWithMode("locateWidget", targetProcedure(), "IOO", pcName_8, hField, hTarget);
         
         if ((hField.isValid()).booleanValue())
         {
            /* This is a SmartDataField. Use its setDataValue method. */
            if ((HandleOps.canQuery(hField, "FILE-NAME")).booleanValue())
            {
               /* Workaround for Issue 9121 - setting the DataValue for a lookup who's 
                        displayed field is not the same as its key field does not refresh
                        the displayed field.  assignNewValue is used instead because this
                        does refresh the displayed field but it is specific to lookups only.
                        When Issue 9121 is fixed this code should be removed so that only
                        set DataValue is done for all SmartDataFields.  */
               if (_isNotEqual( /* {fnarg <func-name> <arg> [handle] */ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "Signature", hField, "I", "initializeLookup"), ""))
               {
                  ErrorManager.silentErrorEnable();
                  ControlFlowOps.invokeInWithMode("assignNewValue", hField, "III", pcValue_4, new character(""), new logical(true));
                  ErrorManager.silentErrorDisable();
                  
                  if ((ErrorManager.isError()).booleanValue())
                  {
                     lInvalid.assign(true);
                  }
               }
               else
               {
                  ErrorManager.silentErrorEnable();
                  /* {set Prop value [handle]} */
                  ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "setDataValue", hField, "I", pcValue_4);
                  ErrorManager.silentErrorDisable();
                  
                  if ((ErrorManager.isError()).booleanValue())
                  {
                     lInvalid.assign(true);
                  }
                  else
                  {
                     ErrorManager.silentErrorEnable();
                     /* {set Prop value [handle]} */
                     ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "setDataModified", hField, "I", true);
                     ErrorManager.silentErrorDisable();
                  }
               }
            }
            else
            {
               if ((HandleOps.canSet(hField, "SCREEN-VALUE")).booleanValue())
               {
                  ErrorManager.silentErrorEnable();
                  hField.unwrapWidget().setScreenValue(pcValue_4);
                  ErrorManager.silentErrorDisable();
                  
                  if ((ErrorManager.isError()).booleanValue())
                  {
                     lInvalid.assign(true);
                  }
                  else
                  {
                     ErrorManager.silentErrorEnable();
                     ControlFlowOps.invokeInWithMode("fieldModified", targetProcedure(), "I", hField);
                     ErrorManager.silentErrorDisable();
                  }
               }
               else
               {
                  lInvalid.assign(true);
               }
            }
         }
         else
         {
            lInvalid.assign(true);
         }
         returnNormal(not(lInvalid));
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical assignWidgetValueList(final character _pcNameList_16, final character _pcValueList_1, final character _pcDelimiter_3)
   {
      character pcNameList_16 = TypeFactory.initInput(_pcNameList_16);
      character pcValueList_1 = TypeFactory.initInput(_pcValueList_1);
      character pcDelimiter_3 = TypeFactory.initInput(_pcDelimiter_3);
      /*------------------------------------------------------------------------------
        Purpose:  Sets the SCREEN-VALUE of objects in a delimited list.
         Params:  INPUT pcNameList  AS CHARACTER
                  INPUT pcValueList AS CHARACTER
                  INPUT pcDelimiter AS CHARACTER
          Notes:  If pcDelimiter is ?, | is used
      ------------------------------------------------------------------------------*/
      character cField = TypeFactory.character();
      character cQualifier = TypeFactory.character();
      character cValue = TypeFactory.character();
      integer iField = TypeFactory.integer();
      logical lAssigned = TypeFactory.logical();
      logical lInvalid = TypeFactory.logical();
      
      return function("assignWidgetValueList", logical.class, new Block((Body) () -> 
      {
         if (_isUnknown(pcDelimiter_3))
         {
            pcDelimiter_3.assign("|");
         }
         
         /* If there is more than one value and the number of values do not 
              match the number of names, false is returned and no assignments are done */
         if (_and(isGreaterThan(numEntries(pcValueList_1, pcDelimiter_3), 1), () -> isNotEqual(numEntries(pcValueList_1, pcDelimiter_3), numEntries(pcNameList_16))))
         {
            returnNormal(false);
         }
         
         /* If the first name in the list is qualified that is stored so that it can be used
              as the qualifier for following non-qualified names in the list.  */
         if (_and(isGreaterThan(numEntries(pcNameList_16), 1), () -> isGreaterThan(numEntries(entry(1, pcNameList_16), "."), 1)))
         {
            cQualifier.assign(substring(entry(1, pcNameList_16), 1, lastIndexOf(entry(1, pcNameList_16), ".")));
         }
         
         IntegerExpression resExpr6 = new IntegerExpression()
         {
            public integer execute()
            {
               return numEntries(pcNameList_16);
            }
         };
         ToClause toClause6 = new ToClause(iField, 1, resExpr6);
         
         doTo("loopLabel9", toClause6, new Block((Body) () -> 
         {
            cField.assign(entry(iField, pcNameList_16));
            
            if (_and(isEqual(numEntries(cField, "."), 1), () -> isNotEqual(cQualifier, "")))
            {
               cField.assign(concat(cQualifier, cField));
            }
            
            cValue.assign(_isGreaterThanOrEqual(numEntries(pcValueList_1, pcDelimiter_3), iField) ? entry(iField, pcValueList_1, pcDelimiter_3) : entry(1, pcValueList_1, pcDelimiter_3));
            lAssigned.assign(ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "assignWidgetValue", targetProcedure(), "II", cField, cValue));
            
            /* If an assignment failed processing of assignments will continue but false
                   will be returned. */
            if (_not(lAssigned))
            {
               lInvalid.assign(true);
            }
         }));
         
         /* do iField to number entries in namelist */
         returnNormal(not(lInvalid));
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical blankWidget(final character _pcNameList_16)
   {
      character pcNameList_16 = TypeFactory.initInput(_pcNameList_16);
      /*------------------------------------------------------------------------------
        Purpose:  Blanks the SCREEN-VALUE of the object or objects in the NameList.
         Params:  INPUT pcNameList AS CHARACTER
          Notes:  
      ------------------------------------------------------------------------------*/
      integer iField = TypeFactory.integer();
      character cField = TypeFactory.character();
      character cQualifier = TypeFactory.character();
      handle hField = TypeFactory.handle();
      handle hTarget = TypeFactory.handle();
      logical lInvalid = TypeFactory.logical();
      
      return function("blankWidget", logical.class, new Block((Body) () -> 
      {
         /* If the first name in the list is qualified that is stored so that it can be used
              as the qualifier for following non-qualified names in the list.  */
         if (_and(isGreaterThan(numEntries(pcNameList_16), 1), () -> isGreaterThan(numEntries(entry(1, pcNameList_16), "."), 1)))
         {
            cQualifier.assign(substring(entry(1, pcNameList_16), 1, lastIndexOf(entry(1, pcNameList_16), ".")));
         }
         
         IntegerExpression resExpr7 = new IntegerExpression()
         {
            public integer execute()
            {
               return numEntries(pcNameList_16);
            }
         };
         ToClause toClause7 = new ToClause(iField, 1, resExpr7);
         
         doTo("loopLabel10", toClause7, new Block((Body) () -> 
         {
            cField.assign(entry(iField, pcNameList_16));
            
            if (_and(isEqual(numEntries(cField, "."), 1), () -> isNotEqual(cQualifier, "")))
            {
               cField.assign(concat(cQualifier, cField));
            }
            
            ControlFlowOps.invokeInWithMode("locateWidget", targetProcedure(), "IOO", cField, hField, hTarget);
            
            if ((hField.isValid()).booleanValue())
            {
               /* This is a SmartDataField. Use its setDataValue method. */
               if ((HandleOps.canQuery(hField, "FILE-NAME")).booleanValue())
               {
                  /* Test for lookup */
                  if (_isNotEqual( /* {fnarg <func-name> <arg> [handle] */ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "Signature", hField, "I", "initializeLookup"), ""))
                  {
                     ControlFlowOps.invokeInWithMode("assignNewValue", hField, "III", new character(""), new character(""), new logical(true));
                     
                     if ((ErrorManager.isError()).booleanValue())
                     {
                        lInvalid.assign(true);
                     }
                  }
                  else
                  {
                     ErrorManager.silentErrorEnable();
                     /* {set Prop value [handle]} */
                     ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "setDataValue", hField, "I", "");
                     ErrorManager.silentErrorDisable();
                     
                     if ((ErrorManager.isError()).booleanValue())
                     {
                        lInvalid.assign(true);
                     }
                     else
                     {
                        ErrorManager.silentErrorEnable();
                        /* {set Prop value [handle]} */
                        ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "setDataModified", hField, "I", true);
                        ErrorManager.silentErrorDisable();
                     }
                  }
               }
               else
               {
                  if (_and(HandleOps.canQuery(hField, "TYPE"), () -> isEqual(hField.unwrapType().getResourceType(), "COMBO-BOX")))
                  {
                     if ((HandleOps.canQuery(hField, "LIST-ITEMS")).booleanValue())
                     {
                        ErrorManager.silentErrorEnable();
                        hField.unwrapCommonListWidget().setListItems(hField.unwrapCommonListWidget().getListItems());
                        ErrorManager.silentErrorDisable();
                        
                        if ((ErrorManager.isError()).booleanValue())
                        {
                           lInvalid.assign(true);
                        }
                     }
                     else
                     {
                        ErrorManager.silentErrorEnable();
                        hField.unwrapCommonListWidget().setListItemPairs(hField.unwrapCommonListWidget().getListItemPairs());
                        ErrorManager.silentErrorDisable();
                        
                        if ((ErrorManager.isError()).booleanValue())
                        {
                           lInvalid.assign(true);
                        }
                     }
                  }
                  else
                  {
                     if ((HandleOps.canSet(hField, "SCREEN-VALUE")).booleanValue())
                     {
                        ErrorManager.silentErrorEnable();
                        hField.unwrapWidget().setScreenValue(new character(""));
                        ErrorManager.silentErrorDisable();
                        
                        /* Assigning the SCREEN-VALUE to blank for certain widgets (e.g. toggle-boxes) 
                                     raises an error but the error status is not true.  */
                        if (_or(ErrorManager.isError(), () -> isGreaterThan(ErrorManager.numErrors(), 0)))
                        {
                           lInvalid.assign(true);
                        }
                     }
                     else
                     {
                        lInvalid.assign(true);
                     }
                  }
                  
                  /* SetDataModified if applicable (no-error as this is in datavis) */
                  if (_not(lInvalid))
                  {
                     ErrorManager.silentErrorEnable();
                     ControlFlowOps.invokeInWithMode("fieldModified", targetProcedure(), "I", hField);
                     ErrorManager.silentErrorDisable();
                  }
               }
            }
            else
            {
               lInvalid.assign(true);
            }
         }));
         
         /* END DO iField */
         /* Return FALSE if any field was invalid. */
         returnNormal(not(lInvalid));
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public integer createDynamicColor(final character _pcName_8)
   {
      character pcName_8 = TypeFactory.initInput(_pcName_8);
      /*------------------------------------------------------------------------------
        Purpose: Creates a Progress 3D color in the color-table and returns its 
                 number.
      Parameter: pcName - colorname to add to the ADM section in .ini                         
          Notes: Returns unknown if the color could not be created.
               - The color is added to the color-table ans the .ini file if it 
                 doesn't exist, menaing that this code is the originator of the 
                 entry in the .ini.
               - It is also possible to manually add or change the entries in 
                 the .ini to point to some other color. This color will be made 
                 dynamic unless it is one of the 16 (0 - 15) reserved colors.   
               - See initWinColors for usage  
      ------------------------------------------------------------------------------*/
      character cValue = TypeFactory.character();
      integer iColor = TypeFactory.integer();
      
      return function("createDynamicColor", integer.class, new Block((Body) () -> 
      {
         cValue.assign(EnvironmentOps.getKeyValue("ADM", pcName_8));
         iColor.assign(new integer(cValue));
         
         if (_isGreaterThan(iColor, 255))
         {
            returnNormal(new integer());
         }
         
         /* If not defined or not enough entries in the color-table then add it */
         if (_or(isUnknown(iColor), () -> isLessThan(ColorTable.getNumEntries(), plus(iColor, 1))))
         {
            /* If undefined; get new number and add it to the ini file. */
            if (_isUnknown(iColor))
            {
               iColor.assign(ColorTable.getNumEntries());
               
               /* only put to 'adm' section if within max entries */
               if (_isLessThan(iColor, 256))
               {
                  EnvironmentOps.setKeyValue("ADM", pcName_8, (valueOf(iColor)).toStringMessage());
               }
               else
               {
                  iColor.setUnknown();
               }
            }
            
            /* add an entry to the color table */
            if (_notUnknown(iColor))
            {
               ColorTable.setNumEntries(new integer(plus(iColor, 1)));
            }
         }
         
         /* Don't mess with the default ADE colors */
         if (_isGreaterThan(iColor, 15))
         {
            ColorTable.setDynamic(iColor, true);
         }
         
         returnNormal(iColor);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical createUiEvents()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Creates custom events for the widget. 
          Notes:  * A Dynamics Repository is required for this API.
      ------------------------------------------------------------------------------*/
      handle hObjectBuffer = TypeFactory.handle();
      handle hWidget = TypeFactory.handle();
      character cEventName = TypeFactory.character();
      character cEventNames = TypeFactory.character();
      character cEventValues = TypeFactory.character();
      logical lUseRepository = TypeFactory.logical();
      decimal dInstanceId = TypeFactory.decimal();
      integer iLoop = TypeFactory.integer();
      
      return function("createUiEvents", logical.class, new Block((Body) () -> 
      {
         /* This API only works with a connected Dynamics Repository. */
         /* {get Prop var [handle]} */ 
         lUseRepository.assign(ControlFlowOps.invokeDynamicFunctionIn(logical.class, "getuseRepository", targetProcedure()));
         
         if (_and(isEqual(lUseRepository, true), () -> gshRepositoryManager.isValid()))
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("InstanceId"));
            dInstanceId.assign(ghProp.unwrapBufferField().value());
            
            /* Get the default frame, window or container handle depending on the object type.
                     * In the case of a container window, the getContainerHandle() call will return
                     * the window handle. In the case of a viewer, it will return the frame's handle. */
            if ((new logical( /* {fnarg <func-name> <arg> [handle] */ ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "instanceOf", targetProcedure(), "I", "Browser"))).booleanValue())
            {
               /* {get Prop var [handle]} */ 
               hWidget.assign(ControlFlowOps.invokeDynamicFunctionIn(handle.class, "getBrowseHandle", targetProcedure()));
            }
            else
            {
               ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
               ghProp.assign(ghProp.unwrapBuffer().bufferField("ContainerHandle"));
               hWidget.assign(ghProp.unwrapBufferField().value());
            }
            /* {get Prop var [handle]} */ 
            
            hObjectBuffer.assign(ControlFlowOps.invokeDynamicFunctionInWithMode("getCacheObjectBuffer", gshRepositoryManager, "I", dInstanceId));
            
            if ((hObjectBuffer.unwrapBuffer().available()).booleanValue())
            {
               cEventNames.assign(hObjectBuffer.unwrapBuffer().bufferField("EventNames").unwrapBufferField().value());
               cEventValues.assign(hObjectBuffer.unwrapBuffer().bufferField("EventActions").unwrapBufferField().value());
            }
            
            loopLabel11:
            for (iLoop.assign(0); _isLessThanOrEqual(iLoop, minus(numEntries(cEventNames), 1)); iLoop.increment())
            {
               cEventName.assign(entry(plus(iLoop, 1), cEventNames));
               
               /* Make sure that this is a valid event for the widget */
               if ((hWidget.isValidEvent(cEventName)).booleanValue())
               {
                  switch (StringHelper.safeTrimTrailing((cEventName).toStringMessage()).toUpperCase())
                  {
                     case "ANY-KEY":
                        {
                           registerTriggerPersistentIn(new EventList("ANY-KEY", hWidget), "processEventProcedure", targetProcedure(), entry(plus(multiply(4, iLoop), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 2), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 3), cEventValues, chr(1)));
                        }
                        break;
                     case "ANY-PRINTABLE":
                        {
                           registerTriggerPersistentIn(new EventList("ANY-PRINTABLE", hWidget), "processEventProcedure", targetProcedure(), entry(plus(multiply(4, iLoop), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 2), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 3), cEventValues, chr(1)));
                        }
                        break;
                     case "BACK-TAB":
                        {
                           registerTriggerPersistentIn(new EventList("BACK-TAB", hWidget), "processEventProcedure", targetProcedure(), entry(plus(multiply(4, iLoop), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 2), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 3), cEventValues, chr(1)));
                        }
                        break;
                     case "BACKSPACE":
                        {
                           registerTriggerPersistentIn(new EventList("BACKSPACE", hWidget), "processEventProcedure", targetProcedure(), entry(plus(multiply(4, iLoop), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 2), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 3), cEventValues, chr(1)));
                        }
                        break;
                     case "BELL":
                        {
                           registerTriggerPersistentIn(new EventList("BELL", hWidget), "processEventProcedure", targetProcedure(), entry(plus(multiply(4, iLoop), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 2), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 3), cEventValues, chr(1)));
                        }
                        break;
                     case "CHOOSE":
                        {
                           registerTriggerPersistentIn(new EventList("CHOOSE", hWidget), "processEventProcedure", targetProcedure(), entry(plus(multiply(4, iLoop), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 2), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 3), cEventValues, chr(1)));
                        }
                        break;
                     case "CLEAR":
                        {
                           registerTriggerPersistentIn(new EventList("CLEAR", hWidget), "processEventProcedure", targetProcedure(), entry(plus(multiply(4, iLoop), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 2), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 3), cEventValues, chr(1)));
                        }
                        break;
                     case "DEFAULT-ACTION":
                        {
                           registerTriggerPersistentIn(new EventList("DEFAULT-ACTION", hWidget), "processEventProcedure", targetProcedure(), entry(plus(multiply(4, iLoop), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 2), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 3), cEventValues, chr(1)));
                        }
                        break;
                     case "DEL":
                        {
                           registerTriggerPersistentIn(new EventList("DEL", hWidget), "processEventProcedure", targetProcedure(), entry(plus(multiply(4, iLoop), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 2), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 3), cEventValues, chr(1)));
                        }
                        break;
                     case "DELETE-CHAR":
                        {
                           registerTriggerPersistentIn(new EventList("DELETE-CHAR", hWidget), "processEventProcedure", targetProcedure(), entry(plus(multiply(4, iLoop), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 2), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 3), cEventValues, chr(1)));
                        }
                        break;
                     case "DELETE-CHARACTER":
                        {
                           registerTriggerPersistentIn(new EventList("DELETE-CHARACTER", hWidget), "processEventProcedure", targetProcedure(), entry(plus(multiply(4, iLoop), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 2), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 3), cEventValues, chr(1)));
                        }
                        break;
                     case "DESELECT":
                        {
                           registerTriggerPersistentIn(new EventList("DESELECT", hWidget), "processEventProcedure", targetProcedure(), entry(plus(multiply(4, iLoop), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 2), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 3), cEventValues, chr(1)));
                        }
                        break;
                     case "DESELECTION":
                        {
                           registerTriggerPersistentIn(new EventList("DESELECTION", hWidget), "processEventProcedure", targetProcedure(), entry(plus(multiply(4, iLoop), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 2), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 3), cEventValues, chr(1)));
                        }
                        break;
                     case "DROP-FILE-NOTIFY":
                        {
                           registerTriggerPersistentIn(new EventList("DROP-FILE-NOTIFY", hWidget), "processEventProcedure", targetProcedure(), entry(plus(multiply(4, iLoop), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 2), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 3), cEventValues, chr(1)));
                        }
                        break;
                     case "EMPTY-SELECTION":
                        {
                           registerTriggerPersistentIn(new EventList("EMPTY-SELECTION", hWidget), "processEventProcedure", targetProcedure(), entry(plus(multiply(4, iLoop), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 2), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 3), cEventValues, chr(1)));
                        }
                        break;
                     case "END-BOX-SELECTION":
                        {
                           registerTriggerPersistentIn(new EventList("END-BOX-SELECTION", hWidget), "processEventProcedure", targetProcedure(), entry(plus(multiply(4, iLoop), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 2), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 3), cEventValues, chr(1)));
                        }
                        break;
                     case "END-ERROR":
                        {
                           registerTriggerPersistentIn(new EventList("END-ERROR", hWidget), "processEventProcedure", targetProcedure(), entry(plus(multiply(4, iLoop), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 2), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 3), cEventValues, chr(1)));
                        }
                        break;
                     case "END-MOVE":
                        {
                           registerTriggerPersistentIn(new EventList("END-MOVE", hWidget), "processEventProcedure", targetProcedure(), entry(plus(multiply(4, iLoop), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 2), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 3), cEventValues, chr(1)));
                        }
                        break;
                     case "END-RESIZE":
                        {
                           registerTriggerPersistentIn(new EventList("END-RESIZE", hWidget), "processEventProcedure", targetProcedure(), entry(plus(multiply(4, iLoop), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 2), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 3), cEventValues, chr(1)));
                        }
                        break;
                     case "ENDKEY":
                        {
                           registerTriggerPersistentIn(new EventList("ENDKEY", hWidget), "processEventProcedure", targetProcedure(), entry(plus(multiply(4, iLoop), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 2), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 3), cEventValues, chr(1)));
                        }
                        break;
                     case "ENTRY":
                        {
                           registerTriggerPersistentIn(new EventList("ENTRY", hWidget), "processEventProcedure", targetProcedure(), entry(plus(multiply(4, iLoop), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 2), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 3), cEventValues, chr(1)));
                        }
                        break;
                     case "ERROR":
                        {
                           registerTriggerPersistentIn(new EventList("ERROR", hWidget), "processEventProcedure", targetProcedure(), entry(plus(multiply(4, iLoop), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 2), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 3), cEventValues, chr(1)));
                        }
                        break;
                     case "GO":
                        {
                           registerTriggerPersistentIn(new EventList("GO", hWidget), "processEventProcedure", targetProcedure(), entry(plus(multiply(4, iLoop), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 2), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 3), cEventValues, chr(1)));
                        }
                        break;
                     case "HELP":
                        {
                           registerTriggerPersistentIn(new EventList("HELP", hWidget), "processEventProcedure", targetProcedure(), entry(plus(multiply(4, iLoop), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 2), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 3), cEventValues, chr(1)));
                        }
                        break;
                     case "LEAVE":
                        {
                           registerTriggerPersistentIn(new EventList("LEAVE", hWidget), "processEventProcedure", targetProcedure(), entry(plus(multiply(4, iLoop), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 2), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 3), cEventValues, chr(1)));
                        }
                        break;
                     case "MOUSE-MENU-CLICK":
                        {
                           registerTriggerPersistentIn(new EventList("MOUSE-MENU-CLICK", hWidget), "processEventProcedure", targetProcedure(), entry(plus(multiply(4, iLoop), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 2), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 3), cEventValues, chr(1)));
                        }
                        break;
                     case "MOUSE-SELECT-CLICK":
                        {
                           registerTriggerPersistentIn(new EventList("MOUSE-SELECT-CLICK", hWidget), "processEventProcedure", targetProcedure(), entry(plus(multiply(4, iLoop), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 2), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 3), cEventValues, chr(1)));
                        }
                        break;
                     case "MOUSE-SELECT-DBLCLICK":
                        {
                           registerTriggerPersistentIn(new EventList("MOUSE-SELECT-DBLCLICK", hWidget), "processEventProcedure", targetProcedure(), entry(plus(multiply(4, iLoop), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 2), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 3), cEventValues, chr(1)));
                        }
                        break;
                     case "PARENT-WINDOW-CLOSE":
                        {
                           registerTriggerPersistentIn(new EventList("PARENT-WINDOW-CLOSE", hWidget), "processEventProcedure", targetProcedure(), entry(plus(multiply(4, iLoop), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 2), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 3), cEventValues, chr(1)));
                        }
                        break;
                     case "RECALL":
                        {
                           registerTriggerPersistentIn(new EventList("RECALL", hWidget), "processEventProcedure", targetProcedure(), entry(plus(multiply(4, iLoop), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 2), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 3), cEventValues, chr(1)));
                        }
                        break;
                     case "RETURN":
                        {
                           registerTriggerPersistentIn(new EventList("RETURN", hWidget), "processEventProcedure", targetProcedure(), entry(plus(multiply(4, iLoop), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 2), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 3), cEventValues, chr(1)));
                        }
                        break;
                     case "SELECT":
                        {
                           registerTriggerPersistentIn(new EventList("SELECT", hWidget), "processEventProcedure", targetProcedure(), entry(plus(multiply(4, iLoop), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 2), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 3), cEventValues, chr(1)));
                        }
                        break;
                     case "SELECTION":
                        {
                           registerTriggerPersistentIn(new EventList("SELECTION", hWidget), "processEventProcedure", targetProcedure(), entry(plus(multiply(4, iLoop), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 2), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 3), cEventValues, chr(1)));
                        }
                        break;
                     case "START-BOX-SELECTION":
                        {
                           registerTriggerPersistentIn(new EventList("START-BOX-SELECTION", hWidget), "processEventProcedure", targetProcedure(), entry(plus(multiply(4, iLoop), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 2), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 3), cEventValues, chr(1)));
                        }
                        break;
                     case "START-MOVE":
                        {
                           registerTriggerPersistentIn(new EventList("START-MOVE", hWidget), "processEventProcedure", targetProcedure(), entry(plus(multiply(4, iLoop), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 2), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 3), cEventValues, chr(1)));
                        }
                        break;
                     case "START-RESIZE":
                        {
                           registerTriggerPersistentIn(new EventList("START-RESIZE", hWidget), "processEventProcedure", targetProcedure(), entry(plus(multiply(4, iLoop), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 2), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 3), cEventValues, chr(1)));
                        }
                        break;
                     case "TAB":
                        {
                           registerTriggerPersistentIn(new EventList("TAB", hWidget), "processEventProcedure", targetProcedure(), entry(plus(multiply(4, iLoop), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 2), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 3), cEventValues, chr(1)));
                        }
                        break;
                     case "VALUE-CHANGED":
                        {
                           registerTriggerPersistentIn(new EventList("VALUE-CHANGED", hWidget), "processEventProcedure", targetProcedure(), entry(plus(multiply(4, iLoop), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 2), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 3), cEventValues, chr(1)));
                        }
                        break;
                     case "WINDOW-CLOSE":
                        {
                           registerTriggerPersistentIn(new EventList("WINDOW-CLOSE", hWidget), "processEventProcedure", targetProcedure(), entry(plus(multiply(4, iLoop), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 2), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 3), cEventValues, chr(1)));
                        }
                        break;
                     case "WINDOW-MAXIMIZED":
                        {
                           registerTriggerPersistentIn(new EventList("WINDOW-MAXIMIZED", hWidget), "processEventProcedure", targetProcedure(), entry(plus(multiply(4, iLoop), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 2), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 3), cEventValues, chr(1)));
                        }
                        break;
                     case "WINDOW-MINIMIZED":
                        {
                           registerTriggerPersistentIn(new EventList("WINDOW-MINIMIZED", hWidget), "processEventProcedure", targetProcedure(), entry(plus(multiply(4, iLoop), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 2), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 3), cEventValues, chr(1)));
                        }
                        break;
                     case "WINDOW-RESIZED":
                        {
                           registerTriggerPersistentIn(new EventList("WINDOW-RESIZED", hWidget), "processEventProcedure", targetProcedure(), entry(plus(multiply(4, iLoop), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 2), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 3), cEventValues, chr(1)));
                        }
                        break;
                     case "WINDOW-RESTORED":
                        {
                           registerTriggerPersistentIn(new EventList("WINDOW-RESTORED", hWidget), "processEventProcedure", targetProcedure(), entry(plus(multiply(4, iLoop), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 2), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 3), cEventValues, chr(1)));
                        }
                        break;
                     case "U1":
                        {
                           registerTriggerPersistentIn(new EventList("U1", hWidget), "processEventProcedure", targetProcedure(), entry(plus(multiply(4, iLoop), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 2), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 3), cEventValues, chr(1)));
                        }
                        break;
                     case "U2":
                        {
                           registerTriggerPersistentIn(new EventList("U2", hWidget), "processEventProcedure", targetProcedure(), entry(plus(multiply(4, iLoop), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 2), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 3), cEventValues, chr(1)));
                        }
                        break;
                     case "U3":
                        {
                           registerTriggerPersistentIn(new EventList("U3", hWidget), "processEventProcedure", targetProcedure(), entry(plus(multiply(4, iLoop), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 2), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 3), cEventValues, chr(1)));
                        }
                        break;
                     case "U4":
                        {
                           registerTriggerPersistentIn(new EventList("U4", hWidget), "processEventProcedure", targetProcedure(), entry(plus(multiply(4, iLoop), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 2), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 3), cEventValues, chr(1)));
                        }
                        break;
                     case "U5":
                        {
                           registerTriggerPersistentIn(new EventList("U5", hWidget), "processEventProcedure", targetProcedure(), entry(plus(multiply(4, iLoop), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 2), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 3), cEventValues, chr(1)));
                        }
                        break;
                     case "U6":
                        {
                           registerTriggerPersistentIn(new EventList("U6", hWidget), "processEventProcedure", targetProcedure(), entry(plus(multiply(4, iLoop), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 2), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 3), cEventValues, chr(1)));
                        }
                        break;
                     case "U7":
                        {
                           registerTriggerPersistentIn(new EventList("U7", hWidget), "processEventProcedure", targetProcedure(), entry(plus(multiply(4, iLoop), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 2), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 3), cEventValues, chr(1)));
                        }
                        break;
                     case "U8":
                        {
                           registerTriggerPersistentIn(new EventList("U8", hWidget), "processEventProcedure", targetProcedure(), entry(plus(multiply(4, iLoop), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 2), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 3), cEventValues, chr(1)));
                        }
                        break;
                     case "U9":
                        {
                           registerTriggerPersistentIn(new EventList("U9", hWidget), "processEventProcedure", targetProcedure(), entry(plus(multiply(4, iLoop), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 2), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 3), cEventValues, chr(1)));
                        }
                        break;
                     case "U10":
                        {
                           registerTriggerPersistentIn(new EventList("U10", hWidget), "processEventProcedure", targetProcedure(), entry(plus(multiply(4, iLoop), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 2), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 1), cEventValues, chr(1)), entry(plus(plus(multiply(4, iLoop), 1), 3), cEventValues, chr(1)));
                        }
                        break;
                  }
               }
            }
            
            /* UI events. */
            /* Return true because the UI events, if any, have been added. 
                     * Even if there are no UI events, still return true because
                     * the attempt to add UI events succeeded.                     */
            returnNormal(true);
         }
         /* valid repository manager */
         /* If we get here, it means that the UI events have not been added from the Repository. */
         returnNormal(false);
      }));
   }

   /** createUiEvents */
   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical destroyPopups()
   {
      /*------------------------------------------------------------------------------
        Purpose: Destroy dynamically created popups kept in FieldPopupMapping.
          Notes: The popups is in every second entry (the first entry is the field 
                 widget) 
      ------------------------------------------------------------------------------*/
      character cFieldPopupMapping = TypeFactory.character();
      integer i = TypeFactory.integer();
      handle hPopup = TypeFactory.handle();
      
      return function("destroyPopups", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FieldPopupMapping"));
         cFieldPopupMapping.assign(ghProp.unwrapBufferField().value());
         
         loopLabel12:
         for (i.assign(2); _isLessThanOrEqual(i, numEntries(cFieldPopupMapping)); i.assign(plus(i, 2)))
         {
            hPopup.assign(handle.fromString(entry(i, cFieldPopupMapping)));
            ErrorManager.silentErrorEnable();
            HandleOps.delete(hPopup);
            ErrorManager.silentErrorDisable();
         }
         
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FieldPopupMapping"));
         ghProp.unwrapBufferField().changeValue(new character(""));
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical disableRadioButton(final character _pcNameList_16, final integer _piButtonNum_3)
   {
      character pcNameList_16 = TypeFactory.initInput(_pcNameList_16);
      integer piButtonNum_3 = TypeFactory.initInput(_piButtonNum_3);
      
      return function("disableRadioButton", logical.class, new Block((Body) () -> 
      {
         /*------------------------------------------------------------------------------
           Purpose:  Disables the specified radio button in the object or objects 
                     in the NameList.
            Params:  INPUT pcNameList  AS CHARACTER
                     INPUT piButtonNum AS INTEGER
             Notes:  Does not support SmartDataFields
         ------------------------------------------------------------------------------*/
         returnNormal(ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "sensitizeRadioButton", targetProcedure(), "III", pcNameList_16, piButtonNum_3, false));
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical disableWidget(final character _pcNameList_16)
   {
      character pcNameList_16 = TypeFactory.initInput(_pcNameList_16);
      /*------------------------------------------------------------------------------
        Purpose:  Disables the object or objects in the NameList.
         Params:  INPUT pcNameList AS CHARACTER
          Notes:  The field is removed from EnabledFields and EnabledHandles so that
                  it remains disabled.
      ------------------------------------------------------------------------------*/
      character cDisplayedFields = TypeFactory.character();
      character cEnabledObjFlds = TypeFactory.character();
      character cEnabledObjHdls = TypeFactory.character();
      character cField = TypeFactory.character();
      character cFieldHandles = TypeFactory.character();
      character cFieldObjectName = TypeFactory.character();
      character cFldsToDisable = TypeFactory.character();
      character cQualifier = TypeFactory.character();
      handle hField = TypeFactory.handle();
      handle hTarget = TypeFactory.handle();
      integer iField = TypeFactory.integer();
      integer iFieldNum = TypeFactory.integer();
      logical lInvalid = TypeFactory.logical();
      
      return function("disableWidget", logical.class, new Block((Body) () -> 
      {
         /* If the first name in the list is qualified that is stored so that it can be used
              as the qualifier for following non-qualified names in the list.  */
         if (_and(isGreaterThan(numEntries(pcNameList_16), 1), () -> isGreaterThan(numEntries(entry(1, pcNameList_16), "."), 1)))
         {
            cQualifier.assign(substring(entry(1, pcNameList_16), 1, lastIndexOf(entry(1, pcNameList_16), ".")));
         }
         
         IntegerExpression resExpr8 = new IntegerExpression()
         {
            public integer execute()
            {
               return numEntries(pcNameList_16);
            }
         };
         ToClause toClause8 = new ToClause(iField, 1, resExpr8);
         
         doTo("loopLabel13", toClause8, new Block((Body) () -> 
         {
            cField.assign(entry(iField, pcNameList_16));
            
            if (_and(isEqual(numEntries(cField, "."), 1), () -> isNotEqual(cQualifier, "")))
            {
               cField.assign(concat(cQualifier, cField));
            }
            
            ControlFlowOps.invokeInWithMode("locateWidget", targetProcedure(), "IOO", cField, hField, hTarget);
            
            if ((hField.isValid()).booleanValue())
            {
               /* This is a SmartDataField. Use its disableField function. 
                        ALso set the flag that it should not re-enabled. */
               if ((HandleOps.canQuery(hField, "FILE-NAME")).booleanValue())
               {
                  ErrorManager.silentErrorEnable();
                  ControlFlowOps.invokeIn("disableField", hField);
                  ErrorManager.silentErrorDisable();
                  
                  if ((ErrorManager.isError()).booleanValue())
                  {
                     lInvalid.assign(true);
                  }
                  else
                  {
                     ErrorManager.silentErrorEnable();
                     ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "setEnableField", hField, "I", false);
                     ErrorManager.silentErrorDisable();
                     
                     if ((ErrorManager.isError()).booleanValue())
                     {
                        lInvalid.assign(true);
                     }
                  }
               }
               else
               {
                  if ((HandleOps.canSet(hField, "SENSITIVE")).booleanValue())
                  {
                     ErrorManager.silentErrorEnable();
                     hField.unwrapSensitive().setSensitive(new logical(false));
                     ErrorManager.silentErrorDisable();
                     
                     if ((ErrorManager.isError()).booleanValue())
                     {
                        lInvalid.assign(true);
                     }
                  }
                  else
                  {
                     if ((HandleOps.canSet(hField, "READ-ONLY")).booleanValue())
                     {
                        ErrorManager.silentErrorEnable();
                        hField.unwrapWidget().setReadOnly(new logical(true));
                        ErrorManager.silentErrorDisable();
                        
                        if ((ErrorManager.isError()).booleanValue())
                        {
                           lInvalid.assign(true);
                        }
                     }
                     else
                     {
                        lInvalid.assign(true);
                     }
                  }
               }
               
               if (_not(lInvalid))
               {
                  ErrorManager.silentErrorEnable();
                  /* Need to get the field name in the object, not necessarily the field name in pcNameList,
                             because it may by qualified with an SDO name (e.g. viewer based on SBO).  */
                  /* {get Prop var [handle]} */
                  cDisplayedFields.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getDisplayedFields", hTarget));
                  ErrorManager.silentErrorDisable();
                  ErrorManager.silentErrorEnable();
                  /* {get Prop var [handle]} */
                  cFieldHandles.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getFieldHandles", hTarget));
                  ErrorManager.silentErrorDisable();
                  ErrorManager.silentErrorEnable();
                  ghProp.assign(handle.fromString(entry(1, hTarget.unwrapADMData().getADMData(), chr(1))));
                  ghProp.assign(ghProp.unwrapBuffer().bufferField("EnabledObjFlds"));
                  cEnabledObjFlds.assign(ghProp.unwrapBufferField().value());
                  ErrorManager.silentErrorDisable();
                  ErrorManager.silentErrorEnable();
                  ghProp.assign(handle.fromString(entry(1, hTarget.unwrapADMData().getADMData(), chr(1))));
                  ghProp.assign(ghProp.unwrapBuffer().bufferField("EnabledObjHdls"));
                  cEnabledObjHdls.assign(ghProp.unwrapBufferField().value());
                  ErrorManager.silentErrorDisable();
                  /* Remove the field from the enabled fields so that it stays disabled. */
                  iFieldNum.assign(lookup(valueOf(hField), cFieldHandles));
                  
                  if (_isGreaterThan(iFieldNum, 0))
                  {
                     ErrorManager.silentErrorEnable();
                     cFieldObjectName.assign(entry(iFieldNum, cDisplayedFields));
                     ErrorManager.silentErrorDisable();
                     ErrorManager.silentErrorEnable();
                     ControlFlowOps.invokeInWithMode("modifyListProperty", targetProcedure(), "IIII", hTarget, new character("REMOVE"), new character("EnabledFields"), cFieldObjectName);
                     ErrorManager.silentErrorDisable();
                     ErrorManager.silentErrorEnable();
                     ControlFlowOps.invokeInWithMode("modifyListProperty", targetProcedure(), "IIII", hTarget, new character("REMOVE"), new character("EnabledHandles"), hField);
                     ErrorManager.silentErrorDisable();
                  }
                  else
                  {
                     iFieldNum.assign(lookup(valueOf(hField), cEnabledObjHdls));
                     
                     if (_isGreaterThan(iFieldNum, 0))
                     {
                        ErrorManager.silentErrorEnable();
                        /* {get Prop var [handle]} */
                        cFldsToDisable.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getEnabledObjFldsToDisable", hTarget));
                        ErrorManager.silentErrorDisable();
                        
                        if (_isEqual(cFldsToDisable, "(All)"))
                        {
                           ErrorManager.silentErrorEnable();
                           cFldsToDisable.assign(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "deleteEntry", hTarget, "III", iFieldNum, cEnabledObjFlds, ""));
                           ErrorManager.silentErrorDisable();
                           ErrorManager.silentErrorEnable();
                           /* {set Prop value [handle]} */
                           ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "setEnabledObjFldsToDisable", hTarget, "I", cFldsToDisable);
                           ErrorManager.silentErrorDisable();
                        }
                        else
                        {
                           if (_isNotEqual(cFldsToDisable, "(None)"))
                           {
                              ErrorManager.silentErrorEnable();
                              cFieldObjectName.assign(entry(iFieldNum, cEnabledObjFlds));
                              ErrorManager.silentErrorDisable();
                              ErrorManager.silentErrorEnable();
                              ControlFlowOps.invokeInWithMode("modifyListProperty", targetProcedure(), "IIII", hTarget, new character("REMOVE"), new character("EnabledObjFldsToDisable"), cFieldObjectName);
                              ErrorManager.silentErrorDisable();
                           }
                        }
                     }
                  }
               }
            }
            else
            {
               lInvalid.assign(true);
            }
         }));
         
         /* END DO iField */
         /* Return FALSE if any field was invalid. */
         returnNormal(not(lInvalid));
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical enableRadioButton(final character _pcNameList_16, final integer _piButtonNum_3)
   {
      character pcNameList_16 = TypeFactory.initInput(_pcNameList_16);
      integer piButtonNum_3 = TypeFactory.initInput(_piButtonNum_3);
      
      return function("enableRadioButton", logical.class, new Block((Body) () -> 
      {
         /*------------------------------------------------------------------------------
           Purpose:  Enables the specified radio button in the object or objects 
                     in the NameList.
            Params:  INPUT pcNameList  AS CHARACTER
                     INPUT piButtonNum AS INTEGER
             Notes:  Does not support SmartDataFields
         ------------------------------------------------------------------------------*/
         returnNormal(ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "sensitizeRadioButton", targetProcedure(), "III", pcNameList_16, piButtonNum_3, true));
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical enableWidget(final character _pcNameList_16)
   {
      character pcNameList_16 = TypeFactory.initInput(_pcNameList_16);
      /*------------------------------------------------------------------------------
        Purpose:  Enables the object or objects in the NameList.
         Params:  INPUT pcNameList AS CHARACTER
          Notes:  The field is added to EnabledFields and EnabledHandles so that
                  it remains enabled.
      ------------------------------------------------------------------------------*/
      character cDisplayedFields = TypeFactory.character();
      character cEnabledObjFlds = TypeFactory.character();
      character cEnabledObjHdls = TypeFactory.character();
      character cField = TypeFactory.character();
      character cFieldHandles = TypeFactory.character();
      character cFieldObjectName = TypeFactory.character();
      character cFldsToDisable = TypeFactory.character();
      character cQualifier = TypeFactory.character();
      handle hField = TypeFactory.handle();
      handle hTarget = TypeFactory.handle();
      integer iField = TypeFactory.integer();
      integer iFieldNum = TypeFactory.integer();
      logical lInvalid = TypeFactory.logical();
      
      return function("enableWidget", logical.class, new Block((Body) () -> 
      {
         /* If the first name in the list is qualified that is stored so that it can be used
              as the qualifier for following non-qualified names in the list.  */
         if (_and(isGreaterThan(numEntries(pcNameList_16), 1), () -> isGreaterThan(numEntries(entry(1, pcNameList_16), "."), 1)))
         {
            cQualifier.assign(substring(entry(1, pcNameList_16), 1, lastIndexOf(entry(1, pcNameList_16), ".")));
         }
         
         IntegerExpression resExpr9 = new IntegerExpression()
         {
            public integer execute()
            {
               return numEntries(pcNameList_16);
            }
         };
         ToClause toClause9 = new ToClause(iField, 1, resExpr9);
         
         doTo("loopLabel14", toClause9, new Block((Body) () -> 
         {
            cField.assign(entry(iField, pcNameList_16));
            
            if (_and(isEqual(numEntries(cField, "."), 1), () -> isNotEqual(cQualifier, "")))
            {
               cField.assign(concat(cQualifier, cField));
            }
            
            ControlFlowOps.invokeInWithMode("locateWidget", targetProcedure(), "IOO", cField, hField, hTarget);
            
            if ((hField.isValid()).booleanValue())
            {
               /* This is a SmartDataField. Use its enableField function. */
               if ((HandleOps.canQuery(hField, "FILE-NAME")).booleanValue())
               {
                  ErrorManager.silentErrorEnable();
                  ControlFlowOps.invokeIn("enableField", hField);
                  ErrorManager.silentErrorDisable();
                  
                  if ((ErrorManager.isError()).booleanValue())
                  {
                     lInvalid.assign(true);
                  }
                  else
                  {
                     ErrorManager.silentErrorEnable();
                     ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "setEnableField", hField, "I", true);
                     ErrorManager.silentErrorDisable();
                     
                     if ((ErrorManager.isError()).booleanValue())
                     {
                        lInvalid.assign(true);
                     }
                  }
               }
               else
               {
                  if ((HandleOps.canSet(hField, "SENSITIVE")).booleanValue())
                  {
                     hField.unwrapSensitive().setSensitive(new logical(true));
                     
                     if ((ErrorManager.isError()).booleanValue())
                     {
                        lInvalid.assign(true);
                     }
                  }
                  else
                  {
                     if ((HandleOps.canSet(hField, "READ-ONLY")).booleanValue())
                     {
                        ErrorManager.silentErrorEnable();
                        hField.unwrapWidget().setReadOnly(new logical(false));
                        ErrorManager.silentErrorDisable();
                        
                        if ((ErrorManager.isError()).booleanValue())
                        {
                           lInvalid.assign(true);
                        }
                     }
                     else
                     {
                        lInvalid.assign(true);
                     }
                  }
               }
               
               if (_not(lInvalid))
               {
                  ErrorManager.silentErrorEnable();
                  /* Need to get the field name in the object, not necessarily the field name in pcNameList,
                             because it may by qualified with an SDO name (e.g. viewer based on SBO).  */
                  /* {get Prop var [handle]} */
                  cDisplayedFields.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getDisplayedFields", hTarget));
                  ErrorManager.silentErrorDisable();
                  ErrorManager.silentErrorEnable();
                  /* {get Prop var [handle]} */
                  cFieldHandles.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getFieldHandles", hTarget));
                  ErrorManager.silentErrorDisable();
                  ErrorManager.silentErrorEnable();
                  ghProp.assign(handle.fromString(entry(1, hTarget.unwrapADMData().getADMData(), chr(1))));
                  ghProp.assign(ghProp.unwrapBuffer().bufferField("EnabledObjFlds"));
                  cEnabledObjFlds.assign(ghProp.unwrapBufferField().value());
                  ErrorManager.silentErrorDisable();
                  ErrorManager.silentErrorEnable();
                  ghProp.assign(handle.fromString(entry(1, hTarget.unwrapADMData().getADMData(), chr(1))));
                  ghProp.assign(ghProp.unwrapBuffer().bufferField("EnabledObjHdls"));
                  cEnabledObjHdls.assign(ghProp.unwrapBufferField().value());
                  ErrorManager.silentErrorDisable();
                  /* Add the field to enabled fields so that it stays enabled. */
                  iFieldNum.assign(lookup(valueOf(hField), cFieldHandles));
                  
                  if (_isGreaterThan(iFieldNum, 0))
                  {
                     ErrorManager.silentErrorEnable();
                     cFieldObjectName.assign(entry(iFieldNum, cDisplayedFields));
                     ErrorManager.silentErrorDisable();
                     ErrorManager.silentErrorEnable();
                     ControlFlowOps.invokeInWithMode("modifyListProperty", targetProcedure(), "IIII", hTarget, new character("ADD"), new character("EnabledFields"), cFieldObjectName);
                     ErrorManager.silentErrorDisable();
                     ErrorManager.silentErrorEnable();
                     ControlFlowOps.invokeInWithMode("modifyListProperty", targetProcedure(), "IIII", hTarget, new character("ADD"), new character("EnabledHandles"), hField);
                     ErrorManager.silentErrorDisable();
                  }
                  else
                  {
                     iFieldNum.assign(lookup(valueOf(hField), cEnabledObjHdls));
                     
                     if (_isGreaterThan(iFieldNum, 0))
                     {
                        ErrorManager.silentErrorEnable();
                        /* {get Prop var [handle]} */
                        cFldsToDisable.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getEnabledObjFldsToDisable", hTarget));
                        ErrorManager.silentErrorDisable();
                        
                        if (_and(isNotEqual(cFldsToDisable, "(All)"), () -> isNotEqual(cFldsToDisable, "(None)")))
                        {
                           ErrorManager.silentErrorEnable();
                           cFieldObjectName.assign(entry(iFieldNum, cEnabledObjFlds));
                           ErrorManager.silentErrorDisable();
                           ErrorManager.silentErrorEnable();
                           ControlFlowOps.invokeInWithMode("modifyListProperty", targetProcedure(), "IIII", hTarget, new character("ADD"), new character("EnabledObjFldsToDisable"), cFieldObjectName);
                           ErrorManager.silentErrorDisable();
                        }
                     }
                  }
               }
            }
            else
            {
               lInvalid.assign(true);
            }
         }));
         
         /* END DO iField */
         /* Return FALSE if any field was invalid. */
         returnNormal(not(lInvalid));
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character formattedWidgetValue(final character _pcName_8)
   {
      character pcName_8 = TypeFactory.initInput(_pcName_8);
      /*------------------------------------------------------------------------------
        Purpose:  Returns the formatted value of an object.
         Params:  INPUT pcName AS CHARACTER
          Notes:  
      ------------------------------------------------------------------------------*/
      character cValue = TypeFactory.character();
      handle hBuffer = TypeFactory.handle();
      handle hField = TypeFactory.handle();
      handle hTarget = TypeFactory.handle();
      
      return function("formattedWidgetValue", character.class, new Block((Body) () -> 
      {
         ControlFlowOps.invokeInWithMode("locateWidget", targetProcedure(), "IOO", pcName_8, hField, hTarget);
         
         if ((hField.isValid()).booleanValue())
         {
            /* This is a SmartDataField. Return its DataValue property.
                   Note that this is the underlying "key" value that is meaningful
                   to the code, not the "DisplayValue" shown to the user. */
            if ((HandleOps.canQuery(hField, "FILE-NAME")).booleanValue())
            {
               /* {get Prop var [handle]} */
               cValue.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getDataValue", hField));
            }
            else
            {
               if ((HandleOps.canQuery(hField, "SCREEN-VALUE")).booleanValue())
               {
                   /* Viewer fields */cValue.assign(hField.unwrapWidget().getScreenValue());
               }
               else
               {
                  ErrorManager.silentErrorEnable();
                  /* {get Prop var [handle]} */
                  hBuffer.assign(ControlFlowOps.invokeDynamicFunctionIn(handle.class, "getQueryRowObject", hTarget));
                  ErrorManager.silentErrorDisable();
                  
                  if ((hBuffer.isValid()).booleanValue())
                  {
                     if (_isEqual(numEntries(pcName_8, "."), 2))
                     {
                        pcName_8.assign(entry(2, pcName_8, "."));
                     }
                     
                     ErrorManager.silentErrorEnable();
                     hField.assign(hBuffer.unwrapBuffer().bufferField(pcName_8));
                     cValue.assign(valueOf(hField.unwrapBufferField().stringValue()));
                     ErrorManager.silentErrorDisable();
                     
                     if ((ErrorManager.isError()).booleanValue())
                     {
                        cValue.setUnknown();
                     }
                  }
                  else
                  {
                     cValue.setUnknown();
                  }
               }
            }
         }
         else
         {
            cValue.setUnknown();
         }
         returnNormal(cValue);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character formattedWidgetValueList(final character _pcNameList_16, final character _pcDelimiter_3)
   {
      character pcNameList_16 = TypeFactory.initInput(_pcNameList_16);
      character pcDelimiter_3 = TypeFactory.initInput(_pcDelimiter_3);
      /*------------------------------------------------------------------------------
        Purpose:  Returns the formatted value of objects in a delimited list.
         Params:  INPUT pcNameList  AS CHARACTER
                  INPUT pcDelimiter AS CHARACTER
          Notes:  If pcDelimiter is ?, | is used
      ------------------------------------------------------------------------------*/
      character cField = TypeFactory.character();
      character cQualifier = TypeFactory.character();
      character cValue = TypeFactory.character();
      character cValueList = TypeFactory.character();
      integer iField = TypeFactory.integer();
      
      return function("formattedWidgetValueList", character.class, new Block((Body) () -> 
      {
         if (_isUnknown(pcDelimiter_3))
         {
            pcDelimiter_3.assign("|");
         }
         
         /* If the first name in the list is qualified that is stored so that it can be used
              as the qualifier for following non-qualified names in the list.  */
         if (_and(isGreaterThan(numEntries(pcNameList_16), 1), () -> isGreaterThan(numEntries(entry(1, pcNameList_16), "."), 1)))
         {
            cQualifier.assign(substring(entry(1, pcNameList_16), 1, lastIndexOf(entry(1, pcNameList_16), ".")));
         }
         
         IntegerExpression resExpr10 = new IntegerExpression()
         {
            public integer execute()
            {
               return numEntries(pcNameList_16);
            }
         };
         ToClause toClause10 = new ToClause(iField, 1, resExpr10);
         
         doTo("loopLabel15", toClause10, new Block((Body) () -> 
         {
            cField.assign(entry(iField, pcNameList_16));
            
            if (_and(isEqual(numEntries(cField, "."), 1), () -> isNotEqual(cQualifier, "")))
            {
               cField.assign(concat(cQualifier, cField));
            }
            
            cValue.assign(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "formattedWidgetValue", targetProcedure(), "I", cField));
            cValueList.assign(concat(cValueList, (_isEqual(iField, 1) ? new character("") : pcDelimiter_3), _isUnknown(cValue) ? new character("?") : cValue));
         }));
         
         /* do iField to number entries in namelist */
         returnNormal(cValueList);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getAllFieldHandles()
   {
      /*------------------------------------------------------------------------------
        Purpose: 
          Notes:   
      ------------------------------------------------------------------------------*/
      character cValue = TypeFactory.character();
      
      return function("getAllFieldHandles", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("allFieldHandles"));
         cValue.assign(ghProp.unwrapBufferField().value());
         returnNormal(cValue);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getAllFieldNames()
   {
      /*------------------------------------------------------------------------------
        Purpose: 
          Notes:   
      ------------------------------------------------------------------------------*/
      character cValue = TypeFactory.character();
      
      return function("getAllFieldNames", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("allFieldNames"));
         cValue.assign(ghProp.unwrapBufferField().value());
         returnNormal(cValue);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public integer getBgcolor()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Returns the Background color of the frame
          Notes:  
      ------------------------------------------------------------------------------*/
      integer iBgcolor = TypeFactory.integer((Long) null);
      handle hFrame = TypeFactory.handle();
      
      return function("getBGColor", integer.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ContainerHandle"));
         hFrame.assign(ghProp.unwrapBufferField().value());
         
         if ((hFrame.isValid()).booleanValue())
         {
            ErrorManager.silentErrorEnable();
            iBgcolor.assign(hFrame.unwrapWidget().getBgColor());
            ErrorManager.silentErrorDisable();
         }
         
         returnNormal(iBgcolor);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public decimal getCol()
   {
      /*------------------------------------------------------------------------------
        Purpose: Return the Col(umn) of the object 
          Notes: Use repositionObject to set the COL.   
      ------------------------------------------------------------------------------*/
      handle hContainer = TypeFactory.handle();
      
      return function("getCol", decimal.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ContainerHandle"));
         hContainer.assign(ghProp.unwrapBufferField().value());
         returnNormal((hContainer.isValid()).booleanValue() ? hContainer.unwrapWidget().getColumn() : new decimal());
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public integer getColor3Dface()
   {
      /*------------------------------------------------------------------------------
        Purpose:  
          Notes:  
      ------------------------------------------------------------------------------*/
      integer iRgb = TypeFactory.integer();
      
      return function("getColor3DFace", integer.class, new Block((Body) () -> 
      {
         ControlFlowOps.invokeWithMode("GetSysColor", "IO", new integer(15), iRgb);
         ErrorManager.silentErrorEnable();
         ColorTable.setRgbValue(giColor3Dface, iRgb);
         ErrorManager.silentErrorDisable();
         returnNormal(giColor3Dface);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public integer getColor3DhighLight()
   {
      /*------------------------------------------------------------------------------
        Purpose:  
          Notes:  
      ------------------------------------------------------------------------------*/
      integer iRgb = TypeFactory.integer();
      
      return function("getColor3DHighLight", integer.class, new Block((Body) () -> 
      {
         ControlFlowOps.invokeWithMode("GetSysColor", "IO", new integer(20), iRgb);
         ErrorManager.silentErrorEnable();
         ColorTable.setRgbValue(giColor3DhighLight, iRgb);
         ErrorManager.silentErrorDisable();
         returnNormal(giColor3DhighLight);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public integer getColor3Dshadow()
   {
      /*------------------------------------------------------------------------------
        Purpose:  
          Notes:  
      ------------------------------------------------------------------------------*/
      integer iRgb = TypeFactory.integer();
      
      return function("getColor3DShadow", integer.class, new Block((Body) () -> 
      {
         ControlFlowOps.invokeWithMode("GetSysColor", "IO", new integer(16), iRgb);
         ErrorManager.silentErrorEnable();
         ColorTable.setRgbValue(giColor3Dshadow, iRgb);
         ErrorManager.silentErrorDisable();
         returnNormal(giColor3Dshadow);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public integer getColorErrorBg()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Returns the error background color number
         Params:  <none>
          Notes:  Used by highlightWidget
      ------------------------------------------------------------------------------*/
      integer iColor = TypeFactory.integer();
      
      return function("getColorErrorBG", integer.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ColorErrorBG"));
         iColor.assign(ghProp.unwrapBufferField().value());
         returnNormal(iColor);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public integer getColorErrorFg()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Returns the error foreground color number
         Params:  <none>
          Notes:  Used by highlightWidget
      ------------------------------------------------------------------------------*/
      integer iColor = TypeFactory.integer();
      
      return function("getColorErrorFG", integer.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ColorErrorFG"));
         iColor.assign(ghProp.unwrapBufferField().value());
         returnNormal(iColor);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public integer getColorInfoBg()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Returns the information background color number
         Params:  <none>
          Notes:  Used by highlightWidget
      ------------------------------------------------------------------------------*/
      integer iColor = TypeFactory.integer();
      
      return function("getColorInfoBG", integer.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ColorInfoBG"));
         iColor.assign(ghProp.unwrapBufferField().value());
         returnNormal(iColor);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public integer getColorInfoFg()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Returns the information foreground color number
         Params:  <none>
          Notes:  Used by highlightWidget
      ------------------------------------------------------------------------------*/
      integer iColor = TypeFactory.integer();
      
      return function("getColorInfoFG", integer.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ColorInfoFG"));
         iColor.assign(ghProp.unwrapBufferField().value());
         returnNormal(iColor);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public integer getColorWarnBg()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Returns the warning background color number
         Params:  <none>
          Notes:  Used by highlightWidget
      ------------------------------------------------------------------------------*/
      integer iColor = TypeFactory.integer();
      
      return function("getColorWarnBG", integer.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ColorWarnBG"));
         iColor.assign(ghProp.unwrapBufferField().value());
         returnNormal(iColor);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public integer getColorWarnFg()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Returns the warning foreground color number
         Params:  <none>
          Notes:  Used by highlightWidget
      ------------------------------------------------------------------------------*/
      integer iColor = TypeFactory.integer();
      
      return function("getColorWarnFG", integer.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ColorWarnFG"));
         iColor.assign(ghProp.unwrapBufferField().value());
         returnNormal(iColor);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getDefaultLayout()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Returns the name of the default for this object.
         Params:  <none>
      ------------------------------------------------------------------------------*/
      character cDefault = TypeFactory.character();
      
      return function("getDefaultLayout", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DefaultLayout"));
         cDefault.assign(ghProp.unwrapBufferField().value());
         returnNormal(cDefault);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical getDisableOnInit()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Returns a flag indicating whether the current object should be
                  left disabled when it is first initialized.
         Params:  <none>
      ------------------------------------------------------------------------------*/
      logical lDisable = TypeFactory.logical();
      
      return function("getDisableOnInit", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DisableOnInit"));
         lDisable.assign(ghProp.unwrapBufferField().value());
         returnNormal(lDisable);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getEnabledObjFlds()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Returns a list of the field names of widgets enabled in this object
                  not associated with data fields.
         Params:  <none>
      ------------------------------------------------------------------------------*/
      character cFields = TypeFactory.character();
      
      return function("getEnabledObjFlds", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("EnabledObjFlds"));
         cFields.assign(ghProp.unwrapBufferField().value());
         returnNormal(cFields);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getEnabledObjHdls()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Returns a list of the handles of widgets enabled in this object
                  not associated with data fields.
         Params:  <none>
      ------------------------------------------------------------------------------*/
      character cHdls = TypeFactory.character();
      
      return function("getEnabledObjHdls", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("EnabledObjHdls"));
         cHdls.assign(ghProp.unwrapBufferField().value());
         returnNormal(cHdls);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public integer getFgcolor()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Retrieves the Foreground Color of the frame 
          Notes:  
      ------------------------------------------------------------------------------*/
      integer iFgcolor = TypeFactory.integer((Long) null);
      handle hFrame = TypeFactory.handle();
      
      return function("getFGColor", integer.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ContainerHandle"));
         hFrame.assign(ghProp.unwrapBufferField().value());
         
         if ((hFrame.isValid()).booleanValue())
         {
            ErrorManager.silentErrorEnable();
            iFgcolor.assign(hFrame.unwrapWidget().getFgColor());
            ErrorManager.silentErrorDisable();
         }
         
         returnNormal(iFgcolor);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getFieldPopupMapping()
   {
      /*------------------------------------------------------------------------------
        Purpose: Returns the mapping of field and object handles and their dynamically 
                 created popup handle. The list can be used directly, but is also used
                 by popupHandle(<field>) to return a specific popup.  
          Notes: The popup widget and this property will be created and set by 
                 widgetwalk in the session manager when this manager is running. 
      ------------------------------------------------------------------------------*/
      character cFieldPopupMapping = TypeFactory.character();
      
      return function("getFieldPopupMapping", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FieldPopupMapping"));
         cFieldPopupMapping.assign(ghProp.unwrapBufferField().value());
         returnNormal(cFieldPopupMapping);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getFieldSecurity()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Returns a character value  (in a list form, comma-seperated) 
                  of the security type corresponding to AllFieldHandles.
                  <security type>,<security type>...
         Params:  <none>
      ------------------------------------------------------------------------------*/
      character cSecurityType = TypeFactory.character();
      
      return function("getFieldSecurity", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FieldSecurity"));
         cSecurityType.assign(ghProp.unwrapBufferField().value());
         returnNormal(cSecurityType);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public decimal getHeight()
   {
      /*------------------------------------------------------------------------------
        Purpose: Return the height of the object 
          Notes: Use resizeObject to set the height. 
      ------------------------------------------------------------------------------*/
      handle hContainer = TypeFactory.handle();
      
      return function("getHeight", decimal.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ContainerHandle"));
         hContainer.assign(ghProp.unwrapBufferField().value());
         
         if ((hContainer.isValid()).booleanValue())
         {
            returnNormal(hContainer.unwrapSizeable().getHeightChars());
         }
         
         returnNormal(new decimal());
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getLayoutOptions()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Returns the list of multi-layout options for the 
                  object.
         Params:  <none>
      ------------------------------------------------------------------------------*/
      character cLayout = TypeFactory.character();
      
      return function("getLayoutOptions", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("LayoutOptions"));
         cLayout.assign(ghProp.unwrapBufferField().value());
         returnNormal(cLayout);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getLayoutVariable()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Returns the name of the &LAYOUT-VARIABLE preprocessor for the 
                  object, which is used as a prefix to the name of the procedure 
                  which resets it.
         Params:  <none>
      ------------------------------------------------------------------------------*/
      character cLayout = TypeFactory.character();
      
      return function("getLayoutVariable", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("LayoutVariable"));
         cLayout.assign(ghProp.unwrapBufferField().value());
         returnNormal(cLayout);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public decimal getMinHeight()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Returns the pre-determined minimum height of a visual object
          Notes:  
      ------------------------------------------------------------------------------*/
      decimal dHeight = TypeFactory.decimal();
      
      return function("getMinHeight", decimal.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("MinHeight"));
         dHeight.assign(ghProp.unwrapBufferField().value());
         
         if (_or(isEqual(dHeight, 0), () -> isUnknown(dHeight)))
         {
            /* {get Prop var [handle]} */ 
            dHeight.assign(ControlFlowOps.invokeDynamicFunctionIn(decimal.class, "getHeight", targetProcedure()));
            /* {set Prop value [handle]} */ 
            ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "setMinHeight", targetProcedure(), "I", dHeight);
         }
         
         returnNormal(dHeight);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public decimal getMinWidth()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Returns the pre-determined minimum width of a visual object
          Notes:  
      ------------------------------------------------------------------------------*/
      decimal dWidth = TypeFactory.decimal();
      
      return function("getMinWidth", decimal.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("MinWidth"));
         dWidth.assign(ghProp.unwrapBufferField().value());
         
         if (_or(isEqual(dWidth, 0), () -> isUnknown(dWidth)))
         {
            /* {get Prop var [handle]} */ 
            dWidth.assign(ControlFlowOps.invokeDynamicFunctionIn(decimal.class, "getWidth", targetProcedure()));
            /* {set Prop value [handle]} */ 
            ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "setMinWidth", targetProcedure(), "I", dWidth);
         }
         
         returnNormal(dWidth);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical getObjectEnabled()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Returns a flag indicating whether the current object is enabled.
         Params:  <none>
      ------------------------------------------------------------------------------*/
      logical lEnabled = TypeFactory.logical();
      
      return function("getObjectEnabled", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ObjectEnabled"));
         lEnabled.assign(ghProp.unwrapBufferField().value());
         returnNormal(lEnabled);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getObjectLayout()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Returns the current Layout Name for the object.
         Params:  <none>
      ------------------------------------------------------------------------------*/
      character cLayout = TypeFactory.character();
      
      return function("getObjectLayout", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ObjectLayout"));
         cLayout.assign(ghProp.unwrapBufferField().value());
         returnNormal(cLayout);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical getObjectSecured()
   {
      /*------------------------------------------------------------------------------
        Purpose: Tells whether the Security has been applied to the object 
          Notes: Set to true by the Repository retrieval process for dynamic objects
      ------------------------------------------------------------------------------*/
      logical lObjectSecured = TypeFactory.logical();
      
      return function("getObjectSecured", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ObjectSecured"));
         lObjectSecured.assign(ghProp.unwrapBufferField().value());
         returnNormal(lObjectSecured);
      }));
   }

   /** getObjectSecured */
   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical getObjectTranslated()
   {
      /*------------------------------------------------------------------------------
        Purpose: Tells whether Translation has been applied to the object 
          Notes: Set to true by the Repository retrieval process for dynamic objects
      ------------------------------------------------------------------------------*/
      logical lObjectTranslated = TypeFactory.logical();
      
      return function("getObjectTranslated", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ObjectTranslated"));
         lObjectTranslated.assign(ghProp.unwrapBufferField().value());
         returnNormal(lObjectTranslated);
      }));
   }

   /** getObjectTranslated */
   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical getPopupButtonsInFields()
   {
      /*------------------------------------------------------------------------------
        Purpose: When set to YES, automatic calendar and calculator popup buttons will 
                 be placed inside, on the right hand side of the field.  
                 When NO, the popup button will be placed outside, to the right of the 
                 field.  
          Notes:  
      ------------------------------------------------------------------------------*/
      logical lPopupButtonsInFields = TypeFactory.logical();
      
      return function("getPopupButtonsInFields", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("PopupButtonsInFields"));
         lPopupButtonsInFields.assign(ghProp.unwrapBufferField().value());
         returnNormal(lPopupButtonsInFields);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical getResizeHorizontal()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Returns TRUE of the object can be resized horizontal
          Notes:  
      ------------------------------------------------------------------------------*/
      logical lResizeHorizontal = TypeFactory.logical();
      
      return function("getResizeHorizontal", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ResizeHorizontal"));
         lResizeHorizontal.assign(ghProp.unwrapBufferField().value());
         returnNormal(lResizeHorizontal);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical getResizeVertical()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Returns TRUE of the object can be resized vertical
          Notes:  
      ------------------------------------------------------------------------------*/
      logical lResizeVertical = TypeFactory.logical();
      
      return function("getResizeVertical", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ResizeVertical"));
         lResizeVertical.assign(ghProp.unwrapBufferField().value());
         returnNormal(lResizeVertical);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public decimal getRow()
   {
      /*------------------------------------------------------------------------------
        Purpose: Return the height of the object 
          Notes: Use repositionObject to set the Row.   
      ------------------------------------------------------------------------------*/
      handle hContainer = TypeFactory.handle();
      
      return function("getRow", decimal.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ContainerHandle"));
         hContainer.assign(ghProp.unwrapBufferField().value());
         returnNormal((hContainer.isValid()).booleanValue() ? hContainer.unwrapWidget().getRow() : new decimal());
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getSecuredTokens()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Return the list of securedtokens for the container 
          Notes:  
      ------------------------------------------------------------------------------*/
      handle hContainerSource = TypeFactory.handle();
      character cObjectName = TypeFactory.character();
      character cRunAttribute = TypeFactory.character();
      character cSecuredTokens = TypeFactory.character();
      logical lObjectSecured = TypeFactory.logical();
      
      return function("getSecuredTokens", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("SecuredTokens"));
         cSecuredTokens.assign(ghProp.unwrapBufferField().value());
         
         if (_isUnknown(cSecuredTokens))
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("ObjectSecured"));
            lObjectSecured.assign(ghProp.unwrapBufferField().value());
            
            /* not (log = TRUE) to treat unknown as false */
            if (_not(isEqual(lObjectSecured, true)))
            {
               ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
               ghProp.assign(ghProp.unwrapBuffer().bufferField("ContainerSource"));
               hContainerSource.assign(ghProp.unwrapBufferField().value());
               
               if ((hContainerSource.isValid()).booleanValue())
               {
                  ghProp.assign(handle.fromString(entry(1, hContainerSource.unwrapADMData().getADMData(), chr(1))));
                  ghProp.assign(ghProp.unwrapBuffer().bufferField("LogicalObjectName"));
                  cObjectName.assign(ghProp.unwrapBufferField().value());
                  ghProp.assign(handle.fromString(entry(1, hContainerSource.unwrapADMData().getADMData(), chr(1))));
                  ghProp.assign(ghProp.unwrapBuffer().bufferField("RunAttribute"));
                  cRunAttribute.assign(ghProp.unwrapBufferField().value());
               }
               else
               {
                  cObjectName.assign("");
                  cRunAttribute.assign("");
               }
               
               if ((gshSecurityManager.isValid()).booleanValue())
               {
                  /* get list of secured tokens for the container instance */
                  ControlFlowOps.invokeInWithMode("tokenSecurityCheck", gshSecurityManager, "IIO", cObjectName, cRunAttribute, cSecuredTokens);
               }
            }
            
            if (_isUnknown(cSecuredTokens))
            {
               cSecuredTokens.assign("");
            }
            
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("ObjectSecured"));
            ghProp.unwrapBufferField().changeValue(new logical(true));
            /* {set Prop value [handle]} */ 
            ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "setSecuredTokens", targetProcedure(), "I", cSecuredTokens);
         }
         /* object not secured */
         returnNormal(cSecuredTokens);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public decimal getWidth()
   {
      /*------------------------------------------------------------------------------
        Purpose: Return the Width of the object 
          Notes: Use resizeObject to set the Width. 
      ------------------------------------------------------------------------------*/
      handle hContainer = TypeFactory.handle();
      
      return function("getWidth", decimal.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ContainerHandle"));
         hContainer.assign(ghProp.unwrapBufferField().value());
         
         if ((hContainer.isValid()).booleanValue())
         {
            returnNormal(hContainer.unwrapSizeable().getWidthChars());
         }
         
         returnNormal(new decimal());
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical hideWidget(final character _pcNameList_16)
   {
      character pcNameList_16 = TypeFactory.initInput(_pcNameList_16);
      /*------------------------------------------------------------------------------
        Purpose:  Hides the object or objects in the NameList.
         Params:  INPUT pcNameList AS CHARACTER
          Notes:  
      ------------------------------------------------------------------------------*/
      integer iField = TypeFactory.integer();
      character cField = TypeFactory.character();
      character cQualifier = TypeFactory.character();
      handle hField = TypeFactory.handle();
      handle hPopup = TypeFactory.handle();
      handle hTarget = TypeFactory.handle();
      logical lInvalid = TypeFactory.logical();
      
      return function("hideWidget", logical.class, new Block((Body) () -> 
      {
         /* If the first name in the list is qualified that is stored so that it can be used
              as the qualifier for following non-qualified names in the list.  */
         if (_and(isGreaterThan(numEntries(pcNameList_16), 1), () -> isGreaterThan(numEntries(entry(1, pcNameList_16), "."), 1)))
         {
            cQualifier.assign(substring(entry(1, pcNameList_16), 1, lastIndexOf(entry(1, pcNameList_16), ".")));
         }
         
         IntegerExpression resExpr11 = new IntegerExpression()
         {
            public integer execute()
            {
               return numEntries(pcNameList_16);
            }
         };
         ToClause toClause11 = new ToClause(iField, 1, resExpr11);
         
         doTo("loopLabel16", toClause11, new Block((Body) () -> 
         {
            cField.assign(entry(iField, pcNameList_16));
            
            if (_and(isEqual(numEntries(cField, "."), 1), () -> isNotEqual(cQualifier, "")))
            {
               cField.assign(concat(cQualifier, cField));
            }
            
            ControlFlowOps.invokeInWithMode("locateWidget", targetProcedure(), "IOO", cField, hField, hTarget);
            
            if ((hField.isValid()).booleanValue())
            {
               /* This is a SmartDataField. */
               if ((HandleOps.canQuery(hField, "FILE-NAME")).booleanValue())
               {
                  ControlFlowOps.invokeIn("hideObject", hField);
               }
               else
               {
                  if ((HandleOps.canSet(hField, "HIDDEN")).booleanValue())
                  {
                     ErrorManager.silentErrorEnable();
                     hField.unwrapWidget().setHidden(new logical(true));
                     ErrorManager.silentErrorDisable();
                     
                     if ((ErrorManager.isError()).booleanValue())
                     {
                        lInvalid.assign(true);
                     }
                     else
                     {
                        ErrorManager.silentErrorEnable();
                        hPopup.assign(ControlFlowOps.invokeDynamicFunctionInWithMode(handle.class, "popupHandle", hTarget, "I", hField));
                        ErrorManager.silentErrorDisable();
                     }
                     
                     if ((hPopup.isValid()).booleanValue())
                     {
                        hPopup.unwrapWidget().setHidden(new logical(true));
                     }
                  }
 /* Viewer fields */                  else
                  {
                     if ((HandleOps.canSet(hField, "VISIBLE")).booleanValue())
                     {
                        ErrorManager.silentErrorEnable();
                        hField.unwrapWidget().setVisible(new logical(false));
                        ErrorManager.silentErrorDisable();
                        
                        if ((ErrorManager.isError()).booleanValue())
                        {
                           lInvalid.assign(true);
                        }
                     }
 /* Browser columns */                     else
                     {
                        lInvalid.assign(true);
                     }
                  }
               }
            }
            else
            {
               lInvalid.assign(true);
            }
         }));
         
         /* END DO iField */
         /* Return FALSE if any field was invalid. */
         returnNormal(not(lInvalid));
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical highlightWidget(final character _pcNameList_16, final character _pcHighlightType_1)
   {
      character pcNameList_16 = TypeFactory.initInput(_pcNameList_16);
      character pcHighlightType_1 = TypeFactory.initInput(_pcHighlightType_1);
      /*------------------------------------------------------------------------------
        Purpose:  Highlights the object or objects in the NameList based on 
                  highlight type.
         Params:  INPUT pcNameList AS CHARACTER
          Notes:  Valid highlight types are:
                  info[rmation] - uses colorInfoBG & FG
                  warn[inf] - uses colorWarnBG & FG
                  err[or] - uses colorErrorBG & FB
                  def[ault] - sets to Windows default
                  Does not support SmartDataFields
      ------------------------------------------------------------------------------*/
      integer iBgcolor = TypeFactory.integer((Long) null);
      integer iFgcolor = TypeFactory.integer((Long) null);
      integer iField = TypeFactory.integer();
      character cField = TypeFactory.character();
      character cQualifier = TypeFactory.character();
      handle hField = TypeFactory.handle();
      handle hTarget = TypeFactory.handle();
      logical lInvalid = TypeFactory.logical();
      
      return function("highlightWidget", logical.class, new Block((Body) () -> 
      {
         /* If the first name in the list is qualified that is stored so that it can be used
              as the qualifier for following non-qualified names in the list.  */
         if (_and(isGreaterThan(numEntries(pcNameList_16), 1), () -> isGreaterThan(numEntries(entry(1, pcNameList_16), "."), 1)))
         {
            cQualifier.assign(substring(entry(1, pcNameList_16), 1, lastIndexOf(entry(1, pcNameList_16), ".")));
         }
         
         IntegerExpression resExpr12 = new IntegerExpression()
         {
            public integer execute()
            {
               return numEntries(pcNameList_16);
            }
         };
         ToClause toClause12 = new ToClause(iField, 1, resExpr12);
         
         doTo("loopLabel17", toClause12, new Block((Body) () -> 
         {
            cField.assign(entry(iField, pcNameList_16));
            
            if (_and(isEqual(numEntries(cField, "."), 1), () -> isNotEqual(cQualifier, "")))
            {
               cField.assign(concat(cQualifier, cField));
            }
            
            ControlFlowOps.invokeInWithMode("locateWidget", targetProcedure(), "IOO", cField, hField, hTarget);
            
            if ((hField.isValid()).booleanValue())
            {
               if (_begins(pcHighlightType_1, "INFO"))
               {
                  ghProp.assign(handle.fromString(entry(1, hTarget.unwrapADMData().getADMData(), chr(1))));
                  ghProp.assign(ghProp.unwrapBuffer().bufferField("ColorInfoBG"));
                  iBgcolor.assign(ghProp.unwrapBufferField().value());
                  ghProp.assign(handle.fromString(entry(1, hTarget.unwrapADMData().getADMData(), chr(1))));
                  ghProp.assign(ghProp.unwrapBuffer().bufferField("ColorInfoFG"));
                  iFgcolor.assign(ghProp.unwrapBufferField().value());
               }
               else
               {
                  if (_begins(pcHighlightType_1, "WARN"))
                  {
                     ghProp.assign(handle.fromString(entry(1, hTarget.unwrapADMData().getADMData(), chr(1))));
                     ghProp.assign(ghProp.unwrapBuffer().bufferField("ColorWarnBG"));
                     iBgcolor.assign(ghProp.unwrapBufferField().value());
                     ghProp.assign(handle.fromString(entry(1, hTarget.unwrapADMData().getADMData(), chr(1))));
                     ghProp.assign(ghProp.unwrapBuffer().bufferField("ColorWarnFG"));
                     iFgcolor.assign(ghProp.unwrapBufferField().value());
                  }
                  else
                  {
                     if (_begins(pcHighlightType_1, "ERR"))
                     {
                        ghProp.assign(handle.fromString(entry(1, hTarget.unwrapADMData().getADMData(), chr(1))));
                        ghProp.assign(ghProp.unwrapBuffer().bufferField("ColorErrorBG"));
                        iBgcolor.assign(ghProp.unwrapBufferField().value());
                        ghProp.assign(handle.fromString(entry(1, hTarget.unwrapADMData().getADMData(), chr(1))));
                        ghProp.assign(ghProp.unwrapBuffer().bufferField("ColorErrorFG"));
                        iFgcolor.assign(ghProp.unwrapBufferField().value());
                     }
                     else
                     {
                        if (_begins(pcHighlightType_1, "DEF"))
                        {
                        }
                        else
                        {
                           returnNormal(false);
                        }
                     }
                  }
               }
               
               if (_and(HandleOps.canSet(hField, "BGCOLOR"), () -> HandleOps.canSet(hField, "FGCOLOR")))
               {
                  ErrorManager.silentErrorEnable();
                  hField.unwrapWidget().setBgColor(iBgcolor);
                  hField.unwrapWidget().setFgColor(iFgcolor);
                  ErrorManager.silentErrorDisable();
                  
                  if ((ErrorManager.isError()).booleanValue())
                  {
                     lInvalid.assign(true);
                  }
               }
               else
               {
                  lInvalid.assign(true);
               }
            }
            else
            {
               lInvalid.assign(true);
            }
         }));
         
         /* END DO iField */
         /* Return FALSE if any field was invalid. */
         returnNormal(not(lInvalid));
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public handle internalWidgetHandle(final character _pcField_2, final character _pcSearchMode_1)
   {
      character pcField_2 = TypeFactory.initInput(_pcField_2);
      character pcSearchMode_1 = TypeFactory.initInput(_pcSearchMode_1);
      /*------------------------------------------------------------------------------
        Purpose:  Return handle of named widget in this object  
          Notes:  This version is for visual objects and does not use
                  search mode, it searches in AllFieldNames for the widget
      ------------------------------------------------------------------------------*/
      character cFieldNames = TypeFactory.character();
      character cFieldHandles = TypeFactory.character();
      integer iFieldPos = TypeFactory.integer();
      handle hField = TypeFactory.handle();
      
      return function("internalWidgetHandle", handle.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("AllFieldNames"));
         cFieldNames.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("AllFieldHandles"));
         cFieldHandles.assign(ghProp.unwrapBufferField().value());
         iFieldPos.assign(lookup(pcField_2, cFieldNames));
         
         if (_isNotEqual(iFieldPos, 0))
         {
            hField.assign(handle.fromString(entry(iFieldPos, cFieldHandles)));
         }
         
         if ((hField.isValid()).booleanValue())
         {
            returnNormal(hField);
         }
         else
         {
            returnNormal(new handle());
         }
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public handle popupHandle(final character _pcField_2)
   {
      character pcField_2 = TypeFactory.initInput(_pcField_2);
      /*------------------------------------------------------------------------------
         Purpose: Returns the dynamically created popup button handle for a field  
      Parameters: pcFieldName - char
                  - The stringed handle of the widget. 
                  - The unique name of the Field in the object.    
           Notes: The popup widget and the corresponding FieldPopupMapping property 
                  will be created by widgetwalk in the session manager when this 
                  manager is running.  
      ------------------------------------------------------------------------------*/
      character cFieldPopupMapping = TypeFactory.character();
      character cFieldNames = TypeFactory.character();
      character cFieldHandles = TypeFactory.character();
      character cHandle = TypeFactory.character();
      handle hPopup = TypeFactory.handle();
      integer iPos = TypeFactory.integer();
      
      return function("popupHandle", handle.class, new Block((Body) () -> 
      {
         /* if not valid widget then assume a name is passed */
         if (_not(handle.fromString(pcField_2).isValid()))
         {
            /* Although this only applies to enabled fields we check the Displayed list
                  as the EnabledFields may change while the object is running */
            /* {get Prop var [handle]} */ 
            cFieldNames.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getDisplayedFields", targetProcedure()));
            iPos.assign(lookup(pcField_2, cFieldNames));
            
            if (_isGreaterThan(iPos, 0))
            {
               /* {get Prop var [handle]} */ 
               cFieldHandles.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getFieldHandles", targetProcedure()));
               cHandle.assign(entry(iPos, cFieldHandles));
            }
            else
            {
               ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
               ghProp.assign(ghProp.unwrapBuffer().bufferField("EnabledObjFlds"));
               cFieldNames.assign(ghProp.unwrapBufferField().value());
               iPos.assign(lookup(pcField_2, cFieldNames));
               
               if (_isGreaterThan(iPos, 0))
               {
                  ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
                  ghProp.assign(ghProp.unwrapBuffer().bufferField("EnabledObjHdls"));
                  cFieldHandles.assign(ghProp.unwrapBufferField().value());
                  cHandle.assign(entry(iPos, cFieldHandles));
               }
            }
         }
         else
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("AllFieldHandles"));
            cFieldHandles.assign(ghProp.unwrapBufferField().value());
            iPos.assign(lookup(pcField_2, cFieldHandles));
            
            if (_isGreaterThan(iPos, 0))
            {
               cHandle.assign(pcField_2);
            }
         }
 /* Passed in a handle */;
         
         if (_and(isGreaterThan(iPos, 0), () -> isGreaterThan(cHandle, "")))
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("FieldPopupMapping"));
            cFieldPopupMapping.assign(ghProp.unwrapBufferField().value());
            iPos.assign(lookup(cHandle, cFieldPopupMapping));
            
            if (_and(isGreaterThan(iPos, 0), () -> isEqual(modulo(iPos, 2), 1)))
            {
               ErrorManager.silentErrorEnable();
               hPopup.assign(handle.fromString(entry(plus(iPos, 1), cFieldPopupMapping)));
               ErrorManager.silentErrorDisable();
            }
         }
         returnNormal(hPopup);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical resetWidgetValue(final character _pcNameList_16)
   {
      character pcNameList_16 = TypeFactory.initInput(_pcNameList_16);
      /*------------------------------------------------------------------------------
        Purpose:  Resets the SCREEN-VALUE of the object or objects in the NameList
                  to their original values from their data source.
         Params:  INPUT pcNameList AS CHARACTER
          Notes:  
      ------------------------------------------------------------------------------*/
      integer iField = TypeFactory.integer();
      character cField = TypeFactory.character();
      character cOnlyField = TypeFactory.character();
      character cQualifier = TypeFactory.character();
      character cValue = TypeFactory.character();
      handle hDataSource = TypeFactory.handle();
      handle hField = TypeFactory.handle();
      handle hTarget = TypeFactory.handle();
      logical lInvalid = TypeFactory.logical();
      
      return function("resetWidgetValue", logical.class, new Block((Body) () -> 
      {
         /* If the first name in the list is qualified that is stored so that it can be used
              as the qualifier for following non-qualified names in the list.  */
         if (_and(isGreaterThan(numEntries(pcNameList_16), 1), () -> isGreaterThan(numEntries(entry(1, pcNameList_16), "."), 1)))
         {
            cQualifier.assign(substring(entry(1, pcNameList_16), 1, lastIndexOf(entry(1, pcNameList_16), ".")));
         }
         
         IntegerExpression resExpr13 = new IntegerExpression()
         {
            public integer execute()
            {
               return numEntries(pcNameList_16);
            }
         };
         ToClause toClause13 = new ToClause(iField, 1, resExpr13);
         
         doTo("loopLabel18", toClause13, new Block((Body) () -> 
         {
            cField.assign(entry(iField, pcNameList_16));
            
            if (_and(isEqual(numEntries(cField, "."), 1), () -> isNotEqual(cQualifier, "")))
            {
               cField.assign(concat(cQualifier, cField));
            }
            
            ControlFlowOps.invokeInWithMode("locateWidget", targetProcedure(), "IOO", cField, hField, hTarget);
            
            if ((hField.isValid()).booleanValue())
            {
               /* {get Prop var [handle]} */
               hDataSource.assign(ControlFlowOps.invokeDynamicFunctionIn(handle.class, "getDataSource", hTarget));
               
               if ((hDataSource.isValid()).booleanValue())
               {
                  if (_isGreaterThan(numEntries(cField, "."), 1))
                  {
                     cOnlyField.assign(entry(numEntries(cField, "."), cField, "."));
                  }
                  else
                  {
                     cOnlyField.assign(cField);
                  }
                  
                  cValue /* {fnarg <func-name> <arg> [handle] */.assign(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "columnValue", hDataSource, "I", cOnlyField));
                  lInvalid.assign(not(new logical(ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "assignWidgetValue", targetProcedure(), "II", cField, cValue))));
               }
               else
               {
                  lInvalid.assign(true);
               }
            }
            else
            {
               lInvalid.assign(true);
            }
         }));
         
         /* END DO iField */
         /* Return FALSE if any field was invalid. */
         returnNormal(not(lInvalid));
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical sensitizeRadioButton(final character _pcNameList_16, final integer _piButtonNum_3, final logical _plEnable_1)
   {
      character pcNameList_16 = TypeFactory.initInput(_pcNameList_16);
      integer piButtonNum_3 = TypeFactory.initInput(_piButtonNum_3);
      logical plEnable_1 = TypeFactory.initInput(_plEnable_1);
      /*------------------------------------------------------------------------------
        Purpose:  Disables/enables the specified radio button for the object or 
                  objects in the NameList.
         Params:  INPUT pcNameList  AS CHARACTER
                  INPUT piButtonNum AS INTEGER
                  INPUT plEnable    AS LOGICAL
          Notes:  
      ------------------------------------------------------------------------------*/
      integer iField = TypeFactory.integer();
      character cField = TypeFactory.character();
      character cLabel = TypeFactory.character();
      character cQualifier = TypeFactory.character();
      handle hField = TypeFactory.handle();
      handle hTarget = TypeFactory.handle();
      logical lInvalid = TypeFactory.logical();
      
      return function("sensitizeRadioButton", logical.class, new Block((Body) () -> 
      {
         /* If the first name in the list is qualified that is stored so that it can be used
              as the qualifier for following non-qualified names in the list.  */
         if (_and(isGreaterThan(numEntries(pcNameList_16), 1), () -> isGreaterThan(numEntries(entry(1, pcNameList_16), "."), 1)))
         {
            cQualifier.assign(substring(entry(1, pcNameList_16), 1, lastIndexOf(entry(1, pcNameList_16), ".")));
         }
         
         IntegerExpression resExpr14 = new IntegerExpression()
         {
            public integer execute()
            {
               return numEntries(pcNameList_16);
            }
         };
         ToClause toClause14 = new ToClause(iField, 1, resExpr14);
         
         doTo("loopLabel19", toClause14, new Block((Body) () -> 
         {
            cField.assign(entry(iField, pcNameList_16));
            
            if (_and(isEqual(numEntries(cField, "."), 1), () -> isNotEqual(cQualifier, "")))
            {
               cField.assign(concat(cQualifier, cField));
            }
            
            ControlFlowOps.invokeInWithMode("locateWidget", targetProcedure(), "IOO", cField, hField, hTarget);
            
            if ((hField.isValid()).booleanValue())
            {
               if ((HandleOps.canQuery(hField, "RADIO-BUTTONS")).booleanValue())
               {
                  ErrorManager.silentErrorEnable();
                  cLabel.assign(entry(minus(multiply(piButtonNum_3, 2), 1), hField.unwrapRadioSet().getRadioButtons(), hField.unwrapCommonListWidget().getDelimiter()));
                  ErrorManager.silentErrorDisable();
                  
                  if ((ErrorManager.isError()).booleanValue())
                  {
                     lInvalid.assign(true);
                  }
                  else
                  {
                     if ((plEnable_1).booleanValue())
                     {
                        ErrorManager.silentErrorEnable();
                        hField.unwrapRadioSet().enable(cLabel);
                        ErrorManager.silentErrorDisable();
                     }
                     else
                     {
                        ErrorManager.silentErrorEnable();
                        hField.unwrapRadioSet().disable(cLabel);
                        ErrorManager.silentErrorDisable();
                     }
                     
                     if ((ErrorManager.isError()).booleanValue())
                     {
                        lInvalid.assign(true);
                     }
                  }
               }
               else
               {
                  lInvalid.assign(true);
               }
            }
            else
            {
               lInvalid.assign(true);
            }
         }));
         
         /* do iField to number in namelist */
         /* Return FALSE if any field was invalid. */
         returnNormal(not(lInvalid));
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setAllFieldHandles(final character _pcValue_4)
   {
      character pcValue_4 = TypeFactory.initInput(_pcValue_4);
      
      return function("setAllFieldHandles", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("AllFieldHandles"));
         ghProp.unwrapBufferField().changeValue(pcValue_4);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setAllFieldNames(final character _pcValue_4)
   {
      character pcValue_4 = TypeFactory.initInput(_pcValue_4);
      
      return function("setAllFieldNames", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("AllFieldNames"));
         ghProp.unwrapBufferField().changeValue(pcValue_4);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setBgcolor(final integer _piBgcolor_1)
   {
      integer piBgcolor_1 = TypeFactory.initInput(_piBgcolor_1);
      /*------------------------------------------------------------------------------
        Purpose: Sets the background color of the frame 
         Params: piBGColor AS INTEGER 
          Notes:  
      ------------------------------------------------------------------------------*/
      handle hFrame = TypeFactory.handle();
      
      return function("setBGColor", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ContainerHandle"));
         hFrame.assign(ghProp.unwrapBufferField().value());
         
         if ((hFrame.isValid()).booleanValue())
         {
            hFrame.unwrapWidget().setBgColor(piBgcolor_1);
         }
         
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("BGCOLOR"));
         ghProp.unwrapBufferField().changeValue(piBgcolor_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setColorErrorBg(final integer _piColor_6)
   {
      integer piColor_6 = TypeFactory.initInput(_piColor_6);
      
      return function("setColorErrorBG", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ColorErrorBG"));
         ghProp.unwrapBufferField().changeValue(piColor_6);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setColorErrorFg(final integer _piColor_6)
   {
      integer piColor_6 = TypeFactory.initInput(_piColor_6);
      
      return function("setColorErrorFG", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ColorErrorFG"));
         ghProp.unwrapBufferField().changeValue(piColor_6);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setColorInfoBg(final integer _piColor_6)
   {
      integer piColor_6 = TypeFactory.initInput(_piColor_6);
      
      return function("setColorInfoBG", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ColorInfoBG"));
         ghProp.unwrapBufferField().changeValue(piColor_6);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setColorInfoFg(final integer _piColor_6)
   {
      integer piColor_6 = TypeFactory.initInput(_piColor_6);
      
      return function("setColorInfoFG", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ColorInfoFG"));
         ghProp.unwrapBufferField().changeValue(piColor_6);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setColorWarnBg(final integer _piColor_6)
   {
      integer piColor_6 = TypeFactory.initInput(_piColor_6);
      
      return function("setColorWarnBG", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ColorWarnBG"));
         ghProp.unwrapBufferField().changeValue(piColor_6);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setColorWarnFg(final integer _piColor_6)
   {
      integer piColor_6 = TypeFactory.initInput(_piColor_6);
      
      return function("setColorWarnFG", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ColorWarnFG"));
         ghProp.unwrapBufferField().changeValue(piColor_6);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setDefaultLayout(final character _pcDefault_1)
   {
      character pcDefault_1 = TypeFactory.initInput(_pcDefault_1);
      
      return function("setDefaultLayout", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DefaultLayout"));
         ghProp.unwrapBufferField().changeValue(pcDefault_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setDisableOnInit(final logical _plDisable_1)
   {
      logical plDisable_1 = TypeFactory.initInput(_plDisable_1);
      
      return function("setDisableOnInit", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DisableOnInit"));
         ghProp.unwrapBufferField().changeValue(plDisable_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setEnabledObjFlds(final character _pcValue_4)
   {
      character pcValue_4 = TypeFactory.initInput(_pcValue_4);
      
      return function("setEnabledObjFlds", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("EnabledObjFlds"));
         ghProp.unwrapBufferField().changeValue(pcValue_4);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setEnabledObjHdls(final character _pcHdls_1)
   {
      character pcHdls_1 = TypeFactory.initInput(_pcHdls_1);
      
      return function("setEnabledObjHdls", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("EnabledObjHdls"));
         ghProp.unwrapBufferField().changeValue(pcHdls_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setFgcolor(final integer _piFgcolor_1)
   {
      integer piFgcolor_1 = TypeFactory.initInput(_piFgcolor_1);
      /*------------------------------------------------------------------------------
        Purpose: Sets the foreground color of the frame 
         Params: piBGColor AS INTEGER 
          Notes:  
      ------------------------------------------------------------------------------*/
      handle hFrame = TypeFactory.handle();
      
      return function("setFGColor", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ContainerHandle"));
         hFrame.assign(ghProp.unwrapBufferField().value());
         
         if ((hFrame.isValid()).booleanValue())
         {
            hFrame.unwrapWidget().setFgColor(piFgcolor_1);
         }
         
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FGCOLOR"));
         ghProp.unwrapBufferField().changeValue(piFgcolor_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setFieldPopupMapping(final character _pcFieldPopupMapping_1)
   {
      character pcFieldPopupMapping_1 = TypeFactory.initInput(_pcFieldPopupMapping_1);
      
      return function("setFieldPopupMapping", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FieldPopupMapping"));
         ghProp.unwrapBufferField().changeValue(pcFieldPopupMapping_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setFieldSecurity(final character _pcSecurityType_1)
   {
      character pcSecurityType_1 = TypeFactory.initInput(_pcSecurityType_1);
      
      return function("setFieldSecurity", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FieldSecurity"));
         ghProp.unwrapBufferField().changeValue(pcSecurityType_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setHideOnInit(final logical _plHide_1)
   {
      logical plHide_1 = TypeFactory.initInput(_plHide_1);
      
      return function("setHideOnInit", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("HideOnInit"));
         ghProp.unwrapBufferField().changeValue(plHide_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setLayoutOptions(final character _pcOptions_1)
   {
      character pcOptions_1 = TypeFactory.initInput(_pcOptions_1);
      
      return function("setLayoutOptions", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("LayoutOptions"));
         ghProp.unwrapBufferField().changeValue(pcOptions_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setMinHeight(final decimal _pdHeight_1)
   {
      decimal pdHeight_1 = TypeFactory.initInput(_pdHeight_1);
      
      return function("setMinHeight", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("MinHeight"));
         ghProp.unwrapBufferField().changeValue(pdHeight_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setMinWidth(final decimal _pdWidth_1)
   {
      decimal pdWidth_1 = TypeFactory.initInput(_pdWidth_1);
      
      return function("setMinWidth", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("MinWidth"));
         ghProp.unwrapBufferField().changeValue(pdWidth_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setObjectLayout(final character _pcLayout_1)
   {
      character pcLayout_1 = TypeFactory.initInput(_pcLayout_1);
      
      return function("setObjectLayout", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ObjectLayout"));
         ghProp.unwrapBufferField().changeValue(pcLayout_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setPopupButtonsInFields(final logical _plPopupButtonsInFields_1)
   {
      logical plPopupButtonsInFields_1 = TypeFactory.initInput(_plPopupButtonsInFields_1);
      
      return function("setPopupButtonsInFields", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("PopupButtonsInFields"));
         ghProp.unwrapBufferField().changeValue(plPopupButtonsInFields_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setResizeHorizontal(final logical _plResizeHorizontal_1)
   {
      logical plResizeHorizontal_1 = TypeFactory.initInput(_plResizeHorizontal_1);
      
      return function("setResizeHorizontal", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ResizeHorizontal"));
         ghProp.unwrapBufferField().changeValue(plResizeHorizontal_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setResizeVertical(final logical _plResizeVertical_1)
   {
      logical plResizeVertical_1 = TypeFactory.initInput(_plResizeVertical_1);
      
      return function("setResizeVertical", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ResizeVertical"));
         ghProp.unwrapBufferField().changeValue(plResizeVertical_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setSecuredTokens(final character _pcSecuredTokens_1)
   {
      character pcSecuredTokens_1 = TypeFactory.initInput(_pcSecuredTokens_1);
      
      return function("setSecuredTokens", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("SecuredTokens"));
         ghProp.unwrapBufferField().changeValue(pcSecuredTokens_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical toggleWidget(final character _pcNameList_16)
   {
      character pcNameList_16 = TypeFactory.initInput(_pcNameList_16);
      /*------------------------------------------------------------------------------
        Purpose:  Reverses the value the object or objects of type LOGICAL in the NameList.
         Params:  INPUT pcNameList AS CHARACTER
          Notes:  
      ------------------------------------------------------------------------------*/
      integer iField = TypeFactory.integer();
      character cField = TypeFactory.character();
      character cQualifier = TypeFactory.character();
      character cFalseFormat = TypeFactory.character();
      character cTrueFormat = TypeFactory.character();
      handle hField = TypeFactory.handle();
      handle hTarget = TypeFactory.handle();
      logical lInvalid = TypeFactory.logical();
      
      return function("toggleWidget", logical.class, new Block((Body) () -> 
      {
         /* If the first name in the list is qualified that is stored so that it can be used
              as the qualifier for following non-qualified names in the list.  */
         if (_and(isGreaterThan(numEntries(pcNameList_16), 1), () -> isGreaterThan(numEntries(entry(1, pcNameList_16), "."), 1)))
         {
            cQualifier.assign(substring(entry(1, pcNameList_16), 1, lastIndexOf(entry(1, pcNameList_16), ".")));
         }
         
         IntegerExpression resExpr15 = new IntegerExpression()
         {
            public integer execute()
            {
               return numEntries(pcNameList_16);
            }
         };
         ToClause toClause15 = new ToClause(iField, 1, resExpr15);
         
         doTo("loopLabel20", toClause15, new Block((Body) () -> 
         {
            cField.assign(entry(iField, pcNameList_16));
            
            if (_and(isEqual(numEntries(cField, "."), 1), () -> isNotEqual(cQualifier, "")))
            {
               cField.assign(concat(cQualifier, cField));
            }
            
            ControlFlowOps.invokeInWithMode("locateWidget", targetProcedure(), "IOO", cField, hField, hTarget);
            
            if ((hField.isValid()).booleanValue())
            {
               if (_and(and(HandleOps.canQuery(hField, "SCREEN-VALUE"), () -> HandleOps.canQuery(hField, "DATA-TYPE")), () -> isEqual(hField.unwrapCommonField().getDataType(), "LOGICAL")))
               {
                  ErrorManager.silentErrorEnable();
                  cTrueFormat.assign(entry(1, hField.unwrapCommonField().getFormat(), "/"));
                  cFalseFormat.assign(entry(2, hField.unwrapCommonField().getFormat(), "/"));
                  ErrorManager.silentErrorDisable();
                  
                  if ((ErrorManager.isError()).booleanValue())
                  {
                     lInvalid.assign(true);
                  }
                  else
                  {
                     if (_isGreaterThan(numEntries(cField, "."), 1))
                     {
                        cField.assign(entry(numEntries(cField, "."), cField, "."));
                     }
                     
                     character condition3 = hField.unwrapWidget().getScreenValue();
                     if (condition3.equals(cTrueFormat))
                     {
                        ErrorManager.silentErrorEnable();
                        hField.unwrapWidget().setScreenValue(cFalseFormat);
                        ErrorManager.silentErrorDisable();
                     }
                     else if (condition3.equals(cFalseFormat))
                     {
                        ErrorManager.silentErrorEnable();
                        hField.unwrapWidget().setScreenValue(cTrueFormat);
                        ErrorManager.silentErrorDisable();
                     }
                     else
                     {
                        lInvalid.assign(true);
                     }
                     
                     if ((ErrorManager.isError()).booleanValue())
                     {
                        lInvalid.assign(true);
                     }
                     else
                     {
                        ErrorManager.silentErrorEnable();
                        /* SetDataModified if applicable (no-error as this is in datavis) */
                        ControlFlowOps.invokeInWithMode("fieldModified", targetProcedure(), "I", hField);
                        ErrorManager.silentErrorDisable();
                     }
                  }
               }
               else
               {
                  lInvalid.assign(true);
               }
            }
            else
            {
               lInvalid.assign(true);
            }
         }));
         
         /* END DO iField */
         /* Return FALSE if any field was invalid. */
         returnNormal(not(lInvalid));
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical viewWidget(final character _pcNameList_16)
   {
      character pcNameList_16 = TypeFactory.initInput(_pcNameList_16);
      /*------------------------------------------------------------------------------
        Purpose:  Views the object or objects in the NameList.
         Params:  INPUT pcNameList AS CHARACTER
          Notes:  
      ------------------------------------------------------------------------------*/
      integer iField = TypeFactory.integer();
      character cField = TypeFactory.character();
      character cQualifier = TypeFactory.character();
      handle hField = TypeFactory.handle();
      handle hPopup = TypeFactory.handle();
      handle hTarget = TypeFactory.handle();
      logical lInvalid = TypeFactory.logical();
      
      return function("viewWidget", logical.class, new Block((Body) () -> 
      {
         /* If the first name in the list is qualified that is stored so that it can be used
              as the qualifier for following non-qualified names in the list.  */
         if (_and(isGreaterThan(numEntries(pcNameList_16), 1), () -> isGreaterThan(numEntries(entry(1, pcNameList_16), "."), 1)))
         {
            cQualifier.assign(substring(entry(1, pcNameList_16), 1, lastIndexOf(entry(1, pcNameList_16), ".")));
         }
         
         IntegerExpression resExpr16 = new IntegerExpression()
         {
            public integer execute()
            {
               return numEntries(pcNameList_16);
            }
         };
         ToClause toClause16 = new ToClause(iField, 1, resExpr16);
         
         doTo("loopLabel21", toClause16, new Block((Body) () -> 
         {
            cField.assign(entry(iField, pcNameList_16));
            
            if (_and(isEqual(numEntries(cField, "."), 1), () -> isNotEqual(cQualifier, "")))
            {
               cField.assign(concat(cQualifier, cField));
            }
            
            ControlFlowOps.invokeInWithMode("locateWidget", targetProcedure(), "IOO", cField, hField, hTarget);
            
            if ((hField.isValid()).booleanValue())
            {
               /* This is a SmartDataField. */
               if ((HandleOps.canQuery(hField, "FILE-NAME")).booleanValue())
               {
                  ControlFlowOps.invokeIn("viewObject", hField);
               }
               else
               {
                  if ((HandleOps.canSet(hField, "HIDDEN")).booleanValue())
                  {
                     ErrorManager.silentErrorEnable();
                     hField.unwrapWidget().setHidden(new logical(false));
                     ErrorManager.silentErrorDisable();
                     
                     if ((ErrorManager.isError()).booleanValue())
                     {
                        lInvalid.assign(true);
                     }
                     else
                     {
                        ErrorManager.silentErrorEnable();
                        hPopup.assign(ControlFlowOps.invokeDynamicFunctionInWithMode(handle.class, "popupHandle", hTarget, "I", hField));
                        ErrorManager.silentErrorDisable();
                     }
                     
                     if ((hPopup.isValid()).booleanValue())
                     {
                        hPopup.unwrapWidget().setHidden(new logical(false));
                     }
                  }
 /* Viewer fields */                  else
                  {
                     if ((HandleOps.canSet(hField, "VISIBLE")).booleanValue())
                     {
                        ErrorManager.silentErrorEnable();
                        hField.unwrapWidget().setVisible(new logical(true));
                        ErrorManager.silentErrorDisable();
                        
                        if ((ErrorManager.isError()).booleanValue())
                        {
                           lInvalid.assign(true);
                        }
                     }
 /* Browser columns */                     else
                     {
                        lInvalid.assign(true);
                     }
                  }
               }
            }
            else
            {
               lInvalid.assign(true);
            }
         }));
         
         /* END DO iField */
         /* Return FALSE if any field was invalid. */
         returnNormal(not(lInvalid));
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public handle widgetHandle(final character _pcName_8)
   {
      character pcName_8 = TypeFactory.initInput(_pcName_8);
      handle hField = TypeFactory.handle();
      handle hTarget = TypeFactory.handle();
      
      return function("widgetHandle", handle.class, new Block((Body) () -> 
      {
         ControlFlowOps.invokeInWithMode("locateWidget", targetProcedure(), "IOO", pcName_8, hField, hTarget);
         
         if ((hField.isValid()).booleanValue())
         {
            returnNormal(hField);
         }
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical widgetIsBlank(final character _pcNameList_16)
   {
      character pcNameList_16 = TypeFactory.initInput(_pcNameList_16);
      /*------------------------------------------------------------------------------
        Purpose: Check if one or several widgets are blank 
      Parameter: pcColumn - column name (comma-separated list)  
          Notes: This was implemented to be able workaround the fact that INPUT-VALUE 
                 performs an implicit validation of format that makes it impossible
                 to check whether a field with a restrictive format really is blank.
               - Returns true if ALL widgets are blank  
      ------------------------------------------------------------------------------*/
      handle hField = TypeFactory.handle();
      character cField = TypeFactory.character();
      character cValue = TypeFactory.character();
      integer iWidget = TypeFactory.integer();
      character cChar = TypeFactory.character();
      character cQualifier = TypeFactory.character();
      handle hTarget = TypeFactory.handle();
      
      return function("widgetIsBlank", logical.class, new Block((Pre) () -> 
      {
         testframeFrame = GenericFrame.createFrame(VisualTestframe.class, "testframe");
      }, 
      (Body) () -> 
      {
         testframeFrame.openScope();
         
         /* If the first name in the list is qualified that is stored so that it can be used
              as the qualifier for following non-qualified names in the list.  */
         if (_and(isGreaterThan(numEntries(pcNameList_16), 1), () -> isGreaterThan(numEntries(entry(1, pcNameList_16), "."), 1)))
         {
            cQualifier.assign(substring(entry(1, pcNameList_16), 1, lastIndexOf(entry(1, pcNameList_16), ".")));
         }
         
         IntegerExpression resExpr17 = new IntegerExpression()
         {
            public integer execute()
            {
               return numEntries(pcNameList_16);
            }
         };
         ToClause toClause17 = new ToClause(iWidget, 1, resExpr17);
         
         doTo("loopLabel22", toClause17, new Block((Body) () -> 
         {
            cField.assign(entry(iWidget, pcNameList_16));
            
            if (_and(isEqual(numEntries(cField, "."), 1), () -> isNotEqual(cQualifier, "")))
            {
               cField.assign(concat(cQualifier, cField));
            }
            
            ControlFlowOps.invokeInWithMode("locateWidget", targetProcedure(), "IOO", cField, hField, hTarget);
            
            /* If the widget is not valid then return unknown. */
            if (_not(hField.isValid()))
            {
               returnNormal(new logical());
            }
            
            /* This is a SmartDataField, check its DataValue */
            if ((HandleOps.canQuery(hField, "FILE-NAME")).booleanValue())
            {
               /* {get Prop var [handle]} */
               cValue.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getDataValue", hField));
               
               if (_isNotEqual(cValue, ""))
               {
                  returnNormal(false);
               }
            }
            else
            {
               if (_isNotEqual(hField.unwrapCommonField().getDataType(), "CHARACTER"))
               {
                  returnNormal(false);
               }
               
               if (_and(HandleOps.canQuery(hField, "FORMAT"), () -> HandleOps.canQuery(hField, "SCREEN-VALUE")))
               {
                  ErrorManager.silentErrorEnable();
                  testframeFrame.widgetCChar().setFormat(hField.unwrapCommonField().getFormat());
                  testframeFrame.widgetCChar().setScreenValue(hField.unwrapWidget().getScreenValue());
                  ErrorManager.silentErrorDisable();
                  
                  if (_isNotEqual(testframeFrame.widgetCChar().getScreenValue(), ""))
                  {
                     returnNormal(false);
                  }
               }
               else
               {
                  if ((HandleOps.canQuery(hField, "SCREEN-VALUE")).booleanValue())
                  {
                     if (_isNotEqual(hField.unwrapWidget().getScreenValue(), ""))
                     {
                        returnNormal(false);
                     }
                  }
                  else
                  {
                     returnNormal(new logical());
                  }
               }
            }
         }));
         
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical widgetIsFocused(final character _pcName_8)
   {
      character pcName_8 = TypeFactory.initInput(_pcName_8);
      /*------------------------------------------------------------------------------
        Purpose:  Returns the TRUE if pcName has focus
         Params:  INPUT pcName AS CHARACTER
          Notes:  Does not support SmartDataFields
      ------------------------------------------------------------------------------*/
      handle hField = TypeFactory.handle();
      handle hTarget = TypeFactory.handle();
      
      return function("widgetIsFocused", logical.class, new Block((Body) () -> 
      {
         ControlFlowOps.invokeInWithMode("locateWidget", targetProcedure(), "IOO", pcName_8, hField, hTarget);
         
         if ((hField.isValid()).booleanValue())
         {
            if (_isEqual(hField, focus()))
            {
               returnNormal(true);
            }
            else
            {
               returnNormal(false);
            }
         }
         else
         {
            returnNormal(new logical());
         }
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical widgetIsModified(final character _pcNameList_16)
   {
      character pcNameList_16 = TypeFactory.initInput(_pcNameList_16);
      /*------------------------------------------------------------------------------
        Purpose:  Returns a logical value indicating whether the value of any of
                  the list of fields passed in has changed since it was displayed.
         Params:  INPUT pcNameList AS CHARACTER
          Notes:  A single LOGICAL value is returned if *any* of the INPUT fields
                  has changed. This makes it easy to determine whether any of the
                  values which are involved in an expression or calculation have 
                  changed.
                  The function returns unknown (?) if any field is invalid or
                  can't be queried.
      ------------------------------------------------------------------------------*/
      integer iField = TypeFactory.integer();
      character cField = TypeFactory.character();
      character cQualifier = TypeFactory.character();
      handle hField = TypeFactory.handle();
      handle hTarget = TypeFactory.handle();
      logical lModified = TypeFactory.logical();
      
      return function("widgetIsModified", logical.class, new Block((Body) () -> 
      {
         /* If the first name in the list is qualified that is stored so that it can be used
              as the qualifier for following non-qualified names in the list.  */
         if (_and(isGreaterThan(numEntries(pcNameList_16), 1), () -> isGreaterThan(numEntries(entry(1, pcNameList_16), "."), 1)))
         {
            cQualifier.assign(substring(entry(1, pcNameList_16), 1, lastIndexOf(entry(1, pcNameList_16), ".")));
         }
         
         IntegerExpression resExpr18 = new IntegerExpression()
         {
            public integer execute()
            {
               return numEntries(pcNameList_16);
            }
         };
         ToClause toClause18 = new ToClause(iField, 1, resExpr18);
         
         doTo("loopLabel23", toClause18, new Block((Body) () -> 
         {
            cField.assign(entry(iField, pcNameList_16));
            
            if (_and(isEqual(numEntries(cField, "."), 1), () -> isNotEqual(cQualifier, "")))
            {
               cField.assign(concat(cQualifier, cField));
            }
            
            ControlFlowOps.invokeInWithMode("locateWidget", targetProcedure(), "IOO", cField, hField, hTarget);
            
            if (_not(hField.isValid()))
            {
               returnNormal(new logical());
            }
            else
            {
               if ((HandleOps.canQuery(hField, "FILE-NAME")).booleanValue())
               {
                  /* This is a SmartDataField. Use its DataModified property. */
                  /* {get Prop var [handle]} */
                  lModified.assign(ControlFlowOps.invokeDynamicFunctionIn(logical.class, "getDataModified", hField));
               }
               else
               {
                  if ((HandleOps.canQuery(hField, "MODIFIED")).booleanValue())
                  {
                     lModified.assign(hField.unwrapWidget().isModified());
                  }
                  else
                  {
                     returnNormal(new logical());
                  }
               }
               
               if ((lModified).booleanValue())
               {
                  returnNormal(true);
               }
            }
         }));
         
         /* END DO iField */
         /* If we fell through, then no field was modified. */
         returnNormal(false);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical widgetIsTrue(final character _pcName_8)
   {
      character pcName_8 = TypeFactory.initInput(_pcName_8);
      handle hBuffer = TypeFactory.handle();
      handle hField = TypeFactory.handle();
      handle hTarget = TypeFactory.handle();
      logical lValue = TypeFactory.logical();
      
      return function("widgetIsTrue", logical.class, new Block((Body) () -> 
      {
         ControlFlowOps.invokeInWithMode("locateWidget", targetProcedure(), "IOO", pcName_8, hField, hTarget);
         
         if ((hField.isValid()).booleanValue())
         {
            if (_and(HandleOps.canQuery(hField, "DATA-TYPE"), () -> isEqual(hField.unwrapCommonField().getDataType(), "LOGICAL")))
            {
               if ((HandleOps.canQuery(hField, "INPUT-VALUE")).booleanValue())
               {
                  ErrorManager.silentErrorEnable();
                  lValue.assign(hField.unwrapWidget().inputValue());
                  ErrorManager.silentErrorDisable();
                  
                  if ((ErrorManager.isError()).booleanValue())
                  {
                     lValue.setUnknown();
                  }
               }
               else
               {
                  ErrorManager.silentErrorEnable();
                  /* {get Prop var [handle]} */
                  hBuffer.assign(ControlFlowOps.invokeDynamicFunctionIn(handle.class, "getQueryRowObject", hTarget));
                  ErrorManager.silentErrorDisable();
                  
                  if ((hBuffer.isValid()).booleanValue())
                  {
                     if (_isEqual(numEntries(pcName_8, "."), 2))
                     {
                        pcName_8.assign(entry(2, pcName_8, "."));
                     }
                     
                     ErrorManager.silentErrorEnable();
                     hField.assign(hBuffer.unwrapBuffer().bufferField(pcName_8));
                     lValue.assign(hField.unwrapBufferField().value());
                     ErrorManager.silentErrorDisable();
                     
                     if ((ErrorManager.isError()).booleanValue())
                     {
                        lValue.setUnknown();
                     }
                  }
                  else
                  {
                     lValue.setUnknown();
                  }
               }
            }
            else
            {
               lValue.setUnknown();
            }
         }
         else
         {
            lValue.setUnknown();
         }
         returnNormal(lValue);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character widgetValue(final character _pcName_8)
   {
      character pcName_8 = TypeFactory.initInput(_pcName_8);
      /*------------------------------------------------------------------------------
        Purpose:  Returns the unformatted value of an object.
         Params:  INPUT pcName AS CHARACTER
          Notes:  Does not support SmartDataFields
      ------------------------------------------------------------------------------*/
      character cValue = TypeFactory.character();
      handle hBuffer = TypeFactory.handle();
      handle hField = TypeFactory.handle();
      handle hTarget = TypeFactory.handle();
      
      return function("widgetValue", character.class, new Block((Body) () -> 
      {
         ControlFlowOps.invokeInWithMode("locateWidget", targetProcedure(), "IOO", pcName_8, hField, hTarget);
         
         if ((hField.isValid()).booleanValue())
         {
            /* This is a SmartDataField. SmartDataFields do not have a standard
                   function for returning an unformatted value so nothing is returned. */
            if ((HandleOps.canQuery(hField, "FILE-NAME")).booleanValue())
            {
               cValue.setUnknown();
            }
            else
            {
               if ((HandleOps.canQuery(hField, "INPUT-VALUE")).booleanValue())
               {
                  ErrorManager.silentErrorEnable();
                  cValue.assign(hField.unwrapWidget().inputValue());
                  ErrorManager.silentErrorDisable();
                  
                  /* INPUT-VALUE will return a 4GL error if the value is blank and the format does not
                           allow blank.  If there is an error, check if the widget is blank and if so return blank */
                  if ((ErrorManager.isError()).booleanValue())
                  {
                     if ((new logical(ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "widgetIsBlank", hTarget, "I", entry(numEntries(pcName_8, "."), pcName_8, ".")))).booleanValue())
                     {
                        cValue.assign("");
                     }
                     else
                     {
                        cValue.setUnknown();
                     }
                  }
               }
 /* Viewer fields */               else
               {
                  ErrorManager.silentErrorEnable();
                  /* {get Prop var [handle]} */
                  hBuffer.assign(ControlFlowOps.invokeDynamicFunctionIn(handle.class, "getQueryRowObject", hTarget));
                  ErrorManager.silentErrorDisable();
                  
                  if ((hBuffer.isValid()).booleanValue())
                  {
                     if (_isEqual(numEntries(pcName_8, "."), 2))
                     {
                        pcName_8.assign(entry(2, pcName_8, "."));
                     }
                     
                     ErrorManager.silentErrorEnable();
                     hField.assign(hBuffer.unwrapBuffer().bufferField(pcName_8));
                     cValue.assign(valueOf(hField.unwrapBufferField().value()));
                     ErrorManager.silentErrorDisable();
                     
                     if ((ErrorManager.isError()).booleanValue())
                     {
                        cValue.setUnknown();
                     }
                  }
                  else
                  {
                     cValue.setUnknown();
                  }
               }
            }
         }
         else
         {
            cValue.setUnknown();
         }
         returnNormal(cValue);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character widgetValueList(final character _pcNameList_16, final character _pcDelimiter_3)
   {
      character pcNameList_16 = TypeFactory.initInput(_pcNameList_16);
      character pcDelimiter_3 = TypeFactory.initInput(_pcDelimiter_3);
      /*------------------------------------------------------------------------------
        Purpose:  Returns the unformatted value of objects in a delimited list.
         Params:  INPUT pcNameList  AS CHARACTER
                  INPUT pcDelimiter AS CHARACTER
          Notes:  If pcDelimiter is ?, | is used
                  Does not support SmartDataFields
      ------------------------------------------------------------------------------*/
      character cField = TypeFactory.character();
      character cQualifier = TypeFactory.character();
      character cValue = TypeFactory.character();
      character cValueList = TypeFactory.character();
      integer iField = TypeFactory.integer();
      
      return function("widgetValueList", character.class, new Block((Body) () -> 
      {
         if (_isUnknown(pcDelimiter_3))
         {
            pcDelimiter_3.assign("|");
         }
         
         /* If the first name in the list is qualified that is stored so that it can be used
              as the qualifier for following non-qualified names in the list.  */
         if (_and(isGreaterThan(numEntries(pcNameList_16), 1), () -> isGreaterThan(numEntries(entry(1, pcNameList_16), "."), 1)))
         {
            cQualifier.assign(substring(entry(1, pcNameList_16), 1, lastIndexOf(entry(1, pcNameList_16), ".")));
         }
         
         IntegerExpression resExpr19 = new IntegerExpression()
         {
            public integer execute()
            {
               return numEntries(pcNameList_16);
            }
         };
         ToClause toClause19 = new ToClause(iField, 1, resExpr19);
         
         doTo("loopLabel24", toClause19, new Block((Body) () -> 
         {
            cField.assign(entry(iField, pcNameList_16));
            
            if (_and(isEqual(numEntries(cField, "."), 1), () -> isNotEqual(cQualifier, "")))
            {
               cField.assign(concat(cQualifier, cField));
            }
            
            cValue.assign(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "widgetValue", targetProcedure(), "I", cField));
            cValueList.assign(concat(cValueList, (_isEqual(iField, 1) ? new character("") : pcDelimiter_3), _isUnknown(cValue) ? new character("?") : cValue));
         }));
         
         /* do iField to number entries in namelist */
         returnNormal(cValueList);
      }));
   }
}
